package com.nordicpeak.flowengine.addflowinstanceasmanager;

import java.io.Serializable;
import java.sql.SQLException;
import java.sql.Timestamp;

import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.time.TimeUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.addflowinstance.AddFlowInstanceModule;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager;

public class AddFlowInstanceCallback implements com.nordicpeak.flowengine.addflowinstance.AddFlowInstanceCallback, Serializable {

	private static final long serialVersionUID = -1518909826970607855L;
	protected static Logger log = Logger.getLogger(AddFlowInstanceCallback.class);

	private final FlowFamilyAddInstanceSettings settings;
	private final User poster;
	private final User manager;
	private final String submitEventText;
	private final String saveEventText;
	private final String returnURL;

	public AddFlowInstanceCallback(FlowFamilyAddInstanceSettings addInstanceSettings, User poster, User manager, String eventText, String saveEventText, String returnURL) {

		super();

		this.settings = addInstanceSettings;
		this.poster = poster;
		this.manager = manager;
		this.submitEventText = eventText;
		this.saveEventText = saveEventText;
		this.returnURL = returnURL;
	}

	public User getPoster() {

		return poster;
	}

	public User getManager() {

		return manager;
	}
	
	@Override
	public void beforeSave(MutableFlowInstanceManager instanceManager, EventType eventType) {

		FlowInstance flowInstance = (FlowInstance) instanceManager.getFlowInstance();
		MutableAttributeHandler flowInstanceAttributeHandler = flowInstance.getAttributeHandler();

		if (eventType == EventType.SUBMITTED) {

			flowInstanceAttributeHandler.setAttribute(Constants.FLOW_INSTANCE_ADDED_BY_MANAGER_ATTRIBUTE, "true");

		} else {

			flowInstanceAttributeHandler.setAttribute(Constants.FLOW_INSTANCE_PREPARED_BY_MANAGER_ATTRIBUTE, "true");
		}
		
		if (settings.isSuppressSubmittedUserSubmitNotification()) {
			
			flowInstanceAttributeHandler.setAttribute(Constants.FLOW_INSTANCE_SUPPRESS_USER_SUBMITTED_NOTIFICATION_ATTRIBUTE, "true");
		}
		
		if (settings.isSuppressManagersSubmitNotification()) {
			
			flowInstanceAttributeHandler.setAttribute(Constants.FLOW_INSTANCE_SUPPRESS_MANAGERS_SUBMITTED_NOTIFICATION_ATTRIBUTE, "true");
		}
		
		if (settings.isSuppressArchiveNotification()) {
			
			flowInstanceAttributeHandler.setAttribute(Constants.FLOW_INSTANCE_SUPPRESS_ARCHIVED_NOTIFICATION_ATTRIBUTE, "true");
		}
	}
	
	@Override
	public void afterSave(FlowInstanceManager instanceManager, EventType eventType, HttpSession session, FlowInstanceEvent saveEvent, AddFlowInstanceModule addFlowInstanceModule) {

		Timestamp now = saveEvent != null ? saveEvent.getAdded() : TimeUtils.getCurrentTimestamp(false);

		if (eventType == EventType.SUBMITTED) {

			try {
				addFlowInstanceModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(instanceManager.getFlowInstance(), EventType.OTHER_EVENT, submitEventText, manager, now, null);

			} catch (SQLException e) {
				log.error("Error adding submitted by manager event", e);
			}

		} else {

			try {
				addFlowInstanceModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(instanceManager.getFlowInstance(), EventType.OTHER_EVENT, saveEventText, manager, now, null);

			} catch (SQLException e) {
				log.error("Error adding saved by manager event", e);
			}
		}
	}

	@Override
	public String getCustomReturnURL(FlowInstanceManager instanceManager) {

		if(instanceManager.getFlowInstance().getFirstSubmitted() != null) {
			return returnURL + "?added=" + instanceManager.getFlowInstanceID();
		}

		return returnURL + "?saved=true";
	}

}
