package com.nordicpeak.flowengine.queries.alternativelistcheckboxquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.MySQLRowLimiter;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.alternativelist.AlternativeListHandlerModule;
import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListProvider;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQuery;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListQueryProvider;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstance;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.populators.FreeTextAlternativePopulator;
import com.nordicpeak.flowengine.queries.alternativelistcheckboxquery.validationerrors.TooFewAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.alternativelistcheckboxquery.validationerrors.TooManyAlternativesSelectedValidationError;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class AlternativeListCheckboxQueryProviderModule extends BaseQueryProviderModule<AlternativeListCheckboxQueryInstance> implements BaseQueryCRUDCallback, AlternativeListQueryProvider {
	
	
	private static final RelationQuery SAVE_QUERY_INSTANCE_RELATION_QUERY = new RelationQuery(AlternativeListCheckboxQueryInstance.ALTERNATIVES_RELATION);
	
	@XSLVariable(prefix = "java.")
	private String missingList = "Missing";
	
	@XSLVariable(prefix = "java.")
	private String countText = "Count";
	
	@XSLVariable(prefix = "java.")
	private String alternativesText = "Alternative";
	
	private AnnotatedDAO<AlternativeListCheckboxQuery> queryDAO;
	private AnnotatedDAO<AlternativeListCheckboxQueryInstance> queryInstanceDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;
	
	private AlternativeListCheckboxQueryCRUD queryCRUD;
	
	private QueryParameterFactory<AlternativeListCheckboxQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<AlternativeListCheckboxQuery, String> listNameParamFactory;
	private QueryParameterFactory<AlternativeListCheckboxQueryInstance, Integer> queryInstanceIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	private AlternativeListHandlerModule alternativeListHandlerModule;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, AlternativeListCheckboxQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);
		
		queryDAO = daoFactory.getDAO(AlternativeListCheckboxQuery.class);
		queryInstanceDAO = daoFactory.getDAO(AlternativeListCheckboxQueryInstance.class);
		
		queryCRUD = new AlternativeListCheckboxQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<AlternativeListCheckboxQuery>(AlternativeListCheckboxQuery.class), "AlternativeListCheckboxQuery", "query", null, this);
		
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		listNameParamFactory = queryDAO.getParamFactory("listName", String.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		
		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());
		
		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}
	
	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {
		
		AlternativeListCheckboxQuery query = new AlternativeListCheckboxQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, null);
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {
		
		AlternativeListCheckboxQuery query = new AlternativeListCheckboxQuery();
		
		query.setQueryID(descriptor.getQueryID());
		
		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);
		
		this.queryDAO.add(query, transactionHandler, null);
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {
		
		AlternativeListCheckboxQuery query = this.getQuery(descriptor.getQueryID(), extraData);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		AlternativeListCheckboxQuery query = this.getQuery(descriptor.getQueryID(), true, transactionHandler);
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {
		
		AlternativeListCheckboxQueryInstance queryInstance = null;
		
		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance = new AlternativeListCheckboxQueryInstance();
			
		} else {
			
			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
			
			if (queryInstance == null) {
				
				return null;
			}
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), true));
		
		if (queryInstance.getQuery() == null) {
			
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.getQuery().scanAttributeTags();
		
		queryInstance.set(descriptor);
		
		//If this is a new query instance copy the default values
		if (descriptor.getQueryInstanceID() == null) {
			
			queryInstance.copyQueryValues();
			queryInstance.getQuery().setAlternativeList(getAlternativeListInstance(queryInstance.getQuery(), null, poster));
		}
		
		return queryInstance;
	}
	
	
	
	@Override
	public ImmutableQueryInstance getImmutableQueryInstance(MutableQueryInstanceDescriptor descriptor, HttpServletRequest req, InstanceMetadata instanceMetadata) throws Throwable {
		
		if (descriptor.getQueryInstanceID() == null) {
			return null;
		}
		
		AlternativeListCheckboxQueryInstance queryInstance = getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			return null;
		}
		
		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID(), false));
		
		if (queryInstance.getQuery() == null) {
			return null;
		}
		
		if (req != null) {
			
			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);
			
			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();
		
		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());
		
		queryInstance.getQuery().scanAttributeTags();
		
		queryInstance.set(descriptor);
		
		return queryInstance;
	}

	@Override
	public QueryResponse getFormHTML(AlternativeListCheckboxQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {
		
		if (queryInstance.getQuery().getAlternativeList() == null || queryInstance.getQuery().getAlternativeList().requiresReload(attributeHandler)) {
			
			queryInstance.getQuery().setAlternativeList(getAlternativeListInstance(queryInstance.getQuery(), attributeHandler, poster));
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}
	
	private AlternativeListCheckboxQuery getQuery(Integer queryID, boolean withAlternativeList) throws SQLException {
		
		HighLevelQuery<AlternativeListCheckboxQuery> getQuery = new HighLevelQuery<AlternativeListCheckboxQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		AlternativeListCheckboxQuery query = queryDAO.get(getQuery);
		
		if (query != null && withAlternativeList) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}
	
	private AlternativeListCheckboxQuery getQuery(Integer queryID, boolean withAlternativeList, TransactionHandler transactionHandler) throws SQLException {
		
		HighLevelQuery<AlternativeListCheckboxQuery> getQuery = new HighLevelQuery<AlternativeListCheckboxQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		AlternativeListCheckboxQuery query = queryDAO.get(getQuery, transactionHandler);
		
		if (query != null && withAlternativeList) {
			query.setAlternativeList(getAlternativeListInstance(query, null, null));
		}
		
		return query;
	}
	
	private AlternativeListCheckboxQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {
		
		HighLevelQuery<AlternativeListCheckboxQueryInstance> query = new HighLevelQuery<AlternativeListCheckboxQueryInstance>(AlternativeListCheckboxQueryInstance.ALTERNATIVES_RELATION, AlternativeListCheckboxQueryInstance.QUERY_RELATION);
		
		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));
		
		return queryInstanceDAO.get(query);
	}
	
	@Override
	public void save(AlternativeListCheckboxQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {
		
		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {
			
			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());
			
			this.queryInstanceDAO.add(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
			
		} else {
			
			this.queryInstanceDAO.update(queryInstance, transactionHandler, SAVE_QUERY_INSTANCE_RELATION_QUERY);
		}
	}
	
	@Override
	public void populate(AlternativeListCheckboxQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {
		
		AlternativeListCheckboxQuery query = queryInstance.getQuery();
		
		List<? extends ImmutableAlternative> availableAlternatives = null;
		
		if (query.getAlternativeList() != null) {
			
			availableAlternatives = queryInstance.getQuery().getAlternativeList().getAlternatives();
		}
		
		if (CollectionUtils.isEmpty(availableAlternatives)) { // If the parent query doesn't have any alternatives then there is no population to do
			
			if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !allowPartialPopulation) {
				
				throw new ValidationException(new ValidationError("RequiredQuery"));
				
			} else {
			
				queryInstance.reset(attributeHandler);
				
				return;
			}
		}
		
		List<AlternativeListCheckboxAlternative> selectedAlternatives = new ArrayList<AlternativeListCheckboxAlternative>(availableAlternatives.size());
		
		for (ImmutableAlternative alternative : availableAlternatives) {
			
			if (req.getParameter("q" + query.getQueryID() + "_alternative" + alternative.getAlternativeID()) != null) {

				selectedAlternatives.add(new AlternativeListCheckboxAlternative(alternative));
			}
		}
		
		int alternativesSelected = selectedAlternatives.size();
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		String freeTextAlternativeValue = null;
		
		if (query.getFreeTextAlternative() != null) {
			
			freeTextAlternativeValue = FreeTextAlternativePopulator.populate(query.getQueryID(), "_freeTextAlternative", req, validationErrors);
		}
		
		if (freeTextAlternativeValue != null) {
			
			if (!validationErrors.isEmpty() && !ValidationUtils.containsValidationErrorWithMessageKey("FreeTextAlternativeValueRequired", validationErrors)) {
				
				allowPartialPopulation = false;
			}
			
			alternativesSelected++;
		}
		
		//If partial population is allowed and the user has not selected any alternatives, skip validation
		if (allowPartialPopulation) {
			
			if (alternativesSelected == 0) {
				
				queryInstance.setAlternatives(null);
				queryInstance.setFreeTextAlternativeValue(null);
				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				
				if (query.isSetAsAttribute()) {
					queryInstance.resetAttribute(attributeHandler);
				}
				
			} else {
				
				queryInstance.setAlternatives(selectedAlternatives);
				queryInstance.setFreeTextAlternativeValue(freeTextAlternativeValue);
				queryInstance.getQueryInstanceDescriptor().setPopulated(true);
				
				if (query.isSetAsAttribute()) {
					
					queryInstance.resetAttribute(attributeHandler);
					queryInstance.setAttribute(attributeHandler);
				}
			}
			
			return;
		}
		
		//Check if this query is required or if the user has selected any alternatives anyway
		if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED || alternativesSelected != 0) {
			
			int totalAvailableAlternatives = availableAlternatives.size() + (query.getFreeTextAlternative() != null ? 1 : 0);
			
			if (query.isRequireAllChecked() && alternativesSelected < totalAvailableAlternatives) {
			
				validationErrors.add(new TooFewAlternativesSelectedValidationError(alternativesSelected, totalAvailableAlternatives));
				
			} else if (queryInstance.getMinChecked() != null && alternativesSelected < queryInstance.getMinChecked()) {
				
				validationErrors.add(new TooFewAlternativesSelectedValidationError(alternativesSelected, queryInstance.getMinChecked()));
				
			} else if (queryInstance.getMaxChecked() != null && alternativesSelected > queryInstance.getMaxChecked()) {
				
				validationErrors.add(new TooManyAlternativesSelectedValidationError(alternativesSelected, queryInstance.getMaxChecked()));
				
			} else if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && alternativesSelected == 0) {
				
				validationErrors.add(new ValidationError("RequiredQuery"));
			}
		}
		
		if (!validationErrors.isEmpty()) {
			
			throw new ValidationException(validationErrors);
		}
		
		queryInstance.setFreeTextAlternativeValue(freeTextAlternativeValue);
		queryInstance.setAlternatives(selectedAlternatives);
		queryInstance.getQueryInstanceDescriptor().setPopulated(!selectedAlternatives.isEmpty() || freeTextAlternativeValue != null);
		
		if (query.isSetAsAttribute()) {
			
			queryInstance.resetAttribute(attributeHandler);
			
			if (queryInstance.getQueryInstanceDescriptor().isPopulated()) {
				
				queryInstance.setAttribute(attributeHandler);
			}
		}
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.queryCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {
		
		AlternativeListCheckboxQuery query = getQuery(descriptor.getQueryID(), false);
		
		if (query == null) {
			
			return false;
		}
		
		this.queryDAO.delete(query, transactionHandler);
		
		return true;
	}
	
	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		AlternativeListCheckboxQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());
		
		if (queryInstance == null) {
			
			return false;
		}
		
		this.queryInstanceDAO.delete(queryInstance, transactionHandler);
		
		return true;
	}
	
	@Override
	public String getTitlePrefix() {
		
		return this.moduleDescriptor.getName();
	}
	
	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {
		
		AlternativeListCheckboxQuery query = getQuery(sourceQueryDescriptor.getQueryID(), false, transactionHandler);
		
		query.setQueryID(copyQueryDescriptor.getQueryID());
		
		this.queryDAO.add(query, transactionHandler, new RelationQuery());
	}
	
	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, AlternativeListCheckboxQueryInstance queryInstance, AttributeHandler attributeHandler) {
		
		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);
		
		if (queryInstance.getQuery().getDescription() != null) {
			
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}
	
	@Override
	protected Class<AlternativeListCheckboxQueryInstance> getQueryInstanceClass() {
		
		return AlternativeListCheckboxQueryInstance.class;
	}
	
	public String getMissingAlternativeList() {
		
		return missingList;
	}
	
	protected Collection<AlternativeListDescriptor> getAlternativeLists() {
		
		return alternativeListHandlerModule.getAlternativeLists();
	}
	
	protected AlternativeListProvider getAlternativeListProvider(String providerID) {
		
		return alternativeListHandlerModule.getProvider(providerID);
	}
	
	protected AlternativeListDescriptor getAlternativeListDescriptor(AlternativeListQuery query) {
		
		if (!StringUtils.isEmpty(query.getAlternativeListName())) {
			
			return alternativeListHandlerModule.getAlternativeList(query.getAlternativeListName());
		}
		
		return null;
	}
	
	protected AlternativeListInstance getAlternativeListInstance(AlternativeListQuery query, AttributeHandler attributeHandler, User poster) {
		
		AlternativeListDescriptor descriptor = getAlternativeListDescriptor(query);
		
		if (descriptor != null) {
			
			return alternativeListHandlerModule.getListInstance(descriptor, attributeHandler, poster);
		}
		
		return null;
	}
	
	@Override
	public Collection<? extends AlternativeListQuery> getQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) {
		
		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<Integer>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE listName = ?", IntegerPopulator.getPopulator());
			idQuery.setString(1, descriptor.getName());
			
			List<Integer> queryIDs = idQuery.executeQuery();
			
			if (!CollectionUtils.isEmpty(queryIDs)) {
				
				List<AlternativeListCheckboxQuery> queries = new ArrayList<AlternativeListCheckboxQuery>(queryIDs.size());
				
				for (Integer queryID : queryIDs) {
					
					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);
					
					if (queryDescriptor != null) {
						
						queries.add((AlternativeListCheckboxQuery) getQuery(queryDescriptor, false));
					}
				}
				
				return queries;
			}
			
		} catch (SQLException e) {
			
			log.error("Unable to get queries using alternative list " + descriptor, e);
		}
		
		return null;
	}

	@Override
	public boolean hasQueriesUsingAlternativeList(AlternativeListDescriptor descriptor) throws SQLException {

		HighLevelQuery<AlternativeListCheckboxQuery> query = new HighLevelQuery<>();
		query.addParameter(listNameParamFactory.getParameter(descriptor.getName()));
		query.setRowLimiter(MySQLRowLimiter.SINGLE_ROW);
		return queryDAO.getBoolean(query);
	}
	
}
