<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/alternativelistdropdownquery.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
	</xsl:template>
		
	<xsl:template match="ShowQueryValues">
	
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
							<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/description"> 
								<xsl:text> hasDescription</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/queryID" />
					</xsl:call-template>
				</div>
				
				<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:choose>
						<xsl:when test="AlternativeListDropDownQueryInstance/alternativeName">
							<xsl:value-of select="AlternativeListDropDownQueryInstance/alternativeName"/>
						</xsl:when>
						<xsl:when test="AlternativeListDropDownQueryInstance/freeTextAlternativeValue">
							<xsl:value-of select="AlternativeListDropDownQueryInstance/freeTextAlternativeValue"/>
						</xsl:when>
					</xsl:choose>
				
			</article>
		
		</div>	
	
	</xsl:template>		
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/queryID)" />
		<xsl:variable name="locked" select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/lockForManagerUpdate = 'true' and RequestMetadata/manager = 'true'" />
	
		<div id="{$queryID}">
	
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/freeTextAlternative">
					<xsl:text> hasFreeTextAlternative</xsl:text>
				</xsl:if>
				<xsl:if test="EnableAjaxPosting"><xsl:text> enableAjaxPosting</xsl:text></xsl:if>
				<xsl:if test="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
	
			<a name="{$queryID}" />
	
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<xsl:apply-templates select="ValidationErrors/validationError"/>
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="AlternativeListDropDownQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/helpText">		
						<xsl:apply-templates select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/helpText" />
					</xsl:if>
					
				</div>
					
				<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/description" disable-output-escaping="yes" />
					</span>		
				</xsl:if>
	
				<xsl:variable name="dropDownName">
					<xsl:value-of select="'q'"/>
					<xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/queryID"/>
					<xsl:value-of select="'_alternative'"/>
				</xsl:variable>
				
				<xsl:choose>
					<xsl:when test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/Alternatives/Alternative">
					
						<fieldset>
							
							<div class="split">
		
								<xsl:variable name="selectedAlternativeID" select="AlternativeListDropDownQueryInstance/alternativeID" />
							
								<select name="{$dropDownName}" style="width: 100%;">
								
									<xsl:if test="$locked">
										<xsl:attribute name="disabled">true</xsl:attribute>
									</xsl:if>
									
									<option value=""><xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/shortDescription"/></option>
									
									<xsl:for-each select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/Alternatives/Alternative">
										<option value="{alternativeID}">
											<xsl:choose>
												<xsl:when test="requestparameters">
													<xsl:if test="requestparameters/parameter[name=$dropDownName]/value = alternativeID">
														<xsl:attribute name="selected"/>
													</xsl:if>
												</xsl:when>
												<xsl:otherwise>
													<xsl:if test="$selectedAlternativeID = alternativeID">
														<xsl:attribute name="selected" />
													</xsl:if>
												</xsl:otherwise>
											</xsl:choose>
											
											<xsl:value-of select="name" />
											
										</option>
									</xsl:for-each>
			
									<xsl:if test="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/freeTextAlternative">
									
										<option value="freeTextAlternative">
											
											<xsl:choose>
												<xsl:when test="requestparameters">
													<xsl:if test="requestparameters/parameter[name=$dropDownName]/value = 'freeTextAlternative'">
														<xsl:attribute name="selected"/>
													</xsl:if>
												</xsl:when>
												<xsl:otherwise>
													<xsl:if test="AlternativeListDropDownQueryInstance/freeTextAlternativeValue">
														<xsl:attribute name="selected" />
													</xsl:if>
												</xsl:otherwise>
											</xsl:choose>	
												
											<xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/freeTextAlternative" />
																		
										</option>
									
									</xsl:if>
									
								</select>
					
							</div>
					
							<div class="freeTextAlternativeValue hidden">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="id" select="concat($dropDownName,'Value')" />
									<xsl:with-param name="name" select="concat($dropDownName,'Value')" />
									<xsl:with-param name="value" select="AlternativeListDropDownQueryInstance/freeTextAlternativeValue" />
									<xsl:with-param name="disabled" select="disabled" />
								</xsl:call-template>
							</div>
						
						</fieldset>
					
					</xsl:when>
					<xsl:otherwise>
					
						<span class="italic html-description">
							<p class="error">
								<xsl:call-template name="replaceLineBreak">
									<xsl:with-param name="string" select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/emptyListDescription" />
								</xsl:call-template>
							</p>
						</span>
						
						<fieldset>
							
							<div class="split">
						
								<select name="{$dropDownName}" style="width: 100%;">
									
									<option value=""><xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/shortDescription"/></option>
									
								</select>
								
							</div>
							
						</fieldset>
						
					</xsl:otherwise>
				</xsl:choose>
				
			</article>
	
		</div>
		
		<script type="text/javascript">
			$(function(){
				initAlternativeListDropDownQuery(
					'<xsl:value-of select="AlternativeListDropDownQueryInstance/AlternativeListDropDownQuery/queryID" />',
					'<xsl:value-of select="$locked" />'
				);
			});
		</script>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.RequiredQuery"/>
			</strong>
		</span>
		
	</xsl:template>		
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
	
</xsl:stylesheet>