package com.nordicpeak.flowengine.alternativelist.provider;

import java.sql.Timestamp;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.enums.Order;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@XMLElement(name = "AlternativeListEvent")
public abstract class BaseAlternativeListEvent<BaseList extends BaseAlternativeList<BaseList, BaseListEvent>, BaseListEvent extends BaseAlternativeListEvent<BaseList, BaseListEvent>> extends GeneratedElementable {
	
	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer eventID;
	
	@DAOManaged
	@OrderBy(order = Order.DESC)
	@XMLElement
	protected Timestamp added;
	
	@DAOManaged(dontUpdateIfNull = true)
	@XMLElement(name = "poster")
	protected User poster;
	
	@DAOManaged
	@XMLElement
	protected String message;
	
	public BaseAlternativeListEvent() {};
	
	public BaseAlternativeListEvent(BaseList list, Timestamp added, User poster, String message) {
		super();
		
		if (list == null) {
			throw new NullPointerException("list can not be null");
		}
		
		setList(list);
		this.added = added;
		this.poster = poster;
		this.message = message;
	}
	
	public Integer getEventID() {
		
		return eventID;
	}
	
	public void setEventID(Integer eventID) {
		
		this.eventID = eventID;
	}
	
	public abstract BaseList getList();
	public abstract void setList(BaseList list);
	
	public Timestamp getAdded() {
		
		return added;
	}
	
	public void setAdded(Timestamp added) {
		
		this.added = added;
	}
	
	public User getPoster() {
		
		return poster;
	}
	
	public void setPoster(User poster) {
		
		this.poster = poster;
	}
	
	public String getMessage() {
		
		return message;
	}
	
	public void setMessage(String message) {
		
		this.message = message;
	}
	
	@Override
	public String toString() {
		
		return " AlternativeListEvent (ID: " + eventID + ", list: " + getList() + ")";
	}
	
}
