package com.nordicpeak.flowengine.queries.attributesumlimitquery;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import se.unlogic.standardutils.annotations.PopulateOnlyIfSet;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.beans.AttributeSumLimitEndpoint;
import com.nordicpeak.flowengine.queries.attributesumlimitquery.api.interfaces.LimitProvider;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;

@Table(name = "attribute_sum_limit_queries")
@XMLElement
public class AttributeSumLimitQuery extends BaseQuery implements LimitProvider{

	private static final long serialVersionUID = -842191226937409416L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate(maxLength = 255, required = true)
	@XMLElement
	private String attributeName;

	@DAOManaged(columnName = "amountLimit")
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "useEndpoint", paramValues = "false")
	@XMLElement
	private Integer limit;

	@DAOManaged
	@WebPopulate
	@PopulateOnlyIfSet(paramNames = "useEndpoint", paramValues = "false")
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date minDate;

	@DAOManaged
	@WebPopulate
	@PopulateOnlyIfSet(paramNames = "useEndpoint", paramValues = "false")
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date maxDate;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useEndpoint;
	
	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private AttributeSumLimitEndpoint endpoint;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showRemaining;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer maxRemainingAmountToShow;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String noneRemainingText;

	@DAOManaged
	@WebPopulate(maxLength = 45)
	@XMLElement
	private String limitExceededText;

	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@RequiredIfSet(paramNames = "useEndpoint", paramValues = "true")
	@XMLElement
	private String apiErrorDescription;	
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<AttributeSumLimitQueryInstance> instances;

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<AttributeSumLimitQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<AttributeSumLimitQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public Date getMinDate() {

		return minDate;
	}

	public void setMinDate(Date minDate) {

		this.minDate = minDate;
	}

	public Date getMaxDate() {

		return maxDate;
	}

	public void setMaxDate(Date maxDate) {

		this.maxDate = maxDate;
	}

	public Integer getLimit() {

		return limit;
	}

	public void setLimit(Integer limit) {

		this.limit = limit;
	}

	public boolean isShowRemaining() {

		return showRemaining;
	}

	public void setShowRemaining(boolean showRemaining) {

		this.showRemaining = showRemaining;
	}

	public Integer getMaxRemainingAmountToShow() {

		return maxRemainingAmountToShow;
	}

	public void setMaxRemainingAmountToShow(Integer maxRemainingAmountToShow) {

		this.maxRemainingAmountToShow = maxRemainingAmountToShow;
	}

	public String getNoneRemainingText() {

		return noneRemainingText;
	}

	public void setNoneRemainingText(String noneRemainingText) {

		this.noneRemainingText = noneRemainingText;
	}

	public String getLimitExceededText() {

		return limitExceededText;
	}

	public void setLimitExceededText(String limitExceededText) {

		this.limitExceededText = limitExceededText;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "AttributeSumLimitQuery (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return "AttributeSumLimitQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);

		showRemaining = xmlParser.getPrimitiveBoolean("showRemaining");
		maxRemainingAmountToShow = xmlParser.getInteger("maxRemainingAmountToShow");
		noneRemainingText = XMLValidationUtils.validateParameter("noneRemainingText", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		limitExceededText = XMLValidationUtils.validateParameter("limitExceededText", xmlParser, false, 1, 45, StringPopulator.getPopulator(), errors);

		useEndpoint = xmlParser.getPrimitiveBoolean("useEndpoint");

		if(useEndpoint) {
			
			apiErrorDescription  = XMLValidationUtils.validateParameter("apiErrorDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
			
		}else {
			
			minDate = XMLValidationUtils.validateParameter("minDate", xmlParser, false, DatePopulator.getPopulator(), errors);
			maxDate = XMLValidationUtils.validateParameter("maxDate", xmlParser, false, DatePopulator.getPopulator(), errors);
			limit = xmlParser.getInteger("limit");
		}
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public AttributeSumLimitEndpoint getEndpoint() {

		return endpoint;
	}

	public void setEndpoint(AttributeSumLimitEndpoint endpoint) {

		this.endpoint = endpoint;
	}

	
	public boolean isUseEndpoint() {
	
		return useEndpoint;
	}

	
	public void setUseEndpoint(boolean useEndpoint) {
	
		this.useEndpoint = useEndpoint;
	}

	
	public String getApiErrorDescription() {
	
		return apiErrorDescription;
	}

	
	public void setApiErrorDescription(String apiErrorDescription) {
	
		this.apiErrorDescription = apiErrorDescription;
	}

}
