<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="scripts">
		/js/attributesumlimitquery.js
	</xsl:variable>

	<xsl:template match="Document">
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues" />
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:if test="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/showRemaining = 'true' or ValidationErrors/validationError">
		
			<xsl:variable name="queryID" select="concat('query_', AttributeSumLimitQueryInstance/AttributeSumLimitQuery/queryID)" />
		
			<div class="query" id="{$queryID}">
				
				<xsl:attribute name="class">
					<xsl:text>query</xsl:text>
					<xsl:if test="AttributeSumLimitQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
				</xsl:attribute>
				
				<a name="{$queryID}" />
				
				<xsl:if test="ValidationErrors/validationError">
					<div id="{$queryID}-validationerrors" class="validationerrors">
						<div class="info-box error">
							
							<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
							
							<div class="marker" />
						</div>
					</div>
				</xsl:if>
		
				<article>
					<xsl:attribute name="class">
						<xsl:text>attributesumlimitquery</xsl:text>
						<xsl:if test="ValidationErrors/validationError"> error</xsl:if>
					</xsl:attribute>
					
					<div class="heading-wrapper">
						
						<h2>
							<xsl:value-of select="AttributeSumLimitQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
						
						<xsl:if test="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/helpText">
							<xsl:apply-templates select="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/helpText" />
						</xsl:if>
					
					</div>
	
					<xsl:if test="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/description">
						<span class="italic">
							<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
							<xsl:value-of select="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/description" disable-output-escaping="yes" />
						</span>
					</xsl:if>
					
					<xsl:if test="AttributeSumLimitQueryInstance/HasLimitProvider = 'true'">
					
						<xsl:if test="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/showRemaining = 'true' and (not(AttributeSumLimitQueryInstance/AttributeSumLimitQuery/maxRemainingAmountToShow) or AttributeSumLimitQueryInstance/AttributeSumLimitQuery/maxRemainingAmountToShow >= AttributeSumLimitQueryInstance/remainingAmount)">
							<span>
								<xsl:choose>
									<xsl:when test="AttributeSumLimitQueryInstance/remainingAmount &lt;= 0 and AttributeSumLimitQueryInstance/AttributeSumLimitQuery/noneRemainingText">
										<xsl:value-of select="AttributeSumLimitQueryInstance/AttributeSumLimitQuery/noneRemainingText" />
									</xsl:when>
									<xsl:otherwise>				
										<xsl:value-of select="$i18n.remainingAmount.part1" />
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="AttributeSumLimitQueryInstance/remainingAmount" />
										<xsl:text>&#160;</xsl:text>
										<xsl:value-of select="$i18n.remainingAmount.part2" />
									</xsl:otherwise>
								</xsl:choose>							
							</span>
						</xsl:if>					
					
					</xsl:if>
					
				</article>
			
			</div>
		
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'LimitExceeded']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:choose>
					<xsl:when test="../../AttributeSumLimitQueryInstance/AttributeSumLimitQuery/limitExceededText">
						<xsl:value-of select="../../AttributeSumLimitQueryInstance/AttributeSumLimitQuery/limitExceededText" />
					</xsl:when>
					<xsl:otherwise>				
						<xsl:value-of select="$i18n.Validation.LimitExceeded" />
					</xsl:otherwise>
				</xsl:choose>				
 			</strong>
		</span>
	
	</xsl:template>
	
		<xsl:template match="validationError[messageKey = 'NoLimiterFound']">
	
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="../../AttributeSumLimitQueryInstance/AttributeSumLimitQuery/apiErrorDescription" />			
 			</strong>
		</span>
	
	</xsl:template>
	
</xsl:stylesheet>