package com.nordicpeak.flowengine.queries.basetablequery;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.basetablequery.interfaces.TableQueryInstance;

@XMLElement
public abstract class BaseTableQueryInstance<DropAlternative extends BaseDropDownAlternative, Row extends TableRow<DropAlternative, Column, Query>, Column extends BaseTableColumn<DropAlternative, Column, Query>, Query extends BaseTableQuery<DropAlternative, Query, Column>, CellValue extends TableCellValue<DropAlternative, Query, Row, Column>> extends BaseQueryInstance implements StringValueQueryInstance, TableQueryInstance {

	private static final long serialVersionUID = -4134276993031885024L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public abstract Query getQuery();

	public abstract void setQuery(Query query);

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		setValues(null);

		if (!CollectionUtils.isEmpty(getQuery().getColumns())) {

			for (Column column : getQuery().getColumns()) {

				column.setSum(null);
			}
		}

		super.reset(attributeHandler);
	}

	public abstract List<CellValue> getValues();

	public abstract void setValues(List<CellValue> values);

	public abstract void setValue(CellValue cellValue);

	public List<CellValue> setValue(List<CellValue> values, CellValue cellValue) {

		if (values == null) {
			values = new ArrayList<>();
		}

		Iterator<CellValue> it = values.iterator();
		while (it.hasNext()) {
			CellValue cell = it.next();

			if (cell.getRow().getRowIdentifier().equals(cellValue.getRow().getRowIdentifier()) && cell.getColumn().getColumnID().equals(cellValue.getColumn().getColumnID())) {

				if (cellValue.getValue() == null) {
					it.remove();

				} else {
					cell.setValue(cellValue.getValue());
				}

				return values;
			}
		}

		values.add(cellValue);

		return values;
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	public abstract String getValue(String rowIdentifier, Integer columnID);

	public abstract CellValue getCellValue(String rowIdentifier, Integer columnID);

	public String getValue(List<CellValue> values, String rowIdentifier, Integer columnID) {

		CellValue cellValue = getCellValue(values, rowIdentifier, columnID);

		if (cellValue == null) {

			return null;
		}

		if (cellValue.getDisplayValue() != null) {

			return cellValue.getDisplayValue();
		}

		return cellValue.getValue();
	}

	public CellValue getCellValue(List<CellValue> values, String rowIdentifier, Integer columnID) {

		if (values != null) {

			for (CellValue cell : values) {

				if (cell.getRow().getRowIdentifier().equals(rowIdentifier) && cell.getColumn().getColumnID().equals(columnID)) {

					return cell;
				}
			}
		}

		return null;
	}

	@Override
	public String getStringValue() {

		if (getValues() == null) {

			return null;
		}

		StringBuilder stringBuilder = new StringBuilder();

		for (Row row : getRows()) {

			String rowName = row.getName();

			boolean first;

			if (rowName != null) {

				first = false;
				stringBuilder.append(rowName);

			} else {

				first = true;
			}

			for (Column column : getQuery().getColumns()) {

				String value = getValue(row.getRowIdentifier(), column.getColumnID());

				if (column.getColumnType() == ColumnType.DROPDOWN) {

					List<DropAlternative> availableAlternatives = column.getDropDownAlternatives();

					Integer alternativeID = NumberUtils.toInt(value);

					DropAlternative selectedAlternative = null;

					if (alternativeID != null) {

						for (DropAlternative alternative : availableAlternatives) {

							if (alternative.getAlternativeID().equals(alternativeID)) {

								selectedAlternative = alternative;
								break;
							}
						}
					}

					if (selectedAlternative != null) {

						value = selectedAlternative.getName();
					}
				}

				if (value == null) {

					value = "";
				}

				if (!first) {

					stringBuilder.append(", ");

				} else {

					first = false;
				}

				stringBuilder.append(value);
			}

			stringBuilder.append(";");
		}

		return stringBuilder.toString();
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		if (getValues() != null) {

			ArrayList<String> columnElementNames2 = new ArrayList<>(getQuery().getColumns().size());
			Map<Integer, String> columnElementNames = new HashMap<>(getQuery().getColumns().size());

			for (Column column : getQuery().getColumns()) {

				columnElementNames.put(column.getColumnID(), BaseTableQuery.generateElementName("Column", columnElementNames2));
			}

			boolean hasRequiredColumn = getQuery().hasRequiredColumn();

			for (Row row : getRows()) {

				Element rowElement = doc.createElement("Row");

				XMLUtils.appendNewElement(doc, rowElement, "ID", row.getRowID());
				XMLUtils.appendNewCDATAElement(doc, rowElement, "Name", row.getName());

				for (Column column : getQuery().getColumns()) {

					Element columnElement = doc.createElement(columnElementNames.get(column.getColumnID()));
					XMLUtils.appendNewElement(doc, columnElement, "ID", column.getColumnID());
					XMLUtils.appendNewCDATAElement(doc, columnElement, "Name", column.getLabel());

					CellValue cellValue = getCellValue(row.getRowIdentifier(), column.getColumnID());

					if (cellValue == null) {

						continue;
					}

					ColumnType columnType = column.getColumnType();

					if (columnType == ColumnType.DROPDOWN) {

						List<DropAlternative> availableAlternatives = column.getDropDownAlternatives();

						Integer alternativeID = NumberUtils.toInt(cellValue.getValue());

						DropAlternative selectedAlternative = null;

						if (alternativeID != null) {

							for (DropAlternative alternative : availableAlternatives) {

								if (alternative.getAlternativeID().equals(alternativeID)) {

									selectedAlternative = alternative;
									break;
								}
							}
						}

						if (selectedAlternative != null) {

							XMLUtils.appendNewCDATAElement(doc, columnElement, "Value", selectedAlternative.getName());
						}

					} else if (columnType == ColumnType.DROPDOWN_ALTERNATIVE_LIST || columnType == ColumnType.LIST_SEARCH) {

						if (!StringUtils.isEmpty(cellValue.getXmlValue())) {

							XMLUtils.appendNewCDATAElement(doc, columnElement, "Value", cellValue.getXmlValue());

						} else if (!StringUtils.isEmpty(cellValue.getDisplayValue())) {

							XMLUtils.appendNewCDATAElement(doc, columnElement, "Value", cellValue.getDisplayValue());

						} else {

							XMLUtils.appendNewCDATAElement(doc, columnElement, "Value", cellValue.getValue());
						}

					} else {

						XMLUtils.appendNewCDATAElement(doc, columnElement, "Value", cellValue.getValue());
					}

					if (columnElement.getChildNodes().getLength() > 2) {
						rowElement.appendChild(columnElement);
					}
				}

				if (rowElement.getChildNodes().getLength() > 0 || hasRequiredColumn && row.isRequired()) {
					element.appendChild(rowElement);
				}
			}
		}

		return element;
	}

	public abstract List<Row> getRows();

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {

		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}
}
