<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/jquery/jquery-ui.js?v=1
		/timepicker/js/jquery.timepicker.js?v=1
		/ckeditor/ckeditor.js
		/ckeditor/adapters/jquery.js
		/ckeditor/init.js
		/js/confirmpost.js
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/timepicker/css/jquery.timepicker.css
		/css/openhierarchy-jquery-ui.css
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/flowengine.tablesorter.js
		/js/flowengine.tablefilter.js
		/js/UserGroupList.js
		/js/calendaradmin.js?v=3
		/js/datatables.min.js
		/js/jquery.ui.datepicker-sv.js
		/js/culling.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/UserGroupList.css?v=1
		/css/datatables.css
		/css/culling.css
	</xsl:variable>

	<xsl:template match="Document">

		<xsl:choose>
			<xsl:when test="CalendarUserAdminExtension">
			
				<xsl:apply-templates select="CalendarUserAdminExtension" />
			
			</xsl:when>
			<xsl:otherwise>
			
				<div id="CalendarAdminModule" class="contentitem errands-wrapper">
					
					<xsl:apply-templates select="ListCalendars" />
					<xsl:apply-templates select="AddCalendar" />
					<xsl:apply-templates select="UpdateCalendar" />
					<xsl:apply-templates select="SelectImport" />
					<xsl:apply-templates select="CullBookings"/> 
				</div>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="ListCalendars">
		
		<div class="floatright marginbottom">
			<label for="name" class="bigmarginleft marginright margintop"><xsl:value-of select="$i18n.Filter" />:</label>
			<input type="text" size="20" class="filter-input" data-tableid="bookings" />
		</div>
		
		<h1>
			<xsl:value-of select="/Document/module/name" />
		</h1>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<table id="bookings" class="oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Calendar.Name" /></span>
					</th>
					<xsl:if test="../EnableProfileSupport = 'true'">
						<th>
							<span><xsl:value-of select="$i18n.Calendar.Profile" /></span>
						</th>
					</xsl:if>
					<th>
						<span><xsl:value-of select="$i18n.Calendar.Enabled" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Calendar.ActiveBookings" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Calendar.BookingSlotCount" /></span>
					</th>
					<th class="no-sort"></th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Calendars/Calendar">
						<xsl:apply-templates select="Calendars/Calendar" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoCalendars" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<article class="buttons">
			
			<a class="btn btn-green btn-right marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/add">
				<xsl:value-of select="$i18n.AddCalendar" />
			</a>
			
			<a class="btn btn-light btn-right" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/import">
				<xsl:value-of select="$i18n.ImportCalender" />
			</a>
	
		</article>
		
	</xsl:template>
	
	<xsl:template match="Calendar" mode="list">
	
		<tr>
			<td data-title="{$i18n.Calendar.Name}"><xsl:value-of select="name" /></td>
			
			<xsl:if test="../../../EnableProfileSupport = 'true'">
				<xsl:variable name="profileID" select="profileID" />
				<td data-title="{$i18n.Calendar.Profile}">
					<xsl:value-of select="../../Profiles/Profile[profileID = $profileID]/name" />
				</td>
			</xsl:if>
			
			<td data-title="{$i18n.Calendar.Enabled}">
				<xsl:choose>
					<xsl:when test="enabled = 'true'">
						<xsl:value-of select="$i18n.Yes" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No" />
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td data-title="{$i18n.Calendar.ActiveBookings}"><xsl:value-of select="activeBookings" /></td>
			<td data-title="{$i18n.Calendar.BookingSlotCount}"><xsl:value-of select="bookingSlotCount" /></td>
			<td class="link">
				<a class="btn btn-light vertical-align-middle" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/export/{calendarID}"><xsl:value-of select="$i18n.Export" /></a>
				<a class="btn btn-green vertical-align-middle marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/update/{calendarID}"><xsl:value-of select="$i18n.Update" /></a>
				<a class="btn btn-red vertical-align-middle marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/delete/{calendarID}" onclick="return confirmPost(this.href, '{$i18n.DeleteConfirm}: {name}?');"><xsl:value-of select="$i18n.Delete" /></a>
				<a class="btn btn-light vertical-align-middle marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/cullbookings/{calendarID}"><xsl:value-of select="$i18n.CullBookings" /></a> 
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="AddCalendar">
	
		<h1>
			<xsl:value-of select="$i18n.AddCalendar" />
		</h1>
		
		<form method="POST" action="{/Document/requestinfo/uri}" name="bookingForm">
			
			<xsl:call-template name="editCalendar" />
		
			<article class="buttons floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
				<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline" />
			</article>
		
		</form>
		
	</xsl:template>
	
	<xsl:template match="UpdateCalendar">
		
		<h1>
			<xsl:value-of select="$i18n.UpdateCalendar" />
		</h1>
		
		<form method="POST" action="{/Document/requestinfo/uri}" name="bookingForm">
			
			<xsl:call-template name="editCalendar"/>
		
			<article class="buttons floatright">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
				<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline" />
			</article>
		
		</form>
		
	</xsl:template>
	
	<xsl:template match="SelectImport">
			
				<h1>
					<xsl:value-of select="$i18n.ImportCalendar.title" />
					<xsl:text> </xsl:text>
					<xsl:value-of select="Calendar/name"/>
				</h1>
						
				<p>
					<xsl:value-of select="$i18n.ImportCalendar.description" />
				</p>			
			
			
	
		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}" enctype="multipart/form-data">

			<div class="floatleft full bigmarginbottom">
				
				<label for="flow" class="floatleft full">
					<xsl:value-of select="$i18n.selectCalendarFile" />
				</label>
				
				<div class="floatleft full">
					<input type="file" name="calendarImportFile" id="calendarImportFile"/>
				</div>
			</div>
			
			<div class="floatright">
				<input type="submit" value="{$i18n.ImportCalendar.submit}" />
			</div>
		
		</form>
		
	
	</xsl:template>
	
	<xsl:template name="editCalendar">
		
		<article class="clearfix">
		
			<xsl:if test="validationException/validationError">
				<div class="bigmarginbottom">
					<xsl:apply-templates select="validationException/validationError"/>
				</div>
			</xsl:if>
			
			<div class="floatleft full bigmarginbottom">
				<label for="name" class="required full nomargin">
					<xsl:value-of select="$i18n.Calendar.Name" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'name'" />
					<xsl:with-param name="id" select="'name'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'enabled'" />
					<xsl:with-param name="name" select="'enabled'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="enabled">
					<xsl:value-of select="$i18n.Calendar.Enabled" />
				</label>
			</div>
			
			<xsl:if test="../EnableProfileSupport = 'true'">
				<div class="floatleft full bigmarginbottom">
					<label for="endpointID" class="required full nomargin">
						<xsl:value-of select="$i18n.Calendar.ProfileTitle" />
					</label>

					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'profileID'" />
						<xsl:with-param name="name" select="'profileID'" />
						<xsl:with-param name="title" select="$i18n.Calendar.ProfileTitle"/>
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'profileID'" />
						<xsl:with-param name="element" select="Profiles/Profile" />
			 			<xsl:with-param name="class" select="'marginright'" />
			 			<xsl:with-param name="addEmptyOption" select="$i18n.ProfileEmptyOption" />
			 			<xsl:with-param name="selectedValue" select="Calendar/profileID" />
					</xsl:call-template>
				</div>
			</xsl:if>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2>
				<xsl:value-of select="$i18n.Settings.Users" />
			</h2>
			
			<div class="full bigmarginbottom"  id="viewModeContainer">
				<label for="viewMode" class="required full">
					<xsl:value-of select="$i18n.Calendar.ViewMode" />
				</label>
				
				<div class="alternative floatleft clearboth">
		
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'viewModeCalendar'" />
						<xsl:with-param name="name" select="'viewMode'" />
						<xsl:with-param name="value" select="'CALENDAR'" />
						<xsl:with-param name="element" select="Calendar" />
					</xsl:call-template>
					
					<xsl:text>&#x20;</xsl:text>	
					<label for="viewModeCalendar"><xsl:value-of select="$i18n.Calendar.ViewMode.Calendar" /></label>
					
				</div>
				
				<div class="alternative clearboth">
		
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'viewModeList'" />
						<xsl:with-param name="name" select="'viewMode'" />
						<xsl:with-param name="value" select="'LIST'" />
						<xsl:with-param name="element" select="Calendar" />
					</xsl:call-template>
					
					<xsl:text>&#x20;</xsl:text>
					<label for="viewModeList"><xsl:value-of select="$i18n.Calendar.ViewMode.List" /></label>
					
				</div>
			
				<div class="alternative clearboth">
		
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'viewModeToggle'" />
						<xsl:with-param name="name" select="'viewMode'" />
						<xsl:with-param name="value" select="'TOGGLE'" />
						<xsl:with-param name="element" select="Calendar" />
					</xsl:call-template>
					
					<xsl:text>&#x20;</xsl:text>	
					<label for="viewModeToggle"><xsl:value-of select="$i18n.Calendar.ViewMode.Toggle" /></label>
					
				</div>
			
				<div class="full bigmarginbottom"  id="calendarWeekViewContainer">
					<label for="calendarWeekViewContainer" class="required full">
						<xsl:value-of select="$i18n.Calendar.WeekView" />
					</label>
					
					<div class="alternative floatleft clearboth">
			
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'calendarWeekViewFalse'" />
							<xsl:with-param name="name" select="'calendarWeekView'" />
							<xsl:with-param name="value" select="'false'" />
							<xsl:with-param name="element" select="Calendar" />
						</xsl:call-template>
						
						<xsl:text>&#x20;</xsl:text>	
						<label for="calendarWeekViewFalse"><xsl:value-of select="$i18n.Calendar.WeekView.Month" /></label>
						
					</div>
					
					<div class="alternative clearboth">
			
						<xsl:call-template name="createRadio">
							<xsl:with-param name="id" select="'calendarWeekViewTrue'" />
							<xsl:with-param name="name" select="'calendarWeekView'" />
							<xsl:with-param name="value" select="'true'" />
							<xsl:with-param name="element" select="Calendar" />
						</xsl:call-template>
						
						<xsl:text>&#x20;</xsl:text>
						<label for="calendarWeekViewTrue"><xsl:value-of select="$i18n.Calendar.WeekView.Week" /></label>
						
					</div>
							
				</div>
						
			</div>

			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showManagers'" />
					<xsl:with-param name="name" select="'showManagers'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showManagers">
					<xsl:value-of select="$i18n.Calendar.ShowManagers" />
				</label>
			</div>

			<div class="floatleft full bigmarginbottom margintop">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'disableLeaving'" />
					<xsl:with-param name="id" select="'disableLeaving'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="disableLeaving">
					<xsl:value-of select="$i18n.Calendar.DisableLeaving" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'disableRebooking'" />
					<xsl:with-param name="id" select="'disableRebooking'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="disableRebooking">
					<xsl:value-of select="$i18n.Calendar.DisableRebooking" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'restrictRebookingToPreviousManager'" />
					<xsl:with-param name="id" select="'restrictRebookingToPreviousManager'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="restrictRebookingToPreviousManager">
					<xsl:value-of select="$i18n.Calendar.RestrictRebookingToPreviousManager" />
				</label>
			</div>			
			
			<div class="floatleft full bigmarginbottom margintop">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'restrictRebookingToCurrentBookingSize'" />
					<xsl:with-param name="id" select="'restrictRebookingToCurrentBookingSize'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="restrictRebookingToCurrentBookingSize" title="{$i18n.Calendar.RestrictRebookingToCurrentBookingSize.Forced}">
					<xsl:value-of select="$i18n.Calendar.RestrictRebookingToCurrentBookingSize" />
				</label>
			</div>			
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hideAddedDateFromUser'" />
					<xsl:with-param name="name" select="'hideAddedDateFromUser'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="hideAddedDateFromUser">
					<xsl:value-of select="$i18n.Calendar.HideAddedDateFromUser" />
				</label>
			</div>
						
			<div class="floatleft full bigmarginbottom margintop">
					
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showFullyBookedDaysInCalendar'"/>
					<xsl:with-param name="name" select="'showFullyBookedDaysInCalendar'"/>
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showFullyBookedDaysInCalendar">
					<xsl:value-of select="$i18n.Calendar.ShowFullyBookedDaysInCalendar" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
					
				<label for="fullyBookedCalendarDayText" class="full nomargin" title="{$i18n.Calendar.FullyBookedCalendarDayText.Title}">
					<xsl:value-of select="$i18n.Calendar.FullyBookedCalendarDayText" />
				</label>
			
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'fullyBookedCalendarDayText'"/>
					<xsl:with-param name="name" select="'fullyBookedCalendarDayText'"/>
					<xsl:with-param name="element" select="Calendar"/>
					<xsl:with-param name="placeholder" select="$i18n.Calendar.FullyBookedDayPlaceholderText"/>
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
					
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showFullyBookedSlotsInList'"/>
					<xsl:with-param name="name" select="'showFullyBookedSlotsInList'"/>
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showFullyBookedSlotsInList">
					<xsl:value-of select="$i18n.Calendar.ShowFullyBookedSlotsInList" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
					
				<label for="fullyBookedListSlotText" class="full nomargin" title="{$i18n.Calendar.FullyBookedListSlotText.Title}">
					<xsl:value-of select="$i18n.Calendar.FullyBookedListSlotText" />
				</label>
			
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'fullyBookedListSlotText'"/>
					<xsl:with-param name="name" select="'fullyBookedListSlotText'"/>
					<xsl:with-param name="element" select="Calendar"/>
					<xsl:with-param name="placeholder" select="$i18n.Calendar.FullyBookedListSlotPlaceholderText"/>
				</xsl:call-template>
			</div>
						
			<xsl:call-template name="hideBookingsPerWeekday" />
									
			<div class="floatleft full bigmarginbottom margintop">
					
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showNotBookableSlots'"/>
					<xsl:with-param name="name" select="'showNotBookableSlots'"/>
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showNotBookableSlots">
					<xsl:value-of select="$i18n.Calendar.ShowNotBookableSlots" />
				</label>
			</div>
			


			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2>
				<xsl:value-of select="$i18n.Settings.Managers" />
			</h2>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showBookings'" />
					<xsl:with-param name="name" select="'showBookings'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showBookings">
					<xsl:value-of select="$i18n.Calendar.ShowBookings" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'requireConfirmation'" />
					<xsl:with-param name="name" select="'requireConfirmation'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="requireConfirmation">
					<xsl:value-of select="$i18n.Calendar.RequireConfirmation" />
				</label>
			</div>			
			
			
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2>
				<xsl:value-of select="$i18n.Settings.Global" />
			</h2>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowMultipleBookings'" />
					<xsl:with-param name="name" select="'allowMultipleBookings'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="allowMultipleBookings">
					<xsl:value-of select="$i18n.Calendar.AllowMultipleBookings" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowDuplicateBookings'" />
					<xsl:with-param name="name" select="'allowDuplicateBookings'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="allowDuplicateBookings">
					<xsl:value-of select="$i18n.Calendar.AllowDuplicateBookings" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'bookingSizeSelectorEnabled'" />
					<xsl:with-param name="name" select="'bookingSizeSelectorEnabled'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="bookingSizeSelectorEnabled">
					<xsl:value-of select="$i18n.Calendar.BookingSizeSelectorEnabled" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowUserBookingSizeSelection'" />
					<xsl:with-param name="name" select="'allowUserBookingSizeSelection'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="allowUserBookingSizeSelection">
					<xsl:value-of select="$i18n.Calendar.AllowUserBookingSizeSelection" />
				</label>
			</div>
						
			<div class="floatleft full bigmarginbottom margintop">
		
				<label class="full nomargin" for="userBookingMaxSize">
					<xsl:value-of select="$i18n.Calendar.UserBookingMaxSize" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'userBookingMaxSize'"/>
					<xsl:with-param name="name" select="'userBookingMaxSize'"/>
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showDuration'" />
					<xsl:with-param name="name" select="'showDuration'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showDuration">
					<xsl:value-of select="$i18n.Calendar.ShowDuration" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'allowPreliminaryBookings'" />
					<xsl:with-param name="name" select="'allowPreliminaryBookings'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="allowPreliminaryBookings">
					<xsl:value-of select="$i18n.Calendar.AllowPreliminaryBookings" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">
		
				<label class="required full nomargin" for="hoursPreliminaryBookingsAreValidFor">
					<xsl:value-of select="$i18n.Calendar.HoursPreliminaryBookingsAreValidFor" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'hoursPreliminaryBookingsAreValidFor'"/>
						<xsl:with-param name="name" select="'hoursPreliminaryBookingsAreValidFor'"/>
						<xsl:with-param name="element" select="Calendar" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'showFreeSlotsAsBoolean'" />
					<xsl:with-param name="name" select="'showFreeSlotsAsBoolean'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="showFreeSlotsAsBoolean" title="{$i18n.Calendar.ShowFreeSlotsAsBoolean.Title}">
					<xsl:value-of select="$i18n.Calendar.ShowFreeSlotsAsBoolean" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hideDisabledManagersInFilter'" />
					<xsl:with-param name="name" select="'hideDisabledManagersInFilter'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="hideDisabledManagersInFilter">
					<xsl:value-of select="$i18n.Calendar.HideDisabledManagersInFilter" />
				</label>
			</div>
			
			<div class="floatleft full bigmarginbottom margintop">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'hideExternalMessages'" />
					<xsl:with-param name="name" select="'hideExternalMessages'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="hideExternalMessages">
					<xsl:value-of select="$i18n.Calendar.HideExternalMessages" />
				</label>
			</div>

			<div class="floatleft full bigmarginbottom">
				<label for="newExternalMessagesAllowedDays" class="full nomargin">
					<xsl:value-of select="$i18n.newExternalMessagesAllowedDays" />
				</label>
				
				<span class="tiny"><xsl:value-of select="$i18n.newExternalMessagesAllowedDaysInfo" /></span>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'newExternalMessagesAllowedDays'" />
					<xsl:with-param name="id" select="'newExternalMessagesAllowedDays'" />
					<xsl:with-param name="size" select="10" />
					<xsl:with-param name="value" select="Calendar/newExternalMessagesAllowedDays" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="price" class="required full nomargin">
					<xsl:value-of select="$i18n.Calendar.Price" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'price'" />
					<xsl:with-param name="id" select="'price'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="full bigmarginbottom">
				<label for="defaultStartTime" class="full">
					<xsl:value-of select="$i18n.Calendar.DefaultStartTime" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'defaultStartTime'" />
					<xsl:with-param name="name" select="'defaultStartTime'" />
					<xsl:with-param name="type" select="'time'" />
					<xsl:with-param name="size" select="10" />
					<xsl:with-param name="element" select="Calendar"/>
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="defaultSlotLength" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.DefaultSlotLength" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'defaultSlotLength'" />
					<xsl:with-param name="id" select="'defaultSlotLength'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
			
				<label for="hoursBeforeBookingLocked" class="required full nomargin">
					<xsl:value-of select="$i18n.Calendar.HoursBeforeBookingLocked" />
				</label>

				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'hoursBeforeBookingLocked'" />
					<xsl:with-param name="name" select="'hoursBeforeBookingLocked'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
			
				<label for="minVisibleDaysRebooking" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.MinVisibleDaysRebooking" />
				</label>

				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'minVisibleDaysRebooking'" />
					<xsl:with-param name="name" select="'minVisibleDaysRebooking'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="description" class="required full nomargin">
					<xsl:value-of select="$i18n.Calendar.Description" />
				</label>
				
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="name" select="'description'" />
					<xsl:with-param name="id" select="'description'" />
					<xsl:with-param name="rows" select="4" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="bookingSlotDefaultDescription" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.BookingSlotDefaultDescription" />
				</label>
				
				<xsl:call-template name="createTextArea">
					<xsl:with-param name="name" select="'bookingSlotDefaultDescription'" />
					<xsl:with-param name="id" select="'bookingSlotDefaultDescription'" />
					<xsl:with-param name="rows" select="4" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="bookingSlotDefaultLocation" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.BookingSlotDefaultLocation" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'bookingSlotDefaultLocation'" />
					<xsl:with-param name="name" select="'bookingSlotDefaultLocation'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
			</div>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.Calendar.Manager.Users"/></h2>
			
			<xsl:call-template name="UserList">
				<xsl:with-param name="connectorURL">
					<xsl:value-of select="/Document/requestinfo/currentURI"/>
					<xsl:text>/</xsl:text>
					<xsl:value-of select="/Document/module/alias"/>
					<xsl:text>/users</xsl:text>
				</xsl:with-param>
				<xsl:with-param name="name" select="'user'"/>
				<xsl:with-param name="users" select="users" />
				<xsl:with-param name="document" select="/Document" />
				<xsl:with-param name="showUsername" select="true()" />
			</xsl:call-template>
			
			<br/>
			
			<h2><xsl:value-of select="$i18n.Calendar.Manager.Groups"/></h2>
			
			<xsl:call-template name="GroupList">
				<xsl:with-param name="connectorURL">
					<xsl:value-of select="/Document/requestinfo/currentURI"/>
					<xsl:text>/</xsl:text>
					<xsl:value-of select="/Document/module/alias"/>
					<xsl:text>/groups</xsl:text>
				</xsl:with-param>
				<xsl:with-param name="name" select="'group'"/>
				<xsl:with-param name="groups" select="groups" />
				<xsl:with-param name="document" select="/Document" />
			</xsl:call-template>
			
			<br/>
						
			<h2><xsl:value-of select="$i18n.Calendar.Admin.Groups"/></h2>
			
			<xsl:call-template name="GroupList">
				<xsl:with-param name="connectorURL">
					<xsl:value-of select="/Document/requestinfo/currentURI"/>
					<xsl:text>/</xsl:text>
					<xsl:value-of select="/Document/module/alias"/>
					<xsl:text>/groups</xsl:text>
				</xsl:with-param>
				<xsl:with-param name="name" select="'adminGroup'"/>
				<xsl:with-param name="groups" select="adminGroups" />
				<xsl:with-param name="document" select="/Document" />
			</xsl:call-template>		

			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.UserNotifications"/></h2>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendEnteredUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendEnteredUserSMS"/>
				<xsl:with-param name="messageField" select="'manualEntranceSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ManualEntranceSMS"/>
				<xsl:with-param name="addManagerToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendExternalMessageReceivedUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExternalMessageReceivedUserSMS"/>
				<xsl:with-param name="messageField" select="'externalMessageReceivedUserSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ExternalMessageReceivedUserSMS"/>
				<xsl:with-param name="addManagerToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendManualReminderSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Notifications.SendManualReminderSMS"/>
				<xsl:with-param name="messageField" select="'manualReminderSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ManualReminderSMS"/>
				<xsl:with-param name="addManagerToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendRebookedUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendRebookedUserSMS"/>
				<xsl:with-param name="messageField" select="'rebookedUserSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.RebookedUserSMS"/>
				<xsl:with-param name="addBookingSlotOldToUserTagsTable" select="'true'"/>
				<xsl:with-param name="addBookingSlotOldManagersToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendExitUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExitUserSMS"/>
				<xsl:with-param name="messageField" select="'userExitSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserExitSMS"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendBookingSlotRebookedUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendBookingSlotRebookedUserSMS"/>
				<xsl:with-param name="messageField" select="'bookingSlotRebookedUserSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.BookingSlotRebookedUserSMS"/>
				<xsl:with-param name="infoText" select="$i18n.Notifications.BookingSlotSMSSentToMultipleUsers"/>
				<xsl:with-param name="addBookingSlotOldToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'sendBookingSlotDeletedUserSMS'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendBookingSlotDeletedUserSMS"/>
				<xsl:with-param name="messageField" select="'bookingSlotDeletedUserSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.BookingSlotDeletedUserSMS"/>
				<xsl:with-param name="infoText" select="$i18n.Notifications.BookingSlotSMSSentToMultipleUsers"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationSMS">
				<xsl:with-param name="toggleField" select="'useReminder'"/>
				<xsl:with-param name="toggleFieldChecked" select="Calendar/reminderMinutes"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendReminderSMS"/>
				<xsl:with-param name="messageField" select="'userReminderSMS'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserReminderSMS"/>
				<xsl:with-param name="valueField" select="'reminderMinutes'"/>
				<xsl:with-param name="valueElement" select="Calendar/reminderMinutes"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendEnteredUserEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendEnteredUserEmail"/>
				<xsl:with-param name="subjectField" select="'manualEntranceEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ManualEntranceEmailSubject"/>
				<xsl:with-param name="messageField" select="'manualEntranceEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ManualEntranceEmailMessage"/>
				<xsl:with-param name="addManagerToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendExternalMessageReceivedUserEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExternalMessageReceivedUserEmail"/>
				<xsl:with-param name="subjectField" select="'externalMessageReceivedUserEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ExternalMessageReceivedUserEmailSubject"/>
				<xsl:with-param name="messageField" select="'externalMessageReceivedUserEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ExternalMessageReceivedUserEmailMessage"/>
				<xsl:with-param name="addManagerToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleLabel" select="$i18n.Notifications.SendManualReminderEmail"/>
				<xsl:with-param name="subjectField" select="'manualReminderEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ManualReminderEmailSubject"/>
				<xsl:with-param name="messageField" select="'manualReminderEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ManualReminderEmailMessage"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendRebookedUserEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendRebookedUserEmail"/>
				<xsl:with-param name="subjectField" select="'rebookedUserEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.RebookedUserEmailSubject"/>
				<xsl:with-param name="messageField" select="'rebookedUserEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.RebookedUserEmailMessage"/>
				<xsl:with-param name="addBookingSlotOldToUserTagsTable" select="'true'"/>
				<xsl:with-param name="addBookingSlotOldManagersToUserTagsTable" select="'true'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExitUserEmail"/>
				<xsl:with-param name="subjectField" select="'userExitEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.UserExitEmailSubject"/>
				<xsl:with-param name="messageField" select="'userExitEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserExitEmailMessage"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendBookingSlotRebookedUserEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendBookingSlotRebookedUserEmail"/>
				<xsl:with-param name="subjectField" select="'bookingSlotRebookedUserEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.BookingSlotRebookedUserEmailSubject"/>
				<xsl:with-param name="messageField" select="'bookingSlotRebookedUserEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.BookingSlotRebookedUserEmailMessage"/>
				<xsl:with-param name="infoText" select="$i18n.Notifications.BookingSlotEmailSentToMultipleUsers"/>
				<xsl:with-param name="addBookingSlotOldToUserTagsTable" select="'true'"/>
			</xsl:call-template>

			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendBookingSlotDeletedUserEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendBookingSlotDeletedUserEmail"/>
				<xsl:with-param name="subjectField" select="'bookingSlotDeletedUserEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.BookingSlotDeletedUserEmailSubject"/>
				<xsl:with-param name="messageField" select="'bookingSlotDeletedUserEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.BookingSlotDeletedUserEmailMessage"/>
				<xsl:with-param name="infoText" select="$i18n.Notifications.BookingSlotEmailSentToMultipleUsers"/>
			</xsl:call-template>
			
			<div class="floatleft full bigmarginbottom bigmargintop">
				<label for="emailSenderName" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.EmailSenderName" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'emailSenderName'" />
					<xsl:with-param name="id" select="'emailSenderName'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="value" select="Calendar/emailSenderName" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">
				<label for="emailSenderAddress" class="full nomargin">
					<xsl:value-of select="$i18n.Calendar.EmailSenderAddress" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'emailSenderAddress'" />
					<xsl:with-param name="id" select="'emailSenderAddress'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="value" select="Calendar/emailSenderAddress" />
				</xsl:call-template>
			</div>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.ManagerNotifications"/></h2>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendEnteredManagerEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendEnteredManagerEmail"/>
				<xsl:with-param name="subjectField" select="'manualEntranceManagerEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ManualEntranceManagerEmailSubject"/>
				<xsl:with-param name="messageField" select="'manualEntranceManagerEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ManualEntranceManagerEmailMessage"/>
				<xsl:with-param name="tagsTable" select="'manager'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendExternalMessageReceivedManagerEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExternalMessageReceivedManagerEmail"/>
				<xsl:with-param name="subjectField" select="'externalMessageReceivedManagerEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ExternalMessageReceivedManagerEmailSubject"/>
				<xsl:with-param name="messageField" select="'externalMessageReceivedManagerEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ExternalMessageReceivedManagerEmailMessage"/>
				<xsl:with-param name="tagsTable" select="'manager'"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendUserRebookedManagerEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.sendRebookedManagerEmail"/>
				<xsl:with-param name="subjectField" select="'userRebookedManagerEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.UserRebookedManagerEmailSubject"/>
				<xsl:with-param name="messageField" select="'userRebookedManagerEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserRebookedManagerEmailMessage"/>
				<xsl:with-param name="title" select="$i18n.Calendar.sendRebookedManagerEmail.Title"/>
				<xsl:with-param name="tagsTable" select="'manager'"/>
			</xsl:call-template>

			<xsl:if test="../EnableRebookingNotificationsOnlyToSlotManagers = 'true'">
				<div class="floatleft full bigmarginbottom margintop">
				
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'sendUserRebookedManagerEmailOnlyToBookingSlotManagers'" />
						<xsl:with-param name="name" select="'sendUserRebookedManagerEmailOnlyToBookingSlotManagers'" />
						<xsl:with-param name="element" select="Calendar" />
						<xsl:with-param name="title" select="$i18n.Calendar.SendUserRebookedManagerEmailOnlyToBookingSlotManagers.Title" />
					</xsl:call-template>
					
					<label class="nomargin" for="sendUserRebookedManagerEmailOnlyToBookingSlotManagers">
						<xsl:value-of select="$i18n.Calendar.SendUserRebookedManagerEmailOnlyToBookingSlotManagers" />
					</label>
					
				</div>
			</xsl:if>
						
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendExitManagerEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.sendExitManagerEmail"/>
				<xsl:with-param name="subjectField" select="'userExitManagerEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.UserExitManagerEmailSubject"/>
				<xsl:with-param name="messageField" select="'userExitManagerEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserExitManagerEmailMessage"/>
				<xsl:with-param name="tagsTable" select="'manager'"/>
			</xsl:call-template>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2><xsl:value-of select="$i18n.GlobalNotifications"/></h2>
						
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendExternalMessageReceivedGlobalEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExternalMessageReceivedGlobalEmail"/>
				<xsl:with-param name="subjectField" select="'externalMessageReceivedGlobalEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.ExternalMessageReceivedGlobalEmailSubject"/>
				<xsl:with-param name="messageField" select="'externalMessageReceivedGlobalEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.ExternalMessageReceivedGlobalEmailMessage"/>
				<xsl:with-param name="addressesField" select="'externalMessageReceivedGlobalEmailAddresses'"/>
				<xsl:with-param name="addressesLabel" select="$i18n.Calendar.ExternalMessageReceivedGlobalEmailAddresses"/>
				<xsl:with-param name="addressesElement" select="Calendar/ExternalMessageReceivedGlobalEmailAddresses/address"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendRebookedGlobalEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendRebookedGlobalEmail"/>
				<xsl:with-param name="subjectField" select="'rebookedGlobalEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.RebookedGlobalEmailSubject"/>
				<xsl:with-param name="messageField" select="'rebookedGlobalEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.RebookedGlobalEmailMessage"/>
				<xsl:with-param name="addressesField" select="'rebookedGlobalEmailAddresses'"/>
				<xsl:with-param name="addressesLabel" select="$i18n.Calendar.RebookedGlobalEmailAddresses"/>
				<xsl:with-param name="addressesElement" select="Calendar/RebookedGlobalEmailAddresses/address"/>
			</xsl:call-template>
			
			<xsl:call-template name="notificationEmail">
				<xsl:with-param name="toggleField" select="'sendExitGlobalEmail'"/>
				<xsl:with-param name="toggleLabel" select="$i18n.Calendar.SendExitGlobalEmail"/>
				<xsl:with-param name="subjectField" select="'userExitGlobalEmailSubject'"/>
				<xsl:with-param name="subjectLabel" select="$i18n.Notifications.UserExitGlobalEmailSubject"/>
				<xsl:with-param name="messageField" select="'userExitGlobalEmailMessage'"/>
				<xsl:with-param name="messageLabel" select="$i18n.Notifications.UserExitGlobalEmailMessage"/>
				<xsl:with-param name="addressesField" select="'exitGlobalEmailAddresses'"/>
				<xsl:with-param name="addressesLabel" select="$i18n.Calendar.ExitGlobalEmailAddresses"/>
				<xsl:with-param name="addressesElement" select="Calendar/ExitGlobalEmailAddresses/address"/>
			</xsl:call-template>
			
			<div class="clearboth marginbottom">
				<br/>
			</div>
			
			<h2 class="floatleft full bigmargintop"><xsl:value-of select="$i18n.Settings.Advanced" /></h2>

			<div class="floatleft full bigmarginbottom">
			
				<label for="exportAttributes" class="floatleft full nomargin">
					<xsl:value-of select="$i18n.Calendar.ExportAttributes" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'exportAttributes'"/>
						<xsl:with-param name="name" select="'exportAttributes'"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="Calendar/ExportAttributes/exportAttribute" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="'useCalendarForInternalUsers'" />
					<xsl:with-param name="id" select="'useCalendarForInternalUsers'" />
					<xsl:with-param name="class" select="'bigmarginbottom'" />
					<xsl:with-param name="element" select="Calendar" />
				</xsl:call-template>
				
				<label class="nomargin" for="useCalendarForInternalUsers">
					<xsl:value-of select="$i18n.Calendar.UseCalendarForInternalUsers" />
<!-- 					TODO: Lgg eventuellt till information i stil med "(ls manualen fr en mer detaljerad beskrivning innan denna instllning anvnds)" -->
				</label>
	
			</div>

			<xsl:if test="ExtensionFragments/ViewFragment">
			
				<xsl:apply-templates select="ExtensionFragments/ViewFragment"/>

				<div class="clearboth marginbottom">
					<br/>
				</div>	
				
			</xsl:if>
			
		</article>
		
		<script type="text/javascript">
			$(function(){
				$("#defaultStartTime").timePicker({
					startTime: "00:00",
					endTime: "23:59",
	    			step: 15
	    		});
			});
		</script>
		
	</xsl:template>
	
	<xsl:template name="notificationSMS">
		<xsl:param name="toggleField" select="null" />
		<xsl:param name="toggleFieldChecked" select="null" />
		<xsl:param name="toggleLabel" />
		<xsl:param name="messageField" />
		<xsl:param name="messageLabel" />
		<xsl:param name="valueField" select="null" />
		<xsl:param name="valueElement" select="null" />		
		<xsl:param name="infoText" select="null" />
		<xsl:param name="tagsTable" select="null" />		
		<xsl:param name="addManagerToUserTagsTable" select="null" />
		<xsl:param name="addBookingSlotOldToUserTagsTable" select="null" />		
		<xsl:param name="addBookingSlotOldManagersToUserTagsTable" select="null" />		
			
		<xsl:call-template name="notificationEmail">
			<xsl:with-param name="toggleField" select="$toggleField" />
			<xsl:with-param name="toggleFieldChecked" select="$toggleFieldChecked" />
			<xsl:with-param name="toggleLabel" select="$toggleLabel" />
			<xsl:with-param name="messageField" select="$messageField" />
			<xsl:with-param name="messageFieldCKEditor" select="false()" />
			<xsl:with-param name="messageLabel" select="$messageLabel" />
			<xsl:with-param name="valueField" select="$valueField" />
			<xsl:with-param name="valueElement" select="$valueElement" />
			<xsl:with-param name="infoText" select="$infoText" />
			<xsl:with-param name="tagsTable" select="$tagsTable" />
			<xsl:with-param name="tagsTableForSMS" select="'true'" />
			<xsl:with-param name="addManagerToUserTagsTable" select="$addManagerToUserTagsTable" />
			<xsl:with-param name="addBookingSlotOldToUserTagsTable" select="$addBookingSlotOldToUserTagsTable" />		
			<xsl:with-param name="addBookingSlotOldManagersToUserTagsTable" select="$addBookingSlotOldManagersToUserTagsTable" />		
		</xsl:call-template>
	
	</xsl:template>
	
	<xsl:template match="CullBookings">
		
		<script>
			$(function() {
				$( "#fromDate" ).datepicker({
					showOn: "button",
					buttonImage: '<xsl:value-of select="$imgPath"/>/calendar_grid.png',
					buttonImageOnly: true,
					buttonText: '<xsl:value-of select="$i18n.FromDate"/>',
					changeMonth: true,
            		changeYear: true,
            		showWeek: true,
            		maxDate: 0,
				});
				
				$( "#toDate" ).datepicker({
					showOn: "button",
					buttonImage: '<xsl:value-of select="$imgPath"/>/calendar_grid.png',
					buttonImageOnly: true,
					buttonText: '<xsl:value-of select="$i18n.ToDate"/>',
					changeMonth: true,
            		changeYear: true,
            		showWeek: true,
            		maxDate: 0,
				});
			});
			
			i18nCullingModule.EMPTY_TABLE = '<xsl:value-of select="$i18n.EmptyTable"/>';
			i18nCullingModule.NEXT = '<xsl:value-of select="$i18n.Next"/>';
			i18nCullingModule.PREVIOUS =  '<xsl:value-of select="$i18n.Previous"/>';
			i18nCullingModule.YES = '<xsl:value-of select="$i18n.Yes"/>';
			i18nCullingModule.NO = '<xsl:value-of select="$i18n.No"/>';
			i18nCullingModule.CONFIRM_DELETE_PART1 = '<xsl:value-of select="$i18n.ConfirmDelete.Part1"/>';
			i18nCullingModule.CONFIRM_DELETE_PART2 = '<xsl:value-of select="$i18n.ConfirmDelete.Part2"/>';
			i18nCullingModule.LENGTH_MENU = '<xsl:value-of select="$i18n.LengthMenu"/>';
		</script>
	
		<div id="CullingDiv">
			<div>
				<h1>
					<xsl:value-of select="$i18n.CullBookingsTitle"/><xsl:value-of select="calendarName"/>
				</h1>
				
				<div id="validationerrors"></div>
				
				
				
				<fieldset class="floatleft full marginbottom">
					<legend><xsl:value-of select="$i18n.Period" /></legend>
					
					<div class="floatleft full bigmarginbottom">
						<div class="floatleft">
							<label for="fromDate" class="floatleft">
								<xsl:value-of select="$i18n.FromDate" />
							</label>
							
							<div class="floatleft clearboth">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'fromDate'" />
									<xsl:with-param name="id" select="'fromDate'" />
									<xsl:with-param name="size" select="10" />
									<xsl:with-param name="class" select="'marginright'" />
								</xsl:call-template>
							</div>
						</div>
						
						<div class="bigmarginleft floatleft">
							<label for="toDate" class="floatleft">
								<xsl:value-of select="$i18n.ToDate" />
							</label>
							
							<div class="floatleft clearboth">
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="'toDate'" />
									<xsl:with-param name="id" select="'toDate'" />
									<xsl:with-param name="size" select="10" />
									<xsl:with-param name="class" select="'marginright'" />
								</xsl:call-template>
							</div>
						</div>
					</div>
				</fieldset>
				
				<div class="floatright clearboth bigmargintop">
					<input type="button" id="showBookings" value="{$i18n.ShowBookings}" data-dateerror="{$i18n.DateError}" data-dateformaterror="{$i18n.DateFormatError}" data-unknownerror="{$i18n.UnknownError}" data-fetchurl="{/Document/ModuleURI}/getallpassedbookingslots/{calendarID}" />
				</div>
			</div>
			
			<div id="bookings-wrapper">
				<h1>
					<xsl:value-of select="$i18n.Found"/>
					<xsl:text> </xsl:text>
					<span id="bookings-count">0</span>
					<xsl:text> </xsl:text>
					<xsl:value-of select="$i18n.Instances"/>
				</h1>
				
				<table id="bookings-table" class="full stripe display">
					<thead>
						<tr>
							<th><input type="checkbox" id="select-all"/></th>
							<th><xsl:value-of select="$i18n.FromDate"/></th>
							<th><xsl:value-of select="$i18n.ToDate"/></th>
							<th class="twenty"><xsl:value-of select="$i18n.NrOfBookings"/></th>
							<th><xsl:value-of select="$i18n.BookingSlotDescription"/></th>
							<th class="hidden">BokningsId</th>
						</tr>
					</thead>
				</table>
				<div class="floatright clearboth bigmargintop">
					<a href="{/Document/ModuleURI}/removeselectedbookingslots/{calendarID}" onclick="return false;" class="btn btn-light" disabled="disabled" id="deleteBookings"><xsl:value-of select="$i18n.CullBookings"/></a>
				</div>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template name="notificationEmail">
		<xsl:param name="toggleField" select="null"/>
		<xsl:param name="toggleFieldChecked" select="null"/>
		<xsl:param name="toggleLabel" />
		<xsl:param name="subjectField" select="null" />
		<xsl:param name="subjectLabel" select="null" />
		<xsl:param name="messageField" />
		<xsl:param name="messageFieldCKEditor" select="true()" />
		<xsl:param name="messageLabel" />
		<xsl:param name="valueField" select="null" />
		<xsl:param name="valueElement" select="null" />
		<xsl:param name="title" select="null" />
		<xsl:param name="addressesField" select="null" />
		<xsl:param name="addressesLabel" select="null" />
		<xsl:param name="addressesElement" select="null" />
		<xsl:param name="infoText" select="null" />
		<xsl:param name="tagsTable" select="'user'" />
		<xsl:param name="tagsTableForSMS" select="'false'" />
		<xsl:param name="addManagerToUserTagsTable" select="'false'" />
		<xsl:param name="addBookingSlotOldToUserTagsTable" select="'false'" />
		<xsl:param name="addBookingSlotOldManagersToUserTagsTable" select="'false'" />
		
		<div class="notification">
		
			<div class="floatleft full bigmarginbottom margintop">
			
				<div class="floatleft">
					
					<xsl:choose>
						<xsl:when test="$toggleField and $toggleFieldChecked">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="name" select="$toggleField" />
								<xsl:with-param name="id" select="$toggleField" />
								<xsl:with-param name="checked" select="'true'" />
								<xsl:with-param name="class">
									<xsl:if test="$valueField">notification-value-toggle</xsl:if>
								</xsl:with-param>
								<xsl:with-param name="title" select="$title" />								
							</xsl:call-template>
						</xsl:when>
						<xsl:when test="$toggleField">
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="name" select="$toggleField" />
								<xsl:with-param name="id" select="$toggleField" />
								<xsl:with-param name="element" select="Calendar" />
								<xsl:with-param name="title" select="$title" />								
								<xsl:with-param name="class">
									<xsl:if test="$valueField">notification-value-toggle</xsl:if>
								</xsl:with-param>
							</xsl:call-template>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="createCheckbox">
								<xsl:with-param name="checked" select="'true'" />
								<xsl:with-param name="disabled" select="'true'" />
								<xsl:with-param name="title" select="$title" />								
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
				
					<label class="nomargin" for="{$toggleField}">
						<xsl:value-of select="$toggleLabel" />
					</label>
					<xsl:text>&#160;</xsl:text>
					<span class="tiny"><a class="notification-text-toggle"><xsl:value-of select="$i18n.ToggleTexts" /></a></span>
				</div>
				
				<xsl:if test="$valueField">
								
					<div class="notification-value floatleft full">
						<xsl:call-template name="createTextField">
							<xsl:with-param name="name" select="$valueField" />
							<xsl:with-param name="id" select="$valueField" />
							<xsl:with-param name="size" select="40" />
							<xsl:with-param name="value" select="$valueElement" />
						</xsl:call-template>
					</div>
				
				</xsl:if>
				
			</div>
			
			<div class="notification-text floatleft full">
			
				<xsl:choose>
					<xsl:when test="$messageField">
						<xsl:if test="not(validationException/validationError/fieldName = $subjectField) and not(validationException/validationError/fieldName = $messageField)">
							<xsl:attribute name="class">notification-text floatleft full hidden</xsl:attribute>
						</xsl:if>
					</xsl:when>
					<xsl:otherwise>
						<xsl:if test="not(validationException/validationError/fieldName = $subjectField)">
							<xsl:attribute name="class">notification-text floatleft full hidden</xsl:attribute>
						</xsl:if>
					</xsl:otherwise>
				</xsl:choose>
			
				<xsl:if test="$infoText">
					<p class="nomargin marginbottom">
						<xsl:value-of select="$infoText" />
					</p>
				</xsl:if>
				
				<xsl:if test="$subjectField and $subjectLabel">
				
					<div class="floatleft full bigmarginbottom">
				
						<label class="nomargin" for="{$subjectField}">
							<xsl:value-of select="$subjectLabel" />
						</label>
				
						<div class="floatleft full">
							<xsl:call-template name="createTextField">
								<xsl:with-param name="id" select="$subjectField"/>
								<xsl:with-param name="name" select="$subjectField"/>
								<xsl:with-param name="title" select="$subjectLabel"/>
								<xsl:with-param name="element" select="Calendar" />
							</xsl:call-template>
						</div>

					</div>
					
				</xsl:if>
				
				<div class="floatleft full bigmarginbottom">
					
					<label for="{$messageField}" class="floatleft full">
						<xsl:value-of select="$messageLabel" />
					</label>
					
					<xsl:variable name="class">
						<xsl:if test="$messageFieldCKEditor">ckeditor</xsl:if>
					</xsl:variable>
					
					<div class="floatleft full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="$messageField"/>
							<xsl:with-param name="name" select="$messageField"/>
							<xsl:with-param name="class" select="$class"/>
							<xsl:with-param name="element" select="Calendar" />
						</xsl:call-template>
					</div>
					
					<xsl:choose>
						<xsl:when test="$tagsTable = 'manager'">
							<xsl:call-template name="addManagerTagsTable"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:call-template name="addUserTagsTable">
								<xsl:with-param name="withManager" select="$addManagerToUserTagsTable" />
								<xsl:with-param name="withBookingSlotOld" select="$addBookingSlotOldToUserTagsTable" />
								<xsl:with-param name="withBookingSlotOldManagers" select="$addBookingSlotOldManagersToUserTagsTable" />
								<xsl:with-param name="SMS" select="$tagsTableForSMS" />
							</xsl:call-template>
						</xsl:otherwise>
					</xsl:choose>
					
				</div>
				
			</div>
			
			<xsl:if test="$addressesField and $addressesLabel">
			
				<div class="floatleft full bigmarginbottom">
				
					<label for="{$addressesField}" class="floatleft full nomargin">
						<xsl:value-of select="$addressesLabel" />
					</label>
					
					<div class="floatleft full">
						<xsl:call-template name="createTextArea">
							<xsl:with-param name="id" select="$addressesField"/>
							<xsl:with-param name="name" select="$addressesField"/>
							<xsl:with-param name="rows" select="5"/>
							<xsl:with-param name="separateListValues" select="'true'"/>
							<xsl:with-param name="element" select="$addressesElement" />
						</xsl:call-template>
					</div>
					
				</div>		
					
			</xsl:if>
			
		</div>
	
	</xsl:template>
	
	<xsl:template name="hideBookingsPerWeekday">
	
		<div class="floatleft full bigmarginbottom margintop">
				
			<xsl:call-template name="createCheckbox">
				<xsl:with-param name="id" select="'hideBookingsPerWeekday'"/>
				<xsl:with-param name="name" select="'hideBookingsPerWeekday'"/>
				<xsl:with-param name="element" select="Calendar" />
			</xsl:call-template>
			
			<label class="nomargin" for="hideBookingsPerWeekday">
				<xsl:value-of select="$i18n.Calendar.HideBookingsPerWeekday" />
			</label>
		</div>
					
		<div id="weekdaysWithHiddenBookingsContainer" class="floatleft full bigmarginbottom">
		
			<div class="marginbottom">
				<xsl:value-of select="$i18n.Calendar.HideBookingsPerWeekday.Info" />
			</div>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'MONDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Monday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'TUESDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Tuesday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'WEDNESDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Wednesday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'THURSDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Thursday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'FRIDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Friday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'SATURDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Saturday" />
			</xsl:call-template>
			
			<xsl:call-template name="weekdayCheckbox">
				<xsl:with-param name="value" select="'SUNDAY'" />
				<xsl:with-param name="text" select="$i18n.Calendar.Sunday" />
			</xsl:call-template>
			
		</div>
				
	</xsl:template>
	
	<xsl:template name="weekdayCheckbox">
		<xsl:param name="value" />
		<xsl:param name="text" />
	
		<label class="nomargin bigmarginleft bigmarginright">
			<xsl:call-template name="createCheckbox">
				<xsl:with-param name="name" select="'weekdaysWithHiddenBookings'"/>
				<xsl:with-param name="element" select="Calendar" />
				<xsl:with-param name="value" select="$value" />
				<xsl:with-param name="valueElementName" select="'dayOfWeek'" />
			</xsl:call-template>
			<xsl:value-of select="$text" />
		</label>
	
	</xsl:template>
	
	<xsl:template name="addUserTagsTable">
		<xsl:param name="withManager" select="'false'"/>
		<xsl:param name="withBookingSlotOld" select="'false'"/>
		<xsl:param name="withBookingSlotOldManagers" select="'false'"/>
		<xsl:param name="SMS" select="'false'"/>
	
		<div class="floatleft margintop full">

			<xsl:choose>
				<xsl:when test="$SMS = 'true'">
					<p><xsl:value-of select="$i18n.TagsTable.Description.SMS"/></p>
				</xsl:when>
				<xsl:otherwise>
					<p><xsl:value-of select="$i18n.TagsTable.Description.Email"/></p>
				</xsl:otherwise>
			</xsl:choose>
		
			<table class="full border">
				<tr>
					<th>
						<xsl:value-of select="$i18n.Tag"/>
					</th>
					<th>
						<xsl:value-of select="$i18n.TagDescription"/>
					</th>
				</tr>
				<tr>
					<td>
						<xsl:text>$calendar.name</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Calendar.Name"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.start</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.Start"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.end</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.End"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.location</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.Location"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.managers</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.Managers"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.managerFirstNames</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.ManagerFirstNames"/>
					</td>
				</tr>
				<xsl:if test="$withBookingSlotOldManagers = 'true'">
					<tr>
						<td>
							<xsl:text>$managersOld</xsl:text>
						</td>
						<td>
							<xsl:value-of select="$i18n.Tag.BookingSlotOld.Managers"/>
						</td>
					</tr>				
				</xsl:if>
				<xsl:if test="$withBookingSlotOld = 'true'">
					<tr>
						<td>
							<xsl:text>$bookingSlotOld.start</xsl:text>
						</td>
						<td>
							<xsl:value-of select="$i18n.Tag.BookingSlotOld.Start"/>
						</td>
					</tr>
					<tr>
						<td>
							<xsl:text>$bookingSlotOld.end</xsl:text>
						</td>
						<td>
							<xsl:value-of select="$i18n.Tag.BookingSlotOld.End"/>
						</td>
					</tr>
				</xsl:if>
				<tr>
					<td>
						<xsl:text>$booking.url</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Booking.URL"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$booking.messagesUrl</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Booking.MessagesURL"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.firstname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Contact.Firstname"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.lastname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Contact.Lastname"/>
					</td>
				</tr>
				<xsl:if test="$withManager = 'true'">
					<tr>
						<td>
							<xsl:text>$manager.firstname</xsl:text>
						</td>
						<td>
							<xsl:value-of select="$i18n.Tag.Manager.Firstname"/>
						</td>
					</tr>
					<tr>
						<td>
							<xsl:text>$manager.lastname</xsl:text>
						</td>
						<td>
							<xsl:value-of select="$i18n.Tag.Manager.Lastname"/>
						</td>
					</tr>
				</xsl:if>
			</table>
		
		</div>
	
	</xsl:template>
	
	<xsl:template name="addManagerTagsTable">
	
		<div class="floatleft margintop full">

			<p><xsl:value-of select="$i18n.TagsTable.Description.Email"/></p>
					
			<table class="full border">
				<tr>
					<th>
						<xsl:value-of select="$i18n.Tag"/>
					</th>
					<th>
						<xsl:value-of select="$i18n.TagDescription"/>
					</th>
				</tr>
				<tr>
					<td>
						<xsl:text>$calendar.name</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Calendar.Name"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.start</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.Start"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$bookingSlot.end</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.BookingSlot.End"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$booking.url</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Booking.URL"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$booking.messagesUrl</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Booking.MessagesURL"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.firstname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Contact.Firstname"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$contact.lastname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Contact.Lastname"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$manager.firstname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Manager.Firstname"/>
					</td>
				</tr>
				<tr>
					<td>
						<xsl:text>$manager.lastname</xsl:text>
					</td>
					<td>
						<xsl:value-of select="$i18n.Tag.Manager.Lastname"/>
					</td>
				</tr>
			</table>
		
		</div>
	
	</xsl:template>
	
	<xsl:template match="CalendarUserAdminExtension">
	
		<xsl:if test="Calendars">
		
			<div class="full floatleft bigmargintop">
	
				<h2><xsl:value-of select="$i18n.Calendars" /></h2>
				
				<div>
				
					<xsl:for-each select="Calendars/Calendar">
				
						<div class="floatleft full border marginbottom">
						
							<div class="floatleft">
								<img src="{$imgPath}/calendar_grid.png" style="margin: 0 5px 3px 1px;"/>
								<a href="{../../CalendarAdminURL}/update/{calendarID}" title="{name}"><xsl:value-of select="name" /></a>
								<xsl:if test="access = 'ADMIN'">
									<xsl:text> (</xsl:text>
									<span><xsl:value-of select="$i18n.Calendar.Admin" /></span>
									<xsl:text>)</xsl:text>
								</xsl:if>
							</div>
						
						</div>
					
					</xsl:for-each>
				
				</div>
			</div>
			
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UpdateFailedCalendarNotFound']">
	
		<p class="error">
			<xsl:value-of select="$i18n.validationError.UpdateFailedCalendarNotFound" />
		</p>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='NoProfileChosen']">
	
		<p class="error">
			<xsl:value-of select="$i18n.validationError.NoProfileChosen" />
		</p>
	
	</xsl:template>

	<xsl:template match="validationError[messageKey='PluginError']">
	
		<p class="error">
			<xsl:value-of select="$i18n.validationError.PluginError" />
		</p>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
	
		<p class="error">
			<xsl:value-of select="$i18n.UnableToParseRequest"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseIcon']">
	
		<p class="error">
			<xsl:value-of select="$i18n.UnableToParseIcon"/>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidFileExtension']">
	
		<p class="error">
			<xsl:value-of select="$i18n.InvalidFileExtension.Part1" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="filename" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.InvalidFileExtension.Part2" />

			<xsl:if test="AllowedExtensions">
			
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.InvalidFileExtension.Part3" />
				<xsl:text>&#160;</xsl:text>
				
				<xsl:apply-templates select="AllowedExtensions/Extension" mode="AllowedExtensions"/>
			</xsl:if>			
		</p>
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<xsl:if test="fieldName and validationErrorType">
			<p class="error">
				<span data-icon-before="!">
					<xsl:choose>
						<xsl:when test="validationErrorType='RequiredField'">
							<xsl:value-of select="$i18n.validationError.RequiredField" />
						</xsl:when>
						<xsl:when test="validationErrorType='InvalidFormat'">
							<xsl:value-of select="$i18n.validationError.InvalidFormat" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooShort'">
							<xsl:value-of select="$i18n.validationError.TooShort" />
						</xsl:when>
						<xsl:when test="validationErrorType='TooLong'">
							<xsl:value-of select="$i18n.validationError.TooLong" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.validationError.Other" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>&#x20;</xsl:text>
					<xsl:choose>
						<xsl:when test="fieldName = 'name'">
							<xsl:value-of select="$i18n.Calendar.Name" />
						</xsl:when>
						<xsl:when test="fieldName = 'description'">
							<xsl:value-of select="$i18n.Calendar.Description" />
						</xsl:when>
						<xsl:when test="fieldName = 'enabled'">
							<xsl:value-of select="$i18n.Calendar.Enabled" />
						</xsl:when>
						<xsl:when test="fieldName = 'price'">
							<xsl:value-of select="$i18n.Calendar.Price" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailSenderName'">
							<xsl:value-of select="$i18n.Calendar.EmailSenderName" />
						</xsl:when>
						<xsl:when test="fieldName = 'emailSenderAddress'">
							<xsl:value-of select="$i18n.Calendar.EmailSenderAddress" />
						</xsl:when>
						<xsl:when test="fieldName = 'exitGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Calendar.ExitGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Calendar.RebookedGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedGlobalEmailAddresses'">
							<xsl:value-of select="$i18n.Calendar.ExternalMessageReceivedGlobalEmailAddresses"/>
						</xsl:when>
						<xsl:when test="fieldName = 'hoursBeforeBookingLocked'">
							<xsl:value-of select="$i18n.Calendar.HoursBeforeBookingLocked"/>
						</xsl:when>
						<xsl:when test="fieldName = 'defaultSlotLength'">
							<xsl:value-of select="$i18n.Calendar.DefaultSlotLength"/>
						</xsl:when>
						<xsl:when test="fieldName = 'reminderMinutes'">
							<xsl:value-of select="$i18n.Calendar.SendReminderSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualEntranceEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ManualEntranceEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualEntranceEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ManualEntranceEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedUserEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedUserEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedUserEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedUserEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualReminderEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ManualReminderEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualReminderEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ManualReminderEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedUserEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.RebookedUserEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedUserEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.RebookedUserEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedManagerEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedManagerEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedManagerEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedManagerEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedGlobalEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedGlobalEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedGlobalEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.RebookedGlobalEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedGlobalEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.RebookedGlobalEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedGlobalEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedGlobalEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.UserExitEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.UserExitEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotRebookedUserEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotRebookedUserEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotRebookedUserEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotRebookedUserEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotDeletedUserEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotDeletedUserEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotDeletedUserEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotDeletedUserEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitManagerEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.UserExitManagerEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitManagerEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.UserExitManagerEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitGlobalEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.UserExitGlobalEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitGlobalEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.UserExitGlobalEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userRebookedManagerEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.UserRebookedManagerEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userRebookedManagerEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.UserRebookedManagerEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualEntranceManagerEmailSubject'">
							<xsl:value-of select="$i18n.Notifications.ManualEntranceManagerEmailSubject"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualEntranceManagerEmailMessage'">
							<xsl:value-of select="$i18n.Notifications.ManualEntranceManagerEmailMessage"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualEntranceSMS'">
							<xsl:value-of select="$i18n.Notifications.ManualEntranceSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'externalMessageReceivedUserSMS'">
							<xsl:value-of select="$i18n.Notifications.ExternalMessageReceivedUserSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'manualReminderSMS'">
							<xsl:value-of select="$i18n.Notifications.ManualReminderSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'rebookedUserSMS'">
							<xsl:value-of select="$i18n.Notifications.RebookedUserSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userExitSMS'">
							<xsl:value-of select="$i18n.Notifications.UserExitSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotRebookedUserSMS'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotRebookedUserSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotDeletedUserSMS'">
							<xsl:value-of select="$i18n.Notifications.BookingSlotDeletedUserSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'userReminderSMS'">
							<xsl:value-of select="$i18n.Notifications.UserReminderSMS"/>
						</xsl:when>
						<xsl:when test="fieldName = 'minVisibleDaysRebooking'">
							<xsl:value-of select="$i18n.Calendar.MinVisibleDaysRebooking"/>
						</xsl:when>
						<xsl:when test="fieldName = 'hoursPreliminaryBookingsAreValidFor'">
							<xsl:value-of select="$i18n.Calendar.HoursPreliminaryBookingsAreValidFor"/>
						</xsl:when>
						<xsl:when test="fieldName = 'fullyBookedCalendarDayText'">
							<xsl:value-of select="$i18n.Calendar.FullyBookedCalendarDayText" />
						</xsl:when>
						<xsl:when test="fieldName = 'fullyBookedListSlotText'">
							<xsl:value-of select="$i18n.Calendar.FullyBookedListSlotText" />
						</xsl:when>
						<xsl:when test="fieldName = 'userBookingMaxSize'">
							<xsl:value-of select="$i18n.Calendar.UserBookingMaxSize" />
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotDefaultDescription'">
							<xsl:value-of select="$i18n.Calendar.BookingSlotDefaultDescription" />
						</xsl:when>
						<xsl:when test="fieldName = 'bookingSlotDefaultLocation'">
							<xsl:value-of select="$i18n.Calendar.BookingSlotDefaultLocation" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="fieldName" />
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>!</xsl:text>
				</span>
			</p>
		</xsl:if>
		<xsl:if test="messageKey">
			<p class="error">
				<span data-icon-before="!">
					<xsl:value-of select="$i18n.validationError.unknownMessageKey" />!
				</span>
			</p>
		</xsl:if>
		<xsl:apply-templates select="message" />
		
	</xsl:template>
	
	<xsl:template match="ViewFragment">
		
		<xsl:value-of select="HTML" disable-output-escaping="yes"/>
		
	</xsl:template>
	
</xsl:stylesheet>