package com.nordicpeak.flowengine.booking.cruds;

import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.utils.crud.BeanFilter;
import se.unlogic.hierarchy.core.utils.crud.FragmentLinkScriptFilter;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.hierarchy.core.utils.crud.ModularCRUD;
import se.unlogic.standardutils.arrays.ArrayUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.populators.SwedishSocialSecurityPopulator;
import se.unlogic.standardutils.populators.TimePopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.booking.BookingConstants;
import com.nordicpeak.flowengine.booking.BookingSlotAdminModule;
import com.nordicpeak.flowengine.booking.BookingUtils;
import com.nordicpeak.flowengine.booking.beans.Booking;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingEvent;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.booking.beans.PrimaryContact;
import com.nordicpeak.flowengine.booking.beans.Repeating;
import com.nordicpeak.flowengine.booking.enums.BookingEventType;
import com.nordicpeak.flowengine.booking.events.BookingManuallyAddedEvent;
import com.nordicpeak.flowengine.booking.events.BookingSlotAddedEvent;
import com.nordicpeak.flowengine.booking.events.BookingSlotDeletedEvent;
import com.nordicpeak.flowengine.booking.events.BookingSlotRebookedEvent;
import com.nordicpeak.flowengine.booking.interfaces.BookingSlotFormExtensionProvider;
import com.nordicpeak.flowengine.booking.listeners.BookingUserTypeElementableListener;
import com.nordicpeak.flowengine.booking.validationerrors.PersonAlreadyInBookingSlotValidationError;
import com.nordicpeak.flowengine.booking.validationerrors.PersonAlreadyInCalendarValidationError;

public class BookingSlotCRUD extends ModularCRUD<BookingSlot, Integer, User, BookingSlotAdminModule> {

	private static final String CALENDAR_REQUEST_ATTRIBUTE_NAME = "calendar";

	protected final AdvancedAnnotatedDAOWrapper<BookingSlot, Integer> bookingSlotDAOWrapper;

	public BookingSlotCRUD(AdvancedAnnotatedDAOWrapper<BookingSlot, Integer> crudDAO, BookingSlotAdminModule callback) {

		super(IntegerBeanIDParser.getInstance(), crudDAO, new AnnotatedRequestPopulator<>(BookingSlot.class), "BookingSlot", "booking slot", "", callback);

		this.bookingSlotDAOWrapper = crudDAO;

		setRequirePostForDelete(true);
	}

	@Override
	public BookingSlot getBean(Integer beanID, String getMode, HttpServletRequest req) throws SQLException, AccessDeniedException {

		BookingSlot bookingSlot = super.getBean(beanID, getMode, req);

		if (bookingSlot != null) {

			BookingUtils.setBookingSlotBookingUsers(bookingSlot, callback.getUserHandler());

			if (bookingSlot.getRepeating() != null) {

				List<BookingSlot> bookingSlots = callback.getBookingSlots(bookingSlot.getRepeating());
				bookingSlot.getRepeating().setBookingSlots(bookingSlots);
			}

			BookingUtils.setNotBookableText(bookingSlot, callback.getBookingSlotNotBookableTextBetween(), callback.getBookingSlotNotBookableTextOpens(), callback.getBookingSlotNotBookableTextClosed());
		}

		return bookingSlot;
	}

	@Override
	protected List<BookingSlot> getAllBeans(User user, HttpServletRequest req, URIParser uriParser) throws SQLException {

		BookingCalendar calendar = getRequestCalendar(req);

		List<BookingSlot> bookingSlots;

		if (calendar.isShowBookings()) {

			bookingSlots = callback.getBookingSlots(calendar, BookingSlot.MANAGERS_RELATION, BookingSlot.CALENDAR_RELATION, BookingSlot.BOOKINGS_RELATION, Booking.CONTACT_RELATION);

		} else {

			bookingSlots = callback.getBookingSlots(calendar, BookingSlot.MANAGERS_RELATION, BookingSlot.CALENDAR_RELATION);
		}

		BookingUtils.setBookingSlotsBookingUsers(calendar, bookingSlots, callback.getUserHandler());

		if (bookingSlots != null) {

			if (beanFilters != null) {

				for (BeanFilter<? super BookingSlot> beanFilter : beanFilters) {

					beanFilter.beansLoaded(bookingSlots, req, uriParser, user);
				}
			}

			BookingUtils.setNotBookableText(bookingSlots, callback.getBookingSlotNotBookableTextBetween(), callback.getBookingSlotNotBookableTextOpens(), callback.getBookingSlotNotBookableTextClosed());
		}

		return bookingSlots;
	}
	
	@Override
	protected void appendBean(BookingSlot bean, Element targetElement, Document doc, User user) {
	
		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
		
		genDoc.addFieldElementableListener(Booking.class, new BookingUserTypeElementableListener(callback.getUserHandler(), bean.getCalendar(), callback.hasSMSSender(), callback.isOverrideContactBySMS()));

		super.appendBean(bean, targetElement, genDoc, user);
	}

	@Override
	protected void appendAllBeans(Document doc, Element listTypeElement, User user, HttpServletRequest req, URIParser uriParser, List<ValidationError> validationError) throws SQLException {

		BookingCalendar calendar = getRequestCalendar(req);
		listTypeElement.appendChild(calendar.toXML(doc));

		if (callback.isCalendarManager(calendar, user)) {

			XMLUtils.appendNewElement(doc, listTypeElement, "AddAccess");
		}

		List<BookingSlot> bookingSlots = getAllBeans(user, req, uriParser);

		XMLUtils.appendNewElement(doc, listTypeElement, "BookingSlotsTitleJSON", BookingUtils.getBookingSlotsTitleJSON(bookingSlots, true).toJson());
		XMLUtils.appendNewElement(doc, listTypeElement, "BookingSlotsJSON", BookingUtils.getBookingSlotsJSON(calendar, true, callback.getPreliminaryBookingText(), bookingSlots).toJson());

		BookingUtils.appendCalendarManagers(doc, listTypeElement, bookingSlots, calendar.isHideDisabledManagersInFilter());

		XMLUtils.append(doc, listTypeElement, typeElementPluralName, bookingSlots);
	}

	@Override
	protected void appendShowFormData(BookingSlot bookingSlot, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws Exception {

		super.appendShowFormData(bookingSlot, doc, showTypeElement, user, req, res, uriParser);

		if (hasCalendarAccess(bookingSlot.getCalendar(), user)) {

			XMLUtils.appendNewElement(doc, showTypeElement, "DeleteAccess");
		}

		if (callback.isCalendarManager(bookingSlot.getCalendar(), user)) {

			XMLUtils.appendNewElement(doc, showTypeElement, "AddAccess");
		}

	}

	@Override
	protected void appendUpdateFormData(BookingSlot bookingSlot, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bookingSlot, doc, updateTypeElement, user, req, uriParser);

		appendShowFormData(bookingSlot, doc, updateTypeElement, user, req, null, uriParser);

		appendExtensionViewFragments(bookingSlot.getCalendar(), bookingSlot, doc, updateTypeElement, user, req, uriParser);
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendAddFormData(doc, addTypeElement, user, req, uriParser);

		BookingCalendar calendar = getRequestCalendar(req);
		addTypeElement.appendChild(calendar.toXML(doc));

		if (req.getParameter("date") != null) {

			XMLUtils.appendNewElement(doc, addTypeElement, "StartDate", req.getParameter("date"));
		}

		if (callback.getAllowedManagers(calendar).containsKey(user.getUserID())) {

			Element managersElement = XMLUtils.appendNewElement(doc, addTypeElement, "DefaultManagers");
			Element userElement = user.toXML(doc);

			AttributeHandler attributeHandler = user.getAttributeHandler();

			if (attributeHandler != null && attributeHandler.isSet(BookingConstants.MANAGER_NAME_ATTRIBUTE)) {

				BookingUtils.setUserElementValue(doc, userElement, "email", attributeHandler.getString(BookingConstants.MANAGER_NAME_ATTRIBUTE));
			}

			managersElement.appendChild(userElement);
		}

		if (calendar.getDefaultStartTime() != null && calendar.getDefaultSlotLength() != null && calendar.getDefaultSlotLength() > 0 && !calendar.isShowDuration()) {

			Calendar cal = Calendar.getInstance();
			cal.setTime(calendar.getDefaultStartTime());
			cal.add(Calendar.MINUTE, calendar.getDefaultSlotLength());

			Time endTime = new Time(cal.getTimeInMillis());

			XMLUtils.appendNewElement(doc, addTypeElement, "DefaultEndTime", TimeUtils.TIME_FORMATTER.format(endTime));
		}

		appendExtensionViewFragments(calendar, null, doc, addTypeElement, user, req, uriParser);
	}

	protected void appendExtensionViewFragments(BookingCalendar calendar, BookingSlot bookingSlot, Document doc, Element element, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		if (!callback.getBookingSlotExtensionProviders().isEmpty()) {

			Element pluginFragments = XMLUtils.appendNewElement(doc, element, "ExtensionFragments");

			for (BookingSlotFormExtensionProvider extensionProvider : callback.getBookingSlotExtensionProviders()) {

				ValidationException validationException = (ValidationException) req.getAttribute(extensionProvider.getProviderID());

				ViewFragment extensionSettings = extensionProvider.getBookingSlotFormViewFragment(calendar, bookingSlot, req, user, uriParser, validationException);

				if (extensionSettings != null) {

					pluginFragments.appendChild(extensionSettings.toXML(doc));
					FragmentLinkScriptFilter.addViewFragment(extensionSettings, req);
				}
			}
		}
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		BookingCalendar calendar = getRequestCalendar(req);

		if (validationErrors != null && calendar == null) {

			return callback.listCalendars(req, user, uriParser, validationErrors);
		}

		return super.list(req, res, user, uriParser, validationErrors);
	}

	@Override
	protected BookingSlot populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		BookingCalendar calendar = getRequestCalendar(req);

		ArrayList<ValidationError> validationErrors = new ArrayList<>();

		BookingSlot bookingSlot = null;

		try {

			bookingSlot = super.populateFromAddRequest(req, user, uriParser);

		} catch (ValidationException e) {

			validationErrors.addAll(e.getErrors());
		}

		if (bookingSlot != null) {

			bookingSlot.setCalendar(calendar);
			bookingSlot.setPoster(user);
			bookingSlot.setAdded(new Timestamp(System.currentTimeMillis()));

			try {

				super.validateAddPopulation(bookingSlot, req, user, uriParser);

			} catch (ValidationException e) {

				validationErrors.addAll(e.getErrors());
			}

			populateAndValidate(bookingSlot, req, user, uriParser, validationErrors);

			if (req.getParameter("addBookings") != null) {

				populateBookings(bookingSlot, req, validationErrors, user);
			}
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		return bookingSlot;
	}

	private void processExtensionRequests(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		if (!callback.getBookingSlotExtensionProviders().isEmpty()) {

			boolean hasErrors = false;

			for (BookingSlotFormExtensionProvider extensionProvider : callback.getBookingSlotExtensionProviders()) {

				try {

					extensionProvider.processBookingSlotFormRequest(bookingSlot, req, user, uriParser);

				} catch (ValidationException e) {

					req.setAttribute(extensionProvider.getProviderID(), e);

					hasErrors = true;
				}
			}

			if (hasErrors) {

				validationErrors.add(new ValidationError("ExtensionErrors"));
			}
		}
	}

	private void populateBookings(BookingSlot bookingSlot, HttpServletRequest req, ArrayList<ValidationError> validationErrors, User user) {

		String[] citizenIDs = req.getParameterValues("citizenID");

		if (!StringUtils.isEmpty(citizenIDs)) {

			List<String> addedIdentifiers = new ArrayList<>(citizenIDs.length);

			Timestamp now = new Timestamp(System.currentTimeMillis());

			for (String citizenID : citizenIDs) {

				String citizenIdentifier = null;
				String rawCitizenID = citizenID;

				if (StringUtils.isEmpty(citizenID)) {

					validationErrors.add(new ValidationError("citizenIdentifier", ValidationErrorType.RequiredField));

				} else {

					citizenID = citizenID.trim().replace("-", "");

					if (!BookingSlotAdminModule.CITIZEN_ID_POPULATOR.validateFormat(citizenID)) {

						validationErrors.add(new ValidationError("citizenIdentifier", ValidationErrorType.InvalidFormat));
					}

					citizenIdentifier = BookingSlotAdminModule.CITIZEN_ID_POPULATOR.getValue(citizenID);
				}

				if (!StringUtils.isEmpty(citizenIdentifier)) {

					Booking booking = new Booking();

					if (citizenIdentifier.length() == 10) {

						citizenIdentifier = SwedishSocialSecurityPopulator.addCentury(citizenIdentifier);
					}

					if (addedIdentifiers.contains(citizenIdentifier) && !bookingSlot.getCalendar().isAllowMultipleBookings()) {

						validationErrors.add(new PersonAlreadyInCalendarValidationError());

					} else if (addedIdentifiers.contains(citizenIdentifier) && !bookingSlot.getCalendar().isAllowDuplicateBookings()) {

						validationErrors.add(new PersonAlreadyInBookingSlotValidationError());

					} else {

						addedIdentifiers.add(citizenIdentifier);

						String phone = ValidationUtils.validateParameter("phone-" + rawCitizenID, req, false, StringSwedishPhoneNumberPopulator.getPopulator(), validationErrors);

						PrimaryContact contact = callback.getPrimaryContact(citizenIdentifier, phone);

						if (contact != null) {

							booking.setPrimaryContact(contact);

						} else {

							validationErrors.add(new ValidationError("ErrorImportingCitizen"));
						}
					}
					
					Integer size = ValidationUtils.validateParameter("size-" + rawCitizenID, req, false, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
					
					booking.setSize(size != null ? size : 1);

					booking.addEvent(new BookingEvent(now, user, BookingEventType.SUBMITTED, callback.getEventAdminSubmittedMessage(), null));
					booking.setAdded(new Timestamp(System.currentTimeMillis()));

					if (bookingSlot.getBookings() == null) {

						bookingSlot.setBookings(new ArrayList<>());
					}

					bookingSlot.getBookings().add(booking);
				}
			}
		}
		
		int totalBookingSize = bookingSlot.getBookings() == null ? 0 : bookingSlot.getBookings().stream().mapToInt(Booking::getSize).sum();

		if (totalBookingSize > bookingSlot.getSize()) {

			validationErrors.add(new ValidationError("TooManyBookings"));
		}
	}

	@Override
	protected BookingSlot populateFromUpdateRequest(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		req.setAttribute("bookingSlotOldStart", bookingSlot.getStart());
		req.setAttribute("bookingSlotOldEnd", bookingSlot.getEnd());

		ArrayList<ValidationError> validationErrors = new ArrayList<>();

		try {

			bookingSlot = super.populateFromUpdateRequest(bookingSlot, req, user, uriParser);

		} catch (ValidationException e) {

			validationErrors.addAll(e.getErrors());
		}

		populateAndValidate(bookingSlot, req, user, uriParser, validationErrors);

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		bookingSlot.setUpdater(user);
		bookingSlot.setUpdated(new Timestamp(System.currentTimeMillis()));

		return bookingSlot;
	}

	protected void populateAndValidate(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		Timestamp now = new Timestamp(System.currentTimeMillis());

		Date date = ValidationUtils.validateParameter("startDate", req, true, DatePopulator.getYearLimitedPopulator(), validationErrors);
		Time startTime = ValidationUtils.validateParameter("startTime", req, true, TimePopulator.getPopulator(), validationErrors);
		Time endTime = ValidationUtils.validateParameter("endTime", req, !bookingSlot.getCalendar().isShowDuration(), TimePopulator.getPopulator(), validationErrors);
		Integer duration = ValidationUtils.validateParameter("duration", req, bookingSlot.getCalendar().isShowDuration(), IntegerPopulator.getPopulator(), validationErrors);
		Date bookableStartDate = ValidationUtils.validateParameter("bookableStartDate", req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
		Time bookableStartTime = ValidationUtils.validateParameter("bookableStartTime", req, bookableStartDate != null, TimePopulator.getPopulator(), validationErrors);
		Date bookableEndDate = ValidationUtils.validateParameter("bookableEndDate", req, false, DatePopulator.getPopulator(), validationErrors);
		Time bookableEndTime = ValidationUtils.validateParameter("bookableEndTime", req, bookableEndDate != null, TimePopulator.getPopulator(), validationErrors);

		if (date != null && startTime != null) {

			Calendar calendarDate = mergeDateAndTime(date, startTime);

			if (new Date().after(calendarDate.getTime())) {

				validationErrors.add(new ValidationError("DateInPast"));
			}

			Timestamp startTimestamp = new Timestamp(calendarDate.getTimeInMillis());
			bookingSlot.setStart(startTimestamp);

			if (bookingSlot.getCalendar().isShowDuration()) {

				if (duration != null) {

					if (duration < 1) {

						validationErrors.add(new ValidationError("duration", ValidationErrorType.TooShort));
					}

					calendarDate.setTime(startTimestamp);

					calendarDate.add(Calendar.MINUTE, duration);

					Timestamp endTimestamp = new Timestamp(calendarDate.getTimeInMillis());
					bookingSlot.setEnd(endTimestamp);
				}

			} else if (endTime != null) {

				calendarDate = mergeDateAndTime(startTimestamp, endTime);

				if (endTime.before(startTime)) { // Midnight crossover

					calendarDate.add(Calendar.DATE, 1);
				}

				Timestamp endTimestamp = new Timestamp(calendarDate.getTimeInMillis());
				bookingSlot.setEnd(endTimestamp);
			}

			if (bookingSlot.isBookableRestricted()) {

				if (bookableStartDate == null && bookableEndDate == null) {

					validationErrors.add(new ValidationError("BookableStartOrEndRequired"));

				} else {

					Timestamp bookableStart = null;
					Timestamp bookableEnd = null;

					if (bookableStartDate != null && bookableStartTime != null) {

						Calendar calendar = mergeDateAndTime(bookableStartDate, bookableStartTime);

						bookableStart = new Timestamp(calendar.getTimeInMillis());
					}

					if (bookableEndDate != null && bookableEndTime != null) {

						Calendar calendar = mergeDateAndTime(bookableEndDate, bookableEndTime);

						bookableEnd = new Timestamp(calendar.getTimeInMillis());
					}

					if (bookableStart != null) {

						if (bookableStart.after(bookingSlot.getStart())) {

							validationErrors.add(new ValidationError("BookableDateAfterDate"));
						}

						if (bookableEnd != null && bookableStart.after(bookableEnd)) {

							validationErrors.add(new ValidationError("BookableStartAfterBookableEnd"));
						}
					}

					if (bookableEnd != null && bookableEnd.after(bookingSlot.getStart())) {

						validationErrors.add(new ValidationError("BookableDateAfterDate"));
					}

					bookingSlot.setBookableStart(bookableStart);
					bookingSlot.setBookableEnd(bookableEnd);
				}
			}
		}

		if (bookingSlot.getSize() < 0) {

			validationErrors.add(new ValidationError("size", ValidationErrorType.InvalidFormat));
		}

		List<User> managers = null;

		String[] managerIDs = req.getParameterValues("manager");

		if (!ArrayUtils.isEmpty(managerIDs)) {

			Map<Integer, User> allowedManagers = callback.getAllowedManagers(bookingSlot.getCalendar());

			if (!CollectionUtils.isEmpty(allowedManagers)) {

				managers = new ArrayList<>(managerIDs.length);

				for (String userIDString : managerIDs) {

					Integer userID = NumberUtils.toInt(userIDString);

					if (userID != null) {

						User manager = allowedManagers.get(userID);

						if (manager != null) {

							managers.add(manager);
						}
					}
				}
			}
		}

		bookingSlot.setManagers(managers);

		if (bookingSlot.getBookingSlotID() == null) { // Add

			if (req.getParameter("repeating") != null) {

				Date toDate = ValidationUtils.validateParameter("repetitionToDate", req, true, DatePopulator.getYearLimitedPopulator(), validationErrors);
				Integer repetitionIntervalInDays;

				if ("custom".equals(req.getParameter("repetitionInterval"))) {

					repetitionIntervalInDays = ValidationUtils.validateParameter("repetitionIntervalCustom", req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);

					if (repetitionIntervalInDays != null) {
						repetitionIntervalInDays *= 7;
					}

				} else {

					repetitionIntervalInDays = ValidationUtils.validateParameter("repetitionInterval", req, true, PositiveStringIntegerPopulator.getPopulator(), validationErrors);
				}

				if (date != null && toDate != null) {

					if (date.compareTo(toDate) >= 0) {

						validationErrors.add(new ValidationError("RepetitionToDateBeforeFromDate"));

					} else if (DateUtils.getDaysBetween(date, toDate) < 7) {

						validationErrors.add(new ValidationError("RepetitionPeriodLessThanOneWeek"));
					}
				}

				if (date != null && toDate != null && repetitionIntervalInDays != null) {

					Repeating repeating = new Repeating(bookingSlot.getCalendar(), new java.sql.Date(date.getTime()), new java.sql.Date(toDate.getTime()), repetitionIntervalInDays, now, user);

					bookingSlot.setRepeating(repeating);
				}
			}

		} else { // Update

			bookingSlot.setRepeating(null);
		}

		processExtensionRequests(bookingSlot, req, user, uriParser, validationErrors);
	}

	private Calendar mergeDateAndTime(Date date, Date time) {

		Calendar calendarDate = Calendar.getInstance();
		Calendar calendarTime = Calendar.getInstance();

		calendarDate.setTime(date);
		calendarTime.setTime(time);

		calendarDate.set(Calendar.HOUR_OF_DAY, calendarTime.get(Calendar.HOUR_OF_DAY));
		calendarDate.set(Calendar.MINUTE, calendarTime.get(Calendar.MINUTE));

		return calendarDate;
	}

	@Override
	protected void beforeAddFilteredBeanCommit(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser) throws Exception {

		super.beforeAddFilteredBeanCommit(bookingSlot, req, user, uriParser);

		Repeating repeating = bookingSlot.getRepeating();

		if (repeating != null) {

			if (repeating.getFromDate().compareTo(repeating.getToDate()) >= 0) {
				throw new RuntimeException("From date is not before to date");
			}

			createRepeatingBookingSlots(repeating, req, bookingSlot);
		}
	}

	private void createRepeatingBookingSlots(Repeating repeating, HttpServletRequest req, BookingSlot lastBookingSlot) throws SQLException {

		RepeatingCRUD.createRepeatingBookingSlots(repeating, getTransactionHandler(req), lastBookingSlot, bookingSlotDAOWrapper, LocalDateTime.now());
	}

	@Override
	public void checkAddAccess(User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		checkCalendarAccess(getRequestCalendar(req), user);
	}

	@Override
	protected void checkListAccess(User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		checkCalendarAccess(getRequestCalendar(req), user);
	}

	@Override
	public void checkShowAccess(BookingSlot bookingSlot, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		checkCalendarAccess(bookingSlot.getCalendar(), user);
	}

	@Override
	public void checkUpdateAccess(BookingSlot bookingSlot, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		checkCalendarAccess(bookingSlot.getCalendar(), user);
	}

	@Override
	protected void checkDeleteAccess(BookingSlot bookingSlot, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		checkCalendarAccess(bookingSlot.getCalendar(), user);
	}

	private void checkCalendarAccess(BookingCalendar calendar, User user) throws AccessDeniedException {

		if (!hasCalendarAccess(calendar, user)) {

			throw new AccessDeniedException("User does not have access to " + calendar);
		}
	}

	private boolean hasCalendarAccess(BookingCalendar calendar, User user) {

		return callback.hasCalendarAccess(calendar, user);
	}

	@Override
	protected ForegroundModuleResponse filteredBeanAdded(BookingSlot bookingSlot, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		List<BookingSlot> repeatingBookingSlots;
		if (bookingSlot.getRepeating() != null && !CollectionUtils.isEmpty(repeatingBookingSlots = callback.getBookingSlots(bookingSlot.getRepeating(), BookingSlot.CALENDAR_RELATION))) {

			for (BookingSlot repeatingBookingSlot : repeatingBookingSlots) {

				processExtensionAddOrUpdate(repeatingBookingSlot, req, user, uriParser, false);
			}

		} else {

			processExtensionAddOrUpdate(bookingSlot, req, user, uriParser, false);
		}

		BookingCalendar calendar = getRequestCalendar(req);

		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/list/" + calendar.getCalendarID());

		callback.getEventHandler().sendEvent(BookingSlot.class, new BookingSlotAddedEvent(bookingSlot, user), EventTarget.ALL);

		if (bookingSlot.getBookings() != null) {

			for (Booking booking : bookingSlot.getBookings()) {

				callback.getEventHandler().sendEvent(Booking.class, new BookingManuallyAddedEvent(booking, user), EventTarget.ALL);
			}
		}

		return null;
	}

	@Override
	protected ForegroundModuleResponse filteredBeanUpdated(BookingSlot bookingSlot, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		processExtensionAddOrUpdate(bookingSlot, req, user, uriParser, true);

		sendBookingSlotRebookedEvent(bookingSlot, req);

		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/showslot/" + bookingSlot.getBookingSlotID());
		return null;
	}

	private void sendBookingSlotRebookedEvent(BookingSlot bookingSlot, HttpServletRequest req) {

		Timestamp bookingSlotOldStart = (Timestamp) req.getAttribute("bookingSlotOldStart");
		Timestamp bookingSlotOldEnd = (Timestamp) req.getAttribute("bookingSlotOldEnd");

		if (bookingSlotOldStart == null || bookingSlotOldEnd == null) {

			return;
		}

		if (bookingSlotOldStart.equals(bookingSlot.getStart()) && bookingSlotOldEnd.equals(bookingSlot.getEnd())) {

			return;
		}

		callback.getEventHandler().sendEvent(BookingSlot.class, new BookingSlotRebookedEvent(bookingSlot, bookingSlotOldStart, bookingSlotOldEnd), EventTarget.ALL);
	}

	@Override
	protected ForegroundModuleResponse filteredBeanDeleted(BookingSlot bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(BookingSlot.class, new BookingSlotDeletedEvent(bean), EventTarget.ALL);

		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/list/" + bean.getCalendar().getCalendarID());
		return null;
	}

	private void processExtensionAddOrUpdate(BookingSlot bookingSlot, HttpServletRequest req, User user, URIParser uriParser, boolean processBookings) throws Exception {

		if (!callback.getBookingSlotExtensionProviders().isEmpty()) {

			for (BookingSlotFormExtensionProvider extensionProvider : callback.getBookingSlotExtensionProviders()) {

				extensionProvider.processBookingSlotAddOrUpdate(bookingSlot, req, user, uriParser, processBookings);
			}
		}
	}

	@Override
	protected ForegroundModuleResponse beanDeleted(BookingSlot bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(BookingSlot.class, new CRUDEvent<>(CRUDAction.DELETE, bean), EventTarget.ALL);

		return super.beanDeleted(bean, req, res, user, uriParser);
	}

	private static BookingCalendar getRequestCalendar(HttpServletRequest req) {

		return (BookingCalendar) req.getAttribute(CALENDAR_REQUEST_ATTRIBUTE_NAME);
	}

	public static void setRequestCalendar(HttpServletRequest req, BookingCalendar calendar) {

		req.setAttribute(CALENDAR_REQUEST_ATTRIBUTE_NAME, calendar);
	}

}
