package com.nordicpeak.flowengine.booking.queries;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.SystemInterface;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.booking.BookingUtils;
import com.nordicpeak.flowengine.booking.beans.BookingCalendar;
import com.nordicpeak.flowengine.booking.beans.BookingSlot;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;

public class BookingQueryRequestProcessor implements QueryRequestProcessor {

	protected static final Logger log = Logger.getLogger(BookingQueryRequestProcessor.class);

	private final BookingQueryInstance queryInstance;
	private final SystemInterface systemInterface;

	public BookingQueryRequestProcessor(BookingQueryInstance queryInstance, SystemInterface systemInterface) {

		this.queryInstance = queryInstance;
		this.systemInterface = systemInterface;
	}

	@Override
	public void processRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Integer managerID = NumberUtils.toInt(req.getParameter("m"));
		String locationString = RequestUtils.parseParameter(req, systemInterface.getEncoding(), req.getParameter("l"));

		BookingCalendar calendar = queryInstance.getQuery().getCalendar();
		List<BookingSlot> bookingSlots = calendar.getBookingSlots();

		if (!CollectionUtils.isEmpty(bookingSlots) && (managerID != null || !StringUtils.isEmpty(locationString))) {

			bookingSlots = new ArrayList<>(bookingSlots);

			bookingSlots.removeIf(slot -> filterByManager(slot, managerID) || filterByLocation(slot, locationString));
		}

		JsonObject jsonObject = new JsonObject(2);
		jsonObject.putField("titleEvents", BookingUtils.getBookingSlotsTitleJSON(bookingSlots, false));
		jsonObject.putField("detailedEvents", BookingUtils.getBookingSlotsJSON(calendar, false, null, bookingSlots));
		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	private boolean filterByManager(BookingSlot bookingSlot, Integer managerID) {

		if (managerID == null) {

			return false;
		}

		return CollectionUtils.find(bookingSlot.getManagers(), manager -> manager.getUserID().equals(managerID)) == null;
	}

	private boolean filterByLocation(BookingSlot bookingSlot, String locationString) {

		if (StringUtils.isEmpty(locationString)) {

			return false;
		}

		return !locationString.equals(bookingSlot.getLocation());
	}

	@Override
	public void close() throws Exception {

	}
}
