package com.nordicpeak.flowengine.integration.boverket;

import java.nio.charset.Charset;
import java.util.List;

import javax.naming.OperationNotSupportedException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionDescriptor;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.MillisecondTimeUnits;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.UserFlowInstanceModule;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.integration.boverket.beans.ControlPlan;
import com.nordicpeak.flowengine.integration.boverket.beans.ControlPoint;
import com.nordicpeak.flowengine.integration.boverket.beans.Errand;
import com.nordicpeak.flowengine.interfaces.FlowInstanceOverviewExtensionProvider;

public class BoverketControlPlanModule extends AnnotatedForegroundModule implements FlowInstanceOverviewExtensionProvider, ViewFragmentModule<ForegroundModuleDescriptor> {

	private static final Charset UTF8 = Charset.forName("UTF-8");

	@XSLVariable(prefix = "java.")
	protected String tabTitle;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;

	@InstanceManagerDependency(required = true)
	private BoverketIntegrationProvider boverketIntegrationProvider;

	protected FlowInstanceAdminModule flowInstanceAdminModule;

	protected UserFlowInstanceModule userFlowInstanceModule;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	protected FlowEngineDAOFactory daoFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		this.daoFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		try {

			viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());
			viewFragmentTransformer.setDebugXML(debugFragmententXML);

		} catch (Exception e) {

			log.error("Unable to parse XSL stylesheet for tab contents in module " + this.moduleDescriptor, e);
		}
	}

	@Override
	public void unload() throws Exception {

		if (flowInstanceAdminModule != null) {

			flowInstanceAdminModule.removeFlowInstanceOverviewExtensionProvider(this);
		}

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.removeFlowInstanceOverviewExtensionProvider(this);
		}

		super.unload();
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.sectionInterface.getSectionDescriptor().toXML(doc));
		document.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(document);

		return doc;
	}

	public SectionDescriptor getSectionDescriptor() {

		return sectionInterface.getSectionDescriptor();
	}

	@InstanceManagerDependency(required = true)
	public void setFlowInstanceAdminModule(FlowInstanceAdminModule flowInstanceAdminModule) {

		if (flowInstanceAdminModule != null) {

			flowInstanceAdminModule.addFlowInstanceOverviewExtensionProvider(this);

		} else {

			this.flowInstanceAdminModule.removeFlowInstanceOverviewExtensionProvider(this);
		}

		this.flowInstanceAdminModule = flowInstanceAdminModule;
	}

	@InstanceManagerDependency(required = true)
	public void setFlowInstanceAdminModule(UserFlowInstanceModule userFlowInstanceModule) {

		if (userFlowInstanceModule != null) {

			userFlowInstanceModule.addFlowInstanceOverviewExtensionProvider(this);

		} else {

			this.userFlowInstanceModule.removeFlowInstanceOverviewExtensionProvider(this);
		}

		this.userFlowInstanceModule = userFlowInstanceModule;
	}

	@Override
	public ExtensionLink getOverviewTabHeaderExtensionLink(FlowInstance flowInstance, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		BoverketIntegrationSettings settings = boverketIntegrationProvider.getIntegrationSettings(flowInstance.getFlow());

		if (settings != null && flowInstance.getAttributeHandler().getString(BoverketIntegrationProvider.BOVERKET_EXTERNAL_ID_ATTRIBUTE) != null) {

			Errand errand = getErrand(flowInstance, true, user);

			String tabTitleSuffix = "";

			if (errand != null) {

				if (errand.getControlPlan() != null) {

					tabTitleSuffix = " (" + errand.getControlPlan().getFinishedCount() + "/" + errand.getControlPlan().getTotalCount() + ")";
				}

				if (!StringUtils.isEmpty(errand.getStatus())) {

					String currentStatus = flowInstance.getAttributeHandler().getString(BoverketIntegrationProvider.BOVERKET_STATUS_ATTRIBUTE);

					if (currentStatus == null || !currentStatus.equals(errand.getStatus())) {

						log.info("Updating status of errand " + errand + " to " + errand.getStatus());

						boverketIntegrationProvider.addOrUpdateFlowInstanceAttribute(BoverketIntegrationProvider.BOVERKET_STATUS_ATTRIBUTE, errand.getStatus(), flowInstance);

						flowInstance.getAttributeHandler().setAttribute(BoverketIntegrationProvider.BOVERKET_STATUS_ATTRIBUTE, errand.getStatus());

					}

				}

				req.setAttribute("Errand", errand);
			}

			return new ExtensionLink(tabTitle + tabTitleSuffix, "#boverket", null, null);
		}

		return null;

	}

	@Override
	public ViewFragment getOverviewTabContentsViewFragment(FlowInstance flowInstance, HttpServletRequest req, URIParser uriParser, User user) throws Exception {

		return getTabContentsViewFragment(flowInstance, req, uriParser, user, null);
	}

	public ControlPlan getControlPlan(String uuid) {

		SimpleRequest request = null;

		try {

			request = new SimpleRequest(boverketIntegrationProvider.getApiURL() + "/kontrollplan/" + uuid);

			request.setConnectionTimeout(boverketIntegrationProvider.getConnectionTimeout() * MillisecondTimeUnits.SECOND);
			request.setReadTimeout(boverketIntegrationProvider.getRequestTimeout() * MillisecondTimeUnits.SECOND);

			String response = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Document responseDoc = XMLUtils.parseXML(response, "UTF-8", false, false);

			XMLParser responseParser = new XMLParser(responseDoc);

			XMLParser xmlParser = responseParser.getNode("/Kontrollplan");

			if (xmlParser != null) {

				return new ControlPlan(xmlParser);
			}

		} catch (Exception e) {

			log.warn("Unable get kontrollplan from Boverket API using endpoint " + request.getEndpoint(), e);

		}

		return null;

	}

	public Errand getErrand(FlowInstance flowInstance, boolean getControlPlan, User user) {

		String uuid = flowInstance.getAttributeHandler().getString(BoverketIntegrationProvider.BOVERKET_EXTERNAL_ID_ATTRIBUTE);

		if (uuid != null) {

			SimpleRequest request = null;

			try {

				log.info("User " + user + " requesting lovarende from Boverket API");

				request = new SimpleRequest(boverketIntegrationProvider.getApiURL() + "/lovarende/" + uuid);

				request.setConnectionTimeout(boverketIntegrationProvider.getConnectionTimeout() * MillisecondTimeUnits.SECOND);
				request.setReadTimeout(boverketIntegrationProvider.getRequestTimeout() * MillisecondTimeUnits.SECOND);

				String response = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

				Document responseDoc = XMLUtils.parseXML(response, "UTF-8", false, false);

				XMLParser responseParser = new XMLParser(responseDoc);

				XMLParser errandParser = responseParser.getNode("/Lovarende");

				Errand errand = new Errand(errandParser);

				if (!StringUtils.isEmpty(errand.getUuid())) {

					if (getControlPlan && !StringUtils.isEmpty(errand.getControlPlanUUID())) {

						ControlPlan controlPlan = getControlPlan(errand.getControlPlanUUID());

						if (controlPlan != null && controlPlan.getControlPoints() != null) {

							String currentPropertyType = null;
							
							for (ControlPoint controlPoint : controlPlan.getControlPoints()) {

								if (!StringUtils.isEmpty(controlPoint.getConstitutionUuid())) {

									controlPoint.setConstitution(getConstitutionText(controlPoint.getConstitutionUuid()));
									
									if(currentPropertyType == null || (currentPropertyType != null && controlPoint.getPropertyType() != null && !currentPropertyType.equals(controlPoint.getPropertyType()))) {
										
										currentPropertyType = controlPoint.getPropertyType();
										
									} else {
										
										controlPoint.setPropertyType("");
										
									}
									
								}

							}

						}

						errand.setControlPlan(controlPlan);

					}

					return errand;
				}

				return null;

			} catch (Exception e) {

				log.warn("Unable get lovarende from Boverket API using endpoint " + request.getEndpoint(), e);

			}

		}

		return null;

	}

	public String getConstitutionText(String uuid) {

		SimpleRequest request = null;

		try {

			request = new SimpleRequest(boverketIntegrationProvider.getApiURL() + "/forfattning/" + uuid);

			request.setConnectionTimeout(boverketIntegrationProvider.getConnectionTimeout() * MillisecondTimeUnits.SECOND);
			request.setReadTimeout(boverketIntegrationProvider.getRequestTimeout() * MillisecondTimeUnits.SECOND);

			String response = HTTPUtils.sendHTTPGetRequest(request, UTF8).getValue();

			Document responseDoc = XMLUtils.parseXML(response, "UTF-8", false, false);

			XMLParser responseParser = new XMLParser(responseDoc);

			XMLParser xmlParser = responseParser.getNode("/Forfattning");

			if (xmlParser != null) {

				return xmlParser.getString("Innehall");
			}

		} catch (Exception e) {

			log.warn("Unable get forfattning from Boverket API using endpoint " + request.getEndpoint(), e);

		}

		return null;

	}

	public ViewFragment getTabContentsViewFragment(FlowInstance flowInstance, HttpServletRequest req, URIParser uriParser, User user, List<ValidationError> validationErrors) throws Exception {

		Document doc = createDocument(req, uriParser, user);
		Element documentElement = doc.getDocumentElement();

		Element tabElement = XMLUtils.appendNewElement(doc, documentElement, "TabContents");

		XMLUtils.append(doc, tabElement, flowInstance);

		if (req.getAttribute("Errand") != null) {

			XMLUtils.append(doc, tabElement, (Errand) req.getAttribute("Errand"));
		}

		XMLUtils.append(doc, tabElement, validationErrors);

		return viewFragmentTransformer.createViewFragment(doc);
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@Override
	public ForegroundModuleResponse processOverviewExtensionRequest(FlowInstance flowInstance, HttpServletRequest req, HttpServletResponse res, URIParser uriParser, User user) throws Exception {

		throw new OperationNotSupportedException();
	}

	@Override
	public String getOverviewExtensionProviderID() {

		return moduleDescriptor.getModuleID().toString();
	}
}
