package com.nordicpeak.flowengine.integration.castor.client;

import java.io.File;
import java.util.List;

import se.unlogic.standardutils.string.TagReplacer;

import com.nordicpeak.flowengine.integration.castor.beans.IntegrationSettings;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.utils.PDFByteAttachment;

public class CaseData {

	private PersonData applicantData;
	private PersonData realEstateOwnerData;
	private PersonData invoiceRecipientData;
	private PersonData inspectionManagerData;
	private PersonData contactPersonData;
	private PersonData appellantData;
	private PersonData coApplicantData;
	private PersonData constructionDeveloperData;

	private File pdfFile;

	private List<PDFByteAttachment> pdfAttachments;

	public CaseData(IntegrationSettings settings, ImmutableFlowInstance flowInstance, File pdfFile, List<PDFByteAttachment> pdfAttachments, TagReplacer tagReplacer) {

		this.applicantData = new PersonData(settings.getCaseNumber(), settings.getCaseName(), settings.getCaseCareOf(), settings.getCaseAddress(), settings.getCaseZIPCode(), settings.getCasePostalAddress(), settings.getCasePhone(), settings.getCaseMobilePhone(), settings.getCaseEmail(), settings.getCaseRealEstateName(), settings.getCaseRealEstateNumber(), settings.getCaseRealEstateAddress(), settings.getCaseRealEstatePostalAddress(), settings.getCaseRealEstateZIPCode(), flowInstance, tagReplacer);

		if (settings.isRealEstateOwnerEnabled()) {

			//@formatter:off
			this.realEstateOwnerData = new PersonData(
				settings.getRealEstateOwnerNumber(), 
				settings.getRealEstateOwnerName(), 
				settings.getRealEstateOwnerCareOf(), 
				settings.getRealEstateOwnerAddress(), 
				settings.getRealEstateOwnerZIPCode(), 
				settings.getRealEstateOwnerPostalAddress(), 
				settings.getRealEstateOwnerPhone(), 
				settings.getRealEstateOwnerMobilePhone(), 
				settings.getRealEstateOwnerEmail(), 
				settings.getRealEstateOwnerRealEstateName(), 
				settings.getRealEstateOwnerRealEstateNumber(), 
				settings.getRealEstateOwnerRealEstateAddress(), 
				settings.getRealEstateOwnerRealEstatePostalAddress(), 
				settings.getRealEstateOwnerRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on

			if (!realEstateOwnerData.isPopulated()) {

				realEstateOwnerData = applicantData;
			}
		}

		if (settings.isInvoiceRecipientEnabled()) {

			//@formatter:off
			this.invoiceRecipientData = new PersonData(
				settings.getInvoiceRecipientNumber(), 
				settings.getInvoiceRecipientName(), 
				settings.getInvoiceRecipientCareOf(), 
				settings.getInvoiceRecipientAddress(), 
				settings.getInvoiceRecipientZIPCode(), 
				settings.getInvoiceRecipientPostalAddress(), 
				settings.getInvoiceRecipientPhone(), 
				settings.getInvoiceRecipientMobilePhone(), 
				settings.getInvoiceRecipientEmail(), 
				settings.getInvoiceRecipientRealEstateName(), 
				settings.getInvoiceRecipientRealEstateNumber(), 
				settings.getInvoiceRecipientRealEstateAddress(), 
				settings.getInvoiceRecipientRealEstatePostalAddress(), 
				settings.getInvoiceRecipientRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on

			if (!invoiceRecipientData.isPopulated()) {

				invoiceRecipientData = applicantData;
			}
		}

		if (settings.isInspectionManagerEnabled()) {

			//@formatter:off
			this.inspectionManagerData = new PersonData(
				settings.getInspectionManagerNumber(), 
				settings.getInspectionManagerName(), 
				settings.getInspectionManagerCareOf(), 
				settings.getInspectionManagerAddress(), 
				settings.getInspectionManagerZIPCode(), 
				settings.getInspectionManagerPostalAddress(), 
				settings.getInspectionManagerPhone(), 
				settings.getInspectionManagerMobilePhone(), 
				settings.getInspectionManagerEmail(), 
				settings.getInspectionManagerRealEstateName(), 
				settings.getInspectionManagerRealEstateNumber(), 
				settings.getInspectionManagerRealEstateAddress(), 
				settings.getInspectionManagerRealEstatePostalAddress(), 
				settings.getInspectionManagerRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on
		}

		if (settings.isContactPersonEnabled()) {

			//@formatter:off
			this.contactPersonData = new PersonData(
				settings.getContactPersonNumber(), 
				settings.getContactPersonName(), 
				settings.getContactPersonCareOf(), 
				settings.getContactPersonAddress(), 
				settings.getContactPersonZIPCode(), 
				settings.getContactPersonPostalAddress(), 
				settings.getContactPersonPhone(), 
				settings.getContactPersonMobilePhone(), 
				settings.getContactPersonEmail(), 
				settings.getContactPersonRealEstateName(), 
				settings.getContactPersonRealEstateNumber(), 
				settings.getContactPersonRealEstateAddress(), 
				settings.getContactPersonRealEstatePostalAddress(), 
				settings.getContactPersonRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on

			if (!contactPersonData.isPopulated()) {

				contactPersonData = applicantData;
			}
		}

		if (settings.isAppellantEnabled()) {

			//@formatter:off
			this.appellantData = new PersonData(
				settings.getAppellantNumber(), 
				settings.getAppellantName(), 
				settings.getAppellantCareOf(), 
				settings.getAppellantAddress(), 
				settings.getAppellantZIPCode(), 
				settings.getAppellantPostalAddress(), 
				settings.getAppellantPhone(), 
				settings.getAppellantMobilePhone(), 
				settings.getAppellantEmail(), 
				settings.getAppellantRealEstateName(), 
				settings.getAppellantRealEstateNumber(), 
				settings.getAppellantRealEstateAddress(), 
				settings.getAppellantRealEstatePostalAddress(), 
				settings.getAppellantRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on
		}

		if (settings.isCoApplicantEnabled()) {

			//@formatter:off
			this.coApplicantData = new PersonData(
				settings.getCoApplicantNumber(), 
				settings.getCoApplicantName(), 
				settings.getCoApplicantCareOf(), 
				settings.getCoApplicantAddress(), 
				settings.getCoApplicantZIPCode(), 
				settings.getCoApplicantPostalAddress(), 
				settings.getCoApplicantPhone(), 
				settings.getCoApplicantMobilePhone(), 
				settings.getCoApplicantEmail(), 
				settings.getCoApplicantRealEstateName(), 
				settings.getCoApplicantRealEstateNumber(), 
				settings.getCoApplicantRealEstateAddress(), 
				settings.getCoApplicantRealEstatePostalAddress(), 
				settings.getCoApplicantRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on
		}

		if (settings.isConstructionDeveloperEnabled()) {
			//@formatter:off
			this.constructionDeveloperData = new PersonData(
				settings.getConstructionDeveloperNumber(), 
				settings.getConstructionDeveloperName(), 
				settings.getConstructionDeveloperCareOf(), 
				settings.getConstructionDeveloperAddress(), 
				settings.getConstructionDeveloperZIPCode(), 
				settings.getConstructionDeveloperPostalAddress(), 
				settings.getConstructionDeveloperPhone(), 
				settings.getConstructionDeveloperMobilePhone(), 
				settings.getConstructionDeveloperEmail(), 
				settings.getConstructionDeveloperRealEstateName(), 
				settings.getConstructionDeveloperRealEstateNumber(), 
				settings.getConstructionDeveloperRealEstateAddress(), 
				settings.getConstructionDeveloperRealEstatePostalAddress(), 
				settings.getConstructionDeveloperRealEstateZIPCode(), 
				flowInstance, 
				tagReplacer
			);
			//@formatter:on

			if (!constructionDeveloperData.isPopulated()) {

				constructionDeveloperData = applicantData;
			}
		}

		this.pdfFile = pdfFile;
		this.pdfAttachments = pdfAttachments;
	}

	public boolean isPopulated() {

		//TODO When documentation is available this needs to be updated with a more exakt list of required parameters
		return pdfFile != null && applicantData.isPopulated();
	}

	@Override
	public String toString() {

		return "(applicantData=" + applicantData + ", pdfFile=" + pdfFile + ")";
	}

	public PersonData getApplicantData() {

		return applicantData;
	}

	public PersonData getRealEstateOwnerData() {

		return realEstateOwnerData;
	}

	public PersonData getInvoiceRecipientData() {

		return invoiceRecipientData;
	}

	public PersonData getInspectionManagerData() {

		return inspectionManagerData;
	}

	public PersonData getContactPersonData() {

		return contactPersonData;
	}

	public PersonData getAppellantData() {

		return appellantData;
	}

	public PersonData getCoApplicantData() {

		return coApplicantData;
	}

	public PersonData getConstructionDeveloperData() {

		return constructionDeveloperData;
	}

	public File getPdfFile() {

		return pdfFile;
	}

	public List<PDFByteAttachment> getPdfAttachments() {

		return pdfAttachments;
	}

}
