package com.nordicpeak.flowengine.changeownerhsa.beans;

import java.io.Serializable;
import java.sql.Timestamp;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.utils.UserUtils;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.HSAIDPopulator;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.string.StringTag;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.beans.Contact;

@Table(name = "ownership_change_requests_hsa")
@XMLElement
public class OwnershipChangeRequest extends GeneratedElementable implements Serializable {
	
	
	private static final long serialVersionUID = -3942415081672454957L;
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer flowInstanceID;
	
	@DAOManaged
	@XMLElement
	private User poster;
	
	@DAOManaged
	@WebPopulate(required = true, populator = HSAIDPopulator.class)
	@XMLElement
	private String hsaID;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String firstName;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String lastName;
	
	@DAOManaged
	@WebPopulate(populator = EmailPopulator.class, maxLength = 255)
	@XMLElement
	private String email;
	
	@DAOManaged
	@WebPopulate(populator = StringSwedishPhoneNumberPopulator.class, maxLength = 50)
	@XMLElement
	private String mobilePhone;
	
	@DAOManaged
	@XMLElement
	private Timestamp added;
	
	@DAOManaged
	@XMLElement
	private boolean declined;
	
	@DAOManaged
	@StringTag
	@XMLElement
	private String declineReason;
	
	private User recipient;
	
	public Integer getFlowInstanceID() {
		
		return flowInstanceID;
	}
	
	public void setFlowInstanceID(Integer flowInstanceID) {
		
		this.flowInstanceID = flowInstanceID;
	}
	
	public String getHsaID() {
		
		return hsaID;
	}
	
	public void setHsaID(String socialSecurityNumber) {
		
		this.hsaID = socialSecurityNumber;
	}
	
	public Timestamp getAdded() {
		
		return added;
	}
	
	public void setAdded(Timestamp added) {
		
		this.added = added;
	}
	
	public User getPoster() {
		return poster;
	}
	
	public void setPoster(User poster) {
		this.poster = poster;
	}
	
	public String getEmail() {
		return email;
	}
	
	public void setEmail(String email) {
		this.email = email;
	}
	
	public String getMobilePhone() {
		return mobilePhone;
	}
	
	public void setMobilePhone(String mobilePhone) {
		this.mobilePhone = mobilePhone;
	}
	
	public String getFirstName() {
		return firstName;
	}
	
	public void setFirstName(String name) {
		this.firstName = name;
	}
	
	public String getLastName() {
		return lastName;
	}
	
	public void setLastName(String lastName) {
		this.lastName = lastName;
	}
	
	public boolean isDeclined() {
		return declined;
	}
	
	public void setDeclined(boolean declined) {
		this.declined = declined;
	}
	
	public String getDeclineReason() {
		return declineReason;
	}
	
	public void setDeclineReason(String declineReason) {
		this.declineReason = declineReason;
	}
	
	public Contact getRecipientContact() {
		
		Contact contact = new Contact();
		
		contact.setCitizenIdentifier(hsaID);
		contact.setFirstname(firstName);
		contact.setLastname(lastName);
		contact.setEmail(email);
		contact.setMobilePhone(mobilePhone);
		
		contact.setContactBySMS(true);
		contact.setContactByEmail(true);
		
		return contact;
	}
	
	public User getRecipient(UserHandler userHandler) {
		
		if (recipient == null && userHandler != null) {
			
			recipient = userHandler.getUserByAttribute("HSAID", hsaID, false, true);
		}
		
		return recipient;
	}
	
	public Contact getPosterContact() {
		
		Contact contact = new Contact();
		
		contact.setFirstname(poster.getFirstname());
		contact.setLastname(poster.getLastname());
		contact.setEmail(poster.getEmail());
		contact.setMobilePhone(UserUtils.getAttribute("mobilePhone", poster));
		
		contact.setContactBySMS(UserUtils.getBooleanAttribute("contactBySMS", poster));
		contact.setContactByEmail(true);
		
		return contact;
	}
}
