package com.nordicpeak.flowengine.evaluators.changeposterevaluator;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.UserValueQuery;
import com.nordicpeak.flowengine.interfaces.UserValueQueryInstance;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;

public class ChangePosterEvaluationProviderModule extends BaseEvaluationProviderModule<ChangePosterEvaluator> {

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "Original poster attributes", description = "Attributes to set to original poster")
	@SplitOnLineBreak
	protected List<String> originalPosterAttributes;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "New poster attributes", description = "Attributes to set to new poster")
	@SplitOnLineBreak
	protected List<String> newPosterAttributes;

	@XSLVariable(prefix = "java.")
	protected String changePosterMessage = "Owner changed";

	protected AnnotatedDAO<ChangePosterEvaluator> evaluatorDAO;

	protected QueryParameterFactory<ChangePosterEvaluator, Integer> evaluatorIDParamFactory;

	protected ChangePosterEvaluatorCRUD evaluatorCRUD;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@InstanceManagerDependency(required = true)
	protected FlowInstanceAdminModule flowInstanceAdminModule;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ChangePosterEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(ChangePosterEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new ChangePosterEvaluatorCRUD(ChangePosterEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, ChangePosterEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (!(queryInstance instanceof UserValueQueryInstance)) {

			log.warn("Query instance " + queryInstance + " does not implement interface " + UserValueQueryInstance.class.getName());

			return null;
		}

		if (queryInstance.getQueryInstanceDescriptor().isPopulated()) {

			UserValueQueryInstance userValueQueryInstance = (UserValueQueryInstance) queryInstance;
			User newPoster = userValueQueryInstance.getUserValue(queryHandler);

			if (newPoster == null) {

				log.warn("Unable to find new poster for flow instance " + queryInstance + " from query instance " + userValueQueryInstance);
				throw new RuntimeException("New poster not found");
			}
		}

		return null;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 25)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException {

		FlowInstanceManager flowInstanceManager = event.getFlowInstanceManager();

		List<UserValueQueryInstance> queryInstances = flowInstanceManager.getQueries(UserValueQueryInstance.class);

		if (queryInstances != null) {

			Timestamp now = TimeUtils.getCurrentTimestamp();

			for (UserValueQueryInstance queryInstance : queryInstances) {

				if (queryInstance == null || !queryInstance.getQueryInstanceDescriptor().isPopulated() || queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors() == null) {

					continue;
				}
				
				User oldPoster = flowInstanceManager.getFlowInstance().getPoster();
				User submitter = event.getEvent().getPoster();

				for (ImmutableEvaluatorDescriptor evaluatorDescriptor : queryInstance.getQueryInstanceDescriptor().getQueryDescriptor().getEvaluatorDescriptors()) {

					if (evaluatorDescriptor.isEnabled() && evaluatorDescriptor.getEvaluatorTypeID().equals(this.evaluatorTypeID)) {

						ChangePosterEvaluator evaluator = getEvaluator(evaluatorDescriptor);

						User newPoster = queryInstance.getUserValue(queryHandler);

						if (newPoster != null) {

							log.info("Changing poster and owner of flow instance " + flowInstanceManager.getFlowInstance() + " from " + oldPoster + " to " + newPoster);

							FlowInstance flowInstance = (FlowInstance) flowInstanceManager.getFlowInstance();
							flowInstance.setPoster(newPoster);
							
							if (flowInstance.getOwners() != null) {
								
								flowInstance.getOwners().remove(oldPoster);
								flowInstance.getOwners().add(newPoster);
							}

							if (oldPoster != null) {

								if (!CollectionUtils.isEmpty(originalPosterAttributes)) {

									String userID = oldPoster.getUserID().toString();

									for (String attribute : originalPosterAttributes) {

										flowInstance.addAttribute(attribute, userID);
									}
								}
							}

							if (!CollectionUtils.isEmpty(newPosterAttributes)) {

								String userID = newPoster.getUserID().toString();

								for (String attribute : newPosterAttributes) {

									flowInstance.addAttribute(attribute, userID);
								}
							}

							HighLevelQuery<FlowInstance> updateQuery = new HighLevelQuery<FlowInstance>(FlowInstance.ATTRIBUTES_RELATION, FlowInstance.OWNERS_RELATION);
							flowInstanceAdminModule.getDAOFactory().getFlowInstanceDAO().update(flowInstance, updateQuery, 20);

							if (evaluator.isShowInLog()) {
								flowInstanceAdminModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstanceManager.getFlowInstance(), EventType.OTHER_EVENT, changePosterMessage + " " + newPoster.getFirstname() + " " + newPoster.getLastname(), submitter, now);
							}

							systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<FlowInstance>(FlowInstance.class, CRUDAction.UPDATE, flowInstance), EventTarget.ALL);

						} else {

							log.warn("Unable to change poster of flow instance " + flowInstanceManager.getFlowInstance() + " from " + oldPoster + " as no new poster was found");
						}
					}
				}
			}
		}
	}


	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ChangePosterEvaluator evaluator = new ChangePosterEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	protected ChangePosterEvaluator getEvaluator(ImmutableEvaluatorDescriptor descriptor) throws SQLException {

		return evaluatorDAO.get(new HighLevelQuery<ChangePosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		ChangePosterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<ChangePosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<ChangePosterEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return UserValueQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		ChangePosterEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<ChangePosterEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

	}

	@Override
	public ChangePosterEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		ChangePosterEvaluator evaluator = new ChangePosterEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	public QueryHandler getQueryHandler() {

		return queryHandler;
	}

}
