package com.nordicpeak.flowengine.queries.childinfoquery.beans;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.childinfoquery.enums.LookupMode;

@Table(name = "child_info_queries")
@XMLElement
public class ChildInfoQuery extends BaseQuery {

	private static final long serialVersionUID = -5353994197787383731L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean dontSetPopulated;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showAddress;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private LookupMode lookupMode = LookupMode.POSTER;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "lookupMode", paramValues = "ATTRIBUTE")
	@XMLElement
	private String lookupAttributeName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ChildInfoQueryInstance> instances;

	public boolean isShowAddress() {

		return showAddress;
	}

	public void setShowAddress(boolean showAddress) {

		this.showAddress = showAddress;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<ChildInfoQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<ChildInfoQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public boolean isDontSetPopulated() {

		return dontSetPopulated;
	}

	public void setDontSetPopulated(boolean dontSetPopulated) {

		this.dontSetPopulated = dontSetPopulated;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return getClass().getSimpleName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return getClass().getSimpleName() + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		dontSetPopulated = xmlParser.getPrimitiveBoolean("dontSetPopulated");
		showAddress = xmlParser.getPrimitiveBoolean("showAddress");

		lookupMode = XMLValidationUtils.validateParameter("lookupMode", xmlParser, true, new EnumPopulator<LookupMode>(LookupMode.class), errors);

		if (lookupMode != null && lookupMode == LookupMode.ATTRIBUTE) {

			lookupAttributeName = XMLValidationUtils.validateParameter("lookupAttributeName", xmlParser, true, 1, 255, StringPopulator.getPopulator(), errors);
		}

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public LookupMode getLookupMode() {

		return lookupMode;
	}

	public void setLookupMode(LookupMode lookupMode) {

		this.lookupMode = lookupMode;
	}

	public String getLookupAttributeName() {

		return lookupAttributeName;
	}

	public void setLookupAttributeName(String lookupAttributeName) {

		this.lookupAttributeName = lookupAttributeName;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

}
