package com.nordicpeak.flowengine.queries.childinfoquery.beans;

import java.lang.reflect.Field;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.childrelationprovider.exceptions.ChildRelationProviderException;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@Table(name = "child_info_query_instances")
@XMLElement
public class ChildInfoQueryInstance extends BaseQueryInstance {

	private static final long serialVersionUID = -6538694830224396801L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(ChildInfoQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private ChildInfoQuery query;

	@XMLElement
	private boolean hasChildrenUnderSecrecy;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(childName = "Children")
	private List<StoredChild> children;

	private ChildRelationProviderException fetchChildrenException;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public ChildInfoQuery getQuery() {

		return query;
	}

	public void setQuery(ChildInfoQuery query) {

		this.query = query;
	}

	public List<StoredChild> getChildren() {

		return children;
	}

	public void setChildren(List<StoredChild> children) {

		this.children = children;
	}

	public boolean isHasChildrenUnderSecrecy() {

		return hasChildrenUnderSecrecy;
	}

	public void setHasChildrenUnderSecrecy(boolean hasChildrenUnderSecrecy) {

		this.hasChildrenUnderSecrecy = hasChildrenUnderSecrecy;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		children = null;
		hasChildrenUnderSecrecy = false;
		fetchChildrenException = null;

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}
		
		super.reset(attributeHandler);
	}

	public ChildRelationProviderException getFetchChildrenException() {

		return fetchChildrenException;
	}

	public void setFetchChildrenException(ChildRelationProviderException fetchChildrenExceptions) {

		this.fetchChildrenException = fetchChildrenExceptions;
	}

	@Override
	public String toString() {

		return getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		return element;
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (!CollectionUtils.isEmpty(children)) {

			XMLUtils.append(doc, element, "Children", children);

		} else if (children != null) {

			XMLUtils.appendNewElement(doc, element, "Children");
		}

		if (fetchChildrenException != null) {

			Element fetchChildrenExceptionElement = XMLUtils.appendNewElement(doc, element, "FetchChildrenException");
			fetchChildrenExceptionElement.appendChild(fetchChildrenException.toXML(doc));
		}

		return element;
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(query.getAttributeName());
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		if(children == null) {
			
			resetAttribute(attributeHandler);
		
		} else {
			
			StringBuilder stringBuilder = new StringBuilder();
			
			for(StoredChild child : children) {
				
				if(stringBuilder.length() != 0) {
					
					stringBuilder.append(" ,");
				}
				
				stringBuilder.append(child.getFirstname() + " " + child.getLastname() + " (" + child.getCitizenIdentifier() + ")");
			}
			
			attributeHandler.setAttribute(query.getAttributeName(), stringBuilder.toString());
		}
	}
}
