package com.nordicpeak.flowengine.queries.childinfoquery.beans;

import java.io.Serializable;
import java.util.Date;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.childrelationprovider.Child;

@Table(name = "child_info_query_children")
@XMLElement(name = "Child")
public class StoredChild extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = -686938949095711911L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer childID;

	@DAOManaged(columnName = "queryInstanceID")
	@ManyToOne
	private ChildInfoQueryInstance queryInstance;

	@DAOManaged
	@XMLElement
	private String citizenIdentifier;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String address;

	@DAOManaged
	@XMLElement
	private String zipCode;

	@DAOManaged
	@XMLElement
	private String postalAddress;
	
	public StoredChild() {}

	public StoredChild(Child child) {

		super();

		citizenIdentifier = child.getCitizenIdentifier();
		firstname = child.getFirstname();
		lastname = child.getLastname();
		address = child.getAddress();
		zipCode = child.getZipCode();
		postalAddress = child.getPostalAddress();
	}

	public Integer getChildID() {

		return childID;
	}

	public void setChildID(Integer childID) {

		this.childID = childID;
	}

	public ChildInfoQueryInstance getQueryInstance() {

		return queryInstance;
	}

	public void setQueryInstance(ChildInfoQueryInstance queryInstance) {

		this.queryInstance = queryInstance;
	}

	public String getCitizenIdentifier() {

		return citizenIdentifier;
	}

	public String getFirstname() {

		return firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getZipCode() {

		return zipCode;
	}

	public void setZipCode(String zipCode) {

		this.zipCode = zipCode;
	}

	public String getPostalAddress() {

		return postalAddress;
	}

	public void setPostalAddress(String postalAddress) {

		this.postalAddress = postalAddress;
	}

	/** @return Returns the age solely based on the birth year ignoring months and days */
	public Integer getAge() {

		if (citizenIdentifier != null) {

			int birthYear = Integer.parseInt(citizenIdentifier.substring(0, 4));

			int currentYear = Integer.parseInt(DateUtils.YEAR_FORMATTER.format(new Date()));

			return currentYear - birthYear;
		}

		return null;
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		XMLUtils.appendNewElement(doc, element, "Age", getAge());

		return element;
	}

	@Override
	public String toString() {

		return firstname + " " + lastname + " (" + citizenIdentifier + ")";
	}

}
