package com.nordicpeak.flowengine.evaluators.citizenidentifier;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CaseInsensitiveStringComparator;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.SwedishSocialSecurity12DigitsWithoutMinusPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluator;

@Table(name = "citizen_identifier_query_state_evaluators")
@XMLElement
public class CitizenIdentifierQueryStateEvaluator extends BaseQueryStateEvaluator{

	private static final long serialVersionUID = 7582032280793947029L;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "citizen_identifier_query_state_evaluator_identifiers", remoteValueColumnName = "citizenIdentifier")
	@WebPopulate(required = true, populator = SwedishSocialSecurity12DigitsWithoutMinusPopulator.class)
	@SplitOnLineBreak
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@XMLElement(name="AllowedCitizenIdentifiers",childName="CitizenIdentifier")
	private List<String> allowedCitizenIdentifiers;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean inclusive;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		super.populate(xmlParser);
		
		List<ValidationError> errors = new ArrayList<ValidationError>(1);
		
		inclusive = xmlParser.getPrimitiveBoolean("inclusive");
		
		allowedCitizenIdentifiers = XMLValidationUtils.validateParameters("AllowedCitizenIdentifiers/CitizenIdentifier", xmlParser, false, SwedishSocialSecurity12DigitsWithoutMinusPopulator.getPopulator(), errors);
		
		if(!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}

	public List<String> getAllowedCitizenIdentifiers() {

		return allowedCitizenIdentifiers;
	}

	public void setAllowedCitizenIdentifiers(List<String> allowedCitizenIdentifiers) {

		this.allowedCitizenIdentifiers = allowedCitizenIdentifiers;
	}

	public boolean isInclusive() {

		return inclusive;
	}

	public void setInclusive(boolean inclusive) {

		this.inclusive = inclusive;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}
}
