package com.nordicpeak.flowengine.integration.cityworks.beans;

import java.util.List;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.integration.base.beans.BaseIntegrationSettings;

@Table(name = "cityworks_integration_settings")
@XMLElement
public class IntegrationSettings extends BaseIntegrationSettings {
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "password")
	@XMLElement
	private String username;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "username")
	@XMLElement
	private String password;
	
	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoGet = true)
	@XMLElement
	private IntegrationEndpoint endpoint;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private Integer problemSid;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private Integer priorityID;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String address;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String coordinates;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String details;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private AttachmentMode attachmentMode;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String callerFirstname;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "enabled")
	@XMLElement
	private String callerLastname;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String callerEmail;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String callerPhone;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String callerWantsFollowUpCall;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendErrorEmail;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "cityworks_integration_email_recipients_error", remoteValueColumnName = "email")
	@WebPopulate(maxLength = 255, populator = EmailPopulator.class)
	@RequiredIfSet(paramNames = "sendErrorEmail")
	@SplitOnLineBreak
	@XMLElement(fixCase = true, childName = "Address")
	private List<String> errorEmailAddresses;
	
	public String getUsername() {
		
		return username;
	}
	
	public void setUsername(String username) {
		
		this.username = username;
	}
	
	public String getPassword() {
		
		return password;
	}
	
	public void setPassword(String password) {
		
		this.password = password;
	}
	
	@Override
	public boolean isPopulated() {
		
		if (isEnabled() || !ObjectUtils.isNull(endpoint, username, password, problemSid, coordinates, details, address, callerFirstname, callerLastname, callerEmail, callerPhone)) {
			return true;
		}
		
		return false;
	}
	
	public IntegrationEndpoint getEndpoint() {
		return endpoint;
	}
	
	public void setEndpoint(IntegrationEndpoint integrationEndpoint) {
		this.endpoint = integrationEndpoint;
	}
	
	public Integer getProblemSid() {
		return problemSid;
	}
	
	public void setProblemSid(Integer problemSid) {
		this.problemSid = problemSid;
	}
	
	public String getAddress() {
		return address;
	}
	
	public void setAddress(String address) {
		this.address = address;
	}
	
	public String getCoordinates() {
		return coordinates;
	}
	
	public void setCoordinates(String cooridnates) {
		this.coordinates = cooridnates;
	}
	
	public String getDetails() {
		return details;
	}
	
	public void setDetails(String details) {
		this.details = details;
	}
	
	public String getCallerFirstname() {
		return callerFirstname;
	}
	
	public void setCallerFirstname(String callerFirstname) {
		this.callerFirstname = callerFirstname;
	}
	
	public String getCallerLastname() {
		return callerLastname;
	}
	
	public void setCallerLastname(String callerLastname) {
		this.callerLastname = callerLastname;
	}
	
	public String getCallerEmail() {
		return callerEmail;
	}
	
	public void setCallerEmail(String callerEmail) {
		this.callerEmail = callerEmail;
	}
	
	public String getCallerPhone() {
		return callerPhone;
	}
	
	public void setCallerPhone(String callerPhone) {
		this.callerPhone = callerPhone;
	}
	
	public String getCallerWantsFollowUpCall() {
		return callerWantsFollowUpCall;
	}
	
	public void setCallerWantsFollowUpCall(String callerWantsFollowUpCall) {
		this.callerWantsFollowUpCall = callerWantsFollowUpCall;
	}
	
	public AttachmentMode getAttachmentMode() {
		return attachmentMode;
	}
	
	public void setAttachmentMode(AttachmentMode attachmentMode) {
		this.attachmentMode = attachmentMode;
	}
	
	public Integer getPriorityID() {
		return priorityID;
	}
	
	public void setPriorityID(Integer priorityID) {
		this.priorityID = priorityID;
	}

	
	public boolean isSendErrorEmail() {
	
		return sendErrorEmail;
	}

	
	public void setSendErrorEmail(boolean sendErrorEmail) {
	
		this.sendErrorEmail = sendErrorEmail;
	}

	
	public List<String> getErrorEmailAddresses() {
	
		return errorEmailAddresses;
	}

	
	public void setErrorEmailAddresses(List<String> errorEmailAddresses) {
	
		this.errorEmailAddresses = errorEmailAddresses;
	}
	
}
