package com.nordicpeak.flowengine.queries.colorpickerquery;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;

@Table(name = "color_picker_queries")
@XMLElement
public class ColorPickerQuery extends BaseQuery {
	
	private static final long serialVersionUID = -842191226937409416L;
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useAlpha;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean usePalette;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useComparison;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private ColorPickerFormat preferedFormat = ColorPickerFormat.HEX;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ColorPickerQueryInstance> instances;
	
	public static long getSerialversionuid() {
		
		return serialVersionUID;
	}
	
	@Override
	public Integer getQueryID() {
		
		return queryID;
	}
	
	public List<ColorPickerQueryInstance> getInstances() {
		
		return instances;
	}
	
	public void setInstances(List<ColorPickerQueryInstance> instances) {
		
		this.instances = instances;
	}
	
	public void setQueryID(int queryID) {
		
		this.queryID = queryID;
	}
	
	public boolean isUseAlpha() {
		
		return useAlpha;
	}
	
	public void setUseAlpha(boolean useDateFields) {
		
		this.useAlpha = useDateFields;
	}
	
	public boolean isUsePalette() {
		
		return usePalette;
	}
	
	public void setUsePalette(boolean useTimeFields) {
		
		this.usePalette = useTimeFields;
	}
	
	public boolean isUseComparison() {
		
		return useComparison;
	}
	
	public void setUseComparison(boolean useMultiDate) {
		
		this.useComparison = useMultiDate;
	}
	
	public ColorPickerFormat getPreferedFormat() {
		return preferedFormat;
	}
	
	public void setPreferedFormat(ColorPickerFormat preferedFormat) {
		this.preferedFormat = preferedFormat;
	}
	
	public void setQueryID(Integer queryID) {
		
		this.queryID = queryID;
	}
	
	@Override
	public String toString() {
		
		if (this.queryDescriptor != null) {
			
			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}
		
		return "ColorPickerQuery (queryID: " + queryID + ")";
	}
	
	@Override
	public String getXSDTypeName() {
		
		return "ColorPickerQuery" + queryID;
	}
	
	@Override
	public void toXSD(Document doc) {
		
		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());
		
		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);
		
		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);
		
		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);
		
		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);
		
		Element valueElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		valueElement.setAttribute("name", "Value");
		valueElement.setAttribute("type", "xs:string");
		valueElement.setAttribute("minOccurs", "1");
		valueElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(valueElement);
		
		doc.getDocumentElement().appendChild(complexTypeElement);
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		
		useAlpha = xmlParser.getPrimitiveBoolean("useDateFields");
		usePalette = xmlParser.getPrimitiveBoolean("useTimeFields");
		useComparison = xmlParser.getPrimitiveBoolean("useMultiDate");
		
		preferedFormat = null;
		String format = XMLValidationUtils.validateParameter("preferedFormat", xmlParser, true, 1, 50, StringPopulator.getPopulator(), errors);
		
		if (!StringUtils.isEmpty(format)) {
			
			preferedFormat = ColorPickerFormat.valueOf(format);
		}
		
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (attributeName != null) {
			
			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
	}
	
	public boolean isSetAsAttribute() {
		
		return setAsAttribute;
	}
	
	public void setSetAsAttribute(boolean setAsAttribute) {
		
		this.setAsAttribute = setAsAttribute;
	}
	
	public String getAttributeName() {
		
		return attributeName;
	}
	
	public void setAttributeName(String attributeName) {
		
		this.attributeName = attributeName;
	}
	
}
