<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/constructionpermitinline.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/constructionpermit.css
	</xsl:variable>

	<xsl:variable name="countOptions">
		<option><value>1</value></option>
		<option><value>2</value></option>
		<option><value>3</value></option>
		<option><value>4</value></option>
		<option><value>5</value></option>
	</xsl:variable>
	
	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ConstructionPermitQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ConstructionPermitQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="ConstructionPermitQueryInstance/ConstructionPermitQuery/queryID" />
					</xsl:call-template>
					
				</div>

				<xsl:if test="ConstructionPermitQueryInstance/ConstructionPermitQuery/description">
					<span class="italic">
						<xsl:value-of select="ConstructionPermitQueryInstance/ConstructionPermitQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:apply-templates select="ConstructionPermitQueryInstance/Values/PermitNodeValue" mode="show"/>
				
			</article>
				
		</div>
		
	</xsl:template>	
	
	<xsl:template match="PermitNodeValue" mode="show">
	
		<div class="full border bigmarginbottom padding border-radius-small floatleft border-box">
		
			<strong>
			
				<xsl:choose>
					<xsl:when test="PermitNode/longTitle">
						<xsl:value-of select="PermitNode/longTitle"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="PermitNode/title"/>
					</xsl:otherwise>
				</xsl:choose>
			</strong>
	
			
			<div class="floatright">
			
				<xsl:if test="PermitNode/requiresCount = 'true'">
			
					<xsl:value-of select="$i18n.Count"/>
					<xsl:text>: </xsl:text>
					<xsl:value-of select="count"/>
				
				</xsl:if>

			</div>			
			
			<xsl:if test="PermitNode/requiresDescription = 'true'">
			
				<div class="floatleft full">
				
					<label>
						<xsl:value-of select="$i18n.Description"/>
						<xsl:text>:</xsl:text>
					</label>
				
					<xsl:value-of select="description"/>
				</div>
			
			</xsl:if>
			
			<xsl:if test="PermitNode/requiresFromToDescription = 'true'">
			
				<div class="full floatleft">

					<div class="floatleft permit-to-from-description permit-split-margin">
										
						<label>
							<xsl:value-of select="$i18n.From"/>
							<xsl:text>:</xsl:text>
						</label>
					
						<xsl:value-of select="fromDescription"/>
					</div>
					
					
					<div class="floatleft permit-to-from-description">
					
						<label>
							<xsl:value-of select="$i18n.To"/>
							<xsl:text>:</xsl:text>
						</label>
					
						<xsl:value-of select="toDescription"/>
						
					</div>
				
				</div>
				
			</xsl:if>
		
		</div>	
	
	</xsl:template>
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', ConstructionPermitQueryInstance/ConstructionPermitQuery/queryID)" />
	
		<xsl:variable name="shortQueryID" select="concat('q', ConstructionPermitQueryInstance/ConstructionPermitQuery/queryID)" />
	
		<div class="query constructionpermitquery" id="{$queryID}">
			
			<xsl:if test="EnableAjaxPosting">
				<xsl:attribute name="class">query constructionpermitquery enableAjaxPosting</xsl:attribute>
			</xsl:if>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError[messageKey]">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ConstructionPermitQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ConstructionPermitQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="ConstructionPermitQueryInstance/ConstructionPermitQuery/helpText">		
						<xsl:apply-templates select="ConstructionPermitQueryInstance/ConstructionPermitQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:if test="ConstructionPermitQueryInstance/ConstructionPermitQuery/description">
					<span class="italic query-description">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description query-description</xsl:attribute></xsl:if>
						<xsl:value-of select="ConstructionPermitQueryInstance/ConstructionPermitQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<xsl:choose>
					<xsl:when test="not(ConstructionPermitQueryInstance/ConstructionPermitQuery/Model)">
					
						<span>
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.NoModelFound"/>
							</strong>
						</span>
					
					</xsl:when>
					<xsl:otherwise>
					
						<div class="permit-tree" id="permit-tree-{ConstructionPermitQueryInstance/ConstructionPermitQuery/queryID}">
							<xsl:if test="ConstructionPermitQueryInstance/ConstructionPermitQuery/selectPermitsDescription">
								<span>
									<xsl:value-of select="ConstructionPermitQueryInstance/ConstructionPermitQuery/selectPermitsDescription" disable-output-escaping="yes"/>
								</span>
							</xsl:if>
							
							<xsl:apply-templates select="ConstructionPermitQueryInstance/ConstructionPermitQuery/Model/RootNodes/PermitNode" mode="tree-roots">
								<xsl:with-param name="values" select="ConstructionPermitQueryInstance/Values/PermitNodeValue"/>
								<xsl:with-param name="shortQueryID" select="$shortQueryID"/>
								<xsl:with-param name="validationErrors" select="ValidationErrors"/>
							</xsl:apply-templates>
						</div>
					
					</xsl:otherwise>
				</xsl:choose>
				
				<div class="subselection-required-template" style="display: none;">
					<span class="subselection-required-error">
						<i data-icon-after="!" title="{$i18n.Error.SubSelectionRequiredField}"/>
						<span class="bigmarginleft"><xsl:value-of select="$i18n.Error.SubSelectionRequiredField"/></span>
					</span>
				</div>
				
			</article>			
		
			<script type="text/javascript">
				initConstructionPermitQuery('<xsl:value-of select="ConstructionPermitQueryInstance/ConstructionPermitQuery/queryID" />');
			</script>		
		
		</div>
		
	</xsl:template>
	
 	<xsl:template match="PermitNode" mode="permit-list">
	
		<xsl:param name="values"/>
		<xsl:param name="shortQueryID"/>
		<xsl:param name="validationErrors"/>
	
		<xsl:choose>
			<xsl:when test="titleNode = 'true'">
				
				<xsl:apply-templates select="ChildNodes/PermitNode" mode="permit-list">
					<xsl:with-param name="values" select="$values"/>
					<xsl:with-param name="shortQueryID" select="$shortQueryID"/>
					<xsl:with-param name="validationErrors" select="$validationErrors"/>
				</xsl:apply-templates>				
				
			</xsl:when>
			<xsl:otherwise>
			
				<xsl:variable name="permitNodeID" select="permitNodeID"/>
			
				<xsl:variable name="valueNode" select="$values[PermitNode/permitNodeID = $permitNodeID]"/>
			
				<div class="hidden full border bigmarginbottom padding border-radius-small floatleft border-box" id="permit-form-{permitNodeID}">
			
					<xsl:if test="$valueNode">
					
						<xsl:attribute name="class">full border bigmarginbottom padding border-radius-small floatleft border-box</xsl:attribute>
					
					</xsl:if>
			
					<strong>
					
						<xsl:choose>
							<xsl:when test="longTitle">
								<xsl:value-of select="longTitle"/>
							</xsl:when>
							<xsl:otherwise>
									<xsl:value-of select="title"/>
							</xsl:otherwise>
						</xsl:choose>
					</strong>
			
					<input type="hidden" name="{$shortQueryID}_permitid[]" id="permit-hidden-{permitNodeID}" value="{permitNodeID}">
					
						<xsl:if test="not($valueNode)">
					
							<xsl:attribute name="disabled">disabled</xsl:attribute>
					
						</xsl:if>
					
					</input>
					
					<div class="floatright">
					
						<xsl:if test="requiresCount = 'true'">
					
							<xsl:value-of select="$i18n.Count"/>
							<xsl:text>:&#160;</xsl:text>
							
							<!-- TODO Validation error for count -->
							
							<xsl:call-template name="createDropdown">
								<xsl:with-param name="name">
									<xsl:value-of select="$shortQueryID"/>
									<xsl:text>_node_</xsl:text>
									<xsl:value-of select="permitNodeID"/>
									<xsl:text>_count</xsl:text>
								</xsl:with-param>
								<xsl:with-param name="element" select="exsl:node-set($countOptions)/option" />
								<xsl:with-param name="labelElementName" select="'value'"/>
								<xsl:with-param name="valueElementName" select="'value'"/>
								<xsl:with-param name="labelDelimiter" select="'st'"/>
								<xsl:with-param name="selectedValue" select="$valueNode/count"/>
								<xsl:with-param name="class">permitcount</xsl:with-param>
							</xsl:call-template>
						
						</xsl:if>
					</div>			
					
					<xsl:if test="requiresDescription = 'true'">
					
						<div class="floatleft full">
						
							<xsl:variable name="fieldName">
								<xsl:value-of select="$shortQueryID"/>
								<xsl:text>_node_</xsl:text>
								<xsl:value-of select="permitNodeID"/>
								<xsl:text>_description</xsl:text>
							</xsl:variable>
						
							<xsl:if test="$validationErrors/validationError[fieldName = $fieldName]">
								<xsl:attribute name="class">
									<xsl:text>floatleft full invalid input-error</xsl:text>
								</xsl:attribute>
							</xsl:if>						
						
							<label for="{$shortQueryID}_node_{permitNodeID}_description">
								<xsl:value-of select="$i18n.Description"/>
								<xsl:text>:</xsl:text>
							</label>
						
							<input type="text" name="{$fieldName}" value="{$valueNode/description}" id="{$fieldName}" style="width: 100%;" maxlength="255"/>
							
							<xsl:apply-templates select="$validationErrors/validationError[fieldName = $fieldName]"/>
						</div>
					
					</xsl:if>
					
					<xsl:if test="requiresFromToDescription = 'true'">
					
						<div class="full floatleft">

							<div class="floatleft permit-to-from-description permit-split-margin">
							
								<xsl:variable name="fieldName">
									<xsl:value-of select="$shortQueryID"/>
									<xsl:text>_node_</xsl:text>
									<xsl:value-of select="permitNodeID"/>
									<xsl:text>_from</xsl:text>
								</xsl:variable>
							
								<xsl:if test="$validationErrors/validationError[fieldName = $fieldName]">
									<xsl:attribute name="class">
										<xsl:text>floatleft permit-to-from-description permit-split-margin invalid input-error</xsl:text>
									</xsl:attribute>
								</xsl:if>	
							
								<label for="{$shortQueryID}_node_{permitNodeID}_from">
									<xsl:value-of select="$i18n.From"/>
									<xsl:text>:</xsl:text>
								</label>
							
								<input type="text" name="{$fieldName}" value="{$valueNode/fromDescription}" id="{$fieldName}" style="width: 100%;" maxlength="255"/>
								
								<xsl:apply-templates select="$validationErrors/validationError[fieldName = $fieldName]"/>
							</div>
							
							
							<div class="floatleft permit-to-from-description">
							
								<xsl:variable name="fieldName">
									<xsl:value-of select="$shortQueryID"/>
									<xsl:text>_node_</xsl:text>
									<xsl:value-of select="permitNodeID"/>
									<xsl:text>_to</xsl:text>
								</xsl:variable>
							
								<xsl:if test="$validationErrors/validationError[fieldName = $fieldName]">
									<xsl:attribute name="class">
										<xsl:text>floatleft permit-to-from-description invalid input-error</xsl:text>
									</xsl:attribute>
								</xsl:if>							
							
								<label for="{$shortQueryID}_node_{permitNodeID}_to">
									<xsl:value-of select="$i18n.To"/>
									<xsl:text>:</xsl:text>
								</label>
							
								<input type="text" name="{$fieldName}" value="{$valueNode/toDescription}" id="{$fieldName}" style="width: 100%;" maxlength="255"/>
								
								<xsl:apply-templates select="$validationErrors/validationError[fieldName = $fieldName]"/>
							</div>
						
						</div>
											
					</xsl:if>
					
				</div>			
			
			</xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>
	
	 <xsl:template match="PermitNode" mode="tree-roots">
	
		<xsl:param name="values"/>
		<xsl:param name="shortQueryID"/>	
		<xsl:param name="validationErrors"/>
		
		<div class="border padding marginbottom">
			<a href="#" class="permit-foldable">
				<img src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/nolines_minus.gif" alt="" class="permit-minus"/>
				<img src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/nolines_plus.gif" alt="" class="permit-plus"/>
				
				<xsl:text> </xsl:text>
				
				<strong>
					<xsl:value-of select="title"/>
				</strong>
			</a>
			
			<div class="hidden permit-children" style="margin-left: 11px !important;">
				<xsl:if test="ChildNodes/PermitNode">
					<ul class="list-style-type-none">
						<xsl:apply-templates select="ChildNodes/PermitNode" mode="tree-child">
							<xsl:with-param name="values" select="$values"/>
							<xsl:with-param name="shortQueryID" select="$shortQueryID"/>
							<xsl:with-param name="validationErrors" select="$validationErrors"/>
						</xsl:apply-templates>
					</ul>
				</xsl:if>
			</div>
		</div>
		
	</xsl:template>
	
	 <xsl:template match="PermitNode" mode="tree-child">
	
		<xsl:param name="values"/>
		<xsl:param name="shortQueryID"/>
		<xsl:param name="validationErrors"/>
		
		<xsl:choose>
		
			<xsl:when test="titleNode = 'true'">
			
				<li class="clearfix">
				
					<xsl:variable name="childNodes" select="ChildNodes"/>
					<xsl:variable name="childSelected" select="$values[PermitNode/permitNodeID = $childNodes/PermitNode/permitNodeID]"/>
					
					<input type="checkbox" id="permit-foldable-{permitNodeID}" class="permit-foldable-checkbox">
					
						<xsl:if test="$childSelected">
						
							<xsl:attribute name="checked">checked</xsl:attribute>
						
						</xsl:if>
					
					</input>
					
					<label for="permit-foldable-{permitNodeID}">
					
						<xsl:attribute name="class">
							<xsl:choose>
								<xsl:when test="$childSelected">checkbox permit-foldable permit-open</xsl:when>
								<xsl:otherwise>checkbox permit-foldable</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
						
						<xsl:value-of select="title"/>
						
					</label>
					
					<ul style="margin-left: 21px !important;">
					
						<xsl:attribute name="class">
							<xsl:choose>
								<xsl:when test="$childSelected">list-style-type-none permit-children</xsl:when>
								<xsl:otherwise>list-style-type-none hidden permit-children</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>

						<xsl:apply-templates select="ChildNodes/PermitNode" mode="tree-child">
							<xsl:with-param name="values" select="$values"/>
							<xsl:with-param name="shortQueryID" select="$shortQueryID"/>
							<xsl:with-param name="validationErrors" select="$validationErrors"/>
						</xsl:apply-templates>				

					</ul>
				</li>
			
			</xsl:when>
			<xsl:otherwise>
				
				<li class="clearfix">
					<input type="checkbox" data-id="{permitNodeID}" class="permit-toggler" id="permit-toggler-{permitNodeID}">
					
						<xsl:variable name="permitNodeID" select="permitNodeID"/>
			
						<xsl:variable name="valueNode" select="$values[PermitNode/permitNodeID = $permitNodeID]"/>
					
						<xsl:if test="$valueNode">
						
							<xsl:attribute name="checked">checked</xsl:attribute>
						
						</xsl:if>
					
					</input>
					
					<label for="permit-toggler-{permitNodeID}" class="checkbox"><xsl:value-of select="title"/></label>
					
					<xsl:apply-templates select="." mode="permit-list">
						<xsl:with-param name="values" select="$values"/>
						<xsl:with-param name="shortQueryID" select="$shortQueryID"/>
						<xsl:with-param name="validationErrors" select="$validationErrors"/>
					</xsl:apply-templates>
				</li>
				
			</xsl:otherwise>
		
		</xsl:choose>
	
	</xsl:template>		
	
	<xsl:template match="validationError[messageKey = 'NoPermitsSelected']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.NoPermitsSelected"/>			
			</strong>
		</span>
		
	</xsl:template>		
	
	<xsl:template match="validationError[messageKey = 'SelectedPermitsNotFound']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.SelectedPermitsNotFound"/>
			</strong>
		</span>
		
	</xsl:template>		
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.Error.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:value-of select="$i18n.Error.TooLongFieldContent.part1"/>
				<xsl:value-of select="currentLength"/>
				<xsl:value-of select="$i18n.Error.TooLongFieldContent.part2"/>
				<xsl:value-of select="maxLength"/>
				<xsl:value-of select="$i18n.Error.TooLongFieldContent.part3"/>
			</xsl:attribute>
		</i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!">
			<xsl:attribute name="title">
				<xsl:choose>
					<xsl:when test="invalidFormatMessage">
						<xsl:value-of select="invalidFormatMessage"/>
					</xsl:when>
					<xsl:otherwise>
							<xsl:value-of select="$i18n.Error.InvalidFormat"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</i>
		
	</xsl:template>	
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>
		
</xsl:stylesheet>