package com.nordicpeak.flowengine.contactdetailsverification;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

public class ContactDetailsVerificationPublicModule extends AnnotatedForegroundModule {
	
	private ContactDetailsVerificationModule parentModule;

	public void setParentModule(ContactDetailsVerificationModule parentModule) {

		this.parentModule = parentModule;
	}

	public ForegroundModuleDescriptor getDescriptor() {

		return moduleDescriptor;
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse verifyEmail(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String uuid = req.getParameter("uuid");
		
		User verificationUser = null;
		
		if (StringUtils.isEmpty(uuid) || (verificationUser = systemInterface.getUserHandler().getUserByAttribute(ContactDetailsVerificationModule.EMAIL_UUID, uuid, false, true)) == null) {
			log.info("Unable to verify email for user " + user + " with uuid " + uuid);
			
			throw new URINotFoundException(uriParser);
		}
		
		Document doc = parentModule.createDocument(req, uriParser, user);
		Element verifyElement = doc.createElement("VerifyEmail");
		doc.getFirstChild().appendChild(verifyElement);
		
		XMLUtils.appendNewElement(doc, verifyElement, "EmailVerified");
		
		parentModule.setEmailVerified(user, verificationUser);
		
		return new SimpleForegroundModuleResponse(doc);
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse verifyPhone(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		String uuid = req.getParameter("uuid");
		
		User verificationUser = null;
		
		if (StringUtils.isEmpty(uuid) || (verificationUser = systemInterface.getUserHandler().getUserByAttribute(ContactDetailsVerificationModule.PHONE_UUID, uuid, false, true)) == null) {
			log.info("Unable to verify phone for user " + user + " with uuid " + uuid);
			
			throw new URINotFoundException(uriParser);
		}
		
		Document doc = parentModule.createDocument(req, uriParser, user);
		Element verifyElement = doc.createElement("VerifyPhone");
		doc.getFirstChild().appendChild(verifyElement);
		
		XMLUtils.appendNewElement(doc, verifyElement, "PhoneVerified");
		
		parentModule.setPhoneVerified(user, verificationUser);
		
		return new SimpleForegroundModuleResponse(doc);
	}
}