var i18nCullingModule = {
	"EMPTY_TABLE": "There are no instances to be shown",
	"NEXT": "Next",
	"PREVIOUS": "Previous",
	"YES": "Yes",
	"NO": "No",
	"CONFIRM_DELETE": "Do you really want to delete the selected",
	"SEARCH": "Search",
	"ZERO_RECORDS": "No matching records found",
	"COMMENT": "Comment"
};

function showComment(id) {
	$.featherlight('<div><label>'+i18nCullingModule.COMMENT+'</label><br>'+$("#"+id).text()+'</div>', null);
	
};

$(function() {
	$("#CullingModule").on("click", ".status-button", function(e) {
		e.preventDefault();
		
		var $button = $(this);
		
		$button.toggleClass("active");
		
		$button.find("input").prop("checked", $button.hasClass("active"));
	});
	
	var datatable = $("#instances-table").DataTable({
		"deferRender": true,
		"info": false,
		"searching": false,
		"order": [[ 1, 'asc' ]],
		"lengthMenu": [ 15, 50, 100, 200, 500 ],
		"select": {
            "style": 'multi',
            "selector": 'td:first-child'
        },
		"initComplete": function(settings, json) {
			
			$(window).trigger("resize");
        },
        "columns": [
        	{
        		"name": "select",
        		"className": 'select-checkbox',
        		"orderable": false,
        		"searchable": false
        	},
    		{
    			"name": "instanceID",
    			"render": function(data, type, row, meta) {
    				return data;
    			}
    		},
    		{
    			"name": "version",
    			"render": function(data, type, row, meta) {
    				return data;
    			}
    		},
    		{
    			"name": "date",
    			"render": function(data, type, row, meta) {
    				return data;
    			}
    		},
    		{
    			"name": "submitted",
    			"render": function(data, type, row, meta) {
    				if (data) {
    					return i18nCullingModule.YES;	
    				}
    				
    				return i18nCullingModule.NO;
    			}
    		}
    	],
    	"dom": 'frtipl',
		"language": {
			"emptyTable": i18nCullingModule.EMPTY_TABLE,
			"lengthMenu": i18nCullingModule.LENGTH_MENU,
			"paginate": {
				"next": i18nCullingModule.NEXT,
				"previous": i18nCullingModule.PREVIOUS
			}
		},
		preDrawCallback: function(settings) {
			
			// Remove callback
			settings.aoPreDrawCallback.splice(0, 1);
			
			settings.oClasses.sPageButton = "btn btn-inline btn-light";
		},
    });
	
	var allDatatable = $("#all-instances-table").DataTable({
		"ajax": $("#all-instances-table").data("fetchurl"),
		"deferRender": true,
		"lengthMenu": [ 15, 50, 100, 200, 500 ],
		"order": [[ 6, 'desc' ]],
		"initComplete": function(settings, json) {
			
			$(window).trigger("resize");
        },
        autoWidth: false,
        "columns": [
    		{
    			"data": 0,
    			"name": "instanceID"
    		},
    		{
    			"data": 1,
    			"name": "familyID"
    		},
    		{
    			"data": 2,
    			"name": "flow",
    			"render": function(data, type, row, meta) {
    				return row[3] + " (ID: " + data + ")";
    			}
    		},
    		{
    			"data": 4,
    			"name": "added"
    		},
    		{
    			"data": 5,
    			"name": "submitted"
    		},
    		{
    			"data": 6,
    			"name": "deletedBy",
    			"render": function(data, type, row, meta) {
    				return row[7] + " (ID: " + data + ")";
    			}
    		},
    		{
    			"data": 8,
    			"name": "deleted"
    		},
    		{
    			"data": 9,
    			"name": "comment",
    			"searchable": false,
    			"orderable": false,
    			"render": function(data, type, row, meta) {
    				if (data) {
						var id = "comment_"+ meta.row;
    					return `<div class='hidden' id='${id}'>` + data +`</div><a href='#' onclick="showComment('${id}');"><span data-icon-before='M' ></a>`;						
    				}
    				return null;
       			}
    		}
    	],
		"language": {
			"emptyTable": i18nCullingModule.EMPTY_TABLE,
			"info":           i18nCullingModule.INFO,
			"infoEmpty":      i18nCullingModule.INFO_EMPTY,
			"infoFiltered":   i18nCullingModule.INFO_FILTERED,
			"infoPostFix":    "",
			"paginate": {
				"next": i18nCullingModule.NEXT,
				"previous": i18nCullingModule.PREVIOUS
			},
			"search": i18nCullingModule.SEARCH,
			"zeroRecords": i18nCullingModule.ZERO_RECORDS,
			"lengthMenu": i18nCullingModule.LENGTH_MENU
		},
		preDrawCallback: function(settings) {
			
			// Remove callback
			settings.aoPreDrawCallback.splice(0, 1);
			
			settings.oClasses.sPageButton = "btn btn-inline btn-light";
		},
    });
	
	$("#instances-wrapper").hide();
	
	datatable.on('change', '#select-all', function() {
		
		var checkbox = $(this);
		
		var visibleRows = datatable.rows();
		
		if (checkbox.prop("checked")) {
		
			visibleRows.select();
			
		} else {
		
			visibleRows.deselect();
		}
	});
	
	var $showInstanceBtn = $("#showInstances");
	var $validationErrors = $("#validationerrors");
	
	$showInstanceBtn.on("click", function(e) {
		e.preventDefault();
		
		$validationErrors.html("");
		$("#instances-wrapper").hide();
		
		$showInstanceBtn.prop("disabled", true);
		
		var fromDate = $("#fromDate").val();
		var toDate = $("#toDate").val();
		
		if (fromDate && toDate) {
			if (new Date(fromDate) > new Date(toDate)) {
				$validationErrors.append('<p class="error">' + $showInstanceBtn.data("dateerror") + '</p>');
				
				$showInstanceBtn.prop("disabled", false);
				
				return;
			}
		}
		
		if (!$("#submitted").is(":checked") && !$("#notSubmitted").is(":checked")) {
			$validationErrors.append('<p class="error">' + $showInstanceBtn.data("submittederror") + '</p>');
			
			$showInstanceBtn.prop("disabled", false);
			
			return;
		}
		
		$.ajax({
			url : $showInstanceBtn.data("fetchurl"),
			beforeSend: function(jqXHR, settings) {
				jqXHR.setRequestHeader('NoLoginRedirect', 'true');
			},
			type: "post",
			dataType : "json",
			contentType: "application/x-www-form-urlencoded;charset=UTF-8",
			data: $("#CullingModule input").serialize(),
			success : function(data) {
				
				if (data.error) {
					if (data.error === "submittederror") {
						$validationErrors.append('<p class="error">' + $showInstanceBtn.data("submittederror") + '</p>');
					}
					else if (data.error === "dateinterval") {
						$validationErrors.append('<p class="error">' + $showInstanceBtn.data("dateerror") + '</p>');
					}
					else if (data.error === "dateformat") {
						$validationErrors.append('<p class="error">' + $showInstanceBtn.data("dateformaterror") + '</p>');
					}
					else {
						$validationErrors.append('<p class="error">' + $showInstanceBtn.data("unknownerror") + '</p>');
					}
					
					$showInstanceBtn.prop("disabled", false);
				}
				else {
					$("#instances-wrapper").show();
					
					$("#instance-count").html(data.instances.length);
					
					datatable.clear();
					datatable.rows.add(data.instances);
					datatable.draw();
					
					$showInstanceBtn.prop("disabled", false);
				}
			},
			error : function() {
				$showInstanceBtn.prop("disabled", false);
				
				$validationErrors.append('<p class="error">' + $showInstanceBtn.data("unknownerror") + '</p>');
			}
		});
	});
	
	var $deleteInstancesBtn = $("#deleteInstances");
	
	datatable.on( 'select.dt', function() {
		updateDeleteButton();
	});
	
	datatable.on( 'deselect.dt', function() {
		updateDeleteButton();
	});
	
	var updateDeleteButton = function() {
		
		var selectedRows = datatable.rows({selected: true});
		
		if (selectedRows[0].length === 0) {
			$deleteInstancesBtn.prop("disabled", true).addClass("btn-light").removeClass("btn-red");
		}
		else {
			$deleteInstancesBtn.prop("disabled", false).removeClass("btn-light").addClass("btn-red");;
		}
	}
	
	$deleteInstancesBtn.on("click", function(e) {
		
		if ($deleteInstancesBtn.prop("disabled")) {
			return false;
		}
		
		var selectedRows = datatable.rows({selected: true});
		
		$deleteInstancesBtn.prop("disabled", true);
		
		if (!confirm(i18nCullingModule.CONFIRM_DELETE_PART1 + " " + selectedRows[0].length + " " + i18nCullingModule.CONFIRM_DELETE_PART2 + "?")) {
			e.preventDefault();
			
			$deleteInstancesBtn.prop("disabled", false);
			
			return false;
		}
		
		var cells = datatable.cells(selectedRows.toArray()[0], "instanceID:name");
		var selectedIDs = cells.data().toArray();
		
		var form = $('<form />', {
			action: $deleteInstancesBtn.prop("href"),
			method: 'post',
			style: 'display: none;'
		});
		
		for (var i = 0; i < selectedIDs.length; i++) {
			var field = $('<input/>');

		    field.attr("type", "hidden");
		    field.attr("name", "flowInstanceID");
		    field.attr("value", selectedIDs[i]);
		
		    form.append(field);
		}
		
		var commentfield = $('<input/>');
		 commentfield.attr("type", "hidden");
		 commentfield.attr("name", "comment");
		 commentfield.attr("value", $("#comment").val());
		 form.append(commentfield);
		
		form.appendTo("body").trigger("submit");
	});
});