package com.nordicpeak.flowengine.queries;

import java.sql.Date;
import java.sql.Time;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.enums.DayOfWeek;
import se.unlogic.standardutils.enums.EnumUtils;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.populators.TimePopulator;
import se.unlogic.standardutils.time.TimeStringyfier;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.enums.DateIntervalRestrictionType;
import com.nordicpeak.flowengine.queries.enums.DateIntervalRestrictionUnit;
import com.nordicpeak.flowengine.queries.enums.DateRestrictionType;
import com.nordicpeak.flowengine.queries.enums.TomorrowRestrictionMode;

@Table(name = "date_time_queries")
@XMLElement
public class DateTimeQuery extends BaseQuery {

	private static final long serialVersionUID = -842191226937409416L;
	
	public static final EnumPopulator<DateRestrictionType> DATE_RESTRICTION_TYPE_POPULATOR = new EnumPopulator<DateRestrictionType>(DateRestrictionType.class);
	public static final EnumPopulator<DateIntervalRestrictionType> DATE_INTERVAL_RESTRICTION_TYPE_POPULATOR = new EnumPopulator<DateIntervalRestrictionType>(DateIntervalRestrictionType.class);
	public static final EnumPopulator<DateIntervalRestrictionUnit> DATE_INTERVAL_RESTRICTION_UNIT_POPULATOR = new EnumPopulator<DateIntervalRestrictionUnit>(DateIntervalRestrictionUnit.class);
	public static final EnumPopulator<DayOfWeek> DAY_OF_WEEK_POPULATOR = new EnumPopulator<DayOfWeek>(DayOfWeek.class);
	

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useDateFields;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useTimeFields;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useMultiDate;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean restrictDateInterval;	

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "restrictDateInterval")
	@XMLElement
	private DateIntervalRestrictionType dateIntervalRestrictionType;
	

	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "restrictDateInterval")
	@XMLElement
	private DateIntervalRestrictionUnit dateIntervalRestrictionUnit;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "restrictDateInterval")
	@XMLElement
	private Integer dateIntervalRestriction;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useMultiTime;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean allowStartTimeAfterEndTime;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "useTimeFields")
	@XMLElement
	private Integer timeInterval;

	@DAOManaged
	@WebPopulate
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date minStartDate;

	@DAOManaged
	@WebPopulate
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date maxStartDate;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private DateRestrictionType startDateRestrictionType;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer restrictedStartDays;

	@DAOManaged
	@WebPopulate
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date minEndDate;
	
	@DAOManaged
	@WebPopulate
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Date maxEndDate;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private DateRestrictionType endDateRestrictionType;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer restrictedEndDays;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "date_time_queries_allowed_days_of_week", remoteValueColumnName = "allowedDayOfWeek")
	@NoDuplicates
	@WebPopulate(paramName = "allowedDayOfWeek")
	@XMLElement(childName = "allowedDayOfWeek")
	private List<DayOfWeek> allowedDaysOfWeek;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean restrictTomorrowOnTime;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private TomorrowRestrictionMode tomorrowRestrictionMode;

	@DAOManaged
	@WebPopulate(populator = TimePopulator.class)
	@XMLElement(valueFormatter = TimeStringyfier.class)
	private Time restrictTomorrowTime;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean restrictCurrentMonth;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean restrictUpcomingMonths;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer numberOfUpcomingMonths;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean endDateRestrictCurrentMonth;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean restrictUpcomingEndMonths;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer numberOfUpcomingEndMonths;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean lockOnOwnershipTransfer;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideTitle;
	
	@DAOManaged
	@WebPopulate(populator = TimePopulator.class)
	@XMLElement(valueFormatter = TimeStringyfier.class)
	private Time earliestTime;
	
	@DAOManaged
	@WebPopulate(populator = TimePopulator.class)
	@XMLElement(valueFormatter = TimeStringyfier.class)
	private Time latestTime;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<DateTimeQueryInstance> instances;

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<DateTimeQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<DateTimeQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public boolean isUseDateFields() {

		return useDateFields;
	}

	public void setUseDateFields(boolean useDateFields) {

		this.useDateFields = useDateFields;
	}

	public boolean isUseTimeFields() {

		return useTimeFields;
	}

	public void setUseTimeFields(boolean useTimeFields) {

		this.useTimeFields = useTimeFields;
	}

	public boolean isUseMultiDate() {

		return useMultiDate;
	}

	public void setUseMultiDate(boolean useMultiDate) {

		this.useMultiDate = useMultiDate;
	}

	public boolean isRestrictDateInterval() {
		
		return restrictDateInterval;
	}
	
	public void setRestrictDateInterval(boolean restrictDateInterval) {
	
		this.restrictDateInterval = restrictDateInterval;
	}
	
	public DateIntervalRestrictionType getDateIntervalRestrictionType() {
	
		return dateIntervalRestrictionType;
	}
	
	public void setDateIntervalRestrictionType(DateIntervalRestrictionType dateIntervalRestrictionType) {
	
		this.dateIntervalRestrictionType = dateIntervalRestrictionType;
	}
	
	public DateIntervalRestrictionUnit getDateIntervalRestrictionUnit() {
	
		return dateIntervalRestrictionUnit;
	}
	
	public void setDateIntervalRestrictionUnit(DateIntervalRestrictionUnit dateIntervalRestrictionUnit) {
	
		this.dateIntervalRestrictionUnit = dateIntervalRestrictionUnit;
	}
	
	public Integer getDateIntervalRestriction() {
	
		return dateIntervalRestriction;
	}
	
	public void setDateIntervalRestriction(Integer dateIntervalRestriction) {
	
		this.dateIntervalRestriction = dateIntervalRestriction;
	}
	
	public boolean isUseMultiTime() {

		return useMultiTime;
	}

	public void setUseMultiTime(boolean useMultiTime) {

		this.useMultiTime = useMultiTime;
	}

	public Integer getTimeInterval() {

		return timeInterval;
	}

	public void setTimeInterval(Integer timeInterval) {

		this.timeInterval = timeInterval;
	}

	public Date getMinStartDate() {
	
		return minStartDate;
	}
	
	public void setMinStartDate(Date minStartDate) {
	
		this.minStartDate = minStartDate;
	}

	public Date getMaxStartDate() {
		
		return maxStartDate;
	}
	
	public void setMaxStartDate(Date maxStartDate) {
	
		this.maxStartDate = maxStartDate;
	}
	
	public Date getMaxEndDate() {
	
		return maxEndDate;
	}
	
	public void setMaxEndDate(Date maxEndDate) {
	
		this.maxEndDate = maxEndDate;
	}
	
	public DateRestrictionType getStartDateRestrictionType() {
	
		return startDateRestrictionType;
	}

	public void setStartDateRestrictionType(DateRestrictionType startDateRestrictionType) {
	
		this.startDateRestrictionType = startDateRestrictionType;
	}
	
	public Integer getRestrictedStartDays() {
	
		return restrictedStartDays;
	}

	public void setRestrictedStartDays(Integer restrictedStartDays) {
	
		this.restrictedStartDays = restrictedStartDays;
	}
	
	public Date getMinEndDate() {
	
		return minEndDate;
	}
	
	public void setMinEndDate(Date minEndDate) {
	
		this.minEndDate = minEndDate;
	}
	
	public DateRestrictionType getEndDateRestrictionType() {
	
		return endDateRestrictionType;
	}

	public void setEndDateRestrictionType(DateRestrictionType endDateRestrictionType) {
	
		this.endDateRestrictionType = endDateRestrictionType;
	}
	
	public Integer getRestrictedEndDays() {
	
		return restrictedEndDays;
	}
	
	public void setRestrictedEndDays(Integer restrictedEndDays) {
	
		this.restrictedEndDays = restrictedEndDays;
	}
	
	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}
	
	public Time getEarliestTime() {
		return earliestTime;
	}
	
	public void setEarliestTime(Time earliestTime) {
		this.earliestTime = earliestTime;
	}
	
	public Time getLatestTime() {
		return latestTime;
	}
	
	public void setLatestTime(Time latestTime) {
		this.latestTime = latestTime;
	}

	public List<DayOfWeek> getAllowedDaysOfWeek() {
		
		return allowedDaysOfWeek;
	}
	
	public void setAllowedDaysOfWeek(List<DayOfWeek> allowedDaysOfWeek) {
	
		this.allowedDaysOfWeek = allowedDaysOfWeek;
	}

	public boolean isRestrictTomorrowOnTime() {
		
		return restrictTomorrowOnTime;
	}
	
	public void setRestrictTomorrowOnTime(boolean restrictTomorrowOnTime) {
	
		this.restrictTomorrowOnTime = restrictTomorrowOnTime;
	}

	public TomorrowRestrictionMode getTomorrowRestrictionMode() {
		
		return tomorrowRestrictionMode;
	}
	
	public void setTomorrowRestrictionMode(TomorrowRestrictionMode tomorrowRestrictionMode) {
	
		this.tomorrowRestrictionMode = tomorrowRestrictionMode;
	}
	
	public Time getRestrictTomorrowTime() {
	
		return restrictTomorrowTime;
	}
	
	public void setRestrictTomorrowTime(Time restrictTomorrowTime) {
	
		this.restrictTomorrowTime = restrictTomorrowTime;
	}

	public boolean isRestrictCurrentMonth() {
		
		return restrictCurrentMonth;
	}
	
	public void setRestrictCurrentMonth(boolean restrictCurrentMonth) {
	
		this.restrictCurrentMonth = restrictCurrentMonth;
	}
	
	public boolean isRestrictUpcomingMonths() {
	
		return restrictUpcomingMonths;
	}
	
	public void setRestrictUpcomingMonths(boolean restrictUpcomingMonths) {
	
		this.restrictUpcomingMonths = restrictUpcomingMonths;
	}
	
	public Integer getNumberOfUpcomingMonths() {
	
		return numberOfUpcomingMonths;
	}
	
	public void setNumberOfUpcomingMonths(Integer numberOfUpcomingMonths) {
	
		this.numberOfUpcomingMonths = numberOfUpcomingMonths;
	}

	public boolean isEndDateRestrictCurrentMonth() {
		
		return endDateRestrictCurrentMonth;
	}
	
	public void setEndDateRestrictCurrentMonth(boolean endDateRestrictCurrentMonth) {
	
		this.endDateRestrictCurrentMonth = endDateRestrictCurrentMonth;
	}
	
	public boolean isRestrictUpcomingEndMonths() {
	
		return restrictUpcomingEndMonths;
	}
	
	public void setRestrictUpcomingEndMonths(boolean restrictUpcomingEndMonths) {
	
		this.restrictUpcomingEndMonths = restrictUpcomingEndMonths;
	}
	
	public Integer getNumberOfUpcomingEndMonths() {
	
		return numberOfUpcomingEndMonths;
	}
	
	public void setNumberOfUpcomingEndMonths(Integer numberOfUpcomingEndMonths) {
	
		this.numberOfUpcomingEndMonths = numberOfUpcomingEndMonths;
	}
	
	@Override
	public String toString() {
		
		if (this.queryDescriptor != null) {
			
			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}
		
		return "DateTimeQuery (queryID: " + queryID + ")";
	}
	
	public boolean isAllowStartTimeAfterEndTime() {
		
		return allowStartTimeAfterEndTime;
	}

	public void setAllowStartTimeAfterEndTime(boolean allowStartTimeAfterEndTime) {
	
		this.allowStartTimeAfterEndTime = allowStartTimeAfterEndTime;
	}

	@Override
	public String getXSDTypeName() {
		
		return "DateTimeQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		Element valueElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		valueElement.setAttribute("name", "Value");
		valueElement.setAttribute("minOccurs", "1");
		valueElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(valueElement);

		Element simpleTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:simpleType");
		valueElement.appendChild(simpleTypeElement);

		Element restrictionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:restriction");
		restrictionElement.setAttribute("base", "xs:string");
		simpleTypeElement.appendChild(restrictionElement);

		Element startDateElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		startDateElement.setAttribute("name", "StartDate");
		startDateElement.setAttribute("type", "xs:date");
		startDateElement.setAttribute("minOccurs", useDateFields ? "1" : "0");
		startDateElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(startDateElement);

		Element endDateElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		endDateElement.setAttribute("name", "EndDate");
		endDateElement.setAttribute("type", "xs:date");
		endDateElement.setAttribute("minOccurs", useDateFields && useMultiDate ? "1" : "0");
		endDateElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(endDateElement);

		Element startTimeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		startTimeElement.setAttribute("name", "StartTime");
		startTimeElement.setAttribute("type", "xs:time");
		startTimeElement.setAttribute("minOccurs", useTimeFields ? "1" : "0");
		startTimeElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(startTimeElement);

		Element endTimeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		endTimeElement.setAttribute("name", "EndTime");
		endTimeElement.setAttribute("type", "xs:time");
		endTimeElement.setAttribute("minOccurs", useMultiTime ? "1" : "0");
		endTimeElement.setAttribute("maxOccurs", "1");
		sequenceElement.appendChild(endTimeElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		useDateFields = xmlParser.getPrimitiveBoolean("useDateFields");
		useTimeFields = xmlParser.getPrimitiveBoolean("useTimeFields");
		useMultiDate = xmlParser.getPrimitiveBoolean("useMultiDate");
		restrictDateInterval = xmlParser.getPrimitiveBoolean("restrictDateInterval");
		dateIntervalRestrictionType = XMLValidationUtils.validateParameter("dateIntervalRestrictionType", xmlParser, false, DATE_INTERVAL_RESTRICTION_TYPE_POPULATOR, errors);
		dateIntervalRestrictionUnit = XMLValidationUtils.validateParameter("dateIntervalRestrictionUnit", xmlParser, false, DATE_INTERVAL_RESTRICTION_UNIT_POPULATOR, errors);
		dateIntervalRestriction = xmlParser.getInteger("dateIntervalRestriction");
		useMultiTime = xmlParser.getPrimitiveBoolean("useMultiTime");
		timeInterval = XMLValidationUtils.validateParameter("timeInterval", xmlParser, useTimeFields, IntegerPopulator.getPopulator(), errors);
		minStartDate = XMLValidationUtils.validateParameter("minStartDate", xmlParser, false, DatePopulator.getPopulator(), errors);
		maxStartDate = XMLValidationUtils.validateParameter("maxStartDate", xmlParser, false, DatePopulator.getPopulator(), errors);
		
		if(minStartDate == null) {
			
			//Older version of query
			minStartDate = XMLValidationUtils.validateParameter("minDate", xmlParser, false, DatePopulator.getPopulator(), errors);
		}
		
		startDateRestrictionType = XMLValidationUtils.validateParameter("startDateRestrictionType", xmlParser, false, DATE_RESTRICTION_TYPE_POPULATOR, errors);
		
		if(startDateRestrictionType == null) {
			
			//Older version of query
			startDateRestrictionType = XMLValidationUtils.validateParameter("dateRestrictionType", xmlParser, false, DATE_RESTRICTION_TYPE_POPULATOR, errors);
		}
		
		restrictedStartDays = XMLValidationUtils.validateParameter("restrictedStartDays", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		
		if(restrictedStartDays == null) {
			
			//Older version of query
			restrictedStartDays = XMLValidationUtils.validateParameter("restrictedDays", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		}
		
		minEndDate = XMLValidationUtils.validateParameter("minEndDate", xmlParser, false, DatePopulator.getPopulator(), errors);
		maxEndDate = XMLValidationUtils.validateParameter("maxEndDate", xmlParser, false, DatePopulator.getPopulator(), errors);
		endDateRestrictionType = XMLValidationUtils.validateParameter("endDateRestrictionType", xmlParser, false, DATE_RESTRICTION_TYPE_POPULATOR, errors);
		restrictedEndDays = XMLValidationUtils.validateParameter("restrictedEndDays", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		allowStartTimeAfterEndTime = xmlParser.getPrimitiveBoolean("allowStartTimeAfterEndTime");
		earliestTime = XMLValidationUtils.validateParameter("earliestTime", xmlParser, false, TimePopulator.getPopulator(), errors);
		latestTime = XMLValidationUtils.validateParameter("latestTime", xmlParser, false, TimePopulator.getPopulator(), errors);
		lockOnOwnershipTransfer = xmlParser.getPrimitiveBoolean("lockOnOwnershipTransfer");

		XMLParser allowedDaysOfWeekParser = xmlParser.getNode("allowedDaysOfWeek");

		if (allowedDaysOfWeekParser != null) {
			
			allowedDaysOfWeek = EnumUtils.toEnum(DayOfWeek.class, allowedDaysOfWeekParser.getStrings("allowedDayOfWeek"));
		}
		
		if(allowedDaysOfWeek == null) {
			
			allowedDaysOfWeek = new ArrayList<DayOfWeek>(7);
			allowedDaysOfWeek.add(DayOfWeek.MONDAY);
			allowedDaysOfWeek.add(DayOfWeek.TUESDAY);
			allowedDaysOfWeek.add(DayOfWeek.WEDNESDAY);
			allowedDaysOfWeek.add(DayOfWeek.THURSDAY);
			allowedDaysOfWeek.add(DayOfWeek.FRIDAY);
			allowedDaysOfWeek.add(DayOfWeek.SATURDAY);
			allowedDaysOfWeek.add(DayOfWeek.SUNDAY);
		}
		
		restrictTomorrowOnTime = xmlParser.getPrimitiveBoolean("restrictTomorrowOnTime");
		restrictTomorrowTime = XMLValidationUtils.validateParameter("restrictTomorrowTime", xmlParser, false, TimePopulator.getPopulator(), errors);
		tomorrowRestrictionMode = EnumUtils.toEnum(TomorrowRestrictionMode.class, xmlParser.getString("tomorrowRestrictionMode"));
		
		restrictCurrentMonth = xmlParser.getPrimitiveBoolean("restrictCurrentMonth");
		restrictUpcomingMonths = xmlParser.getPrimitiveBoolean("restrictUpcomingMonths");
		numberOfUpcomingMonths = XMLValidationUtils.validateParameter("numberOfUpcomingMonths", xmlParser, false, IntegerPopulator.getPopulator(), errors);

		endDateRestrictCurrentMonth = xmlParser.getPrimitiveBoolean("endDateRestrictCurrentMonth");
		restrictUpcomingEndMonths = xmlParser.getPrimitiveBoolean("restrictUpcomingEndMonths");
		numberOfUpcomingEndMonths = XMLValidationUtils.validateParameter("numberOfUpcomingEndMonths", xmlParser, false, IntegerPopulator.getPopulator(), errors);
		
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		if (attributeName != null) {
			
			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}
		
		hideTitle = xmlParser.getPrimitiveBoolean("hideTitle");
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}
	
	public boolean isSetAsAttribute() {
		
		return setAsAttribute;
	}
	
	public void setSetAsAttribute(boolean setAsAttribute) {
		
		this.setAsAttribute = setAsAttribute;
	}
	
	public String getAttributeName() {
		
		return attributeName;
	}
	
	public void setAttributeName(String attributeName) {
		
		this.attributeName = attributeName;
	}
	
	public boolean isLockOnOwnershipTransfer() {
		
		return lockOnOwnershipTransfer;
	}
	
	public void setLockOnOwnershipTransfer(boolean lockOnOwnershipTransfer) {
		
		this.lockOnOwnershipTransfer = lockOnOwnershipTransfer;
	}


}
