package com.nordicpeak.flowengine.documentsigner;

import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringWriter;
import java.io.Writer;
import java.lang.reflect.Field;
import java.net.URISyntaxException;
import java.net.URL;
import java.sql.Date;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.apache.log4j.Level;
import org.apache.pdfbox.io.MemoryUsageSetting;
import org.apache.pdfbox.multipdf.PDFMergerUtility;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
import org.apache.pdfbox.pdmodel.PDDocumentInformation;
import org.apache.pdfbox.pdmodel.common.PDMetadata;
import org.apache.pdfbox.pdmodel.graphics.color.PDOutputIntent;
import org.apache.xmpbox.XMPMetadata;
import org.apache.xmpbox.schema.AdobePDFSchema;
import org.apache.xmpbox.schema.DublinCoreSchema;
import org.apache.xmpbox.schema.PDFAIdentificationSchema;
import org.apache.xmpbox.schema.XMPBasicSchema;
import org.apache.xmpbox.xml.XmpSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xhtmlrenderer.pdf.ITextRenderer;
import org.xml.sax.SAXException;

import se.unlogic.cron4jutils.CronStringValidator;
import se.unlogic.emailutils.framework.EmailUtils;
import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleSMS;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ResourceNotFoundException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.handlers.UserHandler;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.interfaces.sms.SMSSender;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.GenericCRUD;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.crud.MultipartLimitProvider;
import se.unlogic.hierarchy.core.utils.crud.MultipartRequestFilter;
import se.unlogic.hierarchy.core.utils.crud.TransactionRequestFilter;
import se.unlogic.hierarchy.core.utils.usergrouplist.UserGroupListConnector;
import se.unlogic.hierarchy.foregroundmodules.rest.AnnotatedRESTModule;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.hash.HashAlgorithms;
import se.unlogic.standardutils.hash.HashUtils;
import se.unlogic.standardutils.html.HTMLUtils;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.io.FileUtils;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.SwedishSocialSecurity12DigitsWithoutMinusPopulator;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.SwedishPhoneNumberValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.ClassPathURIResolver;
import se.unlogic.standardutils.xml.XMLTransformer;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xsl.URIXSLTransformer;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.http.enums.ContentDisposition;

import com.lowagie.text.Font;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.ColumnText;
import com.lowagie.text.pdf.PdfArray;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfFileSpecification;
import com.lowagie.text.pdf.PdfName;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;
import com.lowagie.text.pdf.RandomAccessFileOrArray;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.documentsigner.beans.DocumentSigner;
import com.nordicpeak.flowengine.documentsigner.beans.SigningDocument;
import com.nordicpeak.flowengine.documentsigner.cruds.SigningDocumentCRUD;
import com.nordicpeak.flowengine.documentsigner.enums.SignerType;
import com.nordicpeak.flowengine.documentsigner.enums.SigningDocumentStatus;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryInstanceNotFoundInQueryProviderException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderErrorException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderNotFoundException;
import com.nordicpeak.flowengine.interfaces.FlowNotificationHandler;
import com.nordicpeak.flowengine.utils.PDFXMLUtils;

import it.sauronsoftware.cron4j.Scheduler;

public class DocumentSignerAdminModule extends AnnotatedRESTModule implements Runnable, MultipartLimitProvider, CRUDCallback<User> {

	public static final AnnotatedBeanTagSourceFactory<SigningDocument> SIGNING_DOCUMENT_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(SigningDocument.class, "$document.");
	public static final AnnotatedBeanTagSourceFactory<Contact> CONTACT_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(Contact.class, "$contact.");
	public static final AnnotatedBeanTagSourceFactory<DocumentSigner> SIGNER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(DocumentSigner.class, "$signer.");
	public static final AnnotatedBeanTagSourceFactory<User> POSTER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<>(User.class, "$poster.");

	@XSLVariable(prefix = "java.")
	private String pdfSignedText = "(Signerat, $hashAlgorithm $hash)";

	@XSLVariable(prefix = "java.")
	private String pdfPageNumber1 = "Page ";

	@XSLVariable(prefix = "java.")
	private String pdfPageNumber2 = " of ";

	@XSLVariable(prefix = "java.")
	private String pdfUnsignedAttachment = "Signeringsunderlag";

	@XSLVariable(prefix = "java.")
	private String pdfSignatureAttachment = "Signatur";
	
	@XSLVariable(prefix = "java.")
	private String signedPDFFileSuffix = "";
	
	@XSLVariable(prefix = "java.")
	private String signedReceiptFileSuffix = "";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Hash algorithm", description = "The hash algorithm to use for signing", required = true)
	protected String hashAlgorithm = HashAlgorithms.SHA256;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Title attribute name", description = "The name of the user attribute that contains their title", required = false)
	protected String titleAttribute = "title";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "PDF XSL stylesheet", description = "The path in classpath relative from this class to the XSL stylesheet used to transform the XHTML for PDF output", required = true)
	protected String pdfStyleSheet = "DocumentSignerPDF.sv.xsl";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Schedule checks and notifications", description = "When this module should check for expired documents (specified in crontab format)", required = true, formatValidator = CronStringValidator.class)
	private String cronSchedule = "0 15 * * *";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Filestore", description = "Directory where documents are stored", required = true)
	protected String filestore;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Temp dir", description = "Directory for temporary files. Should be on the same filesystem as the file store for best performance.")
	protected String tempDir;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max document size", description = "Maxmium allowed document file size in megabytes", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer maxDocumentSize = 100;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Max request size", description = "Maxmium allowed request size in megabytes", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer maxRequestSize = 1000;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "RAM threshold", description = "Maximum size of files in KB to be buffered in RAM during file uploads. Files exceeding the threshold are written to disk instead.", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer ramThreshold = 500;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default CancelInXDays", description = "", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer defaultCancelInXDays = 14;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default SendReminderInXDays", description = "", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer defaultSendReminderInXDays = 7;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default DeleteAfterXDays", description = "", required = true, formatValidator = PositiveStringIntegerValidator.class)
	protected Integer defaultDeleteAfterXDays = 90;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Send email to poster after each signing", description = "When multisigning an information mail is sent to poster for each individual signing. This setting can be overwritten when creating signing document.")
	protected boolean sendEmailWhenSigningPartiallyDone;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Included fonts", description = "Path to the fonts that should be included in the PDF (the paths can be either in filesystem or classpath)")
	private List<String> includedFonts;

	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Search groups", description = "Groups with users that are allowed to be set as managers")
	protected List<Integer> searchFilterGroupIDs;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "Signing SMS message", description = "The SMS sent to the users when they have to sign a document", required = true)
	@XSLVariable(prefix = "java.")
	private String signingInitiatedUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Signing email subject", description = "The subject of emails sent to the users when they have to sign a document", required = true)
	@XSLVariable(prefix = "java.")
	private String signingInitiatedUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signing email message", description = "The message of emails sent to the users when they have to sign a document", required = true)
	@XSLVariable(prefix = "java.")
	private String signingInitiatedUserEmailMessage;

	@ModuleSetting
	@TextAreaSettingDescriptor(name = "Signing canceled SMS message (users)", description = "The SMS sent to the users when a document they had to sign is canceled", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCanceledUserSMS;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Signing canceled email subject (users)", description = "The subject of emails sent to the users when a document they had to sign is canceled", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCanceledUserEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signing canceled email message (users)", description = "The message of emails sent to the users when a document they had to sign is canceled", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCanceledUserEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Signing canceled email subject (owner)", description = "The subject of emails sent to the owner when a document they wanted to be signed is canceled", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCanceledOwnerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signing canceled email message (owner)", description = "The message of emails sent to the owner when a document they wanted to be signed is canceled", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCanceledOwnerEmailMessage;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Signing partially completed email subject (owner)", description = "The message of emails sent to the owner when a document they wanted to be signed is partially completed", required = true)
	@XSLVariable(prefix = "java.")
	private String signingPartiallyCompletedOwnerEmailSubject;
	
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signing partially completed email message (owner)", description = "The message of emails sent to the owner when a document they wanted to be signed is partially completed", required = true)
	@XSLVariable(prefix = "java.")
	private String signingPartiallyCompletedOwnerEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Signing completed email subject (owner)", description = "The subject of emails sent to the owner when a document they wanted to be signed is completed", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCompletedOwnerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Signing completed email message (owner)", description = "The message of emails sent to the owner when a document they wanted to be signed is completed", required = true)
	@XSLVariable(prefix = "java.")
	private String signingCompletedOwnerEmailMessage;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Document expiring email subject (owner)", description = "The subject of emails sent to the owner when a document they created is about to expire", required = true)
	@XSLVariable(prefix = "java.")
	private String documentExpiringOwnerEmailSubject;

	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Document expiring email message (owner)", description = "The message of emails sent to the owner when a document they created is about to expire", required = true)
	@XSLVariable(prefix = "java.")
	private String documentExpiringOwnerEmailMessage;

	@InstanceManagerDependency
	protected SiteProfileHandler profileHandler;

	@InstanceManagerDependency
	protected FlowNotificationHandler flowNotificationHandler;

	@InstanceManagerDependency
	protected SMSSender smsSender;

	private AnnotatedDAO<SigningDocument> documentDAO;

	private AdvancedAnnotatedDAOWrapper<SigningDocument, Integer> documentDAOWrapper;

	private QueryParameterFactory<SigningDocument, User> documentPosterParamFactory;

	@InstanceManagerDependency
	protected DocumentSignerUserModule userModule;

	private SigningDocumentCRUD documentCRUD;

	protected UserGroupListConnector userGroupListConnector;

	protected URIXSLTransformer pdfTransformer;

	private Scheduler scheduler;
	private String removeExpiredDocumentsScheduleID;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		userGroupListConnector = new UserGroupListConnectorWithTitle(systemInterface, getTitleAttribute());
		userGroupListConnector.setUserGroupFilter(searchFilterGroupIDs);
		userGroupListConnector.setOnlyEnabledUsers(true);

		if (!systemInterface.getInstanceHandler().addInstance(DocumentSignerAdminModule.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + DocumentSignerAdminModule.class.getSimpleName() + ", another instance is already registered using this key.");
		}

		initScheduler();
	}

	@Override
	public void update(ForegroundModuleDescriptor descriptor, DataSource dataSource) throws Exception {

		super.update(descriptor, dataSource);

		scheduler.reschedule(removeExpiredDocumentsScheduleID, cronSchedule);
	}

	@Override
	protected void moduleConfigured() throws TransformerConfigurationException, XPathExpressionException, SAXException, IOException, ParserConfigurationException, ClassNotFoundException, URISyntaxException, ResourceNotFoundException, IllegalArgumentException, IllegalAccessException, InstantiationException, SQLException {

		if (ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR)) {

			File filestoreDir = new File(filestore);

			if (!filestoreDir.exists()) {

				if (filestoreDir.getParentFile().exists()) {

					log.info("Creating missing  " + filestoreDir.getName() + " directory for PDF filestore directory");

					if (!filestoreDir.mkdir()) {

						log.error("Unable to create missing " + filestoreDir.getName() + " directory for PDF filestore directory");
					}

				} else {

					log.error("The set PDF filestore directory does not exist");
				}
			}

			if (!FileUtils.directoryExists(tempDir)) {

				log.error("The set temp directory does not exist");
			}
		}

		if (userGroupListConnector != null) {

			userGroupListConnector.setUserGroupFilter(searchFilterGroupIDs);
		}

		if (pdfStyleSheet == null) {

			pdfTransformer = null;

		} else {

			URL styleSheetURL = DocumentSignerAdminModule.class.getResource(pdfStyleSheet);

			if (styleSheetURL != null) {

				try {
					pdfTransformer = new URIXSLTransformer(styleSheetURL.toURI(), ClassPathURIResolver.getInstance(), true);

					log.info("Succesfully parsed PDF stylesheet " + pdfStyleSheet);

				} catch (Exception e) {

					log.error("Unable to cache PDF style sheet " + pdfStyleSheet, e);

					pdfTransformer = null;
				}

			} else {
				log.error("Unable to cache PDF style sheet. Resource " + pdfStyleSheet + " not found");
			}
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, DocumentSignerAdminModule.class.getName(), new XMLDBScriptProvider(DocumentSignerAdminModule.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), true, true, false);

		documentDAO = daoFactory.getDAO(SigningDocument.class);

		documentPosterParamFactory = documentDAO.getParamFactory("poster", User.class);

		documentDAOWrapper = documentDAO.getAdvancedWrapper(Integer.class);
		documentDAOWrapper.getGetQuery().addRelations(SigningDocument.SIGNERS_RELATION);
		documentDAOWrapper.getAddQuery().addRelations(SigningDocument.SIGNERS_RELATION);
		documentDAOWrapper.getUpdateQuery().addRelations(SigningDocument.SIGNERS_RELATION);

		documentCRUD = new SigningDocumentCRUD(documentDAOWrapper, this);
		documentCRUD.addRequestFilter(new MultipartRequestFilter(this));
		documentCRUD.addRequestFilter(new TransactionRequestFilter(dataSource));
	}

	@Override
	public void unload() throws Exception {

		stopScheduler();

		systemInterface.getInstanceHandler().removeInstance(DocumentSignerAdminModule.class, this);

		super.unload();
	}

	protected synchronized void initScheduler() {

		if (scheduler != null) {

			log.warn("Invalid state, scheduler already running!");
			stopScheduler();
		}

		scheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());
		scheduler.setDaemon(true);

		removeExpiredDocumentsScheduleID = scheduler.schedule(cronSchedule, this);

		scheduler.start();
	}

	protected synchronized void stopScheduler() {

		try {
			if (scheduler != null) {

				scheduler.stop();
				scheduler = null;
			}

		} catch (IllegalStateException e) {
			log.error("Error stopping scheduler", e);
		}
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return listDocuments(req, res, user, uriParser, null);
	}

	public ForegroundModuleResponse listDocuments(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws Exception {

		log.info("User " + user + " listing documents");

		Document doc = createDocument(req, uriParser, user);

		Element listDocumentsElement = XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "ListDocuments");

		if (userModule != null) {
			XMLUtils.append(doc, listDocumentsElement, "PendingDocuments", userModule.getPendingDocuments(user));
		}

		List<SigningDocument> documents = getDocuments(user, SigningDocument.SIGNERS_RELATION);

		if (documents != null) {

			Collections.sort(documents);
			XMLUtils.append(doc, listDocumentsElement, "Documents", documents);
		}

		if (validationErrors != null) {

			listDocumentsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			XMLUtils.append(doc, listDocumentsElement, "ValidationErrors", validationErrors);
		}

		return new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), getDefaultBreadcrumb());
	}

	protected List<SigningDocument> getDocuments(User user, Field... relations) throws SQLException {

		HighLevelQuery<SigningDocument> query = new HighLevelQuery<>();
		query.addParameter(documentPosterParamFactory.getParameter(user));

		if (relations != null) {
			query.addRelations(relations);
		}

		return documentDAO.getAll(query);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse showDocument(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		return documentCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse addDocument(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		return documentCRUD.add(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateDocument(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		return documentCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteDocument(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		return documentCRUD.delete(req, res, user, uriParser);
	}

	@RESTMethod(alias = "originalpdf/{documentID}", method = { "get" }, requireLogin = true)
	public ForegroundModuleResponse getOriginalPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "documentID") Integer documentID) throws URINotFoundException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException, SQLException, AccessDeniedException {

		SigningDocument signingDocument = documentDAOWrapper.get(documentID);

		if (signingDocument == null) {
			throw new URINotFoundException(uriParser);
		}

		checkAccess(signingDocument, user);

		File pdfFile = getOriginalPDF(signingDocument);

		if (pdfFile != null) {

			log.info("Sending original PDF for document " + signingDocument + " to user " + user);

			try {
				HTTPUtils.sendFile(pdfFile, signingDocument.getName() + ".pdf", req, res, ContentDisposition.ATTACHMENT);
				return null;

			} catch (IOException e) {
				log.info("Error sending original PDF for document " + signingDocument + " to user " + user + ", " + e);
			}
		}

		log.warn("Unable to find original PDF for document " + signingDocument + " requested by user " + user);
		throw new URINotFoundException(uriParser);
	}

	@RESTMethod(alias = "signedpdf/{documentID}", method = { "get" }, requireLogin = true)
	public ForegroundModuleResponse getSignedPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "documentID") Integer documentID) throws URINotFoundException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException, SQLException, AccessDeniedException {

		SigningDocument signingDocument = documentDAOWrapper.get(documentID);

		if (signingDocument == null) {
			throw new URINotFoundException(uriParser);
		}

		checkAccess(signingDocument, user);

		File pdfFile = getSignedPDF(signingDocument);

		if (pdfFile != null) {

			log.info("Sending signed PDF for document " + signingDocument + " to user " + user);

			try {
				HTTPUtils.sendFile(pdfFile, signingDocument.getName() + signedPDFFileSuffix + ".pdf", req, res, ContentDisposition.ATTACHMENT);
				return null;

			} catch (IOException e) {
				log.info("Error sending signed PDF for document " + signingDocument + " to user " + user + ", " + e);
			}
		}

		log.warn("Unable to find signed PDF for document " + signingDocument + " requested by user " + user);
		throw new URINotFoundException(uriParser);
	}

	@RESTMethod(alias = "signreceiptpdf/{documentID}", method = { "get" }, requireLogin = true)
	public ForegroundModuleResponse getSignReceiptPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "documentID") Integer documentID) throws URINotFoundException, DuplicateFlowInstanceManagerIDException, MissingQueryInstanceDescriptor, QueryProviderNotFoundException, InvalidFlowInstanceStepException, QueryProviderErrorException, QueryInstanceNotFoundInQueryProviderException, SQLException, AccessDeniedException {

		SigningDocument signingDocument = documentDAOWrapper.get(documentID);

		if (signingDocument == null) {
			throw new URINotFoundException(uriParser);
		}

		checkAccess(signingDocument, user);

		File pdfFile = getSignReceiptPDF(signingDocument);

		if (pdfFile != null) {

			log.info("Sending sign receipt PDF for document " + signingDocument + " to user " + user);

			try {
				HTTPUtils.sendFile(pdfFile, signingDocument.getName() + signedReceiptFileSuffix + ".pdf", req, res, ContentDisposition.ATTACHMENT);
				return null;

			} catch (IOException e) {
				log.info("Error sending sign receipt PDF for document " + signingDocument + " to user " + user + ", " + e);
			}
		}

		log.warn("Unable to find sign receipt PDF for " + signingDocument + " requested by user " + user);
		throw new URINotFoundException(uriParser);
	}

	public File getOriginalPDF(SigningDocument signingDocument) {

		return new File(getDocumentDir() + File.separator + "document-" + signingDocument.getDocumentID() + ".pdf");
	}

	public File getSignedPDF(SigningDocument signingDocument) {

		return new File(getDocumentDir() + File.separator + "document-" + signingDocument.getDocumentID() + "-signed.pdf");
	}

	public File getSignReceiptPDF(SigningDocument signingDocument) {

		return new File(getDocumentDir() + File.separator + "document-" + signingDocument.getDocumentID() + "-sign-receipt.pdf");
	}

	protected void generateSignedPDF(SigningDocument signingDocument) throws Exception {

		File inputFile = getOriginalPDF(signingDocument);

		File tmpFile = File.createTempFile("document-" + signingDocument.getDocumentID() + "-tmp-", ".pdf", new File(getTempDir()));

		try {
			generatePDF(signingDocument, inputFile, tmpFile, true, true);

			File outputFile = getSignedPDF(signingDocument);

			writePDFA(tmpFile, outputFile);

		} catch (Exception e) {

			throw new Exception("Error generating PDF for document " + signingDocument, e);

		} finally {

			FileUtils.deleteFile(tmpFile);
		}
	}

	protected void generateSignReceiptPDF(SigningDocument signingDocument) throws Exception {

		File inputFile = getOriginalPDF(signingDocument);

		File tmpFile = File.createTempFile("document-" + signingDocument.getDocumentID() + "-tmp-", ".pdf", new File(getTempDir()));

		try {
			generatePDF(signingDocument, inputFile, tmpFile, false, false);

			File outputFile = getSignReceiptPDF(signingDocument);

			writePDFA(tmpFile, outputFile);

		} catch (Exception e) {

			throw new Exception("Error generating PDF for document " + signingDocument, e);

		} finally {

			FileUtils.deleteFile(tmpFile);
		}
	}

	public void generatePDF(SigningDocument signingDocument, File inputFile, File outputFile, boolean showCitizenIdentifier, boolean addSignatures) throws Exception {

		if (!inputFile.exists() || !inputFile.canRead()) {
			throw new RuntimeException("Unable to find/read input PDF for document " + signingDocument);
		}

		File tmpFile = null;
		RandomAccessFileOrArray randomAccessFile = null;
		OutputStream outputStream = null;

		try {
			// Generate signature page
			log.info("Generate signature page for document " + signingDocument);

			Document doc = XMLUtils.createDomDocument();
			Element documentElement = doc.createElement("Document");
			doc.appendChild(documentElement);

			Element signaturesElement = XMLUtils.appendNewElement(doc, documentElement, "Signatures");
			signaturesElement.appendChild(signingDocument.toXML(doc));

			XMLUtils.appendNewElement(doc, signaturesElement, "ShowCitizenIdentifier", showCitizenIdentifier);
			XMLUtils.appendNewElement(doc, signaturesElement, "AddSignatures", addSignatures);

			StringWriter writer = new StringWriter();

			XMLTransformer.transformToWriter(pdfTransformer.getTransformer(), doc, writer, "UTF-8", "1.1");

			String xml = writer.toString();

			Document document;

			if (systemInterface.getEncoding().equalsIgnoreCase("UTF-8")) {

				document = PDFXMLUtils.parseXML(xml);

			} else {

				document = PDFXMLUtils.parseXML(new ByteArrayInputStream(xml.getBytes("UTF-8")));
			}

			try {
				outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));

				ITextRenderer renderer = new ITextRenderer();
				ResourceLoaderAgent callback = new ResourceLoaderAgent(renderer.getOutputDevice());
				callback.setSharedContext(renderer.getSharedContext());
				renderer.getSharedContext().setUserAgentCallback(callback);

				if (includedFonts != null) {

					for (String font : includedFonts) {

						renderer.getFontResolver().addFont(font, true);
					}
				}

				renderer.setDocument(document, "documentsigner");
				renderer.layout();

				renderer.createPDF(outputStream);

			} finally {

				CloseUtils.close(outputStream);
			}

			// Merge input and signing page
			log.info("Merging input PDF and signing page for document " + signingDocument);
			PDFMergerUtility merger = new PDFMergerUtility();
			merger.addSource(inputFile);
			merger.addSource(outputFile);

			tmpFile = File.createTempFile("document-" + signingDocument.getDocumentID() + "-merge-tmp-", ".pdf", new File(getTempDir()));
			merger.setDestinationFileName(tmpFile.getAbsolutePath());
			merger.mergeDocuments(MemoryUsageSetting.setupMainMemoryOnly());

			// Append signing date to all pages

			log.info("Append signing date to all pages for document " + signingDocument);
			randomAccessFile = new RandomAccessFileOrArray(tmpFile.getAbsolutePath(), false, false);
			PdfReader reader = new PdfReader(randomAccessFile, null);
			outputStream = new BufferedOutputStream(new FileOutputStream(outputFile));

			PdfStamper stamper = new PdfStamper(reader, outputStream);

			Font font;

			if (CollectionUtils.isEmpty(includedFonts)) {

				font = new Font(BaseFont.createFont(), 8f); // Defaults to non-embedded HELVETICA, IText can not embed base 14 fonts (Type 1 fonts).
				log.warn("Using non-embedded font " + font.getFamilyname() + " for inline PDF attachment page numbering.");

			} else {

				font = new Font(BaseFont.createFont(includedFonts.get(0), BaseFont.CP1252, BaseFont.EMBEDDED), 8f);
			}

			BaseFont baseFont = font.getCalculatedBaseFont(true);
			float fontHeight = baseFont.getFontDescriptor(BaseFont.ASCENT, font.getSize()) - baseFont.getFontDescriptor(BaseFont.DESCENT, font.getSize());

			String bottomSignedText = pdfSignedText.replace("$hashAlgorithm", signingDocument.getHashAlgorithm()).replace("$hash", HashUtils.hash(inputFile, signingDocument.getHashAlgorithm()));

			int pageCount = reader.getNumberOfPages();

			for (int pageNumber = 1; pageNumber <= pageCount; pageNumber++) {

				Rectangle pageSize = reader.getPageSize(pageNumber);

				PdfContentByte pageContents = stamper.getOverContent(pageNumber);

				ColumnText columnText = new ColumnText(pageContents);
				columnText.setAlignment(com.lowagie.text.Element.ALIGN_CENTER);
				columnText.setLeading(fontHeight * 3/4f); // Center lower case letters within fontHeight

				float pageBottom = pageSize.getBottom() + 5;
				
//				System.out.println(" L" + columnText.getLeading() + " ML:" + (columnText.getMultipliedLeading() * fontHeight) + " FH:" + fontHeight);
//
//				Rectangle rect = new Rectangle(pageSize.getLeft(), pageBottom + fontHeight + 1, pageSize.getRight(), pageBottom + 1 + 2 * fontHeight);
//				rect.setBorder(Rectangle.BOX);
//				rect.setBorderWidth(2);
//				pageContents.rectangle(rect);
//
//				rect = new Rectangle(pageSize.getLeft(), pageBottom, pageSize.getRight(), pageBottom + fontHeight);
//				rect.setBorder(Rectangle.BOX);
//				rect.setBorderWidth(2);
//				rect.setBorderColor(java.awt.Color.RED);
//				pageContents.rectangle(rect);
				
				columnText.setSimpleColumn(pageSize.getLeft(), pageBottom + fontHeight + 1, pageSize.getRight(), pageBottom + 1 + 2 * fontHeight); // llx, lly, urx, ury)
				columnText.setText(new Phrase(signingDocument.getName(), font));
				
				int writeTextResult = columnText.go();

				if (writeTextResult != ColumnText.NO_MORE_TEXT) {
					log.warn("Unable to fit bottom document name text on page " + pageNumber + ": " + writeTextResult + ", for document " + signingDocument);
				}
				
				columnText.setSimpleColumn(pageSize.getLeft(), pageBottom, pageSize.getRight(), pageBottom + fontHeight); // llx, lly, urx, ury)
				columnText.setText(new Phrase(bottomSignedText, font));

				writeTextResult = columnText.go();

				if (writeTextResult != ColumnText.NO_MORE_TEXT) {
					log.warn("Unable to fit bottom signed text on page " + pageNumber + ": " + writeTextResult + ", for document " + signingDocument);
				}

				String pageNumberText = pdfPageNumber1 + pageNumber + pdfPageNumber2 + pageCount;

				float pageNumberTextWidth = baseFont.getWidthPoint(pageNumberText, font.getSize()) + 0.1f;

				columnText.setAlignment(com.lowagie.text.Element.ALIGN_LEFT);
				columnText.setSimpleColumn(pageSize.getRight() - pageNumberTextWidth - 5, pageBottom + fontHeight, pageSize.getRight() - 5, pageBottom + 2 * fontHeight);
				columnText.setText(new Phrase(pageNumberText, font));

				writeTextResult = columnText.go();

				if (writeTextResult != ColumnText.NO_MORE_TEXT) {
					log.warn("Unable to fit pagenumbering text on page " + pageNumber + ": " + writeTextResult + ", for document " + signingDocument);
				}
			}

			// Append attachments
			PdfArray associatedFilesArray = reader.getCatalog().getAsArray(new PdfName("AF"));

			if (associatedFilesArray == null) {

				associatedFilesArray = new PdfArray();
				reader.getCatalog().put(new PdfName("AF"), associatedFilesArray);
			}

			try {
				PdfFileSpecification fs = StreamPdfFileSpecification.fileEmbedded(stamper.getWriter(), new FileInputStream(inputFile), pdfUnsignedAttachment + ".pdf");
				stamper.getWriter().addFileAttachment(pdfUnsignedAttachment, fs);
				associatedFilesArray.add(fs.getReference());

			} catch (Exception e) {
				log.error("Error appending original PDF for signing to document " + signingDocument, e);
			}

			if (addSignatures) {

				for (DocumentSigner signer : signingDocument.getSigners()) {

					String attachmentName = pdfSignatureAttachment + " - " + signer.getFirstname() + " " + signer.getLastname();

					try {
						//TODO check if this encoding should be hardcoded or not
						PdfFileSpecification fs = StreamPdfFileSpecification.fileEmbedded(stamper.getWriter(), new ByteArrayInputStream(signer.getSignatureData().getBytes("ISO-8859-1")), attachmentName + ".txt");
						stamper.getWriter().addFileAttachment(attachmentName, fs);
						associatedFilesArray.add(fs.getReference());

					} catch (Exception e) {
						log.error("Error appending signatures for signer " + signer + " to signing PDF for document " + signingDocument, e);
					}
				}
			}

			if (associatedFilesArray.isEmpty()) {
				reader.getCatalog().put(new PdfName("AF"), null);
			}

			stamper.close();

		} catch (Exception e) {

			FileUtils.deleteFile(outputFile);

			throw e;

		} finally {

			CloseUtils.close(outputStream);

			if (randomAccessFile != null) {

				try {
					randomAccessFile.close();
				} catch (IOException e) {}
			}

			FileUtils.deleteFile(tmpFile);
		}
	}

	public void writePDFA(File inputFile, File outputFile) throws Exception {

		outputFile.getParentFile().mkdirs();

		PDDocument document = PDDocument.load(inputFile);

		try {
			PDDocumentCatalog catalog = document.getDocumentCatalog();
			PDDocumentInformation info = document.getDocumentInformation();

			XMPMetadata metadata = XMPMetadata.createXMPMetadata();

			PDFAIdentificationSchema id = metadata.createAndAddPFAIdentificationSchema();
			id.setPart(3);
			id.setConformance("A");

			AdobePDFSchema pdfSchema = metadata.createAndAddAdobePDFSchema();
			pdfSchema.setKeywords("Open ePlatform");
			pdfSchema.setProducer("Open ePlatform");

			GregorianCalendar calendar = new GregorianCalendar();

			XMPBasicSchema basicSchema = metadata.createAndAddXMPBasicSchema();
			basicSchema.setModifyDate(calendar);
			basicSchema.setCreateDate(calendar);
			basicSchema.setCreatorTool("Open ePlatform");
			basicSchema.setMetadataDate(new GregorianCalendar());

			DublinCoreSchema dcSchema = metadata.createAndAddDublinCoreSchema();
			dcSchema.setTitle(info.getTitle());
			dcSchema.addCreator("Open ePlatform");
			dcSchema.setDescription("Open ePlatform");

			PDMetadata metadataStream = new PDMetadata(document);
			catalog.setMetadata(metadataStream);

			XmpSerializer serializer = new XmpSerializer();
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			serializer.serialize(metadata, baos, false);
			metadataStream.importXMPMetadata(baos.toByteArray());

			if (!hasColorProfile(catalog)) {

				InputStream colorProfile = DocumentSignerAdminModule.class.getResourceAsStream("sRGB Color Space Profile.icm");

				PDOutputIntent oi = new PDOutputIntent(document, colorProfile);
				oi.setInfo("sRGB IEC61966-2.1");
				oi.setOutputCondition("sRGB IEC61966-2.1");
				oi.setOutputConditionIdentifier("sRGB IEC61966-2.1");
				oi.setRegistryName("http://www.color.org");
				catalog.addOutputIntent(oi);

			}

			document.save(outputFile);

		} finally {

			document.close();
		}
	}

	private boolean hasColorProfile(PDDocumentCatalog catalog) {

		List<PDOutputIntent> list = catalog.getOutputIntents();

		for (PDOutputIntent outputIntent : list) {

			if (outputIntent.getRegistryName() != null && outputIntent.getRegistryName().equalsIgnoreCase("http://www.color.org")) {

				return true;
			}
		}

		return false;
	}

	@Override
	public void run() {

		try {
			log.info("Running checks...");

			//Checking for document signings to cancel
			Timestamp now = new Timestamp(System.currentTimeMillis());

			LowLevelQuery<SigningDocument> queryCancelSigning = new LowLevelQuery<>("SELECT * FROM " + documentDAO.getTableName() + " d WHERE status = 'SIGNING' AND ? > DATE_ADD(startDate, INTERVAL cancelInXDays DAY)");
			queryCancelSigning.addRelation(SigningDocument.SIGNERS_RELATION);
			queryCancelSigning.addParameter(now);

			List<SigningDocument> cancelDocuments = documentDAO.getAll(queryCancelSigning);

			if (cancelDocuments != null) {
				for (SigningDocument document : cancelDocuments) {

					log.info("Canceling expired document: " + document);

					document.setStatus(SigningDocumentStatus.CANCELLED);

					documentDAOWrapper.update(document);

					sendSigningCancelledNotifications(document, null, null);
				}
			}

			//Checking for expired documents to delete
			LowLevelQuery<SigningDocument> queryDeleteOld = new LowLevelQuery<>("SELECT * FROM " + documentDAO.getTableName() + " d WHERE status != 'SIGNING' AND ? > DATE_ADD(COALESCE(completedDate, startDate, added), INTERVAL deleteAfterXDays DAY)");
			queryDeleteOld.addRelation(SigningDocument.SIGNERS_RELATION);
			queryDeleteOld.addParameter(now);

			List<SigningDocument> oldDocuments = documentDAO.getAll(queryDeleteOld);

			if (oldDocuments != null) {

				for (SigningDocument document : oldDocuments) {

					log.info("Deleting expired document: " + document);

					FileUtils.deleteFile(getOriginalPDF(document));
					FileUtils.deleteFile(getSignedPDF(document));
					FileUtils.deleteFile(getSignReceiptPDF(document));

					documentDAOWrapper.delete(document);
				}
			}

			//Checking for expired documents
			LowLevelQuery<SigningDocument> queryWarnExpiring = new LowLevelQuery<>("SELECT * FROM " + documentDAO.getTableName() + " d WHERE deleteAfterXDays > 7 AND expiringSent = 0 AND status != 'SIGNING' AND ? > DATE_ADD(COALESCE(completedDate, startDate, added), INTERVAL deleteAfterXDays DAY)");
			queryWarnExpiring.addRelation(SigningDocument.SIGNERS_RELATION);
			queryWarnExpiring.addParameter(DateUtils.getTodaysDatePlusMinusDays(7));

			List<SigningDocument> expiringDocuments = documentDAO.getAll(queryWarnExpiring);

			if (expiringDocuments != null) {

				for (SigningDocument document : expiringDocuments) {

					log.info("Sending notification about expiring document: " + document);

					LocalDate date = ObjectUtils.getFirstNotNull(document.getCompletedDate(), document.getStartDate(), document.getAdded()).toLocalDateTime().toLocalDate();
					date = date.plus(document.getDeleteAfterXDays(), ChronoUnit.DAYS);

					String message = documentExpiringOwnerEmailMessage.replace("$removalDate", DateUtils.DATE_FORMATTER.format(Date.valueOf(date)));

					sendSigningEmail(document, null, null, true, documentExpiringOwnerEmailSubject, message, null);

					document.setExpiringSent(true);
					documentDAOWrapper.update(document);
				}
			}

			//Checking for pending documents
			LowLevelQuery<SigningDocument> queryReminder = new LowLevelQuery<>("SELECT * FROM " + documentDAO.getTableName() + " d WHERE reminderSent = 0 AND status = 'SIGNING' AND ? > DATE_ADD(startDate, INTERVAL sendReminderInXDays DAY)");
			queryReminder.addRelation(SigningDocument.SIGNERS_RELATION);
			queryReminder.addParameter(now);

			List<SigningDocument> reminderDocuments = documentDAO.getAll(queryReminder);

			if (reminderDocuments != null) {
				for (SigningDocument document : reminderDocuments) {

					log.info("Sending reminder for pending document: " + document);

					sendSigningNotifications(document, null, false);

					document.setReminderSent(true);
					documentDAOWrapper.update(document);
				}
			}

		} catch (Throwable t) {

			log.error("Error encountered during cron checks", t);
		}
	}

	@WebPublic(requireLogin = true)
	public ForegroundModuleResponse checkForExpiringDocuments(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException {

		if (user.isAdmin()) {

			run();
			return new SimpleForegroundModuleResponse("See log", getDefaultBreadcrumb());
		}

		throw new URINotFoundException(uriParser);
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));

		Element moduleElement = moduleDescriptor.toXML(doc);
		XMLUtils.appendNewElement(doc, moduleElement, "fullAlias", getFullAlias());
		document.appendChild(moduleElement);

		if (user != null) {

			Element userElement = (Element) document.appendChild(user.toXML(doc));

			if (user.getAttributeHandler() != null) {

				userElement.appendChild(user.getAttributeHandler().toXML(doc));
			}
		}

		doc.appendChild(document);

		return doc;
	}

	public String getDocumentDir() {

		return filestore;
	}

	@Override
	public String getTempDir() {

		return tempDir;
	}

	@Override
	public long getMaxRequestSize() {

		return maxRequestSize;
	}

	public long getMaxDocumentSize() {

		return maxDocumentSize;
	}

	@Override
	public int getRamThreshold() {

		return ramThreshold;
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public Integer getDefaultCancelInXDays() {

		return defaultCancelInXDays;
	}

	public Integer getDefaultSendReminderInXDays() {

		return defaultSendReminderInXDays;
	}

	public Integer getDefaultDeleteAfterXDays() {

		return defaultDeleteAfterXDays;
	}

	@WebPublic(alias = "users")
	public ForegroundModuleResponse getUsers(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return userGroupListConnector.getUsers(req, res, user, uriParser);
	}

	public UserHandler getUserHandler() {

		return systemInterface.getUserHandler();
	}

	public SiteProfile getCurrentSiteProfile(HttpServletRequest req, User user, URIParser uriParser) {

		if (profileHandler != null) {

			return profileHandler.getCurrentProfile(user, req, uriParser);
		}

		return null;
	}

	public String getSigningURL(SigningDocument document, DocumentSigner signer, URIParser uriParser) {

		String fullContextPath;

		if (uriParser != null) {

			fullContextPath = uriParser.getFullContextPath();

		} else { // Used for system notifications

			SiteProfile profile = profileHandler.getProfile(document.getProfileID());
			fullContextPath = "https://" + profile.getDomains().get(0) + systemInterface.getContextPath();
		}

		return getSigningURL(document, signer, fullContextPath);
	}

	public String getSigningURL(SigningDocument document, DocumentSigner signer, String fullContextPath) {

		String urlEnd = document.getDocumentID().toString();

		if (signer != null && signer.isUnsecure()) {

			urlEnd += "/" + signer.getUnsecureKey();
		}

		return fullContextPath + userModule.getFullAlias() + "/sign/" + urlEnd;
	}

	public String getDocumentURL(SigningDocument document, URIParser uriParser) {

		String fullContextPath;

		if (uriParser != null) {

			fullContextPath = uriParser.getFullContextPath();

		} else { // Used for system notifications

			SiteProfile profile = profileHandler.getProfile(document.getProfileID());
			fullContextPath = "https://" + profile.getDomains().get(0) + systemInterface.getContextPath();
		}

		return fullContextPath + getFullAlias() + "/showdocument/" + document.getDocumentID();
	}

	public void sendSigningNotifications(SigningDocument signingDocument, URIParser uriParser, boolean skipPoster) throws SQLException {

		for (DocumentSigner signer : signingDocument.getSigners()) {

			if (signer.getSignedDate() == null) {

				if (!skipPoster || !signingDocument.getPoster().equals(signer.getUser())) {

					sendSigningNotifications(signingDocument, signer, uriParser);
				}

				if (signingDocument.isUseSequentialSigning()) {
					break;
				}
			}
		}
	}

	public void sendSigningNotifications(SigningDocument signingDocument, DocumentSigner signer, URIParser uriParser) {

		sendSigningEmail(signingDocument, signer, null, false, signingInitiatedUserEmailSubject, signingInitiatedUserEmailMessage, uriParser);
		sendSigningSMS(signingDocument, signer, null, signingInitiatedUserSMS, uriParser);
	}

	public void sendSigningCancelledNotifications(SigningDocument document, DocumentSigner cancellingSigner, URIParser uriParser) {

		Contact contact = null;

		if (cancellingSigner == null) { // Poster or system cancelled

			contact = document.getPosterContact();

		} else { // One of the signers cancelled

			contact = cancellingSigner.getContact();

			sendSigningEmail(document, cancellingSigner, contact, true, signingCanceledOwnerEmailSubject, signingCanceledOwnerEmailMessage, uriParser);
		}

		for (DocumentSigner signer : document.getSigners()) {

			if (signer.equals(cancellingSigner) || (document.isUseSequentialSigning() && DocumentSignerUserModule.isSequentialSigningBlocked(document, signer))) {
				continue;
			}

			sendSigningEmail(document, signer, contact, false, signingCanceledUserEmailSubject, signingCanceledUserEmailMessage, uriParser);
			sendSigningSMS(document, signer, contact, signingCanceledUserSMS, uriParser);
		}
	}
	
	public void sendSigningPartiallyCompletedNotification(SigningDocument signingDocument, URIParser uriParser, User user) {

		//do not send "partially completed" - mail to poster when poster is the signer
		if(!signingDocument.getPoster().equals(user))
		{
			sendSigningEmail(signingDocument, null, null, true, signingPartiallyCompletedOwnerEmailSubject, signingPartiallyCompletedOwnerEmailMessage, uriParser);
		}
	}

	public void sendSigningCompleteNotification(SigningDocument signingDocument, URIParser uriParser) {

		sendSigningEmail(signingDocument, null, null, true, signingCompletedOwnerEmailSubject, signingCompletedOwnerEmailMessage, uriParser);
	}

	private boolean sendSigningEmail(SigningDocument document, DocumentSigner signer, Contact contact, boolean toPoster, String subject, String message, URIParser uriParser) {

		if (subject == null || message == null || (!toPoster && signer.getEmail() == null)) {
			return false;
		}

		if (toPoster && (document.getPoster() == null || !EmailUtils.isValidEmailAddress(document.getPoster().getEmail()))) {
			log.warn("Poster null or invalid email for document " + document);
			return false;
		}

		if (!toPoster && !EmailUtils.isValidEmailAddress(signer.getEmail())) {

			log.warn("Signer " + signer + " from " + document + " has invalid email");
			return false;
		}
		
		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(new SingleTagSource("$document.description", HTMLUtils.replaceLineBreaksWithParagraph(document.getDescription())));

		tagReplacer.addTagSource(SIGNING_DOCUMENT_TAG_SOURCE_FACTORY.getTagSource(document));

		if (contact != null) {

			tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		}

		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(document.getPoster()));
		tagReplacer.addTagSource(SIGNER_TAG_SOURCE_FACTORY.getTagSource(signer));

		if (toPoster) {

			tagReplacer.addTagSource(new SingleTagSource("$documentURL", getDocumentURL(document, uriParser)));
			
			long nrOfSignersLeft = document.getSigners().stream().filter(Objects::nonNull).filter(e->e.getSignedDate()==null).count(); 
			tagReplacer.addTagSource(new SingleTagSource("$nrOfSignersLeft", nrOfSignersLeft));
			

		} else {

			tagReplacer.addTagSource(new SingleTagSource("$signingURL", getSigningURL(document, signer, uriParser)));
		}

		SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());

		try {

			if (toPoster) {

				email.addRecipient(document.getPoster().getEmail());

			} else {

				email.addRecipient(signer.getEmail());
			}

			FlowInstance dummyFlowInstance = new FlowInstance();

			dummyFlowInstance.setProfileID(document.getProfileID());

			email.setMessageContentType(SimpleEmail.HTML);
			email.setSenderName(flowNotificationHandler.getEmailSenderName(dummyFlowInstance));
			email.setSenderAddress(flowNotificationHandler.getEmailSenderAddress(dummyFlowInstance));
			email.setSubject(tagReplacer.replace(subject));
			email.setMessage(EmailUtils.addMessageBody(tagReplacer.replace(message)));

			systemInterface.getEmailHandler().send(email);

		} catch (Exception e) {

			log.error("Error generating/sending email " + email + " document " + document, e);
			return false;
		}

		return true;
	}

	private void sendSigningSMS(SigningDocument document, DocumentSigner signer, Contact contact, String message, URIParser uriParser) {

		if (smsSender == null || message == null) {
			return;
		}

		// Skip signing SMS notifications to poster and internal users
		if (signer.getType() == SignerType.POSTER || signer.getType() == SignerType.INTERNAL_USER) {
			return;
		}

		String mobilePhone = signer.getMobilePhone();

		if (mobilePhone == null) {
			return;
		}

		TagReplacer tagReplacer = new TagReplacer();

		tagReplacer.addTagSource(SIGNING_DOCUMENT_TAG_SOURCE_FACTORY.getTagSource(document));

		if (contact != null) {

			tagReplacer.addTagSource(CONTACT_TAG_SOURCE_FACTORY.getTagSource(contact));
		}

		tagReplacer.addTagSource(POSTER_TAG_SOURCE_FACTORY.getTagSource(document.getPoster()));
		tagReplacer.addTagSource(SIGNER_TAG_SOURCE_FACTORY.getTagSource(signer));

		tagReplacer.addTagSource(new SingleTagSource("$signingURL", getSigningURL(document, signer, uriParser)));

		SimpleSMS sms = new SimpleSMS();

		try {
			//TODO This needs to be rewritten
			FlowInstance dummyFlowInstance = new FlowInstance();
			dummyFlowInstance.setProfileID(document.getProfileID());

			sms.setSenderName(flowNotificationHandler.getSMSSenderName(dummyFlowInstance));
			sms.setMessage(tagReplacer.replace(message));
			sms.addRecipient(mobilePhone);

			smsSender.send(sms);

		} catch (Exception e) {

			log.error("Error generating/sending sms " + sms + " for document " + document, e);
		}
	}

	@WebPublic(alias = "cancel", requireLogin = true)
	public ForegroundModuleResponse cancelSigning(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if (!HTTPUtils.isPost(req)) {

			throw new AccessDeniedException("Cancel signing requests using method " + req.getMethod() + " are not allowed.");
		}

		SigningDocument document = documentCRUD.getRequestedBean(req, res, user, uriParser, GenericCRUD.SHOW);

		if (document == null) {

			throw new URINotFoundException(uriParser);
		}

		if (!user.equals(document.getPoster())) {

			throw new AccessDeniedException("User is not poster for " + document);
		}

		if (document.getStatus() != SigningDocumentStatus.SIGNING) {

			throw new AccessDeniedException("May only cancel documents in signing phase. " + document);
		}

		log.info(user + " cancelling signing of " + document);

		document.setStatus(SigningDocumentStatus.CANCELLED);

		documentDAOWrapper.update(document);

		sendSigningCancelledNotifications(document, null, uriParser);

		redirectToMethod(req, res, "/showdocument/" + document.getDocumentID());
		return null;
	}

	@WebPublic(alias = "sign", requireLogin = true)
	public ForegroundModuleResponse signDocument(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		res.sendRedirect(req.getContextPath() + userModule.getFullAlias() + "/sign/" + uriParser.get(2));
		return null;
	}

	@WebPublic(toLowerCase = true, requireLogin = true)
	public ForegroundModuleResponse validateField(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException {

		String fieldName = req.getParameter("field");
		String fieldValue = req.getParameter("value");
		boolean valid = false;

		log.info("Validating field " + fieldName + " = " + fieldValue + " for user " + user);

		if ("email".equals(fieldName)) {
			valid = EmailPopulator.getPopulator().validateFormat(fieldValue);

		} else if ("mobilePhone".equals(fieldName)) {
			valid = SwedishPhoneNumberValidator.getValidator().validateFormat(fieldValue);

		} else if ("citizenID".equals(fieldName)) {
			valid = SwedishSocialSecurity12DigitsWithoutMinusPopulator.getPopulator().validateFormat(fieldValue);
		}

		JsonObject result = new JsonObject();
		result.putField("valid", valid);

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());

		Writer writer = null;

		try {
			writer = res.getWriter();
			writer.append(result.toJson());
			writer.flush();

		} catch (IOException e) {
			log.warn("Error sending validate field result to " + user, e);

		} finally {
			CloseUtils.close(writer);
		}

		return null;
	}

	public void checkAccess(SigningDocument signingDocument, User user) throws AccessDeniedException {

		if (user == null || !user.equals(signingDocument.getPoster())) {

			throw new AccessDeniedException("User does not have access to signing document " + signingDocument);
		}
	}

	public String getHashAlgorithm() {

		return hashAlgorithm;
	}

	public String getTitleAttribute() {

		return titleAttribute;
	}

	public boolean sendEmailWhenSigningPartiallyDone() {

		return sendEmailWhenSigningPartiallyDone;
	}
}
