package com.nordicpeak.flowengine.queries.dynamictablequery;

import java.lang.reflect.Field;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.queries.basetablequery.TableCellValue;

@Table(name = "dynamic_table_query_instance_values")
@XMLElement(name = "CellValue")
public class DynamicTableCellValue extends TableCellValue<DropDownAlternative, DynamicTableQuery, DynamicTableRow, DynamicTableColumn> {

	private static final long serialVersionUID = 854568573886179411L;

	public static final Field QUERY_INSTANCE_RELATION = ReflectionUtils.getField(DynamicTableCellValue.class, "instance");
	public static final Field COLUMN_RELATION = ReflectionUtils.getField(DynamicTableCellValue.class, "column");
	public static final Field ROW_RELATION = ReflectionUtils.getField(DynamicTableCellValue.class, "row");

	@DAOManaged(columnName = "queryInstanceID")
	@ManyToOne
	private DynamicTableQueryInstance instance;

	@DAOManaged(columnName = "columnID")
	@ManyToOne
	@XMLElement
	private DynamicTableColumn column;

	@DAOManaged(columnName = "rowID")
	@ManyToOne
	private DynamicTableRow row;

	public DynamicTableCellValue() {}

	public DynamicTableCellValue(DynamicTableRow row, DynamicTableColumn column, String value) {

		super(value);

		this.row = row;
		this.column = column;
	}

	public DynamicTableCellValue(DynamicTableRow row, DynamicTableColumn column, String value, String displayValue, String xmlValue) {

		super(value, displayValue, xmlValue);

		this.row = row;
		this.column = column;
	}

	public DynamicTableQueryInstance getInstance() {

		return instance;
	}

	public void setInstance(DynamicTableQueryInstance instance) {

		this.instance = instance;
	}

	@Override
	public DynamicTableColumn getColumn() {

		return column;
	}

	public void setColumn(DynamicTableColumn textField) {

		this.column = textField;
	}

	@Override
	public DynamicTableRow getRow() {

		return row;
	}

	public void setRow(DynamicTableRow tableRow) {

		this.row = tableRow;
	}

	@Override
	public Element toXML(Document doc) {

		Element element = super.toXML(doc);

		if (row != null) {

			Element rowElement = XMLUtils.appendNewElement(doc, element, "Row");

			XMLUtils.appendNewElement(doc, rowElement, "rowID", row.getRowIdentifier());
		}

		return element;
	}
}
