package com.nordicpeak.flowengine.queries.dynamictablequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.queries.basetablequery.BaseTableQueryInstance;

@Table(name = "dynamic_table_query_instances")
@XMLElement
public class DynamicTableQueryInstance extends BaseTableQueryInstance<DropDownAlternative, DynamicTableRow, DynamicTableColumn, DynamicTableQuery, DynamicTableCellValue> {

	private static final long serialVersionUID = -8794537859820177303L;

	public static final Field VALUES_RELATION = ReflectionUtils.getField(DynamicTableQueryInstance.class, "values");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(DynamicTableQueryInstance.class, "query");
	public static final Field ROWS_RELATION = ReflectionUtils.getField(DynamicTableQueryInstance.class, "rows");

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private DynamicTableQuery query;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<DynamicTableCellValue> values;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<DynamicTableRow> rows;

	@Override
	public DynamicTableQuery getQuery() {

		return query;
	}

	@Override
	public void setQuery(DynamicTableQuery query) {

		this.query = query;
	}

	@Override
	public List<DynamicTableCellValue> getValues() {

		return values;
	}

	@Override
	public void setValues(List<DynamicTableCellValue> values) {

		this.values = values;
	}

	@Override
	public void setValue(DynamicTableCellValue cellValue) {

		values = setValue(values, cellValue);
	}

	@Override
	public String getValue(String rowIdentifier, Integer columnID) {

		return getValue(values, rowIdentifier, columnID);
	}

	@Override
	public DynamicTableCellValue getCellValue(String rowIdentifier, Integer columnID) {

		return getCellValue(values, rowIdentifier, columnID);
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		this.rows = null;

		super.reset(attributeHandler);
	}

	@Override
	public List<DynamicTableRow> getRows() {

		return rows;
	}

	public void setRows(List<DynamicTableRow> rows) {

		this.rows = rows;
	}

	public void generateRowsFromValues() {

		if (values == null) {

			return;
		}

		Map<String, DynamicTableRow> rowsMap = new HashMap<>(values.size() / query.getColumns().size());

		for (DynamicTableCellValue value : values) {

			DynamicTableRow row = rowsMap.get(value.getRow().getRowIdentifier());

			if (row == null) {

				row = value.getRow();
				rowsMap.put(row.getRowIdentifier(), row);
			}

			if (row.getValues() == null) {

				row.setValues(new ArrayList<DynamicTableCellValue>());
			}

			row.getValues().add(value);
		}

		List<DynamicTableRow> rows2 = new ArrayList<>(rowsMap.values());

		Collections.sort(rows2);

		rows = rows2;
	}

}
