package com.nordicpeak.flowengine.easitgetitemsalternativelist;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.sql.DataSource;
import javax.xml.ws.BindingProvider;

import org.apache.log4j.Level;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOFactory;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.jaxws.WSUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.MillisecondTimeUnits;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.easit.bpsservice.proxy.BPS;
import com.easit.bpsservice.proxy.BPSService;
import com.easit.bpsservice.proxy.GetItemsRequest;
import com.easit.bpsservice.proxy.GetItemsResponse;
import com.easit.bpsservice.proxy.GetItemsResponse.Items;
import com.easit.bpsservice.proxy.Item;
import com.easit.bpsservice.proxy.Item.Property;
import com.nordicpeak.flowengine.alternativelist.beans.AlternativeListDescriptor;
import com.nordicpeak.flowengine.alternativelist.interfaces.AlternativeListInstance;
import com.nordicpeak.flowengine.alternativelist.provider.BaseAlternativeListProvider;

public class EasitGetItemsAlternativeListProviderModule extends BaseAlternativeListProvider<ListSource, EasitGetItemsAlternativeListInstance, ListSourceCRUD, EasitGetItemsAlternativeListProviderModule, ListSourceEvent> {

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Maximum pages", description = "The maximum number of pages to request")
	private Integer maximumPages = 100;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Debug logging", description = "Toggles debug logging when calling esit web service")
	protected boolean enableDebugLoggning;

	private AnnotatedDAO<EasitGetItemsEndpoint> endPointDAO;
	private QueryParameterFactory<EasitGetItemsEndpoint, Integer> endPointIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, EasitGetItemsAlternativeListProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		AnnotatedDAOFactory daoFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface);

		listDAO = daoFactory.getDAO(ListSource.class).getWrapper(Integer.class);
		listEventDAO = daoFactory.getDAO(ListSourceEvent.class);
		listEventListParamFactory = listEventDAO.getParamFactory(ListSourceEvent.LIST_RELATION, ListSource.class);

		listCRUD = new ListSourceCRUD(listDAO, new AnnotatedRequestPopulator<ListSource>(ListSource.class), this);

		endPointDAO = daoFactory.getDAO(EasitGetItemsEndpoint.class);
		endPointIDParamFactory = endPointDAO.getParamFactory("endpointID", Integer.class);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		if (ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR)) {

		}
	}

	@Override
	public AlternativeListInstance getAlternativeListInstance(String name, AttributeHandler attributeHandler, User poster) {

		EasitGetItemsAlternativeListInstance listInstance = getCachedInstance(name);

		if (listInstance != null) {

			return getAlternativeInstance(listInstance.getListSource(), attributeHandler, poster);
		}

		return null;
	}

	@Override
	public EasitGetItemsAlternativeListInstance getAlternativeInstance(ListSource source, AttributeHandler attributeHandler, User poster) {

		EasitGetItemsAlternativeListInstance listInstance = alternativeListCacheMap.get(source.getSourceID());

		if (listInstance != null) {

			if (source.getCacheRefreshTime() == null) {

				List<ExternalAlternative> alternatives = getAlternatives(source);

				if (alternatives == null) {
					return null;
				}

				listInstance = new EasitGetItemsAlternativeListInstance(new AlternativeListDescriptor(providerID, source.getListName()), getAlternatives(source), System.currentTimeMillis(), source);

			} else if (System.currentTimeMillis() - listInstance.getRetrivedTime() > source.getCacheRefreshTime() * MillisecondTimeUnits.MINUTE) {

				listInstance = cacheAlternatives(source);
			}
		}

		return listInstance;
	}

	protected List<ExternalAlternative> getAlternatives(ListSource source) {

		log.info("Fetching alternatives for list source " + source);

		try {

			BPS proxy = new BPSService().getBPSSoap11();

			BindingProvider bindingProvider = (BindingProvider) proxy;

			WSUtils.setEndpoint(bindingProvider, source.getEndPoint().getWebServiceAddress());
			WSUtils.setUsernamePassword(bindingProvider, source.getEndPoint().getApiKey(), source.getEndPoint().getApiKey());
			WSUtils.setTimeouts(bindingProvider, connectionTimeout, readTimeout);

			List<ExternalAlternative> alternatives = new ArrayList<ExternalAlternative>();
			List<Integer> addedIDs = new ArrayList<Integer>();

			int pageID = 1;

			outer: while (true) {

				GetItemsRequest request = new GetItemsRequest();
				request.setItemViewIdentifier(source.getViewIdentifier());
				request.setPage(pageID);

				GetItemsResponse response = proxy.getItems(request);

				Items items = response.getItems();

				if (items != null && items.getItem() != null) {

					for (Item item : items.getItem()) {

						if (item.getProperty() != null) {

							Integer id = null;
							String name = null;

							for (Property property : item.getProperty()) {

								if (source.getIdProperty().equals(property.getName())) {

									id = NumberUtils.toInt(property.getContent());

								} else if (source.getNameProperty().equals(property.getName())) {

									name = property.getContent();

								}

							}

							if (addedIDs.contains(id)) {

								break outer;
							}

							addedIDs.add(id);

							if (id != null && name != null) {

								ExternalAlternative alternative = new ExternalAlternative(id, name);

								if (source.isSetIdAsAttribute()) {
									alternative.setAttributeValue(alternative.getAlternativeID().toString());
								}

								alternatives.add(alternative);
							}

						}

					}

					pageID++;

					if (pageID > maximumPages) {

						break outer;
					}

				} else {

					break outer;
				}

			}

			Collections.sort(alternatives);

			return alternatives;

		} catch (Exception e) {

			log.warn("Error getting alternatives for " + source, e);
		}

		if (source.getListID() != null && source.isKeepOldListOnRefreshFail()) {

			EasitGetItemsAlternativeListInstance listInstance = alternativeListCacheMap.get(source.getListID());

			if (listInstance != null) {

				return listInstance.getAlternatives();
			}
		}

		return null;
	}

	protected void logDebug(String text) {

		if (enableDebugLoggning) {

			log.info(text);
		}

	}

	public boolean testSource(ListSource source) {

		List<ExternalAlternative> alternatives = getAlternatives(source);

		if (!CollectionUtils.isEmpty(alternatives)) {

			for (ExternalAlternative alternative : alternatives) {

				if (alternative.getAlternativeID() == null || StringUtils.isEmpty(alternative.getName())) {

					return false;
				}
			}

			return true;
		}

		return false;
	}

	@Override
	protected EasitGetItemsAlternativeListInstance getNewInstance(AlternativeListDescriptor descriptor, ListSource list) {

		return new EasitGetItemsAlternativeListInstance(descriptor, getAlternatives(list), System.currentTimeMillis(), list);
	}

	@Override
	protected ListSourceEvent getNewListEvent(ListSource list, Timestamp added, User poster, String message) {

		return new ListSourceEvent(list, added, poster, message);
	}

	public AnnotatedDAO<EasitGetItemsEndpoint> getEndPointDAO() {

		return endPointDAO;
	}

	public EasitGetItemsEndpoint getEndpoint(Integer endPointID) throws SQLException {

		HighLevelQuery<EasitGetItemsEndpoint> query = new HighLevelQuery<EasitGetItemsEndpoint>();

		query.addParameter(endPointIDParamFactory.getParameter(endPointID));

		return endPointDAO.get(query);
	}

}
