<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/js/confirmpost.js
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/evolutionintegration.js?v=1
	</xsl:variable>

	<xsl:variable name="links">
		/css/evolutionintegration.css
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">
	
		<xsl:apply-templates select="ShowIntegrationSettings" />
		<xsl:apply-templates select="UpdateIntegrationSettings" />
			
	</xsl:template>
	
	<xsl:template match="ShowIntegrationSettings">
	
		<a name="evolutionintegrationsettings" />
		
		<xsl:choose>
			<xsl:when test="IntegrationSettings">
					
				<xsl:value-of select="$i18n.IntegrationEnabled"/>
				<xsl:text>:&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="IntegrationSettings/enabled = 'true'">
						<xsl:value-of select="$i18n.Yes"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No"/>
					</xsl:otherwise>
				</xsl:choose>
				
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$i18n.NoIntegrationSettings" />
			</xsl:otherwise>
		</xsl:choose>
		
		<a href="{/Document/ModuleURI}/update/{Flow/flowID}" class="floatright">
			<xsl:value-of select="$i18n.UpdateIntegrationSettings" />
			<img class="alignbottom" src="{/Document/StaticContentURL}/pics/pen.png" alt="" />
		</a>
		
		<xsl:if test="IntegrationSettings">
			<a href="{/Document/ModuleURI}/delete/{Flow/flowID}" class="floatright clearboth" onclick="return confirmPost(this.href, '{$i18n.DeleteIntegrationSettings.Confirm}');">
				<xsl:value-of select="$i18n.DeleteIntegrationSettings" />
				<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
			</a>		
		</xsl:if>
	
	</xsl:template>
	
	<xsl:template match="UpdateIntegrationSettings">
		
		<div id="CastorIntegrationProvider" class="contentitem errands-wrapper border-box">
		
			<h1>
				<xsl:value-of select="$i18n.UpdateIntegrationSettingsTitle"/>
				<xsl:text>:&#x20;</xsl:text>
				<xsl:value-of select="Flow/name"/>
			</h1>
	
			<xsl:apply-templates select="validationException/validationError"/>
	
			<form method="post" action="{/Document/ModuleURI}/update/{Flow/flowID}">
			
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'enabled'" />
						<xsl:with-param name="id" select="'enabled'" />
						<xsl:with-param name="element" select="IntegrationSettings" />       
					</xsl:call-template>
					
					<label for="enabled">
						<xsl:value-of select="$i18n.Enabled" />
					</label>
				</div>
			
				<label for="endpointID" class="floatleft full">
					<xsl:value-of select="$i18n.Endpoint" />
				</label>
				
				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'endpointID'"/>
						<xsl:with-param name="name" select="'endpointID'"/>
						<xsl:with-param name="element" select="Endpoints/Endpoint" /> 
						<xsl:with-param name="labelElementName" select="'name'" />        
						<xsl:with-param name="valueElementName" select="'endpointID'" />
						<xsl:with-param name="selectedValue" select="IntegrationSettings/Endpoint/endpointID"/>
					</xsl:call-template>
				</div>
				
				<div class="floatleft full bigmargintop">
		
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'createCaseOnSpecificStatus'" />
						<xsl:with-param name="id" select="'createCaseOnSpecificStatus'" />
						<xsl:with-param name="element" select="IntegrationSettings" />
					</xsl:call-template>
					
					<label class="marginright" for="createCaseOnSpecificStatus">
						<xsl:value-of select="$i18n.createCaseOnSpecificStatus" />
					</label>
					
				</div>	
				
				<div class="floatleft full bigmarginbottom" data-connector-url="{/Document/requestinfo/currentURI}/{/Document/module/alias}/statuses/{Flow/flowID}">
		
					<p>
						<xsl:value-of select="$i18n.createCaseOnSpecificStatus.Description" />
					</p>
		
					<label class="nomargin" for="caseStatusName">
						<xsl:value-of select="$i18n.caseStatusName" />
					</label>
					
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'caseStatusName'"/>
						<xsl:with-param name="name" select="'caseStatusName'"/>
						<xsl:with-param name="element" select="IntegrationSettings" />          
					</xsl:call-template>
				</div>
				
				<div class="clearboth marginbottom">
					<br/>
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.SupportedTags.Title"/></h2>
					
					<p><xsl:value-of select="$i18n.SupportedTags.Description"/></p>
					
					<p><xsl:value-of select="SupportedTags"/></p>
				
				</div>						
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.CaseData"/></h2>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'unitCode'" />
						<xsl:with-param name="label" select="$i18n.unitCode"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextAreaSetting">
						<xsl:with-param name="name" select="'description'" />
						<xsl:with-param name="label" select="$i18n.description"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextAreaSetting">
						<xsl:with-param name="name" select="'notes'" />
						<xsl:with-param name="label" select="$i18n.notes"/>
					</xsl:call-template>
					
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
				
					<h2><xsl:value-of select="$i18n.ContactDetails"/></h2>	
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'firstname'" />
						<xsl:with-param name="label" select="$i18n.firstname"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'lastname'" />
						<xsl:with-param name="label" select="$i18n.lastname"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'address'" />
						<xsl:with-param name="label" select="$i18n.address"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'coAddress'" />
						<xsl:with-param name="label" select="$i18n.coAddress"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'zipCode'" />
						<xsl:with-param name="label" select="$i18n.zipCode"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'region'" />
						<xsl:with-param name="label" select="$i18n.region"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'email'" />
						<xsl:with-param name="label" select="$i18n.email"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'phone'" />
						<xsl:with-param name="label" select="$i18n.phone"/>
					</xsl:call-template>
					
					<xsl:call-template name="createTextFieldSetting">
						<xsl:with-param name="name" select="'mobilePhone'" />
						<xsl:with-param name="label" select="$i18n.mobilePhone"/>
					</xsl:call-template>
									
				</div>
				
				<div class="clearboth marginbottom">
					<br/>
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
		
					<h2><xsl:value-of select="$i18n.Attachments"/></h2>
				
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="name" select="'addAttachmentsSeparately'" />
						<xsl:with-param name="id" select="'addAttachmentsSeparately'" />
						<xsl:with-param name="element" select="IntegrationSettings" />
					</xsl:call-template>
					
					<label class="marginright" for="addAttachmentsSeparately">
						<xsl:value-of select="$i18n.addAttachmentsSeparately" />
					</label>
					
				</div>
				
				<div class="clearboth marginbottom">
					<br/>
				</div>
				
				<div class="floatleft full bigmargintop bigmarginbottom">
		
					<h2><xsl:value-of select="$i18n.Notifications"/></h2>
				
					<xsl:call-template name="notificationEmail">
						<xsl:with-param name="toggleField" select="'sendErrorEmail'" />
						<xsl:with-param name="addressesField" select="'errorEmailAddresses'" />
						<xsl:with-param name="toggleLabel" select="$i18n.Notifications.ErrorEmail" />
						<xsl:with-param name="addressesLabel" select="$i18n.Notifications.ErrorEmail.EmailAddresses" />
						<xsl:with-param name="addresses" select="IntegrationSettings/ErrorEmailAddresses/Address" />
					</xsl:call-template>
					
				</div>
			
				<div class="floatright clearboth bigmargintop">
					<input type="submit" value="{$i18n.SaveChanges}" />
				</div>
			
			</form>
		
			<div style="display: none;">
		
				<table>
					<tr id="attachmentTypeMapping-template">
						<td>
							<input type="text" name="queryName" class="full"/>
						</td>
						<td>
							<input type="text" name="attachmentTypeID" class="full"/>
						</td>
						<td>
							<input type="hidden" name="attachmentTypeMappingID"/>
							
							<a onclick="deleteAttachmentTypeMapping(this)">
								<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
							</a>				
						</td>
					</tr>
				</table>
				
			</div>
				
		</div>

	</xsl:template>
	
	<xsl:template name="createTextFieldSetting">
		<xsl:param name="name" />
		<xsl:param name="label" />
		
		<div class="floatleft full">
		
			<label for="{$name}">
				<xsl:value-of select="$label" />
			</label>
			
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="$name"/>
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="element" select="IntegrationSettings" />          
			</xsl:call-template>
		</div>
	
	</xsl:template>
	
	<xsl:template name="createTextAreaSetting">
		
		<xsl:param name="name" />
		<xsl:param name="label" />
		
		<div class="floatleft full">
		
			<label for="{$name}">
				<xsl:value-of select="$label" />
			</label>
			
			<xsl:call-template name="createTextArea">
				<xsl:with-param name="id" select="$name"/>
				<xsl:with-param name="name" select="$name"/>
				<xsl:with-param name="element" select="IntegrationSettings" />
			</xsl:call-template>
		</div>
	
	</xsl:template>	
	
	<xsl:template name="notificationEmail">
		<xsl:param name="toggleField" />
		<xsl:param name="addressesField" />
		<xsl:param name="toggleLabel" />
		<xsl:param name="addressesLabel" />
		<xsl:param name="addresses" />
	
		<div class="notification-email">
		
			<div>
			
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="name" select="$toggleField" />
					<xsl:with-param name="id" select="$toggleField" />
					<xsl:with-param name="element" select="IntegrationSettings" />
					<xsl:with-param name="class" select="'notification-email-toggle'" />
				</xsl:call-template>
				
				<label class="marginright" for="{$toggleField}">
					<xsl:value-of select="$toggleLabel" />
				</label>
			</div>
			
			<div class="notification-email-text">
			
				<xsl:if test="not(validationException/validationError/fieldName = $addressesField)">
					<xsl:attribute name="class">notification-email-text hidden</xsl:attribute>
				</xsl:if>
			
				<div>
				
					<label for="{$addressesField}">
						<xsl:value-of select="$addressesLabel" />
					</label>
					
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="$addressesField"/>
						<xsl:with-param name="name" select="$addressesField"/>
						<xsl:with-param name="rows" select="5"/>
						<xsl:with-param name="separateListValues" select="'true'"/>
						<xsl:with-param name="element" select="$addresses" />
					</xsl:call-template>
				</div>
			</div>
		</div>
			
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'endpointID'">
						<xsl:value-of select="$i18n.Endpoint"/>
					</xsl:when>
					<xsl:when test="fieldName = 'unitCode'">
						<xsl:value-of select="$i18n.unitCode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'description'">
						<xsl:value-of select="$i18n.description"/>
					</xsl:when>													
					<xsl:when test="fieldName = 'notes'">
						<xsl:value-of select="$i18n.notes"/>
					</xsl:when>
					<xsl:when test="fieldName = 'firstname'">
						<xsl:value-of select="$i18n.firstname"/>
					</xsl:when>
					<xsl:when test="fieldName = 'lastname'">
						<xsl:value-of select="$i18n.lastname"/>
					</xsl:when>
					<xsl:when test="fieldName = 'address'">
						<xsl:value-of select="$i18n.address"/>
					</xsl:when>
					<xsl:when test="fieldName = 'coAddress'">
						<xsl:value-of select="$i18n.coAddress"/>
					</xsl:when>
					<xsl:when test="fieldName = 'zipCode'">
						<xsl:value-of select="$i18n.zipCode"/>
					</xsl:when>
					<xsl:when test="fieldName = 'region'">
						<xsl:value-of select="$i18n.region"/>
					</xsl:when>
					<xsl:when test="fieldName = 'email'">
						<xsl:value-of select="$i18n.email"/>
					</xsl:when>
					<xsl:when test="fieldName = 'phone'">
						<xsl:value-of select="$i18n.phone"/>
					</xsl:when>
					<xsl:when test="fieldName = 'mobilePhone'">
						<xsl:value-of select="$i18n.mobilePhone"/>
					</xsl:when>
					<xsl:when test="fieldName = 'caseStatusName'">
						<xsl:value-of select="$i18n.caseStatusName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'errorEmailAddresses'">
						<xsl:value-of select="$i18n.Notifications.ErrorEmail.EmailAddresses"/>
					</xsl:when>																					
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>