package com.nordicpeak.flowengine.queries.fixedtablequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basetablequery.BaseTableQuery;

@Table(name = "fixed_table_queries")
@XMLElement
public class FixedTableQuery extends BaseTableQuery<DropDownAlternative, FixedTableQuery, FixedTableColumn> {

	private static final long serialVersionUID = 5116885863935428968L;

	public static final Field COLUMNS_RELATION = ReflectionUtils.getField(FixedTableQuery.class, "columns");
	public static final Field ROWS_RELATION = ReflectionUtils.getField(FixedTableQuery.class, "rows");

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<FixedTableColumn> columns;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<FixedTableRow> rows;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String rowsLabel;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideRowLabels;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer decimalsToShow;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true)
	@XMLElement
	private List<FixedTableQueryInstance> instances;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideTitle;

	@Override
	public List<FixedTableColumn> getColumns() {

		return columns;
	}

	public List<FixedTableQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<FixedTableQueryInstance> instances) {

		this.instances = instances;
	}

	@Override
	public void setColumns(List<FixedTableColumn> alternatives) {

		this.columns = alternatives;
	}

	public boolean isHideTitle() {

		return hideTitle;
	}

	public void setHideTitle(boolean hideTitle) {

		this.hideTitle = hideTitle;
	}

	@Override
	public boolean hasRequiredColumn() {

		if (!CollectionUtils.isEmpty(columns)) {

			for (FixedTableColumn column : columns) {

				if (column.isRequired()) {

					return true;
				}
			}
		}

		return false;
	}

	public List<FixedTableRow> getRows() {

		return rows;
	}

	public void setRows(List<FixedTableRow> rows) {

		this.rows = rows;
	}

	public String getRowsLabel() {

		return rowsLabel;
	}

	public void setRowsLabel(String rowsLabel) {

		this.rowsLabel = rowsLabel;
	}

	@Override
	protected int getMinRowOccurance() {

		int minOccurs = 0;

		if (hasRequiredColumn()) {

			for (FixedTableRow row : rows) {

				if (row.isRequired()) {

					minOccurs++;
				}
			}
		}

		return minOccurs;
	}

	@Override
	protected int getMaxRowOccurance() {

		return rows.size();
	}

	@Override
	protected void populateColumns(XMLParser xmlParser, List<ValidationError> errors) throws ValidationException {

		hideRowLabels = xmlParser.getPrimitiveBoolean("hideRowLabels");

		if (!hideRowLabels) {

			rowsLabel = XMLValidationUtils.validateParameter("rowsLabel", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		}

		List<XMLParser> xmlParsers = xmlParser.getNodes("Columns/Column");

		if (xmlParsers != null) {

			columns = new ArrayList<>();

			for (XMLParser parser : xmlParsers) {

				FixedTableColumn column = new FixedTableColumn();

				column.populate(parser);

				column.setQuery(this);

				columns.add(column);
			}
		}

		xmlParsers = xmlParser.getNodes("Rows/Row");

		hideTitle = xmlParser.getPrimitiveBoolean("hideTitle");

		if (xmlParsers != null) {

			rows = new ArrayList<>();

			for (XMLParser parser : xmlParsers) {

				FixedTableRow row = new FixedTableRow();

				row.populate(parser);

				row.setQuery(this);

				rows.add(row);
			}
		}
	}

	public boolean isHideRowLabels() {

		return hideRowLabels;
	}

	public void setHideRowLabels(boolean hideRowsLabels) {

		this.hideRowLabels = hideRowsLabels;
	}

	@Override
	public Integer getDecimalsToShow() {

		return decimalsToShow;
	}

}
