package com.nordicpeak.flowengine.flowinstanceadmin2;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.naming.OperationNotSupportedException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.cron4jutils.CronStringValidator;
import se.unlogic.fileuploadutils.MultipartRequest;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EnumDropDownSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.Group;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.events.EventHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.HierarchyAnnotatedDAOFactory;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLinkUtils;
import se.unlogic.hierarchy.foregroundmodules.usersessionadmin.UserNameComparator;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.bool.BooleanUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.LowLevelQuery;
import se.unlogic.standardutils.dao.OrderByCriteria;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.enums.EnumUtils;
import se.unlogic.standardutils.enums.Order;
import se.unlogic.standardutils.io.BinarySizeFormater;
import se.unlogic.standardutils.io.BinarySizes;
import se.unlogic.standardutils.io.CloseUtils;
import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.PositiveStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.MultiElementableListener;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.ResponseUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.MessageHandler.ExternalMessageHandlerInstance;
import com.nordicpeak.flowengine.MessageHandler.InternalMessageHandlerInstance;
import com.nordicpeak.flowengine.beans.ExternalMessage;
import com.nordicpeak.flowengine.beans.ExternalMessageReadReceipt;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.FlowOverviewAttribute;
import com.nordicpeak.flowengine.beans.InternalMessage;
import com.nordicpeak.flowengine.beans.MessageTemplate;
import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.cache.FlowCacheAccess;
import com.nordicpeak.flowengine.comparators.GroupNameComparator;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.enums.Priority;
import com.nordicpeak.flowengine.events.ManagersChangedEvent;
import com.nordicpeak.flowengine.events.StatusChangedByManagerEvent;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.exceptions.flow.FlowDisabledException;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.FlowInstanceManagerClosedException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToGetQueryInstanceShowHTMLException;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.DateInterval;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.FlowInstanceAdminUserSettings;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.FormSubmitProgressTracker;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.TableSettings;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.TableSettingsOrderEntry;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.UserManagableFlowInstanceCounts;
import com.nordicpeak.flowengine.flowinstanceadmin2.enums.InstanceFilter;
import com.nordicpeak.flowengine.flowinstanceadmin2.enums.ManagerDisplayMode;
import com.nordicpeak.flowengine.flowinstanceadmin2.enums.OpenMode;
import com.nordicpeak.flowengine.flowinstanceadmin2.enums.TableSettingColumnType;
import com.nordicpeak.flowengine.flowinstanceadmin2.flexjson.TableSettingColumnTypeObjectFactory;
import com.nordicpeak.flowengine.flowinstanceadmin2.flexjson.TableSettingColumnTypeTransformer;
import com.nordicpeak.flowengine.flowinstanceadmin2.flexjson.TableSettingOrderEntryObjectFactory;
import com.nordicpeak.flowengine.flowinstanceadmin2.flexjson.TableSettingOrderEntryTransformer;
import com.nordicpeak.flowengine.flowinstanceadmin2.interfaces.ColoredTagProvider;
import com.nordicpeak.flowengine.flowinstanceadmin2.interfaces.ListOverviewButtonExtension;
import com.nordicpeak.flowengine.flowinstanceadmin2.interfaces.ListOverviewButtonExtensionProvider;
import com.nordicpeak.flowengine.flowinstanceadmin2.interfaces.ManagableFlowInstanceProvider;
import com.nordicpeak.flowengine.flowinstanceadmin2.populators.FlowInstanceChainedPopulator;
import com.nordicpeak.flowengine.interfaces.ExternalMessageExtensionProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceOverviewDescriptionExtensionProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceOverviewHeaderExtensionProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceOverviewHistoryExtensionProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceUpdateStatusFormExtensionProvider;
import com.nordicpeak.flowengine.interfaces.FlowNotificationHandler;
import com.nordicpeak.flowengine.listeners.ExternalMessageExtensionElementableListener;
import com.nordicpeak.flowengine.listeners.FlowInstanceExternalMessageElementableListener;
import com.nordicpeak.flowengine.listeners.FlowInstanceStatusMessageElementableListener;
import com.nordicpeak.flowengine.listeners.FlowStatusManagerAccessElementableListener;
import com.nordicpeak.flowengine.listeners.MessageTemplateTagReplaceElementableListener;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;
import com.nordicpeak.flowengine.validationerrors.UnauthorizedUserNotManagerValidationError;

import flexjson.JSONDeserializer;
import flexjson.JSONSerializer;
import flexjson.factories.BooleanObjectFactory;
import it.sauronsoftware.cron4j.Scheduler;

public class FlowInstanceAdmin2Module extends FlowInstanceAdminModule {

	protected static final Field[] FLOW_INSTANCE_COUNT_RELATIONS = { FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION };
	protected static final Field[] FLOW_INSTANCE_LIST_RELATIONS = { FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION, FlowInstance.STATUS_RELATION, FlowInstance.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.FLOW_FAMILY_RELATION, FlowInstance.ATTRIBUTES_RELATION };
	protected static final Field[] FLOW_INSTANCE_LIST_CACHED_RELATIONS = { FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION, FlowInstance.STATUS_RELATION, FlowInstance.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.FLOW_FAMILY_RELATION };
	protected static final List<Field> FLOW_INSTANCE_SELECTION_REQUIRED_RELATIONS = Arrays.asList(FlowInstance.STATUS_RELATION, FlowInstance.FLOW_RELATION, Flow.FLOW_FAMILY_RELATION, FlowFamily.MANAGER_USERS_RELATION, FlowFamily.MANAGER_GROUPS_RELATION, FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION, FlowInstance.ATTRIBUTES_RELATION);

	protected static final Field[] SET_COLORED_TAG_ACCESS_RELATIONS = { FlowInstance.FLOW_RELATION, FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION, Flow.FLOW_FAMILY_RELATION, FlowFamily.MANAGER_GROUPS_RELATION, FlowFamily.MANAGER_USERS_RELATION, };

	private static final String ORGANIZATION_NUMBER = "organizationNumber";

	//TODO only do distinct and group by when user has restricted groups
	protected final String SELECT_FLOWINSTANCES_WITH_MESSAGES_SQL = "SELECT flowInstance.*, (SELECT externalMessage.messageID FROM flowengine_external_messages externalMessage WHERE externalMessage.flowInstanceID = flowInstance.flowInstanceID limit 1) as externalMessageID, (SELECT internalMessage.messageID FROM flowengine_internal_messages internalMessage WHERE internalMessage.flowInstanceID = flowInstance.flowInstanceID limit 1) as internalMessageID ";
	protected final String SELECT_FLOWINSTANCES_SQL = "SELECT flowInstance.* ";
	protected final String GROUP_FLOWINSTANCES_SQL = " GROUP BY flowInstance.flowInstanceID";
	protected final String ORDER_FLOWINSTANCES_SQL = " ORDER BY lastStatusChange DESC";

	@XSLVariable(prefix = "java.")
	protected String anonymousSubmitter = "Anonymous Submitter";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Default flow instance list page size", description = "", formatValidator = PositiveStringIntegerValidator.class, required = true)
	protected Integer defaultPageSize = 100;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Show history tab as default", description = "Use history tab as default tab when viewing flow instances")
	protected boolean useHistoryTabAsDefaultTab = false;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Show verification modal when submitting external message with attached files", description = "Show verification modal when submitting external message with possibility to verify, download and delete attached files")
	protected boolean externalMessageVerifyAttachedFilesModal = false;

	@ModuleSetting
	@EnumDropDownSettingDescriptor(name = "Default DateInterval", description = "", required = true)
	protected DateInterval defaultDateInterval = DateInterval.ALL;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Interval for expired flow instance notification check", description = "How often this module should check for expired flow instances to notify about (specified in crontab format)", required = true, formatValidator = CronStringValidator.class)
	private String expiredFlowInstanceNotificationCheckerInterval = "0 * * * *";

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Hide flow instance history icon column", description = "Hides the icon column (e.g. get pdf) in the history tab of the flow instance overview page. Also prevents requests to the links.")
	protected boolean hideFlowInstanceHistoryIconColumn = false;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Always show managergroups", description = "Always shows managergroups when selecting to add managers")
	protected boolean alwaysListManagerGroups = false;

	protected AnnotatedDAO<FlowInstanceAdminUserSettings> userSettingsAnnotatedDAO;

	protected QueryParameterFactory<Status, Integer> statusIDParamFactory;
	protected QueryParameterFactory<Status, Flow> statusFlowParamFactory;
	protected QueryParameterFactory<FlowInstanceAdminUserSettings, User> userSettingUserParamFactory;
	protected QueryParameterFactory<FlowFamily, Integer> flowFamilyIDParamFactory;

	protected OrderByCriteria<FlowInstance> flowInstanceOrderByLastStatusChangeCriteria;

	protected CopyOnWriteArrayList<ManagableFlowInstanceProvider> adminFlowInstanceProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<ListOverviewButtonExtensionProvider> listOverviewButtonExtensionLinkProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<FlowInstanceOverviewHeaderExtensionProvider> flowInstanceOverviewHeaderExtensionProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<FlowInstanceOverviewHistoryExtensionProvider> flowInstanceOverviewHistoryExtensionProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<FlowInstanceOverviewDescriptionExtensionProvider> flowInstanceOverviewDescriptionExtensionProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<FlowInstanceUpdateStatusFormExtensionProvider> flowInstanceUpdateStatusFormExtensionProviders = new CopyOnWriteArrayList<>();
	protected CopyOnWriteArrayList<ExternalMessageExtensionProvider> externalMessageExtensionProviders = new CopyOnWriteArrayList<>();

	@InstanceManagerDependency(required = false)
	protected ColoredTagProvider coloredTagProvider;

	private Scheduler scheduler;
	private String expiredFlowInstanceNotificationCheckerScheduleID;

	@InstanceManagerDependency
	protected FlowNotificationHandler flowNotificationHandler;

	@Override
	protected void registerInstance(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.registerInstance(moduleDescriptor, sectionInterface, dataSource);

		if (!systemInterface.getInstanceHandler().addInstance(FlowInstanceAdmin2Module.class, this)) {

			throw new RuntimeException("Unable to register module in global instance handler using key " + FlowInstanceAdmin2Module.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		statusIDParamFactory = daoFactory.getStatusDAO().getParamFactory("statusID", Integer.class);
		statusFlowParamFactory = daoFactory.getStatusDAO().getParamFactory("flow", Flow.class);
		flowFamilyIDParamFactory = daoFactory.getFlowFamilyDAO().getParamFactory("flowFamilyID", Integer.class);

		flowInstanceOrderByLastStatusChangeCriteria = daoFactory.getFlowInstanceDAO().getOrderByCriteria("lastStatusChange", Order.DESC);

		// Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, FlowInstanceAdmin2Module.class.getName(), new XMLDBScriptProvider(FlowInstanceAdmin2Module.class.getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		HierarchyAnnotatedDAOFactory normalDAOFactory = new HierarchyAnnotatedDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler(), false, false, false);

		userSettingsAnnotatedDAO = normalDAOFactory.getDAO(FlowInstanceAdminUserSettings.class);

		userSettingUserParamFactory = userSettingsAnnotatedDAO.getParamFactory("user", User.class);
	}

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		initScheduler();
	}

	@Override
	public void update(ForegroundModuleDescriptor descriptor, DataSource dataSource) throws Exception {

		super.update(descriptor, dataSource);

		scheduler.reschedule(expiredFlowInstanceNotificationCheckerScheduleID, expiredFlowInstanceNotificationCheckerInterval);
	}

	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(FlowInstanceAdmin2Module.class, this);

		flowInstanceOverviewHeaderExtensionProviders.clear();
		flowInstanceOverviewHistoryExtensionProviders.clear();
		flowInstanceOverviewDescriptionExtensionProviders.clear();
		adminFlowInstanceProviders.clear();

		flowInstanceUpdateStatusFormExtensionProviders.clear();
		externalMessageExtensionProviders.clear();

		stopScheduler();

		super.unload();
	}

	protected synchronized void initScheduler() {

		if (scheduler != null) {

			stopScheduler();
		}

		scheduler = new Scheduler(systemInterface.getApplicationName() + " - " + moduleDescriptor.toString());
		scheduler.setDaemon(true);

		expiredFlowInstanceNotificationCheckerScheduleID = scheduler.schedule(expiredFlowInstanceNotificationCheckerInterval, new ExpiredFlowInstanceNotificationChecker(this));

		scheduler.start();
	}

	protected synchronized void stopScheduler() {

		try {

			if (scheduler != null) {

				scheduler.stop();
				scheduler = null;
			}

		} catch (IllegalStateException e) {

			log.error("Error stopping scheduler", e);
		}
	}

	@Override
	protected List<String> getSelectedAttributes() {

		ArrayList<String> attributes = new ArrayList<>();

		attributes.add(Constants.FLOW_INSTANCE_FIRSTNAME_ATTRIBUTE);
		attributes.add(Constants.FLOW_INSTANCE_LASTNAME_ATTRIBUTE);
		attributes.add(Constants.FLOW_INSTANCE_ORGANIZATION_NAME_ATTRIBUTE);

		if (enableExternalID) {

			attributes.add(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE);
		}

		return attributes;
	}

	@WebPublic(alias = "users")
	public ForegroundModuleResponse getUsers(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return userGroupListFlowManagersConnector.getUsers(req, res, user, uriParser);
	}

	@WebPublic(alias = "groups")
	public ForegroundModuleResponse getGroups(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return userGroupListFlowManagersConnector.getGroups(req, res, user, uriParser);
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws SQLException {

		log.info("User " + user + " listing flow instances");

		SiteProfile profile = null;

		if (profileHandler != null) {

			profile = profileHandler.getCurrentProfile(user, req, uriParser);
		}

		FlowCacheAccess flowCacheAccess = getManagedFlowFamilies(user);

		int unassignedInstanceCount = getUnassignedFlowInstanceCount(user, flowCacheAccess, profile);
		int prioritizedInstanceCount = getPrioritizedFlowInstanceCount(user, flowCacheAccess, profile);
		int userAssignedInstanceCount = getMyFlowInstanceCount(user, flowCacheAccess, profile);
		int bookmarkedInstanceCount = getBookmaredFlowInstanceCount(user, flowCacheAccess, profile);
		int activeInstanceCount = getActiveFlowInstanceCount(user, flowCacheAccess, profile);

		if (!adminFlowInstanceProviders.isEmpty()) {

			for (ManagableFlowInstanceProvider adminFlowInstanceProvider : adminFlowInstanceProviders) {

				try {
					UserManagableFlowInstanceCounts externalCounts = adminFlowInstanceProvider.getFlowInstanceCounts(user);

					if (externalCounts != null) {

						unassignedInstanceCount += externalCounts.getUnassignedInstanceCount();
						prioritizedInstanceCount += externalCounts.getPrioritizedInstanceCount();
						userAssignedInstanceCount += externalCounts.getUserAssignedInstanceCount();
						//						bookmarkedInstanceCount += externalCounts.getBookmarkedInstanceCount();
						activeInstanceCount += externalCounts.getActiveInstanceCount();
					}

				} catch (RuntimeException e) {

					log.error("Error getting flow instance counts from provider " + adminFlowInstanceProvider, e);
				}
			}
		}

		Document doc = createDocument(req, uriParser, user);

		XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "fullAlias", getFullAlias());

		Element overviewElement = doc.createElement("OverviewElement");
		doc.getDocumentElement().appendChild(overviewElement);

		XMLUtils.append(doc, overviewElement, validationErrors);

		if (useHistoryTabAsDefaultTab) {
			XMLUtils.appendNewElement(doc, overviewElement, "UseHistoryTabAsDefaultTab");
		}

		XMLUtils.appendNewElement(doc, overviewElement, "UnassignedInstanceCount", unassignedInstanceCount);
		XMLUtils.appendNewElement(doc, overviewElement, "PrioritizedInstanceCount", prioritizedInstanceCount);
		XMLUtils.appendNewElement(doc, overviewElement, "UserAssignedInstanceCount", userAssignedInstanceCount);
		XMLUtils.appendNewElement(doc, overviewElement, "BookmarkedInstanceCount", bookmarkedInstanceCount);
		XMLUtils.appendNewElement(doc, overviewElement, "ActiveInstanceCount", activeInstanceCount);

		XMLUtils.appendNewElement(doc, overviewElement, "PageSize", defaultPageSize);

		if (enableDescriptionColumn) {
			XMLUtils.appendNewElement(doc, overviewElement, "EnableDescriptionColumn");
		}

		if (enableExternalID) {
			XMLUtils.appendNewElement(doc, overviewElement, "EnableExternalIDColumn");
		}

		if (enableSiteProfileSupport && this.profileHandler != null) {

			XMLUtils.append(doc, overviewElement, "SiteProfiles", this.profileHandler.getProfiles());
		}

		if (!adminFlowInstanceProviders.isEmpty()) {

			XMLUtils.appendNewElement(doc, overviewElement, "AdminFlowInstanceProviders");
		}

		if (coloredTagProvider != null) {

			coloredTagProvider.appendListElement(doc, overviewElement, user);
		}

		FlowInstanceAdminUserSettings userSettings = getOrCreateUserSettings(user);

		if (userSettings.getDateInterval() == null) {

			userSettings.setDateInterval(defaultDateInterval);
		}

		XMLUtils.append(doc, overviewElement, userSettings);

		ExtensionLinkUtils.appendExtensionLinks(listOverviewExtensionLinkProviders, user, req, doc, overviewElement);

		List<ScriptTag> scripts = null;

		if (!listOverviewButtonExtensionLinkProviders.isEmpty()) {

			scripts = new ArrayList<>();

			Element listButtonExtensionsElement = XMLUtils.appendNewElement(doc, overviewElement, "ListButtonExtensions");

			for (ListOverviewButtonExtensionProvider extensionProvider : listOverviewButtonExtensionLinkProviders) {

				if (extensionProvider.getAccessInterface() == null || AccessUtils.checkAccess(user, extensionProvider.getAccessInterface())) {

					try {
						ListOverviewButtonExtension extension = extensionProvider.getListOverviewButtonExtension(req, user);

						if (extension != null) {

							listButtonExtensionsElement.appendChild(extension.toXML(doc));

							if (extension.getScripts() != null) {

								scripts.addAll(extension.getScripts());
							}
						}

					} catch (Exception e) {
						log.error("Error appending overview button extension from ListOverviewButtonExtensionProvider " + extensionProvider, e);
					}
				}
			}
		}

		SimpleForegroundModuleResponse moduleResponse = new SimpleForegroundModuleResponse(doc, moduleDescriptor.getName(), this.getDefaultBreadcrumb());

		if (!CollectionUtils.isEmpty(scripts)) {
			moduleResponse.addScripts(scripts);
		}

		return moduleResponse;
	}

	@WebPublic(alias = "instancedata")
	public ForegroundModuleResponse getFlowInstanceData(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());

		Writer writer = ResponseUtils.getCompressedRequestWriter(req, res);

		try {
			InstanceFilter instanceFilter;

			if (uriParser.size() == 3 && !StringUtils.isEmpty(uriParser.get(2)) && (instanceFilter = EnumUtils.toEnum(InstanceFilter.class, uriParser.get(2).toUpperCase())) != null) {

				log.info("Sending flow instances with filter " + instanceFilter + " to user " + user);

				try {
					SiteProfile profile = null;

					if (profileHandler != null) {

						profile = profileHandler.getCurrentProfile(user, req, uriParser);
					}

					//					long start = System.currentTimeMillis();

					FlowCacheAccess flowCacheAccess = getManagedFlowFamilies(user);

					FlowInstanceAdminUserSettings userSettings = getOrCreateUserSettings(user);

					List<FlowInstance> flowInstances = null;

					if (instanceFilter == InstanceFilter.UNASSIGNED) {

						flowInstances = getUnassignedFlowInstances(user, flowCacheAccess, profile);

					} else if (instanceFilter == InstanceFilter.PRIORITIZED) {

						flowInstances = getPrioritizedFlowInstances(user, flowCacheAccess, profile);

					} else if (instanceFilter == InstanceFilter.MINE) {

						flowInstances = getMyFlowInstances(user, flowCacheAccess, profile);

					} else if (instanceFilter == InstanceFilter.BOOKMARKED) {

						flowInstances = getBookmaredFlowInstances(user, flowCacheAccess, profile);

					} else if (instanceFilter == InstanceFilter.ACTIVE) {

						flowInstances = getActiveFlowInstances(user, flowCacheAccess, profile);

					} else if (instanceFilter == InstanceFilter.ALL) {

						Date fromDate = userSettings.calculateFromDate(defaultDateInterval);
						Date toDate = userSettings.getToDate();

						flowInstances = getAllFlowInstances(user, flowCacheAccess, profile, fromDate, toDate);

					} else {
						log.error("Unknown InstanceFilter " + instanceFilter);
					}

					//					start = System.currentTimeMillis() - start;
					//					log.info("Get took " + TimeUtils.millisecondsToString(start));

					flowInstances = appendExternalFlowInstances(instanceFilter, flowInstances, user, profile);

					writer.append("{\"rows\":[");

					if (!CollectionUtils.isEmpty(flowInstances)) {

						if (instanceFilter == InstanceFilter.PRIORITIZED) {

							Date endDate = new Date();

							for (FlowInstance flowInstance : flowInstances) {

								int daysLapsed = DateUtils.getWorkingDays(flowInstance.getLastStatusChange(), endDate);
								float percent = daysLapsed / flowInstance.getStatus().getManagingTime() * 100;

								if (percent >= highPriorityThreshold) {

									flowInstance.setPriority(Priority.HIGH);

								} else if (percent >= mediumPriorityThreshold) {

									flowInstance.setPriority(Priority.MEDIUM);
								}
							}
						}

						Map<Integer, Integer> bookmarkedFlowInstanceMap = getUserBookmarkedFlowInstanceIDsMap(user);

						JsonArray row = new JsonArray(15);
						StringBuilder eventsBuilder = new StringBuilder();
						StringBuilder managersBuilder = new StringBuilder();
						StringBuilder searchExtraBuilder = new StringBuilder();

						for (Iterator<FlowInstance> instanceIterator = flowInstances.iterator(); instanceIterator.hasNext();) {

							FlowInstance flowInstance = instanceIterator.next();
							Flow flow = flowInstance.getFlow();

							row.addNode(flowInstance.getFlowInstanceID());
							row.addNode(flow.getFlowID());
							row.addNode(flow.getFlowFamily().getFlowFamilyID());
							row.addNode(flowInstance.getStatus().getName());
							row.addNode(flow.getName());
							row.addNode(flow.getVersion());
							row.addNode(flow.getFlowType().getName());
							row.addNode(flowInstance.getFirstSubmitted() != null ? DateUtils.DATE_TIME_FORMATTER.format(flowInstance.getFirstSubmitted()) : "");
							row.addNode(FlowInstanceUtils.getSubmitterName(flowInstance, anonymousSubmitter));

							boolean shouldGetManagerUsernames = userSettings != null && userSettings.getManagerDisplayMode() == ManagerDisplayMode.USERNAME;

							getManagersString(managersBuilder, flowInstance.getManagers(), flowInstance.getManagerGroups(), !shouldGetManagerUsernames, shouldGetManagerUsernames);

							row.addNode(managersBuilder.toString());

							if (bookmarkedFlowInstanceMap != null && bookmarkedFlowInstanceMap.get(flowInstance.getFlowInstanceID()) != null) {
								eventsBuilder.append("b");
							}

							if (flowInstance.getExternalMessages() != null) {
								eventsBuilder.append("m");
							}

							if (flowInstance.getInternalMessages() != null) {
								eventsBuilder.append("i");
							}

							row.addNode(eventsBuilder.toString());

							if (enableSiteProfileSupport) {
								row.addNode(flowInstance.getProfileID());
							}

							AttributeHandler attributeHandler = flowInstance.getAttributeHandler();

							if (enableDescriptionColumn) {

								if (flowInstance.getManagerDescription() != null) {

									row.addNode(flowInstance.getManagerDescription());

								} else {

									row.addNode("");
								}
							}

							if (enableExternalID) {

								if (attributeHandler.isSet(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE)) {

									row.addNode(attributeHandler.getString(Constants.FLOW_INSTANCE_EXTERNAL_ID_ATTRIBUTE));

								} else {

									row.addNode("");
								}
							}

							String citizenIdentifier = attributeHandler.getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE);

							if (citizenIdentifier == null && flowInstance.getPoster() != null) {

								citizenIdentifier = flowInstance.getPoster().getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE);
							}

							if (citizenIdentifier != null) {

								searchExtraBuilder.append(citizenIdentifier);
								searchExtraBuilder.append(" ");
							}

							if (flowInstance.getOwners() != null) {

								for (User owner : flowInstance.getOwners()) {

									if (owner.getAttributeHandler().isSet(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE)) {

										searchExtraBuilder.append(owner.getAttributeHandler().getString(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE));
										searchExtraBuilder.append(" ");
									}
								}
							}

							String organizationNumber = attributeHandler.getString(ORGANIZATION_NUMBER);

							if (organizationNumber != null) {

								searchExtraBuilder.append(organizationNumber);
								searchExtraBuilder.append(" ");
							}

							row.addNode(searchExtraBuilder.toString());

							if (!adminFlowInstanceProviders.isEmpty()) {

								if (attributeHandler.isSet("overviewURL")) {

									row.addNode(attributeHandler.getString("overviewURL"));

								} else {

									row.addNode("");
								}
							}

							if (coloredTagProvider != null) {

								row.addNode(coloredTagProvider.getFlowInstanceTableCellData(flowInstance));
							}

							writer.append(row.toJson());

							row.clearNodes();
							searchExtraBuilder.setLength(0);
							eventsBuilder.setLength(0);
							managersBuilder.setLength(0);

							if (instanceIterator.hasNext()) {

								writer.append(",");
							}
						}
					}

					writer.append("]}");

				} catch (IOException e) {

					log.warn("Error sending flow instance data to " + user, e);

				} catch (Exception e) {

					log.error("Error sending flow instance data to " + user, e);
					writer.append("{\"rows\": [], \"error\": \"Internal error\"}");
				}

			} else {

				writer.append("{\"rows\": [], \"error\": \"No parameter\"}");
			}

		} catch (IOException e) {

			log.warn("Error sending flow instance data to " + user);

		} finally {

			CloseUtils.close(writer);
		}

		return null;
	}

	protected StringBuilder getManagersString(StringBuilder stringBuilder, List<User> managers, List<Group> managerGroups, boolean names, boolean usernames) {

		return FlowInstanceUtils.getManagersString(stringBuilder, managers, managerGroups, names, usernames);
	}

	protected List<FlowInstance> appendExternalFlowInstances(InstanceFilter instanceFilter, List<FlowInstance> flowInstances, User user, SiteProfile profile) {

		if (adminFlowInstanceProviders.isEmpty() || instanceFilter == InstanceFilter.BOOKMARKED) {

			return flowInstances;
		}

		List<FlowInstance> externalFlowInstances = null;

		for (ManagableFlowInstanceProvider adminFlowInstanceProvider : adminFlowInstanceProviders) {

			try {
				externalFlowInstances = CollectionUtils.addAndInstantiateIfNeeded(externalFlowInstances, adminFlowInstanceProvider.getFlowInstances(instanceFilter, user));

			} catch (RuntimeException e) {

				log.error("Error getting flow instances from provider " + adminFlowInstanceProvider, e);
			}
		}

		if (CollectionUtils.isEmpty(externalFlowInstances)) {
			return flowInstances;
		}

		if (flowInstances == null) {
			return externalFlowInstances;
		}

		return CollectionUtils.combine(flowInstances, externalFlowInstances);
	}

	public int getUnassignedFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLUnassignedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public int getPrioritizedFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLPrioritizedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public int getMyFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLMyFlowInstances(flowCacheAccess, user, profile), user);
	}

	public int getBookmaredFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLBookmarkedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public int getActiveFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLActiveFlowInstances(flowCacheAccess, user, profile), user);
	}

	protected int getAllFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstanceCount(getSQLAllFlowInstances(flowCacheAccess, user, profile, null, null), user);
	}

	protected int getAllFlowInstanceCount(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile, Date fromDate, Date toDate) throws SQLException {

		return getFlowInstanceCount(getSQLAllFlowInstances(flowCacheAccess, user, profile, fromDate, toDate), user);
	}

	protected int getFlowInstanceCount(String sql, User user) throws SQLException {

		if (sql == null) {
			return 0;
		}

		LowLevelQuery<FlowInstance> query = new LowLevelQuery<>(SELECT_FLOWINSTANCES_SQL + sql + GROUP_FLOWINSTANCES_SQL);
		query.addRelations(FLOW_INSTANCE_COUNT_RELATIONS);

		addFlowInstanceFilterRelations(query);

		List<FlowInstance> flowInstances = daoFactory.getFlowInstanceDAO().getAll(query);

		filterFlowInstances(flowInstances, user);

		return flowInstances != null ? flowInstances.size() : 0;
	}

	public List<FlowInstance> getUnassignedFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstances(getSQLUnassignedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public List<FlowInstance> getPrioritizedFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstances(getSQLPrioritizedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public List<FlowInstance> getPrioritizedFlowInstances(Integer flowFamilyID) throws SQLException {

		return getFlowInstances(getSQLPrioritizedFlowInstances(flowFamilyID));
	}

	public List<FlowInstance> getMyFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstances(getSQLMyFlowInstances(flowCacheAccess, user, profile), user);
	}

	public List<FlowInstance> getBookmaredFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstances(getSQLBookmarkedFlowInstances(flowCacheAccess, user, profile), user);
	}

	public List<FlowInstance> getActiveFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getFlowInstances(getSQLActiveFlowInstances(flowCacheAccess, user, profile), user);
	}

	public List<FlowInstance> getAllFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile) throws SQLException {

		return getAllFlowInstances(user, flowCacheAccess, profile, null, null);
	}

	public List<FlowInstance> getAllFlowInstances(User user, FlowCacheAccess flowCacheAccess, SiteProfile profile, Date fromDate, Date toDate) throws SQLException {

		return getFlowInstances(getSQLAllFlowInstances(flowCacheAccess, user, profile, fromDate, toDate), user);
	}

	protected List<FlowInstance> getFlowInstances(String sql) throws SQLException {

		return getFlowInstances(sql, null, false);
	}

	protected List<FlowInstance> getFlowInstances(String sql, User user) throws SQLException {

		return getFlowInstances(sql, user, true);
	}

	protected List<FlowInstance> getFlowInstances(String sql, User user, boolean filterFlowInstances) throws SQLException {

		if (sql == null) {
			return null;
		}

		LowLevelQuery<FlowInstance> query = new LowLevelQuery<>(SELECT_FLOWINSTANCES_WITH_MESSAGES_SQL + sql + GROUP_FLOWINSTANCES_SQL + ORDER_FLOWINSTANCES_SQL);
		query.addRelations(FLOW_INSTANCE_LIST_RELATIONS);
		query.addCachedRelations(FLOW_INSTANCE_LIST_CACHED_RELATIONS);
		query.addChainedResultSetPopulator(FlowInstanceChainedPopulator.getPopulator());

		if (filterFlowInstances) {

			addFlowInstanceFilterRelations(query);
		}

		List<FlowInstance> flowInstances = daoFactory.getFlowInstanceDAO().getAll(query);

		if (filterFlowInstances) {

			filterFlowInstances(flowInstances, user);
		}

		return flowInstances;
	}

	private Map<Integer, Integer> getUserBookmarkedFlowInstanceIDsMap(User user) throws SQLException {

		List<Integer> bookmarkedFlowInstanceIDs = getUserBookmarkedFlowInstanceIDs(user);

		if (!CollectionUtils.isEmpty(bookmarkedFlowInstanceIDs)) {

			Map<Integer, Integer> bookmarkedFlowInstanceMap = new HashMap<>();

			for (Integer id : bookmarkedFlowInstanceIDs) {
				bookmarkedFlowInstanceMap.put(id, id);
			}

			return bookmarkedFlowInstanceMap;
		}

		return null;
	}

	protected List<Integer> getUserBookmarkedFlowInstanceIDs(User user) throws SQLException {

		ArrayListQuery<Integer> query = new ArrayListQuery<>(dataSource, "SELECT flowInstanceID FROM flowengine_flow_instance_bookmarks WHERE userID = ?", IntegerPopulator.getPopulator());
		query.setInt(1, user.getUserID());

		return query.executeQuery();
	}

	protected String getSQLUnassignedFlowInstances(FlowCacheAccess access, User user, SiteProfile profile) {

		boolean restrictedCheckUser = false;
		boolean restrictedCheckGroups = true;

		if (!hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");
		sql.append(" LEFT JOIN flowengine_flow_instance_managers flowInstanceManager ON flowInstance.flowInstanceID = flowInstanceManager.flowInstanceID");

		appendFlowInstanceListRestrictedSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" WHERE flow.hideFromManager = false AND flowInstance.firstSubmitted IS NOT NULL AND flowStatus.contentType NOT IN ('NEW', 'ARCHIVED')");

		appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" AND flowInstanceManager.userID IS NULL");

		return sql.toString();
	}

	protected String getSQLPrioritizedFlowInstances(FlowCacheAccess access, User user, SiteProfile profile) {

		return getSQLPrioritizedFlowInstances(access, user, profile, null);
	}

	protected String getSQLPrioritizedFlowInstances(Integer flowFamilyID) {

		return getSQLPrioritizedFlowInstances(null, null, null, flowFamilyID);
	}

	protected String getSQLPrioritizedFlowInstances(FlowCacheAccess access, User user, SiteProfile profile, Integer flowFamilyID) {

		boolean restrictedCheckUser = true;
		boolean restrictedCheckGroups = true;

		if (access != null && !hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");

		if (access != null) {

			appendFlowInstanceListRestrictedSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);
		}

		sql.append(" WHERE flow.hideFromManager = false AND flowInstance.firstSubmitted IS NOT NULL AND flowInstance.lastStatusChange IS NOT NULL ");
		sql.append(" AND flowStatus.managingTime IS NOT NULL AND flowStatus.contentType NOT IN ('NEW', 'ARCHIVED')");

		if (access != null) {

			appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);
		}

		/*
		 * Passed working days between dates https://stackoverflow.com/questions/1828948/mysql-function-to-find-the-number-of-working-days-between-two-dates | M
		 * T W T F S S -|-------------- M| 0 1 2 3 4 5 5 T| 4 0 1 2 3 4 4 W| 3 4 0 1 2 3 3 T| 2 3 4 0 1 2 2 F| 1 2 3 4 0 1 1 S| 0 1 2 3 4 0 0 S| 0 1 2 3 4 5 0
		 */
		sql.append(" AND 100 * (5 * (DATEDIFF(NOW(), flowInstance.lastStatusChange) DIV 7) + SUBSTRING('0123455401234434012332340122123401101234000123450', 1 + 7 * WEEKDAY(flowInstance.lastStatusChange) + WEEKDAY(NOW()), 1)) / flowStatus.managingTime > ");
		sql.append(highPriorityThreshold);

		if (flowFamilyID != null) {

			sql.append(" AND flow.flowFamilyID = " + flowFamilyID);
		}

		return sql.toString();
	}

	protected String getSQLMyFlowInstances(FlowCacheAccess access, User user, SiteProfile profile) {

		boolean restrictedCheckUser = true;
		boolean restrictedCheckGroups = false;

		if (!hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");
		sql.append(" LEFT JOIN flowengine_flow_instance_managers flowInstanceManager ON flowInstance.flowInstanceID = flowInstanceManager.flowInstanceID AND flowInstanceManager.userID = ");
		sql.append(user.getUserID());

		sql.append(" WHERE flow.hideFromManager = false AND flowInstance.firstSubmitted IS NOT NULL AND flowStatus.contentType NOT IN ('NEW', 'ARCHIVED')");

		appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" AND flowInstanceManager.userID IS NOT NULL");

		return sql.toString();
	}

	protected String getSQLBookmarkedFlowInstances(FlowCacheAccess access, User user, SiteProfile profile) {

		boolean restrictedCheckUser = true;
		boolean restrictedCheckGroups = true;

		if (!hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");
		sql.append(" INNER JOIN flowengine_flow_instance_bookmarks flowInstanceBookmarks ON flowInstance.flowInstanceID = flowInstanceBookmarks.flowInstanceID");

		appendFlowInstanceListRestrictedSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" WHERE flow.hideFromManager = false AND flowInstance.firstSubmitted IS NOT NULL AND flowStatus.contentType NOT IN ('NEW')");

		appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" AND flowInstanceBookmarks.userID = ");
		sql.append(user.getUserID());

		return sql.toString();
	}

	protected String getSQLActiveFlowInstances(FlowCacheAccess access, User user, SiteProfile profile) {

		boolean restrictedCheckUser = false;
		boolean restrictedCheckGroups = true;

		if (!hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");
		sql.append(" LEFT JOIN flowengine_flow_instance_managers flowInstanceManager ON flowInstance.flowInstanceID = flowInstanceManager.flowInstanceID AND flowInstanceManager.userID = " + user.getUserID());

		appendFlowInstanceListRestrictedSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" WHERE flow.hideFromManager = false AND flowInstance.firstSubmitted IS NOT NULL AND flowStatus.contentType NOT IN ('NEW', 'ARCHIVED')");

		appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" AND flowInstanceManager.userID IS NULL AND EXISTS (SELECT NULL FROM flowengine_flow_instance_managers WHERE flowInstanceID = flowInstance.flowInstanceID AND userID != ");
		sql.append(user.getUserID());
		sql.append(")");

		return sql.toString();
	}

	protected String getSQLAllFlowInstances(FlowCacheAccess access, User user, SiteProfile profile, Date fromDate, Date toDate) {

		boolean restrictedCheckUser = true;
		boolean restrictedCheckGroups = true;

		if (!hasAnyFlowFamilyAccess(access, user, profile, restrictedCheckUser, restrictedCheckGroups)) {
			return null;
		}

		StringBuilder sql = new StringBuilder();

		sql.append("FROM flowengine_flow_instances flowInstance");
		sql.append(" INNER JOIN flowengine_flows flow ON flow.flowID = flowInstance.flowID");
		sql.append(" INNER JOIN flowengine_flow_statuses flowStatus ON flowStatus.statusID = flowInstance.statusID");

		appendFlowInstanceListRestrictedSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(" WHERE flowInstance.firstSubmitted IS NOT NULL AND flowStatus.contentType != 'NEW'");

		appendFlowInstanceDateIntervalSQL(sql, fromDate, toDate);

		appendFlowInstanceListAccessSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		return sql.toString();
	}

	protected void appendFlowInstanceListRestrictedSQL(StringBuilder sql, FlowCacheAccess access, User user, boolean restrictedCheckUser, boolean restrictedCheckGroups) {

		if (access.getFlowFamiliesRestrictedAccess() != null) {

			if (restrictedCheckUser) {
				sql.append(" LEFT JOIN flowengine_flow_instance_managers flowInstanceManager ON flowInstance.flowInstanceID = flowInstanceManager.flowInstanceID AND flowInstanceManager.userID = ");
				sql.append(user.getUserID());
			}

			if (restrictedCheckGroups) {
				sql.append(" LEFT JOIN flowengine_flow_instance_manager_groups flowInstanceManagerGroups ON flowInstance.flowInstanceID = flowInstanceManagerGroups.flowInstanceID");
			}
		}
	}

	protected void appendFlowInstanceDateIntervalSQL(StringBuilder sql, Date fromDate, Date toDate) {

		if (fromDate != null) {

			String fromDateString = DateUtils.DATE_TIME_SECONDS_FORMATTER.format(fromDate);

			sql.append(" AND flowInstance.firstSubmitted >= '" + fromDateString + "'");
		}

		if (toDate != null) {

			toDate = DateUtils.setTimeToMaximum(toDate);

			String toDateString = DateUtils.DATE_TIME_SECONDS_FORMATTER.format(toDate);

			sql.append(" AND flowInstance.firstSubmitted <= '" + toDateString + "'");
		}
	}

	protected void appendFlowInstanceListAccessSQL(StringBuilder sql, FlowCacheAccess access, User user, boolean restrictedCheckUser, boolean restrictedCheckGroups) {

		sql.append(" AND (");

		if (access.getFlowFamiliesFullAccess() != null) {

			sql.append("flow.flowFamilyID IN (");
			appendFlowFamilyIDs(sql, access.getFlowFamiliesFullAccess());
			sql.append(")");
		}

		if (restrictedCheckGroups && user.getGroups() == null) {
			restrictedCheckGroups = false;
		}

		if (access.getFlowFamiliesRestrictedAccess() != null && (restrictedCheckUser || restrictedCheckGroups)) {

			if (access.getFlowFamiliesFullAccess() != null) {
				sql.append(" OR ");
			}

			sql.append("(flow.flowFamilyID IN (");
			appendFlowFamilyIDs(sql, access.getFlowFamiliesRestrictedAccess());
			sql.append(")");

			sql.append(" AND (");

			if (restrictedCheckUser) {
				sql.append("flowInstanceManager.userID IS NOT NULL");
			}

			if (restrictedCheckGroups) {

				if (restrictedCheckUser) {
					sql.append(" OR ");
				}

				sql.append("flowInstanceManagerGroups.groupID IN (");
				appendGroupIDs(sql, (List<Group>) user.getGroups());
				sql.append(")");
			}

			sql.append("))");
		}

		appendFlowInstanceListCacheExtraSQL(sql, access, user, restrictedCheckUser, restrictedCheckGroups);

		sql.append(")");
	}

	protected void appendFlowFamilyIDs(StringBuilder sql, List<FlowFamily> flowFamilies) {

		for (int i = 0, size = flowFamilies.size(); i < size; i++) {
			if (i > 0) {
				sql.append(',');
			}

			sql.append(flowFamilies.get(i).getFlowFamilyID());
		}
	}

	public void appendGroupIDs(StringBuilder sql, List<Group> groups) {

		for (int i = 0, size = groups.size(); i < size; i++) {
			if (i > 0) {
				sql.append(',');
			}

			sql.append(groups.get(i).getGroupID());
		}
	}

	protected boolean hasAnyFlowFamilyAccess(FlowCacheAccess access, User user, SiteProfile profile, boolean restrictedCheckUser, boolean restrictedCheckGroups) {

		if (access.getFlowFamiliesFullAccess() != null) {
			return true;
		}

		if (access.getFlowFamiliesRestrictedAccess() != null && (restrictedCheckUser || restrictedCheckGroups && user.getGroups() != null)) {
			return true;
		}

		return false;
	}

	protected void appendFlowInstanceListCacheExtraSQL(StringBuilder sql, FlowCacheAccess access, User user, boolean restrictedCheckUser, boolean restrictedCheckGroups) {}

	protected void filterFlowInstances(List<FlowInstance> flowInstances, User user) {}

	protected void addFlowInstanceFilterRelations(RelationQuery query) {}

	@Override
	protected Element appendFlowInstanceOverviewElement(Document doc, Element showFlowInstanceOverviewElement, FlowInstance flowInstance, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, SimpleForegroundModuleResponse moduleResponse) {

		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);

		genDoc.addRootElementableListener(FlowInstance.class, new FlowInstanceExternalMessageElementableListener());
		genDoc.addFieldElementableListener(MessageTemplate.class, new MessageTemplateTagReplaceElementableListener(flowInstance, user, getCurrentSiteProfile(req, user, uriParser, flowInstance.getFlow().getFlowFamily())));
		genDoc.addFieldElementableListener(ExternalMessage.class, new ExternalMessageExtensionElementableListener(externalMessageExtensionProviders, flowInstance, req, user, uriParser, true));

		List<FlowOverviewAttribute> overviewAttributes = flowInstance.getFlow().getOverviewAttributes();

		if (overviewAttributes != null) {

			Element flowOverviewAttributesElement = XMLUtils.appendNewElement(doc, showFlowInstanceOverviewElement, "FlowOverviewAttributes");

			for (FlowOverviewAttribute overviewAttribute : overviewAttributes) {

				Element overviewAttributeElement = overviewAttribute.toXML(doc);
				XMLUtils.appendNewElement(doc, overviewAttributeElement, "formattedValue", AttributeTagUtils.replaceTags(overviewAttribute.getValue(), flowInstance.getAttributeHandler()));

				flowOverviewAttributesElement.appendChild(overviewAttributeElement);
			}
		}

		Element flowInstancePreviewElement = appendFlowInstancePreviewElement(genDoc, showFlowInstanceOverviewElement, flowInstance, req, user, uriParser, moduleResponse, getGeneralAccessController());

		if (externalMessageVerifyAttachedFilesModal) {

			XMLUtils.appendNewElement(doc, flowInstancePreviewElement, "ExternalMessageVerifyAttachedFilesModal");
		}

		if (hideFlowInstanceHistoryIconColumn) {

			XMLUtils.appendNewElement(doc, flowInstancePreviewElement, "HideFlowInstanceHistoryIconColumn");
		}

		return flowInstancePreviewElement;
	}

	@Override
	protected List<ViewFragment> appendOverviewData(Document doc, Element showFlowInstanceOverviewElement, FlowInstance flowInstance, HttpServletRequest req, User user, URIParser uriParser) throws SQLException {

		List<ViewFragment> viewFragments = super.appendOverviewData(doc, showFlowInstanceOverviewElement, flowInstance, req, user, uriParser);

		if (!flowInstanceOverviewHeaderExtensionProviders.isEmpty()) {

			Element headerExtensionElement = XMLUtils.appendNewElement(doc, showFlowInstanceOverviewElement, "HeaderExtensions");

			for (FlowInstanceOverviewHeaderExtensionProvider extensionProvider : flowInstanceOverviewHeaderExtensionProviders) {

				try {
					ViewFragment contentsFragment = extensionProvider.getFlowInstanceOverviewHeaderViewFragment(flowInstance, req, uriParser, user);

					if (contentsFragment != null) {

						headerExtensionElement.appendChild(contentsFragment.toXML(doc));
						viewFragments.add(contentsFragment);
					}

				} catch (Exception e) {
					log.error("Error appending header extension from FlowInstanceOverviewExtensionProvider " + extensionProvider, e);
				}
			}
		}

		if (!flowInstanceOverviewHistoryExtensionProviders.isEmpty()) {

			Element historyExtensionElement = XMLUtils.appendNewElement(doc, showFlowInstanceOverviewElement, "HistoryExtensions");

			for (FlowInstanceOverviewHistoryExtensionProvider extensionProvider : flowInstanceOverviewHistoryExtensionProviders) {

				try {
					ViewFragment contentsFragment = extensionProvider.getFlowInstanceOverviewHistoryViewFragment(flowInstance, req, uriParser, user);

					if (contentsFragment != null) {

						historyExtensionElement.appendChild(contentsFragment.toXML(doc));
						viewFragments.add(contentsFragment);
					}

				} catch (Exception e) {
					log.error("Error appending history extension from FlowInstanceOverviewExtensionProvider " + extensionProvider, e);
				}
			}
		}

		if (!flowInstanceOverviewDescriptionExtensionProviders.isEmpty()) {

			Element descriptionExtensionElement = XMLUtils.appendNewElement(doc, showFlowInstanceOverviewElement, "DescriptionExtensions");

			for (FlowInstanceOverviewDescriptionExtensionProvider extensionProvider : flowInstanceOverviewDescriptionExtensionProviders) {

				try {
					ViewFragment contentsFragment = extensionProvider.getFlowInstanceOverviewDescriptionViewFragment(flowInstance, req, uriParser, user);

					if (contentsFragment != null) {

						descriptionExtensionElement.appendChild(contentsFragment.toXML(doc));
						viewFragments.add(contentsFragment);
					}

				} catch (Exception e) {
					log.error("Error appending description extension from FlowInstanceOverviewExtensionProvider " + extensionProvider, e);
				}
			}
		}

		if (coloredTagProvider != null) {

			coloredTagProvider.appendOverviewElement(flowInstance, doc, showFlowInstanceOverviewElement);
		}

		return viewFragments;
	}

	@WebPublic(alias = "steal")
	public ForegroundModuleResponse stealFlowInstance(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException {

		FlowInstance flowInstance;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (flowInstance = getFlowInstance(Integer.parseInt(uriParser.get(2)), null, getUpdateManagerRelations())) != null && !flowInstance.getStatus().getContentType().equals(ContentType.NEW)) {

			try {

				stealFlowInstance(flowInstance, user);

				redirectToMethod(req, res, "/overview/" + flowInstance.getFlowInstanceID());
				return null;

			} catch (FlowDisabledException e) {

				return list(req, res, user, uriParser, FLOW_DISABLED_VALIDATION_ERROR);

			} catch (ValidationException e) {

				log.warn("User " + user + " tried to steal instance " + flowInstance + " but is not an available manager of that instance.");
			}
		}

		return list(req, res, user, uriParser, FLOW_INSTANCE_NOT_FOUND_VALIDATION_ERROR);
	}

	@WebPublic(alias = "stealmultiple")
	public ForegroundModuleResponse stealFlowInstances(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		if (req.getMethod().equalsIgnoreCase("POST")) {

			List<Integer> flowInstanceIDs = NumberUtils.toInt(req.getParameterValues("flowInstanceID"));
			List<FlowInstance> flowInstances;

			if (flowInstanceIDs == null || CollectionUtils.isEmpty(flowInstances = getFlowInstances(flowInstanceIDs, getUpdateManagerRelations()))) {

				redirectToDefaultMethod(req, res);
				return null;
			}

			for (FlowInstance flowInstance : flowInstances) {

				try {

					stealFlowInstance(flowInstance, user);

				} catch (AccessDeniedException e) {

					log.warn("User " + user + " access error while stealing instance " + flowInstance + ", " + e);

				} catch (FlowDisabledException e) {

					log.warn("User " + user + " tried to steal a disabled instance " + flowInstance + ".");

				} catch (ValidationException e) {

					log.warn("User " + user + " tried to steal instance " + flowInstance + " but is not an available manager of that instance.");
				}
			}
		}

		redirectToDefaultMethod(req, res);
		return null;
	}

	private void stealFlowInstance(FlowInstance flowInstance, User user) throws AccessDeniedException, SQLException, FlowDisabledException, ValidationException {

		getGeneralAccessController().checkFlowInstanceAccess(flowInstance, user);

		if (!flowInstance.getFlow().isEnabled() || isOperatingStatusDisabled(flowInstance, true)) {

			throw new FlowDisabledException(flowInstance.getFlow());
		}

		Collection<User> availableManagers = getAllowedManagers(flowInstance);

		if (availableManagers == null || !availableManagers.contains(user)) {

			throw new ValidationException(new UnauthorizedUserNotManagerValidationError(user));
		}

		List<User> previousManagers = flowInstance.getManagers();

		if (!CollectionUtils.isEmpty(previousManagers)) {

			String previousManagersString = FlowInstanceUtils.getManagersString(previousManagers, null);

			log.info("User " + user + " removing old managers from " + flowInstance + ". Old managers: " + previousManagersString);
		}

		flowInstance.setManagers(Arrays.asList(user));

		String newManagersString = FlowInstanceUtils.getManagersString(flowInstance.getManagers(), null);

		log.info("User " + user + " setting managers of instance " + flowInstance + " to " + newManagersString);

		RelationQuery relationQuery = new RelationQuery(FlowInstance.MANAGERS_RELATION);

		daoFactory.getFlowInstanceDAO().update(flowInstance, relationQuery);

		FlowInstanceEvent flowInstanceEvent = flowInstanceEventGenerator.addFlowInstanceEvent(flowInstance, EventType.MANAGERS_UPDATED, newManagersString, user);

		eventHandler.sendEvent(FlowInstance.class, new CRUDEvent<>(CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
		eventHandler.sendEvent(FlowInstance.class, new ManagersChangedEvent(flowInstance, flowInstanceEvent, getSiteProfile(flowInstance), previousManagers, flowInstance.getManagerGroups(), user), EventTarget.ALL);

	}

	protected List<FlowInstance> getFlowInstances(List<Integer> flowInstanceIDs, Field... relations) throws SQLException {

		return getFlowInstances(flowInstanceIDs, Arrays.asList(relations));
	}

	protected List<FlowInstance> getFlowInstances(List<Integer> flowInstanceIDs, List<Field> relations) throws SQLException {

		HighLevelQuery<FlowInstance> query = new HighLevelQuery<>();

		query.addParameter(flowInstanceIDParamFactory.getWhereInParameter(flowInstanceIDs));

		query.addOrderByCriteria(flowInstanceOrderByLastStatusChangeCriteria);

		if (relations != null) {
			query.addRelations(relations);
			query.addCachedRelations(FLOW_INSTANCE_LIST_CACHED_RELATIONS);
		}

		return daoFactory.getFlowInstanceDAO().getAll(query);
	}

	protected Status getStatus(Flow flow, Integer statusID, Field... relations) throws SQLException {

		HighLevelQuery<Status> query = new HighLevelQuery<>();

		query.addParameter(statusIDParamFactory.getParameter(statusID));
		query.addParameter(statusFlowParamFactory.getParameter(flow));

		if (relations != null) {
			query.addRelations(relations);
		}

		return daoFactory.getStatusDAO().get(query);
	}

	private List<Status> getStatuses(Flow flow, Field... relations) throws SQLException {

		HighLevelQuery<Status> query = new HighLevelQuery<>();

		query.addParameter(statusFlowParamFactory.getParameter(flow));

		if (relations != null) {
			query.addRelations(relations);
		}

		return daoFactory.getStatusDAO().getAll(query);
	}

	public List<FlowInstance> getFlowInstanceSelection(boolean requireSameFamily, boolean requireSameFlow, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors, Field... relations) throws IOException, SQLException, AccessDeniedException {

		List<Integer> flowInstanceIDs = NumberUtils.toInt(req.getParameterValues("flowInstanceID"));

		if (flowInstanceIDs == null) {

			return null;
		}

		List<Field> mergedRelations = new ArrayList<>(FLOW_INSTANCE_SELECTION_REQUIRED_RELATIONS.size() + (relations == null ? 0 : relations.length));
		mergedRelations.addAll(FLOW_INSTANCE_SELECTION_REQUIRED_RELATIONS);

		if (relations != null) {
			for (Field field : relations) {
				if (!mergedRelations.contains(field)) {
					mergedRelations.add(field);
				}
			}
		}

		List<FlowInstance> flowInstances = getFlowInstances(flowInstanceIDs, mergedRelations);

		if (flowInstances == null) {

			return null;
		}

		Flow flow = null;
		FlowFamily flowFamily = null;

		for (FlowInstance flowInstance : flowInstances) {

			getGeneralAccessController().checkFlowInstanceAccess(flowInstance, user);

			if (requireSameFlow) {

				if (flow == null) {

					flow = flowInstance.getFlow();

				} else if (!flow.getFlowID().equals(flowInstance.getFlow().getFlowID())) {

					validationErrors.add(new ValidationError("MultipleFlowsSelected"));
				}

			} else if (requireSameFamily) {

				if (flowFamily == null) {

					flowFamily = flowInstance.getFlow().getFlowFamily();

				} else if (!flowFamily.getFlowFamilyID().equals(flowInstance.getFlow().getFlowFamily().getFlowFamilyID())) {

					validationErrors.add(new ValidationError("MultipleFlowFamiliesSelected"));
				}
			}
		}

		return flowInstances;
	}

	@Override
	@WebPublic(alias = "status")
	public ForegroundModuleResponse updateStatus(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException {

		FlowInstance flowInstance;

		if (uriParser.size() == 3 && NumberUtils.isInt(uriParser.get(2)) && (flowInstance = getFlowInstance(Integer.parseInt(uriParser.get(2)), null, getUpdateStatusRelations())) != null && !flowInstance.getStatus().getContentType().equals(ContentType.NEW)) {

			getGeneralAccessController().checkFlowInstanceAccess(flowInstance, user);

			if (!flowInstance.getFlow().isEnabled() || isOperatingStatusDisabled(flowInstance, true)) {

				return list(req, res, user, uriParser, FLOW_DISABLED_VALIDATION_ERROR);
			}

			log.info("User " + user + " requesting update status form for instance " + flowInstance);

			List<ValidationError> validationErrors = null;

			if (req.getMethod().equalsIgnoreCase("POST")) {

				validationErrors = new ArrayList<>();

				TransactionHandler transactionHandler = null;

				FlowInstanceEvent flowInstanceEvent = null;

				Status previousStatus = flowInstance.getStatus();

				Status status = null;

				ExternalMessageHandlerInstance externalMessageHandlerInstance = null;
				InternalMessageHandlerInstance internalMessageHandlerInstance = null;

				ExternalMessage externalMessage = null;
				InternalMessage internalMessage = null;

				req = externalMessageCRUD.parseRequest(req, validationErrors);

				try {

					Integer statusID = NumberUtils.toInt(req.getParameter("statusID"));

					if (statusID == null || (status = getStatus(flowInstance.getFlow(), statusID, Status.MANAGER_USERS_RELATION, Status.MANAGER_GROUPS_RELATION)) == null) {

						validationErrors.add(STATUS_NOT_FOUND_VALIDATION_ERROR);

					} else if (status.getContentType() == ContentType.NEW || status.getContentType() == ContentType.WAITING_FOR_MULTISIGN || status.getContentType() == ContentType.WAITING_FOR_PAYMENT || status.isUseAccessCheck() && !AccessUtils.checkAccess(user, status)) {

						validationErrors.add(INVALID_STATUS_VALIDATION_ERROR);
					}

					if (status != null && !previousStatus.equals(status)) {

						if (status.isAddExternalMessage() && FlowInstanceUtils.isNewExternalMessagesAllowed(flowInstance, status)) {

							externalMessage = externalMessageCRUD.create(req, res, uriParser, user, flowInstance, true, validationErrors, defaultAllowedExternalMessageFileExtensions);

							externalMessageHandlerInstance = messageHandler.getInstance(externalMessage);
						}

						if (status.isAddInternalMessage() && flowInstance.isInternalMessagesEnabled()) {

							internalMessage = internalMessageCRUD.create(req, user, flowInstance, validationErrors, defaultAllowedInternalMessageFileExtensions);

							internalMessageHandlerInstance = messageHandler.getInstance(internalMessage);
						}
					}

					if (status != null && CollectionUtils.isEmpty(validationErrors)) {

						if (!previousStatus.equals(status)) {

							if (status.isRequireSigning()) {

								redirectToMethod(req, res, "/signstatus/" + flowInstance.getFlowInstanceID() + "/" + status.getStatusID());
								return null;
							}

							transactionHandler = new TransactionHandler(dataSource);

							flowInstanceEvent = updateStatus(flowInstance, status, user, transactionHandler);

							messageHandler.add(externalMessageHandlerInstance, transactionHandler);
							messageHandler.add(internalMessageHandlerInstance, transactionHandler);

							processExtensionRequests(status, externalMessage, user, transactionHandler);

							transactionHandler.commit();

							messageHandler.commited(externalMessageHandlerInstance);
							messageHandler.commited(internalMessageHandlerInstance);
						}
					}

				} catch (Throwable t) {

					messageHandler.cleanup(externalMessageHandlerInstance);
					messageHandler.cleanup(internalMessageHandlerInstance);

					throw t;

				} finally {

					TransactionHandler.autoClose(transactionHandler);

					if (req instanceof MultipartRequest) {

						((MultipartRequest) req).deleteFiles();
					}
				}

				if (transactionHandler != null && transactionHandler.isCommited()) {

					sendStatusChangedEvents(flowInstance, flowInstanceEvent, previousStatus, user);

					processCommittedExtensionRequests(status, externalMessage, user);

					redirectToMethod(req, res, "/overview/" + flowInstance.getFlowInstanceID());
					return null;
				}
			}

			Document doc = this.createDocument(req, uriParser, user);

			Element updateInstanceStatusElement = doc.createElement("UpdateInstanceStatus");

			doc.getDocumentElement().appendChild(updateInstanceStatusElement);

			try {

				getUpdateManagersAccessController().checkFlowInstanceAccess(flowInstance, user);

			} catch (AccessDeniedException e) {

				XMLUtils.appendNewElement(doc, updateInstanceStatusElement, "HideUpdateManagers");
			}

			XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);

			MultiElementableListener<Status> multiElementableListener = new MultiElementableListener<>(2);
			multiElementableListener.addElementableListener(new FlowStatusManagerAccessElementableListener(user));
			multiElementableListener.addElementableListener(new FlowInstanceStatusMessageElementableListener(flowInstance));

			genDoc.addFieldElementableListener(Status.class, multiElementableListener);

			genDoc.addFieldElementableListener(MessageTemplate.class, new MessageTemplateTagReplaceElementableListener(flowInstance, user, getCurrentSiteProfile(req, user, uriParser, flowInstance.getFlow().getFlowFamily())));

			updateInstanceStatusElement.appendChild(flowInstance.toXML(genDoc));

			XMLUtils.append(doc, updateInstanceStatusElement, user);

			appendBookmark(doc, updateInstanceStatusElement, flowInstance, req, user);

			XMLUtils.appendNewElement(doc, updateInstanceStatusElement, "FormattedMaxFileSize", BinarySizeFormater.getFormatedSize(maxFileSize * BinarySizes.MegaByte));

			if (externalMessageVerifyAttachedFilesModal) {

				XMLUtils.appendNewElement(doc, updateInstanceStatusElement, "ExternalMessageVerifyAttachedFilesModal");
			}

			appendAllowedMessageFileExtensions(doc, updateInstanceStatusElement);

			if (!FlowInstanceUtils.isOwnersContactable(flowInstance)) {

				XMLUtils.appendNewElement(doc, updateInstanceStatusElement, "NoContactWay");
			}

			if (!CollectionUtils.isEmpty(validationErrors)) {

				XMLUtils.append(doc, updateInstanceStatusElement, validationErrors);
				updateInstanceStatusElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			}

			return new SimpleForegroundModuleResponse(doc, flowInstance.getFlow().getName(), this.getDefaultBreadcrumb());
		}

		return list(req, res, user, uriParser, FLOW_INSTANCE_NOT_FOUND_VALIDATION_ERROR);
	}

	private void appendAllowedMessageFileExtensions(Document doc, Element targetElement) {

		XMLUtils.append(doc, targetElement, "AllowedExternalMessageFileExtensions", "FileExtension", defaultAllowedExternalMessageFileExtensions);
		XMLUtils.append(doc, targetElement, "AllowedInternalMessageFileExtensions", "FileExtension", defaultAllowedInternalMessageFileExtensions);
	}

	public FlowInstanceEvent updateStatus(FlowInstance flowInstance, Status status, User user, TransactionHandler transactionHandler) throws SQLException {

		Status previousStatus = flowInstance.getStatus();

		log.info("User " + user + " changing status of instance " + flowInstance + " from " + previousStatus + " to " + status);

		flowInstance.setStatus(status);
		flowInstance.setLastStatusChange(TimeUtils.getCurrentTimestamp());
		daoFactory.getFlowInstanceDAO().update(flowInstance);

		return flowInstanceEventGenerator.addFlowInstanceEvent(flowInstance, EventType.STATUS_UPDATED, null, user, flowInstance.getLastStatusChange(), null, transactionHandler);
	}

	public void sendStatusChangedEvents(FlowInstance flowInstance, FlowInstanceEvent flowInstanceEvent, Status previousStatus, User user) {

		eventHandler.sendEvent(FlowInstance.class, new CRUDEvent<>(CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
		eventHandler.sendEvent(FlowInstance.class, new StatusChangedByManagerEvent(flowInstance, flowInstanceEvent, getSiteProfile(flowInstance), previousStatus, user), EventTarget.ALL);
	}

	private void processExtensionRequests(Status status, ExternalMessage externalMessage, User user, TransactionHandler transactionHandler) {

		if (!CollectionUtils.isEmpty(flowInstanceUpdateStatusFormExtensionProviders)) {

			for (FlowInstanceUpdateStatusFormExtensionProvider extensionProvider : flowInstanceUpdateStatusFormExtensionProviders) {

				try {

					extensionProvider.processRequest(status, externalMessage, user, transactionHandler);

				} catch (Exception e) {

					log.error("Error processing request for extension provider " + extensionProvider + " for user " + user, e);
				}
			}
		}
	}

	private void processCommittedExtensionRequests(Status status, ExternalMessage externalMessage, User user) {

		if (!CollectionUtils.isEmpty(flowInstanceUpdateStatusFormExtensionProviders)) {

			for (FlowInstanceUpdateStatusFormExtensionProvider extensionProvider : flowInstanceUpdateStatusFormExtensionProviders) {

				try {

					extensionProvider.processCommittedRequest(status, externalMessage, user);

				} catch (Exception e) {

					log.error("Error processing committed request for extension provider " + extensionProvider + " for user " + user, e);
				}
			}
		}
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateStatuses(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		List<ValidationError> validationErrors = new ArrayList<>();

		req = externalMessageCRUD.parseRequest(req, validationErrors);

		try {

			List<FlowInstance> flowInstances = getFlowInstanceSelection(true, true, req, res, user, uriParser, validationErrors, FlowInstance.STATUS_RELATION, FlowInstance.OWNERS_RELATION, FlowFamily.MESSAGE_TEMPLATES_RELATION);

			if (!validationErrors.isEmpty()) {

				return list(req, res, user, uriParser, validationErrors);

			}
			if (flowInstances == null) {

				redirectToDefaultMethod(req, res);

				return null;
			}

			Flow flow = flowInstances.get(0).getFlow();

			if (req.getMethod().equalsIgnoreCase("POST") && req.getParameter("directFromInstanceList") == null && validationErrors.isEmpty()) {

				Integer statusID = NumberUtils.toInt(req.getParameter("statusID"));

				Status status;

				if (statusID == null || (status = getStatus(flow, statusID, Status.MANAGER_USERS_RELATION, Status.MANAGER_GROUPS_RELATION)) == null) {

					validationErrors.add(FlowInstanceAdminModule.STATUS_NOT_FOUND_VALIDATION_ERROR);

				} else if (status.getContentType() == ContentType.NEW || status.getContentType() == ContentType.WAITING_FOR_MULTISIGN || status.getContentType() == ContentType.WAITING_FOR_PAYMENT || status.isUseAccessCheck() && !AccessUtils.checkAccess(user, status) || status.isRequireSigning()) {

					validationErrors.add(FlowInstanceAdminModule.INVALID_STATUS_VALIDATION_ERROR);

				} else if (validationErrors.isEmpty()) {

					FormSubmitProgressTracker formSubmitProgressTracker = FormSubmitProgressTracker.init(flowInstances.size(), moduleDescriptor.getModuleID(), req);

					try {

						for (FlowInstance flowInstance : flowInstances) {

							ExternalMessageHandlerInstance externalMessageHandlerInstance = null;
							InternalMessageHandlerInstance internalMessageHandlerInstance = null;

							ExternalMessage externalMessage = null;
							InternalMessage internalMessage = null;

							if (status.isAddExternalMessage() && FlowInstanceUtils.isNewExternalMessagesAllowed(flowInstance, status)) {

								externalMessage = externalMessageCRUD.create(req, res, uriParser, user, flowInstance, true, validationErrors, defaultAllowedExternalMessageFileExtensions);

								if (externalMessage != null) {

									externalMessage.setMessage(AttributeTagUtils.replaceTags(externalMessage.getMessage(), flowInstance.getAttributeHandler()));

									externalMessageHandlerInstance = messageHandler.getInstance(externalMessage);
								}
							}

							if (status.isAddInternalMessage() && flowInstance.isInternalMessagesEnabled()) {

								internalMessage = internalMessageCRUD.create(req, user, flowInstance, validationErrors, defaultAllowedInternalMessageFileExtensions);

								internalMessageHandlerInstance = messageHandler.getInstance(internalMessage);
							}

							if (validationErrors.isEmpty()) {

								formSubmitProgressTracker.increment();

								TransactionHandler transactionHandler = null;

								FlowInstanceEvent flowInstanceEvent = null;

								Status previousStatus = flowInstance.getStatus();

								try {

									transactionHandler = new TransactionHandler(dataSource);

									flowInstanceEvent = updateStatus(flowInstance, status, user, transactionHandler);

									messageHandler.add(externalMessageHandlerInstance, transactionHandler);
									messageHandler.add(internalMessageHandlerInstance, transactionHandler);

									processExtensionRequests(status, externalMessage, user, transactionHandler);

									transactionHandler.commit();

									messageHandler.commited(externalMessageHandlerInstance);
									messageHandler.commited(internalMessageHandlerInstance);

								} catch (Throwable t) {

									messageHandler.cleanup(externalMessageHandlerInstance);
									messageHandler.cleanup(internalMessageHandlerInstance);

									throw t;

								} finally {

									TransactionHandler.autoClose(transactionHandler);
								}

								sendStatusChangedEvents(flowInstance, flowInstanceEvent, previousStatus, user);

								processCommittedExtensionRequests(status, externalMessage, user);
							}
						}

					} finally {

						formSubmitProgressTracker.cleanup();
					}

					redirectToDefaultMethod(req, res);
					return null;
				}
			}

			log.info("User " + user + " requested form for mass changing flow instance statues");

			Document doc = createDocument(req, uriParser, user);

			XMLUtils.appendNewElement(doc, doc.getDocumentElement(), "fullAlias", getFullAlias());

			Element updateFlowInstanceStatusesElement = doc.createElement("UpdateFlowInstanceStatuses");
			doc.getDocumentElement().appendChild(updateFlowInstanceStatusesElement);

			List<Integer> flowInstanceIDs = new ArrayList<>(flowInstances.size());

			for (FlowInstance flowInstance : flowInstances) {
				flowInstanceIDs.add(flowInstance.getFlowInstanceID());
			}

			XMLUtils.append(doc, updateFlowInstanceStatusesElement, "FlowInstances", "flowInstanceID", flowInstanceIDs);

			XMLUtils.append(doc, updateFlowInstanceStatusesElement, flow);

			List<Status> statuses = getStatuses(flow, Status.MANAGER_USERS_RELATION, Status.MANAGER_GROUPS_RELATION);

			if (statuses != null) {

				Iterator<Status> it = statuses.iterator();
				while (it.hasNext()) {
					Status status = it.next();

					if (status.isRequireSigning() || status.isUseAccessCheck() && !AccessUtils.checkAccess(user, status)) {
						it.remove();
					}
				}
			}

			XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);

			genDoc.addRootElementableListener(Status.class, (document, element, status) -> {

				boolean anyFlowInstanceAllowsNewMessage = false;

				for (FlowInstance flowInstance : flowInstances) {

					Status previousStatus = flowInstance.getStatus();

					if (!previousStatus.equals(status) && FlowInstanceUtils.isNewExternalMessagesAllowed(flowInstance, status)) {

						anyFlowInstanceAllowsNewMessage = true;
						break;
					}
				}

				if (!anyFlowInstanceAllowsNewMessage) {

					XMLUtils.appendNewElement(document, element, "hideSendExternalMessage", true);
				}
			});

			if (flow.isHideInternalMessages()) {

				XMLUtils.appendNewElement(doc, updateFlowInstanceStatusesElement, "hideSendInternalMessage", true);
			}

			XMLUtils.append(genDoc, updateFlowInstanceStatusesElement, "Statuses", statuses);

			appendAllowedMessageFileExtensions(doc, updateFlowInstanceStatusesElement);

			XMLUtils.appendNewElement(doc, updateFlowInstanceStatusesElement, "FormattedMaxFileSize", BinarySizeFormater.getFormatedSize(maxFileSize * BinarySizes.MegaByte));

			if (externalMessageVerifyAttachedFilesModal) {

				XMLUtils.appendNewElement(doc, updateFlowInstanceStatusesElement, "ExternalMessageVerifyAttachedFilesModal");
			}

			if (!CollectionUtils.isEmpty(validationErrors)) {

				XMLUtils.append(doc, updateFlowInstanceStatusesElement, "ValidationErrors", validationErrors);
				updateFlowInstanceStatusesElement.appendChild(RequestUtils.getRequestParameters(req, doc));
			}

			return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());

		} finally {

			if (req instanceof MultipartRequest) {

				((MultipartRequest) req).deleteFiles();
			}
		}
	}

	@WebPublic(alias = "formsubmitprogress")
	public ForegroundModuleResponse getFormSubmitProgress(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return FormSubmitProgressTracker.getFormSubmitProgress(req, res, user, uriParser, moduleDescriptor.getModuleID(), log);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse addManagers(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws IOException, SQLException, AccessDeniedException {

		List<Integer> flowInstanceIDs = NumberUtils.toInt(req.getParameterValues("flowInstanceID"));

		if (flowInstanceIDs == null) {

			redirectToDefaultMethod(req, res);
			return null;
		}

		List<ValidationError> validationErrors = new ArrayList<>();

		List<FlowInstance> flowInstances = getFlowInstances(flowInstanceIDs, getUpdateManagerRelations());

		if (flowInstances != null && flowInstances.get(0).getFlow().getFlowFamily().checkManagerRestrictedAccess(user)) {

			filterRestrictedFlowInstances(flowInstances, user);
		}

		if (CollectionUtils.isEmpty(flowInstances)) {

			redirectToDefaultMethod(req, res);
			return null;
		}

		FlowFamily flowFamily = null;
		Integer profileID = null;

		for (FlowInstance flowInstance : flowInstances) {

			getUpdateManagersAccessController().checkFlowInstanceAccess(flowInstance, user);

			if (flowInstance.getFirstSubmitted() == null) {

				throw new AccessDeniedException("Flow instance is not submitted, " + flowInstance);
			}

			if (flowFamily == null) {

				flowFamily = flowInstance.getFlow().getFlowFamily();

			} else if (!flowFamily.getFlowFamilyID().equals(flowInstance.getFlow().getFlowFamily().getFlowFamilyID())) {

				validationErrors.add(new ValidationError("MultipleFlowFamiliesSelected"));
			}

			if (profileID == null) {

				profileID = flowInstance.getProfileID();

			} else if (!profileID.equals(flowInstance.getProfileID())) {

				validationErrors.add(new ValidationError("MultipleProfilesSelected"));
			}
		}

		if (!validationErrors.isEmpty()) {

			return list(req, res, user, uriParser, validationErrors);
		}

		//At this point only flow instances with the same flow family and profile should be in the list
		Collection<User> availableManagers = getAllowedManagers(flowInstances.get(0));
		Collection<Group> availableManagerGroups = getAllowedManagerGroups(flowInstances.get(0));

		if (req.getMethod().equalsIgnoreCase("POST") && req.getParameter("directFromInstanceList") == null) {

			boolean removeExistingManagers = BooleanUtils.toBoolean(req.getParameter("removeExistingManagers"));
			boolean removeExistingManagerGroups = BooleanUtils.toBoolean(req.getParameter("removeExistingManagerGroups"));
			List<Integer> userIDs = NumberUtils.toInt(req.getParameterValues("managerID"));
			List<Integer> groupIDs = NumberUtils.toInt(req.getParameterValues("managerGroupID"));

			if (userIDs != null || groupIDs != null || removeExistingManagers || removeExistingManagerGroups) {

				List<User> selectedManagers = new ArrayList<>();
				List<Group> selectedManagerGroups = new ArrayList<>();

				if (userIDs != null) {

					if (availableManagers != null) {
						for (User manager : availableManagers) {
							for (Integer userID : userIDs) {

								if (manager.getUserID().equals(userID)) {

									selectedManagers.add(manager);
									break;
								}
							}
						}
					}

					if (CollectionUtils.getSize(selectedManagers) < userIDs.size() && !removeExistingManagers && !removeExistingManagerGroups) {

						validationErrors.add(new ValidationError("OneOrMoreSelectedManagerUsersNotFoundError2"));
					}
				}

				if (groupIDs != null) {

					if (availableManagerGroups != null) {
						for (Group managerGroup : availableManagerGroups) {
							for (Integer groupID : groupIDs) {

								if (managerGroup.getGroupID().equals(groupID)) {

									selectedManagerGroups.add(managerGroup);
									break;
								}
							}
						}
					}

					if (CollectionUtils.getSize(selectedManagerGroups) < groupIDs.size() && !removeExistingManagers && !removeExistingManagerGroups) {

						validationErrors.add(new ValidationError("OneOrMoreSelectedManagerGroupsNotFoundError2"));
					}
				}

				if (validationErrors.isEmpty()) {

					Collections.sort(selectedManagers, UserNameComparator.getInstance());
					Collections.sort(selectedManagerGroups, GroupNameComparator.getInstance());

					String newManagersString = FlowInstanceUtils.getManagersString(selectedManagers, selectedManagerGroups);

					for (FlowInstance flowInstance : flowInstances) {

						if (!selectedManagers.isEmpty() && (flowInstance.getManagers() == null || !flowInstance.getManagers().containsAll(selectedManagers) || removeExistingManagers) || !selectedManagerGroups.isEmpty() && (flowInstance.getManagerGroups() == null || !flowInstance.getManagerGroups().containsAll(selectedManagerGroups) || removeExistingManagerGroups)) {

							log.info("User " + user + " adding " + selectedManagers.size() + " managers to instance " + flowInstance + ". New managers: " + newManagersString);

							List<User> previousManagers = flowInstance.getManagers();

							if (removeExistingManagers) {

								flowInstance.setManagers(selectedManagers);

							} else {

								List<User> combinedManagers = new ArrayList<>(selectedManagers);

								if (!CollectionUtils.isEmpty(previousManagers)) {

									for (User previousManager : previousManagers) {

										if (!combinedManagers.contains(previousManager)) {
											combinedManagers.add(previousManager);
										}
									}

									Collections.sort(combinedManagers, UserNameComparator.getInstance());
								}

								flowInstance.setManagers(combinedManagers);
							}

							List<Group> previousManagerGroups = flowInstance.getManagerGroups();

							if (removeExistingManagerGroups) {

								flowInstance.setManagerGroups(selectedManagerGroups);

							} else {

								List<Group> combinedManagerGroups = new ArrayList<>(selectedManagerGroups);

								if (!CollectionUtils.isEmpty(previousManagerGroups)) {

									for (Group previousManagerGroup : previousManagerGroups) {

										if (!combinedManagerGroups.contains(previousManagerGroup)) {
											combinedManagerGroups.add(previousManagerGroup);
										}
									}

									Collections.sort(combinedManagerGroups, GroupNameComparator.getInstance());
								}

								flowInstance.setManagerGroups(combinedManagerGroups);
							}

							String detailString = FlowInstanceUtils.getManagersString(flowInstance.getManagers(), flowInstance.getManagerGroups());

							saveAndSendEvent(user, flowInstance, previousManagerGroups, previousManagers, detailString);

						} else if (removeExistingManagers || removeExistingManagerGroups) {
							List<Group> previousManagerGroups = flowInstance.getManagerGroups();
							List<User> previousManagers = flowInstance.getManagers();
							if (removeExistingManagers) {
								flowInstance.setManagers(null);
							}
							if (removeExistingManagerGroups) {
								flowInstance.setManagerGroups(null);
							}
							saveAndSendEvent(user, flowInstance, previousManagerGroups, previousManagers, "");

						}

					}

					redirectToDefaultMethod(req, res);
					return null;
				}

			} else {

				validationErrors.add(new ValidationError("managerID", ValidationErrorType.RequiredField));
			}
		}

		log.info("User " + user + " requested form for mass adding flow instance managers");

		Document doc = createDocument(req, uriParser, user);

		Element addFlowInstanceManagersElement = doc.createElement("AddFlowInstanceManagers");
		doc.getDocumentElement().appendChild(addFlowInstanceManagersElement);

		XMLUtils.appendNewElement(doc, addFlowInstanceManagersElement, "FlowName", flowInstances.get(0).getFlow().getName());
		XMLUtils.appendNewElement(doc, addFlowInstanceManagersElement, "FlowID", flowInstances.get(0).getFlow().getFlowID());
		XMLUtils.append(doc, addFlowInstanceManagersElement, "FlowInstances", "flowInstanceID", flowInstanceIDs);
		if (alwaysListManagerGroups) {
			XMLUtils.appendNewElement(doc, addFlowInstanceManagersElement, "alwaysListManagerGroups", alwaysListManagerGroups);
		}

		XMLUtils.append(doc, addFlowInstanceManagersElement, "Managers", availableManagers);
		XMLUtils.append(doc, addFlowInstanceManagersElement, "ManagerGroups", availableManagerGroups);

		if (!CollectionUtils.isEmpty(validationErrors)) {

			XMLUtils.append(doc, addFlowInstanceManagersElement, "ValidationErrors", validationErrors);
			addFlowInstanceManagersElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	private void saveAndSendEvent(User user, FlowInstance flowInstance, List<Group> previousManagerGroups, List<User> previousManagers, String detailString) throws SQLException {

		RelationQuery relationQuery = new RelationQuery(FlowInstance.MANAGERS_RELATION, FlowInstance.MANAGER_GROUPS_RELATION);
		daoFactory.getFlowInstanceDAO().update(flowInstance, relationQuery);

		FlowInstanceEvent flowInstanceEvent = getFlowInstanceEventGenerator().addFlowInstanceEvent(flowInstance, EventType.MANAGERS_UPDATED, detailString, user);

		systemInterface.getEventHandler().sendEvent(FlowInstance.class, new CRUDEvent<>(CRUDAction.UPDATE, flowInstance), EventTarget.ALL);
		systemInterface.getEventHandler().sendEvent(FlowInstance.class, new ManagersChangedEvent(flowInstance, flowInstanceEvent, getSiteProfile(flowInstance), previousManagers, previousManagerGroups, user), EventTarget.ALL);
	}

	protected FlowInstanceAdminUserSettings getOrCreateUserSettings(User user) throws SQLException {

		FlowInstanceAdminUserSettings userSettings = getUserSettings(user);

		if (userSettings == null) {

			userSettings = new FlowInstanceAdminUserSettings();
			userSettings.setUser(user);
		}

		return userSettings;
	}

	protected FlowInstanceAdminUserSettings getUserSettings(User user) throws SQLException {

		HighLevelQuery<FlowInstanceAdminUserSettings> query = new HighLevelQuery<>();
		query.addParameter(userSettingUserParamFactory.getParameter(user));

		return userSettingsAnnotatedDAO.get(query);
	}

	@WebPublic(requireLogin = true, toLowerCase = true)
	public ForegroundModuleResponse saveUserSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		log.info("User " + user + " saving flow instance admin settings");

		res.setCharacterEncoding(systemInterface.getEncoding());
		res.setContentType(JsonUtils.getContentType());

		PrintWriter writer = res.getWriter();
		JsonObject jsonResponse = new JsonObject();

		synchronized (user) {

			boolean isNew = false;
			FlowInstanceAdminUserSettings userSettings = getUserSettings(user);

			if (userSettings == null) {

				isNew = true;
				userSettings = new FlowInstanceAdminUserSettings();
				userSettings.setUser(user);
			}

			userSettings = populateUserSettings(userSettings, req);

			if (userSettings != null) {

				if (isNew) {

					userSettingsAnnotatedDAO.add(userSettings);

				} else {

					userSettingsAnnotatedDAO.update(userSettings);
				}

				jsonResponse.putField("saved", "true");

			} else {

				jsonResponse.putField("error", "Unknown parameters");
			}
		}

		writer.print(jsonResponse.toJson());
		writer.flush();
		return null;
	}

	private FlowInstanceAdminUserSettings populateUserSettings(FlowInstanceAdminUserSettings userSettings, HttpServletRequest req) throws ValidationException {

		String columnOrder = req.getParameter("columnOrder");
		String columnVisibility = req.getParameter("columnVisibility");
		OpenMode openMode = EnumUtils.toEnum(OpenMode.class, req.getParameter("openMode"));
		ManagerDisplayMode managerDisplayMode = EnumUtils.toEnum(ManagerDisplayMode.class, req.getParameter("managerDisplayMode"));

		if (!StringUtils.isEmpty(columnOrder) && !StringUtils.isEmpty(columnVisibility) && openMode != null && managerDisplayMode != null) {

			columnOrder = populateColumnOrder(columnOrder);
			columnVisibility = populateColumnVisibility(columnVisibility);

			userSettings.setColumnOrder(columnOrder);
			userSettings.setColumnVisibility(columnVisibility);
			userSettings.setOpenMode(openMode);
			userSettings.setManagerDisplayMode(managerDisplayMode);

			return userSettings;
		}

		String panelID = req.getParameter("panelID");

		if (!StringUtils.isEmpty(panelID)) {

			validatePanelID(panelID);

			userSettings.setLastSavedTable(panelID);

			String tableSettings = req.getParameter("tableSettings");

			if (!StringUtils.isEmpty(tableSettings)) {

				tableSettings = populateTableSettings(tableSettings);

				if (panelID.equalsIgnoreCase(InstanceFilter.UNASSIGNED.name())) {

					userSettings.setTableUnassigned(tableSettings);

				} else if (panelID.equalsIgnoreCase(InstanceFilter.PRIORITIZED.name())) {

					userSettings.setTablePrioritized(tableSettings);

				} else if (panelID.equalsIgnoreCase(InstanceFilter.MINE.name())) {

					userSettings.setTableMine(tableSettings);

				} else if (panelID.equalsIgnoreCase(InstanceFilter.BOOKMARKED.name())) {

					userSettings.setTableBookmarked(tableSettings);

				} else if (panelID.equalsIgnoreCase(InstanceFilter.ACTIVE.name())) {

					userSettings.setTableActive(tableSettings);

				} else if (panelID.equalsIgnoreCase(InstanceFilter.ALL.name())) {

					userSettings.setTableAll(tableSettings);
				}
			}

			return userSettings;
		}

		DateInterval dateInterval = EnumUtils.toEnum(DateInterval.class, req.getParameter("dateInterval"));
		String fromDateParameter = req.getParameter("fromDate");
		String toDateParameter = req.getParameter("toDate");

		if (dateInterval == DateInterval.CUSTOM && !StringUtils.isEmpty(fromDateParameter) || !StringUtils.isEmpty(toDateParameter)) {

			LocalDate fromDate;
			LocalDate toDate;

			try {

				fromDate = LocalDate.parse(fromDateParameter);

			} catch (DateTimeParseException e) {

				fromDate = null;
			}

			try {

				toDate = LocalDate.parse(toDateParameter);

			} catch (DateTimeParseException e) {

				toDate = null;
			}

			userSettings.setDateInterval(dateInterval);
			userSettings.setFromDate(fromDate != null ? java.sql.Date.valueOf(fromDate) : null);
			userSettings.setToDate(toDate != null ? java.sql.Date.valueOf(toDate) : null);

			return userSettings;
		}

		if (dateInterval != DateInterval.CUSTOM && dateInterval != null) {

			userSettings.setDateInterval(dateInterval);
			userSettings.setFromDate(null);
			userSettings.setToDate(null);

			return userSettings;
		}

		return null;
	}

	private String populateColumnOrder(String columnOrder) {

		List<TableSettingColumnType> orderedColumnTypes = new JSONDeserializer<List<TableSettingColumnType>>().use("values", new TableSettingColumnTypeObjectFactory()).deserialize(columnOrder);

		Set<TableSettingColumnType> uniqueOrderedColumnsTypes = new LinkedHashSet<>(orderedColumnTypes);

		return new JSONSerializer().transform(new TableSettingColumnTypeTransformer(), TableSettingColumnType.class).serialize(uniqueOrderedColumnsTypes);
	}

	private String populateColumnVisibility(String columnVisibility) {

		Map<TableSettingColumnType, Boolean> columnsTypeVisibilites = new JSONDeserializer<Map<TableSettingColumnType, Boolean>>().use("keys", new TableSettingColumnTypeObjectFactory()).use("values", new BooleanObjectFactory()).deserialize(columnVisibility);

		return new JSONSerializer().serialize(columnsTypeVisibilites);
	}

	private void validatePanelID(String panelID) {

		InstanceFilter instanceFilter = InstanceFilter.valueOf(panelID.toUpperCase());

		if (instanceFilter == null) {

			throw new IllegalArgumentException("Invalid panelID: " + panelID);
		}
	}

	private String populateTableSettings(String tableSettings) throws ValidationException {

		TableSettings settings = new JSONDeserializer<TableSettings>().use(TableSettingsOrderEntry.class, new TableSettingOrderEntryObjectFactory()).deserialize(tableSettings, TableSettings.class);

		if (settings.getOrder() == null) {

			throw new ValidationException(new ValidationError("Order cannot be null"));
		}

		LocalDateTime parsedTime = LocalDateTime.ofEpochSecond(settings.getTime(), 0, OffsetDateTime.now().getOffset());

		if (parsedTime == null) {

			throw new ValidationException(new ValidationError("Invalid time"));
		}

		tableSettings = new JSONSerializer().exclude("*.class").transform(new TableSettingOrderEntryTransformer(), TableSettingsOrderEntry.class).deepSerialize(settings);

		return tableSettings;
	}

	@WebPublic(alias = "coloredtag")
	public ForegroundModuleResponse selectColoredTag(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException {

		FlowInstance flowInstance;

		if (coloredTagProvider != null && uriParser.size() >= 3 && NumberUtils.isInt(uriParser.get(2)) && (flowInstance = getFlowInstance(Integer.parseInt(uriParser.get(2)), null, SET_COLORED_TAG_ACCESS_RELATIONS)) != null) {

			getGeneralAccessController().checkFlowInstanceAccess(flowInstance, user);

			if (!flowInstance.getFlow().isEnabled() || isOperatingStatusDisabled(flowInstance, true)) {

				return list(req, res, user, uriParser, FLOW_DISABLED_VALIDATION_ERROR);
			}

			return coloredTagProvider.selectColoredTag(flowInstance, req, res, user, uriParser);
		}

		return null;
	}

	@Override
	public String getFlowInstancePreviewURL(Integer flowInstanceID) {

		return this.getFullAlias() + "/overview/" + flowInstanceID;
	}

	@Override
	@WebPublic(alias = "preview")
	public ForegroundModuleResponse processPreviewRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, AccessDeniedException, ModuleConfigurationException, SQLException, IOException, UnableToGetQueryInstanceShowHTMLException, FlowInstanceManagerClosedException, OperationNotSupportedException {

		throw new OperationNotSupportedException();
	}

	@Override
	@WebPublic(alias = "pdf")
	public ForegroundModuleResponse getEventPDF(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, IOException, AccessDeniedException {

		if (hideFlowInstanceHistoryIconColumn) {

			throw new AccessDeniedException("History icon column is disabled");
		}

		return super.getEventPDF(req, res, user, uriParser);
	}

	@Override
	@WebPublic(alias = "xml")
	public ForegroundModuleResponse getEventXML(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, IOException, AccessDeniedException {

		if (hideFlowInstanceHistoryIconColumn) {

			throw new AccessDeniedException("History icon column is disabled");
		}

		return super.getEventXML(req, res, user, uriParser);
	}

	@Override
	@WebPublic(alias = "signature")
	public ForegroundModuleResponse getEventSignature(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws URINotFoundException, SQLException, IOException, AccessDeniedException, ModuleConfigurationException {

		if (hideFlowInstanceHistoryIconColumn) {

			throw new AccessDeniedException("History icon column is disabled");
		}

		return super.getEventSignature(req, res, user, uriParser);
	}

	@WebPublic(alias = "getreadreceipts")
	public ForegroundModuleResponse getReadReceipts(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws SQLException, AccessDeniedException, IOException, URINotFoundException, ValidationException {

		List<ExternalMessageReadReceipt> readReceipts = externalMessageCRUD.getReadReceipts(user, uriParser, getGeneralAccessController(), true);

		JsonNode json = ExternalMessageReadReceipt.toJson(readReceipts);
		
		if (json != null) {
			
			HTTPUtils.sendReponse(json.toJson(), JsonUtils.getContentType(), res);
			
		} else {
			
			HTTPUtils.sendReponse(new JsonObject().toJson(), JsonUtils.getContentType(), res);
		}
		
		return null;
	}

	@WebPublic(alias = "disablereadreceipt")
	public ForegroundModuleResponse disableReadReceipt(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException {

		try {

			ExternalMessage externalMessage = externalMessageCRUD.disableReadReceipt(user, uriParser, getGeneralAccessController());

			redirectToMethod(req, res, "/overview/" + externalMessage.getFlowInstance().getFlowInstanceID() + "#messages-" + externalMessage.getMessageID());

		} catch (ValidationException e) {

			return list(req, res, user, uriParser, e.getErrors());
		}

		return null;
	}

	public FlowCacheAccess getManagedFlowFamilies(User user) {

		return flowAdminModule.getFlowCache().getManagedFlowFamilies(user);
	}

	public boolean addFlowInstanceOverviewHeaderExtensionProvider(FlowInstanceOverviewHeaderExtensionProvider provider) {

		return flowInstanceOverviewHeaderExtensionProviders.add(provider);
	}

	public boolean removeFlowInstanceOverviewHeaderExtensionProvider(FlowInstanceOverviewHeaderExtensionProvider provider) {

		return flowInstanceOverviewHeaderExtensionProviders.remove(provider);
	}

	public boolean addFlowInstanceOverviewHistoryExtensionProvider(FlowInstanceOverviewHistoryExtensionProvider provider) {

		return flowInstanceOverviewHistoryExtensionProviders.add(provider);
	}

	public boolean removeFlowInstanceOverviewHistoryExtensionProvider(FlowInstanceOverviewHistoryExtensionProvider provider) {

		return flowInstanceOverviewHistoryExtensionProviders.remove(provider);
	}

	public boolean addFlowInstanceOverviewDescriptionExtensionProvider(FlowInstanceOverviewDescriptionExtensionProvider provider) {

		return flowInstanceOverviewDescriptionExtensionProviders.add(provider);
	}

	public boolean removeFlowInstanceOverviewDescriptionExtensionProvider(FlowInstanceOverviewDescriptionExtensionProvider provider) {

		return flowInstanceOverviewDescriptionExtensionProviders.remove(provider);
	}

	public boolean addListOverviewButtonExtensionProvider(ListOverviewButtonExtensionProvider provider) {

		return listOverviewButtonExtensionLinkProviders.add(provider);
	}

	public boolean removeListOverviewButtonExtensionProvider(ListOverviewButtonExtensionProvider provider) {

		return listOverviewButtonExtensionLinkProviders.remove(provider);
	}

	public boolean addFlowInstanceProvider(ManagableFlowInstanceProvider flowInstanceProvider) {

		return adminFlowInstanceProviders.add(flowInstanceProvider);
	}

	public boolean removeFlowInstanceProvider(ManagableFlowInstanceProvider flowInstanceProvider) {

		return adminFlowInstanceProviders.remove(flowInstanceProvider);
	}

	public boolean addExternalMessageFormExtensionProvider(FlowInstanceUpdateStatusFormExtensionProvider flowInstanceUpdateStatusFormExtensionProvider) {

		return flowInstanceUpdateStatusFormExtensionProviders.add(flowInstanceUpdateStatusFormExtensionProvider);
	}

	public boolean removeExternalMessageFormExtensionProvider(FlowInstanceUpdateStatusFormExtensionProvider flowInstanceUpdateStatusFormExtensionProvider) {

		return flowInstanceUpdateStatusFormExtensionProviders.remove(flowInstanceUpdateStatusFormExtensionProvider);
	}

	public boolean addExternalMessageShowExtensionProvider(ExternalMessageExtensionProvider externalMessageExtensionProvider) {

		return externalMessageExtensionProviders.add(externalMessageExtensionProvider);
	}

	public boolean removeExternalMessageShowExtensionProvider(ExternalMessageExtensionProvider externalMessageExtensionProvider) {

		return externalMessageExtensionProviders.remove(externalMessageExtensionProvider);
	}

	public EventHandler getEventHandler() {

		return eventHandler;
	}

	public FlowNotificationHandler getFlowNotificationHandler() {

		return flowNotificationHandler;
	}
}
