package com.nordicpeak.flowengine.flowinstanceadmin2.beans;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.json.JsonUtils;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.SessionUtils;
import se.unlogic.webutils.http.URIParser;

public class FormSubmitProgressTracker {

	private Integer moduleID;

	private HttpSession session;

	private String uuid;

	private Counter counter;

	private FormSubmitProgressTracker(Integer moduleID, HttpSession session, String uuid, Integer total) {

		this.moduleID = moduleID;
		this.session = session;
		this.uuid = uuid;
		this.counter = new Counter(total);
	}

	public static FormSubmitProgressTracker init(Integer total, Integer moduleID, HttpServletRequest req) {

		HttpSession session = req.getSession();
		String uuid = req.getParameter("submitProgressUUID");

		FormSubmitProgressTracker tracker = new FormSubmitProgressTracker(moduleID, session, uuid, total);

		if (uuid != null) {

			SessionUtils.setAttribute(getAttributeName(moduleID, uuid), tracker.counter, session);
		}

		return tracker;
	}

	public void increment() {

		if (isInitialized()) {

			counter.increment();

			String attributeName = getAttributeName(uuid);

			if (counter.isFinished()) {

				SessionUtils.removeAttribute(attributeName, session);
				counter = null;

			} else {

				SessionUtils.setAttribute(attributeName, counter, session);
			}
		}
	}

	public void cleanup() {

		if (isInitialized()) {

			String attributeName = getAttributeName(uuid);

			SessionUtils.removeAttribute(attributeName, session);
		}
	}

	private boolean isInitialized() {

		return !ObjectUtils.containsNull(moduleID, counter, session, uuid) && counter.isInitialized();
	}

	private String getAttributeName(String submitProgressUUID) {

		return getAttributeName(moduleID, submitProgressUUID);
	}

	private static String getAttributeName(Integer moduleID, String submitProgressUUID) {

		return moduleID + "-formSubmitProgress-" + submitProgressUUID;
	}

	public static Counter getCounter(HttpServletRequest req, Integer moduleID, String submitProgressUUID) {

		if (submitProgressUUID == null) {

			return null;
		}

		String attributeName = getAttributeName(moduleID, submitProgressUUID);

		Object attribute = SessionUtils.getAttribute(attributeName, req);

		if (attribute instanceof Counter) {

			return (Counter) attribute;
		}

		return null;
	}

	public static ForegroundModuleResponse getFormSubmitProgress(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, Integer moduleID, Logger log) throws Exception {

		try {

			String submitProgressUUID = uriParser.get(2);

			if (submitProgressUUID != null) {

				log.info("User " + user + " getting form submit progress for UUID: " + submitProgressUUID);

				JsonObject json = new JsonObject(1);

				Counter counter = getCounter(req, moduleID, submitProgressUUID);

				if (counter != null && counter.isInitialized()) {

					JsonObject progressJSON = new JsonObject(3);

					json.putField("progress", progressJSON);

					progressJSON.putField("current", counter.getCurrent());
					progressJSON.putField("total", counter.getTotal());
					progressJSON.putField("percentage", counter.calculatePercentage());

					HTTPUtils.sendReponse(json.toJson(), JsonUtils.getContentType(), res);
				}
			}

		} catch (Exception e) {

			log.error("Error getting form submit progress", e);
		}

		sendProgressTrackerNotFoundError(res);
		return null;
	}

	private static void sendProgressTrackerNotFoundError(HttpServletResponse res) throws IOException {

		res.setStatus(HttpServletResponse.SC_NOT_FOUND);

		JsonObject jsonObject = new JsonObject(1);
		jsonObject.putField("error", "Progress tracker not found");

		HTTPUtils.sendReponse(jsonObject.toJson(), JsonUtils.getContentType(), res);
	}

	private class Counter {

		private Integer current;

		private Integer total;

		public Counter(Integer total) {

			this.current = 0;
			this.total = total;
		}

		private boolean isInitialized() {

			return !ObjectUtils.containsNull(getCurrent(), getTotal());
		}

		private boolean isFinished() {

			return current.equals(total);
		}

		public Integer getCurrent() {

			return current;
		}

		public Integer getTotal() {

			return total;
		}

		private void increment() {

			current++;
		}

		private int calculatePercentage() {

			return 100 * current / total;
		}
	}

}