package com.nordicpeak.flowengine.flowinstanceadminremoteinfo;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.menu.MenuItemDescriptor;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.flowinstanceadmin2.FlowInstanceAdmin2Module;
import com.nordicpeak.flowengine.flowinstanceadminremoteinfo.beans.FlowInstanceAdminRemoteInfoEndpoint;
import com.nordicpeak.flowengine.flowinstanceadminremoteinfo.beans.FlowInstanceAdminRemoteInfoSetting;
import com.nordicpeak.flowengine.flowinstanceadminremoteinfo.cruds.FlowInstanceAdminRemoteInfoEndpointCRUD;
import com.nordicpeak.flowengine.interfaces.APISource;
import com.nordicpeak.flowengine.interfaces.APISourceHandler;
import com.nordicpeak.flowengine.interfaces.APISourceProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowFamily;

public class FlowInstanceAdminRemoteInfoModule extends AnnotatedForegroundModule implements CRUDCallback<User>, ViewFragmentModule<ForegroundModuleDescriptor>, APISourceProvider {

	@XSLVariable(prefix = "java.")
	private String apiSourceShortTypeName;
	
	@XSLVariable(prefix = "java.")
	private String apiSourceFullTypeName;
	
	@XSLVariable(prefix = "java.")
	private String adminExtensionViewTitle = "Flowinstance admin remote info settings";

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Allowed encodings", description = "Encodings allowed to use")
	private List<String> allowedEncodings = Arrays.asList("ISO-8859-1", "UTF-8");

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmentXML;

	private AnnotatedDAOWrapper<FlowInstanceAdminRemoteInfoSetting, Integer> settingDAOWrapper;
	private AnnotatedDAOWrapper<FlowInstanceAdminRemoteInfoEndpoint, Integer> endpointDAOWrapper;
	private AnnotatedDAOWrapper<FlowInstanceAdminRemoteInfoEndpoint, Integer> endpointAdminDAOWrapper;

	private QueryParameterFactory<FlowInstanceAdminRemoteInfoSetting, Integer> flowFamilyIDParamFactory;
	
	private FlowInstanceAdminRemoteInfoEndpointCRUD endpointCRUD;

	private FlowAdminModule flowAdminModule;
	private FlowInstanceAdmin2Module flowInstanceAdminModule;

	@InstanceManagerDependency(required = true)
	protected StaticContentModule staticContentModule;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;

	private FlowInstanceAdminRemoteInfoSettingsFragment remoteInfoSettingsFragment;
	private FlowInstanceAdminRemoteInfoFragment remoteInfoFragment;
	
	private APISourceHandler apiSourceHandler;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, sectionInterface.getSystemInterface().getEncoding());

		super.init(moduleDescriptor, sectionInterface, dataSource);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		viewFragmentTransformer.setDebugXML(debugFragmentXML);

		super.moduleConfigured();
	}

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, FlowInstanceAdminRemoteInfoModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		AnnotatedDAO<FlowInstanceAdminRemoteInfoEndpoint> endpointDAO = daoFactory.getDAO(FlowInstanceAdminRemoteInfoEndpoint.class);
		AnnotatedDAO<FlowInstanceAdminRemoteInfoSetting> settingDAO = daoFactory.getDAO(FlowInstanceAdminRemoteInfoSetting.class);

		settingDAOWrapper = settingDAO.getWrapper(Integer.class);
		settingDAOWrapper.addRelation(FlowInstanceAdminRemoteInfoSetting.ENDPOINT_RELATION);
		settingDAOWrapper.setUseRelationsOnGet(true);
		
		flowFamilyIDParamFactory = settingDAO.getParamFactory("flowFamilyID", Integer.class);

		endpointDAOWrapper = endpointDAO.getWrapper(Integer.class);

		endpointAdminDAOWrapper = endpointDAO.getWrapper(Integer.class);
		endpointAdminDAOWrapper.addRelation(FlowInstanceAdminRemoteInfoEndpoint.SETTINGS_RELATION);
		endpointAdminDAOWrapper.setUseRelationsOnGet(true);
		endpointAdminDAOWrapper.setUseRelationsOnGetAll(true);

		endpointCRUD = new FlowInstanceAdminRemoteInfoEndpointCRUD(endpointAdminDAOWrapper, this);
	}	
	
	@Override
	public void unload() throws Exception {

		systemInterface.getInstanceHandler().removeInstance(FlowInstanceAdminRemoteInfoModule.class, this);

		if (flowAdminModule != null) {

			setFlowAdminModule(null);
		}

		if (flowInstanceAdminModule != null) {

			setFlowInstanceAdminModule(null);
		}

		super.unload();
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (this.flowAdminModule != null) {

			this.flowAdminModule.removeFragmentExtensionViewProvider(remoteInfoSettingsFragment);

			remoteInfoSettingsFragment = null;
		}

		this.flowAdminModule = flowAdminModule;

		if (flowAdminModule != null) {

			remoteInfoSettingsFragment = new FlowInstanceAdminRemoteInfoSettingsFragment(this, priority, adminExtensionViewTitle);

			flowAdminModule.addFragmentExtensionViewProvider(remoteInfoSettingsFragment);
		}
	}

	@InstanceManagerDependency(required = true)
	public void setFlowInstanceAdminModule(FlowInstanceAdmin2Module flowInstanceAdminModule) {

		if (this.flowInstanceAdminModule != null) {

			this.flowInstanceAdminModule.removeFlowInstanceOverviewDescriptionExtensionProvider(remoteInfoFragment);

			remoteInfoFragment = null;
		}

		this.flowInstanceAdminModule = flowInstanceAdminModule;

		if (flowInstanceAdminModule != null) {

			remoteInfoFragment = new FlowInstanceAdminRemoteInfoFragment(this, connectionTimeout, readTimeout);

			flowInstanceAdminModule.addFlowInstanceOverviewDescriptionExtensionProvider(remoteInfoFragment);
		}
	}

	@InstanceManagerDependency
	public void setAPISourceHandler(APISourceHandler apiSourceHandler) throws SQLException {
		
		if (apiSourceHandler != null) {
			
			apiSourceHandler.addAPISourceProvider(this);
			
		} else if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		this.apiSourceHandler = apiSourceHandler;
		
		//Check if this module is fully cached, then trigger an update of the menu items
		if(sectionInterface.getForegroundModuleCache().isCached(moduleDescriptor)) {
			
			this.sectionInterface.getMenuCache().moduleUpdated(moduleDescriptor, this);
		}
	}	
	
	public List<FlowFamily> getFlowFamilies(FlowInstanceAdminRemoteInfoEndpoint endpoint) {

		List<FlowInstanceAdminRemoteInfoSetting> settings = endpoint.getSettings();

		if (CollectionUtils.isEmpty(settings)) {

			return Collections.emptyList();
		}

		List<FlowFamily> flowFamilies = new ArrayList<>();

		for (FlowInstanceAdminRemoteInfoSetting setting : settings) {

			FlowFamily flowFamily = flowAdminModule.getFlowFamily(setting.getFlowFamilyID());

			if (flowFamily != null) {

				flowFamilies.add(flowFamily);
			}
		}

		return flowFamilies;
	}

	public Flow getLatestFlowVersion(FlowFamily flowFamily) {

		return flowAdminModule.getLatestFlowVersion(flowFamily);
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();

		Element documentElement = doc.createElement("Document");

		XMLUtils.appendNewElement(doc, documentElement, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));

		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(documentElement);

		return doc;
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.list(req, res, user, uriParser, null);
	}

	@WebPublic
	public ForegroundModuleResponse show(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.show(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse add(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.add(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse update(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.update(req, res, user, uriParser);
	}

	@WebPublic
	public ForegroundModuleResponse delete(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.delete(req, res, user, uriParser);
	}

	public List<String> getAllowedEncodings() {

		return allowedEncodings;
	}

	protected ModuleViewFragmentTransformer<ForegroundModuleDescriptor> getViewFragmentTransformer() {

		return viewFragmentTransformer;
	}

	protected AnnotatedDAOWrapper<FlowInstanceAdminRemoteInfoSetting, Integer> getSettingDAO() {

		return settingDAOWrapper;
	}

	public AnnotatedDAOWrapper<FlowInstanceAdminRemoteInfoEndpoint, Integer> getEndpointDAO() {

		return endpointDAOWrapper;
	}

	protected SiteProfile getCurrentSiteProfile(HttpServletRequest req, User user, URIParser uriParser, ImmutableFlowFamily flowFamily) {

		return flowAdminModule.getCurrentSiteProfile(req, user, uriParser, flowFamily);
	}

	@Override
	public List<? extends APISource> getAPISources() throws Exception {

		return endpointAdminDAOWrapper.getAll();
	}

	@Override
	public String getShortTypeName() {

		return apiSourceShortTypeName;
	}
	
	@Override
	public String getFullTypeName() {

		return apiSourceFullTypeName;
	}
	
	@Override
	public String getAddURL() {

		return this.getFullAlias() + "/add";
	}

	@Override
	public String getBaseShowURL() {

		return this.getFullAlias() + "/show/";
	}
	
	@Override
	public String getBaseUpdateURL() {

		return this.getFullAlias() + "/update/";
	}

	@Override
	public String getBaseDeleteURL() {

		return this.getFullAlias() + "/delete/";
	}

	@Override
	public boolean isInUse(APISource apiSource) {

		FlowInstanceAdminRemoteInfoEndpoint endpoint = (FlowInstanceAdminRemoteInfoEndpoint)apiSource;
		
		List<FlowInstanceAdminRemoteInfoSetting> settings = endpoint.getSettings();

		if (settings != null) {

			for (FlowInstanceAdminRemoteInfoSetting setting : settings) {

				if (flowAdminModule.getFlowFamily(setting.getFlowFamilyID()) != null) {

					return true;
				}
			}
		}

		return false;
	}

	@EventListener(channel = FlowFamily.class)
	public void processFlowFamilyEvent(CRUDEvent<FlowFamily> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.DELETE) {

			try (TransactionHandler transaction = settingDAOWrapper.getAnnotatedDAO().createTransaction()) {
			
				for (FlowFamily flowFamily : event.getBeans()) {

					log.info("Deleting endpoint settings for flow family ID: " + flowFamily.getFlowFamilyID());

					try {
						HighLevelQuery<FlowInstanceAdminRemoteInfoSetting> query = new HighLevelQuery<>();

						query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));

						settingDAOWrapper.getAnnotatedDAO().delete(query, transaction);

					} catch (SQLException e) {

						log.error("Error endpoint settings for flow family ID: " + flowFamily.getFlowFamilyID(), e);
					}
				}
				
				transaction.commit();
				
			} catch (Exception e) {
				
				log.error("Error deleting endpoint settings for event: " + event, e);
			}
		}
	}

	public APISourceHandler getAPISourceHandler() {

		return apiSourceHandler;
	}
	
	@Override
	public List<? extends MenuItemDescriptor> getVisibleMenuItems() {

		if(this.apiSourceHandler != null) {
			
			return null;
		
		}else {
		
			return super.getVisibleMenuItems();
		}
	}
}
