package com.nordicpeak.flowengine.list;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.date.DateStringyfier;
import se.unlogic.standardutils.date.DateUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;

@XMLElement
public class FlowSelection extends GeneratedElementable {
	
	
	@XMLElement
	private FlowFamily flowFamily;
	
	@XMLElement
	private List<Integer> flowIDs;
	
	private List<Flow> flows;
	
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Timestamp fromDate;
	
	@XMLElement(valueFormatter = DateStringyfier.class)
	private Timestamp toDate;
	
	private ForegroundModuleResponse errorResponse;
	
	@XMLElement
	private boolean allFlows;
	
	public FlowFamily getFlowFamily() {
		
		return flowFamily;
	}
	
	public void setFlowFamily(FlowFamily flowFamily) {
		
		this.flowFamily = flowFamily;
	}
	
	public List<Flow> getFlows() {
		
		return flows;
	}
	
	public void setFlows(List<Flow> flows, boolean allFlows) {
		
		this.allFlows = allFlows;
		this.flows = flows;
		
		if (flows == null) {
			
			flowIDs = null;
			
		} else {
			
			flowIDs = new ArrayList<Integer>(flows.size());
			
			for (Flow flow : flows) {
				
				flowIDs.add(flow.getFlowID());
			}
		}
	}
	
	public Timestamp getFromDate() {
		
		return fromDate;
	}
	
	public void setFromDate(Timestamp fromDate) {
		
		this.fromDate = fromDate;
	}
	
	public Timestamp getToDate() {
		
		return toDate;
	}
	
	public void setToDate(Timestamp toDate) {
		
		this.toDate = toDate;
	}
	
	public ForegroundModuleResponse getErrorResponse() {
		
		return errorResponse;
	}
	
	public void setErrorResponse(ForegroundModuleResponse errorResponse) {
		
		this.errorResponse = errorResponse;
	}
	
	@Override
	public String toString() {
		
		StringBuilder builder = new StringBuilder();
		
		builder.append("flowFamily=" + flowFamily);
		builder.append(", fromDate=" + fromDate);
		builder.append(", toDate=" + toDate);
		
		if (!CollectionUtils.isEmpty(flowIDs)) {
			
			builder.append(", flowIDs=[ ");
			
			for (Integer flowID : flowIDs) {
				
				builder.append(flowID);
				builder.append(", ");
			}
			
			builder.append(" ]");
		}
		
		return builder.toString();
	}
	
	@Override
	public Element toXML(Document doc) {
		
		Element element = super.toXML(doc);
		
		XMLUtils.appendNewElement(doc, element, "URLParameters", toURLParameters());
		
		return element;
	}
	
	public String toURLParameters() {
		
		StringBuilder builder = new StringBuilder();
		
		builder.append("fromDate=" + (fromDate == null ? "" : DateUtils.DATE_FORMATTER.format(fromDate)));
		builder.append("&toDate=" + (toDate == null ? "" : DateUtils.DATE_FORMATTER.format(toDate)));
		
		if (!allFlows && !CollectionUtils.isEmpty(flowIDs)) {
			
			for (Integer flowID : flowIDs) {
				
				builder.append("&flowID=");
				builder.append(flowID);
			}
		}
		
		return builder.toString();
	}
	
}
