package com.nordicpeak.flowengine.voting;

import java.io.Serializable;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CaseInsensitiveStringComparator;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

@Table(name = "flow_family_vote_settings")
@XMLElement(name = "Settings")
public class FlowFamilyVoteSettings extends GeneratedElementable implements Serializable {
	
	private static final long serialVersionUID = 5949025706273639735L;
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer flowFamilyID;
	
	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private String alias;
	
	@DAOManaged(columnName = "siteProfileID")
	@WebPopulate
	@XMLElement
	private Integer siteProfileID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean votingActivated;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private String municipalCode;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer pageSize;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "votingActivated")
	@XMLElement
	private String title;
	
	@DAOManaged
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String description;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "votingActivated")
	@XMLElement
	private Integer votingDays;
	
	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@RequiredIfSet(paramNames = "votingActivated")
	@XMLElement
	private Integer requiredVotes;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "flow_family_vote_settings_published", remoteValueColumnName = "statusName")
	@WebPopulate(required = true, maxLength = 255, paramName = "publishedStatus")
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@XMLElement(fixCase = true, childName = "statusName")
	private List<String> publishedStatuses;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "votingActivated")
	@XMLElement
	private String votePassedStatus;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "votingActivated")
	@XMLElement
	private String voteFailedStatus;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "flow_family_vote_settings_archived", remoteValueColumnName = "statusName")
	@WebPopulate(required = true, maxLength = 255, paramName = "archivedStatus")
	@NoDuplicates(comparator = CaseInsensitiveStringComparator.class)
	@XMLElement(fixCase = true, childName = "statusName")
	private List<String> archivedStatuses;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showStatusInList;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean additionalAttributes;	
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoAttribute1")
	@XMLElement
	private String extraInfoName1;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoName1")
	@XMLElement
	private String extraInfoAttribute1;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideInList1;	
	
	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoAttribute2")
	@XMLElement
	private String extraInfoName2;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoName2")
	@XMLElement
	private String extraInfoAttribute2;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideInList2;	

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoAttribute3")
	@XMLElement
	private String extraInfoName3;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "extraInfoName3")
	@XMLElement
	private String extraInfoAttribute3;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideInList3;	

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideAttachedFiles;	

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean rssFeedActive;	


	public Integer getFlowFamilyID() {
		
		return flowFamilyID;
	}
	
	public void setFlowFamilyID(Integer flowFamilyID) {
		
		this.flowFamilyID = flowFamilyID;
	}
	
	public String getAlias() {
		return alias;
	}
	
	public void setAlias(String alias) {
		this.alias = alias;
	}
	
	public Integer getSiteProfileID() {
		return siteProfileID;
	}
	
	public void setSiteProfileID(Integer siteProfileID) {
		this.siteProfileID = siteProfileID;
	}

	public boolean isVotingActivated() {
		
		return votingActivated;
	}
	
	public void setVotingActivated(boolean votingActivated) {
	
		this.votingActivated = votingActivated;
	}
	
	public String getMunicipalCode() {
		return municipalCode;
	}
	
	public void setMunicipalCode(String municipalCode) {
		this.municipalCode = municipalCode;
	}

	public Integer getPageSize() {
		return pageSize;
	}

	public void setPageSize(Integer pageSize) {
		this.pageSize = pageSize;
	}

	public String getTitle() {
		return title;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	
	public String getDescription() {
		return description;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
	
	public Integer getVotingDays() {
		return votingDays;
	}
	
	public void setVotingDays(Integer votingDays) {
		this.votingDays = votingDays;
	}
	
	public Integer getRequiredVotes() {
		return requiredVotes;
	}
	
	public void setRequiredVotes(Integer requiredVotes) {
		this.requiredVotes = requiredVotes;
	}
	
	public String getVotePassedStatus() {
		return votePassedStatus;
	}
	
	public void setVotePassedStatus(String votePassedStatus) {
		this.votePassedStatus = votePassedStatus;
	}
	
	public String getVoteFailedStatus() {
		return voteFailedStatus;
	}
	
	public void setVoteFailedStatus(String voteFailedStatus) {
		this.voteFailedStatus = voteFailedStatus;
	}
	
	public List<String> getArchivedStatuses() {
		return archivedStatuses;
	}
	
	public void setArchivedStatuses(List<String> visibleStatuses) {
		this.archivedStatuses = visibleStatuses;
	}
	
	public List<String> getPublishedStatuses() {
	
		return publishedStatuses;
	}
	
	public void setPublishedStatuses(List<String> publishedStatuses) {
	
		this.publishedStatuses = publishedStatuses;
	}

	public boolean isShowStatusInList() {
		
		return showStatusInList;
	}
	
	public void setShowStatusInList(boolean showStatusInList) {
	
		this.showStatusInList = showStatusInList;
	}
	
	public boolean isAdditionalAttributes() {
		
		return additionalAttributes;
	}
	
	public void setAdditionalAttributes(boolean additionalAttributes) {
	
		this.additionalAttributes = additionalAttributes;
	}

	public String getExtraInfoName1() {
	
		return extraInfoName1;
	}
	
	public void setExtraInfoName1(String extraInfoName1) {
	
		this.extraInfoName1 = extraInfoName1;
	}
	
	public String getExtraInfoAttribute1() {
	
		return extraInfoAttribute1;
	}

	public void setExtraInfoAttribute1(String extraInfoAttribute1) {
	
		this.extraInfoAttribute1 = extraInfoAttribute1;
	}

	public boolean isHideInList1() {
		
		return hideInList1;
	}
	
	public void setHideInList1(boolean hideInList1) {
	
		this.hideInList1 = hideInList1;
	}

	public String getExtraInfoName2() {
	
		return extraInfoName2;
	}
	
	public void setExtraInfoName2(String extraInfoName2) {
	
		this.extraInfoName2 = extraInfoName2;
	}
	
	public String getExtraInfoAttribute2() {
	
		return extraInfoAttribute2;
	}

	public void setExtraInfoAttribute2(String extraInfoAttribute2) {
	
		this.extraInfoAttribute2 = extraInfoAttribute2;
	}
	
	public boolean isHideInList2() {
		
		return hideInList2;
	}
	
	public void setHideInList2(boolean hideInList2) {
	
		this.hideInList2 = hideInList2;
	}
	
	public String getExtraInfoName3() {
	
		return extraInfoName3;
	}

	public void setExtraInfoName3(String extraInfoName3) {
	
		this.extraInfoName3 = extraInfoName3;
	}

	public String getExtraInfoAttribute3() {
	
		return extraInfoAttribute3;
	}

	public void setExtraInfoAttribute3(String extraInfoAttribute3) {
	
		this.extraInfoAttribute3 = extraInfoAttribute3;
	}
	
	public boolean isHideInList3() {
		
		return hideInList3;
	}

	public void setHideInList3(boolean hideInList3) {
	
		this.hideInList3 = hideInList3;
	}

	public boolean isHideAttachedFiles() {
	
		return hideAttachedFiles;
	}
	
	public void setHideAttachedFiles(boolean hideAttachedFiles) {
	
		this.hideAttachedFiles = hideAttachedFiles;
	}

	public boolean isRssFeedActive() {
		return rssFeedActive;
	}

	public void setRssFeedActive(boolean rssFeedActive) {
		this.rssFeedActive = rssFeedActive;
	}
}
