package com.nordicpeak.flowengine.paymentprovider;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.emailutils.framework.SimpleEmail;
import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.HTMLEditorSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.SimpleViewFragment;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.ViewFragment;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.core.utils.ViewFragmentUtils;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.SingleTagSource;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.Contact;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.FlowPaymentCallback;
import com.nordicpeak.flowengine.interfaces.FlowPaymentProvider;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowFamily;
import com.nordicpeak.flowengine.interfaces.InvoiceLine;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ImmutableFlowInstanceManager;
import com.nordicpeak.flowengine.notifications.StandardFlowNotificationHandler;
import com.nordicpeak.flowengine.paymentprovider.beans.FlowFamilyPaymentSettings;
import com.nordicpeak.flowengine.paymentprovider.beans.GenericFlowPaymentCallback;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;
import com.nordicpeak.openhierarchy.payment.beans.SimpleInvoice;
import com.nordicpeak.openhierarchy.payment.beans.SimpleInvoiceLine;
import com.nordicpeak.openhierarchy.payment.enums.PaymentType;
import com.nordicpeak.openhierarchy.payment.events.DelayedPaymentEvent;
import com.nordicpeak.openhierarchy.payment.exceptions.PaymentConfigurationIDNotUsed;
import com.nordicpeak.openhierarchy.payment.interfaces.Invoice;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentCallback;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentConfigurationInformationProvider;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentProvider;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentResponse;

public class GenericFlowPaymentProvider extends AnnotatedForegroundModule implements FlowPaymentProvider, FlowAdminExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor>, PaymentConfigurationInformationProvider {
	
	private static final AnnotatedBeanTagSourceFactory<Flow> FLOW_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<Flow>(Flow.class, "$flow.");
	private static final AnnotatedBeanTagSourceFactory<FlowInstance> FLOWINSTANCE_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<FlowInstance>(FlowInstance.class, "$flowInstance.");
	
	private static final AnnotatedRequestPopulator<FlowFamilyPaymentSettings> POPULATOR = new AnnotatedRequestPopulator<FlowFamilyPaymentSettings>(FlowFamilyPaymentSettings.class);
	
	@XSLVariable(prefix = "java.")
	private String invoiceDescriptionCustomer = "Customer";
	
	@XSLVariable(prefix = "java.")
	private String invoiceDescriptionCategory = "Category";
	
	@XSLVariable(prefix = "java.")
	private String extensionViewTitle = "Paynova payment settings";
	
	@XSLVariable(prefix = "java.")
	private String onlinePaymentText = "Payed using";
	
	@XSLVariable(prefix = "java.")
	private String orderIDPrefix = "FlowInstance";
	
	@XSLVariable(prefix = "java.")
	private String userNotLoggedIn = "User not logged in";
	
	@XSLVariable(prefix = "java.")
	private String defaultInvoicePaymentEventDescription = "Invoice payment selected";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Invoice payment notification subject", description = "The subject used for invoice payment notification emails", required = true)
	@XSLVariable(prefix = "java.")
	private String invoiceEmailNotificationSubject;
	
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Invoice payment notification subject", description = "The subject used for invoice payment notification emails", required = true)
	@XSLVariable(prefix = "java.")
	private String invoiceEmailNotificationMessage;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Direct payment notification subject", description = "The subject used for direct payment notification emails", required = true)
	@XSLVariable(prefix = "java.")
	private String directPaymentEmailNotificationSubject;
	
	@ModuleSetting
	@HTMLEditorSettingDescriptor(name = "Direct payment notification subject", description = "The subject used for direct payment notification emails", required = true)
	@XSLVariable(prefix = "java.")
	private String directPaymentEmailNotificationMessage;
	
	//TODO email recipient for DelayedPayment flow instance not found
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Priority", description = "The priority of this extension provider compared to other providers. A low value means a higher priority. Valid values are 0 - " + Integer.MAX_VALUE + ".", required = true, formatValidator = NonNegativeStringIntegerValidator.class)
	protected int priority = 0;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Use only invoice description suffix", description = "Use only invoice description suffix if available")
	private boolean useOnlyInvoiceDescriptionSuffix;
	
	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Include flow ID in title", description = "Controls if the flow ID is included in the title of the invoice")
	private boolean includeFlowIDInTitle = true;
	
	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;
	
	@InstanceManagerDependency
	private FlowInstanceAdminModule flowInstanceAdminModule;
	
	@InstanceManagerDependency(required = true)
	private FlowBrowserModule flowBrowserModule;
	
	@InstanceManagerDependency
	protected StandardFlowNotificationHandler standardFlowNotificationHandler;
	
	@InstanceManagerDependency(required = true)
	private PaymentProvider paymentProvider;
	
	private FlowAdminModule flowAdminModule;
	
	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;
	
	private AnnotatedDAO<FlowFamilyPaymentSettings> settingsDAO;
	
	private QueryParameterFactory<FlowFamilyPaymentSettings, Integer> flowFamilyIDParamFactory;
	
	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {
		
		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<ForegroundModuleDescriptor>(sectionInterface.getForegroundModuleXSLTCache(), this, sectionInterface.getSystemInterface().getEncoding());
		
		super.init(moduleDescriptor, sectionInterface, dataSource);
		
		if (!systemInterface.getInstanceHandler().addInstance(FlowPaymentProvider.class, this)) {
			
			throw new RuntimeException("Unable to register module " + this.moduleDescriptor + " in global instance handler using key " + FlowPaymentProvider.class.getSimpleName() + ", another instance is already registered using this key.");
		}
	}
	
	@Override
	public void unload() throws Exception {
		
		systemInterface.getInstanceHandler().removeInstance(FlowPaymentProvider.class, this);
		
		if (flowAdminModule != null) {
			
			setFlowAdminModule(null);
		}
		
		super.unload();
	}
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		super.createDAOs(dataSource);
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, GenericFlowPaymentProvider.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		settingsDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(FlowFamilyPaymentSettings.class);
		
		flowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
	}
	
	@Override
	protected void moduleConfigured() throws Exception {
		
		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);
	}
	
	private String getPaymentProviderConfigurationID(FlowInstanceManager instanceManager) {
		
		return getPaymentProviderConfigurationID(instanceManager.getFlowInstance().getFlow());
	}
	
	private String getPaymentProviderConfigurationID(ImmutableFlow flow) {
		
		return getPaymentProviderConfigurationID(flow.getFlowFamily());
	}
	
	private String getPaymentProviderConfigurationID(ImmutableFlowFamily flowFamily)	{
		
		return "flowfamily-" + flowFamily.getFlowFamilyID();
	}
	
	private Integer getFlowFamilyIDFromPaymentProviderConfigurationID(String configurationID) {
		
		if (configurationID.startsWith("flowfamily-")) {
			
			return NumberUtils.toInt(configurationID.substring("flowfamily-".length()));
		}
		
		return null;
	}
	
	private ViewFragment showPaymentForm(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, Invoice invoice, FlowFamilyPaymentSettings paymentSettings, FlowInstanceManager instanceManager, PaymentCallback callback, List<ValidationError> validationErrors) throws Exception {
		
		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		doc.appendChild(document);
		
		Element paymentElement = doc.createElement("PaymentForm");
		document.appendChild(paymentElement);
		
		paymentElement.appendChild(paymentSettings.toXML(doc));
		
		ViewFragment paymentProviderViewFragment = paymentProvider.showPaymentForm(getPaymentProviderConfigurationID(instanceManager), invoice, req, user, uriParser, callback, validationErrors);
		
		paymentElement.appendChild(paymentProviderViewFragment.toXML(doc));
		
		SimpleViewFragment viewFragment = (SimpleViewFragment) viewFragmentTransformer.createViewFragment(doc);
		
		return new SimpleViewFragment(viewFragment.getHTML(), viewFragment.getDebugXML(), CollectionUtils.combine(viewFragment.getScripts(), paymentProviderViewFragment.getScripts()), CollectionUtils.combine(viewFragment.getLinks(), paymentProviderViewFragment.getLinks()));
	}
	
	@Override
	public ViewFragment pay(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, FlowInstanceManager instanceManager, FlowPaymentCallback callback) throws Exception {
		
		FlowFamilyPaymentSettings paymentSettings = getPaymentSettings(instanceManager.getFlowInstance().getFlow().getFlowFamily());
		
		if (paymentSettings == null) {
			
			log.warn("No payment settings found for flow " + instanceManager.getFlowInstance().getFlow());
			
			Document doc = XMLUtils.createDomDocument();
			Element document = doc.createElement("Document");
			doc.appendChild(document);
			
			Element paymentElement = doc.createElement("NoPaymentSettingsError");
			document.appendChild(paymentElement);
			
			return viewFragmentTransformer.createViewFragment(doc);
		}
		
		Invoice invoice = getInvoice(paymentSettings, instanceManager, user);
		List<ValidationError> validationErrors = null;
		
		try {
			PaymentResponse paymentResponse = paymentProvider.processPayment(getPaymentProviderConfigurationID(instanceManager), invoice, req, res, user, uriParser, new GenericFlowPaymentCallback(instanceManager, getSiteName(instanceManager), callback, req));
			
			if (res.isCommitted()) {
				return null;
			}
			
			if (paymentResponse != null) {
				
				if (paymentResponse.getPaymentType() == PaymentType.DIRECT || paymentResponse.getPaymentType() == PaymentType.INVOICE) {
					
					if (invoice.getTotalSum().compareTo(paymentResponse.getPaymentAmount()) != 0) {
						
						log.error("Got payment response for " + instanceManager + " but payment amount " + paymentResponse.getPaymentAmount().toPlainString() + " does not match invoice amount " + invoice.getTotalSum().toPlainString());
						
						//TODO continue submit and add event log warning or abort submit and notify managers?
					}
					
					if (paymentResponse.getPaymentType() == PaymentType.DIRECT) {
						
						log.info("User " + user + " completed payment of flow instance " + instanceManager);
						
						Map<String, String> eventAttributes = new LinkedHashMap<String, String>(paymentResponse.getPaymentAttributes());
						
						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_DIRECT_PAYMENT_ATTRIBUTE, "true");
						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_PAYED_AMOUNT_ATTRIBUTE, paymentResponse.getPaymentAmount().toPlainString());
						
						String eventDetails = onlinePaymentText + " " + paymentProvider.getProviderName() + " (" + paymentResponse.getPaymentAmount().toPlainString() + " " + paymentResponse.getCurrencyCode() + ")";
						callback.paymentComplete(instanceManager, req, user, true, eventDetails, eventAttributes);
						
						sendDirectPaymentNotifications(paymentSettings, invoice, instanceManager);
						
					} else if (paymentResponse.getPaymentType() == PaymentType.INVOICE) {
						
						log.info("User " + user + " selected invoice payment for flow instance " + instanceManager);
						
						Map<String, String> eventAttributes = new LinkedHashMap<String, String>(paymentResponse.getPaymentAttributes());
						
						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_DIRECT_PAYMENT_ATTRIBUTE, "false");
						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_PAYED_AMOUNT_ATTRIBUTE, paymentResponse.getPaymentAmount().toPlainString());
						
						String eventDetails = paymentSettings.getInvoicePaymentEventDescription() + " (" + paymentResponse.getPaymentAmount().toPlainString() + " " + paymentResponse.getCurrencyCode() + ")";
						callback.paymentComplete(instanceManager, req, user, true, eventDetails, eventAttributes);
						
						sendInvoiceNotifications(paymentSettings, invoice, instanceManager);
					}
					
					res.sendRedirect(callback.getPaymentSuccessURL(instanceManager, req));
					res.getOutputStream().flush();
					return null;
					
				} else {
					
					throw new Exception("Unknown payment type: " + paymentResponse + " for flow instance " + instanceManager);
				}
			}
			
		} catch (ValidationException e) {
			
			validationErrors = new ArrayList<ValidationError>();
			validationErrors.addAll(e.getErrors());
		}
		
		return showPaymentForm(req, res, user, uriParser, invoice, paymentSettings, instanceManager, new GenericFlowPaymentCallback(instanceManager, getSiteName(instanceManager), callback, req), validationErrors);
	}
	
	private String getSiteName(FlowInstanceManager instanceManager) {

		String siteName = null;
		
		SiteProfile profile = flowAdminModule.getSiteProfile(instanceManager);
		
		if(profile != null) {
			
			siteName = profile.getName();
		}
		
		return siteName;
	}

	@EventListener(channel = PaymentProvider.class)
	public void processDelayedPaymentEvent(DelayedPaymentEvent paymentEvent, EventSource source) {
		
		Invoice invoice = paymentEvent.getInvoice();
		PaymentResponse paymentResponse = paymentEvent.getPaymentResponse();
		User user = paymentEvent.getUser();
		
		try {
			if ("flowinstance".equals(paymentEvent.getSourceType())) {
				
				Integer flowInstanceID = paymentEvent.getSourceID();
				FlowInstance flowInstance = flowAdminModule.getFlowInstance(flowInstanceID);
				
				if (flowInstance == null) {
					
					//TODO send email for missing flow instance
					log.error("Got delayed payment event for flow instance ID " + flowInstanceID + " which does not exist");
					
				} else if (flowInstance.getStatus() != null && flowInstance.getStatus().getContentType() == ContentType.WAITING_FOR_PAYMENT) {
					
					if (invoice.getTotalSum().compareTo(paymentResponse.getPaymentAmount()) != 0) {
						
						log.error("Got delayed payment event for " + flowInstance + " but payment amount " + paymentResponse.getPaymentAmount().toPlainString() + " does not match invoice amount " + invoice.getTotalSum().toPlainString());
						
						//TODO continue submit and add event log warning or abort submit and notify managers?
					}
					
					//TODO Use firstSubmitted instead?
					Status nextStatus = flowInstance.getFlow().getDefaultState(flowBrowserModule.getSubmitActionID());

					if (!flowInstance.getStatus().equals(nextStatus)) {
						
						log.info("User " + user + " completed delayed payment of flow instance " + flowInstance);

						ImmutableFlowInstanceManager instanceManager = new ImmutableFlowInstanceManager(flowInstance, flowAdminModule.getQueryHandler(), null, new DefaultInstanceMetadata(null), null);
						FlowFamilyPaymentSettings paymentSettings = getPaymentSettings(instanceManager.getFlowInstance().getFlow().getFlowFamily());

						Map<String, String> eventAttributes = new LinkedHashMap<String, String>(paymentResponse.getPaymentAttributes());

						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_DIRECT_PAYMENT_ATTRIBUTE, "true");
						eventAttributes.put(Constants.FLOW_INSTANCE_EVENT_PAYED_AMOUNT_ATTRIBUTE, paymentResponse.getPaymentAmount().toPlainString());

						String eventDetails = onlinePaymentText + " " + paymentProvider.getProviderName() + " (" + paymentResponse.getPaymentAmount().toPlainString() + " " + paymentResponse.getCurrencyCode() + ")";
						SiteProfile siteProfile = flowAdminModule.getSiteProfile(instanceManager.getFlowInstance());

						flowBrowserModule.paymentComplete(instanceManager, null, user, siteProfile, flowBrowserModule.getSubmitActionID(), true, eventDetails, eventAttributes);

						sendDirectPaymentNotifications(paymentSettings, invoice, instanceManager);

					} else {
						
						log.error("Got delayed payment event for " + flowInstance + " which is already in the submitted status " + nextStatus);
					}
					
				} else {
					
					log.warn("Got delayed payment event for " + flowInstance + " which is not in the waiting for payment status " + flowInstance.getStatus());
				}
			}
		} catch (Exception e) {
			
			log.error("Error handling delayed payment for " + paymentEvent, e);
		}
	}
	
	protected void sendInvoiceNotifications(FlowFamilyPaymentSettings paymentSettings, Invoice invoice, FlowInstanceManager instanceManager) {
		
		if (!CollectionUtils.isEmpty(paymentSettings.getInvoicePaymentNotificationAddresses())) {
			
			if (invoiceEmailNotificationSubject == null || invoiceEmailNotificationMessage == null) {
				
				log.error("Email subject/message templates for invoice payment notifications have not been set, unable to send notifications regarding flow instance " + instanceManager);
				return;
			}
			
			if (standardFlowNotificationHandler == null) {
				
				log.error("StandardFlowNotificationHandler not found, unable to send invoice payment notifications regarding flow instance " + instanceManager);
				return;
			}
			
			sendNotifications(paymentSettings.getInvoicePaymentNotificationAddresses(), paymentSettings, invoice, instanceManager, invoiceEmailNotificationSubject, invoiceEmailNotificationMessage);
		}
	}
	
	protected void sendDirectPaymentNotifications(FlowFamilyPaymentSettings paymentSettings, Invoice invoice, FlowInstanceManager instanceManager) {
		
		if (!CollectionUtils.isEmpty(paymentSettings.getDirectPaymentNotificationAddresses())) {
			
			if (directPaymentEmailNotificationSubject == null || directPaymentEmailNotificationMessage == null) {
				
				log.error("Email subject/message templates for direct payment notifications have not been set, unable to send notifications regarding flow instance " + instanceManager);
				return;
			}
			
			if (standardFlowNotificationHandler == null) {
				
				log.error("StandardFlowNotificationHandler not found, unable to send direct payment notifications regarding flow instance " + instanceManager);
				return;
			}
			
			sendNotifications(paymentSettings.getDirectPaymentNotificationAddresses(), paymentSettings, invoice, instanceManager, directPaymentEmailNotificationSubject, directPaymentEmailNotificationMessage);
		}
	}
	
	protected void sendNotifications(List<String> recipients, FlowFamilyPaymentSettings paymentSettings, Invoice invoice, FlowInstanceManager instanceManager, String subjectTemplate, String messageTemplate) {
		
		TagReplacer tagReplacer = new TagReplacer(4);
		
		tagReplacer.addTagSource(FLOWINSTANCE_TAG_SOURCE_FACTORY.getTagSource((FlowInstance) instanceManager.getFlowInstance()));
		tagReplacer.addTagSource(FLOW_TAG_SOURCE_FACTORY.getTagSource((Flow) instanceManager.getFlowInstance().getFlow()));
		tagReplacer.addTagSource(new SingleTagSource("$invoice.totalSum", invoice.getTotalSum()));
		tagReplacer.addTagSource(new SingleTagSource("$paymentProvider.name", paymentProvider.getProviderName()));
		
		if (flowInstanceAdminModule != null) {
			
			tagReplacer.addTagSource(new SingleTagSource("$flowInstance.url", standardFlowNotificationHandler.getFlowInstanceAdminModuleAlias(instanceManager.getFlowInstance()) + "/overview/" + instanceManager.getFlowInstanceID()));
		}
		
		String subject = tagReplacer.replace(subjectTemplate);
		String message = tagReplacer.replace(messageTemplate);
		
		for (String address : recipients) {
			
			try {
				log.info("Sending notification to address " + address + " regarding flow instance " + instanceManager);
				
				SimpleEmail email = new SimpleEmail(systemInterface.getEncoding());
				
				email.addRecipient(address);
				email.setMessageContentType(SimpleEmail.HTML);
				email.setSenderName(standardFlowNotificationHandler.getEmailSenderName(instanceManager.getFlowInstance()));
				email.setSenderAddress(standardFlowNotificationHandler.getEmailSenderAddress(instanceManager.getFlowInstance()));
				email.setSubject(subject);
				email.setMessage(message);
				
				systemInterface.getEmailHandler().send(email);
				
			} catch (Exception e) {
				
				log.error("Error sending email to address " + address + " regarding payment of flow instance " + instanceManager);
			}
		}
	}
	
	private Invoice getInvoice(FlowFamilyPaymentSettings paymentSettings, FlowInstanceManager instanceManager, User user) {
		
		List<SimpleInvoiceLine> invoiceLines = new ArrayList<SimpleInvoiceLine>();
		
		List<InvoiceLine> queryInvoiceLines = FlowInstanceUtils.getPaymentInvoiceLines(instanceManager);
		
		if (queryInvoiceLines != null) {
			
			for (InvoiceLine invoiceLine : queryInvoiceLines) {
				
				invoiceLines.add(new SimpleInvoiceLine(invoiceLine.getQuantity(), invoiceLine.getUnitPrice(), invoiceLine.getDescription(), invoiceLine.getUnitMeasure()));
			}
		}
		
		ImmutableFlow flow = instanceManager.getFlowInstance().getFlow();
		
		StringBuilder description = new StringBuilder();
		
		if (useOnlyInvoiceDescriptionSuffix && !StringUtils.isEmpty(paymentSettings.getInvoiceDescriptionSuffix())) {
			
			description.append(AttributeTagUtils.replaceTags(paymentSettings.getInvoiceDescriptionSuffix(), instanceManager.getFlowInstance().getAttributeHandler(), false));
			
		} else {
			
			description.append(invoiceDescriptionCustomer);
			description.append(": ");
			
			List<Contact> contacts = standardFlowNotificationHandler.getContacts(instanceManager.getFlowInstance());
			
			if (!CollectionUtils.isEmpty(contacts)) {
				
				for (Iterator<Contact> it = contacts.iterator(); it.hasNext();) {
					Contact contact = it.next();
					
					if (!StringUtils.isEmpty(contact.getOrganizationName())) {
						
						description.append(contact.getOrganizationName());
						
					} else {
						
						description.append(contact.getFirstname());
						description.append(' ');
						description.append(contact.getLastname());
					}
					
					if (it.hasNext()) {
						description.append('\n');
					}
				}
				
			} else if (user != null) {
				
				description.append(user.getFirstname());
				description.append(' ');
				description.append(user.getLastname());
				
			} else {
				
				description.append(userNotLoggedIn);
			}
			
			description.append('\n');
			description.append(invoiceDescriptionCategory);
			description.append(": ");
			description.append(flow.getFlowType().getName());
			
			if (flow.getCategory() != null) {
				description.append(" ");
				description.append(flow.getCategory().getName());
			}
			
			if (!StringUtils.isEmpty(paymentSettings.getInvoiceDescriptionSuffix())) {
				
				description.append('\n');
				description.append(AttributeTagUtils.replaceTags(paymentSettings.getInvoiceDescriptionSuffix(), instanceManager.getFlowInstance().getAttributeHandler()));
			}
		}
		
		String title;
		
		if(includeFlowIDInTitle) {
			
			title = flow.getFlowID() + "-" + flow.getName();
			
		}else {
			
			title = flow.getName();
		}
		
		return new SimpleInvoice("flowinstance", instanceManager.getFlowInstanceID(), orderIDPrefix + "-" + instanceManager.getFlowInstanceID(), title, description.toString(), invoiceLines, instanceManager.getFlowInstance().getAttributeHandler());
	}
	
	@Override
	public int getPriority() {
		
		return priority;
	}
	
	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {
		
		if (!flow.isPaymentSupportEnabled() || !AccessUtils.checkAccess(user, this.moduleDescriptor)) {
			
			return null;
		}
		
		Document doc = createDocument(req, uriParser, user);
		
		Element showViewElement = doc.createElement("ShowPaymentSettings");
		doc.getDocumentElement().appendChild(showViewElement);
		
		showViewElement.appendChild(flow.toXML(doc));
		
		FlowFamilyPaymentSettings paymentSettings = getPaymentSettings(flow.getFlowFamily());
		
		boolean enabled = false;
		
		if(paymentSettings != null) {
		
			XMLUtils.append(doc, showViewElement, paymentSettings);
			enabled = true;
		}
		
		try {
			ViewFragment paymentProviderViewFragment = paymentProvider.showConfiguration(getPaymentProviderConfigurationID(flow), req, user, uriParser);
			
			showViewElement.appendChild(paymentProviderViewFragment.toXML(doc));
			
			SimpleViewFragment viewFragment = (SimpleViewFragment) viewFragmentTransformer.createViewFragment(doc, true);
			
			return new FlowAdminExtensionShowView(new SimpleViewFragment(viewFragment.getHTML(), viewFragment.getDebugXML(), paymentProviderViewFragment.getScripts(), paymentProviderViewFragment.getLinks()), enabled);
			
		} catch (Exception e) {
			
			log.error("Error getting showConfiguration view fragment from payment provider " + paymentProvider, e);
			
			return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
		}
	}
	
	private FlowFamilyPaymentSettings getPaymentSettings(ImmutableFlowFamily flowFamily) throws SQLException {
		
		HighLevelQuery<FlowFamilyPaymentSettings> query = new HighLevelQuery<FlowFamilyPaymentSettings>();
		
		query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));
		
		return settingsDAO.get(query);
	}
	
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {
		
		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");
		
		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));
		
		doc.appendChild(document);
		
		return doc;
	}
	
	@Override
	public String getExtensionViewTitle() {
		
		return extensionViewTitle;
	}
	
	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {
		
		return moduleDescriptor;
	}
	
	@Override
	public List<LinkTag> getLinkTags() {
		
		return links;
	}
	
	@Override
	public List<ScriptTag> getScriptTags() {
		
		return scripts;
	}
	
	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {
		
		if (flowAdminModule == null && this.flowAdminModule != null) {
			
			this.flowAdminModule.removeExtensionViewProvider(this);
		}
		
		this.flowAdminModule = flowAdminModule;
		
		if (this.flowAdminModule != null) {
			
			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse updateFlowPaymentSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);
		
		if (flow == null) {
			
			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
			
		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {
			
			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}
		
		FlowFamilyPaymentSettings paymentSettings = getPaymentSettings(flow.getFlowFamily());
		
		ValidationException validationException = null;
		ValidationException paymentProviderValidationException = null;
		
		if (req.getMethod().equalsIgnoreCase("POST")) {
			
			try {
				paymentProvider.processPaymentConfiguration(getPaymentProviderConfigurationID(flow), req, user, uriParser);
				
			} catch (ValidationException e) {
				
				paymentProviderValidationException = e;
			}
			
			if (paymentSettings == null) {
				
				paymentSettings = new FlowFamilyPaymentSettings();
				paymentSettings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
			}
			
			try {
				POPULATOR.populate(paymentSettings, req);
				
				if (paymentProviderValidationException == null) {
					
					log.info("User " + user + " updated payment settings for flow " + flow);
					
					settingsDAO.addOrUpdate(paymentSettings, null);
					
					flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#paymentsettings");
					return null;
				}
				
			} catch (ValidationException e) {
				
				validationException = e;
			}
		}
		
		log.info("User " + user + " requesting update payment settings form for flow " + flow);
		
		Document doc = createDocument(req, uriParser, user);
		
		Element paymentSettingsElement = doc.createElement("UpdatePaymentSettings");
		doc.getDocumentElement().appendChild(paymentSettingsElement);
		
		paymentSettingsElement.appendChild(flow.toXML(doc));
		
		XMLUtils.append(doc, paymentSettingsElement, paymentSettings);
		
		XMLUtils.appendNewElement(doc, paymentSettingsElement, "DefaultInvoicePaymentEventDescription", defaultInvoicePaymentEventDescription);
		
		ViewFragment paymentProviderSettingsViewFragment = paymentProvider.showConfigurationForm(getPaymentProviderConfigurationID(flow), req, user, uriParser, paymentProviderValidationException);
		
		if (paymentProviderSettingsViewFragment != null) {
			
			paymentSettingsElement.appendChild(paymentProviderSettingsViewFragment.toXML(doc));
		}
		
		if (validationException != null) {
			
			paymentSettingsElement.appendChild(validationException.toXML(doc));
			paymentSettingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}
		
		SimpleForegroundModuleResponse response = new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
		
		if (paymentProviderSettingsViewFragment != null) {
			
			ViewFragmentUtils.appendLinksAndScripts(response, paymentProviderSettingsViewFragment);
		}
		
		return response;
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse deleteFlowPaymentSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		if(!HTTPUtils.isPost(req)) {
			
			throw new AccessDeniedException("Delete requests using method " + req.getMethod() + " are not allowed.");
		}
		
		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);
		
		if (flow == null) {
			
			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));
			
		} else if (!flowAdminModule.hasFlowAccess(user, flow)) {
			
			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}
		
		FlowFamilyPaymentSettings paymentSettings = getPaymentSettings(flow.getFlowFamily());
		
		if (paymentSettings != null) {
			
			log.info("User " + user + " deleting payment settings for flow " + flow);
			
			settingsDAO.delete(paymentSettings);
			
			paymentProvider.deletePaymentConfiguration(getPaymentProviderConfigurationID(flow.getFlowFamily()));
			
		} else {
			
			log.warn("User " + user + " trying to deleting payment settings for flow " + flow + " which has no payment settings");
		}
		
		flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#paymentsettings");
		
		return null;
	}
	
	@EventListener(channel = FlowFamily.class)
	public void processEvent(CRUDEvent<FlowFamily> event, EventSource eventSource) {

		if (event.getAction() == CRUDAction.DELETE) {

			for (FlowFamily flowFamily : event.getBeans()) {

				log.info("Deleting payment settings for flow family ID: " + flowFamily.getFlowFamilyID());

				try {
					HighLevelQuery<FlowFamilyPaymentSettings> query = new HighLevelQuery<FlowFamilyPaymentSettings>();
					
					query.addParameter(flowFamilyIDParamFactory.getParameter(flowFamily.getFlowFamilyID()));
					
					settingsDAO.delete(query);
					
					paymentProvider.deletePaymentConfiguration(getPaymentProviderConfigurationID(flowFamily));

				} catch (Exception e) {

					log.error("Error deleting payment settings for flow family ID: " + flowFamily.getFlowFamilyID(), e);
				}
			}
		}
	}

	@Override
	public ExtensionLink getPaymentConfigurationExtensionLink(String configurationID) throws PaymentConfigurationIDNotUsed {
		
		Integer flowFamilyID = getFlowFamilyIDFromPaymentProviderConfigurationID(configurationID);
		
		if (flowFamilyID != null) {

			FlowFamily flowFamily = flowAdminModule.getFlowFamily(flowFamilyID);

			if (flowFamily == null) {
				throw new PaymentConfigurationIDNotUsed();
			}

			try {
				FlowFamilyPaymentSettings settings = getPaymentSettings(flowFamily);

				if (settings == null) {
					throw new PaymentConfigurationIDNotUsed();
				}

				Flow latestFlow = flowAdminModule.getLatestFlowVersion(flowFamily);

				return new ExtensionLink(latestFlow.getName(), flowAdminModule.getFullAlias() + "/showflow/" + latestFlow.getFlowID(), null, null);

			} catch (SQLException e) {
				log.error("Error getting FlowFamilyPaymentSettings for configurationID " + configurationID, e);
			}
		}
		
		return null;
	}
	
	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}
}
