package com.nordicpeak.flowengine.paymentprovider.beans;

import javax.servlet.http.HttpServletRequest;

import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.interfaces.FlowPaymentCallback;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.openhierarchy.payment.interfaces.PaymentCallback;


public class GenericFlowPaymentCallback implements PaymentCallback {
	
	private final String paymentFormURL;
	private final String processPaymentURL;
	private final boolean hasPaymentStatus;
	private final String siteName;
	
	public GenericFlowPaymentCallback(FlowInstanceManager instanceManager, String siteName, FlowPaymentCallback callback, HttpServletRequest req) {
		super();
		
		paymentFormURL = callback.getPaymentURL(instanceManager, req);
		processPaymentURL = callback.getPaymentURL(instanceManager, req);
		this.siteName = siteName;
		
		ImmutableStatus paymentStatus = instanceManager.getFlowInstance().getFlow().getDefaultState(FlowBrowserModule.PAYMENT_ACTION_ID);
		hasPaymentStatus = paymentStatus != null && paymentStatus.getContentType() == ContentType.WAITING_FOR_PAYMENT;
	}

	@Override
	public String getPaymentFormURL() {
		return paymentFormURL;
	}
	
	@Override
	public String getProcessPaymentURL() {
		return processPaymentURL;
	}

	@Override
	public boolean supportsDelayedPayments() {
		return hasPaymentStatus;
	}

	@Override
	public String getSiteName() {

		return siteName;
	}
	
}
