<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-ui.js?v=1
		/featherlight/js/featherlight.min.js
		/js/confirmpost.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/css/openhierarchy-jquery-ui.css
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/common/js/queryadmin.js
		/js/jquery.ui.datepicker-sv.js
		/js/internalalternativelists.js
		/js/UserGroupList.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/internalalternativelists.css?v=1
		/css/UserGroupList.css?v=1
	</xsl:variable>
	
	<xsl:template match="Document">

		<div class="contentitem alternativelistprovider errands-wrapper">

			<xsl:apply-templates select="ListAlternativeLists" />
			<xsl:apply-templates select="ShowAlternativeList" />
			<xsl:apply-templates select="AddAlternativeList" />
			<xsl:apply-templates select="UpdateAlternativeList" />
			<xsl:apply-templates select="ShowAlternativeListEvents" />
			<xsl:apply-templates select="ShowAlternativeListFlowUsage" />
			
			<xsl:apply-templates select="ShowCombinedList" />
			<xsl:apply-templates select="AddCombinedList" />
			<xsl:apply-templates select="UpdateCombinedList" />

		</div>

	</xsl:template>
	
	<xsl:template match="ListAlternativeLists">

		<h1><xsl:value-of select="$i18n.ListAlternativeLists" /></h1>

		<xsl:apply-templates select="validationException/validationError"/>
		<xsl:apply-templates select="validationError"/>
		
		<p>
			<xsl:value-of select="$i18n.ListAlternativeLists.Description" />
		</p>
		
		<xsl:choose>
			<xsl:when test="not(AlternativeLists/AlternativeList)">
			
				<span>
					<strong>
						<xsl:value-of select="$i18n.ListAlternativeLists.NoListsFound" />
					</strong>
				</span>
				
			</xsl:when>
			<xsl:otherwise>
			
				<table class="full coloredtable sortabletable oep-table">
					<thead class="sortable">
						<tr>
							<xsl:call-template name="ListTableColumns"/>
						</tr>
					</thead>
					<tbody>
						
						<xsl:apply-templates select="AlternativeLists/AlternativeList" mode="list"/>
						
					</tbody>
				</table>
				
			</xsl:otherwise>
		</xsl:choose>
		
		<xsl:if test="CombinedLists/AlternativeList">
			
			<h2 class="bigmargintop"><xsl:value-of select="$i18n.ListAlternativeLists.CombinedLists" /></h2>
			
			<table class="full coloredtable sortabletable oep-table">
				<thead class="sortable">
					<tr>
						<xsl:call-template name="ListTableColumns"/>
					</tr>
				</thead>
				<tbody>
					
					<xsl:apply-templates select="CombinedLists/AlternativeList" mode="list-combined"/>
					
				</tbody>
			</table>
			
		</xsl:if>

		<xsl:if test="HasFullAccess">
			<div class="floatright marginright bigmargintop">
				<a class="btn btn-green btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/add" title="{$i18n.AddInternal}">
					<xsl:value-of select="$i18n.AddInternal" />
				</a>
			</div>
			
			<xsl:if test="AlternativeLists/AlternativeList">
				<div class="floatright marginright bigmargintop">
					<a class="btn btn-green btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addcombined" title="{$i18n.AddCombined}">
						<xsl:value-of select="$i18n.AddCombined" />
					</a>
				</div>
			</xsl:if>		
		</xsl:if>

	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="list-combined">
	
		<tr>
			<td data-title="{$i18n.Name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showcombined/{listID}">
					<xsl:value-of select="name"/>
				</a>
			</td>
			<td data-title="{$i18n.Size}">
				<xsl:value-of select="count(Alternatives/Alternative)"/>
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/combinedusage/{listID}" title="{$i18n.FlowUsage.Button}">
					<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/magnifying_glass.png" alt="" />
				</a>
				
				<xsl:apply-templates select="." mode="combinedListAdminLinks" />

			</td>
		</tr>
		
	</xsl:template>
	
	<xsl:template name="ShowAlternativeListHeaderExtra">
		
		<xsl:if test="AlternativeList/useDateLimits = 'true'">
		
			<th><xsl:value-of select="$i18n.Alternative.ValidFrom" /></th>
			<th><xsl:value-of select="$i18n.Alternative.ValidTo" /></th>
			
		</xsl:if>
		
	</xsl:template>
	
	<xsl:template name="ShowAlternativeListButtonsExtra">
		
		<a class="btn btn-blue floatright margintop marginright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/export/{AlternativeList/listID}"><xsl:value-of select="$i18n.InternalList.ExportToExcel"/></a>
		
	</xsl:template>
	
	<xsl:template match="Alternative" mode="list">
		
		<tr>
			<td class="paddingright" data-title="{$i18n.Show.Alternative.Name}"><xsl:value-of select="name"/></td>
			<td class="paddingright" data-title="{$i18n.Show.Alternative.Value}"><xsl:value-of select="alternativeID"/></td>
			
			<xsl:if test="../../useDateLimits = 'true'">
				
				<td class="paddingright" data-title="{$i18n.Alternative.ValidFrom}"><xsl:value-of select="validFrom"/></td>
				<td class="paddingright" data-title="{$i18n.Alternative.ValidTo}"><xsl:value-of select="validTo"/></td>
				
			</xsl:if>
		</tr>
			
	</xsl:template>
	
	<xsl:template name="editListDetails">
		
		<xsl:if test="not(AlternativeList/Alternatives)">
			<script type="text/javascript">
				$("#addListForm, #updateListForm").prop("enctype", "multipart/form-data");
			</script>
		
			<div class="floatleft full bigmarginbottom">
				
				<label for="file" class="floatleft full">
					<xsl:value-of select="$i18n.InternalList.ImportFromExcel" />
				</label>
				
				<p class="tiny">
					<xsl:copy-of select="$i18n.InternalList.ImportFromExcel.description" />
				</p>
				
				<div class="floatleft full">
					<input type="file" name="importFile" id="file"/>
				</div>
			</div>
		</xsl:if>
		
		<div class="floatleft full marginbottom">
			<div class="floatleft full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'useDateLimits'"/>
					<xsl:with-param name="name" select="'useDateLimits'"/>
					<xsl:with-param name="value" select="'true'"/> 
					<xsl:with-param name="element" select="AlternativeList" />
				</xsl:call-template>
				
				<label for="useDateLimits">
					<xsl:value-of select="$i18n.useDateLimits" />
				</label>
			</div>
		</div>

		<xsl:call-template name="createAlternativesForm">
			<xsl:with-param name="alternatives" select="AlternativeList/Alternatives/Alternative" />
			<xsl:with-param name="allowFreeTextAlternative" select="'false'" />
			<xsl:with-param name="editExtraValues" select="'true'" />
		</xsl:call-template>
		
		<xsl:call-template name="editListAccessControl" />
		
	</xsl:template>
	
	<xsl:template name="editListAccessControl">
	
		<div>
		
			<div class="full bigmarginbottom">
	
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'adminAccessControlEnabled'" />
					<xsl:with-param name="name" select="'adminAccessControlEnabled'" />
					<xsl:with-param name="element" select="AlternativeList" />
				</xsl:call-template>
							
				<label for="adminAccessControlEnabled">
					<xsl:value-of select="$i18n.AdminAccessControlEnabled" />
				</label>
				
			</div>
	
			<div id="adminContainer" class="full bigmarginbottom hidden">
							
				<label>
					<xsl:value-of select="$i18n.AdminGroups" />
				</label>
			
				<xsl:call-template name="GroupList">
					<xsl:with-param name="connectorURL">
						<xsl:value-of select="/Document/requestinfo/currentURI"/>
						<xsl:text>/</xsl:text>
						<xsl:value-of select="/Document/module/alias"/>
						<xsl:text>/groups</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="name" select="'adminGroup'"/>
					<xsl:with-param name="groups" select="Admins/groups" />
				</xsl:call-template>
							
				<label>
					<xsl:value-of select="$i18n.AdminUsers" />
				</label>
			
				<xsl:call-template name="UserList">
					<xsl:with-param name="connectorURL">
						<xsl:value-of select="/Document/requestinfo/currentURI"/>
						<xsl:text>/</xsl:text>
						<xsl:value-of select="/Document/module/alias"/>
						<xsl:text>/users</xsl:text>
					</xsl:with-param>
					<xsl:with-param name="name" select="'adminUser'"/>
					<xsl:with-param name="users" select="Admins/users" />
				</xsl:call-template>
				
			</div>
			
		</div>
	
	</xsl:template>
	
	<xsl:template name="createAlternativesFormHeader">
		
		<div class="floatright bigmargintop">
			
				<xsl:variable name="options">
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Select"/></name>
						<value></value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Descending"/></name>
						<value>d</value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Ascending"/></name>
						<value>a</value>
					</option>
				</xsl:variable>
			
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="class" select="'sorting'"/>
					<xsl:with-param name="name" select="'sorting'"/>
					<xsl:with-param name="width" select="'130px'"/>
					<xsl:with-param name="labelElementName" select="'name'" />
					<xsl:with-param name="valueElementName" select="'value'" />
					<xsl:with-param name="element" select="exsl:node-set($options)/option" />
				</xsl:call-template>

			</div>
		
			<label class="floatleft" style="width: calc(60% - 37px);"><xsl:value-of select="$i18n.Alternative" /></label>
			<label class="floatleft datelimits" style="width: 20%;"><xsl:value-of select="$i18n.Alternative.ValidFrom" /></label>
			<label class="floatleft datelimits"><xsl:value-of select="$i18n.Alternative.ValidTo" /></label>
			
	</xsl:template>
	
	<xsl:template name="createAlternative">
		
		<xsl:param name="alternativeID" />
		<xsl:param name="sortOrder" />
		<xsl:param name="value" />
		<xsl:param name="xmlValue" />
		<xsl:param name="attributeValue" />
		<xsl:param name="disabled" select="null" />
		<xsl:param name="editExtraValues" select="'false'"/>
		<xsl:param name="validFrom" select="''" />
		<xsl:param name="validTo" select="''" />
		
		<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
		
		<xsl:variable name="requestparameters" select="//requestparameters"/>
		
		<div class="alternative full floatleft margintop marginbottom">
			<img class="vertical-align-middle marginright cursor-move" src="{$commonImagePath}/move.png" title="{$i18n.MoveAlternative}"/>
			
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('alternativeID_', $alternativeID)"/>
				<xsl:with-param name="name" select="'alternativeID'"/>
				<xsl:with-param name="value" select="$alternativeID"/>
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('sortorder_', $alternativeID)" />
				<xsl:with-param name="name" select="concat('sortorder_', $alternativeID)" />
				<xsl:with-param name="value" select="$sortOrder" />
				<xsl:with-param name="class" select="'sortorder'" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('alternative-xml-value_', $alternativeID)" />
				<xsl:with-param name="name" select="concat('alternative-xml-value_', $alternativeID)" />
				<xsl:with-param name="value" select="$xmlValue" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('alternative-attribute-value_', $alternativeID)" />
				<xsl:with-param name="name" select="concat('alternative-attribute-value_', $alternativeID)" />
				<xsl:with-param name="value" select="$attributeValue" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="concat('alternative_', $alternativeID)"/>
				<xsl:with-param name="name" select="concat('alternative_', $alternativeID)"/>
				<xsl:with-param name="title" select="name"/>
				<xsl:with-param name="value" select="$value"/>
				<xsl:with-param name="width" select="'calc(60% - 73px)'"/>
				<xsl:with-param name="class" select="'name marginright'" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="concat('alternative-date-from_', $alternativeID)"/>
				<xsl:with-param name="name" select="concat('alternative-date-from_', $alternativeID)"/>
				<xsl:with-param name="title" select="$i18n.Alternative.ValidFrom"/>
				<xsl:with-param name="value">
					<xsl:choose>
						<xsl:when test="$requestparameters">
							<xsl:value-of select="$requestparameters/parameter[name = concat('alternative-date-from_', $alternativeID)]/value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$validFrom"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:with-param>
				<xsl:with-param name="width" select="'calc(20% - 10px)'"/>
				<xsl:with-param name="class" select="'validfrom datelimits marginleft bigmarginright'" />
				<xsl:with-param name="type" select="'date'" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="concat('alternative-date-to_', $alternativeID)"/>
				<xsl:with-param name="name" select="concat('alternative-date-to_', $alternativeID)"/>
				<xsl:with-param name="title" select="$i18n.Alternative.ValidTo"/>
				<xsl:with-param name="value">
					<xsl:choose>
						<xsl:when test="$requestparameters">
							<xsl:value-of select="$requestparameters/parameter[name = concat('alternative-date-to_', $alternativeID)]/value"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$validTo"/>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:with-param>
				<xsl:with-param name="width" select="'calc(20% - 10px)'"/>
				<xsl:with-param name="class" select="'validto datelimits marginright'" />
				<xsl:with-param name="type" select="'date'" />
				<xsl:with-param name="requestparameters" select="$requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			
			<xsl:variable name="helper"><AlternativeExtraValues/></xsl:variable>
			<xsl:apply-templates select="exsl:node-set($helper)/AlternativeExtraValues">
				<xsl:with-param name="alternativeID" select="$alternativeID"/>
				<xsl:with-param name="disabled" select="$disabled"/>
				<xsl:with-param name="context" select="."/>
			</xsl:apply-templates>
			
			<xsl:if test="$editExtraValues = 'true'">
			
				<a href="#" onclick="openAlternativeModal(this, event)" title="{$i18n.EditAlternativeExtra}">
					<img class="vertical-align-middle marginleft" src="{$commonImagePath}/pen.png"/>
				</a>
			
			</xsl:if>
			
			<a href="#" onclick="deleteAlternative(this, event)" title="{$i18n.DeleteAlternative}">
				<img class="vertical-align-middle marginleft" src="{$commonImagePath}/delete.png"/>
			</a>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="Alternative">
		
		<xsl:call-template name="createAlternative">
			<xsl:with-param name="alternativeID" select="alternativeID" />
			<xsl:with-param name="sortOrder" select="sortIndex" />
			<xsl:with-param name="value" select="name" />
			<xsl:with-param name="xmlValue" select="xmlValue" />
			<xsl:with-param name="attributeValue" select="attributeValue" />
			<xsl:with-param name="editExtraValues" select="'true'" />
			<xsl:with-param name="validFrom" select="validFrom" />
			<xsl:with-param name="validTo" select="validTo" />
		</xsl:call-template>
			
	</xsl:template>
	
	<xsl:template match="ShowCombinedList">

		<div class="floatright">
			
			<xsl:apply-templates select="AlternativeList" mode="combinedListAdminLinks" />
			
		</div>

		<h1>
			<xsl:value-of select="$i18n.ShowCombined" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>
		<xsl:apply-templates select="validationError"/>

		<fieldset>
			<legend><xsl:value-of select="$i18n.CombinedList.Lists"/></legend>
		
			<table class="coloredtable sortabletable oep-table">
				<thead class="sortable">
					<tr>
						<th><xsl:value-of select="$i18n.Name" /></th>
						<th><xsl:value-of select="$i18n.Size" /></th>
					</tr>
				</thead>
				<tbody>
					
					<xsl:apply-templates select="AlternativeList/Lists/AlternativeList" mode="list-combined-lists"/>
					
				</tbody>
			</table>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Show.Alternatives"/></legend>
		
			<label class="bigmarginleft">
				<xsl:value-of select="$i18n.CombinedList.ordering"/>
			</label>
			<div class="bigmarginleft">
				<xsl:choose>
					<xsl:when test="AlternativeList/ordering = 'AZ'">
						<xsl:value-of select="$i18n.CombinedList.ordering.az"/>
					</xsl:when>
					<xsl:when test="AlternativeList/ordering = 'ZA'">
						<xsl:value-of select="$i18n.CombinedList.ordering.za"/>
					</xsl:when>
					<xsl:when test="AlternativeList/ordering = 'LIST_ORDER'">
						<xsl:value-of select="$i18n.CombinedList.ordering.listOrder"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="AlternativeList/ordering"/>
					</xsl:otherwise>
				</xsl:choose>
			</div>
		
			<table class="coloredtable sortabletable oep-table clearboth">
				<thead class="sortable">
					<tr>
						<th><xsl:value-of select="$i18n.Show.Alternative.Name" /></th>
						<th><xsl:value-of select="$i18n.Show.Alternative.Value" /></th>
					</tr>
				</thead>
				<tbody>
					
					<xsl:apply-templates select="AlternativeList/Alternatives/Alternative" mode="list"/>
					<xsl:apply-templates select="Alternatives/Alternative" mode="list"/>
					
				</tbody>
			</table>
			
		</fieldset>
		
		<fieldset>
			<legend><xsl:value-of select="$i18n.Events.Title"/></legend>
			
			<p class="nomargin"><xsl:value-of select="$i18n.Events.Description"/></p>
	
				<xsl:choose>
					<xsl:when test="Events/AlternativeListEvent">
					
						<table class="full coloredtable sortabletable oep-table" cellspacing="0">
							<thead>	
								<tr>
									<th><xsl:value-of select="$i18n.Events.message" /></th>
									<th><xsl:value-of select="$i18n.Events.poster" /></th>
									<th><xsl:value-of select="$i18n.Events.added" /></th>
								</tr>
							</thead>
							<tbody>
							
								<xsl:apply-templates select="Events/AlternativeListEvent" mode="list"/>
								
							</tbody>
						</table>
	
					</xsl:when>
					<xsl:otherwise>
						<p><xsl:value-of select="$i18n.Events.NoEvents"/></p>
					</xsl:otherwise>
				</xsl:choose>
				
				<div class="floatright marginright">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/combinedevents/{AlternativeList/listID}" title="{$i18n.Events.ShowAll}">
						<xsl:value-of select="$i18n.Events.ShowAll"/>
					</a>
				</div>
				
		</fieldset>
		
		<a class="btn btn-blue floatright margintop" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}"><xsl:value-of select="$i18n.Back"/></a>

	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="list-combined-lists">
		
		<tr>
			<td class="paddingright" data-title="{$i18n.Name}"><xsl:value-of select="name"/></td>
			<td class="paddingright" data-title="{$i18n.Size}"><xsl:value-of select="count(Alternatives/Alternative)"/></td>
		</tr>
			
	</xsl:template>
	
	<xsl:template match="AddCombinedList">

		<h1>
			<xsl:value-of select="$i18n.AddCombined" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>

		<form id="addCombinedListForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="editCombinedList"/>

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.Add}" />
			</div>

		</form>

	</xsl:template>
	
	<xsl:template match="UpdateCombinedList">

		<h1>
			<xsl:value-of select="$i18n.UpdateCombined" />
			<xsl:text>:&#160;</xsl:text>
			<xsl:value-of select="AlternativeList/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError"/>
		
		<p class="error tiny"><xsl:value-of select="$i18n.Update.Description" /></p>

		<form id="updateCombinedListForm" method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="editCombinedList"/>

			<div class="floatright margintop clearboth">
				<input type="submit" value="{$i18n.SaveChanges}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template name="editCombinedList">

		<div class="floatleft full bigmarginbottom">
			<label for="name" class="floatleft required"><xsl:value-of select="$i18n.Name" /></label>
			<div class="floatleft full">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'name'"/>
					<xsl:with-param name="name" select="'name'"/>
					<xsl:with-param name="title" select="$i18n.Name"/>
					<xsl:with-param name="element" select="AlternativeList" />
				</xsl:call-template>
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">
			<label for="ordering" class="floatleft required"><xsl:value-of select="$i18n.CombinedList.ordering" /></label>
			<div class="floatleft full">
			
				<xsl:variable name="options">
					<option>
						<name><xsl:value-of select="$i18n.CombinedList.ordering.az"/></name>
						<value>AZ</value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.CombinedList.ordering.za"/></name>
						<value>ZA</value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.CombinedList.ordering.listOrder"/></name>
						<value>LIST_ORDER</value>
					</option>
				</xsl:variable>
				
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="id" select="'ordering'"/>
					<xsl:with-param name="name" select="'ordering'"/>
					<xsl:with-param name="element" select="exsl:node-set($options)/option"/>
					<xsl:with-param name="labelElementName" select="'name'" />
					<xsl:with-param name="valueElementName" select="'value'" />
					<xsl:with-param name="selectedValue" select="AlternativeList/ordering" />
				</xsl:call-template>
				
			</div>
		</div>
		
		<div class="floatleft full bigmarginbottom">

			<label class="floatleft full">
				<xsl:value-of select="$i18n.CombinedList.Lists" />
			</label>
			
			<div id="lists-list" class="floatleft full margintop marginbottom sortable" data-url="{/Document/requestinfo/currentURI}/{/Document/module/alias}/lists">
	
				<xsl:choose>
					<xsl:when test="requestparameters">
					
						<xsl:variable name="requestparameters" select="requestparameters"/>
					
						<xsl:for-each select="requestparameters/parameter[name='listID']/value">
							
							<xsl:variable name="paramName">
								<xsl:text>listName-</xsl:text>
								<xsl:value-of select="."/>
							</xsl:variable>
							
							<xsl:call-template name="AlternativeListSearch">
								<xsl:with-param name="name" select="$requestparameters/parameter[name = $paramName]/value"/>
								<xsl:with-param name="id" select="."/>
								<xsl:with-param name="sortIndex" select="$requestparameters/parameter[name = concat('sortorder-', current()/.)]/value"/>
							</xsl:call-template>
							
						</xsl:for-each>
						
					</xsl:when>
					<xsl:otherwise>
						
						<xsl:apply-templates select="AlternativeList/Lists/AlternativeList" mode="search" />
						
					</xsl:otherwise>
				</xsl:choose>
			</div>
			
			<div id="lists-template" style="display: none;">
				<xsl:call-template name="AlternativeListSearch">
					<xsl:with-param name="name"></xsl:with-param>
					<xsl:with-param name="id"></xsl:with-param>
					<xsl:with-param name="sortIndex"></xsl:with-param>
				</xsl:call-template>
			</div>
			
			<div class="floatleft full ui-widget">
				<xsl:call-template name="createTextField">
					<xsl:with-param name="id" select="'lists-search'" />
					<xsl:with-param name="placeholder" select="$i18n.CombinedList.Lists.SearchPlaceholder" />
				</xsl:call-template>
			</div>

		</div>
		
		<xsl:call-template name="editListAccessControl" />
		
	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="search" name="AlternativeListSearch">
		<xsl:param name="name" select="name"/>
		<xsl:param name="id" select="listID"/>
		<xsl:param name="sortIndex" select="position()"/>
		
		<div class="floatleft hover border full marginbottom border-radius lightbackground">
			
			<div class="floatleft padding marginleft">
				<a>
					<img class="cursor-move" src="{$commonImagePath}/move.png" title="{$i18n.MoveAlternative}"/>
				</a>
			</div>
			
			<h3 class="floatleft marginleft" style="margin-top: 3px">
				<xsl:value-of select="$name" />
			</h3>
			
			<input name="listID" hidden="true" value="{$id}"/>
			<input name="listName-{$id}" hidden="true" value="{$name}"/>
			
			<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
			
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('sortorder-', $id)" />
				<xsl:with-param name="name" select="concat('sortorder-', $id)" />
				<xsl:with-param name="value" select="$sortIndex" />
				<xsl:with-param name="class" select="'sortorder'" />
				<xsl:with-param name="requestparameters" select="//requestparameters" />
			</xsl:call-template>
			
			<div class="floatright padding">
				<a class="floatright marginright" href="#" onclick="if(confirm('{$i18n.DeleteCombiningList.Confirm}: {$name}?'))$(this).closest('div.border-radius').remove(); return false;" title="{$i18n.DeleteCombiningList}: {$name}">
					<img src="{$imgPath}/delete.png"/>
				</a>
			</div>
		</div>
		
	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="adminLinks">
	
		<xsl:choose>
			<xsl:when test="not(HideDeleteLink)">
			
				<xsl:apply-imports/>
			
			</xsl:when>
			<xsl:otherwise>
			
				<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/update/{listID}" title="{$i18n.UpdateList}">
					<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/pen.png" alt="" />
				</a>			
			
			</xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>
	
	<xsl:template match="AlternativeList" mode="combinedListAdminLinks">
	
		<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatecombined/{listID}" title="{$i18n.UpdateList}">
			<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/pen.png" alt="" />
		</a>
	
		<xsl:if test="not(HideDeleteLink)">
			
			<xsl:choose>
				<xsl:when test="../Queries/Flow">
				
					<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteListBlockedInUse}'); return false;" title="{$i18n.DeleteListBlockedInUse}">
						<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/delete_gray.png" alt="" />
					</a>
				
				</xsl:when>
				<xsl:otherwise>
				
					<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletecombined/{listID}" onclick="return confirmHyperlinkPost(this);" title="{$i18n.DeleteList}">
						<img class="alignbottom" src="{/Document/requestinfo/contextpath}/static/f/{/Document/module/sectionID}/{/Document/module/moduleID}/pics/delete.png" alt="" />
					</a>
					
				</xsl:otherwise>
			</xsl:choose>
			
		</xsl:if>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TooFewAlternatives']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.TooFewAlternatives" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ValidToMustBeAfterValidTo']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.alternative" />
			<xsl:text>&#160;</xsl:text>
			<xsl:variable name="id" select="displayName" />
			<xsl:value-of select="../../requestparameters/parameter[name = concat('sortorder_', $id)]/value + 1" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.Alternative.ValidTo" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.validationError.ValidToMustBeAfterValidTo" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.Alternative.ValidFrom" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'CantRemoveOwnAccess']">
		
		<p class="error">
			<xsl:value-of select="$i18n.validationError.CantRemoveOwnAccess" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseRequest']">
		
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.UnableToParseRequest" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='FileSizeLimitExceeded']">
	
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.FileSizeLimitExceeded.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.ValidationError.FileSizeLimitExceeded.part2"/>
			<xsl:value-of select="size"/>
			<xsl:value-of select="$i18n.ValidationError.FileSizeLimitExceeded.part3"/>
			<xsl:value-of select="maxFileSize"/>
			<xsl:value-of select="$i18n.ValidationError.FileSizeLimitExceeded.part4"/>
		</p>
			
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='InvalidFileExtension']">
	
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.InvalidFileExtension.Part1" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="filename" />
			<xsl:text>&#160;</xsl:text>
			<xsl:value-of select="$i18n.ValidationError.InvalidFileExtension.Part2" />

			<xsl:if test="AllowedExtensions">
			
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.ValidationError.InvalidFileExtension.Part3" />
				<xsl:text>&#160;</xsl:text>
				
				<xsl:apply-templates select="AllowedExtensions/Extension" mode="AllowedExtensions"/>
			</xsl:if>
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='UnableToParseFile']">
		
		<p class="error">
			<xsl:value-of select="$i18n.ValidationError.UnableToParseFile.part1"/>
			<xsl:value-of select="filename"/>
			<xsl:value-of select="$i18n.ValidationError.UnableToParseFile.part2"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">
	
		<xsl:variable name="fieldName" select="." />
	
		<xsl:choose>
			<xsl:when test="$fieldName = 'ordering'">
				<xsl:value-of select="$i18n.CombinedList.ordering" />
			</xsl:when>
			<xsl:when test="starts-with($fieldName, 'alternative-date-from_')">
			
				<xsl:value-of select="$i18n.validationError.alternative" />
				<xsl:text>&#160;</xsl:text>
				<xsl:variable name="id" select="substring($fieldName, 23)" />
				<xsl:value-of select="../../../requestparameters/parameter[name = concat('sortorder_', $id)]/value + 1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.Alternative.ValidFrom" />
				
			</xsl:when>
			<xsl:when test="starts-with($fieldName, 'alternative-date-to_')">
			
				<xsl:value-of select="$i18n.validationError.alternative" />
				<xsl:text>&#160;</xsl:text>
				<xsl:variable name="id" select="substring($fieldName, 21)" />
				<xsl:value-of select="../../../requestparameters/parameter[name = concat('sortorder_', $id)]/value + 1" />
				<xsl:text>&#160;</xsl:text>
				<xsl:value-of select="$i18n.Alternative.ValidTo" />
				
				<!-- TODO use improved row identification from AccountModuleTemplates.xsl -->
				
			</xsl:when>
			<xsl:when test="messageKey = 'ErrorImportingExcel'">
				<xsl:value-of select="$i18n.ValidationError.ErrorImportingExcel" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="displayName" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
</xsl:stylesheet>