package com.nordicpeak.flowengine.internalalternativelist;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.DatePopulator;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.interfaces.MutableAlternative;
import com.nordicpeak.flowengine.interfaces.PricedAlternative;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternative;
import com.nordicpeak.flowengine.internalalternativelist.beans.InternalAlternativeList;


public class InternalAlternativePopulator {

	public List<InternalAlternative> populate(InternalAlternativeList list, List<InternalAlternative> currentAlternatives, HttpServletRequest req, List<ValidationError> validationErrors) {
		return populate(list, currentAlternatives, req, 255, validationErrors);
	}
	
	public List<InternalAlternative> populate(InternalAlternativeList list, List<InternalAlternative> currentAlternatives, HttpServletRequest req, int maxLength, List<ValidationError> validationErrors) {
		
		String[] alternativeIDs = req.getParameterValues("alternativeID");
		
		List<InternalAlternative> alternatives = new ArrayList<>();
		
		if (alternativeIDs != null) {
			
			for (String alternativeID : alternativeIDs) {
				
				String name = ValidationUtils.validateParameter("alternative_" + alternativeID, req, true, 1, maxLength, validationErrors);
				
				if (name == null) {
					continue;
				}
				
				String sortOrder = req.getParameter("sortorder_" + alternativeID);
				String xmlValue = ValidationUtils.validateParameter("alternative-xml-value_" + alternativeID, req, false, 1, maxLength, validationErrors);
				String attributeValue = ValidationUtils.validateParameter("alternative-attribute-value_" + alternativeID, req, false, 1, maxLength, validationErrors);
				
				if (NumberUtils.isInt(sortOrder)) {
					
					InternalAlternative alternative = new InternalAlternative();
					
					alternative.setName(name);
					alternative.setSortIndex(NumberUtils.toInt(sortOrder));
					
					if (alternative instanceof PricedAlternative) {
						
						Integer price = ValidationUtils.validateParameter("alternativeprice_" + alternativeID, req, false, NonNegativeStringIntegerPopulator.getPopulator(), validationErrors);
						((PricedAlternative) alternative).setPrice(price);
					}
					
					if (list.isUseDateLimits()) {
						
						Date validFrom = ValidationUtils.validateParameter("alternative-date-from_" + alternativeID, req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
						Date validTo = ValidationUtils.validateParameter("alternative-date-to_" + alternativeID, req, false, DatePopulator.getYearLimitedPopulator(), validationErrors);
						
						if (validFrom != null && validTo != null && validFrom.compareTo(validTo) > 0) {
							
							validationErrors.add(new ValidationError("ValidToMustBeAfterValidTo", alternativeID));
						}
						
						alternative.setValidFrom(validFrom);
						alternative.setValidTo(validTo);
					}
					
					if (NumberUtils.isInt(alternativeID)) {
						
						checkForExistingAlternatives(currentAlternatives, alternative, NumberUtils.toInt(alternativeID));
					}
					
					if (!StringUtils.isEmpty(xmlValue)) {
						
						alternative.setExportXMLValue(xmlValue);
					}
					
					if (!StringUtils.isEmpty(attributeValue)) {
						
						alternative.setAttributeValue(attributeValue);
					}
					
					alternatives.add(alternative);
				}
			}
		}
		
		return alternatives;
	}
	
	public List<InternalAlternative> populate(XMLParser xmlParser, List<ValidationError> errors) throws ValidationException {
		
		List<XMLParser> xmlParsers = xmlParser.getNodes("Alternatives/" + InternalAlternative.class.getSimpleName());
		
		if (!CollectionUtils.isEmpty(xmlParsers)) {
			
			List<InternalAlternative> alternatives = new ArrayList<>();
			
			for (XMLParser parser : xmlParsers) {
				
				InternalAlternative alternative = new InternalAlternative();
				alternative.populate(parser);
				alternatives.add(alternative);
			}
			
			return alternatives;
		}
		
		return null;
	}
	
	protected void checkForExistingAlternatives(List<InternalAlternative> currentAlternatives, InternalAlternative alternative, Integer alternativeID) {
		
		if (!CollectionUtils.isEmpty(currentAlternatives)) {
			
			for (MutableAlternative queryAlternative : currentAlternatives) {
				
				if (queryAlternative.getAlternativeID().equals(alternativeID)) {
					
					alternative.setAlternativeID(alternativeID);
					break;
				}
			}
		}
	}
}
