package com.nordicpeak.flowengine.queries.internalcontactdetails;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;
import com.nordicpeak.flowengine.queries.internalcontactdetails.beans.InternalContactDetailQuery;
import com.nordicpeak.flowengine.queries.internalcontactdetails.beans.SelectedManagerAttribute;
import com.nordicpeak.flowengine.queries.internalcontactdetails.beans.SelectedUserAttribute;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;

public class InternalContactDetailQueryCRUD extends BaseQueryCRUD<InternalContactDetailQuery, InternalContactDetailQueryProviderModule> {

	protected AnnotatedDAOWrapper<InternalContactDetailQuery, Integer> queryDAO;

	public InternalContactDetailQueryCRUD(AnnotatedDAOWrapper<InternalContactDetailQuery, Integer> queryDAO, BeanRequestPopulator<InternalContactDetailQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, InternalContactDetailQueryProviderModule callback) {

		super(InternalContactDetailQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.queryDAO = queryDAO;
	}

	@Override
	protected void appendUpdateFormData(InternalContactDetailQuery bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);

		if (callback.getAttributeDescriptors() != null) {

			XMLUtils.append(doc, updateTypeElement, "AvailiableAttributes", callback.getAttributeDescriptors());
		}

		if (callback.isEnableClosestManager()) {
			XMLUtils.appendNewElement(doc, updateTypeElement, "ClosestManagerEnabled");
		}
	}

	@Override
	protected InternalContactDetailQuery populateFromUpdateRequest(InternalContactDetailQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		InternalContactDetailQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();

		populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		if (!callback.isEnableClosestManager()) {

			query.setShowManagerInfo(false);
			query.setRequireManagerSigning(false);
			query.setSelectedManagerAttributes(null);
		}

		if (callback.getAttributeDescriptors() != null) {

			List<SelectedUserAttribute> selectedAttributes = new ArrayList<SelectedUserAttribute>();

			Integer sortIndex = 1;
			
			for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {

				if ("true".equals(req.getParameter("attribute-" + attributeDescriptor.getName()))) {

					selectedAttributes.add(new SelectedUserAttribute(attributeDescriptor, sortIndex++));
				}
			}

			if(selectedAttributes.isEmpty() && !query.isShowName() && !query.isShowEmail() && !query.isShowUsername()) {
				
				validationErrors.add(new ValidationError("NoUserAttributesChosen"));
			}
			
			query.setSelectedUserAttributes(selectedAttributes);

			if (callback.isEnableClosestManager()) {

				List<SelectedManagerAttribute> selectedManagerAttributes = new ArrayList<SelectedManagerAttribute>();

				sortIndex = 1; 
				
				for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {

					if ("true".equals(req.getParameter("managerattribute-" + attributeDescriptor.getName()))) {
						
						selectedManagerAttributes.add(new SelectedManagerAttribute(attributeDescriptor, sortIndex++));
					}
				}

				query.setSelectedManagerAttributes(selectedManagerAttributes);
			}
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

}
