package com.nordicpeak.flowengine.queries.internalcontactdetails.beans;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLPopulationUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.beans.ContactField;
import com.nordicpeak.flowengine.interfaces.ContactFieldQuery;
import com.nordicpeak.flowengine.interfaces.MultiSignQuery;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.UsersClosestManagerQuery;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.internalcontactdetails.InternalContactDetailQueryProviderModule;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesUtils;
import com.nordicpeak.flowengine.queries.internaluserqueries.interfaces.CommonInternalUserQuery;

@Table(name = "internal_contact_detail_queries")
@XMLElement
public class InternalContactDetailQuery extends BaseQuery implements MultiSignQuery, UsersClosestManagerQuery, CommonInternalUserQuery, ContactFieldQuery {

	private static final long serialVersionUID = -842191226937409416L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showUsername;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerInfo;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerUsername;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerEmail;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<SelectedUserAttribute> selectedUserAttributes;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<SelectedManagerAttribute> selectedManagerAttributes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean requireManagerSigning;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<InternalContactDetailQueryInstance> instances;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@Override
	public void toXSD(Document doc) { //TODO test xsd

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);
		
		InternalUserQueriesUtils.toXSD(this, sequenceElement, doc);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		showName = xmlParser.getPrimitiveBoolean("showName");
		showUsername = xmlParser.getPrimitiveBoolean("showUsername");
		showEmail = xmlParser.getPrimitiveBoolean("showEmail");

		if (xmlParser.getNode("SelectedAttributes") != null) {
			
			selectedUserAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedAttributes/SelectedAttribute", SelectedUserAttribute.class, errors);
			
		} else {
			
			selectedUserAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedUserAttributes/SelectedUserAttribute", SelectedUserAttribute.class, errors);
		}

		showManagerInfo = xmlParser.getPrimitiveBoolean("showManagerInfo");
		requireManagerSigning = xmlParser.getPrimitiveBoolean("requireManagerSigning");

		if (showManagerInfo) {

			showManagerName = xmlParser.getPrimitiveBoolean("showManagerName");
			showManagerUsername = xmlParser.getPrimitiveBoolean("showManagerUsername");
			showManagerEmail = xmlParser.getPrimitiveBoolean("showManagerEmail");

			selectedManagerAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedManagerAttributes/SelectedManagerAttribute", SelectedManagerAttribute.class, errors);
		}

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<InternalContactDetailQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<InternalContactDetailQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return this.getClass().getName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return this.getClass().getName() + queryID;
	}

	@Override
	public List<SelectedUserAttribute> getSelectedUserAttributes() {

		return selectedUserAttributes;
	}

	public void setSelectedUserAttributes(List<SelectedUserAttribute> selectedAttributes) {

		this.selectedUserAttributes = selectedAttributes;
	}

	@Override
	public boolean isShowName() {

		return showName;
	}

	public void setShowName(boolean showName) {

		this.showName = showName;
	}

	@Override
	public boolean isShowUsername() {

		return showUsername;
	}

	public void setShowUsername(boolean showUsername) {

		this.showUsername = showUsername;
	}

	@Override
	public boolean isShowEmail() {

		return showEmail;
	}

	public void setShowEmail(boolean showEmail) {

		this.showEmail = showEmail;
	}

	public boolean hasFields() {

		return showEmail || showName || showUsername || selectedUserAttributes != null || (showManagerInfo && (showManagerName || showManagerEmail || showManagerUsername || selectedManagerAttributes != null));
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	@Override
	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	@Override
	public boolean requiresMultipartSigning() {
		return requireManagerSigning;
	}

	@Override
	public boolean isShowManagerInfo() {
		return showManagerInfo;
	}

	public void setShowManagerInfo(boolean showManagerInfo) {
		this.showManagerInfo = showManagerInfo;
	}

	@Override
	public boolean isRequireManagerSigning() {
		return requireManagerSigning;
	}

	public void setRequireManagerSigning(boolean requireManagerSigning) {
		this.requireManagerSigning = requireManagerSigning;
	}

	@Override
	public boolean isShowManagerName() {
		return showManagerName;
	}

	public void setShowManagerName(boolean showManagerName) {
		this.showManagerName = showManagerName;
	}

	@Override
	public boolean isShowManagerUsername() {
		return showManagerUsername;
	}

	public void setShowManagerUsername(boolean showManagerUsername) {
		this.showManagerUsername = showManagerUsername;
	}

	@Override
	public boolean isShowManagerEmail() {
		return showManagerEmail;
	}

	public void setShowManagerEmail(boolean showManagerEmail) {
		this.showManagerEmail = showManagerEmail;
	}

	@Override
	public List<SelectedManagerAttribute> getSelectedManagerAttributes() {
		return selectedManagerAttributes;
	}

	public void setSelectedManagerAttributes(List<SelectedManagerAttribute> selectedManagerAttributes) {
		this.selectedManagerAttributes = selectedManagerAttributes;
	}

	@Override
	public List<ContactField> getContactFields(QueryHandler queryHandler) {

		InternalContactDetailQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryDescriptor().getQueryTypeID(), InternalContactDetailQueryProviderModule.class);

		List<ContactField> contactFields = new ArrayList<>();

		if (showName) {

			contactFields.add(new ContactField("firstname", queryProvider.getContactFieldDisplayName("firstname")));
			contactFields.add(new ContactField("lastname", queryProvider.getContactFieldDisplayName("lastname")));
		}

		if (showUsername) {

			contactFields.add(new ContactField("username", queryProvider.getContactFieldDisplayName("username")));
		}

		if (showEmail) {

			contactFields.add(new ContactField("email", queryProvider.getContactFieldDisplayName("email")));
		}

		if (selectedUserAttributes != null) {

			selectedUserAttributes.forEach(attribute -> contactFields.add(new ContactField(attribute.getName(), attribute.getDisplayName())));
		}

		if (showManagerInfo) {

			if (showManagerName) {

				contactFields.add(new ContactField("managerFirstname", queryProvider.getContactFieldDisplayName("managerFirstname")));
				contactFields.add(new ContactField("managerLastname", queryProvider.getContactFieldDisplayName("managerLastname")));
			}

			if (showManagerUsername) {

				contactFields.add(new ContactField("managerUsername", queryProvider.getContactFieldDisplayName("managerUsername")));
			}

			if (showManagerEmail) {

				contactFields.add(new ContactField("managerEmail", queryProvider.getContactFieldDisplayName("managerEmail")));
			}

			if (selectedManagerAttributes != null) {

				selectedManagerAttributes.forEach(attribute -> contactFields.add(new ContactField("manager-" + attribute.getName(), queryProvider.getExportManager() + " " + attribute.getDisplayName().toLowerCase())));
			}
		}

		return contactFields;
	}

}
