package com.nordicpeak.flowengine.queries.internaluserquery.beans;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfNotSet;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.annotations.Templated;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLPopulationUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.annotations.TextTagReplace;
import com.nordicpeak.flowengine.beans.ContactField;
import com.nordicpeak.flowengine.interfaces.ContactFieldQuery;
import com.nordicpeak.flowengine.interfaces.MultiSignQuery;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.UserValueQuery;
import com.nordicpeak.flowengine.interfaces.UserValuesQuery;
import com.nordicpeak.flowengine.interfaces.UsersClosestManagerQuery;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesUtils;
import com.nordicpeak.flowengine.queries.internaluserqueries.interfaces.CommonInternalUserQuery;
import com.nordicpeak.flowengine.queries.internaluserquery.InternalUserQueryProviderModule;

@Table(name = "internal_user_queries")
@XMLElement
public class InternalUserQuery extends BaseQuery implements UserValueQuery, UserValuesQuery, UsersClosestManagerQuery, MultiSignQuery, CommonInternalUserQuery, ContactFieldQuery {

	private static final long serialVersionUID = -842191226937409416L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean getGroupsByAttribute;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "internal_user_query_group_getting_attributes", remoteValueColumnName = "name")
	@RequiredIfSet(paramNames = "getGroupsByAttribute")
	@WebPopulate(paramName = "groupGettingAttribute")
	@SplitOnLineBreak
	@XMLElement(childName = "groupGettingAttribute")
	private List<String> groupGettingAttributes;

	@TextTagReplace
	@DAOManaged
	@Templated
	@WebPopulate(maxLength = 65535)
	@XMLElement
	private String noGroupFoundByAttributeErrorDescription;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "internal_user_query_groups", remoteValueColumnName = "groupID")
	@WebPopulate(paramName = "group")
	@RequiredIfNotSet(paramNames = "getGroupsByAttribute")
	@XMLElement(childName = "groupID")
	private List<Integer> groupIDs;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showUsername;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showEmail;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean searchCitizenID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean searchTitle;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean searchDepartment;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean limitSearchToMyEmployees;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean excludeInactiveUsers;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean blockPosterSelection;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerInfo;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerUsername;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showManagerEmail;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<SelectedUserAttribute> selectedUserAttributes;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<SelectedManagerAttribute> selectedManagerAttributes;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean maskCitizenIDLastFourDigits;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useForMultipartSigning;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setMultipartsAsOwners;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean requireManagerSigning;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean lockForOwnerUpdate;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<InternalUserQueryInstance> instances;

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		InternalUserQueriesUtils.toXSD(this, sequenceElement, doc);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		groupGettingAttributes = XMLValidationUtils.validateParameters("groupGettingAttributes", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (groupGettingAttributes != null) {

			getGroupsByAttribute = xmlParser.getPrimitiveBoolean("getGroupsByAttribute");
		}

		noGroupFoundByAttributeErrorDescription = XMLValidationUtils.validateParameter("noGroupFoundByAttributeErrorDescription", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		showName = xmlParser.getPrimitiveBoolean("showName");
		showUsername = xmlParser.getPrimitiveBoolean("showUsername");
		showEmail = xmlParser.getPrimitiveBoolean("showEmail");

		if (xmlParser.getNode("SelectedAttributes") != null) {

			selectedUserAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedAttributes/SelectedAttribute", SelectedUserAttribute.class, errors);

		} else {

			selectedUserAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedUserAttributes/SelectedUserAttribute", SelectedUserAttribute.class, errors);
		}

		showManagerInfo = xmlParser.getPrimitiveBoolean("showManagerInfo");

		if (showManagerInfo) {

			showManagerName = xmlParser.getPrimitiveBoolean("showManagerName");
			showManagerUsername = xmlParser.getPrimitiveBoolean("showManagerUsername");
			showManagerEmail = xmlParser.getPrimitiveBoolean("showManagerEmail");

			selectedManagerAttributes = XMLPopulationUtils.populateBeans(xmlParser, "SelectedManagerAttributes/SelectedManagerAttribute", SelectedManagerAttribute.class, errors);
		}

		searchCitizenID = xmlParser.getPrimitiveBoolean("searchCitizenID");
		searchTitle = xmlParser.getPrimitiveBoolean("searchTitle");
		searchDepartment = xmlParser.getPrimitiveBoolean("searchDepartment");
		limitSearchToMyEmployees = xmlParser.getPrimitiveBoolean("limitSearchToMyEmployees");
		excludeInactiveUsers = xmlParser.getPrimitiveBoolean("excludeInactiveUsers");
		blockPosterSelection = xmlParser.getPrimitiveBoolean("blockPosterSelection");
		maskCitizenIDLastFourDigits = xmlParser.getPrimitiveBoolean("maskCitizenIDLastFourDigits");
		lockForOwnerUpdate = xmlParser.getPrimitiveBoolean("lockForOwnerUpdate");

		useForMultipartSigning = xmlParser.getPrimitiveBoolean("useForMultipartSigning");
		setMultipartsAsOwners = xmlParser.getPrimitiveBoolean("setMultipartsAsOwners");

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public boolean isGetGroupsByAttribute() {

		return getGroupsByAttribute;
	}

	public void setGetGroupsByAttribute(boolean getGroupsByAttribute) {

		this.getGroupsByAttribute = getGroupsByAttribute;
	}

	public List<String> getGroupGettingAttributes() {

		return groupGettingAttributes;
	}

	public void setGroupGettingAttributes(List<String> groupGettingAttributes) {

		this.groupGettingAttributes = groupGettingAttributes;
	}

	public String getNoGroupFoundByAttributeErrorDescription() {

		return noGroupFoundByAttributeErrorDescription;
	}

	public void setNoGroupFoundByAttributeErrorDescription(String noGroupFoundByAttributeErrorDescription) {

		this.noGroupFoundByAttributeErrorDescription = noGroupFoundByAttributeErrorDescription;
	}

	public List<Integer> getGroupIDs() {

		return groupIDs;
	}

	public void setGroupIDs(List<Integer> groupIDs) {

		this.groupIDs = groupIDs;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<InternalUserQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<InternalUserQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return this.getClass().getName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return this.getClass().getName() + queryID;
	}

	@Override
	public List<SelectedUserAttribute> getSelectedUserAttributes() {

		return selectedUserAttributes;
	}

	public void setSelectedUserAttributes(List<SelectedUserAttribute> selectedAttributes) {

		this.selectedUserAttributes = selectedAttributes;
	}

	@Override
	public boolean isShowName() {

		return showName;
	}

	public void setShowName(boolean showName) {

		this.showName = showName;
	}

	@Override
	public boolean isShowUsername() {

		return showUsername;
	}

	public void setShowUsername(boolean showUsername) {

		this.showUsername = showUsername;
	}

	@Override
	public boolean isShowEmail() {

		return showEmail;
	}

	public void setShowEmail(boolean showEmail) {

		this.showEmail = showEmail;
	}

	public boolean hasFields() {

		return showEmail || showName || showUsername || selectedUserAttributes != null;
	}

	public boolean isMaskCitizenIDLastFourDigits() {

		return maskCitizenIDLastFourDigits;
	}

	public void setMaskCitizenIDLastFourDigits(boolean maskCitizenIDLastFourDigits) {

		this.maskCitizenIDLastFourDigits = maskCitizenIDLastFourDigits;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	@Override
	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public boolean isSearchCitizenID() {

		return searchCitizenID;
	}

	public boolean isSearchTitle() {

		return searchTitle;
	}

	public boolean isSearchDepartment() {

		return searchDepartment;
	}

	@Override
	public boolean requiresMultipartSigning() {

		return useForMultipartSigning;
	}

	public boolean isUseForMultipartSigning() {

		return useForMultipartSigning;
	}

	public void setUseForMultipartSigning(boolean useForMultipartSigning) {

		this.useForMultipartSigning = useForMultipartSigning;
	}

	public boolean isSetMultipartsAsOwners() {

		return setMultipartsAsOwners;
	}

	public void setSetMultipartsAsOwners(boolean setMultipartsAsOwners) {

		this.setMultipartsAsOwners = setMultipartsAsOwners;
	}

	public void setSearchCitizenID(boolean searchCitizenID) {

		this.searchCitizenID = searchCitizenID;
	}

	public void setSearchTitle(boolean searchTitle) {

		this.searchTitle = searchTitle;
	}

	public void setSearchDepartment(boolean searchDepartment) {

		this.searchDepartment = searchDepartment;
	}

	public boolean isLimitSearchToMyEmployees() {

		return limitSearchToMyEmployees;
	}

	public void setLimitSearchToMyEmployees(boolean limitSearchToMyEmployees) {

		this.limitSearchToMyEmployees = limitSearchToMyEmployees;
	}

	public boolean isExcludeInactiveUsers() {

		return excludeInactiveUsers;
	}

	public void setExcludeInactiveUsers(boolean excludeInactiveUsers) {

		this.excludeInactiveUsers = excludeInactiveUsers;
	}

	@Override
	public boolean isShowManagerInfo() {

		return showManagerInfo;
	}

	public void setShowManagerInfo(boolean showManagerInfo) {

		this.showManagerInfo = showManagerInfo;
	}

	@Override
	public boolean isShowManagerName() {

		return showManagerName;
	}

	public void setShowManagerName(boolean showManagerName) {

		this.showManagerName = showManagerName;
	}

	@Override
	public boolean isShowManagerUsername() {

		return showManagerUsername;
	}

	public void setShowManagerUsername(boolean showManagerUsername) {

		this.showManagerUsername = showManagerUsername;
	}

	@Override
	public boolean isShowManagerEmail() {

		return showManagerEmail;
	}

	public void setShowManagerEmail(boolean showManagerEmail) {

		this.showManagerEmail = showManagerEmail;
	}

	@Override
	public boolean isRequireManagerSigning() {

		return requireManagerSigning;
	}

	public void setRequireManagerSigning(boolean requireManagerSigning) {

		this.requireManagerSigning = requireManagerSigning;
	}

	public boolean isLockForOwnerUpdate() {

		return lockForOwnerUpdate;
	}

	public void setLockForOwnerUpdate(boolean lockForOwnerUpdate) {

		this.lockForOwnerUpdate = lockForOwnerUpdate;
	}

	@Override
	public List<SelectedManagerAttribute> getSelectedManagerAttributes() {

		return selectedManagerAttributes;
	}

	public void setSelectedManagerAttributes(List<SelectedManagerAttribute> selectedManagerAttributes) {

		this.selectedManagerAttributes = selectedManagerAttributes;
	}

	public boolean isBlockPosterSelection() {

		return blockPosterSelection;
	}

	public void setBlockPosterSelection(boolean blockPosterSelection) {

		this.blockPosterSelection = blockPosterSelection;
	}

	@Override
	public List<ContactField> getContactFields(QueryHandler queryHandler) {

		InternalUserQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryDescriptor().getQueryTypeID(), InternalUserQueryProviderModule.class);

		List<ContactField> contactFields = new ArrayList<>();

		if (showName) {

			contactFields.add(new ContactField("firstname", queryProvider.getContactFieldDisplayName("firstname")));
			contactFields.add(new ContactField("lastname", queryProvider.getContactFieldDisplayName("lastname")));
		}

		if (showUsername) {

			contactFields.add(new ContactField("username", queryProvider.getContactFieldDisplayName("username")));
		}

		if (showEmail) {

			contactFields.add(new ContactField("email", queryProvider.getContactFieldDisplayName("email")));
		}

		if (selectedUserAttributes != null) {

			selectedUserAttributes.forEach(attribute -> contactFields.add(new ContactField(attribute.getName(), attribute.getDisplayName())));
		}

		if (showManagerInfo) {

			if (showManagerName) {

				contactFields.add(new ContactField("managerFirstname", queryProvider.getContactFieldDisplayName("managerFirstname")));
				contactFields.add(new ContactField("managerLastname", queryProvider.getContactFieldDisplayName("managerLastname")));
			}

			if (showManagerUsername) {

				contactFields.add(new ContactField("managerUsername", queryProvider.getContactFieldDisplayName("managerUsername")));
			}

			if (showManagerEmail) {

				contactFields.add(new ContactField("managerEmail", queryProvider.getContactFieldDisplayName("managerEmail")));
			}

			if (selectedManagerAttributes != null) {

				selectedManagerAttributes.forEach(attribute -> contactFields.add(new ContactField("manager-" + attribute.getName(), queryProvider.getExportManager() + " " + attribute.getDisplayName().toLowerCase())));
			}
		}

		return contactFields;
	}
}
