package com.nordicpeak.lmsearch.beans;

import java.util.List;

import org.apache.log4j.Logger;

import se.unlogic.standardutils.json.JsonArray;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPoint;
import com.vividsolutions.jts.geom.Point;
import com.vividsolutions.jts.geom.Polygon;

public abstract class BaseLMResponse {
	
	protected Logger log = Logger.getLogger(this.getClass());
	
	public JsonNode getJSONForGeometry(Geometry geometry) {
		
		return getJSONForGeometry(geometry, log);
	}
	
	public static JsonNode getJSONForGeometry(Geometry geometry, Logger log) {
		
		JsonArray coordinatesArray = new JsonArray();
		
		if (geometry instanceof Point) {
			
			Point point = (Point) geometry;
			Coordinate coordinate = point.getCoordinate();
			
			coordinatesArray.addNode(coordinate.x);
			coordinatesArray.addNode(coordinate.y);
			
			if (!Double.isNaN(coordinate.z)) {
				
				coordinatesArray.addNode(coordinate.z);
			}
			
		} else if (geometry instanceof MultiPoint) {
			
			for (Coordinate coordinate : geometry.getCoordinates()) {
				
				JsonArray coordinateArray = new JsonArray();
				coordinateArray.addNode(coordinate.x);
				coordinateArray.addNode(coordinate.y);
				
				if (!Double.isNaN(coordinate.z)) {
					
					coordinateArray.addNode(coordinate.z);
				}
				
				coordinatesArray.addNode(coordinateArray);
			}
		
//		} else if (geometry instanceof LineString) {
			
			//TODO support LineString
			
		} else if (geometry instanceof Polygon) {
			
			Polygon polygon = (Polygon) geometry;
			
			JsonArray polygonArray = new JsonArray();
			
			for (Coordinate coordinate : polygon.getExteriorRing().getCoordinates()) {
				
				JsonArray coordinateArray = new JsonArray();
				coordinateArray.addNode(coordinate.x);
				coordinateArray.addNode(coordinate.y);
				
				if (!Double.isNaN(coordinate.z)) {
					
					coordinateArray.addNode(coordinate.z);
				}
				
				polygonArray.addNode(coordinateArray);
			}
			
			coordinatesArray.addNode(polygonArray);
			
			for (int i = 0; i < polygon.getNumInteriorRing(); i++) {
				
				polygonArray = new JsonArray();
				
				for (Coordinate coordinate : polygon.getInteriorRingN(i).getCoordinates()) {
					
					JsonArray coordinateArray = new JsonArray();
					coordinateArray.addNode(coordinate.x);
					coordinateArray.addNode(coordinate.y);
					
					if (!Double.isNaN(coordinate.z)) {
						
						coordinateArray.addNode(coordinate.z);
					}
					
					polygonArray.addNode(coordinateArray);
				}
				
				coordinatesArray.addNode(polygonArray);
			}
			
		} else {
			
			log.error("Unknown geometry type: " + geometry.getGeometryType());
		}
		
		JsonObject object = new JsonObject();
		
		object.putField("coordinates", coordinatesArray);
		object.putField("type", geometry.getGeometryType());
		
		return object;
	}
	
	@Override
	public String toString() {
	
		return toJson().toString();
	}
	
	public abstract JsonNode toJson();
	
	public static JsonNode listToJson(List<? extends BaseLMResponse> baseLMResponses) {
		
		JsonArray rows = new JsonArray(baseLMResponses.size());
		
		for (BaseLMResponse placeName : baseLMResponses) {
			
			rows.addNode(placeName.toJson());
		}
		
		return rows;
	}
	
	public static JsonNode listToFeatureCollectionJson(List<? extends BaseLMResponse> baseLMResponses) {
		
		JsonArray features = new JsonArray(baseLMResponses.size());
		
		for (BaseLMResponse placeName : baseLMResponses) {
			features.addNode(placeName.toJson());
		}
		
		JsonObject object = new JsonObject();
		
		object.putField("features", features);
		object.putField("type", "FeatureCollection");
		
		return object;
	}
	
}
