package com.nordicpeak.lmsearch.beans;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.Map.Entry;

import com.vividsolutions.jts.geom.Geometry;

import flexjson.JsonNumber;
import se.unlogic.standardutils.json.JsonNode;
import se.unlogic.standardutils.json.JsonObject;
import se.unlogic.standardutils.reflection.ReflectionUtils;

public class HeightResponse extends BaseLMResponse {
	
	public static final Field JSON_NUMBER_INPUT = ReflectionUtils.getField(JsonNumber.class, "input");
	
	private String type;
	private Geometry geometry;
	private Map<String, Object> properties;
	private Map<String, Object> crs;
	
	static {
		JSON_NUMBER_INPUT.setAccessible(true);
	}
	
	public String getType() {
		return type;
	}
	
	public void setType(String type) {
		this.type = type;
	}
	
	public Geometry getGeometry() {
		return geometry;
	}
	
	public void setGeometry(Geometry geometry) {
		this.geometry = geometry;
	}
	
	public Map<String, Object> getProperties() {
		return properties;
	}
	
	public void setProperties(Map<String, Object> properties) {
		this.properties = properties;
	}
	
	public Map<String, Object> getCrs() {
		return crs;
	}
	
	public void setCrs(Map<String, Object> crs) {
		this.crs = crs;
	}
	
	private String getStringValueFromMapObject(Object object) {
		
		if (object instanceof JsonNumber) {
			
			JsonNumber number = (JsonNumber) object;
			try {
				return (String) JSON_NUMBER_INPUT.get(number);
				
			} catch (Exception e) {
				log.error("", e);
			}
		}
		
		return object.toString();
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public JsonNode toJson() {
		
		JsonObject jsonNode = new JsonObject();
		
		jsonNode.putField("type", "Feature");
		jsonNode.putField("geometry", getJSONForGeometry(geometry));
		
		JsonObject propertiesNode = new JsonObject();
		
		for (Entry<String, Object> entry : properties.entrySet()) {
			
			propertiesNode.putField(entry.getKey(), getStringValueFromMapObject(entry.getValue()));
		}
		
		jsonNode.putField("properties", propertiesNode);
		
		JsonObject crsPropertiesNode = new JsonObject();
		
		for (Entry<String, Object> entry : crs.entrySet()) {
			
			if (entry.getValue() instanceof String) {
				
				crsPropertiesNode.putField(entry.getKey(), getStringValueFromMapObject(entry.getValue()));
				
			} else {
				
				Map<String, String> map = (Map<String, String>) entry.getValue();
				
				JsonObject crsInnerPropertiesNode = new JsonObject();
				
				for (Entry<String, String> entry2 : map.entrySet()) {
					
					crsInnerPropertiesNode.putField(entry2.getKey(), getStringValueFromMapObject(entry2.getValue()));
				}
				
				crsPropertiesNode.putField(entry.getKey(), crsInnerPropertiesNode);
			}
		}
		
		jsonNode.putField("crs", crsPropertiesNode);
		
		return jsonNode;
	}
	
}
