package com.nordicpeak.flowengine.managerflowtest;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.GroupMultiListSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.Breadcrumb;
import se.unlogic.hierarchy.core.beans.SimpleAccessInterface;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.ModuleConfigurationException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.AccessInterface;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.events.EventListener;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLink;
import se.unlogic.hierarchy.core.utils.extensionlinks.ExtensionLinkProvider;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.serialization.SerializationUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SessionUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.BaseFlowBrowserModule;
import com.nordicpeak.flowengine.BaseFlowModule;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.FlowBrowserModule;
import com.nordicpeak.flowengine.FlowInstanceAdminModule;
import com.nordicpeak.flowengine.beans.DefaultInstanceMetadata;
import com.nordicpeak.flowengine.beans.EvaluatorDescriptor;
import com.nordicpeak.flowengine.beans.ExtensionView;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowFamily;
import com.nordicpeak.flowengine.beans.FlowForm;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.beans.FlowType;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.RequestMetadata;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.enums.ShowMode;
import com.nordicpeak.flowengine.exceptions.FlowEngineException;
import com.nordicpeak.flowengine.exceptions.evaluation.EvaluationException;
import com.nordicpeak.flowengine.exceptions.evaluationprovider.EvaluationProviderException;
import com.nordicpeak.flowengine.exceptions.flow.FlowDefaultStatusNotFound;
import com.nordicpeak.flowengine.exceptions.flow.FlowNoLongerAvailableException;
import com.nordicpeak.flowengine.exceptions.flow.FlowNotAvailiableInRequestedFormat;
import com.nordicpeak.flowengine.exceptions.flowinstance.InvalidFlowInstanceStepException;
import com.nordicpeak.flowengine.exceptions.flowinstance.MissingQueryInstanceDescriptor;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.DuplicateFlowInstanceManagerIDException;
import com.nordicpeak.flowengine.exceptions.flowinstancemanager.FlowInstanceManagerClosedException;
import com.nordicpeak.flowengine.exceptions.queryinstance.QueryRequestException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToGetQueryInstanceShowHTMLException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToResetQueryInstanceException;
import com.nordicpeak.flowengine.exceptions.queryinstance.UnableToSaveQueryInstanceException;
import com.nordicpeak.flowengine.exceptions.queryprovider.QueryProviderException;
import com.nordicpeak.flowengine.interfaces.FlowBrowserExtensionViewProvider;
import com.nordicpeak.flowengine.interfaces.FlowInstanceAccessController;
import com.nordicpeak.flowengine.interfaces.FlowProcessCallback;
import com.nordicpeak.flowengine.interfaces.Icon;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowFamily;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.managers.ManagerResponse;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager.FlowInstanceManagerRegistery;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class ManagerFlowTestModule extends BaseFlowBrowserModule implements EventListener<CRUDEvent<?>>, ExtensionLinkProvider, FlowProcessCallback {

	protected static final List<Field> LIST_FLOWS_IGNORED_FIELDS = Arrays.asList(FlowType.ALLOWED_ADMIN_GROUPS_RELATION, FlowType.ALLOWED_QUERIES_RELATION, FlowType.ALLOWED_ADMIN_USERS_RELATION, FlowType.CATEGORIES_RELATION, Flow.STATUSES_RELATION, Flow.DEFAULT_FLOW_STATE_MAPPINGS_RELATION, Flow.STEPS_RELATION);

	protected static final ValidationError FLOW_FAMILY_NOT_FOUND_VALIDATION_ERROR = new ValidationError("FlowFamilyNotFound");

	protected static final ValidationError FLOW_FAMILY_ACCESS_DENIED_VALIDATION_ERROR = new ValidationError("FlowFamilyAccessDenied");

	protected static final FlowInstanceAccessController GLOBAL_ACCESS_CONTROLLER = new GlobalAccessController(true);
	private static final FlowInstanceAccessController MANAGER_ACCESS_CONTROLLER = new ManagerAccessController(true);

	private static final FlowInstanceAccessController SUBMITTED_GLOBAL_ACCESS_CONTROLLER = new GlobalAccessController(false);
	private static final FlowInstanceAccessController SUBMITTED_MANAGER_ACCESS_CONTROLLER = new ManagerAccessController(false);

	@ModuleSetting(allowsNull = true)
	@GroupMultiListSettingDescriptor(name = "Global access groups", description = "Groups with access to test all flows")
	protected List<Integer> globalAccessGroups;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "CKEditor connector module alias", description = "The full alias of the CKEditor connector module (relative from the contextpath). Leave empty if you do not want to activate file manager for CKEditor")
	protected String ckConnectorModuleAlias;

	@XSLVariable(prefix = "java.")
	protected String hiddenQueryText = "(hidden)";
	
	@InstanceManagerDependency(required = true)
	protected FlowAdminModule flowAdminModule;
	
	@InstanceManagerDependency
	protected FlowBrowserModule flowBrowserModule;

	protected FlowInstanceAdminModule flowInstanceAdminModule;

	protected StaticContentModule staticContentModule;

	protected AccessInterface globalAccess;

	protected FlowEngineDAOFactory daoFactory;

	protected ExtensionLink overviewExtensionLink;

	@SuppressWarnings("rawtypes")
	private static final Class[] EVENT_LISTENER_CLASSES = new Class[] { Flow.class, Step.class, QueryDescriptor.class, EvaluatorDescriptor.class };

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		eventHandler.addEventListener(CRUDEvent.class, this, EVENT_LISTENER_CLASSES);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		super.moduleConfigured();

		this.globalAccess = new SimpleAccessInterface(globalAccessGroups, null);

		generateOverviewExtensionLink(staticContentModule);
	}

	@Override
	public void unload() throws Exception {

		eventHandler.removeEventListener(CRUDEvent.class, this, EVENT_LISTENER_CLASSES);

		if (flowInstanceAdminModule != null) {

			flowInstanceAdminModule.removeOverviewExtensionLinkProvider(this);
		}

		super.unload();
	}

	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return list(req, res, user, uriParser, (ValidationError) null);
	}

	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, ValidationError validationError) throws ModuleConfigurationException, SQLException {

		return list(req, res, user, uriParser, CollectionUtils.getGenericSingletonList(validationError));
	}

	@Override
	public ForegroundModuleResponse list(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, List<ValidationError> validationErrors) throws ModuleConfigurationException, SQLException {

		log.info("User " + user + " listing flows");

		Document doc = this.createDocument(req, uriParser, user);

		Element listFlowsElement = doc.createElement("ListFlows");
		doc.getDocumentElement().appendChild(listFlowsElement);

		XMLGeneratorDocument generatorDocument = new XMLGeneratorDocument(doc);
		generatorDocument.setIgnoredFields(LIST_FLOWS_IGNORED_FIELDS);

		Collection<Flow> flows = flowAdminModule.getCachedFlows();

		if (!AccessUtils.checkAccess(user, globalAccess)) {

			//Check access and append flows
			for (Flow flow : flows) {

				if(flow.isLatestVersion() && hasAccess(user, flow.getFlowFamily())) {

					Element flowElement = flow.toXML(generatorDocument);

					flowAdminModule.appendFamilyInformation(doc, flowElement, flow, flows);

					listFlowsElement.appendChild(flowElement);
				}
			}

		} else {

			for (Flow flow : flows) {

				if (flow.isLatestVersion()) {
					Element flowElement = flow.toXML(generatorDocument);

					flowAdminModule.appendFamilyInformation(doc, flowElement, flow, flows);

					listFlowsElement.appendChild(flowElement);
				}
			}
		}

		if (validationErrors != null) {

			XMLUtils.append(doc, listFlowsElement, validationErrors);
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	@WebPublic(alias = "family")
	public ForegroundModuleResponse showFlowFamily(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return showFlowFamily(req, res, user, uriParser, null);
	}

	public ForegroundModuleResponse showFlowFamily(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, ValidationError validationError) throws Throwable {

		Integer flowFamilyID = uriParser.getInt(2);

		FlowFamily flowFamily;

		if (flowFamilyID == null || (flowFamily = flowAdminModule.getFlowFamily(flowFamilyID)) == null) {

			return list(req, res, user, uriParser, FLOW_FAMILY_NOT_FOUND_VALIDATION_ERROR);

		} else if (!AccessUtils.checkAccess(user, globalAccess) && !hasAccess(user, flowFamily)) {

			return list(req, res, user, uriParser, FLOW_FAMILY_ACCESS_DENIED_VALIDATION_ERROR);
		}

		log.info("User " + user + " listing flow versions in flow family " + flowFamily);

		Document doc = this.createDocument(req, uriParser, user);

		Element flowVersionsElement = doc.createElement("FlowVersions");

		doc.getDocumentElement().appendChild(flowVersionsElement);

		flowVersionsElement.appendChild(flowFamily.toXML(doc));

		XMLUtils.append(doc, flowVersionsElement, flowAdminModule.getFlowVersions(flowFamily));

		if (validationError != null) {

			XMLUtils.append(doc, flowVersionsElement, validationError);
			flowVersionsElement.appendChild(RequestUtils.getRequestParameters(req, doc, "flowID"));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	@WebPublic(alias = "overview")
	public ForegroundModuleResponse showFlowOverview(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer flowID;
		Flow flow;

		if (uriParser.size() == 3 && (flowID = uriParser.getInt(2)) != null && (flow = getFlow(flowID)) != null) {

			checkFlowFamilyAccess(user, flow.getFlowFamily());
			
			if ("true".equals(req.getParameter("preview"))) {
				
				return testFlowAllSteps(req, res, user, uriParser);
			}

			if (skipOverview(flow, req, res, "testflow")) {

				return null;
			}
			
			Document doc = this.createDocument(req, uriParser, user);

			Element showFlowOverviewElement = doc.createElement("ShowFlowOverview");

			doc.getDocumentElement().appendChild(showFlowOverviewElement);

			flow.setHasTextTags(TextTagReplacer.hasTextTags(flow));
			flow.getFlowFamily().setHasTextTags(TextTagReplacer.hasTextTags(flow.getFlowFamily()));
			flow.setHasFileURLs(FCKUtils.hasAbsoluteFileUrls(flow));
			flow.setHasRelativeURLs(URLRewriter.hasAbsoluteLinkUrls(flow));

			showFlowOverviewElement.appendChild(flow.toXML(doc, this.getCurrentSiteProfile(req, user, uriParser, flow.getFlowFamily()), getAbsoluteFileURL(uriParser, flow), req));

			List<ExtensionView> extensionViews = null;

			if (flowBrowserModule != null && !CollectionUtils.isEmpty(flowBrowserModule.getFlowBrowserExtensionViewProviders())) {

				extensionViews = new ArrayList<ExtensionView>(flowBrowserModule.getFlowBrowserExtensionViewProviders().size());

				for (FlowBrowserExtensionViewProvider extensionProvider : flowBrowserModule.getFlowBrowserExtensionViewProviders()) {

					try {
						ExtensionView extension = extensionProvider.getFlowOverviewExtensionView(flow, req, user, uriParser);

						if (extension != null) {

							extensionViews.add(extension);
						}

					} catch (Exception e) {

						log.error("Error getting extension view from provider " + extensionProvider, e);
					}
				}

				XMLUtils.append(doc, showFlowOverviewElement, "ExtensionViews", extensionViews);
			}

			SimpleForegroundModuleResponse response = new SimpleForegroundModuleResponse(doc, flow.getName(), this.getDefaultBreadcrumb());

			if (!CollectionUtils.isEmpty(extensionViews)) {

				for (ExtensionView extensionView : extensionViews) {

					if (!CollectionUtils.isEmpty(extensionView.getViewFragment().getLinks())) {
						response.addLinks(extensionView.getViewFragment().getLinks());
					}

					if (!CollectionUtils.isEmpty(extensionView.getViewFragment().getScripts())) {

						response.addScripts(extensionView.getViewFragment().getScripts());
					}
				}
			}

			return response;
		}

		return list(req, res, user, uriParser, FlowAdminModule.FLOW_NOT_FOUND_VALIDATION_ERROR);
	}

	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse testFlow(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer flowID = null;
		MutableFlowInstanceManager instanceManager;

		try {
			if (uriParser.size() == 3 && (flowID = NumberUtils.toInt(uriParser.get(2))) != null && getBareFlow(flowID) != null) {

				//Create new instance or get instance from session
				instanceManager = getUnsavedMutableFlowInstanceManager(flowID, getAccessController(user), req.getSession(true), user, user, null, uriParser, req, true, false, false, false, BaseFlowModule.OWNER_REQUEST_METADATA);

				if (instanceManager == null) {

					log.info("User " + user + " requested non-existing flow with ID " + flowID + ", listing flows");
					return list(req, res, user, uriParser, FlowAdminModule.FLOW_NOT_FOUND_VALIDATION_ERROR);
				}

			} else {

				log.info("User " + user + " requested invalid URL, listing flows");
				return list(req, res, user, uriParser, FlowAdminModule.INVALID_LINK_VALIDATION_ERROR);
			}

		} catch (FlowNoLongerAvailableException e) {

			log.info("User " + user + " requested flow " + e.getFlow() + " which is no longer available.");
			return list(req, res, user, uriParser, FlowAdminModule.FLOW_NO_LONGER_AVAILABLE_VALIDATION_ERROR);

		} catch (FlowNotAvailiableInRequestedFormat e) {

			log.info("User " + user + " requested flow " + flowID + " which is not availiable in the requested format.");
			return list(req, res, user, uriParser, FlowAdminModule.FLOW_NOT_AVAILIABLE_IN_REQUESTED_FORMAT_VALIDATION_ERROR);

		} catch (FlowEngineException e) {

			log.error("Unable to get flow instance manager for flowID " + flowID + " requested by user " + user, e);
			return list(req, res, user, uriParser, FlowAdminModule.ERROR_GETTING_FLOW_INSTANCE_MANAGER_VALIDATION_ERROR);
		}

		try {
			return processFlowRequest(instanceManager, this, getAccessController(user), req, res, user, user, uriParser, false, BaseFlowModule.OWNER_REQUEST_METADATA);

		} catch (FlowInstanceManagerClosedException e) {

			log.info("User " + user + " requested flow instance manager for flow instance " + e.getFlowInstance() + " which has already been closed. Removing flow instance manager from session.");

			removeMutableFlowInstanceManagerFromSession(instanceManager, req.getSession(false));

			redirectToMethod(req, res, "/testflow/" + flowID);

			return null;
		}
	}
	
	public ForegroundModuleResponse testFlowAllSteps(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception, Throwable {

		Integer flowID = null;

		try {
			if (uriParser.size() == 3 && (flowID = NumberUtils.toInt(uriParser.get(2))) != null && getBareFlow(flowID) != null) {
				
				//Create new instance or get instance from session, only needed for query request processors
				MutableFlowInstanceManager normalTestFlowInstanceManager = getUnsavedMutableFlowInstanceManager(flowID, getAccessController(user), req.getSession(true), user, user, null, uriParser, req, true, false, false, false, BaseFlowModule.OWNER_REQUEST_METADATA);

				if (normalTestFlowInstanceManager == null) {

					log.info("User " + user + " requested non-existing flow with ID " + flowID + ", listing flows");
					return list(req, res, user, uriParser, FLOW_NOT_FOUND_VALIDATION_ERROR);
				}

				//Create new temporary instance with no evaluators

				Flow flow = getFlow(flowID);

				if (flow == null) {

					log.info("User " + user + " requested non-existing flow with ID " + flowID + ", listing flows");

					return list(req, res, user, uriParser, FLOW_NOT_FOUND_VALIDATION_ERROR);

				} else if (!flow.isInternal() || flow.getSteps() == null) {

					throw new FlowNotAvailiableInRequestedFormat(flowID);
				}

				for (Step step : flow.getSteps()) {

					if (step.getQueryDescriptors() != null) {
						for (QueryDescriptor queryDescriptor : step.getQueryDescriptors()) {

							queryDescriptor.setEvaluatorDescriptors(null);

							if (queryDescriptor.getDefaultQueryState() == QueryState.HIDDEN) {

								queryDescriptor.setDefaultQueryState(QueryState.VISIBLE);
								queryDescriptor.setName(queryDescriptor.getName() + " " + hiddenQueryText);
							}
						}
					}
				}

				SiteProfile profile = getCurrentSiteProfile(req, user, uriParser, flow.getFlowFamily());

				getAccessController(user).checkNewFlowInstanceAccess(flow, user, profile);

				log.info("Creating new instance of flow " + flow + " for user " + user);

				InstanceMetadata instanceMetadata = new DefaultInstanceMetadata(profile);

				MutableFlowInstanceManager instanceManager = new MutableFlowInstanceManager(flow, queryHandler, evaluationHandler, getNewInstanceManagerID(user), req, user, user, instanceMetadata, BaseFlowModule.OWNER_REQUEST_METADATA, getAbsoluteFileURL(uriParser, flow));
				
				try {
					log.info("User " + user + " requested testFlowAllSteps of flow instance " + instanceManager.getFlowInstance());
		
					List<ManagerResponse> managerResponses = instanceManager.getFullFormHTML(queryHandler, req, user, user, getMutableQueryRequestBaseURL(req, normalTestFlowInstanceManager), BaseFlowModule.OWNER_REQUEST_METADATA);
		
					Document doc = createDocument(req, uriParser, user);
					Element flowInstanceManagerPreviewElement = doc.createElement("FlowInstanceManagerAllStepsForm");
					doc.getDocumentElement().appendChild(flowInstanceManagerPreviewElement);
		
					flowInstanceManagerPreviewElement.appendChild(instanceManager.getFlowInstance().toXML(doc));
		
					XMLUtils.append(doc, flowInstanceManagerPreviewElement, "ManagerResponses", managerResponses);
		
					appendFormData(doc, flowInstanceManagerPreviewElement, instanceManager, req, user);
		
					SimpleForegroundModuleResponse moduleResponse = new SimpleForegroundModuleResponse(doc, instanceManager.getFlowInstance().getFlow().getName());
		
					appendLinksAndScripts(moduleResponse, managerResponses);
		
					return moduleResponse;
					
				} catch (FlowInstanceManagerClosedException e) {

					log.info("User " + user + " requested flow instance manager for flow instance " + e.getFlowInstance() + " which has already been closed. Removing flow instance manager from session.");
		
					redirectToMethod(req, res, "/testflowallsteps/" + flowID);
					return null;
					
				} finally {
					
					instanceManager.close(queryHandler);
				}

			} else {

				log.info("User " + user + " requested invalid URL, listing flows");
				return list(req, res, user, uriParser, INVALID_LINK_VALIDATION_ERROR);
			}

		} catch (FlowNoLongerAvailableException e) {

			log.info("User " + user + " requested flow " + e.getFlow() + " which is no longer available.");
			return list(req, res, user, uriParser, FLOW_NO_LONGER_AVAILABLE_VALIDATION_ERROR);

		} catch (FlowNotAvailiableInRequestedFormat e) {

			log.info("User " + user + " requested flow " + flowID + " which is not availiable in the requested format.");
			return list(req, res, user, uriParser, FLOW_NOT_AVAILIABLE_IN_REQUESTED_FORMAT_VALIDATION_ERROR);

		} catch (FlowEngineException e) {

			log.error("Unable to get flow instance manager for flowID " + flowID + " requested by user " + user, e);
			return list(req, res, user, uriParser, ERROR_GETTING_FLOW_INSTANCE_MANAGER_VALIDATION_ERROR);
		}
	}

	@WebPublic(alias = "submitted")
	public ForegroundModuleResponse showSubmittedMessage(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws FlowInstanceManagerClosedException, UnableToGetQueryInstanceShowHTMLException, AccessDeniedException, ModuleConfigurationException, SQLException, se.unlogic.hierarchy.core.exceptions.AccessDeniedException {

		Integer flowID;

		if (uriParser.size() == 3 && (flowID = uriParser.getInt(2)) != null) {

			MutableFlowInstanceManager instanceManager = (MutableFlowInstanceManager) SessionUtils.getAttribute(this.getClass().getName() + "-flow-" + flowID, req);

			if (instanceManager != null) {

				FlowInstance flowInstance = (FlowInstance) instanceManager.getFlowInstance();

				Timestamp now = TimeUtils.getCurrentTimestamp();

				flowInstance.setFlowInstanceID(12345);
				flowInstance.setPoster(user);
				flowInstance.setOwners(Collections.singletonList(user));
				flowInstance.setAdded(now);

				if(flowInstance.getEvents() == null){

					flowInstance.setEvents(new ArrayList<FlowInstanceEvent>(1));
				}

				FlowInstanceEvent submittedEvent = new FlowInstanceEvent();

				submittedEvent.setAdded(now);
				submittedEvent.setPoster(user);
				submittedEvent.setEventType(EventType.SUBMITTED);

				flowInstance.getEvents().add(submittedEvent);

				try {
					ForegroundModuleResponse moduleResponse = showFlowInstance(req, res, user, user, uriParser, instanceManager, getSubmittedAccessController(user), this, "FlowInstanceManagerSubmitted", null, ShowMode.SUBMIT, BaseFlowModule.OWNER_REQUEST_METADATA);

					instanceManager.close(queryHandler);

					return moduleResponse;

				} finally {

					SessionUtils.removeAttribute(this.getClass().getName() + "-flow-" + flowID, req);
				}
			}
		}

		log.info("User " + user + " requested invalid URL, listing flows");
		return list(req, res, user, uriParser, INVALID_LINK_VALIDATION_ERROR);
	}
	
	@WebPublic(alias = "mquery")
	public ForegroundModuleResponse processMutableQueryRequest(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws ModuleConfigurationException, SQLException, AccessDeniedException, IOException, FlowDefaultStatusNotFound, EvaluationException, URINotFoundException, QueryRequestException, QueryProviderException, EvaluationProviderException, InvalidFlowInstanceStepException, MissingQueryInstanceDescriptor, DuplicateFlowInstanceManagerIDException, UnableToResetQueryInstanceException {

		return processMutableQueryRequest(req, res, user, user, uriParser, getAccessController(user), false, false, false, BaseFlowModule.OWNER_REQUEST_METADATA);
	}
	
	protected FlowInstanceAccessController getAccessController(User user) {
		
		if (AccessUtils.checkAccess(user, globalAccess)) {
			
			return GLOBAL_ACCESS_CONTROLLER;
			
		} else {
			
			return MANAGER_ACCESS_CONTROLLER;
		}
	}
	
	protected FlowInstanceAccessController getSubmittedAccessController(User user) {
		
		if (AccessUtils.checkAccess(user, globalAccess)) {
			
			return SUBMITTED_GLOBAL_ACCESS_CONTROLLER;
			
		} else {
			
			return SUBMITTED_MANAGER_ACCESS_CONTROLLER;
		}
	}

	@Override
	protected Flow getBareFlow(Integer flowID) throws SQLException {

		return flowAdminModule.getCachedFlow(flowID);
	}

	@Override
	protected Flow getFlow(Integer flowID) {

		return SerializationUtils.cloneSerializable(flowAdminModule.getCachedFlow(flowID));
	}

	@InstanceManagerDependency
	public void setFlowInstanceAdminModule(FlowInstanceAdminModule flowInstanceAdminModule) {

		if (flowInstanceAdminModule != null) {

			flowInstanceAdminModule.addOverviewExtensionLinkProvider(this);

		} else if (this.flowInstanceAdminModule != null) {

			this.flowInstanceAdminModule.removeOverviewExtensionLinkProvider(this);
		}

		this.flowInstanceAdminModule = flowInstanceAdminModule;
	}

	private void generateOverviewExtensionLink(StaticContentModule staticContentModule) {

		if (staticContentModule != null) {

			overviewExtensionLink = new ExtensionLink(moduleDescriptor.getName(), systemInterface.getContextPath() + this.getFullAlias(), staticContentModule.getModuleContentURL(moduleDescriptor) + "/pics/play.png", "top-right");

		} else {

			overviewExtensionLink = null;
		}
	}

	@InstanceManagerDependency(required = true)
	public void setStaticContentModule(StaticContentModule staticContentModule) {

		generateOverviewExtensionLink(staticContentModule);

		this.staticContentModule = staticContentModule;
	}

	@Override
	public ExtensionLink getExtensionLink(User user) {

		if (hasRequiredDependencies) {

			return overviewExtensionLink;
		}

		return null;
	}

	@Override
	protected String getAbsoluteFileURL(URIParser uriParser, Object object) {

		if (ckConnectorModuleAlias != null) {

			return uriParser.getContextPath() + ckConnectorModuleAlias;
		}

		return null;
	}

	@Override
	protected void redirectToSubmitMethod(MutableFlowInstanceManager instanceManager, HttpServletRequest req, HttpServletResponse res) throws IOException {

		redirectToMethod(req, res, "/submitted/" + instanceManager.getFlowID());
	}

	@Override
	protected void flowInstanceSavedAndClosed(FlowInstanceManager instanceManager, HttpServletRequest req, HttpServletResponse res, User user, FlowInstanceEvent event) throws IOException {

		redirectToMethod(req, res, "/family/" + getFlow(instanceManager.getFlowID()).getFlowFamily().getFlowFamilyID());
	}

	@Override
	protected void closeSubmittedFlowInstanceManager(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		SessionUtils.setAttribute(this.getClass().getName() + "-flow-" + instanceManager.getFlowID(), instanceManager, req);
	}

	@Override
	protected String getBaseUpdateURL(HttpServletRequest req, URIParser uriParser, User user, ImmutableFlowInstance flowInstance, FlowInstanceAccessController accessController) {

		if (!accessController.isMutable(flowInstance, user)) {

			return null;
		}

		return req.getContextPath() + uriParser.getFormattedURI();
	}

	@Override
	public String getSignFailURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}
	
	@Override
	public String getStandalonePaymentURL(FlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getPaymentFailURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getSaveAndSubmitURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getSignSuccessURL(MutableFlowInstanceManager instanceManager, HttpServletRequest req) {

		throw new UnsupportedOperationException();
	}

	@Override
	public String getSubmitActionID() {

		return null;
	}

	@Override
	public String getSaveActionID() {

		return null;
	}

	@Override
	public String getPaymentActionID() {

		return null;
	}

	@Override
	public String getMultiSigningActionID() {

		return null;
	}

	@Override
	public void appendFormData(Document doc, Element baseElement, MutableFlowInstanceManager instanceManager, HttpServletRequest req, User user) {

	}

	@Override
	public void appendShowFlowInstanceData(Document doc, Element baseElement, FlowInstanceManager instanceManager, HttpServletRequest req, User user) {

	}

	@Override
	public AccessInterface getAccessInterface() {

		return moduleDescriptor;
	}

	@Override
	protected FlowInstanceEvent save(MutableFlowInstanceManager instanceManager, User user, User poster, HttpServletRequest req, String actionID, EventType eventType, Map<String,String> eventAttributes, RequestMetadata requestMetadata) throws FlowInstanceManagerClosedException, UnableToSaveQueryInstanceException, SQLException, FlowDefaultStatusNotFound {

		return null;
	}

	@Override
	public Breadcrumb getFlowBreadcrumb(ImmutableFlowInstance flowInstance) {

		return new Breadcrumb(this, flowInstance.getFlow().getName(), "/testflow/" + flowInstance.getFlow().getFlowID());
	}

	private void closeInstanceManagers(Flow flow) {

		int closedCount = FlowInstanceManagerRegistery.getInstance().closeInstances(flow, queryHandler);

		if (closedCount > 0) {
			log.info("Closed " + closedCount + " flow instance managers handling instances of flow " + flow);
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public synchronized void processEvent(CRUDEvent<?> event, EventSource source) {

		log.debug("Received crud event regarding " + event.getAction() + " of " + event.getBeans().size() + " beans with " + event.getBeanClass());

		if (Flow.class.isAssignableFrom(event.getBeanClass()) && (event.getAction() != CRUDAction.ADD)) {

			for (Flow flow : (List<Flow>) event.getBeans()) {

				closeInstanceManagers(flow);
			}

		} else if (Step.class.isAssignableFrom(event.getBeanClass())) {

			for (Step step : (List<Step>) event.getBeans()) {

				closeInstanceManagers(step.getFlow());
			}

		} else if (QueryDescriptor.class.isAssignableFrom(event.getBeanClass())) {

			for (QueryDescriptor queryDescriptor : (List<QueryDescriptor>) event.getBeans()) {

				closeInstanceManagers(queryDescriptor.getStep().getFlow());
			}

		} else if (EvaluatorDescriptor.class.isAssignableFrom(event.getBeanClass())) {

			for (EvaluatorDescriptor evaluatorDescriptor : (List<EvaluatorDescriptor>) event.getBeans()) {

				closeInstanceManagers(evaluatorDescriptor.getQueryDescriptor().getStep().getFlow());
			}
		}
	}

	@Override
	public int getPriority() {

		return 0;
	}

	protected void checkFlowFamilyAccess(User user, ImmutableFlowFamily flowFamily) throws AccessDeniedException {

		if (!hasAccess(user, flowFamily) && !AccessUtils.checkAccess(user, globalAccess)) {

			throw new AccessDeniedException("User is not manager for flow family " + flowFamily);
		}
	}

	@Override
	protected Icon getFlowTypeIcon(Integer flowTypeID) {

		return flowAdminModule.getFlowTypeIcon(flowTypeID);
	}

	@Override
	protected Icon getFlowIcon(Integer flowID) {

		return flowAdminModule.getFlowIcon(flowID);
	}
	
	protected boolean hasAccess(User user, ImmutableFlowFamily flowFamily) {
		
		if (flowFamily.getManagerAccess(user) != null) {
			
			return true;
		}
		
		return false;
	}
	
	@WebPublic(toLowerCase = true)
	public ForegroundModuleResponse getFlowForm(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {
		
		if (flowAdminModule == null) {
			
			throw new URINotFoundException(uriParser);
		}
		
		Integer flowFormID;
		Integer flowID;
		Flow flow;
		
		if (uriParser.size() >= 3 && (flowID = uriParser.getInt(2)) != null && (flow = getBareFlow(flowID)) != null) {
			
			checkFlowFamilyAccess(user, flow.getFlowFamily());
			
			if (uriParser.size() == 3) {
				
				if (CollectionUtils.isEmpty(flow.getFlowForms())) {
					
					log.warn("User " + user + " requested PDF form for flow " + flow + " which has no PDF form available.");
					
				} else {
					
					FlowForm flowForm = flow.getFlowForms().get(0);
					flowForm.setFlow(flow);
					
					return flowAdminModule.sendFlowForm(flowForm, req, res, user, uriParser);
				}
				
			} else if (uriParser.size() == 4 && (flowFormID = uriParser.getInt(3)) != null) {
				
				if (CollectionUtils.isEmpty(flow.getFlowForms())) {
					
					log.warn("User " + user + " requested PDF form for flow " + flow + " which has no PDF form available.");
					
				} else {
					
					for (FlowForm flowForm : flow.getFlowForms()) {
						
						if (flowForm.getFlowFormID().equals(flowFormID)) {
							
							flowForm.setFlow(flow);
							
							return flowAdminModule.sendFlowForm(flowForm, req, res, user, uriParser);
						}
					}
				}
			}
		}
		
		throw new URINotFoundException(uriParser);
	}
}
