<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>
	
	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/tablesorter/js/jquery.tablesorter.min.js
	</xsl:variable>
	
	<xsl:variable name="scripts">
		/js/jquery.ui.touch.min.js
		/js/jquery.blockui.js
		/js/flowengine.helpdialog.js
		/js/flowengine.js?v=1
		/js/flowengine.step-navigator.js
		/js/flowengine.tablesorter.js
		/js/managerflowtestmodule.js?v=2
	</xsl:variable>

	<xsl:variable name="links">
		/css/flowengine.css
	</xsl:variable>

	<xsl:template match="Document">
		
		<xsl:choose>
			<xsl:when test="FlowVersions">
			
				<div id="" class="contentitem errands-wrapper">
					<xsl:apply-templates select="FlowVersions" />
				</div>
			
			</xsl:when>
			<xsl:otherwise>
				
			<div id="FlowBrowser" class="contentitem">
				<xsl:apply-templates select="ListFlows" />
				<xsl:apply-templates select="FlowInstanceManagerForm"/>
				<xsl:apply-templates select="FlowInstanceManagerAllStepsForm"/>
				<xsl:apply-templates select="FlowInstanceManagerPreview"/>
				<xsl:apply-templates select="FlowInstanceManagerSubmitted"/>
				<xsl:apply-templates select="ShowFlowOverview"/>
			</div>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>

	<xsl:template match="ListFlows">

		<div class="floatleft marginbottom">	
			<h1><xsl:value-of select="$i18n.Flowslist.title" /></h1>
			
			<xsl:apply-templates select="validationError"/>
			
			<p>
				<xsl:value-of select="$i18n.Flowlist.description" />
			</p>
		</div>

		<div class="floatright marginbottom">
			<label for="name" class="marginright margintop"><xsl:value-of select="$i18n.statusFilter" />:</label>
			<select id="flow-status-filter" class="bigmarginright" style="width: 180px">
				<option value="all"><xsl:value-of select="$i18n.All" /></option>
				<option value="published"><xsl:value-of select="$i18n.Published" /></option>
				<option value="unpublished"><xsl:value-of select="$i18n.Unpublished" /></option>
			</select>
			<xsl:call-template name="AdditionalListFilters" />
			<label for="flow-filter-input" class="bigmarginleft marginright margintop"><xsl:value-of select="$i18n.Filter" />:</label>
			<input type="text" size="20" id="flow-filter-input" class="flow-filter-input" data-tableid="flowlist" />
		</div>
		
		<table id="flowlist" class="full coloredtable oep-table" cellspacing="0">
			<thead class="sortable">
				<tr>
					<th width="25" class="no-sort"></th>
					<th class="default-sort"><xsl:value-of select="$i18n.flowName" /></th>
					<th><xsl:value-of select="$i18n.flowCategory" /></th>
					<th><xsl:value-of select="$i18n.versions" /></th>
					<th width="115"><xsl:value-of select="$i18n.instances" /></th>
				</tr>
			</thead>
			<tbody>
				<xsl:choose>
					<xsl:when test="not(Flow)">
						<tr>
							<td class="icon"></td>
							<td colspan="6">
								<xsl:value-of select="$i18n.noFlowsFound" />
							</td>
						</tr>
					</xsl:when>
					<xsl:otherwise>
						
						<xsl:apply-templates select="Flow[latestVersion = 'true']" mode="list"/>
						
					</xsl:otherwise>
				</xsl:choose>
			</tbody>
		</table>
		
	</xsl:template>

	<xsl:template name="AdditionalListFilters" />
	
	<xsl:template match="Flow" mode="list">
		
		<tr>
			<xsl:attribute name="class">
				<xsl:if test="HasPublishedVersion">published</xsl:if>
			</xsl:attribute>
			<td class="icon">
				<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{flowID}" width="25" alt="" />
			</td>
			<td data-title="{$i18n.flowName}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/family/{FlowFamily/flowFamilyID}"><xsl:value-of select="name" /></a>
			</td>
			<td data-title="{$i18n.flowCategory}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/family/{FlowFamily/flowFamilyID}"><xsl:value-of select="FlowType/name" /></a>
			</td>
			<td data-title="{$i18n.versions}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/family/{FlowFamily/flowFamilyID}"><xsl:value-of select="VersionCount" /></a>
			</td>
			
			<td data-title="{$i18n.instances}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/family/{FlowFamily/flowFamilyID}"><xsl:value-of select="InstanceCount" /></a>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="FlowVersions">
	
		<h1>
			<xsl:value-of select="Flow[position() = last()]/name"/>
		</h1>
		
		<xsl:apply-templates select="validationError"/>
	
		<p><xsl:value-of select="$i18n.versions.description"/></p>

		<form method="get">
			
			<div class="floatleft full">
				
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'no-preview'"/>
						<xsl:with-param name="name" select="'preview'"/>
						<xsl:with-param name="value" select="'false'"/>
						<xsl:with-param name="checked" select="'true'"/>
					</xsl:call-template>
					
					<label for="no-preview">
						<xsl:value-of select="$i18n.TestSetting.Preview.Normal" />
					</label>
				</div>
			</div>
			
			<div class="floatleft full">
				
				<div class="floatleft full">
					<xsl:call-template name="createRadio">
						<xsl:with-param name="id" select="'preview'"/>
						<xsl:with-param name="name" select="'preview'"/>
						<xsl:with-param name="value" select="'true'"/>
					</xsl:call-template>
					
					<label for="preview">
						<xsl:value-of select="$i18n.TestSetting.Preview.Yes" />
					</label>
				</div>
				
				<p class="tiny" style="padding-left: 19px;">
					<xsl:value-of select="$i18n.TestSetting.Preview.description" />
				</p>
			</div>
			
<!-- 			<div class="floatleft full bigmargintop"> -->
				
<!-- 				<div class="floatleft full"> -->
<!-- 					<xsl:call-template name="createCheckbox"> -->
<!-- 						<xsl:with-param name="id" select="'useTestChildren'" /> -->
<!-- 						<xsl:with-param name="name" select="'useTestChildren'" /> -->
<!-- 						<xsl:with-param name="value" select="'true'" /> -->
<!-- 					</xsl:call-template> -->
					
<!-- 					<label for="useTestChildren"> -->
<!-- 						<xsl:value-of select="$i18n.TestSetting.UseTestChildren" /> -->
<!-- 					</label> -->
<!-- 				</div> -->

<!-- 			</div> -->
			
			<table id="flowversionlist" class="full coloredtable sortabletable oep-table" cellspacing="0">
				<thead>
					<tr>
						<th width="25"></th>
						<th><xsl:value-of select="$i18n.name" /></th>
						<th><xsl:value-of select="$i18n.flowCategory" /></th>
						<th><xsl:value-of select="$i18n.version.title" /></th>
						<th width="25"><xsl:value-of select="$i18n.steps" /></th>
						<th width="25"><xsl:value-of select="$i18n.queries" /></th>
						<th width="25"><xsl:value-of select="$i18n.instances" /></th>
						<th width="75"><xsl:value-of select="$i18n.status" /></th>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Flow" mode="list-versions"/>
				</tbody>
			</table>
			
			<br/>
			
			<div class="floatright marginright">
				
				<input type="button" value="{$i18n.Back}" onclick="window.location = '{/Document/requestinfo/currentURI}/{/Document/module/alias}'" class="marginright"/>
			</div>
		
		</form>
	
	</xsl:template>
	
	<xsl:template match="Flow" mode="list-versions">
		
		<tr>
			<td class="icon">
				<img src="{/Document/requestinfo/currentURI}/{/Document/module/alias}/icon/{flowID}" width="25" alt="" />
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="name" /></a>
			</td>
			<td>
				<xsl:if test="Category">
					<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="Category/name" /></a>
				</xsl:if>
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="version" /></a>
			</td>	
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="count(Steps/Step)" /></a>
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="count(Steps/Step/QueryDescriptors/QueryDescriptor)" /></a>
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?"><xsl:value-of select="flowInstanceCount" /></a>
			</td>
			<td>
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/overview/{flowID}?">
				
					<xsl:choose>
						<xsl:when test="enabled = 'false'">
							<xsl:value-of select="$i18n.disabled" />
						</xsl:when>
						<xsl:when test="published = 'true'">
							<xsl:value-of select="$i18n.published" />
						</xsl:when>
						<xsl:otherwise>
							<xsl:value-of select="$i18n.notPublished" />
						</xsl:otherwise>
					</xsl:choose>
					
				</a>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="FlowInstanceManagerAllStepsForm">
	
		<xsl:if test="ManagerResponse/concurrentModificationLock = 'true'">
			<section class="modal error">
					<span data-icon-before="!">
						<a href="{/Document/requestinfo/uri}?close-reopen=1" onclick="return confirm('{$i18n.FlowInstanceConcurrentlyModifiedConfirm}')" title="{$i18n.FlowInstanceConcurrentlyModifiedLinkTitle}">
							<xsl:value-of select="$i18n.FlowInstanceConcurrentlyModified" />
						</a>
					</span>
				</section>
		</xsl:if>
	
		<xsl:if test="ManagerResponse/ValidationErrors != 'true' and not(validationError)">
			<xsl:apply-templates select="lastFlowAction" />
		</xsl:if>
	
		<xsl:apply-templates select="validationError"/>

		<section class="service has-navigator testing">
			
			<xsl:call-template name="createFlowInstanceManagerFormHeader">
				<xsl:with-param name="showSaveButton">false</xsl:with-param>
			</xsl:call-template>
			
			<xsl:for-each select="ManagerResponses/ManagerResponse">
				
				<xsl:variable name="currentStepID" select="currentStepID" />
				<xsl:variable name="currentStepIndex" select="currentStepIndex" />
				
				<div class="service-navigator-wrap">
					<div>
						<ul class="service-navigator primary">
							<xsl:for-each select="../../FlowInstance/Flow/Steps/Step">
							
								<li>
									<xsl:choose>
										<xsl:when test="$currentStepID = stepID"><xsl:attribute name="class">active</xsl:attribute></xsl:when>
										<xsl:when test="$currentStepIndex >= sortIndex">
											<xsl:attribute name="class">
												<xsl:text>completed</xsl:text>
											</xsl:attribute>
										</xsl:when>
									</xsl:choose>
									
									<span data-step="{position()}">
										<xsl:value-of select="name"/>
									</span>
								</li>
								
							</xsl:for-each>
						</ul>
					</div>
				</div>
				
				<div class="section-full">
					
					<div class="queries">
						
						<xsl:apply-templates select="QueryResponses/QueryResponse"/>
						
						<xsl:if test="not(QueryResponses/QueryResponse/HTML)">
							<p><xsl:value-of select="$i18n.NoQueriesInCurrentStep" /></p>
						</xsl:if>
						
					</div>
				
				</div>
				
			</xsl:for-each>
			
			<div id="ajaxLoadingMessage" style="display: none">
				<h1><xsl:value-of select="$i18n.AjaxLoading" />..</h1><p class="tiny"><a href="javascript:void(0)" onclick="cancelAjaxPost()"><xsl:value-of select="$i18n.AjaxCancel" />.</a></p>
			</div>
			
			<div id="submitLoadingMessage" style="display: none">
				<h1><xsl:value-of select="$i18n.SubmitLoading" />..</h1>
			</div>
			
			<div id="ajaxErrorMessage" style="display: none">
				<h1><xsl:value-of select="$i18n.UnExpectedAjaxError" /></h1>
				<p><xsl:value-of select="$i18n.UnExpectedAjaxErrorDescription" /></p>
				<input id="AjaxRetryButton" type="button" value="{$i18n.AjaxRetry}" onclick="retryAjaxPost()" class="marginright btn btn-blue" /> 
				<input id="AjaxReloadButton" type="button" value="{$i18n.AjaxReload}" onclick="reloadCurrentStep()" class="btn btn-blue" />
			</div>
			
		</section>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='FlowFamilyNotFound']">
	
		<p class="error">
			<xsl:value-of select="$i18n.FlowFamilyNotFound"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey='FlowFamilyAccessDenied']">
	
		<p class="error">
			<xsl:value-of select="$i18n.FlowFamilyAccessDenied"/>
		</p>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownError" />
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:value-of select="fieldName"/>
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>
	
</xsl:stylesheet>