package com.nordicpeak.flowengine.queries.manualcheckboxquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.checkboxquery.Columns;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "manual_checkbox_queries")
@XMLElement
public class ManualCheckboxQuery extends FixedAlternativesBaseQuery {
	
	
	private static final long serialVersionUID = -842191226937409416L;
	
	public static final Field ALTERNATIVES_RELATION = ReflectionUtils.getField(ManualCheckboxQuery.class, "alternatives");
	
	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;
	
	@DAOManaged
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<CheckboxAlternative> alternatives;
	
	@WebPopulate(maxLength = 255)
	@DAOManaged
	@XMLElement
	private String freeTextAlternative;
	
	@DAOManaged
	@WebPopulate(maxLength = 10, required = true)
	@XMLElement
	private Columns columns = Columns.ONE;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hidePrintInfoText;
	
	@DAOManaged
	@OneToMany
	@XMLElement
	private List<ManualCheckboxQueryInstance> instances;
	
	public static long getSerialversionuid() {
		
		return serialVersionUID;
	}
	
	@Override
	public Integer getQueryID() {
		
		return queryID;
	}
	
	@Override
	public List<CheckboxAlternative> getAlternatives() {
		
		return alternatives;
	}
	
	public List<ManualCheckboxQueryInstance> getInstances() {
		
		return instances;
	}
	
	public void setInstances(List<ManualCheckboxQueryInstance> instances) {
		
		this.instances = instances;
	}
	
	public void setQueryID(int queryID) {
		
		this.queryID = queryID;
	}
	
	public void setAlternatives(List<CheckboxAlternative> alternatives) {
		
		this.alternatives = alternatives;
	}
	
	@Override
	public String getFreeTextAlternative() {
		
		return freeTextAlternative;
	}
	
	public void setFreeTextAlternative(String freeTextAlternative) {
		
		this.freeTextAlternative = freeTextAlternative;
	}
	
	public Columns getColumns() {
		return columns;
	}
	
	public void setColumns(Columns columns) {
		this.columns = columns;
	}
	
	@Override
	public String toString() {
		
		if (this.queryDescriptor != null) {
			
			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}
		
		return "ManualCheckboxQuery (queryID: " + queryID + ")";
	}
	
	@Override
	public String getXSDTypeName() {
		
		return "ManualCheckboxQuery" + queryID;
	}
	
	@Override
	public void toXSD(Document doc) {
		
		if (this.alternatives != null) {
			
			int preChecked = 0;
			
			for (CheckboxAlternative alternative : alternatives) {
				
				if (alternative.isChecked()) {
					preChecked++;
				}
			}
			
			toXSD(doc, preChecked, this.alternatives.size());
			
		} else {
			
			toXSD(doc, 1);
		}
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		List<ValidationError> errors = new ArrayList<ValidationError>();
		
		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		
		hidePrintInfoText = xmlParser.getPrimitiveBoolean("hidePrintInfoText");
		freeTextAlternative = XMLValidationUtils.validateParameter("freeTextAlternative", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		
		alternatives = ManualCheckboxQueryCRUD.ALTERNATIVES_POPLATOR.populate(xmlParser, errors);
		
		if (!StringUtils.isEmpty(xmlParser.getString("columns"))) {
			
			columns = XMLValidationUtils.validateParameter("columns", xmlParser, true, new EnumPopulator<Columns>(Columns.class), errors);
		}
		
		if (!errors.isEmpty()) {
			
			throw new ValidationException(errors);
		}
		
	}
}
