package com.nordicpeak.flowengine.queries.manualsignquery;

import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.nordicpeak.flowengine.beans.QueryDescriptor;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.IntegerBasedCRUD;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

public class ManualSignFieldCRUD extends IntegerBasedCRUD<ManualSignField, ManualSignQueryProviderModule> {

	protected AnnotatedDAOWrapper<ManualSignField, Integer> manualSignFieldDAO;

	protected static EnumPopulator<FieldLayout> LAYOUT_POPULATOR = new EnumPopulator<FieldLayout>(FieldLayout.class);

	public ManualSignFieldCRUD(AnnotatedDAOWrapper<ManualSignField, Integer> manualSignFieldDAO, BeanRequestPopulator<ManualSignField> populator, String typeElementName, String typeLogName, String listMethodAlias, ManualSignQueryProviderModule callback) {

		super(manualSignFieldDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.manualSignFieldDAO = manualSignFieldDAO;
	}

	@Override
	protected ForegroundModuleResponse beanAdded(ManualSignField bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(ManualSignField bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanDeleted(ManualSignField bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	public ManualSignField getBean(Integer beanID) throws SQLException, AccessDeniedException {

		HighLevelQuery<ManualSignField> query = new HighLevelQuery<ManualSignField>(ManualSignField.MANUAL_SIGN_QUERY_RELATION);

		query.addParameter(manualSignFieldDAO.getParameterFactory().getParameter(beanID));

		ManualSignField manualSignField = manualSignFieldDAO.getAnnotatedDAO().get(query);

		if (manualSignField != null && manualSignField.getQuery() != null) {

			manualSignField.getQuery().init(callback.getFlowAdminModule().getQueryDescriptor(manualSignField.getQuery().getQueryID()), null);

		}

		return manualSignField;

	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, addTypeElement);

	}

	@Override
	protected void appendUpdateFormData(ManualSignField bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		this.appendFormatValidators(doc, updateTypeElement);

	}

	protected void appendFormatValidators(Document doc, Element element) {

		XMLUtils.append(doc, element, callback.getFormatValidators());

	}

	@Override
	protected ManualSignField populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		ManualSignField manualSignField = super.populateFromAddRequest(req, user, uriParser);

		manualSignField.setQuery((ManualSignQuery) req.getAttribute("ManualSignQuery"));

		manualSignField.setSortIndex(this.getCurrentMaxSortIndex(manualSignField.getQuery()) + 1);

		return manualSignField;
	}

	@Override
	protected void checkUpdateAccess(ManualSignField bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());

	}

	@Override
	protected void checkDeleteAccess(ManualSignField bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());

	}

	private int getCurrentMaxSortIndex(ManualSignQuery manualSignQuery) throws SQLException {

		ObjectQuery<Integer> query = new ObjectQuery<Integer>(manualSignFieldDAO.getAnnotatedDAO().getDataSource(), "SELECT MAX(sortIndex) FROM " + manualSignFieldDAO.getAnnotatedDAO().getTableName() + " WHERE queryID = ?", IntegerPopulator.getPopulator());

		query.setInt(1, manualSignQuery.getQueryID());

		Integer sortIndex = query.executeQuery();

		if (sortIndex == null) {

			return 0;
		}

		return sortIndex;
	}

}
