package com.nordicpeak.flowengine.queries.manualsignquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;

@Table(name = "manual_sign_queries")
@XMLElement
public class ManualSignQuery extends BaseQuery {

	private static final long serialVersionUID = -842191226937409416L;

	public static final Field MANUAL_SIGN_FIELDS_RELATION = ReflectionUtils.getField(ManualSignQuery.class, "fields");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private FieldLayout layout;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<ManualSignField> fields;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hidePrintInfoText;

	@DAOManaged(dontUpdateIfNull = true)
	@OneToMany(autoUpdate = true)
	@XMLElement
	private List<ManualSignQueryInstance> instances;

	private Map<Integer, Integer> manualSignFieldConversionMap;

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<ManualSignField> getFields() {

		return fields;
	}

	public List<ManualSignQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<ManualSignQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setFields(List<ManualSignField> alternatives) {

		this.fields = alternatives;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return this.getClass().getSimpleName() + " (queryID: " + queryID + ")";
	}

	public FieldLayout getLayout() {

		return layout;
	}

	public void setLayout(FieldLayout layout) {

		this.layout = layout;
	}

	public Map<Integer, Integer> getManualSignFieldConversionMap() {

		return manualSignFieldConversionMap;
	}

	public void setManualSignFieldConversionMap(Map<Integer, Integer> textFieldConversionMap) {

		this.manualSignFieldConversionMap = textFieldConversionMap;
	}

	@Override
	public String getXSDTypeName() {

		return this.getClass().getSimpleName() + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	public static String generateElementName(String label, ArrayList<String> fieldElementNames) {

		String elementName = XMLUtils.toValidElementName(label);

		if (fieldElementNames.contains(elementName)) {

			int counter = 1;

			while (fieldElementNames.contains(elementName + counter)) {

				counter++;
			}

			elementName += counter;
		}

		fieldElementNames.add(elementName);

		return elementName;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);

		hidePrintInfoText = xmlParser.getPrimitiveBoolean("hidePrintInfoText");
		layout = XMLValidationUtils.validateParameter("layout", xmlParser, true, ManualSignFieldCRUD.LAYOUT_POPULATOR, errors);

		List<XMLParser> xmlParsers = xmlParser.getNodes("Fields/ManualSignField");

		if (xmlParsers != null) {

			fields = new ArrayList<ManualSignField>();

			for (XMLParser parser : xmlParsers) {

				ManualSignField textField = new ManualSignField();

				textField.populate(parser);

				textField.setQuery(this);

				fields.add(textField);

			}

		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}

	}

}
