package com.nordicpeak.flowengine.queries.manualsignquery;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.formatvalidation.FormatValidator;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class ManualSignQueryProviderModule extends BaseQueryProviderModule<ManualSignQueryInstance>implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	protected String fieldLayoutNewLine = "This variable should be set by your stylesheet";

	@XSLVariable(prefix = "java.")
	protected String fieldLayoutFloat = "This variable should be set by your stylesheet";

	private AnnotatedDAO<ManualSignQuery> queryDAO;
	private AnnotatedDAO<ManualSignQueryInstance> queryInstanceDAO;

	private ManualSignQueryCRUD queryCRUD;
	private ManualSignFieldCRUD manualSignFieldCRUD;

	private QueryParameterFactory<ManualSignQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<ManualSignQueryInstance, Integer> queryInstanceIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, ManualSignQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(ManualSignQuery.class);
		queryInstanceDAO = daoFactory.getDAO(ManualSignQueryInstance.class);

		queryCRUD = new ManualSignQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<ManualSignQuery>(ManualSignQuery.class), "ManualSignQuery", "query", null, this);
		manualSignFieldCRUD = new ManualSignFieldCRUD(daoFactory.getDAO(ManualSignField.class).getWrapper(Integer.class), new AnnotatedRequestPopulator<ManualSignField>(ManualSignField.class), "ManualSignField", "manualsignfield", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ManualSignQuery query = new ManualSignQuery();

		query.setLayout(FieldLayout.FLOAT);

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		ManualSignQuery query = new ManualSignQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		List<Integer> oldManualSignFieldIDs = new ArrayList<Integer>();

		if (query.getFields() != null) {

			for (ManualSignField manualSignField : query.getFields()) {

				oldManualSignFieldIDs.add(manualSignField.getManualSignFieldID());

				manualSignField.setManualSignFieldID(null);
			}

		}

		this.queryDAO.add(query, transactionHandler, null);

		if (query.getFields() != null) {

			Map<Integer, Integer> manualSignFieldConversionMap = new HashMap<Integer, Integer>(oldManualSignFieldIDs.size());

			int index = 0;

			for (Integer oldTextFieldID : oldManualSignFieldIDs) {

				manualSignFieldConversionMap.put(oldTextFieldID, query.getFields().get(index).getManualSignFieldID());

				index++;
			}

			query.setManualSignFieldConversionMap(manualSignFieldConversionMap);

		}

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		ManualSignQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ManualSignQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		ManualSignQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new ManualSignQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private ManualSignQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<ManualSignQuery> query = new HighLevelQuery<ManualSignQuery>(ManualSignQuery.MANUAL_SIGN_FIELDS_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private ManualSignQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<ManualSignQuery> query = new HighLevelQuery<ManualSignQuery>(ManualSignQuery.MANUAL_SIGN_FIELDS_RELATION);

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private ManualSignQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<ManualSignQueryInstance> query = new HighLevelQuery<ManualSignQueryInstance>(ManualSignQueryInstance.QUERY_RELATION, ManualSignQuery.MANUAL_SIGN_FIELDS_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(ManualSignQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(ManualSignQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		queryInstance.getQueryInstanceDescriptor().setPopulated(true);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.show(req, res, user, uriParser);
	}

	@WebPublic(alias = "updatequery")
	public ForegroundModuleResponse updateQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "addmanualsignfield")
	public ForegroundModuleResponse addManualSignField(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		ManualSignQuery query = this.queryCRUD.getRequestedBean(req, res, user, uriParser, null);

		if (query != null) {

			this.checkUpdateQueryAccess(user, query);

			req.setAttribute("ManualSignQuery", query);

			return manualSignFieldCRUD.add(req, res, user, uriParser);

		}

		throw new URINotFoundException(uriParser);
	}

	@WebPublic(alias = "updatemanualsignfield")
	public ForegroundModuleResponse updateManualSignField(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return manualSignFieldCRUD.update(req, res, user, uriParser);
	}

	@WebPublic(alias = "deletemanualsignfield")
	public ForegroundModuleResponse deleteManualSignField(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return manualSignFieldCRUD.delete(req, res, user, uriParser);
	}

	@WebPublic(alias = "sortmanualsignfields")
	public ForegroundModuleResponse sortManualSignFields(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return queryCRUD.sort(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		ManualSignQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		ManualSignQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	public String getFieldLayoutName(FieldLayout layout) {

		if (layout.equals(FieldLayout.FLOAT)) {
			return fieldLayoutFloat;
		} else if (layout.equals(FieldLayout.NEW_LINE)) {
			return fieldLayoutNewLine;
		}

		return layout.toString();

	}

	public void checkUpdateQueryAccess(User user, ManualSignQuery query) throws AccessDeniedException, SQLException {

		flowAdminModule.checkFlowStructureManipulationAccess(user, (Flow) query.getQueryDescriptor().getStep().getFlow());

	}

	public void redirectToQueryConfig(ManualSignQuery query, HttpServletRequest req, HttpServletResponse res) throws IOException {

		res.sendRedirect(req.getContextPath() + this.getFullAlias() + "/config/" + query.getQueryID());

	}

	public List<FormatValidator> getFormatValidators() {

		return null;
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		ManualSignQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		if (query.getFields() != null) {

			for (ManualSignField manualSignField : query.getFields()) {

				manualSignField.setManualSignFieldID(null);
			}
		}

		this.queryDAO.add(query, transactionHandler, new RelationQuery(ManualSignQuery.MANUAL_SIGN_FIELDS_RELATION));
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, ManualSignQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<ManualSignQueryInstance> getQueryInstanceClass() {

		return ManualSignQueryInstance.class;
	}
}
