<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="ManualTextAreaQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ManualTextAreaQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ManualTextAreaQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
				</div>

				<xsl:call-template name="form"/>
				
			</article>
				
		</div>
		
	</xsl:template>	
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', ManualTextAreaQueryInstance/ManualTextAreaQuery/queryID)" />
	
		<div class="query" id="{$queryID}">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="ManualTextAreaQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<xsl:apply-templates select="ValidationErrors/validationError"/>
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="ManualTextAreaQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="ManualTextAreaQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="ManualTextAreaQueryInstance/ManualTextAreaQuery/helpText">
						<xsl:apply-templates select="ManualTextAreaQueryInstance/ManualTextAreaQuery/helpText" />
					</xsl:if>
				
				</div>

				<xsl:call-template name="form"/>
				
			</article>
		
		</div>
		
	</xsl:template>
	
	<xsl:template name="form">
	
		<xsl:if test="ManualTextAreaQueryInstance/ManualTextAreaQuery/description">
			<span class="italic">
				<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
				<xsl:value-of select="ManualTextAreaQueryInstance/ManualTextAreaQuery/description" disable-output-escaping="yes" />
			</span>
		</xsl:if>
		
		<div class="clearboth"/>
		
		<xsl:if test="ManualTextAreaQueryInstance/ManualTextAreaQuery/hidePrintInfoText = 'false'">
			<p class="tiny">
				<xsl:value-of select="$i18n.FilledAfterPrint"/>
			</p>
		</xsl:if>
		
		<div class="marginbottom"/>
		
		<xsl:call-template name="createTextArea">
			<xsl:with-param name="rows">
				<xsl:choose>
					<xsl:when test="ManualTextAreaQueryInstance/ManualTextAreaQuery/rows">
						<xsl:value-of select="ManualTextAreaQueryInstance/ManualTextAreaQuery/rows"/>
					</xsl:when>
					<xsl:otherwise>5</xsl:otherwise>
				</xsl:choose>
			</xsl:with-param>
			<xsl:with-param name="title" select="ManualTextAreaQueryInstance/ManualTextAreaQuery/name"/>
			<xsl:with-param name="width" select="'98%'"/>
			<xsl:with-param name="disabled" select="'true'"/>
		</xsl:call-template>
	
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
		
</xsl:stylesheet>