package com.nordicpeak.flowengine.multisigninghandlers;

import javax.servlet.http.HttpSession;

import com.nordicpeak.flowengine.accesscontrollers.SessionAccessController;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.interfaces.FlowInstanceAccessController;
import com.nordicpeak.flowengine.interfaces.ImmutableFlowInstance;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.webutils.http.SessionUtils;

public class MultiSigning2FlowInstanceAccessController implements FlowInstanceAccessController {
	
	private static final String TAG = MultiSigningHandler2Module.class.getName();
	
	private final MultiSigningHandler2Module multiSigningHandlerModule;
	private final HttpSession session;
	
	public MultiSigning2FlowInstanceAccessController(MultiSigningHandler2Module multiSigningHandlerModule, HttpSession session) {
		
		super();
		this.multiSigningHandlerModule = multiSigningHandlerModule;
		this.session = session;
	}
	
	@Override
	public void checkNewFlowInstanceAccess(Flow flow, User user, SiteProfile profile) throws AccessDeniedException {
		
		throw new AccessDeniedException("Cannot create new flow instances here");
	}
	
	@Override
	public void checkFlowInstanceAccess(ImmutableFlowInstance flowInstance, User user) throws AccessDeniedException {
		
		try {
			if (multiSigningHandlerModule.getSigningSessionParty(flowInstance.getFlowInstanceID(), user) != null) {
				return;
			}
			
			if (flowInstance.getStatus().getContentType() == ContentType.WAITING_FOR_MULTISIGN) {
				
				try {
					if (session != null && session.getAttribute(SessionAccessController.class.getName() + "-" + TAG + "-" + flowInstance.getFlowInstanceID()) != null) {
						return;
					}
				} catch (IllegalStateException e) {}
			}
		} catch (Exception e) {
			throw new RuntimeException("Error checking multi sign access to " + flowInstance, e);
		}
		
		throw new AccessDeniedException("User not a signing party for " + flowInstance);
	}
	
	@Override
	public boolean isMutable(ImmutableFlowInstance flowInstance, User user) {
		
		return false;
	}
	
	public static void setSessionAttribute(Integer flowInstanceID, HttpSession session){

		SessionUtils.setAttribute(SessionAccessController.class.getName() + "-" + TAG + "-" + flowInstanceID, true, session);
	}
	
}
