package com.nordicpeak.flowengine.multisigninghandlers;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.beans.SigningParty;

@Table(name = "multi_signing_sessions")
@XMLElement(name = "SigningSession")
public class SigningSession extends GeneratedElementable implements Serializable {
	
	private static final long serialVersionUID = 3278178719339725352L;
	
	public static final Field SIGNING_PARTIES_RELATION = ReflectionUtils.getField(SigningSession.class, "signingParties");
	
	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer sessionID;
	
	@DAOManaged
	@XMLElement
	private Integer flowInstanceID;
	
	@DAOManaged
	@XMLElement
	private Timestamp added;
	
	@DAOManaged
	@XMLElement
	private Timestamp completedDate;
	
	@DAOManaged
	@XMLElement
	private boolean sequential;
	
	@DAOManaged
	private Integer pdfEventID;
	
	@DAOManaged
	@OneToMany(autoGet = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<SigningSessionParty> signingParties;
	
	public Integer getSessionID() {
		return sessionID;
	}
	
	public void setSessionID(Integer sessionID) {
		this.sessionID = sessionID;
	}
	
	public Integer getFlowInstanceID() {
		return flowInstanceID;
	}
	
	public void setFlowInstanceID(Integer flowInstanceID) {
		this.flowInstanceID = flowInstanceID;
	}
	
	public Timestamp getAdded() {
		return added;
	}
	
	public void setAdded(Timestamp added) {
		this.added = added;
	}
	
	public Timestamp getCompletedDate() {
		return completedDate;
	}
	
	public void setCompletedDate(Timestamp completed) {
		this.completedDate = completed;
	}
	
	public List<SigningSessionParty> getSigningSessionParties() {
		return signingParties;
	}
	
	public void setSigningSessionParties(List<SigningSessionParty> signingParties) {
		this.signingParties = signingParties;
	}
	
	public List<SigningParty> getSigningParties() {
		
		List<SigningParty> parties = new ArrayList<>();
		
		for (SigningSessionParty party : signingParties) {
			
			parties.add(party.getSigningParty());
		}
		
		return parties;
	}
	
	public boolean isSequential() {
		return sequential;
	}
	
	public void setSequential(boolean sequential) {
		this.sequential = sequential;
	}
	
	public boolean isFullySigned() {
		
		if (completedDate != null) {
			return true;
		}
		
		for (SigningSessionParty party : signingParties) {
			
			if (party.getSignedDate() == null) {
				return false;
			}
		}
		
		return true;
	}
	
	@Override
	public String toString() {
		
		return getClass().getSimpleName() + " " + sessionID + " (flowInstanceID " + flowInstanceID + ", " + signingParties.size() + " signers)";
	}

	
	public Integer getPdfEventID() {
	
		return pdfEventID;
	}

	
	public void setPdfEventID(Integer pdfEventID) {
	
		this.pdfEventID = pdfEventID;
	}
}
