package com.nordicpeak.flowengine.evaluators.weightparagraphevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.enums.ModificationAction;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.paragraphinfoquery.Paragraph;
import com.nordicpeak.flowengine.queries.paragraphinfoquery.ParagraphInfoQuery;
import com.nordicpeak.flowengine.queries.paragraphinfoquery.ParagraphInfoQueryInstance;

public class WeightParagraphEvaluationProviderModule extends BaseEvaluationProviderModule<WeightParagraphEvaluator> {

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	protected WeightParagraphEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<WeightParagraphEvaluator> evaluatorDAO;

	protected QueryParameterFactory<WeightParagraphEvaluator, Integer> evaluatorIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, WeightParagraphEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(WeightParagraphEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new WeightParagraphEvaluatorCRUD(WeightParagraphEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return ParagraphInfoQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		WeightParagraphEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<WeightParagraphEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		if (evaluator.getTargetParagraphIDs() != null) {

			List<Paragraph> sourceParagraphs = ((ParagraphInfoQuery) sourceQuery).getParagraphs();
			List<Paragraph> copyParagraphs = ((ParagraphInfoQuery) copyQuery).getParagraphs();

			List<Integer> convertedParagraphIDs = new ArrayList<Integer>(evaluator.getTargetParagraphIDs().size());

			for (Integer integer : evaluator.getTargetParagraphIDs()) {

				int paragraphIndex = 0;

				while (paragraphIndex < sourceParagraphs.size()) {

					if (sourceParagraphs.get(paragraphIndex).getParagraphID().equals(integer)) {

						convertedParagraphIDs.add(copyParagraphs.get(paragraphIndex).getParagraphID());

						break;
					}

					paragraphIndex++;
				}
			}

			evaluator.setTargetParagraphIDs(convertedParagraphIDs);
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

	}

	@Override
	public WeightParagraphEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		WeightParagraphEvaluator evaluator = new WeightParagraphEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		if (evaluator.getTargetParagraphIDs() != null && query instanceof ParagraphInfoQuery && ((ParagraphInfoQuery) query).getParagraphConversionMap() != null) {

			Map<Integer, Integer> conversionMap = ((ParagraphInfoQuery) query).getParagraphConversionMap();

			List<Integer> convertedParagraphIDs = new ArrayList<Integer>(evaluator.getTargetParagraphIDs().size());

			for (Integer paragraphID : evaluator.getTargetParagraphIDs()) {

				convertedParagraphIDs.add(conversionMap.get(paragraphID));
			}

			evaluator.setTargetParagraphIDs(convertedParagraphIDs);
		}

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, WeightParagraphEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (!(queryInstance instanceof ParagraphInfoQueryInstance)) {

			log.warn("Query instance " + queryInstance + " is not supported for evaluator " + evaluatorDescriptor);

			return null;
		}

		if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.HIDDEN) {

			return null;
		}

		ParagraphInfoQueryInstance paragraphInfoQueryInstance = (ParagraphInfoQueryInstance) queryInstance;

		if (paragraphInfoQueryInstance.getQuery().getParagraphs() == null || evaluator.getFromValue() == null || evaluator.getTargetParagraphIDs() == null) {

			return null;
		}

		BigDecimal weight = evaluator.getCurrentWeight();

		if (weight != null && weight.compareTo(BigDecimal.valueOf(evaluator.getFromValue())) >= 0 && (evaluator.getToValue() == null || weight.compareTo(BigDecimal.valueOf(evaluator.getToValue())) <= 0)) {

			if (evaluator.isShowParagraphs()) {

				return showParagraphs(paragraphInfoQueryInstance, evaluator, true, attributeHandler);

			} else {

				return hideParagraphs(paragraphInfoQueryInstance, evaluator, true, attributeHandler);
			}

		} else {

			if (evaluator.isShowParagraphs()) {

				return hideParagraphs(paragraphInfoQueryInstance, evaluator, false, attributeHandler);

			} else {

				return showParagraphs(paragraphInfoQueryInstance, evaluator, false, attributeHandler);
			}
		}
	}

	private EvaluationResponse showParagraphs(ParagraphInfoQueryInstance queryInstance, WeightParagraphEvaluator evaluator, boolean force, MutableAttributeHandler attributeHandler) {

		boolean queryModified = false;

		for (Integer paragraphID : evaluator.getTargetParagraphIDs()) {

			Paragraph paragraph = getParagraph(paragraphID, queryInstance.getQuery());

			if (paragraph != null) {

				if (!force && !paragraph.isVisible()) {

					continue;
				}

				if (queryInstance.getVisibleParagraphs() == null) {

					ArrayList<Paragraph> paragraphList = new ArrayList<Paragraph>(evaluator.getTargetParagraphIDs().size());

					paragraphList.add(paragraph);

					queryInstance.setVisibleParagraphs(paragraphList);

					queryModified = true;

				} else if (!queryInstance.getVisibleParagraphs().contains(paragraph)) {

					queryInstance.getVisibleParagraphs().add(paragraph);

					queryModified = true;
				}
			}
		}

		if (queryModified) {

			Collections.sort(queryInstance.getVisibleParagraphs());

			return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
		}

		return null;
	}

	private EvaluationResponse hideParagraphs(ParagraphInfoQueryInstance queryInstance, WeightParagraphEvaluator evaluator, boolean force, MutableAttributeHandler attributeHandler) {

		if (!CollectionUtils.isEmpty(queryInstance.getVisibleParagraphs())) {

			boolean queryModified = false;

			Iterator<Paragraph> paragraphIterator = queryInstance.getVisibleParagraphs().iterator();

			while (paragraphIterator.hasNext()) {

				Paragraph paragraph = paragraphIterator.next();

				if (!force && paragraph.isVisible()) {

					continue;
				}

				if (evaluator.getTargetParagraphIDs().contains(paragraph.getParagraphID())) {

					paragraphIterator.remove();

					queryModified = true;
				}
			}

			if (queryModified) {

				return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
			}
		}

		return null;
	}

	private Paragraph getParagraph(Integer paragraphID, ParagraphInfoQuery query) {

		for (Paragraph paragraph : query.getParagraphs()) {

			if (paragraph.getParagraphID().equals(paragraphID)) {

				return paragraph;
			}
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		WeightParagraphEvaluator evaluator = new WeightParagraphEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		WeightParagraphEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<WeightParagraphEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<WeightParagraphEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return queryHandler;
	}
}
