package com.nordicpeak.flowengine.evaluators.weightparagraphevaluator;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.EvaluationEventListener;
import com.nordicpeak.flowengine.interfaces.EvaluationHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

@Table(name = "weight_paragraph_evaluators")
@XMLElement
public class WeightParagraphEvaluator extends BaseEvaluator implements EvaluationEventListener {

	private static final long serialVersionUID = -1383195490043894435L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer fromValue;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer toValue;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showParagraphs;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "weight_paragraph_evaluators_target_paragraphs", remoteValueColumnName = "paragraphID")
	@WebPopulate(paramName="paragraphID")
	@XMLElement(fixCase=true,childName="paragraphID")
	private List<Integer> targetParagraphIDs;
	
	private HashMap<Integer, BigDecimal> weightSourceMap;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		fromValue = xmlParser.getInteger("fromValue");
		toValue = xmlParser.getInteger("toValue");
		showParagraphs = xmlParser.getPrimitiveBoolean("showParagraphs");

		targetParagraphIDs = xmlParser.getIntegers("TargetParagraphIDs/paragraphID");
		
		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public boolean supportsEvent(Object event) {

		if (event instanceof SetWeightEvaluationEvent) {

			return true;
		}

		return false;
	}

	@Override
	public List<QueryModification> processEvent(Object event, QueryInstance queryInstance, User user, User poster, EvaluationCallback callback, EvaluationHandler evaluationHandler) {

		SetWeightEvaluationEvent weightEvent = (SetWeightEvaluationEvent) event;

		if (this.weightType != null && weightEvent.getWeightType().equalsIgnoreCase(weightType)) {

			if (weightSourceMap == null) {

				weightSourceMap = new HashMap<Integer, BigDecimal>();
			}

			this.weightSourceMap.put(weightEvent.getSourceID(), weightEvent.getWeight());
		}

		EvaluationResponse response = super.evaluate(queryInstance, user, poster, callback, evaluationHandler, false, weightEvent.getAttributeHandler());

		if (response != null) {

			return response.getModifications();
		}

		return null;
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, EvaluationCallback callback, EvaluationHandler evaluationHandler, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		return null;
	}

	public BigDecimal getCurrentWeight() {

		if (weightSourceMap != null) {

			BigDecimal totalWeight = BigDecimal.ZERO;

			for (BigDecimal weight : weightSourceMap.values()) {

				totalWeight = totalWeight.add(weight);
			}

			return totalWeight;
		}

		return null;
	}

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public Integer getFromValue() {

		return fromValue;
	}

	public void setFromValue(Integer fromValue) {

		this.fromValue = fromValue;
	}

	public Integer getToValue() {

		return toValue;
	}

	public void setToValue(Integer toValue) {

		this.toValue = toValue;
	}

	public boolean isShowParagraphs() {

		return showParagraphs;
	}

	public void setShowParagraphs(boolean showParagraphs) {

		this.showParagraphs = showParagraphs;
	}

	
	public List<Integer> getTargetParagraphIDs() {
	
		return targetParagraphIDs;
	}

	
	public void setTargetParagraphIDs(List<Integer> targetParagraphIDs) {
	
		this.targetParagraphIDs = targetParagraphIDs;
	}
}
