package com.nordicpeak.flowengine.queries.paragraphinfoquery;

import java.sql.SQLException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.AdvancedIntegerBasedCRUD;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.querys.ObjectQuery;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;

public class ParagraphCRUD extends AdvancedIntegerBasedCRUD<Paragraph, ParagraphInfoQueryProviderModule> {

	protected AnnotatedDAOWrapper<Paragraph, Integer> paragraphDAO;

	public ParagraphCRUD(AnnotatedDAOWrapper<Paragraph, Integer> paragraphDAO, BeanRequestPopulator<Paragraph> populator, String typeElementName, String typeLogName, String listMethodAlias, ParagraphInfoQueryProviderModule callback) {

		super(Paragraph.class, paragraphDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);

		this.paragraphDAO = paragraphDAO;
	}

	@Override
	protected ForegroundModuleResponse beanAdded(Paragraph bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	protected ForegroundModuleResponse beanUpdated(Paragraph bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}


	@Override
	protected ForegroundModuleResponse beanDeleted(Paragraph bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<QueryDescriptor>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQuery().getQueryDescriptor()), EventTarget.ALL);

		callback.redirectToQueryConfig(bean.getQuery(), req, res);

		return null;
	}

	@Override
	public Paragraph getBean(Integer beanID) throws SQLException, AccessDeniedException {

		HighLevelQuery<Paragraph> query = new HighLevelQuery<Paragraph>(Paragraph.QUERY_RELATION);

		query.addParameter(paragraphDAO.getParameterFactory().getParameter(beanID));

		Paragraph paragraph = paragraphDAO.getAnnotatedDAO().get(query);

		if(paragraph != null && paragraph.getQuery() != null) {

			paragraph.getQuery().init(callback.getFlowAdminModule().getQueryDescriptor(paragraph.getQuery().getQueryID()), null);

		}

		return paragraph;
	}

	@Override
	protected Paragraph populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		Paragraph paragraph = super.populateFromAddRequest(req, user, uriParser);

		paragraph.setQuery((ParagraphInfoQuery) req.getAttribute("ParagraphInfoQuery"));

		paragraph.setSortIndex(this.getCurrentMaxSortIndex(paragraph.getQuery()) + 1);

		return paragraph;
	}
	
	@Override
	protected void checkUpdateAccess(Paragraph bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	@Override
	protected void checkDeleteAccess(Paragraph bean, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {

		callback.checkUpdateQueryAccess(user, bean.getQuery());
	}

	private int getCurrentMaxSortIndex(ParagraphInfoQuery infoQuery) throws SQLException {

		ObjectQuery<Integer> query = new ObjectQuery<Integer>(paragraphDAO.getAnnotatedDAO().getDataSource(), "SELECT MAX(sortIndex) FROM " + paragraphDAO.getAnnotatedDAO().getTableName() + " WHERE queryID = ?", IntegerPopulator.getPopulator());

		query.setInt(1, infoQuery.getQueryID());

		Integer sortIndex = query.executeQuery();

		if(sortIndex == null){

			return 0;
		}

		return sortIndex;
	}

	@Override
	protected void appendAddFormData(Document doc, Element addTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		addTypeElement.appendChild(((ParagraphInfoQuery) req.getAttribute("ParagraphInfoQuery")).getQueryDescriptor().getStep().getFlow().getFlowType().toXML(doc));
	}

	@Override
	protected void appendUpdateFormData(Paragraph bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		updateTypeElement.appendChild(bean.getQuery().getQueryDescriptor().getStep().getFlow().getFlowType().toXML(doc));
	}
}
