<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />
	
	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath"/>/static/f/<xsl:value-of select="/Document/module/sectionID"/>/<xsl:value-of select="/Document/module/moduleID"/>/pics</xsl:variable>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/queryadmin.js
	</xsl:variable>

	<xsl:template match="Document">

		<div id="AccountModule" class="contentitem errands-wrapper">
		
			<xsl:apply-templates select="ListCharts" />
	
			<xsl:apply-templates select="ShowChart" />
			<xsl:apply-templates select="AddChart" />
			<xsl:apply-templates select="UpdateChart" />
	
			<xsl:apply-templates select="ShowAccount" />
			<xsl:apply-templates select="AddAccount" />
			<xsl:apply-templates select="UpdateAccount" />
		
		</div>

	</xsl:template>
	
	<xsl:template match="ListCharts">
	
		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.Charts" />
			</h1>
		</div>
		
		<xsl:if test="validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationError"/>
			</div>
		</xsl:if>
		
		<p class="floatleft full">
			<xsl:value-of select="$i18n.ListCharts.description"/>
		</p>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Chart.name" /></span>
					</th>
					<th style="width: 40px"/>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Charts/Chart[not(archived = 'true')]">
						<xsl:apply-templates select="Charts/Chart[not(archived = 'true')]" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoCharts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<xsl:if test="Charts/Chart[archived = 'true']">
		
			<h2 class="bigpaddingtop"><xsl:value-of select="$i18n.Charts.Archived" /></h2>
		
			<table class="clearboth full coloredtable oep-table bigmarginbottom">
				<thead class="sortable">
					<tr>
						<th class="default-sort">
							<span><xsl:value-of select="$i18n.Chart.name" /></span>
						</th>
						<th style="width: 40px"/>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Charts/Chart[archived = 'true']" mode="list" />
				</tbody>
			</table>
		
		</xsl:if>
		
		<div class="floatright marginright bigmargintop">
			<a class="btn btn-green btn-inline" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addchart" title="{$i18n.AddChart}">
				<xsl:value-of select="$i18n.AddChart" />
			</a>
		</div>
	
	</xsl:template>
	
	<xsl:template match="Chart" mode="list">
	
		<tr>
			<td data-title="{$i18n.Chart.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showchart/{chartID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td>
				<a class="" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatechart/{chartID}" title="{$i18n.Update}: {name}">
					<img class="vertical-align-sub" src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<xsl:choose>
					<xsl:when test="inUse = 'true'">
					
						<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteChartBlockedInUse}'); return false;" title="{$i18n.DeleteChartBlockedInUse}">
							<img class="vertical-align-sub" src="{$imgPath}/delete_gray.png" alt="" />
						</a>
					
					</xsl:when>
					<xsl:otherwise>
					
						<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletechart/{chartID}" onclick="return confirm('{$i18n.DeleteConfirm}: {name}?');" title="{$i18n.Delete}: {name}">
							<img class="vertical-align-sub" src="{$imgPath}/delete.png" alt="" />
						</a>
						
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="ShowChart">

		<div class="heading-wrapper">
		
			<div class="floatright">
			
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updatechart/{Chart/chartID}" title="{$i18n.Update}">
					<img class="alignbottom" src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<xsl:choose>
					<xsl:when test="Chart/inUse = 'true'">
					
						<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteChartBlockedInUse}'); return false;" title="{$i18n.DeleteChartBlockedInUse}">
							<img class="vertical-align-sub" src="{$imgPath}/delete_gray.png" alt="" />
						</a>
					
					</xsl:when>
					<xsl:otherwise>
					
						<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deletechart/{Chart/chartID}" onclick="return confirm('{$i18n.DeleteConfirm}: {Chart/name}?');" title="{$i18n.Delete}: {Chart/name}">
							<img class="vertical-align-sub" src="{$imgPath}/delete.png" alt="" />
						</a>
						
					</xsl:otherwise>
				</xsl:choose>
				
			</div>

			<h1>
				<xsl:value-of select="$i18n.ShowChart" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="Chart/name"/>
			</h1>
			
		</div>
		
		<h2 class="bigpaddingtop">
			<xsl:value-of select="$i18n.AccountNumberPartDefinitions"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.AccountNumberPartDefinition.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.AccountNumberPartDefinition.length" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.AccountNumberPartDefinition.required" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/AccountNumberPartDefinitions/AccountNumberPartDefinition">
						<xsl:apply-templates select="Chart/AccountNumberPartDefinitions/AccountNumberPartDefinition" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="2"><xsl:value-of select="$i18n.NoAccountNumberPartDefinition" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.Balance"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
					<th style="width: 40px"/>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'BALANCE']">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'BALANCE']" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="3"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<xsl:if test="not(Chart/Accounts/Account[accountType = 'BALANCE'])">
			<a class="floatright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addaccount/{Chart/chartID}?type=balance">
				<xsl:value-of select="$i18n.AddAccount" />
				<img class="vertical-align-sub marginleft" src="{$imgPath}/add.png" alt="" />
			</a>
		</xsl:if>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.Income"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
					<th style="width: 40px"/>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'INCOME' and not(archived = 'true')]">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'INCOME' and not(archived = 'true')]" mode="list" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="3"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<a class="floatright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addaccount/{Chart/chartID}?type=income">
			<xsl:value-of select="$i18n.AddAccount" />
			<img class="vertical-align-sub marginleft" src="{$imgPath}/add.png" alt="" />
		</a>
		
		<h2 class="clearboth bigpaddingtop">
			<xsl:value-of select="$i18n.Accounts.VAT"/>
		</h2>
		
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.Account.name" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.number" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.Account.vat" /></span>
					</th>
					<th style="width: 40px"/>
				</tr>
			</thead>
			<tbody>
				
				<xsl:choose>
					<xsl:when test="Chart/Accounts/Account[accountType = 'VAT' and not(archived = 'true')]">
						<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'VAT' and not(archived = 'true')]" mode="list-vat" />
					</xsl:when>
					<xsl:otherwise>
						<tr>
							<td colspan="4"><xsl:value-of select="$i18n.NoAccounts" /></td>
						</tr>
					</xsl:otherwise>
				</xsl:choose>
				
			</tbody>
		</table>
		
		<a class="floatright" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/addaccount/{Chart/chartID}?type=vat">
			<xsl:value-of select="$i18n.AddAccount" />
			<img class="vertical-align-sub marginleft" src="{$imgPath}/add.png" alt="" />
		</a>
		
		<xsl:if test="Chart/Accounts/Account[accountType = 'INCOME' and archived = 'true']">
		
			<h2 class="clearboth bigpaddingtop">
				<xsl:value-of select="$i18n.Accounts.Archived.Income"/>
			</h2>
		
			<table class="clearboth full coloredtable oep-table bigmarginbottom">
				<thead class="sortable">
					<tr>
						<th class="default-sort">
							<span><xsl:value-of select="$i18n.Account.name" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.Account.number" /></span>
						</th>
						<th style="width: 40px"/>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'INCOME' and archived = 'true']" mode="list" />
				</tbody>
			</table>
		
		</xsl:if>
		
		<xsl:if test="Chart/Accounts/Account[accountType = 'VAT' and archived = 'true']">
		
			<h2 class="clearboth bigpaddingtop">
				<xsl:value-of select="$i18n.Accounts.Archived.VAT"/>
			</h2>
			
			<table class="clearboth full coloredtable oep-table bigmarginbottom">
				<thead class="sortable">
					<tr>
						<th class="default-sort">
							<span><xsl:value-of select="$i18n.Account.name" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.Account.number" /></span>
						</th>
						<th>
							<span><xsl:value-of select="$i18n.Account.vat" /></span>
						</th>
						<th style="width: 40px"/>
					</tr>
				</thead>
				<tbody>
					<xsl:apply-templates select="Chart/Accounts/Account[accountType = 'VAT' and archived = 'true']" mode="list-vat" />
				</tbody>
			</table>
		
		</xsl:if>

		<div class="clearboth floatright marginright bigmargintop">
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}" class="btn btn-light btn-inline">
				<xsl:value-of select="$i18n.Back" />
			</a>
		</div>

	</xsl:template>
	
	<xsl:template match="AccountNumberPartDefinition" mode="list">
	
		<tr>
			<td data-title="{$i18n.AccountNumberPartDefinition.name}">
				<xsl:value-of select="name" />
			</td>
			<td data-title="{$i18n.AccountNumberPartDefinition.length}">
				<xsl:value-of select="length" />
			</td>
			<td data-title="{$i18n.AccountNumberPartDefinition.required}">
				<xsl:choose>
					<xsl:when test="required = 'true'">
						<xsl:value-of select="$i18n.Yes"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No"/>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="Account" mode="list">
	
		<tr>
			<td data-title="{$i18n.Account.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showaccount/{accountID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Account.number}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showaccount/{accountID}">
					<xsl:value-of select="number"/>
				</a>
			</td>
			<td>
				<a class="" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updateaccount/{accountID}" title="{$i18n.Update}: {name}">
					<img class="vertical-align-sub" src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<xsl:choose>
					<xsl:when test="inUse = 'true'">
					
						<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteAccountBlockedInUse}'); return false;" title="{$i18n.DeleteAccountBlockedInUse}">
							<img class="vertical-align-sub" src="{$imgPath}/delete_gray.png" alt="" />
						</a>
					
					</xsl:when>
					<xsl:otherwise>
					
						<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deleteaccount/{accountID}" onclick="return confirm('{$i18n.DeleteConfirm}: {name}?');" title="{$i18n.Delete}: {name}">
							<img class="vertical-align-sub" src="{$imgPath}/delete.png" alt="" />
						</a>
						
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="Account" mode="list-vat">
	
		<tr>
			<td data-title="{$i18n.Account.name}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showaccount/{accountID}">
					<xsl:value-of select="name" />
				</a>
			</td>
			<td data-title="{$i18n.Account.number}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showaccount/{accountID}">
					<xsl:value-of select="number"/>
				</a>
			</td>
			<td data-title="{$i18n.Account.vat}">
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showaccount/{accountID}">
					<xsl:value-of select="vat" />
					<xsl:text>%</xsl:text>
				</a>
			</td>
			<td>
				<a class="" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updateaccount/{accountID}" title="{$i18n.Update}: {name}">
					<img class="vertical-align-sub" src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<xsl:choose>
					<xsl:when test="inUse = 'true'">
					
						<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteAccountBlockedInUse}'); return false;" title="{$i18n.DeleteAccountBlockedInUse}">
							<img class="vertical-align-sub" src="{$imgPath}/delete_gray.png" alt="" />
						</a>
					
					</xsl:when>
					<xsl:otherwise>
					
						<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deleteaccount/{accountID}" onclick="return confirm('{$i18n.DeleteConfirm}: {name}?');" title="{$i18n.Delete}: {name}">
							<img class="vertical-align-sub" src="{$imgPath}/delete.png" alt="" />
						</a>
						
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="AddChart">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.AddChart" />
			</h1>
		</div>

		<form method="POST" action="{/Document/requestinfo/uri}" name="chartForm">

			<xsl:call-template name="chartForm" >
				<xsl:with-param name="mode">add</xsl:with-param>
			</xsl:call-template>

			<div class="floatright marginright bigmargintop">
				<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline next" />
				
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/list" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');">
					<xsl:value-of select="$i18n.Cancel" />
				</a>
			</div>

		</form>

	</xsl:template>

	<xsl:template match="UpdateChart">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.UpdateChart" />
			</h1>
		</div>

		<form method="POST" action="{/Document/requestinfo/uri}" name="chartForm">

			<xsl:call-template name="chartForm" >
				<xsl:with-param name="mode">update</xsl:with-param>
			</xsl:call-template>

			<div class="floatright marginright bigmargintop">
				<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showchart/{Chart/chartID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');">
					<xsl:value-of select="$i18n.Cancel" />
				</a>
			</div>

		</form>

	</xsl:template>

	<xsl:template name="chartForm">
		<xsl:param name="mode"/>

		<xsl:if test="validationException/validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationException/validationError" />
			</div>
		</xsl:if>

		<div class="full bigmarginbottom">
			<label for="name" class="full required">
				<xsl:value-of select="$i18n.Chart.name" />
			</label>

			<xsl:call-template name="createTextField">
				<xsl:with-param name="name" select="'name'" />
				<xsl:with-param name="id" select="'name'" />
				<xsl:with-param name="element" select="Chart" />
			</xsl:call-template>
		</div>

		<xsl:if test="$mode = 'update'">
			<div class="full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'archived'" />
					<xsl:with-param name="name" select="'archived'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Chart" />
				</xsl:call-template>
	
				<label for="archived">
					<xsl:value-of select="$i18n.Chart.archived" />
				</label>
			</div>
		</xsl:if>
		
		<h2 class="bigpaddingtop">
			<xsl:value-of select="$i18n.AccountNumberPartDefinitions"/>
		</h2>
		
		<script type="text/javascript">
			Alternativesi18n = {
				deleteConfirm: "<xsl:value-of select="$i18n.DeleteConfirm"/>",
			};
		</script>
		
		<div class="alternatives-container full floatleft bigmarginbottom">
		
			<div class="floatright bigmargintop" style="margin-right: 18px">
			
				<xsl:variable name="options">
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Select"/></name>
						<value></value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Descending"/></name>
						<value>d</value>
					</option>
					<option>
						<name><xsl:value-of select="$i18n.AutoSortAlternatives.Ascending"/></name>
						<value>a</value>
					</option>
				</xsl:variable>
			
				<xsl:call-template name="createDropdown">
					<xsl:with-param name="class" select="'sorting'"/>
					<xsl:with-param name="name" select="'sorting'"/>
					<xsl:with-param name="width" select="'130px'"/>
					<xsl:with-param name="labelElementName" select="'name'" />
					<xsl:with-param name="valueElementName" select="'value'" />
					<xsl:with-param name="element" select="exsl:node-set($options)/option" />
				</xsl:call-template>

			</div>
		
			<label class="floatleft" style="width: calc(60% - 37px);"><xsl:value-of select="$i18n.AccountNumberPartDefinition.name" /></label>
			<label class="floatleft" style="width: calc(20% - 10px);"><xsl:value-of select="$i18n.AccountNumberPartDefinition.length" /></label>
			<label class="floatleft"><xsl:value-of select="$i18n.AccountNumberPartDefinition.required" /></label>
		
			<div class="alternatives full floatleft marginleft sortable">
				
				<xsl:choose>
					<xsl:when test="requestparameters">
						<xsl:apply-templates select="requestparameters/parameter[starts-with(name,'alternative_')]" mode="numberDefinitionAlternative" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:apply-templates select="Chart/AccountNumberPartDefinitions/AccountNumberPartDefinition" />
					</xsl:otherwise>
				</xsl:choose>
			
			</div>
			
			<div class="alternative-template" style="display: none;">
				<xsl:call-template name="createNumberDefinitionAlternative">
					<xsl:with-param name="alternativeID" select="''" />
					<xsl:with-param name="sortOrder" select="'sortIndex'" />
					<xsl:with-param name="value" select="''" />
					<xsl:with-param name="required" select="''" />
					<xsl:with-param name="length" select="''" />
					<xsl:with-param name="disabled" select="'true'" />
				</xsl:call-template>
			</div>
			
			<div class="floatright margintop" style="margin-right: 21px">
				<a href="#" onclick="addAlternative(this, event)" title="{$i18n.AccountNumberPartDefinitions.Add}">
					<xsl:value-of select="$i18n.AccountNumberPartDefinitions.Add"/>
				</a>
				<a href="#" onclick="addAlternative(this, event)" title="{$i18n.AccountNumberPartDefinitions.Add}">
					<img class="vertical-align-sub marginleft" src="{$imgPath}/add.png"/>
				</a>
			</div>
		
		</div>

	</xsl:template>
	
	<xsl:template match="AccountNumberPartDefinition">
		
		<xsl:call-template name="createNumberDefinitionAlternative">
			<xsl:with-param name="alternativeID" select="accountPartDefinitionID" />
			<xsl:with-param name="sortOrder" select="sortIndex" />
			<xsl:with-param name="value" select="name" />
			<xsl:with-param name="required" select="required" />
			<xsl:with-param name="length" select="length" />
		</xsl:call-template>
			
	</xsl:template>
	
	<xsl:template match="parameter" mode="numberDefinitionAlternative">
		
		<xsl:variable name="id" select="substring(name,13,47)" />
		
		<xsl:call-template name="createNumberDefinitionAlternative">
			<xsl:with-param name="alternativeID" select="$id" />
			<xsl:with-param name="sortOrder" select="../parameter[name = concat('sortorder_',$id)]/value" />
			<xsl:with-param name="value" select="value" />
			<xsl:with-param name="required" select="../parameter[name = concat('alternative-required_',$id)]/value" />
			<xsl:with-param name="length" select="../parameter[name = concat('alternative-length_',$id)]/value" />
		</xsl:call-template>
			
	</xsl:template>
	
	<xsl:template name="createNumberDefinitionAlternative">
		
		<xsl:param name="alternativeID" />
		<xsl:param name="sortOrder" />
		<xsl:param name="value" />
		<xsl:param name="required" />
		<xsl:param name="length" />
		<xsl:param name="disabled" select="null" />
		
		<div class="alternative full floatleft margintop marginbottom">
			<img class="vertical-align-sub marginright cursor-move" src="{$imgPath}/move.png" title="{$i18n.AccountNumberPartDefinitions.Move}"/>
			
			<!-- TODO Use of //requestparameters below is very slow and needs to be replaced -->
			
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('alternativeID_', $alternativeID)"/>
				<xsl:with-param name="name" select="'alternativeID'"/>
				<xsl:with-param name="value" select="$alternativeID"/>
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createHiddenField">
				<xsl:with-param name="id" select="concat('sortorder_', $alternativeID)" />
				<xsl:with-param name="name" select="concat('sortorder_', $alternativeID)" />
				<xsl:with-param name="value" select="$sortOrder" />
				<xsl:with-param name="class" select="'sortorder'" />
				<xsl:with-param name="requestparameters" select="//requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="concat('alternative_', $alternativeID)"/>
				<xsl:with-param name="name" select="concat('alternative_', $alternativeID)"/>
				<xsl:with-param name="value" select="$value"/>
				<xsl:with-param name="width" select="'calc(60% - 73px)'"/>
				<xsl:with-param name="class" select="'name bigmarginright'" />
				<xsl:with-param name="requestparameters" select="//requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="concat('alternative-length_', $alternativeID)"/>
				<xsl:with-param name="name" select="concat('alternative-length_', $alternativeID)"/>
				<xsl:with-param name="value" select="$length"/>
				<xsl:with-param name="width" select="'calc(20% - 10px)'"/>
				<xsl:with-param name="class" select="'marginright'" />
				<xsl:with-param name="requestparameters" select="//requestparameters" />
				<xsl:with-param name="disabled" select="$disabled"/>
			</xsl:call-template>
			<div class="display-inline-block marginright" style="width: calc(20% - 10px)">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="concat('alternative-required_', $alternativeID)"/>
					<xsl:with-param name="name" select="concat('alternative-required_', $alternativeID)"/>
					<xsl:with-param name="checked" select="$required" />
					<xsl:with-param name="requestparameters" select="//requestparameters" />
					<xsl:with-param name="disabled" select="$disabled"/>
				</xsl:call-template>
			</div>
			
			<a href="#" onclick="deleteAlternative(this, event)" title="{$i18n.AccountNumberPartDefinitions.Delete}">
				<img class="vertical-align-sub marginleft" src="{$imgPath}/delete.png"/>
			</a>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="ShowAccount">

		<div class="heading-wrapper">
		
			<div class="floatright">
			
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/updateaccount/{Account/accountID}" title="{$i18n.Update}">
					<img class="alignbottom" src="{$imgPath}/pen.png" alt="" />
				</a>
				
				<xsl:choose>
					<xsl:when test="Account/inUse = 'true'">
					
						<a class="marginleft" href="#" onclick="alert('{$i18n.DeleteAccountBlockedInUse}'); return false;" title="{$i18n.DeleteAccountBlockedInUse}">
							<img class="vertical-align-sub" src="{$imgPath}/delete_gray.png" alt="" />
						</a>
					
					</xsl:when>
					<xsl:otherwise>
					
						<a class="marginleft" href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/deleteaccount/{Account/accountID}" onclick="return confirm('{$i18n.DeleteConfirm}: {Chart/name}?');" title="{$i18n.Delete}: {Chart/name}">
							<img class="vertical-align-sub" src="{$imgPath}/delete.png" alt="" />
						</a>
						
					</xsl:otherwise>
				</xsl:choose>
				
			</div>

			<h1>
				<xsl:value-of select="$i18n.ShowAccount" />
				<xsl:text>: </xsl:text>
				<xsl:value-of select="Account/name"/>
			</h1>
			
		</div>
		
		<h2 class="bigpaddingtop">
			<xsl:value-of select="$i18n.AccountNumberPart"/>
		</h2>
	
		<table class="clearboth full coloredtable oep-table bigmarginbottom">
			<thead class="sortable">
				<tr>
					<th class="default-sort">
						<span><xsl:value-of select="$i18n.AccountNumberPart.definition" /></span>
					</th>
					<th>
						<span><xsl:value-of select="$i18n.AccountNumberPart.value" /></span>
					</th>
				</tr>
			</thead>
			<tbody>
				<xsl:apply-templates select="Account/Chart/AccountNumberPartDefinitions/AccountNumberPartDefinition" mode="list-account">
					<xsl:with-param name="account" select="Account"/>
				</xsl:apply-templates>
			</tbody>
		</table>
		
		<div class="clearboth floatright marginright bigmargintop">
			<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showchart/{Account/Chart/chartID}" class="btn btn-light btn-inline">
				<xsl:value-of select="$i18n.Back" />
			</a>
		</div>

	</xsl:template>
	
	<xsl:template match="AccountNumberPartDefinition" mode="list-account">
		<xsl:param name="account"/>
	
		<tr>
			<td data-title="{$i18n.AccountNumberPart.definition}">
				<xsl:value-of select="name" />
			</td>
			<td data-title="{$i18n.AccountNumberPart.value}">
				<xsl:value-of select="$account/AccountNumberParts/AccountNumberPart[AccountNumberPartDefinition/accountPartDefinitionID = current()/accountPartDefinitionID]/value" />
			</td>
		</tr>
	
	</xsl:template>
	
	<xsl:template match="AddAccount">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.AddAccount" />
			</h1>
		</div>

		<form method="POST" action="{/Document/requestinfo/uri}" name="accountForm">

			<input type="hidden" name="type" value="{AccountType}"/>

			<xsl:call-template name="accountForm" >
				<xsl:with-param name="mode">add</xsl:with-param>
				<xsl:with-param name="chart" select="Chart"/>
			</xsl:call-template>

			<div class="floatright marginright bigmargintop">
				<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline next" />
				
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showchart/{Chart/chartID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');">
					<xsl:value-of select="$i18n.Cancel" />
				</a>
			</div>

		</form>

	</xsl:template>

	<xsl:template match="UpdateAccount">

		<div class="heading-wrapper">
			<h1>
				<xsl:value-of select="$i18n.UpdateAccount" />
			</h1>
		</div>

		<form method="POST" action="{/Document/requestinfo/uri}" name="accountForm">

			<xsl:call-template name="accountForm" >
				<xsl:with-param name="mode">update</xsl:with-param>
				<xsl:with-param name="chart" select="Account/Chart"/>
			</xsl:call-template>

			<div class="floatright marginright bigmargintop">
				<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline next" />
				
				<a href="{/Document/requestinfo/currentURI}/{/Document/module/alias}/showchart/{Account/Chart/chartID}" class="btn btn-light btn-inline" onclick="return confirm('{$i18n.CancelConfirm}?');">
					<xsl:value-of select="$i18n.Cancel" />
				</a>
			</div>

		</form>

	</xsl:template>

	<xsl:template name="accountForm">
		<xsl:param name="mode" />
		<xsl:param name="chart" />

		<xsl:if test="validationException/validationError">
			<div class="bigmarginbottom">
				<xsl:apply-templates select="validationException/validationError" />
			</div>
		</xsl:if>
		
		<div class="full bigmarginbottom">
			<label for="name" class="full required">
				<xsl:value-of select="$i18n.Account.name" />
			</label>

			<xsl:call-template name="createTextField">
				<xsl:with-param name="name" select="'name'" />
				<xsl:with-param name="id" select="'name'" />
				<xsl:with-param name="element" select="Account" />
			</xsl:call-template>
		</div>
		
		<xsl:if test="AccountType = 'VAT' or Account/accountType = 'VAT'">
		
			<div class="full bigmarginbottom">
				<label for="vat" class="full required">
					<xsl:value-of select="$i18n.Account.vat" />
				</label>
	
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'vat'" />
					<xsl:with-param name="id" select="'vat'" />
					<xsl:with-param name="element" select="Account" />
				</xsl:call-template>
			</div>
		
		</xsl:if>
		
		<xsl:if test="$mode = 'update'">
			<div class="full">
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'archived'" />
					<xsl:with-param name="name" select="'archived'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="Account" />
				</xsl:call-template>
	
				<label for="archived">
					<xsl:value-of select="$i18n.Account.archived" />
				</label>
			</div>
		</xsl:if>
		
		<h2 class="bigpaddingtop">
			<xsl:value-of select="$i18n.AccountNumberParts"/>
		</h2>

		<xsl:apply-templates select="$chart/AccountNumberPartDefinitions/AccountNumberPartDefinition" mode="account">
			<xsl:with-param name="account" select="Account"/>
			<xsl:with-param name="requestparameters" select="requestparameters"/>
		</xsl:apply-templates>

	</xsl:template>
	
	<xsl:template match="AccountNumberPartDefinition" mode="account">
		<xsl:param name="account"/>
		<xsl:param name="requestparameters"/>
	
		<div class="full bigmarginbottom">
			<label for="part-{accountPartDefinitionID}">
				<xsl:attribute name="class">
					<xsl:text>class</xsl:text>
					<xsl:if test="required = 'true'"> required</xsl:if>
				</xsl:attribute>
			
				<xsl:value-of select="name" />
				
				<xsl:if test="length">
					<xsl:text> (</xsl:text>
					<xsl:value-of select="$i18n.AccountNumberPartDefinition.length"/>
					<xsl:text> </xsl:text>
					<xsl:value-of select="length"/>
					<xsl:text>)</xsl:text>
				</xsl:if>
			</label>

			<xsl:call-template name="createTextField">
				<xsl:with-param name="name" select="concat('part-', accountPartDefinitionID)" />
				<xsl:with-param name="id" select="concat('part-', accountPartDefinitionID)" />
				<xsl:with-param name="value" select="$account/AccountNumberParts/AccountNumberPart[AccountNumberPartDefinition/accountPartDefinitionID = current()/accountPartDefinitionID]/value" />
				<xsl:with-param name="requestparameters" select="$requestparameters"/>	
			</xsl:call-template>
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ShowFailedChartNotFound' or messageKey = 'UpdateFailedChartNotFound' or messageKey = 'DeleteFailedChartNotFound']">
		<p class="error">
			<xsl:value-of select="$i18n.Validation.ChartNotFound" />
		</p>
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ShowFailedAccountNotFound' or messageKey = 'UpdateFailedAccountNotFound' or messageKey = 'DeleteFailedAccountNotFound']">
		<p class="error">
			<xsl:value-of select="$i18n.Validation.AccountNotFound" />
		</p>
	</xsl:template>

	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.Validation.RequiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.Validation.InvalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.Validation.TooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.Validation.TooLong" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.Validation.UnknownError" />
					</xsl:otherwise>
				</xsl:choose>

				<xsl:text>&#x20;</xsl:text>

				<xsl:choose>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.Chart.name" />
					</xsl:when>
					<xsl:when test="fieldName = 'vat'">
						<xsl:value-of select="$i18n.Account.vat" />
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'part-')">
					
						<xsl:value-of select="$i18n.AccountNumberPart" />
						<xsl:text>&#160;</xsl:text>
						<xsl:variable name="id" select="substring(fieldName, 6)" />
						<xsl:value-of select="../../Account/Chart/AccountNumberPartDefinitions/AccountNumberPartDefinition[accountPartDefinitionID = $id]/name" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'alternative_')">
					
						<xsl:value-of select="$i18n.AccountNumberPartDefinition" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="$i18n.Row" />
						<xsl:text>&#160;</xsl:text>
						<xsl:variable name="id" select="substring(fieldName, 13)" />
						<xsl:value-of select="../../requestparameters/parameter[name = concat('sortorder_', $id)]/value + 1" />
						
					</xsl:when>
					<xsl:when test="starts-with(fieldName, 'alternative-length_')">
					
						<xsl:value-of select="$i18n.AccountNumberPartDefinition" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="$i18n.AccountNumberPartDefinition.Length" />
						<xsl:text>:&#160;</xsl:text>
						<xsl:value-of select="$i18n.Row" />
						<xsl:text>&#160;</xsl:text>
						<xsl:variable name="id" select="substring(fieldName, 20)" />
						<xsl:value-of select="../../requestparameters/parameter[name = concat('sortorder_', $id)]/value + 1" />
						
						<xsl:variable name="name" select="../../requestparameters/parameter[name = concat('alternative_', $id)]/value" />
						
						<xsl:if test="$name != ''">
							<xsl:text>&#160;</xsl:text>
							<xsl:value-of select="$name" />
						</xsl:if>
						
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName" />
					</xsl:otherwise>
				</xsl:choose>
			</p>
		</xsl:if>

		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.Validation.UnknownFault" />
			</p>
		</xsl:if>

	</xsl:template>

</xsl:stylesheet>