package com.nordicpeak.flowengine.paymentlog.beans;

import java.lang.reflect.Field;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.paymentlog.enums.AccountType;

@XMLElement
@Table(name = "payment_accounts")
public class Account extends GeneratedElementable {
	
	public static final Field ACCOUNT_NUMBER_PARTS_RELATION = ReflectionUtils.getField(Account.class, "accountNumberParts");
	public static final Field CHART_RELATION = ReflectionUtils.getField(Account.class, "chart");
	
	@DAOManaged(autoGenerated = true)
	@Key
	@XMLElement
	private Integer accountID;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;
	
	@DAOManaged
	@XMLElement
	private AccountType accountType;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean archived;
	
	@DAOManaged
	@XMLElement
	private Integer vat;
	
	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<AccountNumberPart> accountNumberParts;
	
	@DAOManaged(columnName = "chartID")
	@ManyToOne
	@XMLElement(fixCase = true)
	private ChartOfAccounts chart;
	
	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<Transaction> transactions;
	
	@DAOManaged
	@OneToMany(remoteField = "account")
	@XMLElement(fixCase = true)
	private List<FlowFamilyTransactionSettings> accountFlowFamilySettings;
	
	@DAOManaged
	@OneToMany(remoteField = "vatAccount")
	@XMLElement(fixCase = true)
	private List<FlowFamilyTransactionSettings> vatAccountFlowFamilySettings;
	
	@DAOManaged
	@OneToMany(remoteField = "account")
	@XMLElement(fixCase = true)
	private List<PaymentProductTransactionSettings> accountPaymentProductSettingsList;
	
	@DAOManaged
	@OneToMany(remoteField = "vatAccount")
	@XMLElement(fixCase = true)
	private List<PaymentProductTransactionSettings> vatAccountPaymentProductSettingsList;
	
	public Integer getAccountID() {
		return accountID;
	}
	
	public void setAccountID(Integer accountID) {
		this.accountID = accountID;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public AccountType getAccountType() {
		return accountType;
	}
	
	public void setAccountType(AccountType accountType) {
		this.accountType = accountType;
	}
	
	public Integer getVat() {
		return vat;
	}
	
	public void setVat(Integer vat) {
		this.vat = vat;
	}
	
	public List<AccountNumberPart> getAccountNumberParts() {
		return accountNumberParts;
	}
	
	public void setAccountNumberParts(List<AccountNumberPart> accountNumberParts) {
		this.accountNumberParts = accountNumberParts;
	}
	
	public ChartOfAccounts getChart() {
		return chart;
	}
	
	public void setChart(ChartOfAccounts chart) {
		this.chart = chart;
	}
	
	public List<Transaction> getTransactions() {
		return transactions;
	}
	
	public void setTransactions(List<Transaction> transactions) {
		this.transactions = transactions;
	}
	
	public boolean isArchived() {
		return archived;
	}
	
	public void setArchived(boolean archived) {
		this.archived = archived;
	}

	public List<FlowFamilyTransactionSettings> getAccountFlowFamilySettings() {

		return accountFlowFamilySettings;
	}

	public void setAccountFlowFamilySettings(List<FlowFamilyTransactionSettings> accountFlowFamilySettings) {

		this.accountFlowFamilySettings = accountFlowFamilySettings;
	}

	public List<FlowFamilyTransactionSettings> getVatAccountFlowFamilySettings() {

		return vatAccountFlowFamilySettings;
	}

	public void setVatAccountFlowFamilySettings(List<FlowFamilyTransactionSettings> vatAccountFlowFamilySettings) {

		this.vatAccountFlowFamilySettings = vatAccountFlowFamilySettings;
	}

	public List<PaymentProductTransactionSettings> getAccountPaymentProductSettingsList() {

		return accountPaymentProductSettingsList;
	}

	public void setAccountPaymentProductSettingsList(List<PaymentProductTransactionSettings> accountPaymentProductSettingsList) {

		this.accountPaymentProductSettingsList = accountPaymentProductSettingsList;
	}

	public List<PaymentProductTransactionSettings> getVatAccountPaymentProductSettingsList() {

		return vatAccountPaymentProductSettingsList;
	}

	public void setVatAccountPaymentProductSettingsList(List<PaymentProductTransactionSettings> vatAccountPaymentProductSettingsList) {

		this.vatAccountPaymentProductSettingsList = vatAccountPaymentProductSettingsList;
	}

	@Override
	public Element toXML(Document doc) {
		
		Element element = super.toXML(doc);
		
		if (chart != null && chart.getAccountNumberPartDefinitions() != null) {
			
			StringBuilder builder = new StringBuilder();
			
			boolean first = true;
			for (AccountNumberPartDefinition numberPartDefinition : chart.getAccountNumberPartDefinitions()) {
				
				if (first) {
					
					first = false;
					
				} else {
					
					builder.append(" - ");
				}
				
				AccountNumberPart foundNumberPart = null;
				
				if (accountNumberParts != null) {
					for (AccountNumberPart numberPart : accountNumberParts) {
						
						if (numberPart.getAccountNumberPartDefinition() != null && numberPart.getAccountNumberPartDefinition().getAccountPartDefinitionID().equals(numberPartDefinition.getAccountPartDefinitionID())) {
							
							foundNumberPart = numberPart;
							break;
						}
					}
				}
				
				if (foundNumberPart == null) {
					
					if (numberPartDefinition.getLength() == null) {
					
						builder.append("\u2007"); // U+2007 FIGURE SPACE
						
					} else {
						
						StringUtils.repeatString("\u2007", numberPartDefinition.getLength(), builder);
					}
					
				} else {
					
					if (numberPartDefinition.getLength() == null) {
						
						builder.append(foundNumberPart.getValue());

					} else {
						
						builder.append(String.format("%0" + numberPartDefinition.getLength() + "d", foundNumberPart.getValue()));
					}
				}
			}
			
			XMLUtils.appendNewElement(doc, element, "number", builder.toString());
		}
		
		return element;
	}

	@Override
	public String toString() {
		
		return StringUtils.toLogFormat(name, 30) + " (accountID: " + accountID + ")";
	}
}
