package com.nordicpeak.flowengine.paymentlog.cruds;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.utils.crud.IntegerBeanIDParser;
import se.unlogic.hierarchy.core.utils.crud.ModularCRUD;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.CRUDDAO;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGeneratorDocument;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.paymentlog.AccountModule;
import com.nordicpeak.flowengine.paymentlog.beans.Account;
import com.nordicpeak.flowengine.paymentlog.beans.AccountNumberPartDefinition;
import com.nordicpeak.flowengine.paymentlog.beans.ChartOfAccounts;
import com.nordicpeak.flowengine.paymentlog.listeners.AccountInUseElementableListener;
import com.nordicpeak.flowengine.paymentlog.listeners.ChartInUseElementableListener;
import com.nordicpeak.flowengine.paymentlog.populators.AccountNumberPartDefinitionPopulator;

public class ChartCRUD extends ModularCRUD<ChartOfAccounts, Integer, User, AccountModule> {
	
	protected static AccountNumberPartDefinitionPopulator ALTERNATIVES_POPLATOR = new AccountNumberPartDefinitionPopulator();
	
	public ChartCRUD(CRUDDAO<ChartOfAccounts, Integer> crudDAO, AccountModule callback) {
		super(IntegerBeanIDParser.getInstance(), crudDAO, new AnnotatedRequestPopulator<>(ChartOfAccounts.class), "Chart", "chart", "/list", callback);
	}
	
	@Override
	protected void checkDeleteAccess(ChartOfAccounts chart, User user, HttpServletRequest req, URIParser uriParser) throws AccessDeniedException, URINotFoundException, SQLException {
		super.checkDeleteAccess(chart, user, req, uriParser);
		
		//TODO if used by flow family or has transactions deny, maybe allow if archived
		if (callback.isChartInUse(chart)) {
			throw new AccessDeniedException("Chart or its accounts in use");
		}
	}
	
	@Override
	protected void appendBean(ChartOfAccounts chart, Element targetElement, Document doc, User user) {
		
		if (!CollectionUtils.isEmpty(chart.getAccounts())) {
			for (Account account : chart.getAccounts()) {
				account.setChart(chart);
			}
		}
		
		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
		genDoc.addIgnoredField(Account.CHART_RELATION);
		
		genDoc.addFieldElementableListener(Account.class, new AccountInUseElementableListener(callback));
		
		Element element = chart.toXML(genDoc);
		
		try {
			if (callback.isChartInUse(chart)) {
				XMLUtils.appendNewElement(doc, element, "inUse", true);
			}
		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
		
		targetElement.appendChild(element);
	}
	
	@Override
	protected void appendAllBeans(Document doc, Element listTypeElement, User user, HttpServletRequest req, URIParser uriParser, List<ValidationError> validationError) throws SQLException {
		
		XMLGeneratorDocument genDoc = new XMLGeneratorDocument(doc);
		genDoc.addIgnoredField(Account.CHART_RELATION);
		
		genDoc.addRootElementableListener(ChartOfAccounts.class, new ChartInUseElementableListener(callback));
		
		XMLUtils.append(genDoc, listTypeElement, this.typeElementPluralName, getAllBeans(user, req, uriParser));
	}

	@Override
	protected ChartOfAccounts populateFromAddRequest(HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		ChartOfAccounts chart = super.populateFromAddRequest(req, user, uriParser);
		
		populate(chart, null, req, user, uriParser);
		
		return chart;
	}
	
	@Override
	protected ChartOfAccounts populateFromUpdateRequest(ChartOfAccounts oldChart, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		ChartOfAccounts chart = super.populateFromUpdateRequest(oldChart, req, user, uriParser);
		
		populate(chart, oldChart, req, user, uriParser);
		
		return chart;
	}
	
	protected ChartOfAccounts populate(ChartOfAccounts chart, ChartOfAccounts oldChart, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		List<AccountNumberPartDefinition> numberPartDefinitions = ALTERNATIVES_POPLATOR.populate(oldChart != null ? oldChart.getAccountNumberPartDefinitions() : null, req, validationErrors);
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		chart.setAccountNumberPartDefinitions(numberPartDefinitions);
		
		return chart;
	}
	
	@Override
	protected ForegroundModuleResponse filteredBeanAdded(ChartOfAccounts chart, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/showchart/" + chart.getChartID());
		return null;
	}
	
	@Override
	protected ForegroundModuleResponse filteredBeanUpdated(ChartOfAccounts chart, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		res.sendRedirect(req.getContextPath() + callback.getFullAlias() + "/showchart/" + chart.getChartID());
		return null;
	}
	
}
