package com.nordicpeak.flowengine.paymentlog.populators;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.paymentlog.beans.AccountNumberPartDefinition;


public class AccountNumberPartDefinitionPopulator {

	public List<AccountNumberPartDefinition> populate(List<AccountNumberPartDefinition> currentAlternatives, HttpServletRequest req, List<ValidationError> validationErrors) {
		return populate(currentAlternatives, req, 255, validationErrors);
	}
	
	public List<AccountNumberPartDefinition> populate(List<AccountNumberPartDefinition> currentAlternatives, HttpServletRequest req, int maxLength, List<ValidationError> validationErrors) {
		
		String[] alternativeIDs = req.getParameterValues("alternativeID");
		
		List<AccountNumberPartDefinition> alternatives = new ArrayList<AccountNumberPartDefinition>();
		
		if (alternativeIDs != null) {
			
			for (String alternativeID : alternativeIDs) {
				
				String name = ValidationUtils.validateParameter("alternative_" + alternativeID, req, true, 1, maxLength, validationErrors);
				
				if (name == null) {
					continue;
				}
				
				String sortOrder = req.getParameter("sortorder_" + alternativeID);
				Integer length = ValidationUtils.validateParameter("alternative-length_" + alternativeID, req, false, NonNegativeStringIntegerPopulator.getPopulator(), validationErrors);
				boolean required = "true".equals(req.getParameter("alternative-required_" + alternativeID));
				
				if (NumberUtils.isInt(sortOrder) && validationErrors.isEmpty()) {
					
					AccountNumberPartDefinition numberPartDefinition = new AccountNumberPartDefinition();
					
					numberPartDefinition.setName(name);
					numberPartDefinition.setSortIndex(NumberUtils.toInt(sortOrder));
					numberPartDefinition.setRequired(required);
					numberPartDefinition.setLength(length);
					
					if (NumberUtils.isInt(alternativeID)) {
						
						checkForExistingAlternatives(currentAlternatives, numberPartDefinition, NumberUtils.toInt(alternativeID));
					}
					
					alternatives.add(numberPartDefinition);
				}
			}
		}
		
		return alternatives;
	}
	
	protected void checkForExistingAlternatives(List<AccountNumberPartDefinition> currentAlternatives, AccountNumberPartDefinition alternative, Integer alternativeID) {
		
		if (!CollectionUtils.isEmpty(currentAlternatives)) {
			
			for (AccountNumberPartDefinition currentAlternative : currentAlternatives) {
				
				if (currentAlternative.getAccountPartDefinitionID().equals(alternativeID)) {
					
					alternative.setAccountPartDefinitionID(alternativeID);
					break;
				}
			}
		}
	}
}
