package com.nordicpeak.flowengine.paymentlog.populators;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.paymentlog.beans.AccountNumberPart;
import com.nordicpeak.flowengine.paymentlog.beans.AccountNumberPartDefinition;

public class AccountNumberPartPopulator {
	
	public List<AccountNumberPart> populate(List<AccountNumberPartDefinition> definitions, HttpServletRequest req, List<ValidationError> validationErrors) {
		return populate(definitions, req, 255, validationErrors);
	}
	
	public List<AccountNumberPart> populate(List<AccountNumberPartDefinition> definitions, HttpServletRequest req, int maxLength, List<ValidationError> validationErrors) {
		
		List<AccountNumberPart> alternatives = new ArrayList<AccountNumberPart>();
		
		for (AccountNumberPartDefinition numberPartDefinition : definitions) {
			
			Integer value = ValidationUtils.validateParameter("part-" + numberPartDefinition.getAccountPartDefinitionID(), req, numberPartDefinition.isRequired(), NonNegativeStringIntegerPopulator.getPopulator(), validationErrors);
			
			if (value == null) {
				continue;
			}
			
			if (numberPartDefinition.getLength() != null && value.toString().length() > numberPartDefinition.getLength()) {
				
				validationErrors.add(new ValidationError("part-" + numberPartDefinition.getAccountPartDefinitionID(), ValidationErrorType.TooLong));
				continue;
			}
			
			AccountNumberPart accountNumberPart = new AccountNumberPart();
			
			accountNumberPart.setAccountNumberPartDefinition(numberPartDefinition);
			accountNumberPart.setValue(value);
			
			alternatives.add(accountNumberPart);
		}
		
		return alternatives;
	}
}
