<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/personquery.js?v=1
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/personquery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="PersonQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article>
				
				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="PersonQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="PersonQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="PersonQueryInstance/PersonQuery/queryID" />
					</xsl:call-template>
					
				</div>
				
				<xsl:if test="PersonQueryInstance/PersonQuery/description">
					<span class="italic">
						<xsl:value-of select="PersonQueryInstance/PersonQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<fieldset>

					<div class="split">
						<strong class="block"><xsl:value-of select="$i18n.CitizenID" /></strong>
						<xsl:value-of select="PersonQueryInstance/citizenID" />						
					</div>				
			
				</fieldset>
				
				<fieldset>
					
					<div class="split">
						<strong class="block"><xsl:value-of select="$i18n.FirstnameAndLastname" /></strong>
						<xsl:value-of select="PersonQueryInstance/firstname" />
						<xsl:text>&#160;</xsl:text>
						<xsl:value-of select="PersonQueryInstance/lastname" />
					</div>
					
					<div class="split odd"></div>
					
				</fieldset>
				
				<xsl:if test="PersonQueryInstance/careOf">
				
					<fieldset>
				
						<div class="split">
							<strong class="block"><xsl:value-of select="$i18n.CareOf" /></strong>
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="PersonQueryInstance/careOf" />
							</xsl:call-template>
						</div>
					
					</fieldset>
				
				</xsl:if>
				
				<fieldset>
				
					<div class="split">
						<strong class="block"><xsl:value-of select="$i18n.Address" /></strong>
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="PersonQueryInstance/address" />
						</xsl:call-template>
					</div>
					
					<div class="split odd">
						<strong class="block"><xsl:value-of select="$i18n.ZipCodeAndPostalAddress" /></strong>
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="PersonQueryInstance/zipCode" />
						</xsl:call-template>
						<xsl:text>&#160;</xsl:text>
						<xsl:call-template name="printValue">
							<xsl:with-param name="value" select="PersonQueryInstance/postalAddress" />
						</xsl:call-template>
					</div>
					
				</fieldset>
				
				<xsl:if test="PersonQueryInstance/PersonQuery/useForMultipartSigning = 'true'">
					
					<fieldset>
					
						<div class="split">
							<strong class="block"><xsl:value-of select="$i18n.Email" /></strong>
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="PersonQueryInstance/email" />
							</xsl:call-template>
						</div>
						
						<div class="split odd">
							<strong class="block"><xsl:value-of select="$i18n.Phone" /></strong>
							<xsl:call-template name="printValue">
								<xsl:with-param name="value" select="PersonQueryInstance/phone" />
							</xsl:call-template>
						</div>
						
					</fieldset>
					
				</xsl:if>
				
			</article>
			
		</div>
		
	</xsl:template>
	
	<xsl:template name="printValue">
		
		<xsl:param name="value" />
		
		<xsl:choose>
			<xsl:when test="$value">
				<xsl:value-of select="$value"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>-</xsl:text>
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', PersonQueryInstance/PersonQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', PersonQueryInstance/PersonQuery/queryID)" />
	
		<div id="{$queryID}">
			
			<xsl:attribute name="class">
				<xsl:text>query personquery</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="PersonQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
			<input name="connectorURL" disabled="true" hidden="true" value="{queryRequestURL}"/>
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
				
					<div class="info-box error">
					
						<xsl:choose>
							<xsl:when test="ValidationErrors/validationError[not(fieldName)]">
								<xsl:apply-templates select="ValidationErrors/validationError[not(fieldName)]"/>
							</xsl:when>
							<xsl:otherwise>
								<span/>
							</xsl:otherwise>
						</xsl:choose>
				
						<div class="marker"></div>
					</div>
				
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="PersonQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="PersonQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
					<xsl:if test="PersonQueryInstance/PersonQuery/helpText">		
						<xsl:apply-templates select="PersonQueryInstance/PersonQuery/helpText" />
					</xsl:if>
				
				</div>
				
				<xsl:if test="PersonQueryInstance/PersonQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="PersonQueryInstance/PersonQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<fieldset>

					<div class="split">

						<xsl:variable name="fieldName" select="concat($shortQueryID, '_citizenID')" />
					
						<xsl:if test="ValidationErrors/validationError[fieldName = $fieldName]">
							<xsl:attribute name="class">
								<xsl:text>split invalid input-error</xsl:text>
							</xsl:attribute>
						</xsl:if>
					
						<label for="{$fieldName}" >
							<xsl:attribute name="class">
								<xsl:if test="PersonQueryInstance/PersonQuery/fieldCitizenID = 'REQUIRED'">required</xsl:if>
							</xsl:attribute>
						
							<xsl:value-of select="$i18n.CitizenIDWithFormat" />
						</label>
						
						<xsl:call-template name="createTextField">
							<xsl:with-param name="id" select="$fieldName" />
							<xsl:with-param name="name" select="$fieldName" />
							<xsl:with-param name="title" select="$i18n.CitizenIDWithFormat"/>
							<xsl:with-param name="size" select="50"/>
							<xsl:with-param name="value" select="PersonQueryInstance/citizenID" />
						</xsl:call-template>
						
						<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $fieldName]"/>
						
					</div>
					
					<div class="split odd">
					
						<a class="btn btn-green btn-inline fetchbutton" onclick="event.preventDefault(); event.stopPropagation(); $(this).closest('fieldset').find('input').trigger('change');"><xsl:value-of select="$i18n.Lookup"/></a>
					
					</div>
					
					<div style="display: none">
					
						<i class="template-invalid-format" data-icon-after="!" title="{$i18n.ValidationError.InvalidFormat}"></i>
						
						<p class="template-block-poster-selection error">
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.ValidationError.BlockPosterSelection"/>
							</strong>
						</p>
						
						<p class="template-not-found error">
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.ValidationError.PersonNotFound"/>
							</strong>
						</p>
						
						<p class="template-provider-error error">
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.ValidationError.ProviderError"/>
							</strong>
						</p>
						
						<p class="template-max-lookups-reached error">
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.Error.MaxLookupsReached.Part1"/>
								<xsl:text>&#160;</xsl:text>
								<xsl:value-of select="/Document/MaxLookUps" />
								<xsl:text>&#160;</xsl:text>
								<xsl:value-of select="$i18n.Error.MaxLookupsReached.Part2"/>
							</strong>
						</p>
						
						<p class="template-unknown-error error">
							<strong data-icon-before="!">
								<xsl:value-of select="$i18n.Error.UnknownError"/>
							</strong>
						</p>
					
					</div>
			
				</fieldset>
				
				<div class="person-wrapper hidden">
				
					<fieldset>
						
						<div class="split">
						
							<strong><xsl:value-of select="$i18n.FirstnameAndLastname" /></strong>
							<div class="name"><xsl:if test="PersonQueryInstance/firstname and PersonQueryInstance/lastname"><xsl:value-of select="PersonQueryInstance/firstname" /><xsl:text>&#160;</xsl:text><xsl:value-of select="PersonQueryInstance/lastname" /></xsl:if></div>
							
						</div>
					
						<div class="split odd"></div>
							
					</fieldset>
					
					<fieldset>
						<xsl:if test="not(PersonQueryInstance/careOf)"><xsl:attribute name="class">hidden</xsl:attribute></xsl:if>
						<div class="split">
							<strong class="block"><xsl:value-of select="$i18n.CareOf" /></strong>
							<div class="careof">
								<xsl:call-template name="printValue">
									<xsl:with-param name="value" select="PersonQueryInstance/careOf" />
								</xsl:call-template>
							</div>
						</div>
						
					</fieldset>
					
					<fieldset>
					
						<div class="split">
							<strong class="block"><xsl:value-of select="$i18n.Address" /></strong>
							<div class="address">
								<xsl:call-template name="printValue">
									<xsl:with-param name="value" select="PersonQueryInstance/address" />
								</xsl:call-template>
							</div>
						</div>
						
						<div class="split odd">
							<strong class="block"><xsl:value-of select="$i18n.ZipCodeAndPostalAddress" /></strong>
							<div class="postal">
								<xsl:call-template name="printValue">
									<xsl:with-param name="value" select="PersonQueryInstance/zipCode" />
								</xsl:call-template>
								<xsl:text>&#160;</xsl:text>
								<xsl:call-template name="printValue">
									<xsl:with-param name="value" select="PersonQueryInstance/postalAddress" />
								</xsl:call-template>
							</div>
						</div>
					
					</fieldset>
					
					<xsl:if test="PersonQueryInstance/PersonQuery/useForMultipartSigning = 'true'">
						
						<fieldset>
							
							<xsl:variable name="emailID">
								<xsl:value-of select="$shortQueryID"/>
								<xsl:value-of select="'_email'"/>
							</xsl:variable>
							
							<xsl:variable name="classEmail">
								<xsl:if test="ValidationErrors/validationError[fieldName = $emailID]">
									<xsl:text>invalid input-error</xsl:text>
								</xsl:if>
							</xsl:variable>
							
							<div class="split {$classEmail}">
							
								<label>
									<xsl:value-of select="$i18n.Email"/>
								</label>
						
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="$emailID"/>
									<xsl:with-param name="title" select="$i18n.Email"/>
									<xsl:with-param name="value" select="PersonQueryInstance/email"/>
									<xsl:with-param name="class" select="$classEmail"/>
									<xsl:with-param name="size" select="'50'"/>
								</xsl:call-template>
								
								<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $emailID]"/>
							
							</div>
							
							<xsl:variable name="phoneID">
								<xsl:value-of select="$shortQueryID"/>
										<xsl:value-of select="'_phone'"/>
							</xsl:variable>
							
							<xsl:variable name="classPhone">
								<xsl:if test="ValidationErrors/validationError[fieldName = $phoneID]">
									<xsl:text>invalid input-error</xsl:text>
								</xsl:if>
							</xsl:variable>
							
							<div class="split odd {$classPhone}">
							
								<label>
									<xsl:value-of select="$i18n.Phone"/>
								</label>
						
								<xsl:call-template name="createTextField">
									<xsl:with-param name="name" select="$phoneID"/>
									<xsl:with-param name="title" select="$i18n.Phone"/>
									<xsl:with-param name="value" select="PersonQueryInstance/phone"/>
									<xsl:with-param name="class" select="$classPhone"/>
									<xsl:with-param name="size" select="'50'"/>
								</xsl:call-template>
								
								<xsl:apply-templates select="ValidationErrors/validationError[fieldName = $phoneID]"/>
							
							</div>
							
						</fieldset>
						
					</xsl:if>
				
				</div>
				
			</article>
		
		</div>
		
		<script type="text/javascript">$(function(){initPersonQuery('<xsl:value-of select="PersonQueryInstance/PersonQuery/queryID" />');});</script>
		
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'BlockPosterSelection']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ValidationError.BlockPosterSelection"/>
			</strong>
		</span>
		
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'PersonNotFound']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ValidationError.PersonNotFound"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ProviderError']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ValidationError.ProviderError"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'EmailOrPhoneRequired']">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ValidationError.EmailOrPhoneRequired"/>
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'TooLong']">
		
		<xsl:variable name="message">
			<xsl:value-of select="$i18n.ValidationError.TooLongFieldContent.part1"/>
			<xsl:value-of select="currentLength"/>
			<xsl:value-of select="$i18n.ValidationError.TooLongFieldContent.part2"/>
			<xsl:value-of select="maxLength"/>
			<xsl:value-of select="$i18n.ValidationError.TooLongFieldContent.part3"/>
		</xsl:variable>
		
		<i data-icon-after="!" title="{$message}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.ValidationError.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'InvalidFormat']">
		
		<i data-icon-after="!" title="{$i18n.ValidationError.InvalidFormat}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.ValidationError.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>
	
</xsl:stylesheet>