<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>
	
	<xsl:template match="ShowQueryValues">
		
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="PlotQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<article class="plotquery show-mode">
				
				<div class="heading-wrapper">
				
					<h2>
						<xsl:value-of select="PlotQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
					</h2>
					
				</div>
				
				<xsl:if test="PlotQueryInstance/PlotQuery/description">
					<span class="italic">
						<xsl:value-of select="PlotQueryInstance/PlotQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'propertyUnitDesignation'" />
					<xsl:with-param name="value" select="PlotQueryInstance/propertyUnitDesignation" />
					<xsl:with-param name="text" select="$i18n.PropertyUnitDesignation" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'objectID'" />
					<xsl:with-param name="value" select="PlotQueryInstance/objectID" />
					<xsl:with-param name="text" select="$i18n.ObjectID" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'address'" />
					<xsl:with-param name="value" select="PlotQueryInstance/address" />
					<xsl:with-param name="text" select="$i18n.Address" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'location'" />
					<xsl:with-param name="value" select="PlotQueryInstance/location" />
					<xsl:with-param name="text" select="$i18n.Location" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'area'" />
					<xsl:with-param name="value" select="PlotQueryInstance/area" />
					<xsl:with-param name="text" select="$i18n.Area" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'connectionFee'" />
					<xsl:with-param name="value">
						<xsl:choose>
							<xsl:when test="PlotQueryInstance/connectionFee"><xsl:value-of select="PlotQueryInstance/connectionFee" /></xsl:when>
							<xsl:otherwise><xsl:value-of select="$i18n.NoConnectionFee" /></xsl:otherwise>
						</xsl:choose>
					</xsl:with-param>
					<xsl:with-param name="text" select="$i18n.ConnectionFee" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'price'" />
					<xsl:with-param name="value" select="PlotQueryInstance/price" />
					<xsl:with-param name="text" select="$i18n.Price" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>

				<xsl:call-template name="showPlotField">
					<xsl:with-param name="name" select="'comment'" />
					<xsl:with-param name="value" select="PlotQueryInstance/comment" />
					<xsl:with-param name="text" select="$i18n.Comment" />
					<xsl:with-param name="hiddenFields" select="HiddenFields" />
				</xsl:call-template>
				
			</article>
				
		</div>
		
	</xsl:template>	
	
	<xsl:template name="showPlotField">
	
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="text" />
		<xsl:param name="hiddenFields" />	
		
		<xsl:if test="not($hiddenFields/HiddenField = $name)">
		
			<p>
				<strong><xsl:value-of select="$text" /></strong>
				<br/>
				<xsl:value-of select="$value" />
			</p>
			
		</xsl:if>	
	
	</xsl:template>
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="shortQueryID" select="concat('q', PlotQueryInstance/PlotQuery/queryID)" />
	
		<xsl:variable name="queryID" select="concat('query_', PlotQueryInstance/PlotQuery/queryID)" />
	
		<div class="query plotquery" id="{$queryID}">
		
			<xsl:attribute name="class">
				<xsl:text>query plotquery</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="PlotQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
			
			<a name="{$queryID}" />
		
			<xsl:choose>
				<xsl:when test="ValidationErrors/validationError">

					<div id="{$queryID}-validationerrors" class="validationerrors">
						<xsl:if test="ValidationErrors/validationError[messageKey]">
							<div class="info-box error">
								<xsl:apply-templates select="ValidationErrors/validationError[messageKey]"/>
							</div>
						</xsl:if>
					</div>
					<article class="nopadding"></article>
					
				</xsl:when>
				<xsl:otherwise>
				
					<article>
						
							<xsl:attribute name="class">
			
								<xsl:if test="ValidationErrors/validationError[messageKey]">
									<xsl:text> error</xsl:text>
								</xsl:if>
								
							</xsl:attribute>
						
							<div class="heading-wrapper">
								
								<h2>
									<xsl:value-of select="PlotQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
								</h2>
								
								<xsl:if test="PlotQueryInstance/PlotQuery/helpText">		
									<xsl:apply-templates select="PlotQueryInstance/PlotQuery/helpText" />
								</xsl:if>
								
							</div>
							
							<xsl:if test="PlotQueryInstance/PlotQuery/description">
								<span class="italic">
									<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
									<xsl:value-of select="PlotQueryInstance/PlotQuery/description" disable-output-escaping="yes" />
								</span>		
							</xsl:if>	
							
							<fieldset>
							
								<div class="split">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'propertyUnitDesignation'" />
										<xsl:with-param name="value" select="PlotQueryInstance/propertyUnitDesignation" />
										<xsl:with-param name="text" select="$i18n.PropertyUnitDesignation" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
			
							
								<div class="split odd">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'objectID'" />
										<xsl:with-param name="value" select="PlotQueryInstance/objectID" />
										<xsl:with-param name="text" select="$i18n.ObjectID" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
							
								<div class="split">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'address'" />
										<xsl:with-param name="value" select="PlotQueryInstance/address" />
										<xsl:with-param name="text" select="$i18n.Address" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
			
				
								<div class="split odd">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'location'" />
										<xsl:with-param name="value" select="PlotQueryInstance/location" />
										<xsl:with-param name="text" select="$i18n.Location" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
			
								<div class="split">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'area'" />
										<xsl:with-param name="value" select="PlotQueryInstance/area" />
										<xsl:with-param name="text" select="$i18n.Area" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
			
								<div class="split odd">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'connectionFee'" />
										<xsl:with-param name="value">
											<xsl:choose>
												<xsl:when test="PlotQueryInstance/connectionFee"><xsl:value-of select="PlotQueryInstance/connectionFee" /></xsl:when>
												<xsl:otherwise><xsl:value-of select="$i18n.NoConnectionFee" /></xsl:otherwise>
											</xsl:choose>
										</xsl:with-param>
										<xsl:with-param name="text" select="$i18n.ConnectionFee" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
				
								<div class="split">
								
									<xsl:call-template name="disabledTextField">
										<xsl:with-param name="name" select="'price'" />
										<xsl:with-param name="value" select="PlotQueryInstance/price" />
										<xsl:with-param name="text" select="$i18n.Price" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
							
								<div class="split">

									<xsl:call-template name="disabledTextArea">
										<xsl:with-param name="name" select="'comment'" />
										<xsl:with-param name="value" select="PlotQueryInstance/comment" />
										<xsl:with-param name="text" select="$i18n.Comment" />
										<xsl:with-param name="hiddenFields" select="HiddenFields" />
									</xsl:call-template>
								</div>
			
						
							</fieldset>			
			
					</article>		
						
				</xsl:otherwise>
			</xsl:choose>

		</div>
		
	</xsl:template>
	
	<xsl:template name="disabledTextField">
	
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="text" />
		<xsl:param name="hiddenFields" />
	
		<xsl:if test="not($hiddenFields/HiddenField = $name)">
		
			<label for="{$name}" >
				<xsl:value-of select="$text" />
			</label>
			
			<xsl:call-template name="createTextField">
				<xsl:with-param name="id" select="$name" />
				<xsl:with-param name="name" select="$name" />
				<xsl:with-param name="value" select="$value" />
				<xsl:with-param name="title" select="$text" />
				<xsl:with-param name="size" select="50" />
				<xsl:with-param name="disabled" select="'true'" />
			</xsl:call-template>

		</xsl:if>
	
	</xsl:template>
	
	<xsl:template name="disabledTextArea">
	
		<xsl:param name="name" />
		<xsl:param name="value" />
		<xsl:param name="text" />
		<xsl:param name="hiddenFields" />
		
		<xsl:if test="not($hiddenFields/HiddenField = $name)">
		
			<label for="{$name}" >
				<xsl:value-of select="$text" />
			</label>
			
			<xsl:call-template name="createTextArea">
				<xsl:with-param name="id" select="$name" />
				<xsl:with-param name="name" select="$name" />
				<xsl:with-param name="value" select="$value" />
				<xsl:with-param name="title" select="$text" />
				<xsl:with-param name="disabled" select="'true'" />
				<xsl:with-param name="rows" select="6" />
			</xsl:call-template>

		</xsl:if>
		
	</xsl:template>
	
	<xsl:template match="validationError[validationErrorType = 'RequiredField']">
		
		<i data-icon-after="!" title="{$i18n.RequiredField}"></i>
		
	</xsl:template>
	
	<xsl:template match="validationError[messageKey != '']">
		
		<span>
			<strong data-icon-before="!">
				
				<xsl:choose>
					<xsl:when test="messageKey = 'UnableToGetDataError'">
		
						<xsl:value-of select="$i18n.UnableToGetDataErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotUnavailableError'">
					
						<xsl:value-of select="$i18n.PlotUnavailableErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotLockedError'">
					
						<xsl:value-of select="$i18n.PlotLockedErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotLockExpiredError'">
					
						<xsl:value-of select="$i18n.PlotLockExpiredErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotSoldError'">
					
						<xsl:value-of select="$i18n.PlotSoldErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotNotBookableError'">
					
						<xsl:value-of select="$i18n.PlotNotBookableErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotCurrentlyNotBookableError'">
					
						<xsl:value-of select="$i18n.PlotCurrentlyNotBookableErrorText" />
					
					</xsl:when>
					<xsl:when test="messageKey = 'PlotLockError'">
					
						<xsl:value-of select="$i18n.PlotLockErrorText" />
					
					</xsl:when>
				</xsl:choose>
			
			</strong>
		</span>
		
	</xsl:template>
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>	
		
</xsl:stylesheet>