package com.nordicpeak.flowengine.queries.predefineduserquery;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;
import com.nordicpeak.flowengine.queries.internaluserqueries.InternalUserQueriesAttributeDescriptor;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.PredefinedUserQuery;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.SelectedDropDownAttribute;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.SelectedManagerAttribute;
import com.nordicpeak.flowengine.queries.predefineduserquery.beans.SelectedUserAttribute;

public class PredefinedUserQueryCRUD extends BaseQueryCRUD<PredefinedUserQuery, PredefinedUserQueryProviderModule> {
	
	protected AnnotatedDAOWrapper<PredefinedUserQuery, Integer> queryDAO;
	
	public PredefinedUserQueryCRUD(AnnotatedDAOWrapper<PredefinedUserQuery, Integer> queryDAO, BeanRequestPopulator<PredefinedUserQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, PredefinedUserQueryProviderModule callback) {
		
		super(PredefinedUserQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.queryDAO = queryDAO;
	}
	
	@Override
	protected void appendUpdateFormData(PredefinedUserQuery query, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		
		super.appendUpdateFormData(query, doc, updateTypeElement, user, req, uriParser);
		
		if (query.getUserIDs() != null) {
			XMLUtils.append(doc, updateTypeElement, "Users", callback.getUserHandler().getUsers(query.getUserIDs(), false, true));
		}
		
		if (query.getGroupIDs() != null) {
			XMLUtils.append(doc, updateTypeElement, "Groups", callback.getGroupHandler().getGroups(query.getGroupIDs(), true));
		}
		
		if (query.getPreSelectedUserID() != null) {
			
			User preSelectedUser = callback.getUserHandler().getUser(query.getPreSelectedUserID(), false, true);
			XMLUtils.append(doc, updateTypeElement, "PreSelectedUsers", Collections.singletonList(preSelectedUser));
		}
		
		if (callback.getAttributeDescriptors() != null) {
			
			XMLUtils.append(doc, updateTypeElement, "AvailiableAttributes", callback.getAttributeDescriptors());
		}
		
		if (callback.isEnableClosestManager()) {
			XMLUtils.appendNewElement(doc, updateTypeElement, "ClosestManagerEnabled");
		}
	}
	
	@Override
	protected PredefinedUserQuery populateFromUpdateRequest(PredefinedUserQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		PredefinedUserQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		if (!callback.isEnableClosestManager()) {

			query.setShowManagerInfo(false);
			query.setRequireManagerSigning(false);
			query.setSelectedManagerAttributes(null);
			query.setLimitSelectionToMyEmployees(false);
		}
		
		if (query.getPreSelectedUserID() != null) {
			
			List<User> selectableUsers = callback.getSelectableUsers(query);
			
			boolean allowedUser = false;
			
			if (selectableUsers != null) {
				for (User selectableUser : selectableUsers) {
					
					if (selectableUser.getUserID().equals(query.getPreSelectedUserID())) {
						allowedUser = true;
						break;
					}
				}
			}
			
			if (!allowedUser) {
				validationErrors.add(new ValidationError("InvalidPreSelectedUser"));
			}
		}
		
		if (callback.getAttributeDescriptors() != null) {
			
			List<SelectedUserAttribute> selectedUserAttributes = new ArrayList<SelectedUserAttribute>();
			
			Integer sortIndex = 1;
			
			for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {
				
				if ("true".equals(req.getParameter("attribute-" + attributeDescriptor.getName()))) {
					
					selectedUserAttributes.add(new SelectedUserAttribute(attributeDescriptor, sortIndex++));
				}
			}
			
			query.setSelectedUserAttributes(selectedUserAttributes);
			
			List<SelectedDropDownAttribute> selectedDropDownAttributes = new ArrayList<SelectedDropDownAttribute>();
			
			for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {
				
				if ("true".equals(req.getParameter("attribute-dropdown-" + attributeDescriptor.getName()))) {
					
					selectedDropDownAttributes.add(new SelectedDropDownAttribute(attributeDescriptor));
				}
			}
			
			query.setSelectedDropDownAttributes(selectedDropDownAttributes);
			
			if (callback.isEnableClosestManager()) {

				List<SelectedManagerAttribute> selectedManagerAttributes = new ArrayList<SelectedManagerAttribute>();

				sortIndex = 1;
				
				for (InternalUserQueriesAttributeDescriptor attributeDescriptor : callback.getAttributeDescriptors()) {

					if ("true".equals(req.getParameter("managerattribute-" + attributeDescriptor.getName()))) {

						selectedManagerAttributes.add(new SelectedManagerAttribute(attributeDescriptor, sortIndex++));
					}
				}

				query.setSelectedManagerAttributes(selectedManagerAttributes);
			}
		}
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		return query;
	}
	
}
