package com.nordicpeak.flowengine.queries.pricetablequery;

import java.io.IOException;
import java.lang.reflect.Field;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.CRUDAction;
import se.unlogic.hierarchy.core.enums.EventTarget;
import se.unlogic.hierarchy.core.events.CRUDEvent;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.exceptions.URINotFoundException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableQuery;
import com.nordicpeak.flowengine.queries.pricetablequery.beans.PriceTableRow;

public class PriceTableQueryCRUD extends BaseQueryCRUD<PriceTableQuery, PriceTableQueryProviderModule> {
	
	
	protected AnnotatedDAOWrapper<PriceTableQuery, Integer> queryDAO;
	
	public PriceTableQueryCRUD(AnnotatedDAOWrapper<PriceTableQuery, Integer> queryDAO, BeanRequestPopulator<PriceTableQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, PriceTableQueryProviderModule callback) {
		
		super(PriceTableQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
		
		this.queryDAO = queryDAO;
	}
	
	@Override
	public PriceTableQuery getBean(Integer beanID) throws SQLException, AccessDeniedException {
		
		PriceTableQuery query = super.getBean(beanID);
		
		if (query != null && !CollectionUtils.isEmpty(query.getRows())) {
			
			for (PriceTableRow row : query.getRows()) {
				
				row.setQuery(query);
			}
		}
		
		return query;
	}
	
	@Override
	protected PriceTableQuery populateFromUpdateRequest(PriceTableQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {
		
		PriceTableQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> validationErrors = new ArrayList<>();
		
		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);
		
		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
		
		return query;
	}
	
	@Override
	protected void appendShowFormData(PriceTableQuery bean, Document doc, Element showTypeElement, User user, HttpServletRequest req, HttpServletResponse res, URIParser uriParser) throws SQLException, IOException, Exception {
		
		super.appendShowFormData(bean, doc, showTypeElement, user, req, res, uriParser);
		
		XMLUtils.appendNewElement(doc, showTypeElement, "showFlowURL", callback.getFlowAdminModule().getFlowQueryRedirectURL(req, bean.getQueryDescriptor().getStep().getFlow().getFlowID()));
	}
	
	@Override
	protected void appendUpdateFormData(PriceTableQuery bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {
		
		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);
		
		updateTypeElement.appendChild(bean.getQueryDescriptor().getStep().getFlow().getFlowType().toXML(doc));
	}
	
	@Override
	protected ForegroundModuleResponse beanUpdated(PriceTableQuery bean, HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) bean.getQueryDescriptor()), EventTarget.ALL);
		
		callback.getFlowAdminModule().addFlowFamilyEvent(callback.getFlowAdminModule().getEventQueryUpdatedMessage() + " \"" + bean.getQueryDescriptor().getName() + "\"", bean.getQueryDescriptor().getStep().getFlow(), user);
		
		callback.redirectToQueryConfig(bean, req, res);
		
		return null;
	}
	
	@Override
	protected List<Field> getBeanRelations() {
		
		return Arrays.asList(PriceTableQuery.ROWS_RELATION);
	}
	
	public ForegroundModuleResponse sortRows(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		PriceTableQuery query = this.getRequestedBean(req, res, user, uriParser, UPDATE);
		
		if (query != null) {
			
			this.checkUpdateAccess(query, user, req, uriParser);
			
			if (req.getMethod().equalsIgnoreCase("POST")) {
				
				List<PriceTableRow> tableRows = query.getRows();
				
				if (!CollectionUtils.isEmpty(tableRows)) {
					
					for (PriceTableRow tableRow : tableRows) {
						
						String sortIndex = req.getParameter("sortorder_" + tableRow.getRowID());
						
						if (NumberUtils.isInt(sortIndex)) {
							
							tableRow.setSortIndex(NumberUtils.toInt(sortIndex));
						}
					}
					
					queryDAO.update(query);
					
					callback.getEventHandler().sendEvent(QueryDescriptor.class, new CRUDEvent<>(CRUDAction.UPDATE, (QueryDescriptor) query.getQueryDescriptor()), EventTarget.ALL);
					
					callback.redirectToQueryConfig(query, req, res);
					
					return null;
				}
			}
			
			Document doc = this.callback.createDocument(req, uriParser, user);
			Element updateTypeElement = doc.createElement("SortRows");
			doc.getFirstChild().appendChild(updateTypeElement);
			
			updateTypeElement.appendChild(query.toXML(doc));
			
			return new SimpleForegroundModuleResponse(doc);
			
		}
		
		throw new URINotFoundException(uriParser);
	}
}
