package com.nordicpeak.flowengine.queries.pricetablequery.beans;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.BaseInvoiceLine;
import com.nordicpeak.flowengine.interfaces.ColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.PaymentQueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basetablequery.interfaces.TableQueryInstance;
import com.nordicpeak.flowengine.queries.pricetablequery.PriceTableQueryProviderModule;

@Table(name = "price_table_query_instances")
@XMLElement
public class PriceTableQueryInstance extends BaseQueryInstance implements StringValueQueryInstance, ColumnExportableQueryInstance, PaymentQueryInstance, TableQueryInstance {

	private static final long serialVersionUID = -8794537859820177303L;

	public static final Field VALUES_RELATION = ReflectionUtils.getField(PriceTableQueryInstance.class, "values");
	public static final Field QUERY_RELATION = ReflectionUtils.getField(PriceTableQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private PriceTableQuery query;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<PriceTableRowValue> values;

	private BigDecimal sum;

	@XMLElement(name = "sum")
	private String formattedSum;

	private BigDecimal count;

	@XMLElement(name = "count")
	private String formattedCount;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		values = null;
		setSum(null, null);
		setCount(null, null);

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}

		super.reset(attributeHandler);
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(query.getAmountSumAttributeName());
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		BigDecimal amountSum = BigDecimal.ZERO;

		if (values != null) {
			for (PriceTableRowValue value : values) {

				amountSum = amountSum.add(value.getValue());
			}
		}

		attributeHandler.setAttribute(query.getAmountSumAttributeName(), PriceTableQueryProviderModule.formatBigDecimal(amountSum, query.getDecimalSetting()));
	}

	@Override
	public PriceTableQuery getQuery() {

		return query;
	}

	public void setQuery(PriceTableQuery query) {

		this.query = query;
	}

	public List<PriceTableRowValue> getValues() {

		return values;
	}

	public void setValues(List<PriceTableRowValue> values) {

		this.values = values;
	}

	public PriceTableRowValue getRowValue(String rowIdentifier) {

		return getRowValue(values, rowIdentifier);
	}

	@Override
	public String getStringValue() {

		StringBuilder stringBuilder = new StringBuilder();

		stringBuilder.append(sum);
		stringBuilder.append(" kr");

		if (!CollectionUtils.isEmpty(values)) {
			for (PriceTableRowValue value : values) {

				stringBuilder.append(", ");
				stringBuilder.append(value.getRow().getLabel() + ":" + PriceTableQueryProviderModule.formatBigDecimal(value.getValue(), query.getDecimalSetting()));

				if (!StringUtils.isEmpty(value.getComment())) {

					stringBuilder.append(" \"" + value.getComment() + "\"");
				}
			}
		}

		return stringBuilder.toString();

	}

	protected List<PriceTableRow> getRows() {

		return query.getRows();
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	public PriceTableRowValue getRowValue(List<PriceTableRowValue> values, String rowIdentifier) {

		if (values != null) {

			for (PriceTableRowValue rowValue : values) {

				if (rowValue.getRow().getRowIdentifier().equals(rowIdentifier)) {

					return rowValue;
				}
			}
		}

		return null;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewElement(doc, element, "Sum", sum);

		if (getValues() != null) {

			for (PriceTableRow row : getRows()) {

				Element rowElement = doc.createElement("PriceTableRow");

				XMLUtils.appendNewElement(doc, rowElement, "ID", row.getRowID());
				XMLUtils.appendNewCDATAElement(doc, rowElement, "Name", row.getName());
				XMLUtils.appendNewElement(doc, rowElement, "Price", PriceTableQueryProviderModule.formatBigDecimal(row.getPrice(), query.getDecimalSetting()));

				if (StringUtils.isEmpty(row.getSum()) || row.getSum().equals("?")) {

					XMLUtils.appendNewElement(doc, rowElement, "Sum", 0);

				} else {

					XMLUtils.appendNewElement(doc, rowElement, "Sum", row.getSum());
				}

				PriceTableRowValue rowValue = getRowValue(row.getRowIdentifier());

				if (rowValue != null && rowValue.getValue() != null) {

					XMLUtils.appendNewElement(doc, rowElement, "Value", PriceTableQueryProviderModule.formatBigDecimal(rowValue.getValue(), query.getDecimalSetting()));
					XMLUtils.appendNewElement(doc, rowElement, "Comment", rowValue.getComment());
				}

				if ((rowValue != null && rowValue.getValue() != null) || row.isRequired()) {
					element.appendChild(rowElement);
				}
			}
		}

		return element;
	}

	@Override
	public List<String> getColumnLabels(QueryHandler queryHandler) {

		PriceTableQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryInstanceDescriptor().getQueryDescriptor().getQueryTypeID(), PriceTableQueryProviderModule.class);

		if (sum == null) {

			queryProvider.calculateQueryInstanceSummary(this, getRows());
		}

		List<String> labels = new ArrayList<String>();

		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportSum());

		if (getRows() != null) {

			for (PriceTableRow row : getRows()) {

				labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + row.getLabel() + " " + PriceTableQueryProviderModule.formatBigDecimal(row.getPrice(), query.getDecimalSetting()) + " kr (" + queryProvider.getExportAmount() + ")");
			}

			if (query.isShowComments()) {

				for (PriceTableRow row : getRows()) {

					labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + row.getLabel() + " (" + queryProvider.getExportComment() + ")");
				}
			}
		}

		return labels;
	}

	@Override
	public List<String> getColumnValues() {

		List<String> values = new ArrayList<String>();

		values.add(formattedSum);

		if (getRows() != null) {

			for (PriceTableRow row : getRows()) {

				PriceTableRowValue value = getRowValue(row.getRowIdentifier());

				if (value != null) {

					values.add(PriceTableQueryProviderModule.formatBigDecimal(value.getValue(), query.getDecimalSetting()));

				} else {

					values.add("0");
				}
			}

			if (query.isShowComments()) {

				for (PriceTableRow row : getRows()) {

					PriceTableRowValue value = getRowValue(row.getRowIdentifier());

					if (value != null) {

						values.add(value.getComment());

					} else {

						values.add(null);
					}
				}
			}
		}

		return values;
	}

	/** Returns null if sum is not calculated or could not be calculated
	 * 
	 * @return */
	public BigDecimal getSum() {

		return sum;
	}

	public void setSum(BigDecimal sum, String formattedSum) {

		this.sum = sum;
		this.formattedSum = formattedSum;

		query.getColumns().get(0).setSum(sum);
	}

	public String getFormattedSum() {

		return formattedSum;
	}

	public BigDecimal getCount() {

		return count;
	}

	public void setCount(BigDecimal count, String formattedCount) {

		this.count = count;
		this.formattedCount = formattedCount;
		
		query.getColumns().get(1).setSum(count);
	}

	public String getFormattedCount() {

		return formattedCount;
	}

	@Override
	public List<BaseInvoiceLine> getInvoiceLines() {

		if (getRows() != null) {

			List<BaseInvoiceLine> invoiceLines = new ArrayList<BaseInvoiceLine>(getRows().size());

			for (PriceTableRow row : getRows()) {

				PriceTableRowValue value = getRowValue(row.getRowIdentifier());

				if (value != null) {

					if (row.getPrice().compareTo(BigDecimal.ZERO) > 0 && value.getValue().compareTo(BigDecimal.ZERO) > 0) {

						invoiceLines.add(new BaseInvoiceLine(value.getValue(), row.getPrice(), row.getName(), ""));
					}
				}
			}

			return invoiceLines;
		}

		return null;
	}

}
