package com.nordicpeak.flowengine.queries.priorityquery;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.populators.AlternativesPopulator;

public class PriorityAlternativesPopulator extends AlternativesPopulator<PriorityAlternative> {

	public PriorityAlternativesPopulator(Class<PriorityAlternative> alternativeClass) {

		super(alternativeClass);
	}

	public PriorityAlternativesPopulator(Class<PriorityAlternative> alternativeClass, String xmlParentNodeName) {

		super(alternativeClass, xmlParentNodeName);
	}

	@Override
	public List<PriorityAlternative> populate(List<PriorityAlternative> currentAlternatives, HttpServletRequest req, List<ValidationError> validationErrors) {

		String[] alternativeIDs = req.getParameterValues("alternativeID");

		List<PriorityAlternative> alternatives = new ArrayList<PriorityAlternative>();

		if (alternativeIDs != null) {

			for (String alternativeID : alternativeIDs) {

				String name = ValidationUtils.validateParameter("alternative_" + alternativeID, req, true, 1, 255, validationErrors);

				if (name == null) {

					continue;
				}

				String sortOrder = req.getParameter("sortorder_" + alternativeID);
				boolean allowMultiple = req.getParameter("allowmultiple_" + alternativeID) != null;

				if (NumberUtils.isInt(sortOrder)) {

					PriorityAlternative alternative = this.getNewAlternativeInstance();

					alternative.setName(name);
					alternative.setSortIndex(NumberUtils.toInt(sortOrder));
					alternative.setAllowMultiple(allowMultiple);

					if (NumberUtils.isInt(alternativeID)) {

						this.checkForExistingAlternatives(currentAlternatives, alternative, NumberUtils.toInt(alternativeID));
					}

					alternatives.add(alternative);
				}
			}
		}

		return alternatives;
	}
}
