package com.nordicpeak.flowengine.queue.beans;

import java.io.Serializable;

import se.unlogic.emailutils.populators.EmailPopulator;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.populators.StringSwedishPhoneNumberPopulator;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.queries.contactdetailquery.ContactDetailQueryInstance;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.personprovider.Person;

@XMLElement
public abstract class BaseContact extends GeneratedElementable implements Serializable {

	private static final long serialVersionUID = 7634780772195411124L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 50)
	@XMLElement
	private String firstname;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 50)
	@XMLElement
	private String lastname;

	@DAOManaged
	@WebPopulate(maxLength = 50, populator = EmailPopulator.class)
	@XMLElement
	private String email;

	@DAOManaged
	@WebPopulate(maxLength = 50, populator = StringSwedishPhoneNumberPopulator.class)
	@XMLElement
	private String phone;

	@DAOManaged
	@WebPopulate(maxLength = 50, populator = StringSwedishPhoneNumberPopulator.class)
	@XMLElement
	private String mobilePhone;

	@DAOManaged
	@XMLElement
	private String citizenIdentifier;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String address;

	@DAOManaged
	@WebPopulate(maxLength = 50)
	@XMLElement
	private String zipCode;

	@DAOManaged
	@WebPopulate(maxLength = 50)
	@XMLElement
	private String postalAddress;

	public BaseContact() {}

	public BaseContact(User user) {

		firstname = user.getFirstname();
		lastname = user.getLastname();

		AttributeHandler attributeHandler = user.getAttributeHandler();

		if (attributeHandler != null) {

			citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);
			mobilePhone = attributeHandler.getString("mobilePhone");
			phone = attributeHandler.getString("phone");
			address = attributeHandler.getString("address");
			postalAddress = attributeHandler.getString("postalAddress");
			zipCode = attributeHandler.getString("zipCode");
		}
	}

	public BaseContact(ContactDetailQueryInstance contactDetailQueryInstance, User user) {

		if (user != null) {

			firstname = user.getFirstname();
			lastname = user.getLastname();
			citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		} else {

			firstname = contactDetailQueryInstance.getFirstname();
			lastname = contactDetailQueryInstance.getLastname();
			citizenIdentifier = contactDetailQueryInstance.getCitizenID();
		}

		email = contactDetailQueryInstance.getEmail();
		phone = contactDetailQueryInstance.getPhone();
		mobilePhone = contactDetailQueryInstance.getMobilePhone();
		address = contactDetailQueryInstance.getAddress();
		zipCode = contactDetailQueryInstance.getZipCode();
		postalAddress = contactDetailQueryInstance.getPostalAddress();
	}

	public BaseContact(Person person, User user) {

		firstname = person.getFirstname();
		lastname = person.getLastname();
		citizenIdentifier = person.getCitizenIdentifier();
		address = person.getAddress();
		zipCode = person.getZipCode();
		postalAddress = person.getPostalAddress();

		if (user != null) {

			email = user.getEmail();

			AttributeHandler attributeHandler = user.getAttributeHandler();

			if (attributeHandler != null) {

				mobilePhone = attributeHandler.getString("mobilePhone");
				phone = attributeHandler.getString("phone");
			}
		}
	}

	public String getMobilePhone() {

		return mobilePhone;
	}

	public void setMobilePhone(String mobilePhone) {

		this.mobilePhone = mobilePhone;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getPhone() {

		return phone;
	}

	public void setPhone(String phone) {

		this.phone = phone;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public void setCitizenIdentifier(String citizenIdentifier) {

		this.citizenIdentifier = citizenIdentifier;
	}

	public String getFirstname() {

		return firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public String getCitizenIdentifier() {

		return citizenIdentifier;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getZipCode() {

		return zipCode;
	}

	public void setZipCode(String zipcode) {

		this.zipCode = zipcode;
	}

	public String getPostalAddress() {

		return postalAddress;
	}

	public void setPostalAddress(String postalAddress) {

		this.postalAddress = postalAddress;
	}

}
