package com.nordicpeak.flowengine.queue.query;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.BeanRequestPopulator;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUD;
import com.nordicpeak.flowengine.queue.beans.Queue;

public class QueueQueryCRUD extends BaseQueryCRUD<QueueQuery, QueueQueryProviderModule> {

	public QueueQueryCRUD(AnnotatedDAOWrapper<QueueQuery, Integer> queryDAO, BeanRequestPopulator<QueueQuery> populator, String typeElementName, String typeLogName, String listMethodAlias, QueueQueryProviderModule callback) {

		super(QueueQuery.class, queryDAO, populator, typeElementName, typeLogName, listMethodAlias, callback);
	}

	@Override
	protected void appendUpdateFormData(QueueQuery query, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(query, doc, updateTypeElement, user, req, uriParser);

		Element queuesElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Queues");

		List<Queue> queues = callback.getQueues();

		if (!CollectionUtils.isEmpty(queues)) {
			for (Queue queue : queues) {

				queuesElement.appendChild(queue.toXML(doc));
			}
		}

		if (query.getQueue() != null && (CollectionUtils.isEmpty(queues) || !queues.contains(query.getQueue()))) {

			queuesElement.appendChild(query.getQueue().toXML(doc));
		}
	}

	@Override
	protected QueueQuery populateFromUpdateRequest(QueueQuery bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		QueueQuery query = super.populateFromUpdateRequest(bean, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<>();

		this.populateQueryDescriptor((QueryDescriptor) query.getQueryDescriptor(), req, validationErrors);

		Integer queueID = ValidationUtils.validateParameter("queueID", req, true, IntegerPopulator.getPopulator(), validationErrors);

		if (queueID != null) {

			Queue queue = callback.getQueue(queueID);

			if (queue == null || !queue.isEnabled()) {

				if (query.getQueue() == null || !query.getQueue().getQueueID().equals(queueID)) {

					validationErrors.add(new ValidationError("queueID", ValidationErrorType.InvalidFormat));
				}

			} else {

				query.setQueue(queue);
			}
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}

		return query;
	}

	@Override
	protected List<Field> getBeanRelations() {

		return Arrays.asList(QueueQuery.QUEUE_RELATION);
	}

}
