<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
		/jquery/jquery-migrate.js?v=1
		/featherlight/js/featherlight.min.js
	</xsl:variable>
	
	<xsl:variable name="globallinks">
		/featherlight/css/featherlight.min.css?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/randombookingnumbergenerator.js
	</xsl:variable>

	<xsl:variable name="links">
		/css/randombookingnumbergenerator.css
	</xsl:variable>

	<xsl:template match="Document">
				
		<xsl:choose>
			<xsl:when test="SelectBookingSlotInterval or ShowAdminBookingNumber or ShowUserBookingNumber">

					<xsl:apply-templates select="SelectBookingSlotInterval" />
					<xsl:apply-templates select="ShowAdminBookingNumber" />
					<xsl:apply-templates select="ShowUserBookingNumber" />

			</xsl:when>
			<xsl:otherwise>
						
				<div id="RandomBookingNumberGeneratorModule" class="contentitem errands-wrapper">
						
					<xsl:apply-templates select="ListIntervals" />
					<xsl:apply-templates select="AddInterval" />
					<xsl:apply-templates select="UpdateInterval" />
						
				</div>
				
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template match="ListIntervals">
		
		<h1>
			<xsl:value-of select="/Document/module/name" />
		</h1>
	
		<table class="oep-table bigmarginbottom">
			<thead>
				<tr>
					<th class="full">
						<xsl:value-of select="$i18n.Interval.Name" />
					</th>
					<th>
						<xsl:value-of select="$i18n.Interval.Enabled" />
					</th>
					<th>
						<xsl:value-of select="$i18n.Interval.Range" />
					</th>
					<th>
						<!-- Buttons -->
					</th>
				</tr>
			</thead>
			<tbody>
			
				<xsl:choose>
					<xsl:when test="Intervals/RandomBookingInterval">
							
						<xsl:apply-templates select="Intervals/RandomBookingInterval" mode="list" />
						
					</xsl:when>
					<xsl:otherwise>
							
						<tr>
							<td colspan="4">
								<xsl:value-of select="$i18n.NoIntervals" />
							</td>
						</tr>
						
					</xsl:otherwise>
				</xsl:choose>
			
			</tbody>
		</table>
		
		<div>

			<div class="buttons">
	
				<a class="btn btn-green btn-right" href="{/Document/ModuleURI}/addinterval">
					<xsl:value-of select="$i18n.AddInterval" />
				</a>
			
			</div>
		
			<xsl:call-template name="createAttributeTable" />
	
		</div>
	
	</xsl:template>
	
	<xsl:template match="RandomBookingInterval" mode="list">
	
		<tr>
			<td>
				<xsl:value-of select="name" />
			</td>
			<td>
				<xsl:choose>
					<xsl:when test="enabled = 'true'">
						<xsl:value-of select="$i18n.Yes" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="$i18n.No" />
					</xsl:otherwise>
				</xsl:choose>
			</td>
			<td>
				<xsl:value-of select="count(Numbers/RandomBookingNumber)" />
			</td>
			<td class="nowrap">
				<a class="btn btn-green btn-inline" href="{/Document/ModuleURI}/updateinterval/{intervalID}">
					<xsl:value-of select="$i18n.Update" />
				</a>
				<xsl:choose>
					<xsl:when test="noDeleteAccess">
						<a class="btn btn-light btn-inline" href="javascript:void(0)" onclick="return alert('{$i18n.DeleteInterval.Disabled}');">
							<xsl:value-of select="$i18n.Delete" />
						</a>
					</xsl:when>
					<xsl:otherwise>
						<a class="btn btn-red btn-inline" href="{/Document/ModuleURI}/deleteinterval/{intervalID}" onclick="return confirm('{$i18n.DeleteInterval.Confirm}: {name}?');">
							<xsl:value-of select="$i18n.Delete" />
						</a>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>	
	
	</xsl:template>
	
	<xsl:template match="AddInterval">
	
		<h1>
			<xsl:value-of select="$i18n.AddInterval" />
		</h1>
		
		<form method="POST" action="{/Document/ModuleURI}/addinterval">
			
			<xsl:call-template name="IntervalFormBaseFields" />

			<xsl:call-template name="IntervalFormAddFields" />
		
			<article class="buttons">
				<input type="submit" value="{$i18n.Add}" class="btn btn-green btn-inline floatright" />
				<a href="{/Document/ModuleURI}/listintervals" class="btn btn-light btn-inline floatright" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
			</article>
		
		</form>
	
	</xsl:template>
	
	<xsl:template match="UpdateInterval">
	
		<h1>
			<xsl:value-of select="$i18n.UpdateInterval" />
			<xsl:text>: </xsl:text>
			<xsl:value-of select="RandomBookingInterval/name" />
		</h1>
		
		<form method="POST" action="{/Document/ModuleURI}/updateinterval/{RandomBookingInterval/intervalID}">
			
			<xsl:call-template name="IntervalFormBaseFields" />
			
			<xsl:call-template name="IntervalFormUpdateFields" />
		
			<article class="buttons">
				<input type="submit" value="{$i18n.SaveChanges}" class="btn btn-green btn-inline floatright" />
				<a href="{/Document/ModuleURI}/listintervals" class="btn btn-light btn-inline floatright" onclick="return confirm('{$i18n.CancelConfirm}?');"><xsl:value-of select="$i18n.Cancel" /></a>
			</article>
		
		</form>
		
	</xsl:template>
	
	<xsl:template name="IntervalFormBaseFields">
		
		<article class="clearfix bigmarginbottom">
		
			<xsl:if test="validationException/validationError">
				<div class="bigmarginbottom">
					<xsl:apply-templates select="validationException/validationError"/>
				</div>
			</xsl:if>
			
			<div class="floatleft full bigmarginbottom">
				<label for="name" class="required full nomargin">
					<xsl:value-of select="$i18n.Interval.Name" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'name'" />
					<xsl:with-param name="id" select="'name'" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="element" select="RandomBookingInterval" />
				</xsl:call-template>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="'enabled'" />
					<xsl:with-param name="name" select="'enabled'" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="RandomBookingInterval" />
				</xsl:call-template>
				
				<label class="nomargin" for="enabled">
					<xsl:value-of select="$i18n.Interval.Enabled" />
				</label>
			</div>
			
		</article>
		
	</xsl:template>
			
	<xsl:template name="IntervalFormAddFields">
		
		<article class="clearfix bigmarginbottom">
		
			<div class="floatleft full bigmarginbottom">
				<label for="name" class="required full nomargin">
					<xsl:value-of select="$i18n.Interval.Range" />
				</label>
				
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="'range'" />
					<xsl:with-param name="id" select="'range'" />
					<xsl:with-param name="size" select="40" />
				</xsl:call-template>
			</div>
			
		</article>
		
	</xsl:template>
			
	<xsl:template name="IntervalFormUpdateFields">
		
		<article class="clearfix bigmarginbottom">
		
			<table class="oep-table">
				<thead>
					<tr>
						<th class="ten">
							<xsl:value-of select="$i18n.Number.Index" />
						</th>
						<th class="ten">
							<xsl:value-of select="$i18n.Number.Enabled" />
						</th>
						<th>
							<xsl:value-of select="$i18n.Number.Description" />
						</th>
					</tr>
				</thead>
				<tbody>
				
					<xsl:apply-templates select="RandomBookingInterval/Numbers/RandomBookingNumber" />
				
				</tbody>
			</table>
			
		</article>
		
	</xsl:template>
			
	<xsl:template match="RandomBookingNumber">
		
		<tr>
			<td>
				<xsl:value-of select="index" />
			</td>
			<td>
				<xsl:call-template name="createCheckbox">
					<xsl:with-param name="id" select="concat(numberID, '-enabled')" />
					<xsl:with-param name="name" select="concat(numberID, '-enabled')" />
					<xsl:with-param name="value" select="'true'" />
					<xsl:with-param name="element" select="." />
					<xsl:with-param name="elementName" select="'enabled'" />
				</xsl:call-template>
			</td>
			<td>
				<xsl:call-template name="createTextField">
					<xsl:with-param name="name" select="concat(numberID, '-description')" />
					<xsl:with-param name="id" select="concat(numberID, '-description')" />
					<xsl:with-param name="size" select="40" />
					<xsl:with-param name="element" select="." />
					<xsl:with-param name="value" select="description" />
				</xsl:call-template>
			</td>
		</tr>
		
	</xsl:template>
		
	<xsl:template match="SelectBookingSlotInterval">

		<xsl:if test="validationException/validationError">
			<div class="bigmargintop">
				<xsl:apply-templates select="validationException/validationError"/>
			</div>
		</xsl:if>

		<div class="full">
			
			<label for="randomBookingInterval">
				<xsl:value-of select="$i18n.BookingSlot.Interval" />
			</label>			
		
			<xsl:call-template name="createDropdown">
				<xsl:with-param name="id" select="'randomBookingInterval'"/>
				<xsl:with-param name="name" select="'randomBookingInterval'"/>
				<xsl:with-param name="element" select="Intervals/RandomBookingInterval"/>
				<xsl:with-param name="labelElementName" select="'name'"/>
				<xsl:with-param name="valueElementName" select="'intervalID'"/>
				<xsl:with-param name="selectedValue" select="SelectedIntervalID"/>
				<xsl:with-param name="addEmptyOption" select="$i18n.BookingSlot.Interval.Choose"/>
			</xsl:call-template>
		
		</div>
	
	</xsl:template>
		
	<xsl:template match="ShowAdminBookingNumber">
	
		<div class="random-booking-number">
		
			<span>
				<xsl:value-of select="$i18n.Booking.Number" />
				<xsl:text>: </xsl:text>
				
				<a id="randomBookingNumberLink" href="javascript:void(0)" data-icon-after="w">
					<xsl:choose>
						<xsl:when test="RandomBookingNumberMapping">
							
							<xsl:apply-templates select="RandomBookingNumberMapping" />						
							
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="$i18n.Booking.Number.None" />
						
						</xsl:otherwise>
					</xsl:choose>
				</a>
				
			</span>
			
		</div>
		
		<div style="display: none;">
	
			<div id="selectRandomBookingNumberModal" class="modal-content contentitem">
			
				<div class="modal-header bigmarginbottom">
					<h2>
						<xsl:value-of select="$i18n.Modal.ChooseNumber"/>
					</h2>
				</div>
				
				<div class="modal-body">
				
					<xsl:if test="validationError">
						<div class="bigmarginbottom">
							<xsl:apply-templates select="validationError"/>
						</div>
					</xsl:if>

					<form method="post" action="{/Document/requestinfo/uri}">
						
						<div class="bigmargintop bigmarginbottom">
							<xsl:call-template name="createNumberDropdown">
								<xsl:with-param name="id" select="'randomBookingNumber'"/>
								<xsl:with-param name="name" select="'randomBookingNumber'"/>
								<xsl:with-param name="class" select="'full'"/>
								<xsl:with-param name="valueElementName" select="'numberID'"/>
								<xsl:with-param name="labelElementName" select="'index'"/>
								<xsl:with-param name="labelDelimiter" select="' - '" />
								<xsl:with-param name="labelElementName2" select="'description'" />
								<xsl:with-param name="element" select="Numbers/RandomBookingNumber"/>
								<xsl:with-param name="selectedValue" select="RandomBookingNumberMapping/RandomBookingNumber/numberID"/>
								<xsl:with-param name="addEmptyOption" select="$i18n.Booking.Number.None"/>
							</xsl:call-template>
						</div>
								
						<div class="modal-buttons">
							<input type="submit" value="{$i18n.Modal.Submit}" class="btn btn-green floatright" />
							<input type="button" value="{$i18n.Modal.Cancel}" class="btn btn-light floatright cancel bigmarginright" />
						</div>
					
					</form>

				</div>
				
			</div>
		
		</div>	
	
	</xsl:template>
		
	<xsl:template match="ShowUserBookingNumber">
	
		<div class="random-booking-number">
		
			<span>
				<xsl:value-of select="$i18n.Booking.Number" />
				<xsl:text>: </xsl:text>

				<xsl:apply-templates select="RandomBookingNumberMapping" />						

			</span>
			
		</div>
	
	</xsl:template>
	
	<xsl:template match="RandomBookingNumberMapping">
	
		<xsl:value-of select="index" />
	
		<span class="random-booking-number-description">
			<xsl:if test="description">
				<xsl:text> - </xsl:text>
				<xsl:value-of select="description" />
			</xsl:if>
		</span>
		
	</xsl:template>
	
	<xsl:template name="createNumberDropdown">

		<xsl:param name="id" select="null" />
		<xsl:param name="name" select="null" />
		<xsl:param name="class" select="null" />
		<xsl:param name="valueElementName" select="null" />
		<xsl:param name="labelElementName" select="null" />
		<xsl:param name="labelDelimiter" select="null" />
		<xsl:param name="labelElementName2" select="null" />
		<xsl:param name="element" />
		<xsl:param name="selectedValue" select="''" />
		<xsl:param name="requestparameters" select="requestparameters" />
		<xsl:param name="addEmptyOption" select="null" />

		<select>
			<xsl:if test="$class != ''">
				<xsl:attribute name="class">
					<xsl:value-of select="$class" /> 
				</xsl:attribute>
			</xsl:if>

			<xsl:if test="$name != ''">
				<xsl:attribute name="name">
					<xsl:value-of select="$name" /> 
				</xsl:attribute>
			</xsl:if>

			<xsl:if test="$id != ''">
				<xsl:attribute name="id">
					<xsl:value-of select="$id" />
				</xsl:attribute>
			</xsl:if>

			<xsl:if test="$addEmptyOption != ''">
				<option value="">
					<xsl:value-of select="$addEmptyOption" />
				</option>
			</xsl:if>

			<xsl:variable name="paramValue" select="$requestparameters/parameter[name=$name]/value" />

			<xsl:for-each select="$element">
				<option value="{*[name()=$valueElementName]}">
					<xsl:choose>
						<xsl:when test="$requestparameters">
							<xsl:if test="$paramValue = *[local-name()=$valueElementName]">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<xsl:if test="*[name()=$valueElementName] = $selectedValue">
								<xsl:attribute name="selected">selected</xsl:attribute>
							</xsl:if>
						</xsl:otherwise>
					</xsl:choose>

					<xsl:value-of select="*[name()=$labelElementName]" />
					
					<xsl:if test="*[name()=$labelElementName2]">
	
						<xsl:if test="$labelDelimiter">
							<xsl:text>&#160;</xsl:text>
							<xsl:value-of select="$labelDelimiter" />
						</xsl:if>

						<xsl:value-of select="*[name()=$labelElementName2]" />
						
					</xsl:if>
				</option>
			</xsl:for-each>
		</select>

	</xsl:template>
	
	<xsl:template name="createAttributeTable">
	
		<div class="attribute-table">
		
			<p class="floatleft clearboth"><xsl:value-of select="$i18n.Attributes.Description" /></p>
			
			<table class="floatleft clearboth border">
				<thead>
					<th><xsl:value-of select="$i18n.Attributes.Name" /></th>
					<th><xsl:value-of select="$i18n.Attributes.Value" /></th>
				</thead>
				<tbody>
					<tr>
						<td>prefix.randomBookingNumberID</td>
						<td><xsl:value-of select="$i18n.Attribute.ID" /></td>
					</tr>
					<tr>
						<td>prefix.randomBookingNumberDescription</td>
						<td><xsl:value-of select="$i18n.Attribute.Description" /></td>
					</tr>
				</tbody>
			</table>
			
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'ExistingNumberMappings']">
		<p class="error">
			<xsl:value-of select="$i18n.validation.ExistingNumberMappings" />
		</p>
	</xsl:template>
	
	<xsl:template match="validationError">
		<xsl:if test="fieldName and validationErrorType and not(messageKey)">
			<p class="error">
				<xsl:choose>
					<xsl:when test="validationErrorType='RequiredField'">
						<xsl:value-of select="$i18n.validation.requiredField" />
					</xsl:when>
					<xsl:when test="validationErrorType='InvalidFormat'">
						<xsl:value-of select="$i18n.validation.invalidFormat" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooShort'">
						<xsl:value-of select="$i18n.validation.tooShort" />
					</xsl:when>
					<xsl:when test="validationErrorType='TooLong'">
						<xsl:value-of select="$i18n.validation.tooLong" />
					</xsl:when>														
					<xsl:otherwise>
						<xsl:value-of select="$i18n.validation.unknownError" />
					</xsl:otherwise>
				</xsl:choose>
				
				<xsl:text>&#x20;</xsl:text>
				
				<xsl:choose>
					<xsl:when test="displayName">
						<xsl:value-of select="displayName"/>
					</xsl:when>
					<xsl:when test="fieldName = 'name'">
						<xsl:value-of select="$i18n.Interval.Name"/>
					</xsl:when>
					<xsl:when test="fieldName = 'range'">
						<xsl:value-of select="$i18n.Interval.Range"/>
					</xsl:when>
					<xsl:when test="fieldName = 'description'">
						<xsl:value-of select="$i18n.Number.Description"/>
					</xsl:when>
					<xsl:when test="fieldName = 'randomBookingInterval'">
						<xsl:value-of select="$i18n.BookingSlot.Interval"/>
					</xsl:when>
					<xsl:when test="fieldName = 'randomBookingNumber'">
						<xsl:value-of select="$i18n.Booking.Number"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="fieldName"/>
					</xsl:otherwise>
				</xsl:choose>			
			</p>
		</xsl:if>
		
		<xsl:if test="messageKey">
			<p class="error">
				<xsl:value-of select="$i18n.validation.unknownFault" />
			</p>
		</xsl:if>
		
	</xsl:template>	
	
</xsl:stylesheet>