package com.nordicpeak.flowengine.randomevent;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.CheckboxSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.EventListener;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.LinkTag;
import se.unlogic.hierarchy.core.beans.ScriptTag;
import se.unlogic.hierarchy.core.beans.SimpleForegroundModuleResponse;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.enums.EventSource;
import se.unlogic.hierarchy.core.exceptions.AccessDeniedException;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.core.utils.AccessUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.hierarchy.core.utils.ModuleViewFragmentTransformer;
import se.unlogic.hierarchy.core.utils.ViewFragmentModule;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.hierarchy.foregroundmodules.staticcontent.StaticContentModule;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.RelationQuery;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.random.RandomUtils;
import se.unlogic.standardutils.time.TimeUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.FlowAdminModule;
import com.nordicpeak.flowengine.MessageHandler;
import com.nordicpeak.flowengine.beans.ExternalMessage;
import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.FlowAdminExtensionShowView;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.beans.FlowInstanceEvent;
import com.nordicpeak.flowengine.enums.EventType;
import com.nordicpeak.flowengine.events.SubmitEvent;
import com.nordicpeak.flowengine.interfaces.FlowAdminExtensionViewProvider;
import com.nordicpeak.flowengine.managers.FlowInstanceManager;
import com.nordicpeak.flowengine.utils.FlowInstanceUtils;

public class RandomEventGeneratorModule extends AnnotatedForegroundModule implements FlowAdminExtensionViewProvider, ViewFragmentModule<ForegroundModuleDescriptor> {

	private static final AnnotatedRequestPopulator<RandomEventSettings> POPULATOR = new AnnotatedRequestPopulator<>(RandomEventSettings.class);
	
	private static final RelationQuery RELATION_QUERY = new RelationQuery(FlowInstance.ATTRIBUTES_RELATION);
	
	private static final RelationQuery EVENT_ATTRIBUTE_RELATION_QUERY = new RelationQuery(FlowInstanceEvent.ATTRIBUTES_RELATION);
	
	@XSLVariable(name = "java.extensionViewTitle")
	private String extensionViewTitle;
	
	@XSLVariable(prefix = "java.")
	private String flowEventDescription = "RandomEventGenerator";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Provider priority", description = "The priority of this provider", required = true)
	private Integer priority = 100;

	@ModuleSetting
	@CheckboxSettingDescriptor(name = "Enable fragment XML debug", description = "Enables debugging of fragment XML")
	private boolean debugFragmententXML;

	@ModuleSetting(allowsNull = true)
	@TextAreaSettingDescriptor(name = "Supported actionID's", description = "The action ID's which will trigger export XML to be generated and stored when a submit event is detected", required = true)
	private List<String> supportedActionIDs;

	@ModuleSetting(allowsNull = true)
	@TextFieldSettingDescriptor(name = "Editor CSS", description = "Path to the desired CSS stylesheet for CKEditor (relative from the contextpath)", required = false)
	protected String cssPath;
	
	@InstanceManagerDependency(required = true)
	private StaticContentModule staticContentModule;

	@InstanceManagerDependency(required = true)
	protected MessageHandler messageHandler;
	
	private FlowAdminModule flowAdminModule;

	private AnnotatedDAO<RandomEventSettings> settingsDAO;
	private QueryParameterFactory<RandomEventSettings, Integer> flowFamilyIDParamFactory;

	private ModuleViewFragmentTransformer<ForegroundModuleDescriptor> viewFragmentTransformer;
	
	protected boolean configured;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		super.createDAOs(dataSource);

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RandomEventGeneratorModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		settingsDAO = daoFactory.getDAO(RandomEventSettings.class);

		flowFamilyIDParamFactory = settingsDAO.getParamFactory("flowFamilyID", Integer.class);
	}

	@Override
	protected void moduleConfigured() throws Exception {

		this.configured = ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.WARN);

		this.viewFragmentTransformer = new ModuleViewFragmentTransformer<>(sectionInterface.getForegroundModuleXSLTCache(), this, systemInterface.getEncoding());

		this.viewFragmentTransformer.setDebugXML(debugFragmententXML);
	}

	@Override
	public void unload() throws Exception {

		if (flowAdminModule != null) {

			setFlowAdminModule(null);
		}

		super.unload();
	}

	@Override
	public FlowAdminExtensionShowView getShowView(Flow flow, HttpServletRequest req, User user, URIParser uriParser) throws TransformerConfigurationException, TransformerException, SQLException {

		if (!AccessUtils.checkAccess(user, this.moduleDescriptor)) {

			return null;
		}

		Document doc = createDocument(req, uriParser, user);

		Element showViewElement = doc.createElement("ShowRandomEventSettings");
		doc.getDocumentElement().appendChild(showViewElement);

		RandomEventSettings randomEventSettings = getIntegrationSettings(flow);

		boolean enabled = false;
		
		if(randomEventSettings != null) {
			
			XMLUtils.append(doc, showViewElement, randomEventSettings);
			enabled = randomEventSettings.isEnabled();
		}

		showViewElement.appendChild(flow.toXML(doc));

		return new FlowAdminExtensionShowView(viewFragmentTransformer.createViewFragment(doc, true), enabled);
	}

	private RandomEventSettings getIntegrationSettings(Flow flow) throws SQLException {

		HighLevelQuery<RandomEventSettings> query = new HighLevelQuery<>();

		query.addParameter(flowFamilyIDParamFactory.getParameter(flow.getFlowFamily().getFlowFamilyID()));

		return settingsDAO.get(query);
	}

	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();
		Element document = doc.createElement("Document");

		document.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		document.appendChild(this.moduleDescriptor.toXML(doc));
		XMLUtils.appendNewElement(doc, document, "ModuleURI", req.getContextPath() + getFullAlias());
		XMLUtils.appendNewElement(doc, document, "StaticContentURL", staticContentModule.getModuleContentURL(moduleDescriptor));

		doc.appendChild(document);

		return doc;
	}

	@EventListener(channel = FlowInstanceManager.class, priority = 48)
	public void processEvent(SubmitEvent event, EventSource eventSource) throws SQLException, IOException {

		RandomEventSettings randomEventSettings = getIntegrationSettings((Flow)event.getFlowInstanceManager().getFlowInstance().getFlow());

		if (randomEventSettings == null || !randomEventSettings.isEnabled()) {

			return;
		}
		
		if (event.getEvent().getEventType() != EventType.SUBMITTED || event.getActionID() == null || !supportedActionIDs.contains(event.getActionID())) {

			return;
		}
		
		int randomValue = RandomUtils.getRandomInt(0, 100);
		
		if(randomValue <= randomEventSettings.getChance()){
			
			log.info("Adding random event to flow instance " + event.getFlowInstanceManager());

			flowAdminModule.getFlowInstanceEventGenerator().addFlowInstanceEvent(event.getFlowInstanceManager().getFlowInstance(), EventType.OTHER_EVENT, randomEventSettings.getEventDescription(), null);

			if(randomEventSettings.isOverrideSubmittedMessage()) {
				
				event.getEvent().getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_ALTERNATE_SUBMIT_MESSAGE_HEADER, randomEventSettings.getSubmittedMessageHeading());
				event.getEvent().getAttributeHandler().setAttribute(Constants.FLOW_INSTANCE_ALTERNATE_SUBMIT_MESSAGE_TEXT, randomEventSettings.getSubmittedMessageText());
			
				flowAdminModule.getDAOFactory().getFlowInstanceEventDAO().update(event.getEvent(), EVENT_ATTRIBUTE_RELATION_QUERY);
			}
			
			if(randomEventSettings.isSetAsAttribute()){
				
				((MutableAttributeHandler)event.getFlowInstanceManager().getFlowInstance().getAttributeHandler()).setAttribute(randomEventSettings.getAttributeName(), randomEventSettings.getAttributeValue());
				
				FlowInstanceUtils.setDescriptions((FlowInstance)event.getFlowInstanceManager().getFlowInstance());
				
				flowAdminModule.getDAOFactory().getFlowInstanceDAO().update((FlowInstance)event.getFlowInstanceManager().getFlowInstance(), RELATION_QUERY);
			}
			
			if(randomEventSettings.isAddExternalMessage()){
				
				FlowInstance flowInstance = (FlowInstance) event.getFlowInstanceManager().getFlowInstance();
				
				ExternalMessage externalMessage = new ExternalMessage();
				
				externalMessage.setAdded(TimeUtils.getCurrentTimestamp());
				externalMessage.setFlowInstance(flowInstance);
				externalMessage.setMessage(randomEventSettings.getExternalMessageText());
				externalMessage.setPostedByManager(true);
				externalMessage.setSystemMessage(true);
				externalMessage.setPoster(null);
				
				messageHandler.add(externalMessage);
			}
		}
	}

	@WebPublic(alias = "update")
	public ForegroundModuleResponse updateFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		}
		if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		RandomEventSettings randomEventSettings = getIntegrationSettings(flow);

		ValidationException validationException = null;

		if (req.getMethod().equalsIgnoreCase("POST")) {

			if (randomEventSettings == null) {

				randomEventSettings = new RandomEventSettings();
				randomEventSettings.setFlowFamilyID(flow.getFlowFamily().getFlowFamilyID());
			}

			try {
				POPULATOR.populate(randomEventSettings, req);

				List<ValidationError> validationErrors = new ArrayList<>();

				if (!validationErrors.isEmpty()) {

					throw new ValidationException(validationErrors);
				}

				log.info("User " + user + " updating random event settings for flow " + flow);

				if (randomEventSettings.isPopulated()) {

					settingsDAO.addOrUpdate(randomEventSettings, null);

				} else {

					settingsDAO.delete(randomEventSettings);
				}

				flowAdminModule.addFlowFamilyEvent(flowAdminModule.getEventFunctionConfigured() + " " + flowEventDescription, flow.getFlowFamily(), user);
				flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#randomeventsettings");

				return null;

			} catch (ValidationException e) {

				validationException = e;
			}
		}

		log.info("User " + user + " requesting update random event settings form for flow " + flow);

		Document doc = createDocument(req, uriParser, user);

		Element randomEventSettingsElement = doc.createElement("UpdateRandomEventSettings");
		doc.getDocumentElement().appendChild(randomEventSettingsElement);

		randomEventSettingsElement.appendChild(flow.toXML(doc));

		XMLUtils.append(doc, randomEventSettingsElement, randomEventSettings);

		if (validationException != null) {

			randomEventSettingsElement.appendChild(validationException.toXML(doc));
			randomEventSettingsElement.appendChild(RequestUtils.getRequestParameters(req, doc));
		}

		return new SimpleForegroundModuleResponse(doc, this.getDefaultBreadcrumb());
	}

	@WebPublic(alias = "delete")
	public ForegroundModuleResponse deleteFlowIntegrationSettings(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		Flow flow = flowAdminModule.getRequestedFlow(req, user, uriParser);

		if (flow == null) {

			return flowAdminModule.list(req, res, user, uriParser, new ValidationError("FlowNotFound"));

		}
		if (!flowAdminModule.hasFlowAccess(user, flow)) {

			throw new AccessDeniedException("User does not have access to flow type " + flow.getFlowType());
		}

		RandomEventSettings randomEventSettings = getIntegrationSettings(flow);

		if (randomEventSettings != null) {

			log.info("User " + user + " deleting random event settings for flow " + flow);

			settingsDAO.delete(randomEventSettings);

			//TODO add event to flow family history

		} else {

			log.warn("User " + user + " trying to deleting random event settings for flow " + flow + " which has no random event settings");
		}

		flowAdminModule.redirectToMethod(req, res, "/showflow/" + flow.getFlowID() + "#httpintegrationsettings");

		return null;
	}

	@Override
	public String getExtensionViewTitle() {

		return extensionViewTitle;
	}

	@Override
	public int getPriority() {

		return priority;
	}

	@Override
	public ForegroundModuleDescriptor getModuleDescriptor() {

		return moduleDescriptor;
	}

	@Override
	public List<LinkTag> getLinkTags() {

		return links;
	}

	@Override
	public List<ScriptTag> getScriptTags() {

		return scripts;
	}

	@InstanceManagerDependency(required = true)
	public void setFlowAdminModule(FlowAdminModule flowAdminModule) {

		if (flowAdminModule == null && this.flowAdminModule != null) {

			this.flowAdminModule.removeExtensionViewProvider(this);
		}

		this.flowAdminModule = flowAdminModule;

		if (this.flowAdminModule != null) {

			this.flowAdminModule.addExtensionViewProvider(this);
		}
	}
	
	@Override
	public int getModuleID() {

		return moduleDescriptor.getModuleID();
	}
}
