package com.nordicpeak.flowengine.queries.ratingscalequery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;

import se.unlogic.standardutils.annotations.RequiredIfNotSet;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesBaseQuery;

@Table(name = "rating_scale_queries")
@XMLElement
public class RatingScaleQuery extends FixedAlternativesBaseQuery {

	private static final long serialVersionUID = 6187777672035800269L;

	public static final Field ALTERNATIVES_RELATION = ReflectionUtils.getField(RatingScaleQuery.class, "alternatives");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@OneToMany(autoUpdate = true, autoAdd = true)
	@XMLElement(fixCase = true)
	private List<RatingScaleAlternative> alternatives;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideAlternativeLabels;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideEdgeLabels;

	@DAOManaged
	@WebPopulate(maxLength = 50)
	@RequiredIfNotSet(paramNames = "hideEdgeLabels")
	@XMLElement
	private String smallestValueLabel;

	@DAOManaged
	@WebPopulate(maxLength = 50)
	@RequiredIfNotSet(paramNames = "hideEdgeLabels")
	@XMLElement
	private String largestValueLabel;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<RatingScaleQueryInstance> instances;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideTitle;

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	@Override
	public List<RatingScaleAlternative> getAlternatives() {

		return alternatives;
	}

	public List<RatingScaleQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<RatingScaleQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setAlternatives(List<RatingScaleAlternative> alternatives) {

		this.alternatives = alternatives;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "RatingScaleQuery (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return "RatingScaleQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		toXSD(doc, 1);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		hideTitle = xmlParser.getPrimitiveBoolean("hideTitle");
		
		smallestValueLabel = XMLValidationUtils.validateParameter("smallestValueLabel", xmlParser, false, 1, 50, StringPopulator.getPopulator(), errors);
		largestValueLabel = XMLValidationUtils.validateParameter("largestValueLabel", xmlParser, false, 1, 50, StringPopulator.getPopulator(), errors);
		
		hideAlternativeLabels = xmlParser.getPrimitiveBoolean("hideAlternativeLabels");

		if (!StringUtils.isEmpty(smallestValueLabel) && !StringUtils.isEmpty(largestValueLabel)) {
			
			hideEdgeLabels = xmlParser.getPrimitiveBoolean("hideEdgeLabels");
		}
		
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		alternatives = RatingScaleQueryCRUD.ALTERNATIVES_POPLATOR.populate(xmlParser, errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}

	}

	public boolean isHideAlternativeLabels() {

		return hideAlternativeLabels;
	}

	public void setHideAlternativeLabels(boolean hideAlternativeLabels) {

		this.hideAlternativeLabels = hideAlternativeLabels;
	}

	public boolean isHideEdgeLabels() {

		return hideEdgeLabels;
	}

	public void setHideEdgeLabels(boolean hideEdgeLabels) {

		this.hideEdgeLabels = hideEdgeLabels;
	}

	public String getSmallestValueLabel() {

		return smallestValueLabel;
	}

	public void setSmallestValueLabel(String smallestValueLabel) {

		this.smallestValueLabel = smallestValueLabel;
	}

	public String getLargestValueLabel() {

		return largestValueLabel;
	}

	public void setLargestValueLabel(String largestValueLabel) {

		this.largestValueLabel = largestValueLabel;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public boolean isHideTitle() {

		return hideTitle;
	}

	public void setHideTitle(boolean hideTitle) {

		this.hideTitle = hideTitle;
	}

	@Override
	public String getFreeTextAlternative() {

		// Not used
		return null;
	}
}
