<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1"/>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/ratingscalequery.js
	</xsl:variable>
	
	<xsl:variable name="links">
		/css/ratingscalequery.css
	</xsl:variable>

	<xsl:template match="Document">	
		
		<xsl:apply-templates select="ShowQueryValues"/>
		<xsl:apply-templates select="ShowQueryForm"/>
		
	</xsl:template>

	<xsl:template match="ShowQueryValues">
	
		<div class="query">
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="RatingScaleQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<article>
				
				<div class="heading-wrapper">
				
					<xsl:if test="not(RatingScaleQueryInstance/RatingScaleQuery/hideTitle = 'true')">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="RatingScaleQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="RatingScaleQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>
					
					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="RatingScaleQueryInstance/RatingScaleQuery/queryID" />
					</xsl:call-template>
					
				</div>
				
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="RatingScaleQueryInstance/RatingScaleQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<div class="clearboth"/>
				
				<xsl:if test="RatingScaleQueryInstance/RatingScaleAlternative">
					<xsl:value-of select="RatingScaleQueryInstance/RatingScaleAlternative/name"/>
				</xsl:if>
				
			</article>
		
		</div>		
	
	</xsl:template>
		
	<xsl:template match="ShowQueryForm">
	
		<xsl:variable name="queryID" select="concat('query_', RatingScaleQueryInstance/RatingScaleQuery/queryID)" />
	
		<div class="query ratingscalequery" id="{$queryID}">
		
			<xsl:attribute name="class">
				<xsl:text>query ratingscalequery</xsl:text>
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/hideTitle = 'true'"> notitle</xsl:if>
				<xsl:if test="EnableAjaxPosting"><xsl:text> enableAjaxPosting</xsl:text></xsl:if>
				<xsl:if test="RatingScaleQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>
		
			<a name="{$queryID}" />
		
			<xsl:if test="ValidationErrors/validationError">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<xsl:apply-templates select="ValidationErrors/validationError"/>
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>
	
			<article>
			
				<xsl:if test="ValidationErrors/validationError">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
			
				<div class="heading-wrapper">
					
					<xsl:if test="not(RatingScaleQueryInstance/RatingScaleQuery/hideTitle = 'true')">
						<h2>
							<xsl:attribute name="class">
								<xsl:if test="RatingScaleQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
									<xsl:text>required</xsl:text>
								</xsl:if>
							</xsl:attribute>
							<xsl:value-of select="RatingScaleQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name"/>
						</h2>
					</xsl:if>	
						
					<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/helpText">		
						<xsl:apply-templates select="RatingScaleQueryInstance/RatingScaleQuery/helpText" />
					</xsl:if>
					
				</div>
				
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'"><xsl:attribute name="class">italic html-description</xsl:attribute></xsl:if>
						<xsl:value-of select="RatingScaleQueryInstance/RatingScaleQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/hideEdgeLabels = 'false'">
					<div class="edge-label bold floatleft hugemargintop hugemarginright">
						<xsl:value-of select="RatingScaleQueryInstance/RatingScaleQuery/smallestValueLabel"></xsl:value-of>
					</div>
				</xsl:if>
				
				<xsl:apply-templates select="RatingScaleQueryInstance/RatingScaleQuery/Alternatives/RatingScaleAlternative" mode="form"/>
				
				<xsl:if test="RatingScaleQueryInstance/RatingScaleQuery/hideEdgeLabels = 'false'">
					<div class="edge-label bold floatleft hugemargintop hugemarginleft">
						<xsl:value-of select="RatingScaleQueryInstance/RatingScaleQuery/largestValueLabel"></xsl:value-of>
					</div>
				</xsl:if>

			</article>
		
		</div>
		
		<script type="text/javascript">$(function(){initRatingScaleQuery('<xsl:value-of select="RatingScaleQueryInstance/RatingScaleQuery/queryID" />');});</script>
		
	</xsl:template>
	
	<xsl:template match="RatingScaleAlternative" mode="form">
	
		<div class="alternative floatleft">

			<xsl:variable name="radioID">
				<xsl:value-of select="'q'"/>
				<xsl:value-of select="../../queryID"/>
				<xsl:value-of select="'_alternative'"/>
				<xsl:value-of select="alternativeID" />
			</xsl:variable>
		
			<xsl:variable name="alternativeID" select="alternativeID"/>
		
			<xsl:call-template name="createRadio">
				<xsl:with-param name="id" select="$radioID" />
				<xsl:with-param name="name">
					<xsl:value-of select="'q'"/>
					<xsl:value-of select="../../queryID"/>
					<xsl:value-of select="'_alternative'"/>
				</xsl:with-param>
				<xsl:with-param name="value" select="alternativeID"/>
				<xsl:with-param name="elementName" select="'alternativeID'" />
				<xsl:with-param name="element" select="../../../RatingScaleAlternative[alternativeID = $alternativeID]" />
				<xsl:with-param name="requestparameters" select="../../../../requestparameters"/>
			</xsl:call-template>
			
			<label for="{$radioID}" class="radio">
			
				<xsl:if test="../../hideAlternativeLabels = 'true'">
					<xsl:attribute name="class">radio hide-label</xsl:attribute>
				</xsl:if>
			
				<xsl:value-of select="name" />
			
			</label>
				
		</div>
	
	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'RequiredQuery']">
		
		<span>
			<strong data-icon-before="!">
			<xsl:value-of select="$i18n.RequiredQuery"/>
			</strong>
		</span>
		
	</xsl:template>		
	
	<xsl:template match="validationError">
		
		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError"/>
			</strong>
		</span>
		
	</xsl:template>			
	
</xsl:stylesheet>