package com.nordicpeak.flowengine.remoteflowinstance;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.SectionInterface;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.modules.descriptors.ForegroundModuleDescriptor;
import se.unlogic.hierarchy.foregroundmodules.rest.AnnotatedRESTModule;
import se.unlogic.hierarchy.foregroundmodules.rest.RESTMethod;
import se.unlogic.hierarchy.foregroundmodules.rest.URIParam;
import se.unlogic.hierarchy.foregroundmodules.rest.responsehandlers.FilteredSerializingResponseHandler;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfile;
import se.unlogic.openhierarchy.foregroundmodules.siteprofile.interfaces.SiteProfileHandler;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.UserFlowInstanceModule;
import com.nordicpeak.flowengine.beans.FlowInstance;
import com.nordicpeak.flowengine.cache.FlowCacheAccess;
import com.nordicpeak.flowengine.flowinstanceadmin2.FlowInstanceAdmin2Module;
import com.nordicpeak.flowengine.flowinstanceadmin2.beans.UserManagableFlowInstanceCounts;
import com.nordicpeak.flowengine.flowinstanceadmin2.enums.InstanceFilter;

public class HTTPRemoteFlowInstanceModule extends AnnotatedRESTModule {

	private static final ArrayList<FlowInstance> EMPTY_LIST = new ArrayList<FlowInstance>(0);
	private static final UserManagableFlowInstanceCounts EMPTY_FLOW_INSTANCE_COUNTS = new UserManagableFlowInstanceCounts();

	@InstanceManagerDependency(required = true)
	private FlowInstanceAdmin2Module flowInstanceAdmin2Module;

	@InstanceManagerDependency(required = true)
	private UserFlowInstanceModule userFlowInstanceModule;

	@InstanceManagerDependency
	protected SiteProfileHandler profileHandler;

	@Override
	public void init(ForegroundModuleDescriptor moduleDescriptor, SectionInterface sectionInterface, DataSource dataSource) throws Exception {

		super.init(moduleDescriptor, sectionInterface, dataSource);

		FilteredSerializingResponseHandler responseHandler = new FilteredSerializingResponseHandler(HTTPRemoteFlowInstanceShared.REPLACEMENT_CLASSES, HTTPRemoteFlowInstanceShared.ALLOWED_CLASSES);

		this.addResponseHandler(responseHandler);
	}

	@RESTMethod(alias = "managerflowinstances/{username}", method = "get")
	public UserManagableFlowInstanceCounts getManagerFlowInstanceCounts(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "username") String username) throws Throwable {

		log.info("User " + user + " requesting flow instance counts for username " + username + " from " + req.getRemoteAddr());

		try {
			User requestedUser = systemInterface.getUserHandler().getUserByUsername(username, true, true);

			if (requestedUser == null) {
				return EMPTY_FLOW_INSTANCE_COUNTS;
			}

			FlowCacheAccess flowCacheAccess = flowInstanceAdmin2Module.getManagedFlowFamilies(requestedUser);

			UserManagableFlowInstanceCounts flowInstanceCounts = new UserManagableFlowInstanceCounts();

			flowInstanceCounts.setActiveInstanceCount(flowInstanceAdmin2Module.getActiveFlowInstanceCount(requestedUser, flowCacheAccess, null));
			//		flowInstanceCounts.setBookmarkedInstanceCount(flowInstanceAdmin2Module.getBookmaredFlowInstanceCount(requestedUser, flowCacheAccess, null));
			flowInstanceCounts.setPrioritizedInstanceCount(flowInstanceAdmin2Module.getPrioritizedFlowInstanceCount(requestedUser, flowCacheAccess, null));
			flowInstanceCounts.setUnassignedInstanceCount(flowInstanceAdmin2Module.getUnassignedFlowInstanceCount(requestedUser, flowCacheAccess, null));
			flowInstanceCounts.setUserAssignedInstanceCount(flowInstanceAdmin2Module.getMyFlowInstanceCount(requestedUser, flowCacheAccess, null));

			return flowInstanceCounts;

		} catch (Exception e) {

			log.error("Exception when user " + user + " requesting flow instance counts for username " + username + " from " + req.getRemoteAddr(), e);
			return EMPTY_FLOW_INSTANCE_COUNTS;
		}
	}

	@RESTMethod(alias = "managerflowinstances/{username}/{instanceFilter}", method = "get")
	public List<FlowInstance> getManagerFlowinstances(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "username") String username, @URIParam(name = "instanceFilter") InstanceFilter instanceFilter) throws Throwable {

		log.info("User " + user + " requesting flow instances for username " + username + " and type " + instanceFilter + " from " + req.getRemoteAddr());
		
		try {
			User requestedUser = systemInterface.getUserHandler().getUserByUsername(username, true, true);

			if (requestedUser == null) {
				return EMPTY_LIST;
			}

			FlowCacheAccess flowCacheAccess = flowInstanceAdmin2Module.getManagedFlowFamilies(requestedUser);

			List<FlowInstance> flowInstances = getFlowInstances(requestedUser, flowCacheAccess, instanceFilter);

			if (flowInstances == null) {

				return EMPTY_LIST;
			}

			for (FlowInstance flowInstance : flowInstances) {

				MutableAttributeHandler attributeHandler = flowInstance.getAttributeHandler();

				attributeHandler.setAttribute("overviewURL", uriParser.getFullContextPath() + flowInstanceAdmin2Module.getFullAlias() + "/overview/" + flowInstance.getFlowInstanceID());

				if (flowInstance.getProfileID() != null && profileHandler != null) {

					SiteProfile siteProfile = profileHandler.getProfile(flowInstance.getProfileID());

					if (siteProfile != null) {

						attributeHandler.setAttribute("siteProfileName", siteProfile.getName());
					}
				}
			}

			return flowInstances;

		} catch (Exception e) {

			log.error("Exception when user " + user + " requesting flow instances for username " + username + " and type " + instanceFilter + " from " + req.getRemoteAddr(), e);
			return EMPTY_LIST;
		}
	}

	private List<FlowInstance> getFlowInstances(User requestedUser, FlowCacheAccess flowCacheAccess, InstanceFilter instanceFilter) throws SQLException {

		if (instanceFilter == InstanceFilter.ACTIVE) {

			return flowInstanceAdmin2Module.getActiveFlowInstances(requestedUser, flowCacheAccess, null);

		} else if (instanceFilter == InstanceFilter.BOOKMARKED) {

			return flowInstanceAdmin2Module.getBookmaredFlowInstances(requestedUser, flowCacheAccess, null);

		} else if (instanceFilter == InstanceFilter.PRIORITIZED) {

			return flowInstanceAdmin2Module.getPrioritizedFlowInstances(requestedUser, flowCacheAccess, null);

		} else if (instanceFilter == InstanceFilter.UNASSIGNED) {

			return flowInstanceAdmin2Module.getUnassignedFlowInstances(requestedUser, flowCacheAccess, null);

		} else if (instanceFilter == InstanceFilter.MINE) {

			return flowInstanceAdmin2Module.getMyFlowInstances(requestedUser, flowCacheAccess, null);
			
		} else if (instanceFilter == InstanceFilter.ALL) {

			return flowInstanceAdmin2Module.getAllFlowInstances(requestedUser, flowCacheAccess, null);
			
		}else {
			
			throw new RuntimeException("Unsupported enum value " + instanceFilter);
		}
	}

	@RESTMethod(alias = "userflowinstances/{citizenIdentifier}", method = "get")
	public List<FlowInstance> getUserFlowinstances(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser, @URIParam(name = "citizenIdentifier") Long citizenIdentifier) throws Throwable {

		log.info("User " + user + " requesting flow instances for citizenIdentifier " + citizenIdentifier + " from " + req.getRemoteAddr());

		try {
			User requestedUser = systemInterface.getUserHandler().getUserByAttribute(Constants.USER_CITIZEN_IDENTIFIER_ATTRIBUTE, citizenIdentifier.toString(), false, false);

			if (requestedUser == null) {

				return EMPTY_LIST;
			}

			List<FlowInstance> flowInstances = userFlowInstanceModule.getFlowInstances(requestedUser, true, false);

			if (flowInstances == null) {

				return EMPTY_LIST;
			}

			for (FlowInstance flowInstance : flowInstances) {

				MutableAttributeHandler attributeHandler = flowInstance.getAttributeHandler();

				attributeHandler.setAttribute("UserFlowInstanceModuleURL", uriParser.getFullContextPath() + userFlowInstanceModule.getFullAlias());

				if (flowInstance.getProfileID() != null && profileHandler != null) {

					SiteProfile siteProfile = profileHandler.getProfile(flowInstance.getProfileID());

					if (siteProfile != null) {

						attributeHandler.setAttribute("siteProfileName", siteProfile.getName());
					}
				}
			}

			return flowInstances;
			
		} catch (Exception e) {

			log.error("Exception when user " + user + " requesting flow instances for citizenIdentifier " + citizenIdentifier + " from " + req.getRemoteAddr(), e);
			return EMPTY_LIST;
		}
	}
}
