package com.nordicpeak.flowengine.remoteflowinstance;

import java.io.ObjectStreamException;
import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;

import se.unlogic.hierarchy.core.beans.Group;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.handlers.SimpleAttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.foregroundmodules.userproviders.SimpleUser;
import se.unlogic.standardutils.i18n.Language;

public class RemoteUser implements Serializable {

	private static final long serialVersionUID = -1330205626987497936L;

	private Integer userID;

	private Timestamp added;

	private String email;

	private String firstname;

	private String lastname;

	private String username;

	private Language language;

	private boolean admin;

	private boolean enabled;

	private Timestamp lastLogin;

	private String preferedDesign;

	private boolean formProvider;

	// Automatically converted to simple groups on read
	private List<RemoteGroup> groups;

	private SimpleAttributeHandler attributeHandler;

	public RemoteUser() {}

	public RemoteUser(User user) {

		userID = user.getUserID();
		added = user.getAdded();
		email = user.getEmail();
		firstname = user.getFirstname();
		lastname = user.getLastname();
		username = user.getUsername();
		language = user.getLanguage();
		admin = user.isAdmin();
		enabled = user.isEnabled();
		lastLogin = user.getLastLogin();
		preferedDesign = user.getPreferedDesign();
		formProvider = user.hasFormProvider();

		if (user.getAttributeHandler() != null && !user.getAttributeHandler().isEmpty()) {

			attributeHandler = new SimpleAttributeHandler(user.getAttributeHandler().getAttributeMap());
		}

		if (user.getGroups() != null) {

			groups = new ArrayList<>(user.getGroups().size());

			for (Group group : user.getGroups()) {

				groups.add(new RemoteGroup(group));
			}
		}
	}

	public boolean isAdmin() {
		return admin;
	}

	public String getEmail() {
		return email;
	}

	public String getFirstname() {
		return firstname;
	}

	public Timestamp getCurrentLogin() {
		return null;
	}

	public Timestamp getLastLogin() {
		return lastLogin;
	}

	public String getLastname() {
		return lastname;
	}

	public String getPassword() {
		return null;
	}

	public String getUsername() {
		return username;
	}

	public Integer getUserID() {
		return userID;
	}

	public boolean isEnabled() {
		return enabled;
	}

	public Timestamp getAdded() {
		return added;
	}

	public Collection<RemoteGroup> getGroups() {
		return groups;
	}

	public Language getLanguage() {
		return language;
	}

	public String getPreferedDesign() {
		return preferedDesign;
	}

	public boolean hasFormProvider() {
		return formProvider;
	}

	public void setCurrentLogin(Timestamp currentLogin) {
		throw new UnsupportedOperationException();
	}

	public AttributeHandler getAttributeHandler() {
		return attributeHandler;
	}

	@SuppressWarnings("unchecked")
	public Object readResolve() throws ObjectStreamException {

		SimpleUser user = new SimpleUser();

		user.setUserID(userID);
		user.setAdded(added);
		user.setEmail(email);
		user.setFirstname(firstname);
		user.setLastname(lastname);
		user.setUsername(username);
		user.setLanguage(language);
		user.setAdmin(admin);
		user.setEnabled(enabled);
		user.setLastLogin(lastLogin);
		user.setPreferedDesign(preferedDesign);
		user.setFormProvider(formProvider);
		
		if (groups != null) {
			
			user.setGroups((List<Group>)(List<?>) groups);
		}

		if (attributeHandler != null) {

			MutableAttributeHandler userAttributeHandler = user.getAttributeHandler();
			
			for (Entry<String, String> entry : attributeHandler.getAttributeMap().entrySet()) {

				userAttributeHandler.setAttribute(entry.getKey(), entry.getValue());
			}
		}

		return user;
	}
}
