package com.nordicpeak.flowengine.queries.remoteinfoquery;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLParserPopulateable;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.APISource;

@Table(name = "remote_info_query_endpoints")
@XMLElement
public class RemoteInfoQueryEndpoint extends GeneratedElementable implements XMLParserPopulateable, Serializable, APISource {

	private static final long serialVersionUID = -8348666528294482053L;

	public static final Field QUERIES_RELATION = ReflectionUtils.getField(RemoteInfoQueryEndpoint.class, "queries");

	@DAOManaged
	@Key
	@XMLElement
	private Integer endpointID;

	@DAOManaged
	@OneToMany
	@XMLElement(fixCase = true)
	private List<RemoteInfoQuery> queries;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 1024)
	private String address;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	private String username;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	private String password;

	@DAOManaged
	@WebPopulate(required = true)
	private String encoding;
	
	public Element toXMLFull(Document doc) {
		
		Element element = toXML(doc);
		
		XMLUtils.appendNewElement(doc, element, "address", address);
		XMLUtils.appendNewElement(doc, element, "username", username);
		XMLUtils.appendNewElement(doc, element, "password", password);
		XMLUtils.appendNewElement(doc, element, "encoding", encoding);
		
		return element;
	}

	@Override
	public String toString() {

		return name + " (ID: " + endpointID + ")";
	}

	public Integer getEndpointID() {

		return endpointID;
	}

	public void setEndpointID(Integer endpointID) {

		this.endpointID = endpointID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getAddress() {

		return address;
	}

	public void setAddress(String address) {

		this.address = address;
	}

	public String getUsername() {

		return username;
	}

	public void setUsername(String username) {

		this.username = username;
	}

	public String getPassword() {

		return password;
	}

	public void setPassword(String password) {

		this.password = password;
	}

	public String getEncoding() {

		return encoding;
	}

	public void setEncoding(String encoding) {

		this.encoding = encoding;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		this.endpointID = XMLValidationUtils.validateParameter("endpointID", xmlParser, true, PositiveStringIntegerPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public Charset getCharset() {

		return Charset.forName(encoding);
	}

	public List<RemoteInfoQuery> getQueries() {

		return queries;
	}

	public void setQueries(List<RemoteInfoQuery> queries) {

		this.queries = queries;
	}
}
