package com.nordicpeak.flowengine.queries.remoteinfoquery;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextAreaSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.menu.MenuItemDescriptor;
import se.unlogic.hierarchy.core.utils.CRUDCallback;
import se.unlogic.hierarchy.foregroundmodules.AnnotatedForegroundModule;
import se.unlogic.standardutils.annotations.SplitOnLineBreak;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AdvancedAnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.querys.ArrayListQuery;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.Flow;
import com.nordicpeak.flowengine.beans.QueryDescriptor;
import com.nordicpeak.flowengine.beans.Step;
import com.nordicpeak.flowengine.dao.FlowEngineDAOFactory;
import com.nordicpeak.flowengine.interfaces.APISource;
import com.nordicpeak.flowengine.interfaces.APISourceHandler;
import com.nordicpeak.flowengine.interfaces.APISourceProvider;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;

public class RemoteInfoQueryEndpointAdminModule extends AnnotatedForegroundModule implements CRUDCallback<User>, APISourceProvider {
	
	@XSLVariable(prefix = "java.")
	private String apiSourceShortTypeName;
	
	@XSLVariable(prefix = "java.")
	private String apiSourceFullTypeName;
	
	@ModuleSetting
	@SplitOnLineBreak
	@TextAreaSettingDescriptor(name = "Allowed encodings", description = "Encodings allowed to use")
	private List<String> allowedEncodings = new ArrayList<String>(Arrays.asList("ISO-8859-1", "UTF-8"));
	
	@InstanceManagerDependency
	private QueryHandler queryHandler;
	
	private AnnotatedDAO<RemoteInfoQueryEndpoint> endpointDAO;
	private AnnotatedDAO<RemoteInfoQuery> queryDAO;
	private AnnotatedDAOWrapper<QueryDescriptor, Integer> queryDescriptorDAOWrapper;
	private AdvancedAnnotatedDAOWrapper<RemoteInfoQueryEndpoint, Integer> endpointDAOWrapper;
	
	private QueryParameterFactory<RemoteInfoQuery, Integer> queryIDParamFactory;
	
	private RemoteInfoQueryEndpointCRUD endpointCRUD;
	
	private APISourceHandler apiSourceHandler;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RemoteInfoQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}
		
		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(RemoteInfoQuery.class);
		endpointDAO = daoFactory.getDAO(RemoteInfoQueryEndpoint.class);
		
		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);

		endpointDAOWrapper = endpointDAO.getAdvancedWrapper(Integer.class);
		endpointDAOWrapper.getGetAllQuery().addRelations(RemoteInfoQueryEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetAllQuery().disableAutoRelations(true);
		endpointDAOWrapper.getGetQuery().addRelations(RemoteInfoQueryEndpoint.QUERIES_RELATION);
		endpointDAOWrapper.getGetQuery().disableAutoRelations(true);
		
		endpointCRUD = new RemoteInfoQueryEndpointCRUD(endpointDAOWrapper, this);
		
		FlowEngineDAOFactory flowDAOFactory = new FlowEngineDAOFactory(dataSource, systemInterface.getUserHandler(), systemInterface.getGroupHandler());
		
		queryDescriptorDAOWrapper = flowDAOFactory.getQueryDescriptorDAO().getWrapper(Integer.class);
		queryDescriptorDAOWrapper.addRelations(QueryDescriptor.STEP_RELATION, Step.FLOW_RELATION, Flow.FLOW_TYPE_RELATION, Flow.CATEGORY_RELATION);
		queryDescriptorDAOWrapper.setUseRelationsOnGet(true);
	}
	
	@InstanceManagerDependency
	public void setAPISourceHandler(APISourceHandler apiSourceHandler) throws SQLException {
		
		if (apiSourceHandler != null) {
			
			apiSourceHandler.addAPISourceProvider(this);
			
		} else if (this.apiSourceHandler != null) {
			
			this.apiSourceHandler.removeAPISourceProvider(this);
		}
		
		this.apiSourceHandler = apiSourceHandler;
		
		//Check if this module is fully cached, then trigger an update of the menu items
		if(sectionInterface.getForegroundModuleCache().isCached(moduleDescriptor)) {
			
			this.sectionInterface.getMenuCache().moduleUpdated(moduleDescriptor, this);
		}
	}
	
	@Override
	public ForegroundModuleResponse defaultMethod(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.list(req, res, user, uriParser, null);
	}
	
	@WebPublic
	public ForegroundModuleResponse show(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.show(req, res, user, uriParser);
	}
	
	@WebPublic
	public ForegroundModuleResponse add(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.add(req, res, user, uriParser);
	}
	
	@WebPublic
	public ForegroundModuleResponse update(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.update(req, res, user, uriParser);
	}
	
	@WebPublic
	public ForegroundModuleResponse delete(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Throwable {

		return endpointCRUD.delete(req, res, user, uriParser);
	}
	
	public List<RemoteInfoQuery> getQueries(RemoteInfoQueryEndpoint endpoint) {
		
		try {
			ArrayListQuery<Integer> idQuery = new ArrayListQuery<Integer>(queryDAO.getDataSource(), "SELECT queryID FROM " + queryDAO.getTableName() + " WHERE endpointID = ?", IntegerPopulator.getPopulator());
			idQuery.setInt(1, endpoint.getEndpointID());
			
			List<Integer> queryIDs = idQuery.executeQuery();
			
			if (!CollectionUtils.isEmpty(queryIDs)) {
				
				List<RemoteInfoQuery> queries = new ArrayList<RemoteInfoQuery>(queryIDs.size());
				
				for (Integer queryID : queryIDs) {
					
					QueryDescriptor queryDescriptor = queryDescriptorDAOWrapper.get(queryID);
					
					if (queryDescriptor != null) {
						
						queries.add((RemoteInfoQuery) getQuery(queryDescriptor));
					}
				}
				
				return queries;
			}
			
		} catch (SQLException e) {
			
			log.error("Unable to get queries using endpoint " + endpoint, e);
		}
		
		return null;
	}
	
	public Query getQuery(MutableQueryDescriptor descriptor) throws SQLException {
		
		RemoteInfoQuery query = this.getQuery(descriptor.getQueryID());
		
		if (query == null) {
			
			return null;
		}
		
		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());
		
		return query;
	}
	
	public RemoteInfoQuery getQuery(Integer queryID) throws SQLException {
		
		HighLevelQuery<RemoteInfoQuery> getQuery = new HighLevelQuery<RemoteInfoQuery>();
		
		getQuery.addParameter(queryIDParamFactory.getParameter(queryID));
		
		RemoteInfoQuery query = queryDAO.get(getQuery);
		
		return query;
	}

	@Override
	public Document createDocument(HttpServletRequest req, URIParser uriParser, User user) {

		Document doc = XMLUtils.createDomDocument();

		Element documentElement = doc.createElement("Document");
		documentElement.appendChild(RequestUtils.getRequestInfoAsXML(doc, req, uriParser));
		documentElement.appendChild(this.moduleDescriptor.toXML(doc));

		doc.appendChild(documentElement);

		return doc;
	}

	@Override
	public String getTitlePrefix() {

		return moduleDescriptor.getName();
	}

	public List<String> getAllowedEncodings() {

		return allowedEncodings;
	}

	@Override
	public List<? extends APISource> getAPISources() throws Exception {

		return endpointDAOWrapper.getAll();
	}
	
	@Override
	public String getShortTypeName() {

		return apiSourceShortTypeName;
	}
	
	@Override
	public String getFullTypeName() {

		return apiSourceFullTypeName;
	}

	@Override
	public String getAddURL() {

		return this.getFullAlias() + "/add";
	}
	
	@Override
	public String getBaseShowURL() {

		return this.getFullAlias() + "/show/";
	}
	
	@Override
	public String getBaseUpdateURL() {

		return this.getFullAlias() + "/update/";
	}

	@Override
	public String getBaseDeleteURL() {

		return this.getFullAlias() + "/delete/";
	}

	@Override
	public boolean isInUse(APISource apiSource) throws Exception {

		RemoteInfoQueryEndpoint endpoint = (RemoteInfoQueryEndpoint)apiSource;
		
		return endpoint.getQueries() != null;
	}

	public APISourceHandler getAPISourceHandler() {

		return apiSourceHandler;
	}
	
	@Override
	public List<? extends MenuItemDescriptor> getVisibleMenuItems() {

		if(this.apiSourceHandler != null) {
			
			return null;
		
		}else {
		
			return super.getVisibleMenuItems();
		}
	}
}