package com.nordicpeak.flowengine.queries.remoteinfoquery;

import java.lang.reflect.Field;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.BaseInvoiceLine;
import com.nordicpeak.flowengine.interfaces.InvoiceLine;
import com.nordicpeak.flowengine.interfaces.NonColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.PaymentQueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@Table(name = "remote_info_query_instances")
@XMLElement
public class RemoteInfoQueryInstance extends BaseQueryInstance implements NonColumnExportableQueryInstance, PaymentQueryInstance{

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(RemoteInfoQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private RemoteInfoQuery query;

	@DAOManaged
	@XMLElement(cdata = true)
	private String infoText;
	
	@DAOManaged
	@XMLElement(cdata = true)
	private String invoiceLineDescription;
	
	@DAOManaged
	@XMLElement
	private Integer price;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewCDATAElement(doc, element, "InfoText", infoText);
		
		return element;
	}

	@Override
	public RemoteInfoQuery getQuery() {

		return query;
	}

	public void setQuery(RemoteInfoQuery query) {

		this.query = query;
	}

	public String getInfoText() {
	
		return infoText;
	}

	
	public void setInfoText(String infoText) {
	
		this.infoText = infoText;
	}
	
	
	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		if (query.isSetAsAttribute() && attributeHandler.isSet(query.getAttributeName()) && attributeHandler.getString(query.getAttributeName()).equals(query.getDescription())) {

			attributeHandler.removeAttribute(query.getAttributeName());
		}
		
		this.infoText = null;
		this.invoiceLineDescription = null;
		this.price = null;

		super.reset(attributeHandler);
	}

	@Override
	public List<? extends InvoiceLine> getInvoiceLines() {
		
		if(invoiceLineDescription != null && price != null && price != 0) {
			
			return Collections.singletonList(new BaseInvoiceLine(1, price, invoiceLineDescription,""));
		}

		return null;
	}

	
	public String getInvoiceLineDescription() {
	
		return invoiceLineDescription;
	}

	
	public void setInvoiceLineDescription(String invoiceLineDescription) {
	
		this.invoiceLineDescription = invoiceLineDescription;
	}

	
	public Integer getPrice() {
	
		return price;
	}

	
	public void setPrice(Integer price) {
	
		this.price = price;
	}

}
