package com.nordicpeak.flowengine.queries.remoteinfoquery;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.AttributeTagUtils;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.html.HTMLUtils;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.string.AnnotatedBeanTagSourceFactory;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.string.TagReplacer;
import se.unlogic.standardutils.string.URLEncodingTagReplacer;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.HTTPResponse;
import se.unlogic.webutils.http.HTTPUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.SimpleRequest;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.utils.CurrentUserAttributeTagUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;
import com.nordicpeak.flowengine.utils.UserAttributeTagUtils;

public class RemoteInfoQueryProviderModule extends BaseQueryProviderModule<RemoteInfoQueryInstance> implements BaseQueryCRUDCallback {

	public static final AnnotatedBeanTagSourceFactory<User> USER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$user.");
	
	public static final AnnotatedBeanTagSourceFactory<User> CURRENT_USER_TAG_SOURCE_FACTORY = new AnnotatedBeanTagSourceFactory<User>(User.class, "$currentUser.");
	
	public String REQUIRED_HEADER = "InfoQueryResponse";
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;
	
	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;
	
	private AnnotatedDAO<RemoteInfoQuery> queryDAO;
	private AnnotatedDAO<RemoteInfoQueryInstance> queryInstanceDAO;
	private AnnotatedDAO<RemoteInfoQueryEndpoint> endpointsDAO;

	private RemoteInfoQueryCRUD queryCRUD;

	private QueryParameterFactory<RemoteInfoQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<RemoteInfoQueryInstance, Integer> queryInstanceIDParamFactory;
	private QueryParameterFactory<RemoteInfoQueryEndpoint, Integer> endpointIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, RemoteInfoQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(RemoteInfoQuery.class);
		queryInstanceDAO = daoFactory.getDAO(RemoteInfoQueryInstance.class);
		endpointsDAO = daoFactory.getDAO(RemoteInfoQueryEndpoint.class);

		queryCRUD = new RemoteInfoQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<RemoteInfoQuery>(RemoteInfoQuery.class), "RemoteInfoQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		endpointIDParamFactory = endpointsDAO.getParamFactory("endpointID", Integer.class);
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		RemoteInfoQuery query = new RemoteInfoQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		RemoteInfoQuery query = new RemoteInfoQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);
		
		if(query.getEndpoint() != null){
			
			RemoteInfoQueryEndpoint endpoint = getEndpoint(query.getEndpoint().getEndpointID());
			
			if(endpoint != null) {
				
				query.setEndpoint(endpoint);
				
			} else {
				
				query.setEndpoint(null);
			}
		}

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		RemoteInfoQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		RemoteInfoQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		RemoteInfoQueryInstance queryInstance;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new RemoteInfoQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}
		
		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private RemoteInfoQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<RemoteInfoQuery> query = new HighLevelQuery<RemoteInfoQuery>();
		
		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private RemoteInfoQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<RemoteInfoQuery> query = new HighLevelQuery<RemoteInfoQuery>();
		
		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private RemoteInfoQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<RemoteInfoQueryInstance> query = new HighLevelQuery<RemoteInfoQueryInstance>(RemoteInfoQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(RemoteInfoQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		RemoteInfoQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		RemoteInfoQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		RemoteInfoQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, RemoteInfoQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
		
		if(queryInstance.getInfoText() != null){

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "InfoText", JTidyUtils.getXHTML(queryInstance.getInfoText(), systemInterface.getEncoding()));
		}

	}

	@Override
	protected Class<RemoteInfoQueryInstance> getQueryInstanceClass() {

		return RemoteInfoQueryInstance.class;
	}

	public List<RemoteInfoQueryEndpoint> getEndpoints() throws SQLException {

		HighLevelQuery<RemoteInfoQueryEndpoint> query = new HighLevelQuery<RemoteInfoQueryEndpoint>();
		
		return endpointsDAO.getAll(query);
	}

	public RemoteInfoQueryEndpoint getEndpoint(Integer endpointID) throws SQLException {

		HighLevelQuery<RemoteInfoQueryEndpoint> query =  new HighLevelQuery<RemoteInfoQueryEndpoint>();
		
		query.addParameter(endpointIDParamFactory.getParameter(endpointID));
		
		return endpointsDAO.get(query);
	}
	
	protected void getEndpointData(RemoteInfoQueryInstance queryInstance, AttributeHandler attributeHandler, User poster, User currentUser) throws IOException{
		
		RemoteInfoQueryEndpoint endpoint = queryInstance.getQuery().getEndpoint();
		
		String address = endpoint.getAddress();
		
		address = AttributeTagUtils.replaceTags(address, attributeHandler, false, true);

		if(poster != null) {
		
			address = UserAttributeTagUtils.replaceTags(address, poster);
			
			TagReplacer tagReplacer = new URLEncodingTagReplacer();
			
			tagReplacer.addTagSource(USER_TAG_SOURCE_FACTORY.getTagSource(poster));
			
			address = tagReplacer.replace(address);
			
		}
		
		if(currentUser != null) {
			
			address = CurrentUserAttributeTagUtils.replaceTags(address, currentUser, true);
			
			TagReplacer tagReplacer = new URLEncodingTagReplacer();
			
			tagReplacer.addTagSource(CURRENT_USER_TAG_SOURCE_FACTORY.getTagSource(currentUser));
			
			address = tagReplacer.replace(address);
		}		
		
		SimpleRequest simpleRequest = new SimpleRequest(address);
		
		simpleRequest.setFollowRedirects(false);
		simpleRequest.setConnectionTimeout(connectionTimeout * 1000);
		simpleRequest.setReadTimeout(readTimeout * 1000);
		
		if(endpoint.getUsername() != null && endpoint.getPassword() != null) {
			
			simpleRequest.setUsername(endpoint.getUsername());
			simpleRequest.setPassword(endpoint.getPassword());
		}
		
		List<Entry<String, String>> headerEntries = new ArrayList<Entry<String, String>>(2);
		
		headerEntries.add(new SimpleEntry<String, String>("NoLoginRedirect", "NoLoginRedirect"));
		simpleRequest.setHeaders(headerEntries);
		
		HTTPResponse response = HTTPUtils.sendHTTPGetRequest(simpleRequest, endpoint.getCharset());
		
		if(response.getHeaders().get("InfoQueryResponse") == null && getCaseInsensitiveMapValue(response.getHeaders(), "InfoQueryResponse") == null) {
			
			throw new IOException("Got bad response for API call to address " + address);
		}
		
		queryInstance.setInfoText(response.getValue());
		
		if(queryInstance.getQuery().isHideOnEmptyResponse() && StringUtils.isEmpty(queryInstance.getInfoText())) {
			
			queryInstance.setInfoText(null);
		}
		
		String invoiceLineDescription = CollectionUtils.getFirstValue(response.getHeaders().get("invoiceDescription"));
		Integer price = NumberUtils.toInt(CollectionUtils.getFirstValue(response.getHeaders().get("price")));
		
		if(!StringUtils.isEmpty(invoiceLineDescription) && price != null) {
			
			queryInstance.setInvoiceLineDescription(StringUtils.substring(invoiceLineDescription, 255));
			queryInstance.setPrice(price);
		
		}else {
			
			queryInstance.setInvoiceLineDescription(null);
			queryInstance.setPrice(null);
		}
		
	}

	private static List<String> getCaseInsensitiveMapValue(Map<String,List<String>> map, String key) {
		
		for(Entry<String, List<String>> entry : map.entrySet()) {
			
			if(entry != null && entry.getKey() != null && entry.getKey().equalsIgnoreCase(key)) {
				
				return entry.getValue();
			}
		}
		
		return null;
	}
	
	@Override
	public QueryResponse getFormHTML(RemoteInfoQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if(validationErrors == null){
			
			if(queryInstance.getQuery().getEndpoint() == null){
				
				validationErrors = CollectionUtils.getList(new ValidationError("ModuleConfigurationError"));
				
			}else{
				
				try {

					getEndpointData(queryInstance, attributeHandler, poster, user);
					
				} catch (IOException e) {

					validationErrors = CollectionUtils.getList(new ValidationError("UnableToGetDataError"));
					
					log.warn("Error getting data from endpoint " + queryInstance.getQuery().getEndpoint() + " for query " + queryInstance + " requested by " + user, e);
				}
			}
		}
		
		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}
	
	@Override
	public void populate(RemoteInfoQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if(queryInstance.getQuery().getEndpoint() == null){
			
			if(allowPartialPopulation != true) {
			
				throw new ValidationException(new ValidationError("ModuleConfigurationError"));
			}
			
		}else{
			
			try {
				getEndpointData(queryInstance, attributeHandler, poster, user);
					
				if (queryInstance.getQuery().isSetAsAttribute()) {

					if(queryInstance.getInfoText() == null) {
						
						attributeHandler.removeAttribute(queryInstance.getQuery().getAttributeName());
						
					}else {
						
						attributeHandler.setAttribute(queryInstance.getQuery().getAttributeName(), HTMLUtils.removeHTMLTags(queryInstance.getInfoText()));
					}
				}	
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(!queryInstance.getQuery().isDontSetPopulated() || (queryInstance.getInfoText() == null && queryInstance.getQuery().isHideOnEmptyResponse()));
				
			} catch (IOException e) {

				log.warn("Error getting data from endpoint " + queryInstance.getQuery().getEndpoint() + " for query " + queryInstance + " requested by " + user, e);
				
				queryInstance.getQueryInstanceDescriptor().setPopulated(false);
				
				queryInstance.reset(attributeHandler);
				
				if(allowPartialPopulation != true) {
					
					throw new ValidationException(new ValidationError("UnableToGetDataError"));
				}
			}
		}
	}
}
