package com.nordicpeak.flowengine.queries.remotesinglesignquery;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.remotesinglesignquery.api.RemoteSinglesignQueryEndpoint;

@Table(name = "remote_singlesign_queries")
@XMLElement
public class RemoteSinglesignQuery extends BaseQuery {

	private static final long serialVersionUID = 8732607760189948382L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean hideCitizenIdentifierInPDF;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setMultipartsAsOwners;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<RemoteSinglesignQueryInstance> instances;

	@DAOManaged(columnName = "endpointID")
	@ManyToOne(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement
	private RemoteSinglesignQueryEndpoint endpoint;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private String endpointErrorText;

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<RemoteSinglesignQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<RemoteSinglesignQueryInstance> instances) {

		this.instances = instances;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public RemoteSinglesignQueryEndpoint getEndpoint() {

		return endpoint;
	}

	public void setEndpoint(RemoteSinglesignQueryEndpoint endpoint) {

		this.endpoint = endpoint;
	}

	public String getEndpointErrorText() {

		return endpointErrorText;
	}

	public void setEndpointErrorText(String endpointErrorText) {

		this.endpointErrorText = endpointErrorText;
	}

	public boolean isHideCitizenIdentifierInPDF() {

		return hideCitizenIdentifierInPDF;
	}

	public void setHideCitizenIdentifierInPDF(boolean hideCitizenIdetifierInPDF) {

		this.hideCitizenIdentifierInPDF = hideCitizenIdetifierInPDF;
	}

	public boolean isSetMultipartsAsOwners() {

		return setMultipartsAsOwners;
	}

	public void setSetMultipartsAsOwners(boolean setMultipartsAsOwners) {

		this.setMultipartsAsOwners = setMultipartsAsOwners;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return "InfoQuery (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return "InfoQuery" + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		doc.getDocumentElement().appendChild(complexTypeElement);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		hideCitizenIdentifierInPDF = xmlParser.getPrimitiveBoolean("hideCitizenIdetifierInPDF");
		setMultipartsAsOwners = xmlParser.getPrimitiveBoolean("setMultipartsAsOwners");
		endpointErrorText = XMLValidationUtils.validateParameter("endpointErrorText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}
}
