package com.nordicpeak.flowengine.queries.remotesinglesignquery;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;

import com.nordicpeak.flowengine.beans.SigningParty;
import com.nordicpeak.flowengine.interfaces.ColumnExportableQueryInstance;
import com.nordicpeak.flowengine.interfaces.MultiSignQueryinstance;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.StringValueQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;

@Table(name = "remote_singlesign_query_instances")
@XMLElement
public class RemoteSinglesignQueryInstance extends BaseQueryInstance implements StringValueQueryInstance, ColumnExportableQueryInstance, MultiSignQueryinstance {

	private static final long serialVersionUID = -7761759005604863873L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(RemoteSinglesignQueryInstance.class, "query");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private RemoteSinglesignQuery query;

	@DAOManaged
	@XMLElement
	private String citizenID;

	@DAOManaged
	@XMLElement
	private String firstname;

	@DAOManaged
	@XMLElement
	private String lastname;

	@DAOManaged
	@XMLElement
	private String email;

	@DAOManaged
	@XMLElement
	private String mobilePhone;

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	@Override
	public String toString() {

		return this.getClass().getSimpleName() + " (queryInstanceID=" + queryInstanceID + ")";
	}

	@Override
	public RemoteSinglesignQuery getQuery() {

		return query;
	}

	public void setQuery(RemoteSinglesignQuery query) {

		this.query = query;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		if (query.isSetAsAttribute()) {

			resetAttribute(attributeHandler);
		}

		this.citizenID = null;
		this.firstname = null;
		this.lastname = null;
		this.email = null;
		this.mobilePhone = null;

		super.reset(attributeHandler);
	}

	public void setAttribute(MutableAttributeHandler attributeHandler) {

		if (citizenID != null) {

			//			attributeHandler.setAttribute(query.getAttributeName() + ".firstname", firstname);
			//			attributeHandler.setAttribute(query.getAttributeName() + ".lastname", lastname);
			//			attributeHandler.setAttribute(query.getAttributeName() + ".citizenID", citizenID);
			//			attributeHandler.setAttribute(query.getAttributeName() + ".email", email);
			//			attributeHandler.setAttribute(query.getAttributeName() + ".mobilePhone", mobilePhone);
		}
	}

	public void resetAttribute(MutableAttributeHandler attributeHandler) {

		//		attributeHandler.removeAttribute(query.getAttributeName() + ".firstname");
		//		attributeHandler.removeAttribute(query.getAttributeName() + ".lastname");
		//		attributeHandler.removeAttribute(query.getAttributeName() + ".citizenID");
		//		attributeHandler.removeAttribute(query.getAttributeName() + ".email");
		//		attributeHandler.removeAttribute(query.getAttributeName() + ".mobilePhone");
	}

	public String getCitizenID() {

		return citizenID;
	}

	public void setCitizenID(String citizenID) {

		this.citizenID = citizenID;
	}

	public String getFirstname() {

		return firstname;
	}

	public void setFirstname(String firstname) {

		this.firstname = firstname;
	}

	public String getLastname() {

		return lastname;
	}

	public void setLastname(String lastname) {

		this.lastname = lastname;
	}

	public String getEmail() {

		return email;
	}

	public void setEmail(String email) {

		this.email = email;
	}

	public String getMobilePhone() {

		return mobilePhone;
	}

	public void setMobilePhone(String mobilePhone) {

		this.mobilePhone = mobilePhone;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewCDATAElement(doc, element, "Firstname", firstname);
		XMLUtils.appendNewCDATAElement(doc, element, "Lastname", lastname);
		XMLUtils.appendNewCDATAElement(doc, element, "CitizenIdentifier", citizenID);
		XMLUtils.appendNewCDATAElement(doc, element, "Email", email);
		XMLUtils.appendNewCDATAElement(doc, element, "MobilePhone", mobilePhone);

		return element;
	}

	@Override
	public String getStringValue() {

		return firstname + " " + lastname;
	}

	@Override
	public List<String> getColumnLabels(QueryHandler queryHandler) {

		RemoteSinglesignQueryProviderModule queryProvider = queryHandler.getQueryProvider(getQueryInstanceDescriptor().getQueryDescriptor().getQueryTypeID(), RemoteSinglesignQueryProviderModule.class);

		List<String> labels = new ArrayList<String>();

		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportFirstName());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportLastName());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportCitizenID());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportEmail());
		labels.add(getQueryInstanceDescriptor().getQueryDescriptor().getName() + ": " + queryProvider.getExportMobilePhone());

		return labels;
	}

	@Override
	public List<String> getColumnValues() {

		List<String> values = new ArrayList<String>();

		values.add(firstname);
		values.add(lastname);
		values.add(citizenID);
		values.add(email);
		values.add(mobilePhone);

		return values;
	}

	@Override
	public List<? extends SigningParty> getSigningParties() {

		if (citizenID == null) {
			
			return null;
		}

		return Collections.singletonList(new SigningParty(firstname, lastname, email, mobilePhone, citizenID, query.isSetMultipartsAsOwners()));
	}
}
