package com.nordicpeak.flowengine.reservation;

import java.util.Enumeration;
import java.util.List;

import javax.servlet.http.HttpSessionEvent;

import org.apache.log4j.Logger;

import se.unlogic.hierarchy.core.interfaces.SystemInterface;
import se.unlogic.hierarchy.core.interfaces.listeners.SessionListener;
import se.unlogic.standardutils.collections.CollectionUtils;

import com.nordicpeak.flowengine.Constants;
import com.nordicpeak.flowengine.managers.MutableFlowInstanceManager;

public class AbortedReservationListener implements SessionListener {

	private Logger log = Logger.getLogger(this.getClass());

	private ReservationModule reservationModule;

	public AbortedReservationListener(ReservationModule reservationModule) {

		this.reservationModule = reservationModule;
	}

	public void init(SystemInterface systemInterface) throws Exception {

		if (systemInterface.getSessionListenerHandler() != null) {

			systemInterface.getSessionListenerHandler().addListener(this);

		} else {

			log.warn("No session listener handler available, unable to listen for expired sessions");
		}
	}

	public void unload(SystemInterface systemInterface) throws Exception {

		if (systemInterface.getSessionListenerHandler() != null) {

			systemInterface.getSessionListenerHandler().removeListener(this);
		}
	}

	@Override
	public void sessionCreated(HttpSessionEvent event) {}

	@Override
	public void sessionDestroyed(HttpSessionEvent event) {

		List<MutableFlowInstanceManager> flowInstanceManagers = getFlowInstanceManagers(event);

		if (flowInstanceManagers == null) {

			return;
		}

		try {

			for (MutableFlowInstanceManager flowInstanceManager : flowInstanceManagers) {

				String flowInstanceManagerID = flowInstanceManager.getInstanceManagerID();

				int deletedCount = reservationModule.deleteReservationsByFlowInstanceManagerID(flowInstanceManagerID);

				log.info("Deleted " + deletedCount + " reservations for flowInstanceManager " + flowInstanceManager);
			}

		} catch (Exception e) {

			log.error("Error deleting reservations", e);
		}
	}

	public List<MutableFlowInstanceManager> getFlowInstanceManagers(HttpSessionEvent event) {

		List<MutableFlowInstanceManager> flowInstanceManagers = null;

		Enumeration<String> enumeration = event.getSession().getAttributeNames();

		while (enumeration.hasMoreElements()) {

			String attributeName = enumeration.nextElement();

			if (attributeName.startsWith(Constants.FLOW_INSTANCE_SESSION_PREFIX)) {

				MutableFlowInstanceManager flowInstanceManager = (MutableFlowInstanceManager) event.getSession().getAttribute(attributeName);

				flowInstanceManagers = CollectionUtils.addAndInstantiateIfNeeded(flowInstanceManagers, flowInstanceManager);
			}
		}

		return flowInstanceManagers;
	}

}
