package com.nordicpeak.flowengine.queries.ssbt;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.ManyToOne;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryInstance;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryUtils;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;
import com.nordicpeak.flowengine.queries.ssbt.enums.CompanyType;
import com.nordicpeak.flowengine.queries.ssbt.enums.ErrorType;

@Table(name = "ssbt2_company_detail_query_instances")
@XMLElement
public class CompanyDetailsQueryInstance extends BaseQueryInstance implements FixedAlternativesQueryInstance {

	private static final long serialVersionUID = 4807658445190901111L;

	public static final Field QUERY_RELATION = ReflectionUtils.getField(CompanyDetailsQueryInstance.class, "query");
	public static final Field SAVED_COMPANIES_RELATION = ReflectionUtils.getField(CompanyDetailsQueryInstance.class, "savedCompanies");

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryInstanceID;

	//UD0001
	@DAOManaged
	@XMLElement
	private String companyName;

	//From SSBTEN
	@DAOManaged
	@XMLElement
	private String organizationNumber;

	//From SSBTEN
	@DAOManaged
	@XMLElement
	private Integer nameProtectionNumber;

	//UD0003
	@DAOManaged
	@XMLElement
	private String legalEntityAddress;

	@DAOManaged
	@XMLElement
	private String legalEntityCOAddress;

	@DAOManaged
	@XMLElement
	private String legalEntityZip;

	@DAOManaged
	@XMLElement
	private String legalEntityPostalAddress;

	//UD0004
	@DAOManaged
	@XMLElement
	private String email;

	//UD0006
	@DAOManaged
	@XMLElement
	private String phone;

	//UD0023
	@DAOManaged
	@XMLElement
	private String civilRegistrationAddress;

	@DAOManaged
	@XMLElement
	private String civilRegistrationCOAddress;

	@DAOManaged
	@XMLElement
	private String civilRegistrationZip;

	@DAOManaged
	@XMLElement
	private String civilRegistrationPostalAddress;

	//UD0025
	@DAOManaged
	@XMLElement
	private CompanyType companyType;

	@DAOManaged
	@XMLElement
	private String companyTypeDescription;

	//UD0029
	@DAOManaged
	@XMLElement
	private String fTaxRegistered;

	//UD0031
	@DAOManaged
	@XMLElement
	private String vatRegistered;

	//UD0033
	@DAOManaged
	@XMLElement
	private String employerRegistration;

	//UD0045
	@DAOManaged
	@XMLElement
	private String signing;

	//UD0046
	@DAOManaged
	@XMLElement
	private String businessDescription;

	//UD0047
	@DAOManaged
	@XMLElement
	private String officers;

	//UD0048
	@DAOManaged
	@XMLElement
	private String shares;

	@DAOManaged(columnName = "queryID")
	@ManyToOne
	@XMLElement
	private CompanyDetailsQuery query;

	@DAOManaged
	@OneToMany(autoAdd = true, autoUpdate = true, autoGet = true)
	@XMLElement(fixCase = true)
	private List<SavedCompany> savedCompanies;

	//Non persisted list of available companies
	@XMLElement(fixCase = true)
	private List<Company> companies;

	@XMLElement
	private ErrorType error;

	@Override
	public CompanyDetailsQuery getQuery() {

		return query;
	}

	@Override
	public Element toExportXML(Document doc, QueryHandler queryHandler) throws Exception {

		Element element = getBaseExportXML(doc);

		XMLUtils.appendNewCDATAElement(doc, element, "CompanyName", companyName);

		XMLUtils.appendNewCDATAElement(doc, element, "OrganizationNumber", organizationNumber);

		XMLUtils.appendNewCDATAElement(doc, element, "NameProtectionNumber", nameProtectionNumber);

		XMLUtils.appendNewCDATAElement(doc, element, "LegalEntityAddress", legalEntityAddress);
		XMLUtils.appendNewCDATAElement(doc, element, "LegalEntityCOAddress", legalEntityCOAddress);
		XMLUtils.appendNewCDATAElement(doc, element, "LegalEntityZip", legalEntityZip);
		XMLUtils.appendNewCDATAElement(doc, element, "LegalEntityPostalAddress", legalEntityPostalAddress);

		XMLUtils.appendNewCDATAElement(doc, element, "Email", email);
		XMLUtils.appendNewCDATAElement(doc, element, "Phone", phone);

		XMLUtils.appendNewCDATAElement(doc, element, "CivilRegistrationAddress", civilRegistrationAddress);
		XMLUtils.appendNewCDATAElement(doc, element, "CivilRegistrationCOAddress", civilRegistrationCOAddress);
		XMLUtils.appendNewCDATAElement(doc, element, "CivilRegistrationZip", civilRegistrationZip);
		XMLUtils.appendNewCDATAElement(doc, element, "CivilRegistrationPostalAddress", civilRegistrationPostalAddress);

		XMLUtils.appendNewCDATAElement(doc, element, "CompanyType", companyType);
		XMLUtils.appendNewCDATAElement(doc, element, "CompanyTypeDescription", companyTypeDescription);

		XMLUtils.appendNewCDATAElement(doc, element, "Signing", signing);

		XMLUtils.appendNewCDATAElement(doc, element, "BusinessDescription", businessDescription);

		XMLUtils.appendNewCDATAElement(doc, element, "Officers", officers);

		XMLUtils.appendNewCDATAElement(doc, element, "Shares", shares);

		XMLUtils.appendNewCDATAElement(doc, element, "FTaxRegistered", fTaxRegistered);

		XMLUtils.appendNewCDATAElement(doc, element, "VATRegistered", vatRegistered);

		XMLUtils.appendNewCDATAElement(doc, element, "EmployerRegistration", employerRegistration);

		return element;
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(HttpServletRequest req, User user, User poster, URIParser uriParser, QueryHandler queryHandler, InstanceRequestMetadata requestMetadata) throws Exception {

		return BaseQueryUtils.getGenericQueryInstanceProvider(this.getClass(), queryHandler, queryInstanceDescriptor.getQueryDescriptor().getQueryTypeID()).getQueryRequestProcessor(this, req, user, poster, uriParser, requestMetadata);
	}

	public Integer getQueryInstanceID() {

		return queryInstanceID;
	}

	public void setQueryInstanceID(Integer queryInstanceID) {

		this.queryInstanceID = queryInstanceID;
	}

	public void setQuery(CompanyDetailsQuery query) {

		this.query = query;
	}

	public List<Company> getCompanies() {

		return companies;
	}

	public void setCompanies(List<Company> companies) {

		this.companies = companies;
	}

	public ErrorType getError() {

		return error;
	}

	public void setError(ErrorType error) {

		this.error = error;
	}

	@Override
	public void reset(MutableAttributeHandler attributeHandler) {

		companyName = null;
		organizationNumber = null;
		nameProtectionNumber = null;
		legalEntityAddress = null;
		legalEntityCOAddress = null;
		legalEntityZip = null;
		legalEntityPostalAddress = null;
		email = null;
		phone = null;
		civilRegistrationAddress = null;
		civilRegistrationCOAddress = null;
		civilRegistrationZip = null;
		civilRegistrationPostalAddress = null;
		companyType = null;
		companyTypeDescription = null;
		fTaxRegistered = null;
		vatRegistered = null;
		employerRegistration = null;
		signing = null;
		businessDescription = null;
		officers = null;
		shares = null;

		this.phone = null;

		//this.companies = null;
		savedCompanies = null;
		error = null;

		if (query.isSetAsAttribute()) {
			resetAttributes(attributeHandler);
		}

		super.reset(attributeHandler);
	}

	public void resetAttributes(MutableAttributeHandler attributeHandler) {

		attributeHandler.removeAttribute(query.getAttributeName() + ".organizationNumber");
		attributeHandler.removeAttribute(query.getAttributeName() + ".companyName");
		attributeHandler.removeAttribute(query.getAttributeName() + ".companyType");
		attributeHandler.removeAttribute(query.getAttributeName() + ".companyTypeDescription");
	}

	public void setAttributes(MutableAttributeHandler attributeHandler) {

		attributeHandler.setAttribute(query.getAttributeName() + ".organizationNumber", organizationNumber);
		attributeHandler.setAttribute(query.getAttributeName() + ".companyName", companyName);
		attributeHandler.setAttribute(query.getAttributeName() + ".companyType", companyType);
		attributeHandler.setAttribute(query.getAttributeName() + ".companyTypeDescription", companyTypeDescription);
	}

	public static long getSerialversionuid() {

		return serialVersionUID;
	}

	public static Field getQueryRelation() {

		return QUERY_RELATION;
	}

	public String getCompanyName() {

		return companyName;
	}

	public String getOrganizationNumber() {

		return organizationNumber;
	}

	public Integer getNameProtectionNumber() {

		return nameProtectionNumber;
	}

	public String getLegalEntityAddress() {

		return legalEntityAddress;
	}

	public String getLegalEntityCOAddress() {

		return legalEntityCOAddress;
	}

	public String getLegalEntityZip() {

		return legalEntityZip;
	}

	public String getLegalEntityPostalAddress() {

		return legalEntityPostalAddress;
	}

	public String getEmail() {

		return email;
	}

	public String getPhone() {

		return phone;
	}

	public String getCivilRegistrationAddress() {

		return civilRegistrationAddress;
	}

	public String getCivilRegistrationCOAddress() {

		return civilRegistrationCOAddress;
	}

	public String getCivilRegistrationZip() {

		return civilRegistrationZip;
	}

	public String getCivilRegistrationPostalAddress() {

		return civilRegistrationPostalAddress;
	}

	public CompanyType getCompanyType() {

		return companyType;
	}

	public String getCompanyTypeDescription() {

		return companyTypeDescription;
	}

	public String getSigning() {

		return signing;
	}

	public String getBusinessDescription() {

		return businessDescription;
	}

	public String getOfficers() {

		return officers;
	}

	public String getShares() {

		return shares;
	}

	public void setCompanyDetails(Company company) {

		this.organizationNumber = company.getOrganizationNumber();
		this.nameProtectionNumber = company.getNameProtectionNumber();
		this.companyType = company.getCompanyType();
		this.companyTypeDescription = company.getCompanyTypeDescription();

		CompanyDetails companyDetails = company.getCompanyDetails();

		this.fTaxRegistered = companyDetails.getfTaxRegistered();
		this.vatRegistered = companyDetails.getVatRegistered();
		this.employerRegistration = companyDetails.getEmployerRegistration();

		this.companyName = companyDetails.getCompanyName();
		this.legalEntityAddress = companyDetails.getLegalEntityAddress();
		this.legalEntityCOAddress = companyDetails.getLegalEntityCOAddress();
		this.legalEntityZip = companyDetails.getLegalEntityZip();
		this.legalEntityPostalAddress = companyDetails.getLegalEntityPostalAddress();
		this.email = companyDetails.getEmail();
		this.phone = companyDetails.getPhone();
		this.civilRegistrationAddress = companyDetails.getCivilRegistrationAddress();
		this.civilRegistrationCOAddress = companyDetails.getCivilRegistrationCOAddress();
		this.civilRegistrationZip = companyDetails.getCivilRegistrationZip();
		this.civilRegistrationPostalAddress = companyDetails.getCivilRegistrationPostalAddress();
		this.signing = companyDetails.getSigning();
		this.businessDescription = companyDetails.getBusinessDescription();
		this.officers = companyDetails.getOfficers();
		this.shares = companyDetails.getShares();
	}

	@Override
	public List<? extends ImmutableAlternative> getAlternatives() {

		if (companyType != null) {

			ArrayList<ImmutableAlternative> alternatives = new ArrayList<ImmutableAlternative>(2);

			if (companyType == CompanyType.E && nameProtectionNumber == null) {

				alternatives.add(new UnregisteredCompanyTypeAlternative());

			} else {

				alternatives.add(companyType.getAlternative());
			}

			if (email != null) {

				alternatives.add(CompanyDetailsQuery.UD0004_ALTERNATIVE);
			}

			if (phone != null) {

				alternatives.add(CompanyDetailsQuery.UD0006_ALTERNATIVE);
			}

			if (fTaxRegistered != null) {

				if (fTaxRegistered.equalsIgnoreCase("Ja")) {

					alternatives.add(CompanyDetailsQuery.UD0029_ALTERNATIVE_YES);

				} else if (fTaxRegistered.equalsIgnoreCase("Nej")) {

					alternatives.add(CompanyDetailsQuery.UD0029_ALTERNATIVE_NO);

				} else if (fTaxRegistered.equalsIgnoreCase(CompanyDetailsQuery.ROLE_ACCESS_DENIED)) {

					alternatives.add(CompanyDetailsQuery.UD0029_ALTERNATIVE_ACCESS_DENIED);
				}
			}

			if (vatRegistered != null) {

				if (vatRegistered.equalsIgnoreCase("Ja")) {

					alternatives.add(CompanyDetailsQuery.UD0031_ALTERNATIVE_YES);

				} else if (vatRegistered.equalsIgnoreCase("Nej")) {

					alternatives.add(CompanyDetailsQuery.UD0031_ALTERNATIVE_NO);

				} else if (vatRegistered.equalsIgnoreCase(CompanyDetailsQuery.ROLE_ACCESS_DENIED)) {

					alternatives.add(CompanyDetailsQuery.UD0031_ALTERNATIVE_ACCESS_DENIED);
				}
			}

			if (employerRegistration != null) {

				if (employerRegistration.equalsIgnoreCase("Ja")) {

					alternatives.add(CompanyDetailsQuery.UD0033_ALTERNATIVE_YES);

				} else if (employerRegistration.equalsIgnoreCase("Nej")) {

					alternatives.add(CompanyDetailsQuery.UD0033_ALTERNATIVE_NO);

				} else if (employerRegistration.equalsIgnoreCase(CompanyDetailsQuery.ROLE_ACCESS_DENIED)) {

					alternatives.add(CompanyDetailsQuery.UD0033_ALTERNATIVE_ACCESS_DENIED);
				}
			}

			return alternatives;
		}

		return null;
	}

	@Override
	public String getFreeTextAlternativeValue() {

		return null;
	}

	public List<SavedCompany> getSavedCompanies() {

		return savedCompanies;
	}

	public void setSavedCompanies(List<SavedCompany> savedCompanies) {

		this.savedCompanies = savedCompanies;
	}

}
