package com.nordicpeak.flowengine.queries.ssbt;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.ws.BindingProvider;

import org.apache.log4j.Level;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.bolagsverket.schema.ssbt.fel.FelTyp;
import se.bolagsverket.schema.ssbt.foretag.Aktier;
import se.bolagsverket.schema.ssbt.foretag.Aktieslag;
import se.bolagsverket.schema.ssbt.foretag.ForetagId;
import se.bolagsverket.schema.ssbt.foretag.Foretagsform;
import se.bolagsverket.schema.ssbt.foretag.PersonId;
import se.bolagsverket.schema.ssbt.foretag.PersonIdentitetsbeteckning;
import se.bolagsverket.schema.ssbt.metadata.Anvandningsomrade;
import se.bolagsverket.schema.ssbt.metadata.Datakonsument;
import se.bolagsverket.schema.ssbt.metadata.Part;
import se.bolagsverket.schema.ssbt.metadata.PartId;
import se.bolagsverket.schema.ssbt.metadata.Service;
import se.bolagsverket.schema.ssbt.metadata.Tidsenhet;
import se.bolagsverket.schema.ssbt.metadata.Ttl;
import se.bolagsverket.schema.ssbt.rollintyg.RollIntyg;
import se.bolagsverket.schema.ssbten.SsbtenService;
import se.bolagsverket.schema.ssbten.SsbtenServicePorts;
import se.bolagsverket.schema.ssbten.engagemang.EngagemangBegaran;
import se.bolagsverket.schema.ssbten.engagemang.EngagemangBegaranDetaljer;
import se.bolagsverket.schema.ssbten.engagemang.EngagemangBegaranMetadata;
import se.bolagsverket.schema.ssbten.engagemang.EngagemangSvar;
import se.bolagsverket.schema.ssbten.engagemang.Foretag;
import se.bolagsverket.schema.ssbten.engagemang.ForetagEngagemang;
import se.bolagsverket.schema.ssbten.engagemang.Foretagsformer;
import se.bolagsverket.schema.ssbtgu.v2.SsbtguService;
import se.bolagsverket.schema.ssbtgu.v2.SsbtguServicePorts;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.GrundlaggandeUppgiftId;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.GrundlaggandeUppgifterBegaran;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.GrundlaggandeUppgifterBegaranDetaljer;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.GrundlaggandeUppgifterBegaranMetadata;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.GrundlaggandeUppgifterSvar;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0001;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0003;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0004;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0006;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0023;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0029;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0031;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0033;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0045;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0046;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0047;
import se.bolagsverket.schema.ssbtgu.v2.grundlaggandeuppgifter.Ud0048;
import se.bolagsverket.schema.ssbtro.SsbtroService;
import se.bolagsverket.schema.ssbtro.SsbtroServicePorts;
import se.bolagsverket.schema.ssbtro.roll.RollBegaran;
import se.bolagsverket.schema.ssbtro.roll.RollBegaranDetaljer;
import se.bolagsverket.schema.ssbtro.roll.RollBegaranMetadata;
import se.bolagsverket.schema.ssbtro.roll.RollSvar;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.annotations.XSLVariable;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.hierarchy.core.utils.FCKUtils;
import se.unlogic.hierarchy.core.utils.ModuleUtils;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.enums.EnumUtils;
import se.unlogic.standardutils.jaxws.WSUtils;
import se.unlogic.standardutils.random.RandomUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.NonNegativeStringIntegerValidator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.RequestUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.url.URLRewriter;

import com.nordicpeak.flowengine.beans.InstanceRequestMetadata;
import com.nordicpeak.flowengine.beans.QueryResponse;
import com.nordicpeak.flowengine.enums.QueryState;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.MutableQueryInstanceDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryContentFilter;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.interfaces.QueryRequestProcessor;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryCRUDCallback;
import com.nordicpeak.flowengine.queries.basequery.BaseQueryProviderModule;
import com.nordicpeak.flowengine.queries.ssbt.enums.CompanyType;
import com.nordicpeak.flowengine.queries.ssbt.enums.ErrorType;
import com.nordicpeak.flowengine.queries.ssbt.enums.TargetEnvironment;
import com.nordicpeak.flowengine.queries.ssbt.exceptions.IncompleteResponseException;
import com.nordicpeak.flowengine.utils.CitizenIdentifierUtils;
import com.nordicpeak.flowengine.utils.JTidyUtils;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class CompanyDetailsQueryProviderModule extends BaseQueryProviderModule<CompanyDetailsQueryInstance> implements BaseQueryCRUDCallback {

	@XSLVariable(prefix = "java.")
	private String shareCapital;

	@XSLVariable(prefix = "java.")
	private String shareCount;

	@XSLVariable(prefix = "java.")
	private String distributionOfIssuedShares;

	@XSLVariable(prefix = "java.")
	private String shareType;

	@XSLVariable(prefix = "java.")
	private String count;

	@XSLVariable(prefix = "java.")
	private String voteValue;

	@XSLVariable(prefix = "java.")
	private String hardCodedCompanyName;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Consumer ID", description = "The consumer ID to use for SSBT queries.", required = true)
	private String consumerID;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Consumer name", description = "The consumer name to use for SSBT queries.", required = true)
	private String consumerName;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Keystore path", description = "Path to the keystore.", required = true)
	private String keyStorePath;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Keystore password", description = "The password to the keystore and certificate.", required = true)
	private String keyStorePass;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "Connection timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	private Integer connectionTimeout = 5;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Request timeout", description = "Request timeout in seconds", formatValidator = NonNegativeStringIntegerValidator.class, required = true)
	private Integer requestTimeout = 10;

	private AnnotatedDAO<CompanyDetailsQuery> queryDAO;
	private AnnotatedDAO<CompanyDetailsQueryInstance> queryInstanceDAO;

	private CompanyDetailsQueryCRUD queryCRUD;

	private QueryParameterFactory<CompanyDetailsQuery, Integer> queryIDParamFactory;
	private QueryParameterFactory<CompanyDetailsQueryInstance, Integer> queryInstanceIDParamFactory;

	private boolean configured;

	private HashMap<TargetEnvironment, SsbtenService> enServices;
	private HashMap<TargetEnvironment, SsbtguService> guServices;
	private HashMap<TargetEnvironment, SsbtroService> roServices;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, CompanyDetailsQueryProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		SimpleAnnotatedDAOFactory daoFactory = new SimpleAnnotatedDAOFactory(dataSource);

		queryDAO = daoFactory.getDAO(CompanyDetailsQuery.class);
		queryInstanceDAO = daoFactory.getDAO(CompanyDetailsQueryInstance.class);

		queryCRUD = new CompanyDetailsQueryCRUD(queryDAO.getWrapper(Integer.class), new AnnotatedRequestPopulator<CompanyDetailsQuery>(CompanyDetailsQuery.class), "CompanyDetailsQuery", "query", null, this);

		queryIDParamFactory = queryDAO.getParamFactory("queryID", Integer.class);
		queryInstanceIDParamFactory = queryInstanceDAO.getParamFactory("queryInstanceID", Integer.class);
		//queryInstanceQueryParamFactory = queryInstanceDAO.getParamFactory("query", CompanyDetailsQuery.class);
	}

	@Override
	protected synchronized void moduleConfigured() throws Exception {

		super.moduleConfigured();

		if (ModuleUtils.checkRequiredModuleSettings(moduleDescriptor, this, systemInterface, Level.ERROR)) {

			try {
				HashMap<TargetEnvironment, SsbtenService> enServiceMap = new HashMap<TargetEnvironment, SsbtenService>(3);

				SsbtenService enTestService = new SsbtenServicePorts().getSsbtenService();
				WSUtils.setEndpoint((BindingProvider) enTestService, "https://ssbtgu-accept2.bolagsverket.se/ssbten-dft-web/SsbtServicePorts/Test/SsbtEnTestService");
				WSUtils.setKeyStore((BindingProvider) enTestService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) enTestService, connectionTimeout, requestTimeout);
				enServiceMap.put(TargetEnvironment.TEST, enTestService);

				SsbtenService enAcceptService = new SsbtenServicePorts().getSsbtenService();
				WSUtils.setEndpoint((BindingProvider) enAcceptService, "https://ssbtgu-accept2.bolagsverket.se/ssbten-dft-web/SsbtenServicePorts/SsbtenService");
				WSUtils.setKeyStore((BindingProvider) enAcceptService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) enAcceptService, connectionTimeout, requestTimeout);
				enServiceMap.put(TargetEnvironment.ACCEPTANCE, enAcceptService);

				SsbtenService enProdService = new SsbtenServicePorts().getSsbtenService();
				WSUtils.setEndpoint((BindingProvider) enProdService, "https://ssbtgu.bolagsverket.se/ssbten-dft-web/SsbtenServicePorts/SsbtenService");
				WSUtils.setKeyStore((BindingProvider) enProdService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) enProdService, connectionTimeout, requestTimeout);
				enServiceMap.put(TargetEnvironment.PRODUCTION, enProdService);

				HashMap<TargetEnvironment, SsbtguService> guServiceMap = new HashMap<TargetEnvironment, SsbtguService>(3);

				SsbtguService guTestService = new SsbtguServicePorts().getSsbtguService();
				WSUtils.setEndpoint((BindingProvider) guTestService, "https://ssbtgu-accept2.bolagsverket.se/ssbtgu-dft-web-2/SsbtServicePorts/Test/SsbtTestService");
				WSUtils.setKeyStore((BindingProvider) guTestService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) guTestService, connectionTimeout, requestTimeout);
				guServiceMap.put(TargetEnvironment.TEST, guTestService);

				SsbtguService guAcceptService = new SsbtguServicePorts().getSsbtguService();
				WSUtils.setEndpoint((BindingProvider) guAcceptService, "https://ssbtgu-accept2.bolagsverket.se/ssbtgu-dft-web-2/SsbtServicePorts/SsbtService");
				WSUtils.setKeyStore((BindingProvider) guAcceptService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) guAcceptService, connectionTimeout, requestTimeout);
				guServiceMap.put(TargetEnvironment.ACCEPTANCE, guAcceptService);

				SsbtguService guProdService = new SsbtguServicePorts().getSsbtguService();
				WSUtils.setEndpoint((BindingProvider) guProdService, "https://ssbtgu.bolagsverket.se/ssbtgu-dft-web-2/SsbtServicePorts/SsbtService");
				WSUtils.setKeyStore((BindingProvider) guProdService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) guProdService, connectionTimeout, requestTimeout);
				guServiceMap.put(TargetEnvironment.PRODUCTION, guProdService);

				HashMap<TargetEnvironment, SsbtroService> roServiceMap = new HashMap<TargetEnvironment, SsbtroService>(3);
				
				SsbtroService roTestService = new SsbtroServicePorts().getSsbtroService();
				WSUtils.setEndpoint((BindingProvider) roTestService, "https://ssbtgu-accept2.bolagsverket.se/ssbtro-dft-web/SsbtServicePorts/Test/SsbtRoTestService");
				WSUtils.setKeyStore((BindingProvider) roTestService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) roTestService, connectionTimeout, requestTimeout);
				roServiceMap.put(TargetEnvironment.TEST, roTestService);

				SsbtroService roAcceptService = new SsbtroServicePorts().getSsbtroService();
				WSUtils.setEndpoint((BindingProvider) roAcceptService, "https://ssbtgu-accept2.bolagsverket.se/ssbtro-dft-web/SsbtroServicePorts/SsbtroService");
				WSUtils.setKeyStore((BindingProvider) roAcceptService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) roAcceptService, connectionTimeout, requestTimeout);
				roServiceMap.put(TargetEnvironment.ACCEPTANCE, roAcceptService);

				SsbtroService roProdService = new SsbtroServicePorts().getSsbtroService();
				WSUtils.setEndpoint((BindingProvider) roProdService, "https://ssbtgu.bolagsverket.se/ssbtro-dft-web/SsbtroServicePorts/SsbtroService");
				WSUtils.setKeyStore((BindingProvider) roProdService, keyStorePath, keyStorePass);
				WSUtils.setTimeouts((BindingProvider) roProdService, connectionTimeout, requestTimeout);
				roServiceMap.put(TargetEnvironment.PRODUCTION, roProdService);
				
				this.enServices = enServiceMap;
				this.guServices = guServiceMap;
				this.roServices = roServiceMap;

				configured = true;

				return;

			} catch (Exception e) {

				log.error("Error creating proxy instances", e);
			}
		}

		configured = false;

		enServices = null;
		guServices = null;
	}

	@Override
	public Query createQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		CompanyDetailsQuery query = new CompanyDetailsQuery();

		query.setQueryID(descriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query importQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap, QueryContentFilter contentFilter) throws Throwable {

		CompanyDetailsQuery query = new CompanyDetailsQuery();

		query.setQueryID(descriptor.getQueryID());

		query.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(query.getClass())));
		
		contentFilter.filterHTML(query);

		this.queryDAO.add(query, transactionHandler, null);

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, boolean extraData) throws SQLException {

		CompanyDetailsQuery query = this.getQuery(descriptor.getQueryID());

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public Query getQuery(MutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		CompanyDetailsQuery query = this.getQuery(descriptor.getQueryID(), transactionHandler);

		if (query == null) {

			return null;
		}

		query.init(descriptor, getFullAlias() + "/config/" + descriptor.getQueryID());

		return query;
	}

	@Override
	public QueryInstance getQueryInstance(MutableQueryInstanceDescriptor descriptor, String instanceManagerID, HttpServletRequest req, User user, User poster, InstanceMetadata instanceMetadata) throws SQLException {

		CompanyDetailsQueryInstance queryInstance = null;

		//Check if we should create a new instance or get an existing one
		if (descriptor.getQueryInstanceID() == null) {

			queryInstance = new CompanyDetailsQueryInstance();

		} else {

			queryInstance = getQueryInstance(descriptor.getQueryInstanceID());

			if (queryInstance == null) {

				return null;
			}
		}

		queryInstance.setQuery(getQuery(descriptor.getQueryDescriptor().getQueryID()));

		if (queryInstance.getQuery() == null) {

			return null;
		}

		if (req != null) {

			FCKUtils.setAbsoluteFileUrls(queryInstance.getQuery(), RequestUtils.getFullContextPathURL(req) + ckConnectorModuleAlias);

			URLRewriter.setAbsoluteLinkUrls(queryInstance.getQuery(), req, true);
		}

		queryInstance.getQuery().scanAttributeTags();

		TextTagReplacer.replaceTextTags(queryInstance.getQuery(), instanceMetadata.getSiteProfile());

		queryInstance.set(descriptor);

		return queryInstance;
	}

	private CompanyDetailsQuery getQuery(Integer queryID) throws SQLException {

		HighLevelQuery<CompanyDetailsQuery> query = new HighLevelQuery<CompanyDetailsQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query);
	}

	private CompanyDetailsQuery getQuery(Integer queryID, TransactionHandler transactionHandler) throws SQLException {

		HighLevelQuery<CompanyDetailsQuery> query = new HighLevelQuery<CompanyDetailsQuery>();

		query.addParameter(queryIDParamFactory.getParameter(queryID));

		return queryDAO.get(query, transactionHandler);
	}

	private CompanyDetailsQueryInstance getQueryInstance(Integer queryInstanceID) throws SQLException {

		HighLevelQuery<CompanyDetailsQueryInstance> query = new HighLevelQuery<CompanyDetailsQueryInstance>(CompanyDetailsQueryInstance.QUERY_RELATION);

		query.addParameter(queryInstanceIDParamFactory.getParameter(queryInstanceID));

		return queryInstanceDAO.get(query);
	}

	@Override
	public void save(CompanyDetailsQueryInstance queryInstance, TransactionHandler transactionHandler, InstanceRequestMetadata requestMetadata) throws Throwable {

		//Check if the query instance has an ID set and if the ID of the descriptor has changed
		if (queryInstance.getQueryInstanceID() == null || !queryInstance.getQueryInstanceID().equals(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID())) {

			queryInstance.setQueryInstanceID(queryInstance.getQueryInstanceDescriptor().getQueryInstanceID());

			this.queryInstanceDAO.add(queryInstance, transactionHandler, null);

		} else {

			this.queryInstanceDAO.update(queryInstance, transactionHandler, null);
		}
	}

	@Override
	public void populate(CompanyDetailsQueryInstance queryInstance, HttpServletRequest req, User user, User poster, boolean allowPartialPopulation, MutableAttributeHandler attributeHandler, InstanceRequestMetadata requestMetadata) throws ValidationException {

		if (requestMetadata.isManager()) {
			
			if (queryInstance.getQueryInstanceDescriptor().isPopulated() || allowPartialPopulation) {
				
				return;
			}
				
			throw new ValidationException(new ValidationError("RequiredQuery"));
		}
		
		CompanyDetailsQuery query = queryInstance.getQuery();
		
		String fieldName = "q" + query.getQueryID() + "_organizationNumber";
		String organizationNumber = req.getParameter(fieldName);
		
		if(StringUtils.isEmpty(organizationNumber)){
			
			organizationNumber = null;
		}
		
		Company company = getCompany(organizationNumber, queryInstance);

		if (company == null) {

			if (organizationNumber != null) {
				
				throw new ValidationException(new ValidationError("InvalidCompany"));
			}
			
			if (queryInstance.getQueryInstanceDescriptor().getQueryState() == QueryState.VISIBLE_REQUIRED && !allowPartialPopulation) {

				throw new ValidationException(new ValidationError("RequiredQuery"));
			}

			queryInstance.reset(attributeHandler);
			queryInstance.getQueryInstanceDescriptor().setPopulated(false);
			return;
		}

		if (company.getCompanyDetails() == null) {

			getCompanyDetails(user, company, queryInstance);

			if (company.getCompanyDetails() == null) {

				queryInstance.reset(attributeHandler);

				throw new ValidationException(new ValidationError("ErrorGettingCompanyDetails"));
			}
		}

		if (query.isShowCompaniesAsRadiosAndKeepShowing() && !CollectionUtils.isEmpty(queryInstance.getCompanies())) {
			
			List<SavedCompany> savedCompanies = new ArrayList<>(queryInstance.getCompanies().size());
			
			for (Company company2 : queryInstance.getCompanies()) {
				
				savedCompanies.add(new SavedCompany(company2));
			}
			
			queryInstance.setSavedCompanies(savedCompanies);
		}
		
		queryInstance.setCompanyDetails(company);
		
		if (query.isSetAsAttribute()) {

			queryInstance.resetAttributes(attributeHandler);
			queryInstance.setAttributes(attributeHandler);
		}
		
		queryInstance.getQueryInstanceDescriptor().setPopulated(true);
	}

	@Override
	public QueryRequestProcessor getQueryRequestProcessor(CompanyDetailsQueryInstance queryInstance, HttpServletRequest req, User user, User poster, URIParser uriParser, InstanceRequestMetadata requestMetadata) throws Exception {

		Company company = getCompany(req.getParameter("organizationNumber"), queryInstance);

		if (company != null && company.getCompanyDetails() == null) {

			getCompanyDetails(user, company, queryInstance);
		}

		return new CompanyDetailsQueryRequestProcessor(this, company);
	}

	private void getCompanyDetails(User user, Company company, CompanyDetailsQueryInstance queryInstance) {

		log.info("Getting company details for company " + company + " requested by user " + user);

		CompanyDetailsQuery query = queryInstance.getQuery();
		
		PersonIdentitetsbeteckning personIdentitetsbeteckning = new PersonIdentitetsbeteckning();

		ForetagId foretagId = new ForetagId();
		foretagId.setPersonIdentitetsbeteckning(personIdentitetsbeteckning);

		if (company.getCompanyType() == CompanyType.E) {

			personIdentitetsbeteckning.setPersonnummer(company.getOrganizationNumber());

			if (company.getNameProtectionNumber() != null) {

				foretagId.setNamnskyddslopnummer(company.getNameProtectionNumber());
			}

		} else {

			personIdentitetsbeteckning.setOrganisationsnummer(company.getOrganizationNumber());
		}
		
		RollIntyg rollIntyg = null;
		
		if(query.usesUD0029() || query.usesUD0031() || query.usesUD0033()) {
			
			RollBegaranMetadata rollBegaranMetadata = new RollBegaranMetadata();
			
			rollBegaranMetadata.setMeddelandeId("Message-" + RandomUtils.getRandomInt(1000, 10000));
			rollBegaranMetadata.setTransaktionId("Transaction-" + RandomUtils.getRandomInt(1000, 10000));

			try {
				rollBegaranMetadata.setTidstampel(DatatypeFactory.newInstance().newXMLGregorianCalendar((GregorianCalendar) GregorianCalendar.getInstance()));

			} catch (DatatypeConfigurationException e) {

				throw new RuntimeException(e);
			}

			Ttl ttl = new Ttl();
			ttl.setTidsenhet(Tidsenhet.MILLISEKUNDER);
			ttl.setValue(5000);
			rollBegaranMetadata.setTTL(ttl);

			String citizenIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);
			
			Part part = new Part();
			part.setPartNamn(user.getFirstname() + " " + user.getLastname());

			PartId partId = new PartId();
			partId.setPersonnummer(citizenIdentifier);
			part.setPartId(partId);

			rollBegaranMetadata.setAnvandare(part);
			rollBegaranMetadata.setAnvandningsomrade(Anvandningsomrade.INDIREKT_ATERANVANDNING);
			rollBegaranMetadata.setDatakonsument(getDataKonsument(queryInstance.getQuery().getServiceName()));

			PersonIdentitetsbeteckning roPersonIdentitetsbeteckning = new PersonIdentitetsbeteckning();
			roPersonIdentitetsbeteckning.setPersonnummer(citizenIdentifier);
			
			PersonId personId = new PersonId();
			personId.setPersonIdentitetsbeteckning(roPersonIdentitetsbeteckning);
			
			Foretagsform foretagsform = new Foretagsform();
			foretagsform.setForetagsformKod(company.getCompanyType().toString());
			
			RollBegaranDetaljer rollBegaranDetaljer = new RollBegaranDetaljer();
			rollBegaranDetaljer.setForetagId(foretagId);
			rollBegaranDetaljer.setPersonId(personId);
			rollBegaranDetaljer.setForetagsform(foretagsform);
			
			RollBegaran rollBegaran = new RollBegaran();
			rollBegaran.setSchemaVersion("1.2.0");
			rollBegaran.setRollBegaranMetadata(rollBegaranMetadata);
			rollBegaran.setRollBegaranDetaljer(rollBegaranDetaljer);
			
			SsbtroService service = roServices.get(query.getTargetEnvironment());
			
			try {
				RollSvar rollSvar = service.getForetagsroll(rollBegaran);
				
				if(rollSvar.getRollSvarDetaljer().getForetagRoll() != null) {
					
					rollIntyg = rollSvar.getRollSvarDetaljer().getForetagRoll().getRollIntyg();
				}
				
			} catch (Exception e) {

				company.setCompanyDetails(null);
				company.setError(true);
				log.error("Error getting roles for company " + company + " requested by user " + user, e);
				return;
			}
		}
		
		GrundlaggandeUppgifterBegaranMetadata requestMetadata = new GrundlaggandeUppgifterBegaranMetadata();

		requestMetadata.setMeddelandeId("Message-" + RandomUtils.getRandomInt(1000, 10000));
		requestMetadata.setTransaktionId("Transaction-" + RandomUtils.getRandomInt(1000, 10000));

		try {
			requestMetadata.setTidstampel(DatatypeFactory.newInstance().newXMLGregorianCalendar((GregorianCalendar) GregorianCalendar.getInstance()));

		} catch (DatatypeConfigurationException e) {

			throw new RuntimeException(e);
		}

		Ttl ttl = new Ttl();
		ttl.setTidsenhet(Tidsenhet.MILLISEKUNDER);
		ttl.setValue(5000);
		requestMetadata.setTTL(ttl);

		Part part = new Part();
		part.setPartNamn(user.getFirstname() + " " + user.getLastname());

		PartId partId = new PartId();
		partId.setPersonnummer(CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user));
		part.setPartId(partId);

		requestMetadata.setAnvandare(part);
		requestMetadata.setAnvandningsomrade(Anvandningsomrade.INDIREKT_ATERANVANDNING);
		requestMetadata.setDatakonsument(getDataKonsument(queryInstance.getQuery().getServiceName()));

		GrundlaggandeUppgifterBegaranDetaljer requestDetails = new GrundlaggandeUppgifterBegaranDetaljer();
		requestDetails.setForetagId(foretagId);
		requestDetails.setRollIntyg(rollIntyg);

		//Generic UD codes
		if (!isExcluded(company)) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0001);
		}

		if (query.usesUD0004()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0004);
		}
		
		if (query.usesUD0006()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0006);
		}

		//		if(query.usesUD0025()){
		//
		//			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0025);
		//		}

		if (query.usesUD0029()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0029);
		}
		
		if (query.usesUD0031()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0031);
		}
		
		if (query.usesUD0033()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0033);
		}
		
		if (query.usesUD0046()) {

			requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0046);
		}

		//Type dependent UD codes
		if (company.getCompanyType() == CompanyType.E) {

			if (query.usesUD0023()) {

				requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0023);
			}

		} else {

			if (query.usesUD0003()) {

				requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0003);
			}

			if (query.usesUD0045()) {

				requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0045);
			}

			if (query.usesUD0047()) {

				requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0047);
			}

			if (query.usesUD0048()) {

				requestDetails.getGrundlaggandeUppgiftId().add(GrundlaggandeUppgiftId.UD_0048);
			}
		}

		GrundlaggandeUppgifterBegaran request = new GrundlaggandeUppgifterBegaran();
		request.setSchemaVersion("2.4.0");
		request.setGrundlaggandeUppgifterBegaranMetadata(requestMetadata);
		request.setGrundlaggandeUppgifterBegaranDetaljer(requestDetails);

		SsbtguService service = guServices.get(query.getTargetEnvironment());

		try {
			GrundlaggandeUppgifterSvar response = service.getForetagsuppgifter(request);

			CompanyDetails companyDetails = new CompanyDetails();

			//Generic UD codes
			if (company.isHardCoded()) {

				companyDetails.setCompanyName(user.getFirstname() + " " + user.getLastname());
			
			}else if(company.getCompanyType() == CompanyType.E){
				
				companyDetails.setCompanyName(company.getCompanyName());

			} else {

				Ud0001 ud0001 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0001();

				if (ud0001.getFel() != null && ud0001.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

					throw new IncompleteResponseException(ud0001, ud0001.getFel());
				}

				companyDetails.setCompanyName(ud0001.getForetagNamn());
			}

			if (query.usesUD0004() && !isExcluded(company)) {

				Ud0004 ud0004 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0004();

				if (ud0004.getFel() != null && ud0004.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

					throw new IncompleteResponseException(ud0004, ud0004.getFel());
				}

				companyDetails.setEmail(ud0004.getEPostAdress());
			}
			
			if (query.usesUD0006() && !isExcluded(company)) {

				Ud0006 ud0006 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0006();

				if (ud0006.getFel() != null && ud0006.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ && ud0006.getFel().getTyp() != FelTyp.FORETAG_FINNS_EJ) {

					throw new IncompleteResponseException(ud0006, ud0006.getFel());
				}

				String phone = ud0006.getTelefonnummer();
				
				if (phone != null && StringUtils.isEmpty(phone)) {
					phone = null;
				}
				
				companyDetails.setPhone(phone);
			}

			//			if(query.usesUD0025() && !company.isHardCoded()){
			//
			//				Ud0025 ud0025 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0025();
			//
			//				if(ud0025.getFel() != null && ud0025.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ){
			//
			//					throw new IncompleteResponseException(ud0025,ud0025.getFel());
			//				}
			//
			//				companyDetails.setCompanyType(getCompanyType(ud0025.getForetagsform().getForetagsformKod()));
			//				companyDetails.setCompanyTypeDescription(ud0025.getForetagsform().getForetagsformBeskrivning());
			//			}

			if (query.usesUD0029()) {

				Ud0029 ud0029 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0029();

				if (ud0029.getFel() != null && ud0029.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ && ud0029.getFel().getTyp() != FelTyp.EJ_BEHORIG) {

					throw new IncompleteResponseException(ud0029, ud0029.getFel());
				}

				if(ud0029.getFel() != null && ud0029.getFel().getTyp() == FelTyp.EJ_BEHORIG) {
					
					companyDetails.setfTaxRegistered(CompanyDetailsQuery.ROLE_ACCESS_DENIED);
					
				}else if (ud0029.getSkatteform() != null) {
					
					companyDetails.setfTaxRegistered(ud0029.getSkatteform().toString());
				}
			}
			
			if (query.usesUD0031()) {

				Ud0031 ud0031 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0031();

				if (ud0031.getFel() != null && ud0031.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ && ud0031.getFel().getTyp() != FelTyp.EJ_BEHORIG) {

					throw new IncompleteResponseException(ud0031, ud0031.getFel());
				}

				if(ud0031.getFel() != null && ud0031.getFel().getTyp() == FelTyp.EJ_BEHORIG) {
					
					companyDetails.setVatRegistered(CompanyDetailsQuery.ROLE_ACCESS_DENIED);
					
				}else if (ud0031.getMomsregistrerad() != null) {
					
					companyDetails.setVatRegistered(ud0031.getMomsregistrerad().toString());
				}
			}
			
			if (query.usesUD0033()) {

				Ud0033 ud0033 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0033();

				if (ud0033.getFel() != null && ud0033.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ && ud0033.getFel().getTyp() != FelTyp.EJ_BEHORIG) {

					throw new IncompleteResponseException(ud0033, ud0033.getFel());
				}

				if(ud0033.getFel() != null && ud0033.getFel().getTyp() == FelTyp.EJ_BEHORIG) {
					
					companyDetails.setEmployerRegistration(CompanyDetailsQuery.ROLE_ACCESS_DENIED);
					
				}else if (ud0033.getArbetsgivarregistrerad() != null) {
					
					companyDetails.setEmployerRegistration(ud0033.getArbetsgivarregistrerad().toString());
				}
			}
			
			if (query.usesUD0046() && !isExcluded(company)) {

				Ud0046 ud0046 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0046();

				if (ud0046.getFel() != null && ud0046.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

					throw new IncompleteResponseException(ud0046, ud0046.getFel());
				}

				companyDetails.setBusinessDescription(ud0046.getVerksamhetsbeskrivning());
			}

			//Type dependent UD codes
			if (company.getCompanyType() == CompanyType.E) {

				if (query.usesUD0023()) {

					Ud0023 ud0023 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0023();

					if (ud0023.getFel() != null && ud0023.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ && ud0023.getFel().getTyp() != FelTyp.EJ_BEHORIG & ud0023.getFel().getTyp() != FelTyp.OGILTIG_BEGARAN) {

						throw new IncompleteResponseException(ud0023, ud0023.getFel());
					}

					if (ud0023.getPostadress() != null) {

						StringBuilder addressBuilder = new StringBuilder();

						if (ud0023.getPostadress().getUtdelningsadress1() != null) {

							addressBuilder.append(ud0023.getPostadress().getUtdelningsadress1());
						}

						if (ud0023.getPostadress().getUtdelningsadress2() != null) {

							if (addressBuilder.length() > 0) {

								addressBuilder.append(", ");
							}

							addressBuilder.append(ud0023.getPostadress().getUtdelningsadress2());
						}

						companyDetails.setCivilRegistrationAddress(addressBuilder.toString());
						companyDetails.setCivilRegistrationCOAddress(ud0023.getPostadress().getCOAdress());
						companyDetails.setCivilRegistrationPostalAddress(ud0023.getPostadress().getPostort());
						companyDetails.setCivilRegistrationZip(ud0023.getPostadress().getPostnummer());
					}
				}

			} else {

				if (query.usesUD0003()) {

					Ud0003 ud0003 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0003();

					if (ud0003.getFel() != null && ud0003.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

						throw new IncompleteResponseException(ud0003, ud0003.getFel());
					}

					StringBuilder addressBuilder = new StringBuilder();

					if (ud0003.getPostadress().getUtdelningsadress1() != null) {

						addressBuilder.append(ud0003.getPostadress().getUtdelningsadress1());
					}

					if (ud0003.getPostadress().getUtdelningsadress2() != null) {

						if (addressBuilder.length() > 0) {

							addressBuilder.append(", ");
						}

						addressBuilder.append(ud0003.getPostadress().getUtdelningsadress2());
					}

					companyDetails.setLegalEntityAddress(addressBuilder.toString());
					companyDetails.setLegalEntityCOAddress(ud0003.getPostadress().getCOAdress());
					companyDetails.setLegalEntityPostalAddress(ud0003.getPostadress().getPostort());
					companyDetails.setLegalEntityZip(ud0003.getPostadress().getPostnummer());
				}

				if (query.usesUD0045()) {

					Ud0045 ud0045 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0045();

					if (ud0045.getFel() != null && ud0045.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

						throw new IncompleteResponseException(ud0045, ud0045.getFel());
					}

					companyDetails.setSigning(ud0045.getFirmateckningBeskrivning());
				}

				if (query.usesUD0047()) {

					Ud0047 ud0047 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0047();

					if (ud0047.getFel() != null && ud0047.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

						throw new IncompleteResponseException(ud0047, ud0047.getFel());
					}

					companyDetails.setOfficers(ud0047.getSamladeFunktionarerBeskrivning());
				}

				if (query.usesUD0048()) {

					Ud0048 ud0048 = response.getGrundlaggandeUppgifterSvarDetaljer().getUD0048();

					if (ud0048.getFel() != null && ud0048.getFel().getTyp() != FelTyp.UPPGIFT_FINNS_EJ) {

						throw new IncompleteResponseException(ud0048, ud0048.getFel());
					}

					if (ud0048.getAktier() != null) {

						Aktier aktier = ud0048.getAktier();

						StringBuilder shareDescriptionBuilder = new StringBuilder();

						shareDescriptionBuilder.append(shareCapital + ": " + aktier.getAktiekapital().getValue().intValue() + " " + aktier.getAktiekapital().getRedovisningsvaluta() + "\n");

						shareDescriptionBuilder.append("\n");

						shareDescriptionBuilder.append(shareCount + ": " + aktier.getAktierAntal() + "\n");

						if (!CollectionUtils.isEmpty(aktier.getAktieslag())) {

							shareDescriptionBuilder.append("\n");

							shareDescriptionBuilder.append(distributionOfIssuedShares + "\n");

							boolean first = true;

							for (Aktieslag aktieslag : aktier.getAktieslag()) {

								if (first) {
									first = false;
								} else {
									shareDescriptionBuilder.append("\n");
								}

								shareDescriptionBuilder.append(shareType + ": " + aktieslag.getAktieslagBeteckning() + "\n");

								shareDescriptionBuilder.append(count + ": " + aktieslag.getAktieslagAntal() + "\n");

								shareDescriptionBuilder.append(voteValue + ": " + aktieslag.getAktieslagRostvarde() + "\n");
							}
						}

						companyDetails.setShares(shareDescriptionBuilder.toString());
					}
				}
			}

			company.setError(false);
			company.setCompanyDetails(companyDetails);

		} catch (Exception e) {

			company.setCompanyDetails(null);
			company.setError(true);
			log.error("Error getting details for company " + company + " requested by user " + user, e);
		}
	}

	private boolean isExcluded(Company company) {

		return company.isHardCoded() || (company.getCompanyType() == CompanyType.E && company.getNameProtectionNumber() == null);
	}

	private Company getCompany(String organizationNumber, CompanyDetailsQueryInstance queryInstance) {

		if (organizationNumber != null && !CollectionUtils.isEmpty(queryInstance.getCompanies())) {

			for (Company company : queryInstance.getCompanies()) {

				if (company.getOrganizationNumber().equals(organizationNumber)) {

					return company;
				}
			}
		}

		return null;
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureQuery(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.queryCRUD.update(req, res, user, uriParser);
	}

	@Override
	public boolean deleteQuery(ImmutableQueryDescriptor descriptor, TransactionHandler transactionHandler) throws SQLException {

		CompanyDetailsQuery query = getQuery(descriptor.getQueryID());

		if (query == null) {

			return false;
		}

		this.queryDAO.delete(query, transactionHandler);

		return true;
	}

	@Override
	public boolean deleteQueryInstance(ImmutableQueryInstanceDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		CompanyDetailsQueryInstance queryInstance = this.getQueryInstance(descriptor.getQueryInstanceID());

		if (queryInstance == null) {

			return false;
		}

		this.queryInstanceDAO.delete(queryInstance, transactionHandler);

		return true;
	}

	@Override
	public String getTitlePrefix() {

		return this.moduleDescriptor.getName();
	}

	@Override
	public void copyQuery(MutableQueryDescriptor sourceQueryDescriptor, MutableQueryDescriptor copyQueryDescriptor, TransactionHandler transactionHandler, Map<Integer, ImmutableStatus> statusConversionMap) throws SQLException {

		CompanyDetailsQuery query = getQuery(sourceQueryDescriptor.getQueryID(), transactionHandler);

		query.setQueryID(copyQueryDescriptor.getQueryID());

		this.queryDAO.add(query, transactionHandler, null);
	}

	@Override
	protected void appendPDFData(Document doc, Element showQueryValuesElement, CompanyDetailsQueryInstance queryInstance, AttributeHandler attributeHandler) {

		super.appendPDFData(doc, showQueryValuesElement, queryInstance, attributeHandler);

		if (queryInstance.getQuery().getDescription() != null) {

			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "Description", JTidyUtils.getXHTML(queryInstance.getQuery().getDescription(attributeHandler), systemInterface.getEncoding()));
			XMLUtils.appendNewCDATAElement(doc, showQueryValuesElement, "isHTMLDescription", queryInstance.getQuery().getDescription().contains("<") && queryInstance.getQuery().getDescription().contains(">"));
		}
	}

	@Override
	protected Class<CompanyDetailsQueryInstance> getQueryInstanceClass() {

		return CompanyDetailsQueryInstance.class;
	}

	public String getCompanyDetailsView(Company company, User user) throws TransformerConfigurationException, TransformerException {

		log.info("Transforming company " + company + " for user " + user);

		if (company == null) {

			return "";
		}

		Document doc = XMLUtils.createDomDocument();

		Element documentElement = doc.createElement("Document");
		doc.appendChild(documentElement);

		Element companyDetailsViewElement = XMLUtils.appendNewElement(doc, documentElement, "CompanyDetailsView");

		Element companyElement = company.toXML(doc);
		companyDetailsViewElement.appendChild(companyElement);

		if (company.getCompanyDetails() != null) {

			Element companyDetailsElement = company.getCompanyDetails().toXML(doc);
			companyElement.appendChild(companyDetailsElement);

			XMLUtils.appendNewElement(doc, companyDetailsElement, "companyType", company.getCompanyType());
			XMLUtils.appendNewElement(doc, companyDetailsElement, "companyTypeDescription", company.getCompanyTypeDescription());
			XMLUtils.appendNewElement(doc, companyDetailsElement, "organizationNumber", company.getOrganizationNumber());
			XMLUtils.appendNewElement(doc, companyDetailsElement, "nameProtectionNumber", company.getNameProtectionNumber());
		}

		return transformQuery(doc, queryTransformer, false);
	}

	@Override
	public QueryResponse getFormHTML(CompanyDetailsQueryInstance queryInstance, HttpServletRequest req, User user, User poster, List<ValidationError> validationErrors, boolean enableAjaxPosting, String queryRequestURL, InstanceRequestMetadata requestMetadata, AttributeHandler attributeHandler) throws Throwable {

		if (queryInstance.getCompanies() == null && !requestMetadata.isManager()) {
			
			getCompanies(user, poster, queryInstance, requestMetadata.isManager());
		}

		return super.getFormHTML(queryInstance, req, user, poster, validationErrors, enableAjaxPosting, queryRequestURL, requestMetadata, attributeHandler);
	}

	private void getCompanies(User user, User poster, CompanyDetailsQueryInstance queryInstance, boolean manager) {

		if (queryInstance.getQuery().getTargetEnvironment() == null || queryInstance.getQuery().getServiceName() == null) {

			queryInstance.setError(ErrorType.QUERY_NOT_CONFIGURED);
			return;
		}

		if (user == null) {

			queryInstance.setError(ErrorType.USER_NOT_LOGGED_ID);
			return;
		}

		if (!user.equals(poster) || manager) {

			queryInstance.setError(ErrorType.WRONG_USER);
			queryInstance.setCompanies(Collections.<Company> emptyList());
			return;
		}
		
		String citizienIdentifier = CitizenIdentifierUtils.getUserOrManagerCitizenIdentifier(user);

		if (citizienIdentifier == null) {

			queryInstance.setError(ErrorType.USER_HAS_NO_CITIZEN_IDENTIFIER);
			queryInstance.setCompanies(Collections.<Company> emptyList());
			return;
		}

		log.info("Getting companies for user " + user);

		EngagemangBegaranMetadata requestMetadata = new EngagemangBegaranMetadata();

		requestMetadata.setMeddelandeId("Message-" + RandomUtils.getRandomInt(1000, 10000));
		requestMetadata.setTransaktionId("Transaction-" + RandomUtils.getRandomInt(1000, 10000));
		try {
			requestMetadata.setTidstampel(DatatypeFactory.newInstance().newXMLGregorianCalendar((GregorianCalendar) GregorianCalendar.getInstance()));

		} catch (DatatypeConfigurationException e) {

			throw new RuntimeException(e);
		}

		Ttl ttl = new Ttl();
		ttl.setTidsenhet(Tidsenhet.MILLISEKUNDER);
		ttl.setValue(5000);
		requestMetadata.setTTL(ttl);

		Part part = new Part();
		part.setPartNamn(user.getFirstname() + " " + user.getLastname());

		PartId partId = new PartId();
		partId.setPersonnummer(citizienIdentifier);
		part.setPartId(partId);

		requestMetadata.setAnvandare(part);
		requestMetadata.setAnvandningsomrade(Anvandningsomrade.INDIREKT_ATERANVANDNING);
		requestMetadata.setDatakonsument(getDataKonsument(queryInstance.getQuery().getServiceName()));

		PersonIdentitetsbeteckning personIdentitetsbeteckning = new PersonIdentitetsbeteckning();
		personIdentitetsbeteckning.setPersonnummer(citizienIdentifier);

		PersonId personId = new PersonId();
		personId.setPersonIdentitetsbeteckning(personIdentitetsbeteckning);

		Foretagsformer foretagsformer = new Foretagsformer();

		for (CompanyType companyType : queryInstance.getQuery().getCompanyTypes()) {

			foretagsformer.getForetagsformKod().add(companyType.toString());
		}

		EngagemangBegaranDetaljer requestDetails = new EngagemangBegaranDetaljer();
		requestDetails.setPersonId(personId);
		requestDetails.setForetagsformer(foretagsformer);

		EngagemangBegaran request = new EngagemangBegaran();
		request.setSchemaVersion("1.4.0");
		request.setEngagemangBegaranMetadata(requestMetadata);
		request.setEngagemangBegaranDetaljer(requestDetails);

		SsbtenService service = enServices.get(queryInstance.getQuery().getTargetEnvironment());

		try {
			EngagemangSvar response = service.getForetagsengagemang(request);

			List<Company> companies = new ArrayList<Company>();

			boolean unregisteredCompanyFound = false;

			for (ForetagEngagemang engagemang : response.getEngagemangSvarDetaljer().getForetagEngagemang()) {

				for (Foretag foretag : engagemang.getForetag()) {

					Company company = new Company();

					company.setCompanyType(getCompanyType(foretag.getForetagsform().getForetagsformKod()));
					
					if(queryInstance.getQuery().isShowCompanyTypeDescription()){
						
						company.setCompanyTypeDescription(foretag.getForetagsform().getForetagsformBeskrivning());
					}

					if (company.getCompanyType() == CompanyType.E) {

						company.setOrganizationNumber(foretag.getForetagId().getPersonIdentitetsbeteckning().getPersonnummer());

					} else {

						company.setOrganizationNumber(foretag.getForetagId().getPersonIdentitetsbeteckning().getOrganisationsnummer());
					}

					if (foretag.getForetagNamn() != null) {

						company.setCompanyName(foretag.getForetagNamn());
						
						if(company.getCompanyType() == CompanyType.E){
							
							unregisteredCompanyFound = true;
						}

					} else {
						company.setCompanyName(foretag.getNamnskydd().get(0).getForetagNamn());
						company.setNameProtectionNumber(foretag.getNamnskydd().get(0).getNamnskyddslopnummer());
					}

					companies.add(company);
				}
			}
			
			if(unregisteredCompanyFound){
				
				List<String> registeredOrganizationNumbers = new ArrayList<String>(companies.size());
				
				for(Company company : companies){
					
					if(company.getCompanyType() == CompanyType.E && company.getNameProtectionNumber() != null){
						
						registeredOrganizationNumbers.add(company.getOrganizationNumber());
					}
				}
				
				if(registeredOrganizationNumbers != null){
					
					Iterator<Company> iterator = companies.iterator();
					
					while(iterator.hasNext()){
						
						Company company = iterator.next();
						
						if(company.getCompanyType() == CompanyType.E && company.getNameProtectionNumber() == null && registeredOrganizationNumbers.contains(company.getOrganizationNumber())){
							
							iterator.remove();
						}
					}
				}
			}

			queryInstance.setError(null);
			queryInstance.setCompanies(companies);

		} catch (Exception e) {

			if(e.getMessage() != null && (e.getMessage().contains("Access inte tillten") || e.getMessage().contains("Ett internt fel har intrffat"))) {

				log.warn("Error getting companies for user " + user, e);
				
			}else {
				
				log.error("Error getting companies for user " + user, e);
			}

			queryInstance.setError(ErrorType.SERVICE_ERROR);
			return;
		}
	}

	private CompanyType getCompanyType(String foretagsformKod) {

		return EnumUtils.toEnum(CompanyType.class, foretagsformKod);
	}

	private Datakonsument getDataKonsument(String serviceNamn) {

		PartId partId = new PartId();
		partId.setOrganisationsnummer(this.consumerID);

		Service service = new Service();
		service.setServiceNamn(serviceNamn);

//		JuridiskForm juridiskForm = new JuridiskForm();
//		juridiskForm.setJuridiskFormKod(?);
		
		Datakonsument servicePart = new Datakonsument();
		servicePart.setPartId(partId);
		servicePart.setService(service);
//		servicePart.setJuridiskForm(juridiskForm);
		servicePart.setPartNamn(this.consumerName);

		return servicePart;
	}

	public boolean isConfigured() {

		return configured;
	}
}
