<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/ssbtqueries.js?v=1
	</xsl:variable>

	<xsl:template match="Document">

		<xsl:apply-templates select="ShowQueryValues" />
		<xsl:apply-templates select="ShowQueryForm" />
		<xsl:apply-templates select="CompanyDetailsView" />

	</xsl:template>

	<xsl:template match="ShowQueryValues">

		<div>
			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="CompanyDetailsQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>

			<article>

				<div class="heading-wrapper">
					<h2>
						<xsl:attribute name="class">
							<xsl:if test="CompanyDetailsQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="CompanyDetailsQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name" />
					</h2>

					<xsl:call-template name="createUpdateButton">
						<xsl:with-param name="queryID" select="CompanyDetailsQueryInstance/CompanyDetailsQuery/queryID" />
					</xsl:call-template>

				</div>

				<xsl:if test="CompanyDetailsQueryInstance/CompanyDetailsQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'">
							<xsl:attribute name="class">italic html-description</xsl:attribute>
						</xsl:if>
						<xsl:value-of select="CompanyDetailsQueryInstance/CompanyDetailsQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>
				
				<xsl:if test="CompanyDetailsQueryInstance/CompanyDetailsQuery/showCompaniesAsRadiosAndKeepShowing = 'true'">
					
					<xsl:variable name="selectedAlternativeID" select="CompanyDetailsQueryInstance/organizationNumber" />
					
					<strong>
						<xsl:value-of select="$i18n.SelectCompany" />
					</strong>
					
					<ul class="bigmarginbottom">
						<xsl:for-each select="CompanyDetailsQueryInstance/SavedCompanies/SavedCompany">
							
							<li>
								
								<xsl:choose>
									<xsl:when test="$selectedAlternativeID = organizationNumber">
										
										<strong>
											<xsl:value-of select="companyName" />
											<xsl:text> (</xsl:text>
											<xsl:value-of select="organizationNumber" />
											<xsl:text>)</xsl:text>
										</strong>
										
									</xsl:when>
									<xsl:otherwise>
									
										<xsl:value-of select="companyName" />
										<xsl:text> (</xsl:text>
										<xsl:value-of select="organizationNumber" />
										<xsl:text>)</xsl:text>
										
									</xsl:otherwise>
								</xsl:choose>
								
							</li>
							
						</xsl:for-each>
					</ul>
					
				</xsl:if>

				<xsl:apply-templates select="CompanyDetailsQueryInstance" mode="preview"/>
				
			</article>

		</div>

	</xsl:template>

	<xsl:template match="ShowQueryForm">

		<xsl:variable name="queryID" select="concat('query_', CompanyDetailsQueryInstance/CompanyDetailsQuery/queryID)" />
		<xsl:variable name="isManager" select="RequestMetadata/manager = 'true'" />

		<div id="{$queryID}" data-requesturl="{queryRequestURL}" data-target="companydetails-{$queryID}">

			<xsl:attribute name="class">
				<xsl:text>query</xsl:text>
				<xsl:if test="EnableAjaxPosting"> enableAjaxPosting</xsl:if>
				<xsl:if test="CompanyDetailsQueryInstance/QueryInstanceDescriptor/QueryDescriptor/mergeWithPreviousQuery = 'true'"> mergewithpreviousquery</xsl:if>
			</xsl:attribute>

			<a name="{$queryID}" />

			<xsl:if test="ValidationErrors/validationError or CompanyDetailsQueryInstance/error">
				<div id="{$queryID}-validationerrors" class="validationerrors">
					<div class="info-box error">
						<xsl:apply-templates select="ValidationErrors/validationError" />
						<div class="marker"></div>
					</div>
				</div>
			</xsl:if>

			<article>

				<xsl:if test="ValidationErrors/validationError or CompanyDetailsQueryInstance/error">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>

				<div class="heading-wrapper">

					<h2>
						<xsl:attribute name="class">
							<xsl:if test="CompanyDetailsQueryInstance/QueryInstanceDescriptor/queryState = 'VISIBLE_REQUIRED'">
								<xsl:text>required</xsl:text>
							</xsl:if>
						</xsl:attribute>
						<xsl:value-of select="CompanyDetailsQueryInstance/QueryInstanceDescriptor/QueryDescriptor/name" />
					</h2>

					<xsl:if test="CompanyDetailsQueryInstance/CompanyDetailsQuery/helpText">
						<xsl:apply-templates select="CompanyDetailsQueryInstance/CompanyDetailsQuery/helpText" />
					</xsl:if>

				</div>

				<xsl:if test="CompanyDetailsQueryInstance/CompanyDetailsQuery/description">
					<span class="italic">
						<xsl:if test="/Document/useCKEditorForDescription = 'true'">
							<xsl:attribute name="class">italic html-description</xsl:attribute>
						</xsl:if>
						<xsl:value-of select="CompanyDetailsQueryInstance/CompanyDetailsQuery/description" disable-output-escaping="yes" />
					</span>
				</xsl:if>

				<xsl:choose>
					<xsl:when test="not(CompanyDetailsQueryInstance/error)">

						<fieldset>

							<xsl:variable name="dropDownName">
								<xsl:value-of select="'q'" />
								<xsl:value-of select="CompanyDetailsQueryInstance/CompanyDetailsQuery/queryID" />
								<xsl:value-of select="'_organizationNumber'" />
							</xsl:variable>

							<div class="split">

								<xsl:variable name="selectedAlternativeID" select="CompanyDetailsQueryInstance/organizationNumber" />
								
								<xsl:choose>
									<xsl:when test="CompanyDetailsQueryInstance/CompanyDetailsQuery/showCompaniesAsRadiosAndKeepShowing = 'true' and $isManager">
										
										<strong class="floatleft full">
											<xsl:value-of select="$i18n.SelectedCompany" />
										</strong>
										
										<xsl:choose>
											<xsl:when test="CompanyDetailsQueryInstance/SavedCompanies/SavedCompany">
											
												<xsl:for-each select="CompanyDetailsQueryInstance/SavedCompanies/SavedCompany">
													
													<div class="alternative floatleft clearboth">
														
														<xsl:variable name="radioID">
															<xsl:value-of select="$dropDownName" />
															<xsl:value-of select="'-'" />
															<xsl:value-of select="organizationNumber" />
														</xsl:variable>
														
														<xsl:call-template name="createRadio">
															<xsl:with-param name="id" select="$radioID" />
															<xsl:with-param name="value" select="organizationNumber" />
															<xsl:with-param name="checked" select="$selectedAlternativeID = organizationNumber" />
															<xsl:with-param name="disabled" select="true()"/>
														</xsl:call-template>
														
														<label for="{$radioID}" class="radio disabled">
															<xsl:value-of select="companyName" />
															<xsl:text> (</xsl:text>
															<xsl:value-of select="organizationNumber" />
															<xsl:text>)</xsl:text>
														</label>
														
													</div>
													
												</xsl:for-each>
												
											</xsl:when>
											<xsl:otherwise>
											
												<p class="error">
													<xsl:value-of select="$i18n.NoSelectedCompany" />
												</p>
											
											</xsl:otherwise>
										</xsl:choose>
										
										
									</xsl:when>								
									<xsl:when test="CompanyDetailsQueryInstance/CompanyDetailsQuery/showCompaniesAsRadiosAndKeepShowing = 'true'">
										
										<strong class="floatleft full">
											<xsl:value-of select="$i18n.SelectCompany" />
										</strong>
										
										<xsl:for-each select="CompanyDetailsQueryInstance/Companies/Company">
											
											<div class="alternative floatleft clearboth">
												
												<xsl:variable name="radioID">
													<xsl:value-of select="$dropDownName" />
													<xsl:value-of select="'-'" />
													<xsl:value-of select="organizationNumber" />
												</xsl:variable>
												
												<xsl:call-template name="createRadio">
													<xsl:with-param name="id" select="$radioID" />
													<xsl:with-param name="name" select="$dropDownName" />
													<xsl:with-param name="value" select="organizationNumber" />
													<xsl:with-param name="checked" select="$selectedAlternativeID = organizationNumber" />
												</xsl:call-template>
												
												<label for="{$radioID}" class="radio">
													<xsl:value-of select="companyName" />
													<xsl:text> (</xsl:text>
													<xsl:value-of select="organizationNumber" />
													<xsl:text>)</xsl:text>
												</label>
												
											</div>
											
										</xsl:for-each>
										
									</xsl:when>
									<xsl:when test="$isManager">
										
										<xsl:choose>
											<xsl:when test="CompanyDetailsQueryInstance/organizationNumber">
												
												<select name="{$dropDownName}" style="width: auto; font-weight: normal;" disabled="true">
		
													<option value="{CompanyDetailsQueryInstance/organizationNumber}" selected="true">
		
														<xsl:value-of select="CompanyDetailsQueryInstance/companyName" />
														<xsl:text> (</xsl:text>
														<xsl:value-of select="CompanyDetailsQueryInstance/organizationNumber" />
														<xsl:text>)</xsl:text>
		
													</option>
		
												</select>
												
											</xsl:when>
											<xsl:otherwise>
												
												<p class="error">
													<xsl:value-of select="$i18n.NoSelectedCompany" />
												</p>											
												
											</xsl:otherwise>
										</xsl:choose>
										
										
									</xsl:when>
									<xsl:otherwise>
									
										<select name="{$dropDownName}" style="width: auto; font-weight: normal;" onchange="updateCompanyDetails('{queryRequestURL}','companydetails-{$queryID}', this, '{CompanyDetailsQueryInstance/CompanyDetailsQuery/queryID}')">

											<option value="">
												<xsl:value-of select="$i18n.SelectCompany" />
											</option>

											<xsl:for-each select="CompanyDetailsQueryInstance/Companies/Company">
												<option value="{organizationNumber}">
													<xsl:choose>
														<xsl:when test="requestparameters">
															<xsl:if test="requestparameters/parameter[name=$dropDownName]/value = organizationNumber">
																<xsl:attribute name="selected" />
															</xsl:if>
														</xsl:when>
														<xsl:otherwise>
															<xsl:if test="$selectedAlternativeID = organizationNumber">
																<xsl:attribute name="selected" />
															</xsl:if>
														</xsl:otherwise>
													</xsl:choose>

													<xsl:value-of select="companyName" />
													<xsl:text> (</xsl:text>
													<xsl:value-of select="organizationNumber" />
													<xsl:text>)</xsl:text>

												</option>
											</xsl:for-each>

										</select>
									
									</xsl:otherwise>
								</xsl:choose>

							</div>

						</fieldset>

						<div id="companydetails-{$queryID}">
							<xsl:if test="CompanyDetailsQueryInstance/organizationNumber">

								<h3>
									<xsl:value-of select="$i18n.SelectedCompanyDetails.Title" />
								</h3>
								
								<xsl:if test="$isManager">
																														
									<p class="error">
										<xsl:value-of select="$i18n.CompanyDetailsMightBeOutdated" />
									</p>
									
								</xsl:if>

								<xsl:apply-templates select="CompanyDetailsQueryInstance" />

							</xsl:if>
						</div>
						
						<script type="text/javascript">$(function(){initCompanyDetailsQuery('<xsl:value-of select="CompanyDetailsQueryInstance/CompanyDetailsQuery/queryID" />');});</script>

					</xsl:when>
					<xsl:otherwise>

						<xsl:apply-templates select="CompanyDetailsQueryInstance/error" />

					</xsl:otherwise>
				</xsl:choose>

			</article>

		</div>

	</xsl:template>

	<xsl:template match="CompanyDetailsView">

		<h3>
			<xsl:value-of select="$i18n.SelectedCompanyDetails.Title" />
		</h3>

		<xsl:choose>
			<xsl:when test="Company/error = 'true'">

				<p class="error">
					<xsl:value-of select="$i18n.ErrorGettingCompanyDetails" />
				</p>

			</xsl:when>
			<xsl:otherwise>

				<xsl:apply-templates select="Company/CompanyDetails" />

			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

	<xsl:template match="CompanyDetails | CompanyDetailsQueryInstance">

		<xsl:choose>

			<xsl:when test="companyType = 'E'">

				<fieldset>
					<xsl:if test="companyName">
						<div>
							
							<!-- 
							<xsl:choose>
								<xsl:when test="nameProtectionNumber">
									<label>Registrerat fretagsnamn</label>
									<label>Fretagets namn</label>
								</xsl:when>
								<xsl:otherwise>
									<label>Fretagets namn</label>
								</xsl:otherwise>
							</xsl:choose>
							 -->
							
							<label>Fretagets namn</label>
							
							<input disabled="true" size="107" value="{companyName}" type="text" />
						</div>
					</xsl:if>


					<div class="split">
						<label>Organisationsnummer</label>
						<input disabled="true" size="50" value="{organizationNumber}" type="text" title="Organisationsnummer"/>
					</div>

					<xsl:if test="companyTypeDescription">
						<div class="split odd">
							<label>Fretagsform</label>
							<input disabled="true" size="50" value="{companyTypeDescription}" type="text" title="Fretagsform"/>
						</div>
					</xsl:if>
				</fieldset>

				<xsl:if test="businessDescription">
					<fieldset>
						<label>Verksamhetsbeskrivning</label>
						<pre>
							<xsl:value-of select="businessDescription"/>
						</pre>
					</fieldset>
				</xsl:if>

				<xsl:if test="civilRegistrationAddress">
					<div class="heading-wrapper">
						<h2 style="font-style: normal; margin-top:30px;">Folkbokfringsadress</h2>
					</div>
	
					<xsl:if test="civilRegistrationCOAddress">
					
						<fieldset>
							<div class="split">
								<label>c/o</label>
								<input disabled="true" size="50" value="{civilRegistrationCOAddress}" title="Adress" type="text" />
							</div>
						</fieldset>
					
					</xsl:if>
	
					<fieldset>
						<div class="split">
							<label>Utdelningsadress</label>
							<input disabled="true" size="50" title="Adress" type="text" value="{civilRegistrationAddress}"/>
						</div>
	
						<div class="split">
							<div class="left">
								<label class="floatleft">Postnummer</label>
								<input class="floatleft clearboth" disabled="true" size="15" title="Postnummer" type="text" value="{civilRegistrationZip}" />
							</div>
	
							<div class="right">
								<label class="floatleft">Postort</label>
								<input class="floatleft clearboth" disabled="true" size="28" title="Postort" type="text" value="{civilRegistrationPostalAddress}"/>
							</div>
						</div>
					</fieldset>
				</xsl:if>

			</xsl:when>
			<xsl:otherwise>

				<fieldset>
					<xsl:if test="companyName">
						<div>
							<!-- <label>Registrerat fretagsnamn</label> -->
							<label>Fretagets namn</label>
							<input disabled="true" size="107" value="{companyName}" type="text" />
						</div>
					</xsl:if>


					<div class="split">
						<label>Organisationsnummer</label>
						<input disabled="true" size="50" value="{organizationNumber}" type="text" title="Organisationsnummer"/>
					</div>

					<xsl:if test="companyTypeDescription">
						<div class="split odd">
							<label>Fretagsform</label>
							<input disabled="true" size="50" value="{companyTypeDescription}" type="text" title="Fretagsform"/>
						</div>
					</xsl:if>
				</fieldset>

				<xsl:if test="signing">
					<fieldset>
						<label>Firmateckning</label>
						<pre>
							<xsl:value-of select="signing"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="officers">
					<fieldset>
						<label>Samlade funktionrer</label>
						<pre>
							<xsl:value-of select="officers"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="shares">
					<fieldset>
						<label>Sammanstllning av aktiekapital</label>
						<pre>
							<xsl:value-of select="shares"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="businessDescription">
					<fieldset>
						<label>Verksamhetsbeskrivning</label>
						<pre>
							<xsl:value-of select="businessDescription"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="legalEntityAddress">

					<div class="heading-wrapper">
						<h2 style="font-style: normal; margin-top:30px;">Postadress</h2>
					</div>
				
					<xsl:if test="legalEntityCOAddress">
					
						<fieldset>
							<div class="split">
								<label>c/o</label>
								<input disabled="true" size="50" value="{legalEntityCOAddress}" title="Adress" type="text" />
							</div>
						</fieldset>
					
					</xsl:if>
	
					<fieldset>
						<div class="split">
							<label>Utdelningsadress</label>
							<input disabled="true" size="50" title="Adress" type="text" value="{legalEntityAddress}"/>
						</div>
	
						<div class="split">
							<div class="left">
								<label class="floatleft">Postnummer</label>
								<input class="floatleft clearboth" disabled="true" size="15" title="Postnummer" type="text" value="{legalEntityZip}" />
							</div>
	
							<div class="right">
								<label class="floatleft">Postort</label>
								<input class="floatleft clearboth" disabled="true" size="28" title="Postort" type="text" value="{legalEntityPostalAddress}"/>
							</div>
						</div>
					</fieldset>
				
				</xsl:if>
				
			</xsl:otherwise>

		</xsl:choose>

		<xsl:if test="fTaxRegistered">
			<fieldset>
				<strong class="block">F-skatt eller FA-skatt</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="fTaxRegistered = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="fTaxRegistered"/>
							
						</xsl:otherwise>
					</xsl:choose>
				
				</pre>
			</fieldset>
		</xsl:if>
		
		<xsl:if test="vatRegistered">
			<fieldset>
				<strong class="block">Momsregistrering</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="vatRegistered = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="vatRegistered"/>
							
						</xsl:otherwise>
					</xsl:choose>
				
				</pre>
			</fieldset>
		</xsl:if>
		
		<xsl:if test="employerRegistration">
			<fieldset>
				<strong class="block">Arbetsgivarregistrering</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="employerRegistration = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="employerRegistration"/>
							
						</xsl:otherwise>
					</xsl:choose>
				
				</pre>
			</fieldset>
		</xsl:if>

		<xsl:if test="email or phone">
			<div class="heading-wrapper">
				<h2 style="font-style: normal; margin-top:30px;">Kontaktuppgifter</h2>
			</div>

			<fieldset>
			
				<xsl:if test="email">
					<div class="split">
						<label>Fretagets e-postadress</label>
						<input disabled="true" size="50" title="Fretagets e-postadress" value="{email}" type="text" />
					</div>
				</xsl:if>
				
				<xsl:if test="phone">
					<div class="split">
						<xsl:if test="email">
							<xsl:attribute name="class">split odd</xsl:attribute>
						</xsl:if>
						
						<label>Fretagets telefonnummer</label>
						<input disabled="true" size="50" title="Fretagets telefonnummer" value="{phone}" type="text" />
					</div>
				</xsl:if>
				
			</fieldset>
		</xsl:if>

	</xsl:template>

	<xsl:template match="CompanyDetailsQueryInstance" mode="preview">

		<xsl:choose>

			<xsl:when test="companyType = 'E'">

				<fieldset>
					<xsl:if test="companyName">
						<div class="bigmarginbottom">
							
							<strong class="block">
								<xsl:choose>
									<xsl:when test="nameProtectionNumber">
										<!-- Registrerat fretagsnamn -->
										Fretagets namn
									</xsl:when>
									<xsl:otherwise>
										Fretagets namn
									</xsl:otherwise>
								</xsl:choose>
							</strong>
							
							<xsl:value-of select="companyName"/>
						</div>
					</xsl:if>


					<div class="split">
						<strong class="block">Organisationsnummer</strong>
						<xsl:value-of select="organizationNumber"/>
					</div>

					<xsl:if test="companyTypeDescription">
						<div class="split odd">
							<strong class="block">Fretagsform</strong>
							<xsl:value-of select="companyTypeDescription"/>
						</div>
					</xsl:if>
				</fieldset>

				<xsl:if test="businessDescription">
					<fieldset>
						<strong class="block">Verksamhetsbeskrivning</strong>
						<textarea style="width: 98%" disabled="true">
							<xsl:value-of select="businessDescription"/>
						</textarea>
					</fieldset>
				</xsl:if>

				<xsl:if test="civilRegistrationAddress">
					<div class="heading-wrapper">
						<h2 style="font-style: normal; margin-top:30px;">Folkbokfringsadress</h2>
					</div>
	
					<xsl:if test="civilRegistrationCOAddress">
					
						<fieldset>
							<div class="split">
								<strong class="block">c/o</strong>
								<xsl:value-of select="civilRegistrationCOAddress"/>
							</div>
						</fieldset>
					
					</xsl:if>
	
					<fieldset>
						<div class="split">
							<strong class="block">Postadress</strong>
							<xsl:value-of select="civilRegistrationAddress"/>
						</div>
	
						<div class="split">
							<div class="left">
								<strong class="block">Postnummer</strong>
								<xsl:value-of select="civilRegistrationZip"/>
							</div>
	
							<div class="right">
								<strong class="block">Postort</strong>
								<xsl:value-of select="civilRegistrationPostalAddress"/>
							</div>
						</div>
					</fieldset>
				</xsl:if>

			</xsl:when>
			<xsl:otherwise>

				<fieldset>
					<xsl:if test="companyName">
						<div class="bigmarginbottom">
							<strong class="block">Fretagets namn</strong>
							<xsl:value-of select="companyName"/>
						</div>
					</xsl:if>

					<div class="split">
						<strong class="block">Organisationsnummer</strong>
						<xsl:value-of select="organizationNumber"/>
					</div>

					<xsl:if test="companyTypeDescription">
						<div class="split odd">
							<strong class="block">Fretagsform</strong>
							<xsl:value-of select="companyTypeDescription"/>
						</div>
					</xsl:if>
				</fieldset>

				<xsl:if test="signing">
					<fieldset>
						<strong class="block">Firmateckning</strong>
						<pre>
							<xsl:value-of select="signing"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="officers">
					<fieldset>
						<strong class="block">Samlade funktionrer</strong>
						<pre>
							<xsl:value-of select="officers"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="shares">
					<fieldset>
						<strong class="block">Aktiekapital/Aktier</strong>
						<pre>
							<xsl:value-of select="shares"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="businessDescription">
					<fieldset>
						<strong class="block">Verksamhetsbeskrivning</strong>
						<pre>
							<xsl:value-of select="businessDescription"/>
						</pre>
					</fieldset>
				</xsl:if>
				
				<xsl:if test="legalEntityAddress">

					<div class="heading-wrapper">
						<h2 style="font-style: normal; margin-top:30px;">Postadress</h2>
					</div>
				
					<xsl:if test="legalEntityCOAddress">
					
						<fieldset>
							<div class="split">
								<strong class="block">c/o</strong>
								<xsl:value-of select="legalEntityCOAddress"/>
							</div>
						</fieldset>
					
					</xsl:if>
	
					<fieldset>
						<div class="split">
							<strong class="block">Utdelningsadress</strong>
							<xsl:value-of select="legalEntityAddress"/>
						</div>
	
						<div class="split">
							<div class="left">
								<strong class="block">Postnummer</strong>
								<xsl:value-of select="legalEntityZip"/>
							</div>
	
							<div class="right">
								<strong class="block">Postort</strong>
								<xsl:value-of select="legalEntityPostalAddress"/>
							</div>
						</div>
					</fieldset>
				
				</xsl:if>
				
			</xsl:otherwise>

		</xsl:choose>

		<xsl:if test="fTaxRegistered">
			<fieldset>
				<strong class="block">F-skatt eller FA-skatt</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="fTaxRegistered = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="fTaxRegistered"/>
							
						</xsl:otherwise>
					</xsl:choose>
				</pre>
			</fieldset>
		</xsl:if>
		
		<xsl:if test="vatRegistered">
			<fieldset>
				<strong class="block">Momsregistrering</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="vatRegistered = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="vatRegistered"/>
							
						</xsl:otherwise>
					</xsl:choose>
				</pre>
			</fieldset>
		</xsl:if>
		
		<xsl:if test="employerRegistration">
			<fieldset>
				<strong class="block">Arbetsgivarregistrering</strong>
				<pre>
					<xsl:choose>
						<xsl:when test="employerRegistration = 'ROLE-ACCESS-DENIED'">
						
							<xsl:value-of select="$i18n.RoleAccessDenied"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:value-of select="employerRegistration"/>
						</xsl:otherwise>
					</xsl:choose>
				</pre>
			</fieldset>
		</xsl:if>

		<xsl:if test="email or phone">
			<div class="heading-wrapper">
				<h2 style="font-style: normal; margin-top:30px;">Kontaktuppgifter</h2>
			</div>

			<fieldset>
				
				<xsl:if test="email">
					<div class="split ">
						<strong class="block">Fretagets e-postadress</strong>
						<xsl:value-of select="email"/>
					</div>
				</xsl:if>
				
				<xsl:if test="phone">
					<div class="split ">
						<xsl:if test="email">
							<xsl:attribute name="class">split odd</xsl:attribute>
						</xsl:if>
					
						<strong class="block">Fretagets telefonnummer</strong>
						<xsl:value-of select="phone"/>
					</div>
				</xsl:if>
				
			</fieldset>
		</xsl:if>
		
	</xsl:template>

	<xsl:template match="validationError[messageKey = 'RequiredQuery']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.RequiredQuery" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="error[.='QUERY_NOT_CONFIGURED']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.QUERY_NOT_CONFIGURED" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="error[.= 'USER_NOT_LOGGED_ID']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.USER_NOT_LOGGED_ID" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="error[.= 'USER_HAS_NO_CITIZEN_IDENTIFIER']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.USER_HAS_NO_CITIZEN_IDENTIFIER" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="error[.= 'WRONG_USER']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.WRONG_USER" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="error[.= 'SERVICE_ERROR']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.SERVICE_ERROR" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError[messageKey = 'InvalidCompany']">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.InvalidCompany" />
			</strong>
		</span>

	</xsl:template>

	<xsl:template match="validationError">

		<span>
			<strong data-icon-before="!">
				<xsl:value-of select="$i18n.UnknownValidationError" />
			</strong>
		</span>

	</xsl:template>

</xsl:stylesheet>