package com.nordicpeak.flowengine.evaluators.sendtextfieldsmsevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.beans.Status;
import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;

public class SendTextFieldSMSEvaluatorCRUD extends BaseEvaluatorCRUD<SendTextFieldSMSEvaluator, SendTextFieldSMSEvaluationProviderModule> {

	List<ContentType> unSubmittedContentTypes = CollectionUtils.getList(ContentType.NEW, ContentType.WAITING_FOR_MULTISIGN, ContentType.WAITING_FOR_PAYMENT);
	
	private static final AnnotatedRequestPopulator<SendTextFieldSMSEvaluator> POPULATOR = new AnnotatedRequestPopulator<SendTextFieldSMSEvaluator>(SendTextFieldSMSEvaluator.class);

	public SendTextFieldSMSEvaluatorCRUD(Class<SendTextFieldSMSEvaluator> beanClass, AnnotatedDAOWrapper<SendTextFieldSMSEvaluator, Integer> evaluatorDAO, SendTextFieldSMSEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SendTextFieldSMSEvaluator", "send text field SMS evaluator", null, callback);
	}

	@Override
	protected SendTextFieldSMSEvaluator populateFromUpdateRequest(SendTextFieldSMSEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		SendTextFieldSMSEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> errors = new ArrayList<ValidationError>();

		Integer textFieldID = bean.getTextFieldID();

		boolean found = false;

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		if (query.getFields() != null) {

			for (TextField field : query.getFields()) {

				if (field.getTextFieldID().equals(textFieldID)) {

					found = true;
				}
			}
		}

		if (!found) {

			errors.add(new ValidationError("TextFieldNotFound"));
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
		
		return evaluator;
	}

	@Override
	protected void appendUpdateFormData(SendTextFieldSMSEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<TextField> fields = query.getFields();

		if (fields != null) {

			Element fieldsElement = doc.createElement("TextFields");
			updateTypeElement.appendChild(fieldsElement);

			for (TextField field : fields) {

				Element fieldElement = doc.createElement("TextField");
				fieldsElement.appendChild(fieldElement);

				XMLUtils.appendNewElement(doc, fieldElement, "fieldID", field.getTextFieldID());
				XMLUtils.appendNewCDATAElement(doc, fieldElement, "name", field.getLabel());
			}
		}
		
		List<Status> submittedStatuses = new ArrayList<>();

		//Remove statuses with content types "before" submitted
		for(Status status : callback.getFlowAdminModule().getCachedFlow(bean.getEvaluatorDescriptor().getQueryDescriptor().getStep().getFlow().getFlowID()).getStatuses()) {
			
			if(!unSubmittedContentTypes.contains(status.getContentType())) {
				
				submittedStatuses.add(status);
			}
		}
		
		XMLUtils.append(doc, updateTypeElement, "Statuses", submittedStatuses);

		XMLUtils.appendNewElement(doc, updateTypeElement, "AvailableTags", callback.getAvailableTags());
	}

}
