package com.nordicpeak.flowengine.evaluators.setattributefromalternativeid;

import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

public class SetAttributeFromAlternativeIDEvaluationProviderModule extends BaseEvaluationProviderModule<SetAttributeFromAlternativeIDEvaluator> {
	
	protected SetAttributeFromAlternativeIDEvaluatorCRUD evaluatorCRUD;
	
	protected AnnotatedDAO<SetAttributeFromAlternativeIDEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<SetAttributeFromAlternativeIDEvaluator, Integer> evaluatorIDParamFactory;
	
	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetAttributeFromAlternativeIDEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetAttributeFromAlternativeIDEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new SetAttributeFromAlternativeIDEvaluatorCRUD(SetAttributeFromAlternativeIDEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetAttributeFromAlternativeIDEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (evaluator.getAttributeName() == null) {
			return null;
		}
		
		if (!(queryInstance instanceof FixedAlternativesQueryInstance)) {
			
			log.warn("Query instance " + queryInstance + " is not of the type " + FixedAlternativesQueryInstance.class.getName());
			
			return null;
		}
		
		FixedAlternativesQueryInstance fixedAlternativesQueryInstance = ((FixedAlternativesQueryInstance) queryInstance);
		
		List<? extends ImmutableAlternative> selectedAlternatives = fixedAlternativesQueryInstance.getAlternatives();
		
		if (selectedAlternatives == null) {
			
			attributeHandler.removeAttribute(evaluator.getAttributeName());
			
		} else {
			
			StringBuilder builder = new StringBuilder();
			
			for (ImmutableAlternative alternative : selectedAlternatives) {
				
				if (builder.length() > 0) {
					builder.append(",");
				}
				
				builder.append(alternative.getAlternativeID());
			}
			
			attributeHandler.setAttribute(evaluator.getAttributeName(), builder.toString());
		}
		
		return null;
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		SetAttributeFromAlternativeIDEvaluator evaluator = new SetAttributeFromAlternativeIDEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return FixedAlternativesQuery.class.isAssignableFrom(queryClass);
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		SetAttributeFromAlternativeIDEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetAttributeFromAlternativeIDEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	protected Class<SetAttributeFromAlternativeIDEvaluator> getEvaluatorClass() {
		
		return SetAttributeFromAlternativeIDEvaluator.class;
	}
	
	protected AnnotatedDAO<SetAttributeFromAlternativeIDEvaluator> getEvaluatorDAO() {
		
		return evaluatorDAO;
	}
	
	protected QueryParameterFactory<SetAttributeFromAlternativeIDEvaluator, Integer> getEvaluatorIDParameterFactory() {
		
		return evaluatorIDParamFactory;
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		SetAttributeFromAlternativeIDEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetAttributeFromAlternativeIDEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<SetAttributeFromAlternativeIDEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	public QueryHandler getQueryHandler() {
		
		return this.queryHandler;
	}
	
	@Override
	public SetAttributeFromAlternativeIDEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		SetAttributeFromAlternativeIDEvaluator evaluator = getEvaluatorClass().newInstance();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
