package com.nordicpeak.flowengine.evaluators.setcontactdetailsevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_contact_details_evaluators")
@XMLElement
public class SetContactDetailsEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer emailTextFieldID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer phoneTextFieldID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private Integer mobilePhoneTextFieldID;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean overwriteExistingContactDetails;

	public Integer getEmailTextFieldID() {

		return emailTextFieldID;
	}

	public void setEmailTextFieldID(Integer emailTextFieldID) {

		this.emailTextFieldID = emailTextFieldID;
	}

	public Integer getPhoneTextFieldID() {

		return phoneTextFieldID;
	}

	public void setPhoneTextFieldID(Integer phoneTextFieldID) {

		this.phoneTextFieldID = phoneTextFieldID;
	}

	public Integer getMobilePhoneTextFieldID() {

		return mobilePhoneTextFieldID;
	}

	public void setMobilePhoneTextFieldID(Integer mobilePhoneTextFieldID) {

		this.mobilePhoneTextFieldID = mobilePhoneTextFieldID;
	}

	public boolean isOverwriteExistingContactDetails() {

		return overwriteExistingContactDetails;
	}

	public void setOverwriteExistingContactDetails(boolean overwriteExistingContactDetails) {

		this.overwriteExistingContactDetails = overwriteExistingContactDetails;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		emailTextFieldID = XMLValidationUtils.validateParameter("emailTextFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		phoneTextFieldID = XMLValidationUtils.validateParameter("phoneTextFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);
		mobilePhoneTextFieldID = XMLValidationUtils.validateParameter("mobilePhoneTextFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		overwriteExistingContactDetails = xmlParser.getPrimitiveBoolean("overwriteExistingContactDetails");

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

}
