package com.nordicpeak.flowengine.evaluators.setpudfromcitizenaddressevaluator;

import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressResponseType;
import se.lantmateriet.namespace.distribution.produkter.belagenhetsadress.v4.BelagenhetsadressType;
import se.lantmateriet.namespace.distribution.produkter.fastighet.v2.RegisterenhetsreferensType;
import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.ModuleSetting;
import se.unlogic.hierarchy.core.annotations.TextFieldSettingDescriptor;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.AttributeHandler;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.datatypes.SimpleEntry;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.enums.ModificationAction;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.IconMapQuery;
import com.nordicpeak.flowengine.queries.iconmapquery.beans.IconMapQueryInstance;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQueryInstance;
import com.nordicpeak.lmsearch.LMSearchProvider;
import com.nordicpeak.lmsearch.beans.ParsedAddress;
import com.nordicpeak.lmsearch.utils.AddressUtils;

public class SetPUDFromCitizenAddressEvaluationProviderModule extends BaseEvaluationProviderModule<SetPUDFromCitizenAddressEvaluator> {

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Connection timeout", description = "The maximum time in seconds to wait for a connection")
	private Integer connectionTimeout = 10;

	@ModuleSetting
	@TextFieldSettingDescriptor(name = "Read timeout", description = "The maximum time in seconds to wait for a response")
	private Integer readTimeout = 10;

	@InstanceManagerDependency(required = true)
	private LMSearchProvider lmSearchProvider;
	
	protected SetPUDFromCitizenAddressEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetPUDFromCitizenAddressEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetPUDFromCitizenAddressEvaluator, Integer> evaluatorIDParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetPUDFromCitizenAddressEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetPUDFromCitizenAddressEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetPUDFromCitizenAddressEvaluatorCRUD(SetPUDFromCitizenAddressEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetPUDFromCitizenAddressEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		IconMapQueryInstance iconMapQueryInstance = ((IconMapQueryInstance) queryInstance);

		if (evaluator.isInitialized()) {

			iconMapQueryInstance.setInitializeErrorMessage(null);

			return null;
		}

		evaluator.setInitialized(true);

		if (!(queryInstance instanceof IconMapQueryInstance)) {

			log.warn("Query instance " + queryInstance + " is not of the type " + TextFieldQueryInstance.class.getName());

			return null;
		}

		String oldValue = iconMapQueryInstance.getPudObjectID();

		Entry<String, String> pudEntry = getPUDFromCitizenAddress(user, evaluator, callback);

		if (pudEntry != null) {

			iconMapQueryInstance.setPropertyUnitDesignation(pudEntry.getValue());
			iconMapQueryInstance.setPudObjectID(pudEntry.getKey());

			if (!ObjectUtils.compare(oldValue, pudEntry.getKey())) {

				return new EvaluationResponse(Collections.singletonList(new QueryModification(queryInstance, ModificationAction.RELOAD, attributeHandler)));
			}
			
		} else if (evaluator.getNoPUDFoundMessage() != null) {

			iconMapQueryInstance.setInitializeErrorMessage(evaluator.getNoPUDFoundMessage());

		}

		return null;
	}

	private Entry<String, String> getPUDFromCitizenAddress(User user, SetPUDFromCitizenAddressEvaluator evaluator, EvaluationCallback callback) {

		if(user == null) {
			
			return null;
		}
		
		AttributeHandler attributeHandler = user.getAttributeHandler();

		if (attributeHandler != null) {

			String address = attributeHandler.getString("address");

			if (!StringUtils.isEmpty(address)) {

				ParsedAddress parsedAddress = AddressUtils.getAddressNumber(address);
				
				String searchString = parsedAddress.getStreet();
				
				if(!StringUtils.isEmpty(parsedAddress.getNumber())){
					
					searchString += " " + parsedAddress.getNumber();
				}
				
				BelagenhetsadressResponseType belagenhetsadress = lmSearchProvider.getLmConnection().searchAddressInMunicipality(lmSearchProvider.getMunicipality(), searchString);

				if (belagenhetsadress == null || CollectionUtils.isEmpty(belagenhetsadress.getBelagenhetsadressMember())) {

					return null;
				}

				BelagenhetsadressType addressType = belagenhetsadress.getBelagenhetsadressMember().get(0).getBelagenhetsadress();
				
				List<Double> coordinates = addressType.getAdressplatsattribut().getAdressplatspunkt().getPoint().getPos().getValue();

				int x = coordinates.get(1).intValue();
				int y = coordinates.get(0).intValue();

				RegisterenhetsreferensType registerEnhet = lmSearchProvider.getLmConnection().searchRegisterUnit(x, y);
				
				if (registerEnhet != null) {
					
					return new SimpleEntry<String, String>(registerEnhet.getObjektidentitet(), registerEnhet.getBeteckning());
				}

			}

		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetPUDFromCitizenAddressEvaluator evaluator = new SetPUDFromCitizenAddressEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return IconMapQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetPUDFromCitizenAddressEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetPUDFromCitizenAddressEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	protected Class<SetPUDFromCitizenAddressEvaluator> getEvaluatorClass() {

		return SetPUDFromCitizenAddressEvaluator.class;
	}

	protected AnnotatedDAO<SetPUDFromCitizenAddressEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetPUDFromCitizenAddressEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetPUDFromCitizenAddressEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetPUDFromCitizenAddressEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<SetPUDFromCitizenAddressEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}

	@Override
	public SetPUDFromCitizenAddressEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetPUDFromCitizenAddressEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}
}
