package com.nordicpeak.flowengine.evaluators.settextfieldvalueattribute;

import java.util.ArrayList;
import java.util.List;

import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.EvaluationEventListener;
import com.nordicpeak.flowengine.interfaces.EvaluationHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

@Table(name = "set_textfield_value_attribute_evaluators")
@XMLElement
public class SetTextFieldValueAttributeEvaluator extends BaseEvaluator implements EvaluationEventListener {

	private static final long serialVersionUID = 4660678625236716384L;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer textFieldID;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement(cdata = true)
	private String attributeName;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String noAttributeValueFoundMessage;

	private boolean initialized;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		textFieldID = XMLValidationUtils.validateParameter("textFieldID", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		noAttributeValueFoundMessage = XMLValidationUtils.validateParameter("noAttributeValueFoundMessage", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getTextFieldID() {

		return textFieldID;
	}

	public void setTextFieldID(Integer textFieldID) {

		this.textFieldID = textFieldID;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public String getNoAttributeValueFoundMessage() {

		return noAttributeValueFoundMessage;
	}

	public void setNoAttributeValueFoundMessage(String noAttributeValueFoundMessage) {

		this.noAttributeValueFoundMessage = noAttributeValueFoundMessage;
	}

	@Override
	public boolean supportsEvent(Object event) {

		return false;
	}

	@Override
	public List<QueryModification> processEvent(Object event, QueryInstance queryInstance, User user, User poster, EvaluationCallback callback, EvaluationHandler evaluationHandler) {

		return null;
	}

	public boolean isInitialized() {

		return initialized;
	}

	public void setInitialized(boolean initialized) {

		this.initialized = initialized;
	}
}
