<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="globalscripts">
		/jquery/jquery.js?v=1
	</xsl:variable>	

	<xsl:variable name="scripts">
		/js/tableattributesummary.js
	</xsl:variable>

	<xsl:template match="Document">

		<div class="contentitem errands-wrapper">

			<xsl:apply-templates select="UpdateTableAttributeSummaryEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateTableAttributeSummaryEvaluator">
	
		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="TableAttributeSummaryEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="TableAttributeSummaryEvaluator" />
			</xsl:call-template>
			
			<div class="floatleft full bigmarginbottom">

				<label for="typeColumnID" class="marginbottom">
					<xsl:value-of select="$i18n.SelectColumn.Type" />
				</label>
				
				<div class="full">
				
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'typeColumnID'" />
						<xsl:with-param name="name" select="'typeColumnID'" />
						<xsl:with-param name="selectedValue" select="TableAttributeSummaryEvaluator/typeColumnID" />
						<xsl:with-param name="element" select="Columns/Column[columnType = 'DROPDOWN']" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'columnID'" />
						<xsl:with-param name="addEmptyOption" select="$i18n.SelectColumn" />
					</xsl:call-template>
					
				</div>
						
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<label for="valueColumnID" class="marginbottom">
					<xsl:value-of select="$i18n.SelectColumn.Value" />
				</label>
				
				<div class="full">
					
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'valueColumnID'" />
						<xsl:with-param name="name" select="'valueColumnID'" />
						<xsl:with-param name="selectedValue" select="TableAttributeSummaryEvaluator/valueColumnID" />
						<xsl:with-param name="element" select="Columns/Column[columnType = 'STRING']" />
						<xsl:with-param name="labelElementName" select="'name'" />
						<xsl:with-param name="valueElementName" select="'columnID'" />
						<xsl:with-param name="addEmptyOption" select="$i18n.SelectColumn" />
					</xsl:call-template>
					
				</div>
							
			</div>
			
			<div class="floatleft full bigmarginbottom bigmargintop">

				<xsl:apply-templates select="Columns/Column[DropDownAlternatives/DropDownAlternative]" />
			
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.submit}" />
			</div>

		</form>
			
	</xsl:template>
	
	<xsl:template match="Column">
	
	 	<table id="column-{columnID}-alternatives" class="column-alternatives oep-table hidden">

			<thead>
				<tr>
				
					<th><xsl:value-of select="$i18n.Alternatives" /></th>
					<th><xsl:value-of select="$i18n.Attributes" /></th>
						
				</tr>
			</thead>

			<tbody>
				<xsl:apply-templates select="DropDownAlternatives/DropDownAlternative" />	 	
			</tbody>
	 	
	 	</table>
	
	</xsl:template>
	
	<xsl:template match="DropDownAlternative">
	
		<tr>
		
	 		<td><span class="bigmarginright"><xsl:value-of select="name" /></span></td>
	 		
	 		<td>
	 		
	 			<xsl:call-template name="createTextField">
	 				<xsl:with-param name="name" select="concat('alternative-', alternativeID)" />
	 				<xsl:with-param name="value" select="value" />
	 				<xsl:with-param name="disabled" select="true" />
	 			</xsl:call-template>
	 			
	 		</td>		

		</tr>
	
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'typeColumnID'">
				<xsl:value-of select="$i18n.SelectColumn.Type" />
			</xsl:when>
			<xsl:when test="$fieldName = 'valueColumnID'">
				<xsl:value-of select="$i18n.SelectColumn.Value" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>