package com.nordicpeak.flowengine.evaluators.tableattributesummaryevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.basetablequery.BaseTableColumn;
import com.nordicpeak.flowengine.queries.basetablequery.BaseTableQuery;
import com.nordicpeak.flowengine.queries.dynamictablequery.DropDownAlternative;

public class TableAttributeSummaryEvaluatorCRUD extends BaseEvaluatorCRUD<TableAttributeSummaryEvaluator, TableAttributeSummaryEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<TableAttributeSummaryEvaluator> POPULATOR = new AnnotatedRequestPopulator<TableAttributeSummaryEvaluator>(TableAttributeSummaryEvaluator.class);

	public TableAttributeSummaryEvaluatorCRUD(Class<TableAttributeSummaryEvaluator> beanClass, AnnotatedDAOWrapper<TableAttributeSummaryEvaluator, Integer> evaluatorDAO, TableAttributeSummaryEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "TableAttributeSummaryEvaluator", "table attribute summary evaluator", null, callback);
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	@Override
	protected void appendUpdateFormData(TableAttributeSummaryEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);

		BaseTableQuery query = (BaseTableQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<BaseTableColumn> columns = query.getColumns();

		if (columns != null) {

			Element columnsElement = XMLUtils.appendNewElement(doc, updateTypeElement, "Columns");

			for (BaseTableColumn column : columns) {

				Element columnElement = XMLUtils.appendNewElement(doc, columnsElement, "Column");

				XMLUtils.appendNewElement(doc, columnElement, "columnID", column.getColumnID());
				XMLUtils.appendNewElement(doc, columnElement, "columnType", column.getColumnType());
				XMLUtils.appendNewCDATAElement(doc, columnElement, "name", column.getLabel());

				Element dropdownAlternativesElement = XMLUtils.appendNewElement(doc, columnElement, "DropDownAlternatives");

				List<DropDownAlternative> alternatives = column.getDropDownAlternatives();

				if (!CollectionUtils.isEmpty(alternatives)) {

					for (DropDownAlternative alternative : alternatives) {

						Element alternativeElement = XMLUtils.append(doc, dropdownAlternativesElement, alternative);

						String attribute = getAttributeByAlternativeID(evaluator.getAlternativeAttributes(), alternative.getAlternativeID());

						if (attribute != null) {

							XMLUtils.appendNewElement(doc, alternativeElement, "value", attribute);
						}
					}
				}
			}
		}
	}

	private String getAttributeByAlternativeID(List<AlternativeAttribute> attributes, Integer alternativeID) {

		if (!CollectionUtils.isEmpty(attributes) && alternativeID != null) {

			for (AlternativeAttribute attribute : attributes) {

				if (alternativeID.equals(attribute.getAlternativeID())) {

					return attribute.getAttribute();
				}
			}
		}

		return null;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	protected TableAttributeSummaryEvaluator populateFromUpdateRequest(TableAttributeSummaryEvaluator evaluator, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		evaluator = super.populateFromUpdateRequest(evaluator, req, user, uriParser);

		List<ValidationError> validationErrors = new ArrayList<ValidationError>();
		
		BaseTableQuery query = (BaseTableQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<BaseTableColumn> columns = query.getColumns();

		List<AlternativeAttribute> attributes = new ArrayList<AlternativeAttribute>();

		if (!CollectionUtils.isEmpty(columns)) {

			for (BaseTableColumn column : columns) {

				List<DropDownAlternative> alternatives = column.getDropDownAlternatives();

				if (!CollectionUtils.isEmpty(alternatives)) {

					for (DropDownAlternative alternative : alternatives) {

						String attributeValue = req.getParameter("alternative-" + alternative.getAlternativeID());

						if (!StringUtils.isEmpty(attributeValue)) {

							if(attributeValue.length() > 255) {
								
								validationErrors.add(new ValidationError(alternative.getName(), ValidationErrorType.TooLong));
							}
							
							AlternativeAttribute attribute = new AlternativeAttribute();

							attribute.setAlternativeID(alternative.getAlternativeID());
							attribute.setEvaluator(evaluator);
							attribute.setAttribute(attributeValue);

							attributes.add(attribute);
						}
					}
				}
			}
		}

		evaluator.setAlternativeAttributes(attributes);

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}
		
		return evaluator;
	}

}
