<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xmlns:exsl="http://exslt.org/common">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:variable name="links">
		/css/buttonspinner.css
	</xsl:variable>

	<xsl:variable name="scripts">
		/js/treservadecision.js?v=1
	</xsl:variable>

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl"/>

	<xsl:variable name="imgPath"><xsl:value-of select="/Document/requestinfo/contextpath" />/static/f/<xsl:value-of select="/Document/module/sectionID" />/<xsl:value-of select="/Document/module/moduleID" />/pics</xsl:variable>

	<xsl:template match="Document">
		
		<script>
			i18n = {
				LoadingDecisions: "<xsl:value-of select="$i18n.LoadingDecisions" />",
				ErrorGettingDecisions: "<xsl:value-of select="$i18n.ErrorGettingDecisions" />",
			};
		</script>
	
		<xsl:apply-templates select="TabContentContainer" />
		
		<xsl:apply-templates select="TabContent" />
			
	</xsl:template>
	
	<xsl:template match="TabContentContainer">
		
		<div id="treserva-decision" class="tabs-content errands-wrapper clearfix">
			
			<div class="spinner-wrapper">
				<p id="treserva-decision-info"></p>
			</div>
			
			<div id="treserva-decision-content" data-url="{ExtensionURL}/tabcontent"></div>
			
		</div>
		
	</xsl:template>
	
	<xsl:template match="TabContent">
	
		<xsl:choose>
			<xsl:when test="Decisions">
			
				<div class="bigmarginbottom">
		
					<xsl:call-template name="createFilterMonthDropdown" />
				
				</div>
				
				<form id="treserva-decision-form" class="hidden" method="post" action="{ExtensionURL}/senddecisions">
				
					<table id="treserva-decision-table" class="full coloredtable sortabletable oep-table bigmarginbottom" cellspacing="0">
						<thead>
							<tr>
								<th style="width:20px;"></th>
								<th><xsl:value-of select="$i18n.Decision.Number" /></th>
								<th><xsl:value-of select="$i18n.Decision.Name" /></th>
								<th><xsl:value-of select="$i18n.Decision.Date" /></th>
								<th><xsl:value-of select="$i18n.Decision.SentDate" /></th>
								<th style="width:20px;"></th>
							</tr>
						</thead>
						<tbody>
							<xsl:apply-templates select="Decisions/Decision" />
						</tbody>
					</table>
					
					<div class="floatright clearboth bigmargintop">
						<input id="treserva-decision-form-preview-button" class="btn btn-green" type="submit" value="{$i18n.Decision.Preview}" />
					</div>
				
				</form>
				
				<xsl:call-template name="PreviewDecisionsModal" />
				
			</xsl:when>
			<xsl:otherwise>
			
				<p>
					<xsl:value-of select="$i18n.NoDecisions" />
				</p>
			
			</xsl:otherwise>
		</xsl:choose>
	
	
	</xsl:template>
	
	<xsl:template match="Decision">
		
		<tr data-intended-month="{intendedMonth}" data-name="{name}" data-number="{number}" data-message="{message}">
			<td>
				<input style="display:block;" type="checkbox" name="decisionID" value="{decisionID}">
					<xsl:if test="sent = 'true'">
						<xsl:attribute name="disabled">true</xsl:attribute>
					</xsl:if>
				</input>
				<input type="hidden" name="decision-message-{decisionID}" value="{message}" />
				<input type="hidden" name="decision-number-{decisionID}" value="{number}" />
			</td>
			<td>
				<xsl:value-of select="number" />
			</td>
			<td>
				<xsl:value-of select="name" />
			</td>
			<td>
				<xsl:value-of select="date" />
			</td>
			<td>
				<xsl:value-of select="sentDate" />
			</td>
			<td>
				<xsl:if test="Document">
					<a href="{../../ExtensionURL}/decisiondocument/{number}" title="">
						<img alt="" src="{$imgPath}/pdf.png" />
					</a>
				</xsl:if>
			</td>
		</tr>
			
	</xsl:template>
	
	<xsl:template name="createFilterMonthDropdown">
						
		<label for="treserva-decision-month-filter">
			<xsl:value-of select="$i18n.Month" />
		</label>
		
		<select id="treserva-decision-month-filter">
			
			<option value="">
				<xsl:value-of select="$i18n.Month.Select" />
			</option>
						
			<xsl:for-each select="FilterMonths/FilterMonth">
				
				<option value="{value}">
					<xsl:call-template name="printMonth">
						<xsl:with-param name="index" select="index" />
					</xsl:call-template>
				</option>
				
			</xsl:for-each>
	
		</select>
	
	</xsl:template>

	<xsl:template name="printMonth">
		<xsl:param name="index" />
		
		<xsl:choose>
			<xsl:when test="index = 1"><xsl:value-of select="$i18n.Month.January" /></xsl:when>
			<xsl:when test="index = 2"><xsl:value-of select="$i18n.Month.February" /></xsl:when>
			<xsl:when test="index = 3"><xsl:value-of select="$i18n.Month.March" /></xsl:when>
			<xsl:when test="index = 4"><xsl:value-of select="$i18n.Month.April" /></xsl:when>
			<xsl:when test="index = 5"><xsl:value-of select="$i18n.Month.May" /></xsl:when>
			<xsl:when test="index = 6"><xsl:value-of select="$i18n.Month.June" /></xsl:when>
			<xsl:when test="index = 7"><xsl:value-of select="$i18n.Month.July" /></xsl:when>
			<xsl:when test="index = 8"><xsl:value-of select="$i18n.Month.August" /></xsl:when>
			<xsl:when test="index = 9"><xsl:value-of select="$i18n.Month.September" /></xsl:when>
			<xsl:when test="index = 10"><xsl:value-of select="$i18n.Month.October" /></xsl:when>
			<xsl:when test="index = 11"><xsl:value-of select="$i18n.Month.November" /></xsl:when>
			<xsl:when test="index = 12"><xsl:value-of select="$i18n.Month.December" /></xsl:when>
			<xsl:otherwise><xsl:value-of select="i18n.Month.Unknown" /></xsl:otherwise>
		</xsl:choose>
	
	</xsl:template>
	
	<xsl:template name="PreviewDecisionsModal">
	
		<div class="hidden">
			
			<div id="treserva-preview-decisions-modal" class="contentitem">
		
				<div class="modal-content">
				
					<div class="modal-header bigmarginbottom">
						
						<h1>
							<xsl:value-of select="$i18n.Decision.Preview" />
						</h1>
						
					</div>
					
					<div class="modal-body">
					
						<div id="treserva-preview-decision-template" class="hidden bigmargintop bigmarginbottom">
						
							<h2 class="marginbottom">
								<span id="treserva-preview-decision-template-name"></span>
								<span><xsl:text> (</xsl:text></span>
								<span id="treserva-preview-decision-template-number"></span>
								<span><xsl:text>)</xsl:text></span>
							</h2>
						
							<textarea class="bigmarginbottom" id="treserva-preview-decision-template-message" disabled="true" rows="20"></textarea>
							
						</div>
					
					</div>
					
					<div class="modal-buttons bigmargintop">

						<input id="treserva-preview-decisions-modal-submit-button" type="button" class="btn btn-green bigmargintop bigmarginleft floatright" value="{$i18n.Decision.Send}"/>
						
						<input type="button" class="btn btn-light bigmargintop floatright" value="{$i18n.Decision.Cancel}" onclick="$.featherlight.close();"/>
						
					</div>
				
				</div>
			
			</div>
		
		</div>
	
	</xsl:template>
	
</xsl:stylesheet>