package com.nordicpeak.flowengine.integration.treservaddecision.beans;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;

import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import flexjson.JSON;

@XMLElement
public class Decision extends GeneratedElementable {

	private static final DateTimeFormatter DATE_TIME_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
	private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyy-MM-dd");

	@JSON(name = "beslutId")
	@XMLElement
	private String decisionID;

	@JSON(name = "beslutNr")
	@XMLElement
	private String number;

	@JSON(name = "beslutName")
	@XMLElement
	private String name;

	@JSON(name = "name")
	@XMLElement
	private String manager;

	@JSON(name = "beslutsdatum")
	@XMLElement
	private String date;

	@XMLElement
	private String sentDate;

	@JSON(name = "avserManad")
	@XMLElement
	private String intendedMonth;

	@JSON(name = "beslutsmotivering")
	@XMLElement
	private String reason;

	@XMLElement
	private String message;

	@XMLElement
	private DecisionDocument document;

	@XMLElement
	private boolean sent;

	@Override
	public String toString() {

		return "(decisionID=" + decisionID + ", number=" + number + ", name=" + name + ")";
	}

	public String getDecisionID() {

		return decisionID;
	}

	public void setDecisionID(String decisionID) {

		this.decisionID = decisionID;
	}

	public String getNumber() {

		return number;
	}

	public void setNumber(String number) {

		this.number = number;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getManager() {

		return manager;
	}

	public void setManager(String manager) {

		this.manager = manager;
	}

	public String getDate() {

		return date;
	}

	public void setDate(String date) {

		this.date = date;
	}

	public void formatDate() {

		if (date == null) {

			return;
		}

		try {

			date = LocalDateTime.parse(date, DATE_TIME_FORMATTER).format(DATE_FORMATTER);

		} catch (DateTimeParseException e) {}
	}

	public String getSentDate() {

		return sentDate;
	}

	public void setSentDate(String sentDate) {

		this.sentDate = sentDate;
	}

	public void formatAndSetSentDate(LocalDateTime sentDate) {

		this.sentDate = sentDate.format(DATE_TIME_FORMATTER);
	}

	public String getIntendedMonth() {

		return intendedMonth;
	}

	public LocalDateTime getParsedIntendedMonth() {

		try {

			return LocalDateTime.parse(intendedMonth, DATE_TIME_FORMATTER);

		} catch (DateTimeParseException e) {

			return null;
		}
	}

	public void setIntendedMonth(String intendedMonth) {

		this.intendedMonth = intendedMonth;
	}

	public String getReason() {

		return reason;
	}

	public void setReason(String reason) {

		this.reason = reason;
	}

	public String getMessage() {

		return message;
	}

	public void setMessage(String message) {

		this.message = message;
	}

	public void generateMessage(String messageTemplate) {

		if (messageTemplate == null) {

			return;
		}

		String messageReason = reason != null ? reason.replace("\\r\\n", "\n") : "";
		String messageDate = date != null ? date : "";

		setMessage(messageTemplate.replace("${number}", number).replace("${reason}", messageReason).replace("${date}", messageDate).replace("${manager}", manager));
	}

	public DecisionDocument getDocument() {

		return document;
	}

	public void setDocument(DecisionDocument document) {

		this.document = document;
	}

	public boolean isSent() {

		return sent;
	}

	public void setSent(boolean sent) {

		this.sent = sent;
	}

}
