package com.nordicpeak.flowengine.evaluators.setweightfromparameterevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;
import com.nordicpeak.flowengine.interfaces.ParameterEvaluator;

@Table(name = "set_weight_param_evaluators")
@XMLElement
public class SetWeightFromParameterEvaluator extends BaseEvaluator implements ParameterEvaluator {

	private static final long serialVersionUID = 1129410235430246625L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement(cdata = true)
	private String paramName;
	
	@DAOManaged
	@WebPopulate(required=true, maxLength=255)
	@XMLElement
	private String weightType;
	
	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;

	private boolean initialized;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		paramName = XMLValidationUtils.validateParameter("paramName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		weight = xmlParser.getInt("weight");

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}
	
	public String getWeightType() {
		return weightType;
	}
	
	public void setWeightType(String weightType) {
		this.weightType = weightType;
	}
	
	public Integer getWeight() {
		return weight;
	}
	
	public void setWeight(Integer weight) {
		this.weight = weight;
	}

	@Override
	public String toString() {
		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {
		return true;
	}

	@Override
	public boolean requiresInitialization() {
		return true;
	}

	@Override
	public String getParamName() {
		return paramName;
	}

	public void setParamName(String expression) {
		this.paramName = expression;
	}

	public boolean isInitialized() {
		return initialized;
	}

	public void setInitialized(boolean initialized) {
		this.initialized = initialized;
	}
}
