$(function() {
	
	var list = $("#address-list");
	var url = list.data("url");
	var selectByStreetNumber = list.data("selectByStreetNumber");
	var searchInput = $('#address-search');
	 
	searchInput.autocomplete({
		source: function(request, response) {
			return searchAddress(request, response, url, searchInput, selectByStreetNumber);
		},
		select: function( event, ui ) {
			
			addEntry(ui.item, list);
			
			searchInput.val("");
			
			return false;
		},
		focus: function(event, ui) {
	        event.preventDefault();
	    }
	});
	 
	var template = getTemplate();
	template.find("input").prop("disabled", true);
	
	function searchAddress(request, response, searchURL, searchInput, selectByStreetNumber) {
		
		searchInput.addClass("ui-autocomplete-loading");
		
		$.ajax({
			url : searchURL,
			dataType : "json",
			contentType: "application/x-www-form-urlencoded;charset=UTF-8",
			data : {
				q : encodeURIComponent(request.term)
			},
			success : function(data) {
				
				if (data != undefined && data.length > 0) {
					
					var addresses = {};
					
					data.forEach(function(item) {
						
						var street = item[5];
						var postalAddress = item[6];
						var streetNumber = item[7];
						
						if (!street || !postalAddress || (selectByStreetNumber && !streetNumber)) {
							
							return;
						}
						
						var fullAddress = street + " " + postalAddress;
						
						addresses[fullAddress] = {
								label : fullAddress,
								street : street,
								postalAddress : postalAddress,								
						}
						
						if (selectByStreetNumber) {
							
							street += " " + streetNumber;
							fullAddress = street + " " + postalAddress;
							
							addresses[fullAddress] = {
									label : fullAddress,
									street : street,
									postalAddress : postalAddress,								
							}
						}
					});
					
					response(addresses);
					
				} else {
					
					response(null);
				}
				
				searchInput.removeClass("ui-autocomplete-loading");
				
			},
			error : function() {
				
				searchInput.removeClass("ui-autocomplete-loading");
			}
		});
		
	}
	
	function getTemplate() {
		
		return $('#address-template');
	}
	
	function addEntry(item, list) {
		
		var exists = false;
		
		$("input[name='addressID']").each(function() {
			
			if ($(this).siblings('input[name^="street"]').val() == item.street && $(this).siblings('input[name^="postalAddress"]').val() == item.postalAddress) {
				
				exists = true;
				return false;
			}
		});
		
		if (exists) {
			
			return;
		}
		
		var clone = getTemplate().children().clone();
		
		var id = generateUUID();
		
		clone.find("h3").text(item.label);
		clone.find("input").prop("disabled", false);
		clone.find("input[name='addressID']").val(id);
		clone.find("input[name='street-']").val(item.street).attr("name", 'street-' + id);
		clone.find("input[name='postalAddress-']").val(item.postalAddress).attr("name", 'postalAddress-' + id);
		
		list.append(clone);
		clone.show();
	}
	
	function generateUUID() {
		var chars = '0123456789abcdef'.split('');
		var uuid = [], rnd = Math.random;
		var r;
		uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
		uuid[14] = '4';
		for (var i = 0; i < 36; i++) {
			if (!uuid[i]) {
				r = 0 | rnd()*16;
				uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r & 0xf];
			}
		}
		return uuid.join('');
	}
});
