package com.nordicpeak.flowengine.evaluators.userage;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.NonNegativeStringIntegerPopulator;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;

import com.nordicpeak.flowengine.evaluators.basequerystateevaluator.BaseQueryStateEvaluator;

@Table(name = "user_age_query_state_evaluators")
@XMLElement
public class UserAgeQueryStateEvaluator extends BaseQueryStateEvaluator {

	private static final long serialVersionUID = 7582032280793947029L;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer minAge;

	@DAOManaged
	@WebPopulate(populator = NonNegativeStringIntegerPopulator.class)
	@XMLElement
	private Integer maxAge;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean calculateBasedOnYearOnly;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAttributeIfAgeInRange;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "setAttributeIfAgeInRange")
	@XMLElement
	private String attributeName;
	
	@DAOManaged
	@WebPopulate
	@RequiredIfSet(paramNames = "setAttributeIfAgeInRange")
	@XMLElement
	private String attributeValue;

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		super.populate(xmlParser);

		minAge = xmlParser.getInteger("minAge");
		maxAge = xmlParser.getInteger("maxAge");
		
		setAttributeIfAgeInRange = xmlParser.getPrimitiveBoolean("setAttributeIfAgeInRange");
		attributeName = xmlParser.getString("attributeName");
		attributeValue = xmlParser.getString("attributeValue");
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getMinAge() {

		return minAge;
	}

	public void setMinAge(Integer minAge) {

		this.minAge = minAge;
	}

	public Integer getMaxAge() {

		return maxAge;
	}

	public void setMaxAge(Integer maxAge) {

		this.maxAge = maxAge;
	}

	
	public boolean isCalculateBasedOnYearOnly() {
	
		return calculateBasedOnYearOnly;
	}

	
	public void setCalculateBasedOnYearOnly(boolean yearOnly) {
	
		this.calculateBasedOnYearOnly = yearOnly;
	}

	
	public boolean isSetAttributeIfAgeInRange() {
	
		return setAttributeIfAgeInRange;
	}

	
	public void setSetAttributeIfAgeInRange(boolean setAttributeIfAgeInRange) {
	
		this.setAttributeIfAgeInRange = setAttributeIfAgeInRange;
	}

	
	public String getAttributeName() {
	
		return attributeName;
	}

	
	public void setAttributeName(String attributeName) {
	
		this.attributeName = attributeName;
	}

	
	public String getAttributeValue() {
	
		return attributeValue;
	}

	
	public void setAttributeValue(String attributeValue) {
	
		this.attributeValue = attributeValue;
	}
}
