package com.nordicpeak.flowengine.queries.userlistquery.beans;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.standardutils.annotations.PopulateOnlyIfRequired;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.numbers.NumberUtils;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.interfaces.MultiSignQuery;
import com.nordicpeak.flowengine.interfaces.UserValueQuery;
import com.nordicpeak.flowengine.interfaces.UserValuesQuery;
import com.nordicpeak.flowengine.queries.basequery.BaseQuery;
import com.nordicpeak.flowengine.queries.userlistquery.enums.UserSourceType;

@Table(name = "userlist_queries")
@XMLElement
public class UserListQuery extends BaseQuery implements UserValueQuery, UserValuesQuery, MultiSignQuery {

	private static final long serialVersionUID = -842191226937409429L;

	@DAOManaged
	@Key
	@XMLElement
	private Integer queryID;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private UserSourceType userSourceType;

	@DAOManaged
	@RequiredIfSet(paramNames = "userSourceType", paramValues = "LIST_USER_PROVIDER")
	@PopulateOnlyIfRequired
	@WebPopulate(maxLength = 255)
	@XMLElement
	private String providerIdentifier;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "userlist_query_groups", remoteValueColumnName = "groupID")
	@RequiredIfSet(paramNames = "userSourceType", paramValues = "GROUPS")
	@PopulateOnlyIfRequired	
	@WebPopulate(paramName = "group")
	@XMLElement(childName = "groupID")
	private List<Integer> groupIDs;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer minSelectedUsers;

	@DAOManaged
	@WebPopulate(populator = PositiveStringIntegerPopulator.class)
	@XMLElement
	private Integer maxSelectedUsers;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showCitizenIdentifier;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showContactInformation;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useForMultipartSigning;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setMultipartsAsOwners;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean showUsername;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean setAsAttribute;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean excludeInactiveUsers;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "setAsAttribute")
	@XMLElement
	private String attributeName;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<UserListQueryInstance> instances;

	@Override
	public Integer getQueryID() {

		return queryID;
	}

	public List<UserListQueryInstance> getInstances() {

		return instances;
	}

	public void setInstances(List<UserListQueryInstance> instances) {

		this.instances = instances;
	}

	public Integer getMaxSelectedUsers() {

		return maxSelectedUsers;
	}

	public void setMaxSelectedUsers(Integer maxSelectedUsers) {

		this.maxSelectedUsers = maxSelectedUsers;
	}

	public void setQueryID(int queryID) {

		this.queryID = queryID;
	}

	public void setQueryID(Integer queryID) {

		this.queryID = queryID;
	}

	public boolean isShowCitizenIdentifier() {

		return showCitizenIdentifier;
	}

	public void setShowCitizenIdentifier(boolean showCitizenIdentifier) {

		this.showCitizenIdentifier = showCitizenIdentifier;
	}

	public boolean isShowContactInformation() {

		return showContactInformation;
	}

	public Integer getMinSelectedUsers() {

		return minSelectedUsers;
	}

	public void setMinSelectedUsers(Integer minSelectedUsers) {

		this.minSelectedUsers = minSelectedUsers;
	}

	public void setShowContactInformation(boolean showContactInformation) {

		this.showContactInformation = showContactInformation;
	}

	@Override
	public boolean requiresMultipartSigning() {

		return useForMultipartSigning;
	}

	public void setUseForMultipartSigning(boolean useForMultiSigning) {

		this.useForMultipartSigning = useForMultiSigning;
	}

	public boolean isSetMultipartsAsOwners() {

		return setMultipartsAsOwners;
	}

	public void setSetMultipartsAsOwners(boolean setMultipartsAsOwners) {

		this.setMultipartsAsOwners = setMultipartsAsOwners;
	}

	@Override
	public String toString() {

		if (this.queryDescriptor != null) {

			return queryDescriptor.getName() + " (queryID: " + queryID + ")";
		}

		return getClass().getSimpleName() + " (queryID: " + queryID + ")";
	}

	@Override
	public String getXSDTypeName() {

		return getClass().getSimpleName() + queryID;
	}

	@Override
	public void toXSD(Document doc) {

		Element complexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		complexTypeElement.setAttribute("name", getXSDTypeName());

		Element complexContentElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexContent");
		complexTypeElement.appendChild(complexContentElement);

		Element extensionElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:extension");
		extensionElement.setAttribute("base", "Query");
		complexContentElement.appendChild(extensionElement);

		Element sequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		extensionElement.appendChild(sequenceElement);

		Element nameElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		nameElement.setAttribute("name", "Name");
		nameElement.setAttribute("type", "xs:string");
		nameElement.setAttribute("minOccurs", "1");
		nameElement.setAttribute("maxOccurs", "1");
		nameElement.setAttribute("fixed", queryDescriptor.getName());
		sequenceElement.appendChild(nameElement);

		Element userElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		userElement.setAttribute("name", "User");
		userElement.setAttribute("type", getXSDTypeName() + "User");

		userElement.setAttribute("minOccurs", Integer.toString(NumberUtils.toPrimitiveInt(minSelectedUsers)));

		if (maxSelectedUsers != null) {

			userElement.setAttribute("maxOccurs", maxSelectedUsers.toString());

		} else {

			userElement.setAttribute("maxOccurs", "unbounded");
		}

		sequenceElement.appendChild(userElement);

		doc.getDocumentElement().appendChild(complexTypeElement);

		// User type
		Element userComplexTypeElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:complexType");
		userComplexTypeElement.setAttribute("name", getXSDTypeName() + "User");

		Element userSequenceElement = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:sequence");
		userComplexTypeElement.appendChild(userSequenceElement);

		addElementType(doc, userSequenceElement, "UserID", "xs:string");
		addElementType(doc, userSequenceElement, "Username", "xs:string", false);
		addElementType(doc, userSequenceElement, "Firstname", "xs:string");
		addElementType(doc, userSequenceElement, "Lastname", "xs:string");
		addElementType(doc, userSequenceElement, "CitizenIdentifier", "xs:string", false);
		addElementType(doc, userSequenceElement, "Email", "xs:string", false);
		addElementType(doc, userSequenceElement, "Phone", "xs:string", false);

		doc.getDocumentElement().appendChild(userComplexTypeElement);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type) {

		addElementType(doc, sequenceElement, name, type, null, true);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, boolean required) {

		addElementType(doc, sequenceElement, name, type, null, required);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed) {

		addElementType(doc, sequenceElement, name, type, fixed, true);
	}

	public void addElementType(Document doc, Element sequenceElement, String name, String type, String fixed, boolean required) {

		Element element = doc.createElementNS("http://www.w3.org/2001/XMLSchema", "xs:element");
		element.setAttribute("name", name);
		element.setAttribute("type", type);
		element.setAttribute("minOccurs", required ? "1" : "0");
		element.setAttribute("maxOccurs", "1");

		if (!StringUtils.isEmpty(fixed)) {
			element.setAttribute("fixed", fixed);
		}

		sequenceElement.appendChild(element);
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		description = XMLValidationUtils.validateParameter("description", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		helpText = XMLValidationUtils.validateParameter("helpText", xmlParser, false, 1, 65535, StringPopulator.getPopulator(), errors);
		providerIdentifier = XMLValidationUtils.validateParameter("providerIdentifier", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		minSelectedUsers = xmlParser.getInteger("minSelectedUsers");
		maxSelectedUsers = xmlParser.getInteger("maxSelectedUsers");
		showCitizenIdentifier = xmlParser.getPrimitiveBoolean("showCitizenIdentifier");
		showContactInformation = xmlParser.getPrimitiveBoolean("showContactInformation");
		useForMultipartSigning = xmlParser.getPrimitiveBoolean("useForMultipartSigning");
		setMultipartsAsOwners = xmlParser.getPrimitiveBoolean("setMultipartsAsOwners");
		showUsername = xmlParser.getPrimitiveBoolean("showUsername");

		attributeName = XMLValidationUtils.validateParameter("attributeName", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		if (attributeName != null) {

			setAsAttribute = xmlParser.getPrimitiveBoolean("setAsAttribute");
		}

		excludeInactiveUsers = xmlParser.getPrimitiveBoolean("excludeInactiveUsers");

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	public String getProviderIdentifier() {

		return providerIdentifier;
	}

	public void setProviderIdentifier(String providerIdentifier) {

		this.providerIdentifier = providerIdentifier;
	}

	public boolean isSetAsAttribute() {

		return setAsAttribute;
	}

	public void setSetAsAttribute(boolean setAsAttribute) {

		this.setAsAttribute = setAsAttribute;
	}

	public boolean isExcludeInactiveUsers() {

		return excludeInactiveUsers;
	}

	public void setExcludeInactiveUsers(boolean excludeInactiveUsers) {

		this.excludeInactiveUsers = excludeInactiveUsers;
	}

	public String getAttributeName() {

		return attributeName;
	}

	public void setAttributeName(String attributeName) {

		this.attributeName = attributeName;
	}

	public UserSourceType getUserSourceType() {

		return userSourceType;
	}

	public void setUserSourceType(UserSourceType userSourceType) {

		this.userSourceType = userSourceType;
	}

	public List<Integer> getGroupIDs() {

		return groupIDs;
	}

	public void setGroupIDs(List<Integer> groupIDs) {

		this.groupIDs = groupIDs;
	}

}
