package com.nordicpeak.flowengine.evaluators.userquerymailevaluator;

import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "user_query_mail_evaluators")
@XMLElement
public class UserQueryMailEvaluator extends BaseEvaluator {
	
	private static final long serialVersionUID = 5501896243489595373L;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean enableStatusFiltering;
	
	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "user_query_mail_evaluator_statuses", remoteValueColumnName = "statusID")
	@WebPopulate(paramName = "statusID")
	@XMLElement(fixCase = true, childName = "statusID")
	private List<Integer> statusIDs;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String emailSubject;
	
	@DAOManaged
	@WebPopulate(required = true, maxLength = 65536)
	@XMLElement
	private String emailMessage;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean attachPDF;
	
	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean sendPDFAttachmentsSeparately;
	
	public String getEmailSubject() {
		
		return emailSubject;
	}
	
	public void setEmailSubject(String emailSubject) {
		
		this.emailSubject = emailSubject;
	}
	
	public String getEmailMessage() {
		
		return emailMessage;
	}
	
	public void setEmailMessage(String emailMessage) {
		
		this.emailMessage = emailMessage;
	}
	
	public boolean attachPDF() {
		
		return attachPDF;
	}
	
	public void setAttachPDF(boolean attachPDF) {
		
		this.attachPDF = attachPDF;
	}
	
	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {
		
		emailSubject = xmlParser.getString("emailSubject");
		emailMessage = xmlParser.getString("emailMessage");
		attachPDF = xmlParser.getPrimitiveBoolean("attachPDF");
		enableStatusFiltering = xmlParser.getPrimitiveBoolean("enableStatusFiltering");
		statusIDs = xmlParser.getIntegers("StatusIDs/statusID");
	}
	
	@Override
	public String toString() {
		
		return evaluatorDescriptor.toString();
	}
	
	public boolean enableStatusFiltering() {
		
		return enableStatusFiltering;
	}
	
	public void setEnableStatusFiltering(boolean enableStatusFiltering) {
		
		this.enableStatusFiltering = enableStatusFiltering;
	}
	
	public List<Integer> getStatusIDs() {
		
		return statusIDs;
	}
	
	public void setStatusIDs(List<Integer> statusIDs) {
		
		this.statusIDs = statusIDs;
	}
	
	public boolean sendPDFAttachmentsSeparately() {
		
		return sendPDFAttachmentsSeparately;
	}
	
	public void setSendPDFAttachmentsSeparately(boolean sendPDFAttachmentsSeparately) {
		
		this.sendPDFAttachmentsSeparately = sendPDFAttachmentsSeparately;
	}
}
