$(function() {

	setQueryRequiredFunctions["UserTreeQueryInstance"] = makeUserTreeQueryRequired;
});

function initUserTreeQuery(queryID) {

	waitUntilUserTreeQueryScriptsAreLoaded(queryID);
}

function waitUntilUserTreeQueryScriptsAreLoaded(queryID) {
	
	if(typeof $.ui.fancytree === "undefined" || typeof $.ui.fancytree._FancytreeClass.prototype.clearFilter === "undefined" || typeof $.ui.fancytree._FancytreeClass.prototype.getPersistData === "undefined") {
		
		setTimeout(function() { waitUntilUserTreeQueryScriptsAreLoaded(queryID); }, 5);
		
	} else {
		
		initUserTreeQuery2(queryID);
	} 
}

function initUserTreeQuery2(queryID) {

	var query = $("#query_" + queryID);
	var $tree = $("#tree" + queryID);
	var $usertree = $("#usertree" + queryID);
	
	if ($tree.length > 0) {

		var fetchUsersURL = $usertree.data("fetchurl");
		var requireEmail = $usertree.data("requireemail") == true;
		var onlyleafselection = $tree.data("nofolders") == true;
		var selectedInput = query.find("input.selectedkey");
		var selectedUserInput = query.find("input.selecteduser");
		
		if(fetchUsersURL == ""){
			if(console !== undefined){
				console.error("UserTree fetchurl missing!");
			}
		}
		
		// https://github.com/mar10/fancytree/wiki
		$usertree.fancytree({
			extensions: ["filter"],
			autoActivate: false,	// Automatically activate a node when it is focused (using keys).
			autoScroll: true,		// Automatically scroll nodes into visible area.
			clickFolderMode: 1,		// 1:activate, 2:expand, 3:activate and expand, 4:activate (dblclick expands)
			debugLevel: 1,			// 0:quiet, 1:normal, 2:debug
			minExpandLevel: 2,		// x: nodes x from invisible root (inclusive) is not collapsible
			titlesTabbable: true,	// Add all node titles to TAB chain
			quicksearch: true,		// Jump to nodes when pressing first character
			keyboard: false,
			source: [{title: UserTree.i18nInitialNodeTitle, statusNodeType: "nodata", icon: UserTree.PicEmpty}],
			
			// https://github.com/mar10/fancytree/wiki/ExtFilter
			filter: {
				mode: 'hide', // 'dimm' | 'hide'
				highlight: true,
			},
			
			focus: function(event, data) {
		        data.node.scrollIntoView(true);
		    },
		    
		    beforeActivate: function(event, data) {
		    	
		    	var node = data.node;
		    	
		    	if(!node.isActive() && node.isStatusNode()){
		    		
		    		return false;
		    	}
		    	
		    	if(requireEmail && node.data.email == null){
					return false;
				}
	    	},
		    
		    activate: function(event, data) {
		    	selectedUserInput.val(data.node.key);
	    	},
		});
		
		var usertree = $usertree.fancytree("getTree");
		
		// https://github.com/mar10/fancytree/wiki
		$tree.fancytree({
			extensions: ["persist", "filter"],
			autoActivate: false,	// Automatically activate a node when it is focused (using keys).
			autoScroll: true,		// Automatically scroll nodes into visible area.
			clickFolderMode: 4,		// 1:activate, 2:expand, 3:activate and expand, 4:activate (dblclick expands)
			debugLevel: 1,			// 0:quiet, 1:normal, 2:debug
			minExpandLevel: 2,		// x: nodes x from invisible root (inclusive) is not collapsible
			titlesTabbable: true,	// Add all node titles to TAB chain
			quicksearch: true,		// Jump to nodes when pressing first character
			keyboard: true,
			source: window["UserTreeQueryTree" + queryID],
			
			// https://github.com/mar10/fancytree/wiki/ExtPersist
			persist: {
				cookiePrefix: "fancytree-usertreequery-" + queryID,
				overrideSource: false,  // true: cookie takes precedence over `source` data attributes.
				store: "session",     // 'cookie': use cookie, 'local': use localStore, 'session': use sessionStore
				types: "expanded",  // which status types to store "active expanded focus selected"
				expandLazy: true
			},
			
			// https://github.com/mar10/fancytree/wiki/ExtFilter
			filter: {
				mode: 'hide', // 'dimm' | 'hide'
				highlight: true,
			},
			
			focus: function(event, data) {
		        data.node.scrollIntoView(true);
		    },
		    
		    beforeActivate: function(event, data){
				if(onlyleafselection && data.node.hasChildren()){
					return false;
				}
			},
		    
		    activate: function(event, data) {
		    	
		    	selectedInput.val(data.node.key);
		    	
		    	usertree.clear();
		    	
		    	var root = usertree.getRootNode();
		    	
		    	var loadingNode = root.addNode({
					title: UserTree.i18nLoading,
					icon: UserTree.PicLoading,
				});
		    	loadingNode.statusNodeType = "loading";
	    		
	    		$.ajax({
    			  url: fetchUsersURL,
    			  data: { selectedNodeKey: data.node.key},
    			  
    			}).done(function(result){
    				
    				usertree.clear();
    				
    				if (result.users !== undefined) {
    					
    					if(result.users.length > 0){
    						
    						updateUserTree(usertree, result.users, requireEmail);
    						
    						if(selectedUserInput.val() != ""){
    							
    							var activatedNode = usertree.activateKey(selectedUserInput.val(), {activeVisible: true});
    							
    							if(activatedNode == null){
    								
    								selectedUserInput.val("");
    							}
    						}
    						
    					} else {
    						
    						var emptyNode = root.addNode({
        						title: UserTree.i18nEmpty,
        					    icon: UserTree.PicEmpty,
        					});
        					emptyNode.statusNodeType = "nodata";
    					}
    					
    				} else {
    					
        				var errorNode = root.addNode({
        					title: UserTree.i18nFailed,
        				    icon: UserTree.PicFailed,
        				});
        				errorNode.statusNodeType = "error";
    				}
    				
    			}).fail(function(jqXHR, textStatus, errorThrown){
    				
    				usertree.clear();
    				
    				var errorNode = root.addNode({
    					title: UserTree.i18nFailed + ": " + jqXHR.status + " " + jqXHR.statusText,
    				    icon: UserTree.PicFailed,
    				});
    				errorNode.statusNodeType = "error";
    			});
	    	},
		});
		
		var tree = $tree.fancytree("getTree");
		
		userTreefilterField(tree, "filterField" + queryID);
		userTreefilterField(usertree, "filterField2" + queryID);
		
		query.closest("form").on("submit", function(){
			tree.clearFilter();
			usertree.clearFilter();
		});
	}
}

function userTreeGetUserIcon(user, requireEmail){

	if(requireEmail && user.email == null){
		return UserTree.PicUserDisabled;
	}
	
	return UserTree.PicUser;
}

function userTreeGetUserTitle(user, requireEmail){

	var name = user.firstname + " " + user.lastname;
	
	if(requireEmail && user.email == null){
		name += " (" + UserTree.i18nEmailMissing + ")";
	}
	
	return name;
}

function updateUserTree(tree, usersJson, requireEmail) {
	
	var root = tree.getRootNode();
	
	for (var i = 0; i < usersJson.length; i++) {
		
	    var user = usersJson[i];
		
	    root.addChildren({
			key: user.providerUserID,
			title: userTreeGetUserTitle(user, requireEmail),
			tooltip: user.email,
			icon: userTreeGetUserIcon(user, requireEmail),
		    extraClasses: "user",
		    email: user.email,
		});
	}
}

function userTreefilterField(tree, inputname){
	
	var KEYCODE_ENTER = 13;
	var KEYCODE_ESC = 27;
	
	$("input[name='" + inputname + "']").on("keyup", function(event){
		
		var $this = $(this);
        var filter = $this.val();

		if (event && event.keyCode == KEYCODE_ESC || $.trim(filter) === "") {
			
			$this.val("");
			
			tree.clearFilter();
			return;
		}
		
		tree.filterNodes(filter, {autoExpand: true});
		
	}).on("keypress", function(event){
		
		if(event.keyCode == KEYCODE_ENTER){
			event.preventDefault();
		}
	});
}

function userTreeQueryCreateShowTree(queryID) {

	var tree = $("#tree" + queryID);
	
	if (tree.length > 0) {
		
		// https://github.com/mar10/fancytree/wiki
		tree.fancytree({
			extensions: [],
			autoActivate: false, 	// Automatically activate a node when it is focused (using keys).
			autoScroll: true, 		// Automatically scroll nodes into visible area.
			clickFolderMode: 1, 	// 1:activate, 2:expand, 3:activate and expand, 4:activate (dblclick expands)
			debugLevel: 1, 			// 0:quiet, 1:normal, 2:debug
			minExpandLevel: 99, 	// x: nodes x from invisible root (inclusive) is not collapsible
			titlesTabbable: false,  // Add all node titles to TAB chain
			quicksearch: false,     // Jump to nodes when pressing first character
			keyboard: false,
			source: window["UserTreeQueryTree" + queryID],
			
			beforeActivate: function(event, data) {
	    		return false;
	    	},
		
			beforeSelect: function(event, data) {
	    		return false;
	    	}
		});
	}
}

function userTreeQuerySetSelected(queryID, key){
	
	var tree = $("#tree" + queryID).fancytree("getTree");
	
	tree.activateKey(key, {activeVisible: true});
}

function makeUserTreeQueryRequired(queryID) {

	$("#query_" + queryID).find(".heading-wrapper h2").addClass("required");
}