package com.nordicpeak.flowengine.evaluators.setattributefromweightevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.utils.TextTagReplacer;

public class SetAttributeFromWeightEvaluationProviderModule extends BaseEvaluationProviderModule<SetAttributeFromWeightEvaluator> {
	
	protected SetAttributeFromWeightEvaluatorCRUD evaluatorCRUD;
	
	protected AnnotatedDAO<SetAttributeFromWeightEvaluator> evaluatorDAO;
	
	protected QueryParameterFactory<SetAttributeFromWeightEvaluator, Integer> evaluatorIDParamFactory;
	
	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {
		
		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetAttributeFromWeightEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));
		
		if (upgradeResult.isUpgrade()) {
			
			log.info(upgradeResult.toString());
		}
		
		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetAttributeFromWeightEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);
		
		evaluatorCRUD = new SetAttributeFromWeightEvaluatorCRUD(SetAttributeFromWeightEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}
	
	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {
		
		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}
	
	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetAttributeFromWeightEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {
		
		if (evaluator.getFromValue() == null || evaluator.getAttributeName() == null) {
			return null;
		}
		
		BigDecimal weight = evaluator.getWeight();
		
		if (weight != null && weight.compareTo(BigDecimal.valueOf(evaluator.getFromValue())) >= 0 && (evaluator.getToValue() == null || weight.compareTo(BigDecimal.valueOf(evaluator.getToValue())) <= 0)) {
			
			attributeHandler.setAttribute(evaluator.getAttributeName(), evaluator.getAttributeValue());
			
		} else {
			
			attributeHandler.removeAttribute(evaluator.getAttributeName());
		}
		
		return null;
	}
	
	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {
		
		return true;
	}
	
	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {
		
		SetAttributeFromWeightEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetAttributeFromWeightEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));
		
		if (evaluator != null) {
			
			if (instanceMetadata != null && instanceMetadata.getSiteProfile() != null) {
				
				TextTagReplacer.replaceTextTags(evaluator, instanceMetadata.getSiteProfile());
			}
			
			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}
		
		return evaluator;
	}
	
	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		SetAttributeFromWeightEvaluator evaluator = new SetAttributeFromWeightEvaluator();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		
		return evaluator;
	}
	
	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {
		
		SetAttributeFromWeightEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<SetAttributeFromWeightEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));
		
		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}
	
	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {
		
		evaluatorDAO.delete(new HighLevelQuery<SetAttributeFromWeightEvaluator>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);
		
		return true;
	}
	
	protected Class<SetAttributeFromWeightEvaluator> getEvaluatorClass() {
		
		return SetAttributeFromWeightEvaluator.class;
	}
	
	@Override
	public SetAttributeFromWeightEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {
		
		SetAttributeFromWeightEvaluator evaluator = getEvaluatorClass().newInstance();
		
		evaluator.setEvaluatorID(descriptor.getEvaluatorID());
		
		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));
		
		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
		return evaluator;
	}
}
