<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateSetTextFieldValueWeightEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateSetTextFieldValueWeightEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="SetTextFieldValueWeightEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="SetTextFieldValueWeightEvaluator" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<label for="expression" class="floatleft full">
					<xsl:value-of select="$i18n.expression" />
					<span class="required">*</span>
				</label>

				<p><xsl:value-of select="$i18n.expressionDescription" /><br/><xsl:value-of select="$i18n.expressionDescription2"/></p>

				<div class="floatleft full">
					<xsl:call-template name="createTextArea">
						<xsl:with-param name="id" select="'expression'" />
						<xsl:with-param name="name" select="'expression'" />
						<xsl:with-param name="element" select="SetTextFieldValueWeightEvaluator" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<label for="valueSuffix" class="floatleft full">
					<xsl:value-of select="$i18n.valueSuffix" />
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'valueSuffix'" />
						<xsl:with-param name="name" select="'valueSuffix'" />
						<xsl:with-param name="element" select="SetTextFieldValueWeightEvaluator" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full marginbottom">
	
				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'displayDecimals'" />
						<xsl:with-param name="name" select="'displayDecimals'" />
						<xsl:with-param name="value" select="'true'" />
						<xsl:with-param name="element" select="SetTextFieldValueWeightEvaluator" />
					</xsl:call-template>
	
					<label for="displayDecimals">
						<xsl:value-of select="$i18n.displayDecimals" />
					</label>
				</div>
			</div>					
			
			<div class="floatleft full bigmarginbottom">

				<label for="invalidExpressionMessage" class="floatleft full">
					<xsl:value-of select="$i18n.invalidExpressionMessage" />
					<span class="required">*</span>
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'invalidExpressionMessage'" />
						<xsl:with-param name="name" select="'invalidExpressionMessage'" />
						<xsl:with-param name="element" select="SetTextFieldValueWeightEvaluator" />
					</xsl:call-template>
				</div>
			</div>			

			<div class="floatleft full bigmarginbottom">

				<xsl:variable name="fieldsCount" select="count(TextFields/TextField)" />
	
				<label class="floatleft full">
					<xsl:value-of select="$i18n.fields.title" />
				</label>
		
				<xsl:choose>
					<xsl:when test="$fieldsCount > 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.description" />
						</div>
					
						<xsl:apply-templates select="TextFields/TextField">
							<xsl:with-param name="queryStateEvaluator" select="SetTextFieldValueWeightEvaluator" />
							<xsl:with-param name="requestparameters" select="requestparameters" />
						</xsl:apply-templates>
					</xsl:when>
					<xsl:when test="$fieldsCount = 1">
						
						<div class="clearboth floatleft marginbottom">
							<xsl:value-of select="$i18n.fields.single.description" />
						</div>
						
						<xsl:call-template name="createHiddenField">
							<xsl:with-param name="id" select="'textFieldID'" />
							<xsl:with-param name="name" select="'textFieldID'" />
							<xsl:with-param name="value" select="TextFields/TextField/fieldID" />
						</xsl:call-template>
						
						<p class="floatleft full bold nomargin"><xsl:value-of select="TextFields/TextField/name" /></p>
						
					</xsl:when>
					<xsl:otherwise>
						<p>
							<xsl:value-of select="$i18n.QueryHasNoTextFields" />
						</p>
					</xsl:otherwise>
				</xsl:choose>
			
			</div>

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.submit}" />
			</div>

		</form>

	</xsl:template>

	<xsl:template match="TextField">

		<xsl:param name="queryStateEvaluator" />
		<xsl:param name="requestparameters" />

		<div class="floatleft full marginbottom">

			<div class="floatleft full">
				<xsl:call-template name="createRadio">
					<xsl:with-param name="id">
						<xsl:value-of select="'textfield'"/>
						<xsl:value-of select="fieldID"/>
					</xsl:with-param>				
					<xsl:with-param name="name" select="'textFieldID'" />
					<xsl:with-param name="value" select="fieldID" />
					<xsl:with-param name="element" select="$queryStateEvaluator" />
					<xsl:with-param name="requestparameters" select="$requestparameters" />
				</xsl:call-template>

				<label for="textfield{fieldID}">
					<xsl:value-of select="name" />
				</label>
			</div>
			
		</div>

	</xsl:template>
	
	<xsl:template match="validationError[messageKey = 'TextFieldNotFound']">

		<p class="error">
			<xsl:value-of select="$i18n.TextFieldNotFound" />
		</p>
		
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'expression'">
				<xsl:value-of select="$i18n.expression" />
			</xsl:when>
			<xsl:when test="$fieldName = 'invalidExpressionMessage'">
				<xsl:value-of select="$i18n.invalidExpressionMessage" />
			</xsl:when>
			<xsl:when test="$fieldName = 'valueSuffix'">
				<xsl:value-of select="$i18n.valueSuffix" />
			</xsl:when>
			<xsl:when test="$fieldName = 'textFieldID'">
				<xsl:value-of select="$i18n.fields.title" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>