package com.nordicpeak.flowengine.evaluators.setweightbyalternativeidevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.InstanceManagerDependency;
import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableAlternative;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryHandler;
import com.nordicpeak.flowengine.interfaces.QueryInstance;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQuery;
import com.nordicpeak.flowengine.queries.fixedalternativesquery.FixedAlternativesQueryInstance;

public class SetWeightByAlternativeIDEvaluationProviderModule extends BaseEvaluationProviderModule<SetWeightByAlternativeIDEvaluator> {

	protected SetWeightByAlternativeIDEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetWeightByAlternativeIDEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetWeightByAlternativeIDEvaluator, Integer> evaluatorIDParamFactory;

	@InstanceManagerDependency(required = true)
	protected QueryHandler queryHandler;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetWeightByAlternativeIDEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetWeightByAlternativeIDEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetWeightByAlternativeIDEvaluatorCRUD(SetWeightByAlternativeIDEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetWeightByAlternativeIDEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		if (evaluator.getWeightType() == null) {

			return null;
		}

		if (!(queryInstance instanceof FixedAlternativesQueryInstance)) {

			log.warn("Query instance " + queryInstance + " does not implement interface " + FixedAlternativesQueryInstance.class.getName());

			return null;
		}

		List<? extends ImmutableAlternative> alternatives = ((FixedAlternativesQueryInstance) queryInstance).getAlternatives();

		BigDecimal totalWeight = BigDecimal.ZERO;

		if (!CollectionUtils.isEmpty(alternatives)) {

			for (ImmutableAlternative alternative : alternatives) {

				BigDecimal weight = BigDecimal.valueOf(alternative.getAlternativeID());

				if (weight != null) {

					totalWeight = totalWeight.add(weight);
				}
			}
		}

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), totalWeight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetWeightByAlternativeIDEvaluator evaluator = new SetWeightByAlternativeIDEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return FixedAlternativesQuery.class.isAssignableFrom(queryClass);
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetWeightByAlternativeIDEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
	}

	@Override
	public SetWeightByAlternativeIDEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetWeightByAlternativeIDEvaluator evaluator = new SetWeightByAlternativeIDEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	protected Class<SetWeightByAlternativeIDEvaluator> getEvaluatorClass() {

		return SetWeightByAlternativeIDEvaluator.class;
	}

	protected AnnotatedDAO<SetWeightByAlternativeIDEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected QueryParameterFactory<SetWeightByAlternativeIDEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetWeightByAlternativeIDEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public QueryHandler getQueryHandler() {

		return this.queryHandler;
	}
}
