<?xml version="1.0" encoding="ISO-8859-1" ?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" version="4.0" encoding="ISO-8859-1" />

	<xsl:include href="classpath://se/unlogic/hierarchy/core/utils/xsl/Common.xsl" />

	<xsl:variable name="scripts">
		/js/setweightbycontactfieldevaluator.js
	</xsl:variable>
	
	<xsl:template match="Document">

		<div class="contentitem">

			<xsl:apply-templates select="UpdateSetWeightByContactFieldEvaluator" />
		</div>

	</xsl:template>

	<xsl:template match="UpdateSetWeightByContactFieldEvaluator">

		<h1>
			<xsl:value-of select="$i18n.UpdateEvaluatorDescriptor.title" />
			<xsl:text>&#x20;</xsl:text>
			<xsl:value-of select="SetWeightByContactFieldEvaluator/EvaluatorDescriptor/name" />
		</h1>

		<xsl:apply-templates select="validationException/validationError" />

		<form method="post" action="{/Document/requestinfo/uri}">

			<xsl:call-template name="createCommonFieldsForm">
				<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" />
			</xsl:call-template>

			<div class="floatleft full bigmarginbottom">

				<label for="weightType" class="floatleft full">
					<xsl:value-of select="$i18n.WeightType" />
					<span class="required">*</span>
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'weightType'" />
						<xsl:with-param name="name" select="'weightType'" />
						<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" />
					</xsl:call-template>
				</div>
			</div>

			<div class="floatleft full bigmarginbottom">

				<label for="contactField" class="floatleft full">
					<xsl:value-of select="$i18n.ContactField" />
				</label>

				<div class="floatleft full">
					<xsl:call-template name="createDropdown">
						<xsl:with-param name="id" select="'contactField'" />
						<xsl:with-param name="name" select="'contactField'" />
						<xsl:with-param name="element" select="ContactFields/ContactField" />
						<xsl:with-param name="valueElementName" select="'name'" />
						<xsl:with-param name="labelElementName" select="'displayName'" />
						<xsl:with-param name="selectedValue" select="SetWeightByContactFieldEvaluator/contactField" />
					</xsl:call-template>
				</div>
			</div>
			
			<p class="floatleft full">
				<xsl:value-of select="$i18n.Value.Description" />
			</p>
			
			<table id="value-table" class="full coloredtable bigmarginbottom">
				<thead>
					<tr>
						<th><xsl:value-of select="$i18n.Value.MatchedValue" /></th>
						<th><xsl:value-of select="$i18n.Value.Weight" /></th>
						<th width="16px"></th>
					</tr>
				</thead>
				<tbody>
				
					<xsl:choose>
						<xsl:when test="requestparameters">
						
							<xsl:apply-templates select="requestparameters/parameter[name = 'valueID']/value" mode="parameter"/>
						
						</xsl:when>
						<xsl:otherwise>
						
							<xsl:apply-templates select="SetWeightByContactFieldEvaluator/Values/Value"/>
						
						</xsl:otherwise>
					</xsl:choose>					
				
				</tbody>
			</table>	
			
			<div class="floatright bigmarginbottom">
				<a onclick="addValue();">
					<xsl:value-of select="$i18n.AddValue" />
					<img class="alignbottom marginleft" src="{/Document/StaticContentURL}/pics/add.png" alt="" />
				</a>
			</div>	
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setWeightIfNoValueMatches'" />
						<xsl:with-param name="name" select="'setWeightIfNoValueMatches'" />
						<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" /> 
					</xsl:call-template>
						
					<label for="setWeightIfNoValueMatches">
						<xsl:value-of select="$i18n.SetWeightIfNoValueMatches" />
					</label>
				</div>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'weightIfNoValueMatches'" />
						<xsl:with-param name="name" select="'weightIfNoValueMatches'" />
						<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" />
					</xsl:call-template>
				</div>
			</div>
			
			<div class="floatleft full bigmarginbottom">

				<div class="floatleft full">
					<xsl:call-template name="createCheckbox">
						<xsl:with-param name="id" select="'setWeightIfContactFieldIsEmpty'" />
						<xsl:with-param name="name" select="'setWeightIfContactFieldIsEmpty'" />
						<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" /> 
					</xsl:call-template>
						
					<label for="setWeightIfContactFieldIsEmpty">
						<xsl:value-of select="$i18n.SetWeightIfContactFieldIsEmpty" />
					</label>
				</div>

				<div class="floatleft full">
					<xsl:call-template name="createTextField">
						<xsl:with-param name="id" select="'weightIfContactFieldIsEmpty'" />
						<xsl:with-param name="name" select="'weightIfContactFieldIsEmpty'" />
						<xsl:with-param name="element" select="SetWeightByContactFieldEvaluator" />
					</xsl:call-template>
				</div>
			</div>						

			<div class="floatright clearboth">
				<input type="submit" value="{$i18n.UpdateSetWeightByContactFieldEvaluator.submit}" />
			</div>

		</form>
		
		<div style="display: none;">
			
			<table>
				<xsl:call-template name="createValue">
					<xsl:with-param name="valueID" select="'template'" />
					<xsl:with-param name="rowID" select="'value-template'"/>
				</xsl:call-template>
			</table>
			
		</div>		

	</xsl:template>
	
	<xsl:template match="Value">

		<xsl:call-template name="createValue">
			<xsl:with-param name="valueID" select="valueID" />
			<xsl:with-param name="matchedValue" select="matchedValue" />
			<xsl:with-param name="weight" select="weight" />
		</xsl:call-template>

	</xsl:template>	
	
	<xsl:template match="value" mode="parameter">
		<xsl:param name="requestparameters" select="../../." />
	
		<xsl:variable name="matchedValueParam" select="concat('matchedValue-', .)"/>
		<xsl:variable name="weightParam" select="concat('weight-', .)"/>
		
		<xsl:call-template name="createValue">
			<xsl:with-param name="valueID" select="." />
			<xsl:with-param name="matchedValue" select="$requestparameters/parameter[name = $matchedValueParam]/value" />
			<xsl:with-param name="weight" select="$requestparameters/parameter[name = $weightParam]/value" />
		</xsl:call-template>

	</xsl:template>	
	
	<xsl:template name="createValue">
		<xsl:param name="valueID"  />
		<xsl:param name="matchedValue" select="null" />
		<xsl:param name="weight" select="null" />
		<xsl:param name="rowID" select="concat('value-', $valueID)" />
	
		<xsl:variable name="matchedValueParam" select="concat('matchedValue-', $valueID)"/>
		<xsl:variable name="weightParam" select="concat('weight-', $valueID)"/>

		<tr id="{$rowID}">
			<td>
				<input type="text" name="{$matchedValueParam}" value="{$matchedValue}" class="full"/>
			</td>
			<td>
				<input type="text" name="{$weightParam}" value="{$weight}" class="full"/>
			</td>
			<td>
				<input type="hidden" name="valueID" value="{$valueID}"/>
				
				<a onclick="deleteValue(this)">
					<img class="alignbottom" src="{/Document/StaticContentURL}/pics/delete.png" alt="" />
				</a>				
			</td>			
		</tr>
			
	</xsl:template>
	
	<xsl:template match="fieldName">

		<xsl:variable name="fieldName" select="." />

		<xsl:choose>
			<xsl:when test="$fieldName = 'weightType'">
				<xsl:value-of select="$i18n.WeightType" />
			</xsl:when>
			<xsl:when test="$fieldName = 'contactField'">
				<xsl:value-of select="$i18n.ContactField" />
			</xsl:when>
			<xsl:when test="starts-with($fieldName, 'matchedValue-')">
				<xsl:value-of select="$i18n.Value.MatchedValue" />
			</xsl:when>
			<xsl:when test="starts-with($fieldName, 'weight-')">
				<xsl:value-of select="$i18n.Value.Weight" />
			</xsl:when>
			<xsl:when test="$fieldName = 'weightIfNoValueMatches'">
				<xsl:value-of select="$i18n.SetWeightIfContactFieldIsEmpty" />
			</xsl:when>
			<xsl:when test="$fieldName = 'weightIfContactFieldIsEmpty'">
				<xsl:value-of select="$i18n.SetWeightIfNoValueMatches" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$fieldName" />
			</xsl:otherwise>
		</xsl:choose>

	</xsl:template>

</xsl:stylesheet>