package com.nordicpeak.flowengine.evaluators.setweightbycontactfieldevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.object.ObjectUtils;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_contact_field_evaluators")
@XMLElement
public class SetWeightByContactFieldEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String contactField;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private boolean setWeightIfNoValueMatches;

	@DAOManaged
	@WebPopulate
	@XMLElement
	@RequiredIfSet(paramNames = "setWeightIfNoValueMatches")
	private Integer weightIfNoValueMatches;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private boolean setWeightIfContactFieldIsEmpty;

	@DAOManaged
	@WebPopulate
	@XMLElement
	@RequiredIfSet(paramNames = "setWeightIfContactFieldIsEmpty")
	private Integer weightIfContactFieldIsEmpty;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement(fixCase = true)
	private List<SetWeightByContactFieldEvaluatorValue> values;

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	public String getContactField() {

		return contactField;
	}

	public void setContactField(String contactField) {

		this.contactField = contactField;
	}

	public boolean isSetWeightIfNoValueMatches() {

		return setWeightIfNoValueMatches;
	}

	public void setSetWeightIfNoValueMatches(boolean setWeightIfNoValueMatches) {

		this.setWeightIfNoValueMatches = setWeightIfNoValueMatches;
	}

	public Integer getWeightIfNoValueMatches() {

		return weightIfNoValueMatches;
	}

	public void setWeightIfNoValueMatches(Integer weightIfNoValueMatches) {

		this.weightIfNoValueMatches = weightIfNoValueMatches;
	}

	public boolean isSetWeightIfContactFieldIsEmpty() {

		return setWeightIfContactFieldIsEmpty;
	}

	public void setSetWeightIfContactFieldIsEmpty(boolean setWeightIfContactFieldIsEmpty) {

		this.setWeightIfContactFieldIsEmpty = setWeightIfContactFieldIsEmpty;
	}

	public Integer getWeightIfContactFieldIsEmpty() {

		return weightIfContactFieldIsEmpty;
	}

	public void setWeightIfContactFieldIsEmpty(Integer weightIfContactFieldIsEmpty) {

		this.weightIfContactFieldIsEmpty = weightIfContactFieldIsEmpty;
	}

	public List<SetWeightByContactFieldEvaluatorValue> getValues() {

		return values;
	}

	public void setValues(List<SetWeightByContactFieldEvaluatorValue> matchedValues) {

		this.values = matchedValues;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);
		contactField = XMLValidationUtils.validateParameter("contactField", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		setWeightIfNoValueMatches = xmlParser.getPrimitiveBoolean("setWeightIfNoValueMatches");

		if (setWeightIfNoValueMatches) {

			weightIfNoValueMatches = XMLValidationUtils.validateParameter("weightIfNoValueMatches", xmlParser, true, IntegerPopulator.getPopulator(), errors);
		}

		setWeightIfContactFieldIsEmpty = xmlParser.getPrimitiveBoolean("setWeightIfContactFieldIsEmpty");

		if (setWeightIfContactFieldIsEmpty) {

			setWeightIfContactFieldIsEmpty(XMLValidationUtils.validateParameter("weightIfContactFieldIsEmpty", xmlParser, true, IntegerPopulator.getPopulator(), errors));
		}

		List<XMLParser> weightNodes = xmlParser.getNodes("Values/Value");

		if (!CollectionUtils.isEmpty(weightNodes)) {

			values = new ArrayList<>(weightNodes.size());

			for (XMLParser parser : weightNodes) {

				SetWeightByContactFieldEvaluatorValue value = new SetWeightByContactFieldEvaluatorValue();

				value.populate(parser);

				value.setEvaluator(this);

				values.add(value);
			}
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public boolean isConfigured() {

		if (ObjectUtils.containsNull(weightType, contactField)) {

			return false;
		}

		if ((values == null || CollectionUtils.filter(values, value -> !value.isConfigured()) != null) && !setWeightIfContactFieldIsEmpty) {

			return false;
		}

		if(setWeightIfContactFieldIsEmpty && weightIfContactFieldIsEmpty == null) {
			
			return false;
		}
		
		return true;
	}

}
