package com.nordicpeak.flowengine.evaluators.setweightbycontactfieldevaluator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.populators.IntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationErrorType;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;
import se.unlogic.webutils.validation.ValidationUtils;

import com.nordicpeak.flowengine.beans.ContactField;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.ContactFieldQuery;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;

public class SetWeightByContactFieldEvaluatorCRUD extends BaseEvaluatorCRUD<SetWeightByContactFieldEvaluator, SetWeightByContactFieldEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SetWeightByContactFieldEvaluator> POPULATOR = new AnnotatedRequestPopulator<>(SetWeightByContactFieldEvaluator.class);

	public SetWeightByContactFieldEvaluatorCRUD(Class<SetWeightByContactFieldEvaluator> beanClass, AnnotatedDAOWrapper<SetWeightByContactFieldEvaluator, Integer> evaluatorDAO, SetWeightByContactFieldEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetWeightByContactFieldEvaluator", "set weight by contact field evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(SetWeightByContactFieldEvaluator evaluator, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(evaluator, doc, updateTypeElement, user, req, uriParser);

		Query query = callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), false);

		if (query instanceof ContactFieldQuery) {

			ContactFieldQuery contactFieldQuery = (ContactFieldQuery) query;

			List<ContactField> contactFields = contactFieldQuery.getContactFields(callback.getQueryHandler());

			XMLUtils.append(doc, updateTypeElement, "ContactFields", contactFields);
		}
	}

	@Override
	protected SetWeightByContactFieldEvaluator populateFromUpdateRequest(SetWeightByContactFieldEvaluator evaluator, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		evaluator = super.populateFromUpdateRequest(evaluator, req, user, uriParser);

		evaluator.setValues(populateValues(evaluator, req, user, uriParser));

		return evaluator;
	}

	private List<SetWeightByContactFieldEvaluatorValue> populateValues(SetWeightByContactFieldEvaluator evaluator, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException {

		String[] valueIDs = req.getParameterValues("valueID");

		if (valueIDs == null) {

			return null;
		}

		List<SetWeightByContactFieldEvaluatorValue> values = null;
		List<ValidationError> validationErrors = new ArrayList<>();

		for (String valueID : valueIDs) {

			String matchedValue = ValidationUtils.validateParameter("matchedValue-" + valueID, callback.getMatchedValueDisplayName(), req, true, 1, 255, StringPopulator.getPopulator(), validationErrors);
			Integer weight = ValidationUtils.validateParameter("weight-" + valueID, callback.getWeightDisplayName(), req, true, null, null, IntegerPopulator.getPopulator(), validationErrors);

			if (validationErrors.isEmpty()) {

				SetWeightByContactFieldEvaluatorValue value = new SetWeightByContactFieldEvaluatorValue(evaluator, matchedValue, weight);

				values = CollectionUtils.addAndInstantiateIfNeeded(values, value);
			}
		}

		if (!validationErrors.isEmpty()) {

			throw new ValidationException(validationErrors);
		}

		return values;
	}

	@Override
	protected void validateUpdatePopulation(SetWeightByContactFieldEvaluator evaluator, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, SQLException, Exception {

		super.validateUpdatePopulation(evaluator, req, user, uriParser);

		Query query = callback.getQueryHandler().getQuery((MutableQueryDescriptor) evaluator.getEvaluatorDescriptor().getQueryDescriptor(), false);

		if (query instanceof ContactFieldQuery) {

			ContactFieldQuery contactFieldQuery = (ContactFieldQuery) query;

			List<ContactField> contactFields = contactFieldQuery.getContactFields(callback.getQueryHandler());

			boolean selectedContactFieldExists = CollectionUtils.find(contactFields, cf -> cf.getName().equals(evaluator.getContactField())) != null;

			if (!selectedContactFieldExists) {

				throw new ValidationException(new ValidationError("contactField", ValidationErrorType.InvalidFormat));
			}
		}
	}

}