package com.nordicpeak.flowengine.evaluators.setweightbyexistingflowinstancestatusevaluator;

import java.util.ArrayList;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.EnumPopulator;
import se.unlogic.standardutils.populators.PositiveStringIntegerPopulator;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.enums.ContentType;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_exist_flow_instance_status_evaluators")
@XMLElement
public class SetWeightByExistingFlowInstanceStatusEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -336493853396860255L;

	private static final EnumPopulator<ContentType> CONTENT_TYPE_POPULATOR = new EnumPopulator<>(ContentType.class);

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@WebPopulate(required = true)
	@XMLElement
	private Integer weight;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "set_weight_by_exist_flow_instance_status_evaluator_content_types", remoteValueColumnName = "contentType")
	@WebPopulate(required = true)
	@XMLElement(childName = "contentType")
	private List<ContentType> contentTypes;

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<ValidationError>();

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		weight = XMLValidationUtils.validateParameter("weight", xmlParser, false, PositiveStringIntegerPopulator.getPopulator(), errors);

		contentTypes = XMLValidationUtils.validateParameters("contentTypes/contentType", xmlParser, false, CONTENT_TYPE_POPULATOR, errors); //TODO test

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
	}

	@Override
	public String toString() {

		return evaluatorDescriptor.toString();
	}

	@Override
	public boolean forceAjaxPosting() {

		return true;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public Integer getWeight() {

		return weight;
	}

	public void setWeight(Integer weight) {

		this.weight = weight;
	}

	public List<ContentType> getContentTypes() {

		return contentTypes;
	}

	public void setContentTypes(List<ContentType> contentTypes) {

		this.contentTypes = contentTypes;
	}

}
