package com.nordicpeak.flowengine.evaluators.setweightbyflowinstancestatus;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.hierarchy.core.interfaces.attributes.MutableAttributeHandler;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.xml.XMLGenerator;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.beans.EvaluationResponse;
import com.nordicpeak.flowengine.beans.QueryModification;
import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluationProviderModule;
import com.nordicpeak.flowengine.evaluators.setweightevaluator.SetWeightEvaluationEvent;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.Evaluator;
import com.nordicpeak.flowengine.interfaces.ImmutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.ImmutableFlow;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.InstanceMetadata;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

public class SetWeightByFlowInstanceStatusEvaluationProviderModule extends BaseEvaluationProviderModule<SetWeightByFlowInstanceStatusEvaluator> {

	protected SetWeightByFlowInstanceStatusEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<SetWeightByFlowInstanceStatusEvaluator> evaluatorDAO;

	protected QueryParameterFactory<SetWeightByFlowInstanceStatusEvaluator, Integer> evaluatorIDParamFactory;

	private AnnotatedDAOWrapper<SetWeightByFlowInstanceStatusEvaluator, Integer> evaluatorDAOWrapper;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, SetWeightByFlowInstanceStatusEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(SetWeightByFlowInstanceStatusEvaluator.class);
		evaluatorDAOWrapper = evaluatorDAO.getWrapper("evaluatorID", Integer.class);

		evaluatorDAOWrapper.addRelation(SetWeightByFlowInstanceStatusEvaluator.VALUE_RELATION);
		evaluatorDAOWrapper.setUseRelationsOnGet(true);
		evaluatorDAOWrapper.setUseRelationsOnUpdate(true);
		evaluatorDAOWrapper.setUseRelationsOnAdd(true);

		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new SetWeightByFlowInstanceStatusEvaluatorCRUD(SetWeightByFlowInstanceStatusEvaluator.class, evaluatorDAOWrapper, this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	public List<? extends ImmutableStatus> getFlowStatuses(Integer flowID) {

		ImmutableFlow flow = flowAdminModule.getFlow(flowID);

		if (flow != null) {

			return flow.getStatuses();
		}

		return null;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return true;
	}

	protected AnnotatedDAO<SetWeightByFlowInstanceStatusEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	protected Class<SetWeightByFlowInstanceStatusEvaluator> getEvaluatorClass() {

		return SetWeightByFlowInstanceStatusEvaluator.class;
	}

	protected QueryParameterFactory<SetWeightByFlowInstanceStatusEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		SetWeightByFlowInstanceStatusEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		handleStatusConversion(evaluator, statusConversionMap);

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

	}

	@Override
	public SetWeightByFlowInstanceStatusEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		SetWeightByFlowInstanceStatusEvaluator evaluator = getEvaluatorClass().newInstance();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		evaluator.populate(descriptor.getImportParser().getNode(XMLGenerator.getElementName(evaluator.getClass())));

		handleStatusConversion(evaluator, statusConversionMap);

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	private void handleStatusConversion(SetWeightByFlowInstanceStatusEvaluator evaluator, Map<Integer, ImmutableStatus> statusConversionMap) {

		List<SetWeightByFlowInstanceStatusEvaluatorValues> values = evaluator.getWeightByFlowInstanceEvaluatorValueList();
		if (!CollectionUtils.isEmpty(values)) {

			//make sure each statusID is converted to the new id
			values.stream().filter(Objects::nonNull).forEach(e -> {
				try {
					e.setStatusID(statusConversionMap.get(e.getStatusID()).getStatusID());
				}catch(Exception ex) {
					log.warn("Unable to convert status "+e.getStatusID()+" to new status");
				}
			});
		}

	}

	protected List<Integer> getTargetQueryIDs(SetWeightByFlowInstanceStatusEvaluator evaluator, EvaluationCallback callback) {

		return Collections.singletonList(callback.getQueryInstance().getQueryInstanceDescriptor().getQueryDescriptor().getQueryID());
	}

	@Override
	public EvaluationResponse evaluate(QueryInstance queryInstance, User user, User poster, SetWeightByFlowInstanceStatusEvaluator evaluator, EvaluationCallback callback, boolean hasQueryValidationErrors, MutableAttributeHandler attributeHandler) {

		List<SetWeightByFlowInstanceStatusEvaluatorValues> listOfValues = evaluator.getWeightByFlowInstanceEvaluatorValueList();

		if (user == null || CollectionUtils.isEmpty(listOfValues)) {

			return null;
		}

		BigDecimal weight = BigDecimal.ZERO;

		if (callback.getFlowInstance().getStatus() != null) {
			try {
				log.info("Checking for flow instance status" + callback.getFlowInstance());
				
				SetWeightByFlowInstanceStatusEvaluatorValues currentEvaluatorValues = null;
				
				for (SetWeightByFlowInstanceStatusEvaluatorValues evaluatorValues : listOfValues) {
					if (evaluatorValues.getStatusID().equals(callback.getFlowInstance().getStatus().getStatusID())) {
						currentEvaluatorValues = evaluatorValues;
						
						break;
					}
				}
				
				if (currentEvaluatorValues != null && currentEvaluatorValues.getWeight() != null) {
					
					weight = BigDecimal.valueOf(currentEvaluatorValues.getWeight());
				}
				
			} catch (Exception e) {
				
				log.error("Error checking current flow instance " + callback.getFlowInstance(), e);
			}
		}

		List<QueryModification> queryModifications = callback.sendEvent(new SetWeightEvaluationEvent(evaluator.getEvaluatorID(), evaluator.getWeightType(), weight, attributeHandler), true);

		if (queryModifications != null) {

			return new EvaluationResponse(queryModifications);
		}

		return null;
	}

	@Override
	public Evaluator createEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		SetWeightByFlowInstanceStatusEvaluator evaluator = new SetWeightByFlowInstanceStatusEvaluator();

		evaluator.setEvaluatorID(descriptor.getEvaluatorID());

		getEvaluatorDAO().add(evaluator, transactionHandler, null);

		evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());

		return evaluator;

	}

	@Override
	public Evaluator getEvaluator(MutableEvaluatorDescriptor descriptor, InstanceMetadata instanceMetadata) throws Throwable {

		SetWeightByFlowInstanceStatusEvaluator evaluator = evaluatorDAOWrapper.getAnnotatedDAO().get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())));

		if (evaluator != null) {

			evaluator.init(descriptor, getFullAlias() + "/config/" + descriptor.getEvaluatorID());
		}

		return evaluator;
	}

	@Override
	public boolean deleteEvaluator(ImmutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler) throws Throwable {

		evaluatorDAO.delete(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(descriptor.getEvaluatorID())), transactionHandler);

		return true;
	}

	public SetWeightByFlowInstanceStatusEvaluator getEvaluator(Integer evaluatorID) throws SQLException {

		return evaluatorDAOWrapper.getAnnotatedDAO().get(new HighLevelQuery<>(evaluatorIDParamFactory.getParameter(evaluatorID)));
	}

}
