package com.nordicpeak.flowengine.evaluators.setweightbyflowinstancestatus;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.collections.CollectionUtils;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.populators.StringPopulator;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLElement;
import se.unlogic.standardutils.xml.XMLParser;
import se.unlogic.standardutils.xml.XMLValidationUtils;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluator;

@Table(name = "set_weight_by_flow_instance_status_evaluators")
@XMLElement
public class SetWeightByFlowInstanceStatusEvaluator extends BaseEvaluator {

	private static final long serialVersionUID = -1383195490043894435L;

	private ValidationError validationErrorMissingValues = new ValidationError("MissingStatusValues");
	private ValidationError validationErrorWeightNameMissing = new ValidationError("WeightNameMissing");

	public static final Field VALUE_RELATION = ReflectionUtils.getField(SetWeightByFlowInstanceStatusEvaluator.class, "setWeightByFlowInstanceStatusEvaluatorValueList");

	@DAOManaged
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String weightType;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@XMLElement
	private List<SetWeightByFlowInstanceStatusEvaluatorValues> setWeightByFlowInstanceStatusEvaluatorValueList;

	private HashMap<Integer, BigDecimal> weightSourceMap;

	public String getWeightType() {

		return weightType;
	}

	public void setWeightType(String weightType) {

		this.weightType = weightType;
	}

	@Override
	public boolean requiresInitialization() {

		return true;
	}

	public List<SetWeightByFlowInstanceStatusEvaluatorValues> getWeightByFlowInstanceEvaluatorValueList() {

		return setWeightByFlowInstanceStatusEvaluatorValueList;
	}

	public void setWeightByFlowInstanceStatusValueList(List<SetWeightByFlowInstanceStatusEvaluatorValues> setWeightByFlowInstanceEvaluatorValueList) {

		this.setWeightByFlowInstanceStatusEvaluatorValueList = setWeightByFlowInstanceEvaluatorValueList;
	}

	@Override
	public void populate(XMLParser xmlParser) throws ValidationException {

		List<ValidationError> errors = new ArrayList<>(1);

		weightType = XMLValidationUtils.validateParameter("weightType", xmlParser, false, 1, 255, StringPopulator.getPopulator(), errors);

		try {

			List<XMLParser> valueNodes = xmlParser.getNodes("setWeightByFlowInstanceStatusEvaluatorValueList/SetWeightByFlowInstanceStatusEvaluatorValues");

			if (!CollectionUtils.isEmpty(valueNodes)) {

				setWeightByFlowInstanceStatusEvaluatorValueList = new ArrayList<>(valueNodes.size());

				for (XMLParser parser : valueNodes) {

					SetWeightByFlowInstanceStatusEvaluatorValues weightQueryStateEvaluatorValues = new SetWeightByFlowInstanceStatusEvaluatorValues();

					weightQueryStateEvaluatorValues.populate(parser);

					weightQueryStateEvaluatorValues.setWeightQueryStateEvaluator(this);

					setWeightByFlowInstanceStatusEvaluatorValueList.add(weightQueryStateEvaluatorValues);
				}
			}
		} catch (ValidationException ve) {
			errors.addAll(ve.getErrors());
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}

	}

	public BigDecimal getCurrentWeight() {

		if (weightSourceMap != null) {

			BigDecimal totalWeight = BigDecimal.ZERO;

			for (BigDecimal weight : weightSourceMap.values()) {

				totalWeight = totalWeight.add(weight);
			}

			return totalWeight;
		}

		return null;
	}

	public void validate() throws ValidationException {

		List<ValidationError> validationErrors = new ArrayList<>();

		if (StringUtils.isEmpty(weightType)) {
			validationErrors.add(validationErrorWeightNameMissing);
		}

		if (CollectionUtils.isEmpty(setWeightByFlowInstanceStatusEvaluatorValueList)) {
			validationErrors.add(validationErrorMissingValues);
			throw new ValidationException(validationErrors);
		}

		boolean valueSet = false;
		for (int i = 0; i < setWeightByFlowInstanceStatusEvaluatorValueList.size(); i++) {
			SetWeightByFlowInstanceStatusEvaluatorValues evaluatorvalue = setWeightByFlowInstanceStatusEvaluatorValueList.get(i);
			if (evaluatorvalue.getWeight() != null) {
				valueSet = true;
				break;
			}
		}

		if (!valueSet) {
			addValidationError(validationErrors, validationErrorMissingValues);
		}

		if (!validationErrors.isEmpty()) {
			throw new ValidationException(validationErrors);
		}
	}

	private void addValidationError(List<ValidationError> validationErrors, ValidationError validationError) {

		if (!validationErrors.contains(validationError)) {
			validationErrors.add(validationError);
		}
	}

}
