package com.nordicpeak.flowengine.evaluators.setweightbytextfieldvalueevaluator;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.standardutils.dao.AnnotatedDAOWrapper;
import se.unlogic.standardutils.validation.ValidationError;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.standardutils.xml.XMLUtils;
import se.unlogic.webutils.http.URIParser;
import se.unlogic.webutils.populators.annotated.AnnotatedRequestPopulator;

import com.nordicpeak.flowengine.evaluators.baseevaluator.BaseEvaluatorCRUD;
import com.nordicpeak.flowengine.interfaces.MutableQueryDescriptor;
import com.nordicpeak.flowengine.queries.textfieldquery.TextField;
import com.nordicpeak.flowengine.queries.textfieldquery.TextFieldQuery;

public class SetWeightByTextFieldValueEvaluatorCRUD extends BaseEvaluatorCRUD<SetWeightByTextFieldValueEvaluator, SetWeightByTextFieldValueEvaluationProviderModule> {

	private static final AnnotatedRequestPopulator<SetWeightByTextFieldValueEvaluator> POPULATOR = new AnnotatedRequestPopulator<SetWeightByTextFieldValueEvaluator>(SetWeightByTextFieldValueEvaluator.class);

	public SetWeightByTextFieldValueEvaluatorCRUD(Class<SetWeightByTextFieldValueEvaluator> beanClass, AnnotatedDAOWrapper<SetWeightByTextFieldValueEvaluator, Integer> evaluatorDAO, SetWeightByTextFieldValueEvaluationProviderModule callback) {

		super(beanClass, evaluatorDAO, POPULATOR, "SetWeightByTextFieldValueEvaluator", "set weight by text field value evaluator", null, callback);
	}

	@Override
	protected void appendUpdateFormData(SetWeightByTextFieldValueEvaluator bean, Document doc, Element updateTypeElement, User user, HttpServletRequest req, URIParser uriParser) throws Exception {

		super.appendUpdateFormData(bean, doc, updateTypeElement, user, req, uriParser);

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		List<TextField> fields = query.getFields();

		if (fields != null) {

			Element fieldsElement = doc.createElement("TextFields");
			updateTypeElement.appendChild(fieldsElement);

			for (TextField field : fields) {

				Element fieldElement = doc.createElement("TextField");
				fieldsElement.appendChild(fieldElement);

				XMLUtils.appendNewElement(doc, fieldElement, "fieldID", field.getTextFieldID());
				XMLUtils.appendNewCDATAElement(doc, fieldElement, "name", field.getLabel());
			}
		}
	}

	@Override
	protected SetWeightByTextFieldValueEvaluator populateFromUpdateRequest(SetWeightByTextFieldValueEvaluator bean, HttpServletRequest req, User user, URIParser uriParser) throws ValidationException, Exception {

		SetWeightByTextFieldValueEvaluator evaluator = super.populateFromUpdateRequest(bean, req, user, uriParser);
		
		List<ValidationError> errors = new ArrayList<ValidationError>();

		Integer textFieldID = bean.getTextFieldID();

		boolean found = false;

		TextFieldQuery query = (TextFieldQuery) callback.getQueryHandler().getQuery((MutableQueryDescriptor) bean.getEvaluatorDescriptor().getQueryDescriptor(), false);

		if (query.getFields() != null) {

			for (TextField field : query.getFields()) {

				if (field.getTextFieldID().equals(textFieldID)) {

					found = true;
				}
			}
		}

		if (!found) {

			errors.add(new ValidationError("TextFieldNotFound"));
		}

		if (!errors.isEmpty()) {

			throw new ValidationException(errors);
		}
		
		return evaluator;
	}

}
