package com.nordicpeak.flowengine.evaluators.weightapi.beans;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.nio.charset.Charset;
import java.util.List;

import se.unlogic.standardutils.annotations.NoDuplicates;
import se.unlogic.standardutils.annotations.RequiredIfSet;
import se.unlogic.standardutils.annotations.WebPopulate;
import se.unlogic.standardutils.dao.annotations.DAOManaged;
import se.unlogic.standardutils.dao.annotations.Key;
import se.unlogic.standardutils.dao.annotations.OneToMany;
import se.unlogic.standardutils.dao.annotations.OrderBy;
import se.unlogic.standardutils.dao.annotations.SimplifiedRelation;
import se.unlogic.standardutils.dao.annotations.Table;
import se.unlogic.standardutils.reflection.ReflectionUtils;
import se.unlogic.standardutils.string.StringUtils;
import se.unlogic.standardutils.xml.GeneratedElementable;
import se.unlogic.standardutils.xml.XMLElement;

import com.nordicpeak.flowengine.interfaces.APISource;

@Table(name = "weight_from_api_sources")
@XMLElement(name = "APISource")
public class WeightAPISource extends GeneratedElementable implements Serializable, APISource {

	public static final Field EVENTS_RELATION = ReflectionUtils.getField(WeightAPISource.class, "events");

	private static final long serialVersionUID = 3296400205039536096L;

	@Key
	@DAOManaged(autoGenerated = true)
	@XMLElement
	private Integer sourceID;

	@DAOManaged
	@OrderBy
	@WebPopulate(required = true, maxLength = 255)
	@XMLElement
	private String name;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 1000)
	@XMLElement
	private String url;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "password")
	@XMLElement
	private String username;

	@DAOManaged
	@WebPopulate(maxLength = 255)
	@RequiredIfSet(paramNames = "username")
	@XMLElement
	private String password;

	@DAOManaged
	@WebPopulate(required = true, maxLength = 50)
	@XMLElement
	private String encoding;

	@DAOManaged
	@OneToMany
	@XMLElement
	private List<WeightAPISourceEvent> events;

	@DAOManaged
	@WebPopulate
	@XMLElement
	private boolean useAccessRestriction;

	@DAOManaged
	@OneToMany(autoAdd = true, autoGet = true, autoUpdate = true)
	@SimplifiedRelation(table = "weight_from_api_source_flowfamilies", remoteValueColumnName = "flowFamilyID")
	@WebPopulate(paramName = "flowfamilyID")
	@NoDuplicates
	@XMLElement(childName = "flowFamilyID")
	@RequiredIfSet(paramNames = "useAccessRestriction")
	private List<Integer> flowFamilies;

	public Integer getSourceID() {

		return sourceID;
	}

	public void setSourceID(Integer sourceID) {

		this.sourceID = sourceID;
	}

	public String getName() {

		return name;
	}

	public void setName(String name) {

		this.name = name;
	}

	public String getUrl() {

		return url;
	}

	public void setUrl(String url) {

		this.url = url;
	}

	public String getUsername() {

		return username;
	}

	public void setUsername(String username) {

		this.username = username;
	}

	public String getPassword() {

		return password;
	}

	public void setPassword(String password) {

		this.password = password;
	}

	public String getEncoding() {

		return encoding;
	}

	public void setEncoding(String encoding) {

		this.encoding = encoding;
	}

	public Charset getCharset() {

		return Charset.forName(encoding);
	}

	@Override
	public String toString() {

		return StringUtils.toLogFormat(name, 30) + " (ID: " + sourceID + ")";
	}

	public List<Integer> getFlowFamilies() {

		return flowFamilies;
	}

	public void setFlowFamilies(List<Integer> flowFamilies) {

		this.flowFamilies = flowFamilies;
	}

	public List<WeightAPISourceEvent> getEvents() {

		return events;
	}

	public void setEvents(List<WeightAPISourceEvent> events) {

		this.events = events;
	}

	public boolean isUseAccessRestriction() {

		return useAccessRestriction;
	}

	public void setUseAccessRestriction(boolean useAccessRestriction) {

		this.useAccessRestriction = useAccessRestriction;
	}

	//TODO rename sourceID field
	@Override
	public Integer getEndpointID() {

		return sourceID;
	}

	//TODO rename url field
	@Override
	public String getAddress() {

		return url;
	}

}
