package com.nordicpeak.flowengine.evaluators.weightquerystateevaluator;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;

import se.unlogic.hierarchy.core.annotations.WebPublic;
import se.unlogic.hierarchy.core.beans.User;
import se.unlogic.hierarchy.core.interfaces.ForegroundModuleResponse;
import se.unlogic.standardutils.dao.AnnotatedDAO;
import se.unlogic.standardutils.dao.HighLevelQuery;
import se.unlogic.standardutils.dao.QueryParameterFactory;
import se.unlogic.standardutils.dao.SimpleAnnotatedDAOFactory;
import se.unlogic.standardutils.dao.TransactionHandler;
import se.unlogic.standardutils.db.tableversionhandler.TableVersionHandler;
import se.unlogic.standardutils.db.tableversionhandler.UpgradeResult;
import se.unlogic.standardutils.db.tableversionhandler.XMLDBScriptProvider;
import se.unlogic.standardutils.validation.ValidationException;
import se.unlogic.webutils.http.URIParser;

import com.nordicpeak.flowengine.evaluators.calculatequerystateevaluator.CalculatedQueryStateEvaluationProviderModule;
import com.nordicpeak.flowengine.interfaces.EvaluationCallback;
import com.nordicpeak.flowengine.interfaces.ImmutableStatus;
import com.nordicpeak.flowengine.interfaces.MutableEvaluatorDescriptor;
import com.nordicpeak.flowengine.interfaces.Query;
import com.nordicpeak.flowengine.interfaces.QueryInstance;

public class WeightQueryStateEvaluationProviderModule extends CalculatedQueryStateEvaluationProviderModule<WeightQueryStateEvaluator> {

	protected WeightQueryStateEvaluatorCRUD evaluatorCRUD;

	protected AnnotatedDAO<WeightQueryStateEvaluator> evaluatorDAO;

	protected QueryParameterFactory<WeightQueryStateEvaluator, Integer> evaluatorIDParamFactory;

	@Override
	protected void createDAOs(DataSource dataSource) throws Exception {

		//Automatic table version handling
		UpgradeResult upgradeResult = TableVersionHandler.upgradeDBTables(dataSource, WeightQueryStateEvaluationProviderModule.class.getName(), new XMLDBScriptProvider(this.getClass().getResourceAsStream("DB script.xml")));

		if (upgradeResult.isUpgrade()) {

			log.info(upgradeResult.toString());
		}

		evaluatorDAO = new SimpleAnnotatedDAOFactory(dataSource).getDAO(WeightQueryStateEvaluator.class);
		evaluatorIDParamFactory = evaluatorDAO.getParamFactory("evaluatorID", Integer.class);

		evaluatorCRUD = new WeightQueryStateEvaluatorCRUD(WeightQueryStateEvaluator.class, evaluatorDAO.getWrapper("evaluatorID", Integer.class), this);
	}

	@WebPublic(alias = "config")
	public ForegroundModuleResponse configureEvaluator(HttpServletRequest req, HttpServletResponse res, User user, URIParser uriParser) throws Exception {

		return this.evaluatorCRUD.update(req, res, user, uriParser);
	}

	@Override
	protected BigDecimal getCalculatedValue(QueryInstance queryInstance, WeightQueryStateEvaluator evaluator, User poster) throws ValidationException {

		return evaluator.getCurrentWeight();
	}

	@Override
	protected Class<WeightQueryStateEvaluator> getEvaluatorClass() {

		return WeightQueryStateEvaluator.class;
	}

	@Override
	public boolean supportsQueryType(Class<? extends Query> queryClass) {

		return true;
	}
	
	@Override
	protected boolean supportsQueryInstance(QueryInstance queryInstance) {

		return true;
	}

	@Override
	protected AnnotatedDAO<WeightQueryStateEvaluator> getEvaluatorDAO() {

		return evaluatorDAO;
	}

	@Override
	protected QueryParameterFactory<WeightQueryStateEvaluator, Integer> getEvaluatorIDParameterFactory() {

		return evaluatorIDParamFactory;
	}

	@Override
	public void copyEvaluator(MutableEvaluatorDescriptor sourceEvaluatorDescriptor, MutableEvaluatorDescriptor copyEvaluatorDescriptor, Query sourceQuery, Query copyQuery, Map<Integer, ImmutableStatus> statusConversionMap, TransactionHandler transactionHandler) throws SQLException {

		WeightQueryStateEvaluator evaluator = evaluatorDAO.get(new HighLevelQuery<WeightQueryStateEvaluator>(evaluatorIDParamFactory.getParameter(sourceEvaluatorDescriptor.getEvaluatorID())));

		evaluator.setEvaluatorID(copyEvaluatorDescriptor.getEvaluatorID());

		this.evaluatorDAO.add(evaluator, transactionHandler, null);
		
	}

	@Override
	public WeightQueryStateEvaluator importEvaluator(MutableEvaluatorDescriptor descriptor, TransactionHandler transactionHandler, Query query, Map<Integer, ImmutableStatus> statusConversionMap) throws Throwable {

		WeightQueryStateEvaluator evaluator = importEvaluatorDescriptor(descriptor, transactionHandler, query, statusConversionMap);

		this.evaluatorDAO.add(evaluator, transactionHandler, null);

		return evaluator;
	}

	@Override
	protected List<Integer> getTargetQueryIDs(WeightQueryStateEvaluator evaluator, EvaluationCallback callback) {

		return Collections.singletonList(callback.getQueryInstance().getQueryInstanceDescriptor().getQueryDescriptor().getQueryID());
	}
}
